{% extends 'is_core/base.html' %}
{% load forms static i18n %}

{% block title %}{% trans 'Login' %}{% endblock %}

{% block body %}
{% block sticky-flash %}{{ block.super }}{% endblock %}
<div class="login-page">
  <div class="container login">
    <div class="row">
      <div class="col-md-4 col-md-offset-4">
        <div class="login-panel panel panel-default">
          <div class="panel-heading">
            <h3 class="panel-title">{% trans 'Please Sign In' %}</h3>
          </div>
          <div class="panel-body">
            <h4>{% trans 'Fill your credentials' %}</h4>
            <br>
            {% form_renderer form template="is_core/forms/login_form.html" class_attr='login-form' id_attr='login-form' %}
              {% block below-login-form %}{% endblock %}
          </div>
        </div>
      </div>
  </div>
</div>
{% endblock %}
