if(typeof Math.imul == "undefined" || (Math.imul(0xffffffff,5) == 0)) {
    Math.imul = function (a, b) {
        var ah  = (a >>> 16) & 0xffff;
        var al = a & 0xffff;
        var bh  = (b >>> 16) & 0xffff;
        var bl = b & 0xffff;
        // the shift by 0 fixes the sign on the high part
        // the final |0 converts the unsigned value into a signed value
        return ((al * bl) + (((ah * bl + al * bh) << 16) >>> 0)|0);
    }
}

/**
 * React v15.3.1
 *
 * Copyright 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 */
!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e();else if("function"==typeof define&&define.amd)define([],e);else{var t;t="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,t.React=e()}}(function(){return function e(t,n,r){function o(i,s){if(!n[i]){if(!t[i]){var u="function"==typeof require&&require;if(!s&&u)return u(i,!0);if(a)return a(i,!0);var l=new Error("Cannot find module '"+i+"'");throw l.code="MODULE_NOT_FOUND",l}var c=n[i]={exports:{}};t[i][0].call(c.exports,function(e){var n=t[i][1][e];return o(n?n:e)},c,c.exports,e,t,n,r)}return n[i].exports}for(var a="function"==typeof require&&require,i=0;i<r.length;i++)o(r[i]);return o}({1:[function(e,t,n){"use strict";var r=e(40),o=e(148),a={focusDOMComponent:function(){o(r.getNodeFromInstance(this))}};t.exports=a},{148:148,40:40}],2:[function(e,t,n){"use strict";function r(){var e=window.opera;return"object"==typeof e&&"function"==typeof e.version&&parseInt(e.version(),10)<=12}function o(e){return(e.ctrlKey||e.altKey||e.metaKey)&&!(e.ctrlKey&&e.altKey)}function a(e){switch(e){case k.topCompositionStart:return M.compositionStart;case k.topCompositionEnd:return M.compositionEnd;case k.topCompositionUpdate:return M.compositionUpdate}}function i(e,t){return e===k.topKeyDown&&t.keyCode===_}function s(e,t){switch(e){case k.topKeyUp:return C.indexOf(t.keyCode)!==-1;case k.topKeyDown:return t.keyCode!==_;case k.topKeyPress:case k.topMouseDown:case k.topBlur:return!0;default:return!1}}function u(e){var t=e.detail;return"object"==typeof t&&"data"in t?t.data:null}function l(e,t,n,r){var o,l;if(E?o=a(e):R?s(e,n)&&(o=M.compositionEnd):i(e,n)&&(o=M.compositionStart),!o)return null;N&&(R||o!==M.compositionStart?o===M.compositionEnd&&R&&(l=R.getData()):R=v.getPooled(r));var c=g.getPooled(o,t,n,r);if(l)c.data=l;else{var p=u(n);null!==p&&(c.data=p)}return h.accumulateTwoPhaseDispatches(c),c}function c(e,t){switch(e){case k.topCompositionEnd:return u(t);case k.topKeyPress:var n=t.which;return n!==w?null:(S=!0,P);case k.topTextInput:var r=t.data;return r===P&&S?null:r;default:return null}}function p(e,t){if(R){if(e===k.topCompositionEnd||s(e,t)){var n=R.getData();return v.release(R),R=null,n}return null}switch(e){case k.topPaste:return null;case k.topKeyPress:return t.which&&!o(t)?String.fromCharCode(t.which):null;case k.topCompositionEnd:return N?null:t.data;default:return null}}function d(e,t,n,r){var o;if(o=T?c(e,n):p(e,n),!o)return null;var a=y.getPooled(M.beforeInput,t,n,r);return a.data=o,h.accumulateTwoPhaseDispatches(a),a}var f=e(16),h=e(20),m=e(140),v=e(21),g=e(95),y=e(99),b=e(158),C=[9,13,27,32],_=229,E=m.canUseDOM&&"CompositionEvent"in window,x=null;m.canUseDOM&&"documentMode"in document&&(x=document.documentMode);var T=m.canUseDOM&&"TextEvent"in window&&!x&&!r(),N=m.canUseDOM&&(!E||x&&x>8&&x<=11),w=32,P=String.fromCharCode(w),k=f.topLevelTypes,M={beforeInput:{phasedRegistrationNames:{bubbled:b({onBeforeInput:null}),captured:b({onBeforeInputCapture:null})},dependencies:[k.topCompositionEnd,k.topKeyPress,k.topTextInput,k.topPaste]},compositionEnd:{phasedRegistrationNames:{bubbled:b({onCompositionEnd:null}),captured:b({onCompositionEndCapture:null})},dependencies:[k.topBlur,k.topCompositionEnd,k.topKeyDown,k.topKeyPress,k.topKeyUp,k.topMouseDown]},compositionStart:{phasedRegistrationNames:{bubbled:b({onCompositionStart:null}),captured:b({onCompositionStartCapture:null})},dependencies:[k.topBlur,k.topCompositionStart,k.topKeyDown,k.topKeyPress,k.topKeyUp,k.topMouseDown]},compositionUpdate:{phasedRegistrationNames:{bubbled:b({onCompositionUpdate:null}),captured:b({onCompositionUpdateCapture:null})},dependencies:[k.topBlur,k.topCompositionUpdate,k.topKeyDown,k.topKeyPress,k.topKeyUp,k.topMouseDown]}},S=!1,R=null,O={eventTypes:M,extractEvents:function(e,t,n,r){return[l(e,t,n,r),d(e,t,n,r)]}};t.exports=O},{140:140,158:158,16:16,20:20,21:21,95:95,99:99}],3:[function(e,t,n){"use strict";function r(e,t){return e+t.charAt(0).toUpperCase()+t.substring(1)}var o={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridRow:!0,gridColumn:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},a=["Webkit","ms","Moz","O"];Object.keys(o).forEach(function(e){a.forEach(function(t){o[r(t,e)]=o[e]})});var i={background:{backgroundAttachment:!0,backgroundColor:!0,backgroundImage:!0,backgroundPositionX:!0,backgroundPositionY:!0,backgroundRepeat:!0},backgroundPosition:{backgroundPositionX:!0,backgroundPositionY:!0},border:{borderWidth:!0,borderStyle:!0,borderColor:!0},borderBottom:{borderBottomWidth:!0,borderBottomStyle:!0,borderBottomColor:!0},borderLeft:{borderLeftWidth:!0,borderLeftStyle:!0,borderLeftColor:!0},borderRight:{borderRightWidth:!0,borderRightStyle:!0,borderRightColor:!0},borderTop:{borderTopWidth:!0,borderTopStyle:!0,borderTopColor:!0},font:{fontStyle:!0,fontVariant:!0,fontWeight:!0,fontSize:!0,lineHeight:!0,fontFamily:!0},outline:{outlineWidth:!0,outlineStyle:!0,outlineColor:!0}},s={isUnitlessNumber:o,shorthandPropertyExpansions:i};t.exports=s},{}],4:[function(e,t,n){"use strict";var r=e(3),o=e(140),a=(e(66),e(142),e(113)),i=e(153),s=e(159),u=(e(161),s(function(e){return i(e)})),l=!1,c="cssFloat";if(o.canUseDOM){var p=document.createElement("div").style;try{p.font=""}catch(e){l=!0}void 0===document.documentElement.style.cssFloat&&(c="styleFloat")}var d={createMarkupForStyles:function(e,t){var n="";for(var r in e)if(e.hasOwnProperty(r)){var o=e[r];null!=o&&(n+=u(r)+":",n+=a(r,o,t)+";")}return n||null},setValueForStyles:function(e,t,n){var o=e.style;for(var i in t)if(t.hasOwnProperty(i)){var s=a(i,t[i],n);if("float"!==i&&"cssFloat"!==i||(i=c),s)o[i]=s;else{var u=l&&r.shorthandPropertyExpansions[i];if(u)for(var p in u)o[p]="";else o[i]=""}}}};t.exports=d},{113:113,140:140,142:142,153:153,159:159,161:161,3:3,66:66}],5:[function(e,t,n){"use strict";function r(){this._callbacks=null,this._contexts=null}var o=e(132),a=e(162),i=e(25);e(154);a(r.prototype,{enqueue:function(e,t){this._callbacks=this._callbacks||[],this._contexts=this._contexts||[],this._callbacks.push(e),this._contexts.push(t)},notifyAll:function(){var e=this._callbacks,t=this._contexts;if(e){e.length!==t.length?o("24"):void 0,this._callbacks=null,this._contexts=null;for(var n=0;n<e.length;n++)e[n].call(t[n]);e.length=0,t.length=0}},checkpoint:function(){return this._callbacks?this._callbacks.length:0},rollback:function(e){this._callbacks&&(this._callbacks.length=e,this._contexts.length=e)},reset:function(){this._callbacks=null,this._contexts=null},destructor:function(){this.reset()}}),i.addPoolingTo(r),t.exports=r},{132:132,154:154,162:162,25:25}],6:[function(e,t,n){"use strict";function r(e){var t=e.nodeName&&e.nodeName.toLowerCase();return"select"===t||"input"===t&&"file"===e.type}function o(e){var t=T.getPooled(S.change,O,e,N(e));C.accumulateTwoPhaseDispatches(t),x.batchedUpdates(a,t)}function a(e){b.enqueueEvents(e),b.processEventQueue(!1)}function i(e,t){R=e,O=t,R.attachEvent("onchange",o)}function s(){R&&(R.detachEvent("onchange",o),R=null,O=null)}function u(e,t){if(e===M.topChange)return t}function l(e,t,n){e===M.topFocus?(s(),i(t,n)):e===M.topBlur&&s()}function c(e,t){R=e,O=t,I=e.value,D=Object.getOwnPropertyDescriptor(e.constructor.prototype,"value"),Object.defineProperty(R,"value",U),R.attachEvent?R.attachEvent("onpropertychange",d):R.addEventListener("propertychange",d,!1)}function p(){R&&(delete R.value,R.detachEvent?R.detachEvent("onpropertychange",d):R.removeEventListener("propertychange",d,!1),R=null,O=null,I=null,D=null)}function d(e){if("value"===e.propertyName){var t=e.srcElement.value;t!==I&&(I=t,o(e))}}function f(e,t){if(e===M.topInput)return t}function h(e,t,n){e===M.topFocus?(p(),c(t,n)):e===M.topBlur&&p()}function m(e,t){if((e===M.topSelectionChange||e===M.topKeyUp||e===M.topKeyDown)&&R&&R.value!==I)return I=R.value,O}function v(e){return e.nodeName&&"input"===e.nodeName.toLowerCase()&&("checkbox"===e.type||"radio"===e.type)}function g(e,t){if(e===M.topClick)return t}var y=e(16),b=e(17),C=e(20),_=e(140),E=e(40),x=e(88),T=e(97),N=e(121),w=e(128),P=e(129),k=e(158),M=y.topLevelTypes,S={change:{phasedRegistrationNames:{bubbled:k({onChange:null}),captured:k({onChangeCapture:null})},dependencies:[M.topBlur,M.topChange,M.topClick,M.topFocus,M.topInput,M.topKeyDown,M.topKeyUp,M.topSelectionChange]}},R=null,O=null,I=null,D=null,A=!1;_.canUseDOM&&(A=w("change")&&(!("documentMode"in document)||document.documentMode>8));var L=!1;_.canUseDOM&&(L=w("input")&&(!("documentMode"in document)||document.documentMode>11));var U={get:function(){return D.get.call(this)},set:function(e){I=""+e,D.set.call(this,e)}},F={eventTypes:S,extractEvents:function(e,t,n,o){var a,i,s=t?E.getNodeFromInstance(t):window;if(r(s)?A?a=u:i=l:P(s)?L?a=f:(a=m,i=h):v(s)&&(a=g),a){var c=a(e,t);if(c){var p=T.getPooled(S.change,c,n,o);return p.type="change",C.accumulateTwoPhaseDispatches(p),p}}i&&i(e,s,t)}};t.exports=F},{121:121,128:128,129:129,140:140,158:158,16:16,17:17,20:20,40:40,88:88,97:97}],7:[function(e,t,n){"use strict";function r(e,t){return Array.isArray(t)&&(t=t[1]),t?t.nextSibling:e.firstChild}function o(e,t,n){c.insertTreeBefore(e,t,n)}function a(e,t,n){Array.isArray(t)?s(e,t[0],t[1],n):v(e,t,n)}function i(e,t){if(Array.isArray(t)){var n=t[1];t=t[0],u(e,t,n),e.removeChild(n)}e.removeChild(t)}function s(e,t,n,r){for(var o=t;;){var a=o.nextSibling;if(v(e,o,r),o===n)break;o=a}}function u(e,t,n){for(;;){var r=t.nextSibling;if(r===n)break;e.removeChild(r)}}function l(e,t,n){var r=e.parentNode,o=e.nextSibling;o===t?n&&v(r,document.createTextNode(n),o):n?(m(o,n),u(r,o,t)):u(r,e,t)}var c=e(8),p=e(12),d=e(70),f=(e(40),e(66),e(112)),h=e(134),m=e(135),v=f(function(e,t,n){e.insertBefore(t,n)}),g=p.dangerouslyReplaceNodeWithMarkup,y={dangerouslyReplaceNodeWithMarkup:g,replaceDelimitedText:l,processUpdates:function(e,t){for(var n=0;n<t.length;n++){var s=t[n];switch(s.type){case d.INSERT_MARKUP:o(e,s.content,r(e,s.afterNode));break;case d.MOVE_EXISTING:a(e,s.fromNode,r(e,s.afterNode));break;case d.SET_MARKUP:h(e,s.content);break;case d.TEXT_CONTENT:m(e,s.content);break;case d.REMOVE_NODE:i(e,s.fromNode)}}}};t.exports=y},{112:112,12:12,134:134,135:135,40:40,66:66,70:70,8:8}],8:[function(e,t,n){"use strict";function r(e){if(v){var t=e.node,n=e.children;if(n.length)for(var r=0;r<n.length;r++)g(t,n[r],null);else null!=e.html?p(t,e.html):null!=e.text&&f(t,e.text)}}function o(e,t){e.parentNode.replaceChild(t.node,e),r(t)}function a(e,t){v?e.children.push(t):e.node.appendChild(t.node)}function i(e,t){v?e.html=t:p(e.node,t)}function s(e,t){v?e.text=t:f(e.node,t)}function u(){return this.node.nodeName}function l(e){return{node:e,children:[],html:null,text:null,toString:u}}var c=e(9),p=e(134),d=e(112),f=e(135),h=1,m=11,v="undefined"!=typeof document&&"number"==typeof document.documentMode||"undefined"!=typeof navigator&&"string"==typeof navigator.userAgent&&/\bEdge\/\d/.test(navigator.userAgent),g=d(function(e,t,n){t.node.nodeType===m||t.node.nodeType===h&&"object"===t.node.nodeName.toLowerCase()&&(null==t.node.namespaceURI||t.node.namespaceURI===c.html)?(r(t),e.insertBefore(t.node,n)):(e.insertBefore(t.node,n),r(t))});l.insertTreeBefore=g,l.replaceChildWithTree=o,l.queueChild=a,l.queueHTML=i,l.queueText=s,t.exports=l},{112:112,134:134,135:135,9:9}],9:[function(e,t,n){"use strict";var r={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg"};t.exports=r},{}],10:[function(e,t,n){"use strict";function r(e,t){return(e&t)===t}var o=e(132),a=(e(154),{MUST_USE_PROPERTY:1,HAS_BOOLEAN_VALUE:4,HAS_NUMERIC_VALUE:8,HAS_POSITIVE_NUMERIC_VALUE:24,HAS_OVERLOADED_BOOLEAN_VALUE:32,injectDOMPropertyConfig:function(e){var t=a,n=e.Properties||{},i=e.DOMAttributeNamespaces||{},u=e.DOMAttributeNames||{},l=e.DOMPropertyNames||{},c=e.DOMMutationMethods||{};e.isCustomAttribute&&s._isCustomAttributeFunctions.push(e.isCustomAttribute);for(var p in n){s.properties.hasOwnProperty(p)?o("48",p):void 0;var d=p.toLowerCase(),f=n[p],h={attributeName:d,attributeNamespace:null,propertyName:p,mutationMethod:null,mustUseProperty:r(f,t.MUST_USE_PROPERTY),hasBooleanValue:r(f,t.HAS_BOOLEAN_VALUE),hasNumericValue:r(f,t.HAS_NUMERIC_VALUE),hasPositiveNumericValue:r(f,t.HAS_POSITIVE_NUMERIC_VALUE),hasOverloadedBooleanValue:r(f,t.HAS_OVERLOADED_BOOLEAN_VALUE)};if(h.hasBooleanValue+h.hasNumericValue+h.hasOverloadedBooleanValue<=1?void 0:o("50",p),u.hasOwnProperty(p)){var m=u[p];h.attributeName=m}i.hasOwnProperty(p)&&(h.attributeNamespace=i[p]),l.hasOwnProperty(p)&&(h.propertyName=l[p]),c.hasOwnProperty(p)&&(h.mutationMethod=c[p]),s.properties[p]=h}}}),i=":A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD",s={ID_ATTRIBUTE_NAME:"data-reactid",ROOT_ATTRIBUTE_NAME:"data-reactroot",ATTRIBUTE_NAME_START_CHAR:i,ATTRIBUTE_NAME_CHAR:i+"\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040",properties:{},getPossibleStandardName:null,_isCustomAttributeFunctions:[],isCustomAttribute:function(e){for(var t=0;t<s._isCustomAttributeFunctions.length;t++){var n=s._isCustomAttributeFunctions[t];if(n(e))return!0}return!1},injection:a};t.exports=s},{132:132,154:154}],11:[function(e,t,n){"use strict";function r(e){return!!l.hasOwnProperty(e)||!u.hasOwnProperty(e)&&(s.test(e)?(l[e]=!0,!0):(u[e]=!0,!1))}function o(e,t){return null==t||e.hasBooleanValue&&!t||e.hasNumericValue&&isNaN(t)||e.hasPositiveNumericValue&&t<1||e.hasOverloadedBooleanValue&&t===!1}var a=e(10),i=(e(40),e(66),e(131)),s=(e(161),new RegExp("^["+a.ATTRIBUTE_NAME_START_CHAR+"]["+a.ATTRIBUTE_NAME_CHAR+"]*$")),u={},l={},c={createMarkupForID:function(e){return a.ID_ATTRIBUTE_NAME+"="+i(e)},setAttributeForID:function(e,t){e.setAttribute(a.ID_ATTRIBUTE_NAME,t)},createMarkupForRoot:function(){return a.ROOT_ATTRIBUTE_NAME+'=""'},setAttributeForRoot:function(e){e.setAttribute(a.ROOT_ATTRIBUTE_NAME,"")},createMarkupForProperty:function(e,t){var n=a.properties.hasOwnProperty(e)?a.properties[e]:null;if(n){if(o(n,t))return"";var r=n.attributeName;return n.hasBooleanValue||n.hasOverloadedBooleanValue&&t===!0?r+'=""':r+"="+i(t)}return a.isCustomAttribute(e)?null==t?"":e+"="+i(t):null},createMarkupForCustomAttribute:function(e,t){return r(e)&&null!=t?e+"="+i(t):""},setValueForProperty:function(e,t,n){var r=a.properties.hasOwnProperty(t)?a.properties[t]:null;if(r){var i=r.mutationMethod;if(i)i(e,n);else{if(o(r,n))return void this.deleteValueForProperty(e,t);if(r.mustUseProperty)e[r.propertyName]=n;else{var s=r.attributeName,u=r.attributeNamespace;u?e.setAttributeNS(u,s,""+n):r.hasBooleanValue||r.hasOverloadedBooleanValue&&n===!0?e.setAttribute(s,""):e.setAttribute(s,""+n)}}}else if(a.isCustomAttribute(t))return void c.setValueForAttribute(e,t,n)},setValueForAttribute:function(e,t,n){r(t)&&(null==n?e.removeAttribute(t):e.setAttribute(t,""+n))},deleteValueForAttribute:function(e,t){e.removeAttribute(t)},deleteValueForProperty:function(e,t){var n=a.properties.hasOwnProperty(t)?a.properties[t]:null;if(n){var r=n.mutationMethod;if(r)r(e,void 0);else if(n.mustUseProperty){var o=n.propertyName;n.hasBooleanValue?e[o]=!1:e[o]=""}else e.removeAttribute(n.attributeName)}else a.isCustomAttribute(t)&&e.removeAttribute(t)}};t.exports=c},{10:10,131:131,161:161,40:40,66:66}],12:[function(e,t,n){"use strict";var r=e(132),o=e(8),a=e(140),i=e(145),s=e(146),u=(e(154),{dangerouslyReplaceNodeWithMarkup:function(e,t){if(a.canUseDOM?void 0:r("56"),t?void 0:r("57"),"HTML"===e.nodeName?r("58"):void 0,"string"==typeof t){var n=i(t,s)[0];e.parentNode.replaceChild(n,e)}else o.replaceChildWithTree(e,t)}});t.exports=u},{132:132,140:140,145:145,146:146,154:154,8:8}],13:[function(e,t,n){"use strict";var r=e(158),o=[r({ResponderEventPlugin:null}),r({SimpleEventPlugin:null}),r({TapEventPlugin:null}),r({EnterLeaveEventPlugin:null}),r({ChangeEventPlugin:null}),r({SelectEventPlugin:null}),r({BeforeInputEventPlugin:null})];t.exports=o},{158:158}],14:[function(e,t,n){"use strict";var r={onClick:!0,onDoubleClick:!0,onMouseDown:!0,onMouseMove:!0,onMouseUp:!0,onClickCapture:!0,onDoubleClickCapture:!0,onMouseDownCapture:!0,onMouseMoveCapture:!0,onMouseUpCapture:!0},o={getHostProps:function(e,t){if(!t.disabled)return t;var n={};for(var o in t)!r[o]&&t.hasOwnProperty(o)&&(n[o]=t[o]);return n}};t.exports=o},{}],15:[function(e,t,n){"use strict";var r=e(16),o=e(20),a=e(40),i=e(101),s=e(158),u=r.topLevelTypes,l={mouseEnter:{registrationName:s({onMouseEnter:null}),dependencies:[u.topMouseOut,u.topMouseOver]},mouseLeave:{registrationName:s({onMouseLeave:null}),dependencies:[u.topMouseOut,u.topMouseOver]}},c={eventTypes:l,extractEvents:function(e,t,n,r){if(e===u.topMouseOver&&(n.relatedTarget||n.fromElement))return null;if(e!==u.topMouseOut&&e!==u.topMouseOver)return null;var s;if(r.window===r)s=r;else{var c=r.ownerDocument;s=c?c.defaultView||c.parentWindow:window}var p,d;if(e===u.topMouseOut){p=t;var f=n.relatedTarget||n.toElement;d=f?a.getClosestInstanceFromNode(f):null}else p=null,d=t;if(p===d)return null;var h=null==p?s:a.getNodeFromInstance(p),m=null==d?s:a.getNodeFromInstance(d),v=i.getPooled(l.mouseLeave,p,n,r);v.type="mouseleave",v.target=h,v.relatedTarget=m;var g=i.getPooled(l.mouseEnter,d,n,r);return g.type="mouseenter",g.target=m,g.relatedTarget=h,o.accumulateEnterLeaveDispatches(v,g,p,d),[v,g]}};t.exports=c},{101:101,158:158,16:16,20:20,40:40}],16:[function(e,t,n){"use strict";var r=e(157),o=r({bubbled:null,captured:null}),a=r({topAbort:null,topAnimationEnd:null,topAnimationIteration:null,topAnimationStart:null,topBlur:null,topCanPlay:null,topCanPlayThrough:null,topChange:null,topClick:null,topCompositionEnd:null,topCompositionStart:null,topCompositionUpdate:null,topContextMenu:null,topCopy:null,topCut:null,topDoubleClick:null,topDrag:null,topDragEnd:null,topDragEnter:null,topDragExit:null,topDragLeave:null,topDragOver:null,topDragStart:null,topDrop:null,topDurationChange:null,topEmptied:null,topEncrypted:null,topEnded:null,topError:null,topFocus:null,topInput:null,topInvalid:null,topKeyDown:null,topKeyPress:null,topKeyUp:null,topLoad:null,topLoadedData:null,topLoadedMetadata:null,topLoadStart:null,topMouseDown:null,topMouseMove:null,topMouseOut:null,topMouseOver:null,topMouseUp:null,topPaste:null,topPause:null,topPlay:null,topPlaying:null,topProgress:null,topRateChange:null,topReset:null,topScroll:null,topSeeked:null,topSeeking:null,topSelectionChange:null,topStalled:null,topSubmit:null,topSuspend:null,topTextInput:null,topTimeUpdate:null,topTouchCancel:null,topTouchEnd:null,topTouchMove:null,topTouchStart:null,topTransitionEnd:null,topVolumeChange:null,topWaiting:null,topWheel:null}),i={topLevelTypes:a,PropagationPhases:o};t.exports=i},{157:157}],17:[function(e,t,n){"use strict";var r=e(132),o=e(18),a=e(19),i=e(58),s=e(108),u=e(117),l=(e(154),{}),c=null,p=function(e,t){e&&(a.executeDispatchesInOrder(e,t),e.isPersistent()||e.constructor.release(e))},d=function(e){return p(e,!0)},f=function(e){return p(e,!1)},h=function(e){return"."+e._rootNodeID},m={injection:{injectEventPluginOrder:o.injectEventPluginOrder,injectEventPluginsByName:o.injectEventPluginsByName},putListener:function(e,t,n){"function"!=typeof n?r("94",t,typeof n):void 0;var a=h(e),i=l[t]||(l[t]={});i[a]=n;var s=o.registrationNameModules[t];s&&s.didPutListener&&s.didPutListener(e,t,n)},getListener:function(e,t){var n=l[t],r=h(e);return n&&n[r]},deleteListener:function(e,t){var n=o.registrationNameModules[t];n&&n.willDeleteListener&&n.willDeleteListener(e,t);var r=l[t];if(r){var a=h(e);delete r[a]}},deleteAllListeners:function(e){var t=h(e);for(var n in l)if(l.hasOwnProperty(n)&&l[n][t]){var r=o.registrationNameModules[n];r&&r.willDeleteListener&&r.willDeleteListener(e,n),delete l[n][t]}},extractEvents:function(e,t,n,r){for(var a,i=o.plugins,u=0;u<i.length;u++){var l=i[u];if(l){var c=l.extractEvents(e,t,n,r);c&&(a=s(a,c))}}return a},enqueueEvents:function(e){e&&(c=s(c,e))},processEventQueue:function(e){var t=c;c=null,e?u(t,d):u(t,f),c?r("95"):void 0,i.rethrowCaughtError()},__purge:function(){l={}},__getListenerBank:function(){return l}};t.exports=m},{108:108,117:117,132:132,154:154,18:18,19:19,58:58}],18:[function(e,t,n){"use strict";function r(){if(s)for(var e in u){var t=u[e],n=s.indexOf(e);if(n>-1?void 0:i("96",e),!l.plugins[n]){t.extractEvents?void 0:i("97",e),l.plugins[n]=t;var r=t.eventTypes;for(var a in r)o(r[a],t,a)?void 0:i("98",a,e)}}}function o(e,t,n){l.eventNameDispatchConfigs.hasOwnProperty(n)?i("99",n):void 0,l.eventNameDispatchConfigs[n]=e;var r=e.phasedRegistrationNames;if(r){for(var o in r)if(r.hasOwnProperty(o)){var s=r[o];a(s,t,n)}return!0}return!!e.registrationName&&(a(e.registrationName,t,n),!0)}function a(e,t,n){l.registrationNameModules[e]?i("100",e):void 0,l.registrationNameModules[e]=t,l.registrationNameDependencies[e]=t.eventTypes[n].dependencies}var i=e(132),s=(e(154),null),u={},l={plugins:[],eventNameDispatchConfigs:{},registrationNameModules:{},registrationNameDependencies:{},possibleRegistrationNames:null,injectEventPluginOrder:function(e){s?i("101"):void 0,s=Array.prototype.slice.call(e),r()},injectEventPluginsByName:function(e){var t=!1;for(var n in e)if(e.hasOwnProperty(n)){var o=e[n];u.hasOwnProperty(n)&&u[n]===o||(u[n]?i("102",n):void 0,u[n]=o,t=!0)}t&&r()},getPluginModuleForEvent:function(e){var t=e.dispatchConfig;if(t.registrationName)return l.registrationNameModules[t.registrationName]||null;for(var n in t.phasedRegistrationNames)if(t.phasedRegistrationNames.hasOwnProperty(n)){var r=l.registrationNameModules[t.phasedRegistrationNames[n]];if(r)return r}return null},_resetEventPlugins:function(){s=null;for(var e in u)u.hasOwnProperty(e)&&delete u[e];l.plugins.length=0;var t=l.eventNameDispatchConfigs;for(var n in t)t.hasOwnProperty(n)&&delete t[n];var r=l.registrationNameModules;for(var o in r)r.hasOwnProperty(o)&&delete r[o]}};t.exports=l},{132:132,154:154}],19:[function(e,t,n){"use strict";function r(e){return e===y.topMouseUp||e===y.topTouchEnd||e===y.topTouchCancel}function o(e){return e===y.topMouseMove||e===y.topTouchMove}function a(e){return e===y.topMouseDown||e===y.topTouchStart}function i(e,t,n,r){var o=e.type||"unknown-event";e.currentTarget=b.getNodeFromInstance(r),t?v.invokeGuardedCallbackWithCatch(o,n,e):v.invokeGuardedCallback(o,n,e),e.currentTarget=null}function s(e,t){var n=e._dispatchListeners,r=e._dispatchInstances;if(Array.isArray(n))for(var o=0;o<n.length&&!e.isPropagationStopped();o++)i(e,t,n[o],r[o]);else n&&i(e,t,n,r);e._dispatchListeners=null,e._dispatchInstances=null}function u(e){var t=e._dispatchListeners,n=e._dispatchInstances;if(Array.isArray(t)){for(var r=0;r<t.length&&!e.isPropagationStopped();r++)if(t[r](e,n[r]))return n[r]}else if(t&&t(e,n))return n;return null}function l(e){var t=u(e);return e._dispatchInstances=null,e._dispatchListeners=null,t}function c(e){var t=e._dispatchListeners,n=e._dispatchInstances;Array.isArray(t)?h("103"):void 0,e.currentTarget=t?b.getNodeFromInstance(n):null;var r=t?t(e):null;return e.currentTarget=null,e._dispatchListeners=null,e._dispatchInstances=null,r}function p(e){return!!e._dispatchListeners}var d,f,h=e(132),m=e(16),v=e(58),g=(e(154),e(161),{injectComponentTree:function(e){d=e},injectTreeTraversal:function(e){f=e}}),y=m.topLevelTypes,b={isEndish:r,isMoveish:o,isStartish:a,executeDirectDispatch:c,executeDispatchesInOrder:s,executeDispatchesInOrderStopAtTrue:l,hasDispatches:p,getInstanceFromNode:function(e){return d.getInstanceFromNode(e)},getNodeFromInstance:function(e){return d.getNodeFromInstance(e)},isAncestor:function(e,t){return f.isAncestor(e,t)},getLowestCommonAncestor:function(e,t){return f.getLowestCommonAncestor(e,t)},getParentInstance:function(e){return f.getParentInstance(e)},traverseTwoPhase:function(e,t,n){return f.traverseTwoPhase(e,t,n)},traverseEnterLeave:function(e,t,n,r,o){return f.traverseEnterLeave(e,t,n,r,o)},injection:g};t.exports=b},{132:132,154:154,16:16,161:161,58:58}],20:[function(e,t,n){"use strict";function r(e,t,n){var r=t.dispatchConfig.phasedRegistrationNames[n];return b(e,r)}function o(e,t,n){var o=t?y.bubbled:y.captured,a=r(e,n,o);a&&(n._dispatchListeners=v(n._dispatchListeners,a),n._dispatchInstances=v(n._dispatchInstances,e))}function a(e){e&&e.dispatchConfig.phasedRegistrationNames&&m.traverseTwoPhase(e._targetInst,o,e)}function i(e){if(e&&e.dispatchConfig.phasedRegistrationNames){var t=e._targetInst,n=t?m.getParentInstance(t):null;m.traverseTwoPhase(n,o,e)}}function s(e,t,n){if(n&&n.dispatchConfig.registrationName){var r=n.dispatchConfig.registrationName,o=b(e,r);o&&(n._dispatchListeners=v(n._dispatchListeners,o),n._dispatchInstances=v(n._dispatchInstances,e))}}function u(e){e&&e.dispatchConfig.registrationName&&s(e._targetInst,null,e)}function l(e){g(e,a)}function c(e){g(e,i)}function p(e,t,n,r){m.traverseEnterLeave(n,r,s,e,t)}function d(e){g(e,u)}var f=e(16),h=e(17),m=e(19),v=e(108),g=e(117),y=(e(161),f.PropagationPhases),b=h.getListener,C={accumulateTwoPhaseDispatches:l,accumulateTwoPhaseDispatchesSkipTarget:c,accumulateDirectDispatches:d,accumulateEnterLeaveDispatches:p};t.exports=C},{108:108,117:117,16:16,161:161,17:17,19:19}],21:[function(e,t,n){"use strict";function r(e){this._root=e,this._startText=this.getText(),this._fallbackText=null}var o=e(162),a=e(25),i=e(125);o(r.prototype,{destructor:function(){this._root=null,this._startText=null,this._fallbackText=null},getText:function(){return"value"in this._root?this._root.value:this._root[i()]},getData:function(){if(this._fallbackText)return this._fallbackText;var e,t,n=this._startText,r=n.length,o=this.getText(),a=o.length;for(e=0;e<r&&n[e]===o[e];e++);var i=r-e;for(t=1;t<=i&&n[r-t]===o[a-t];t++);var s=t>1?1-t:void 0;return this._fallbackText=o.slice(e,s),this._fallbackText}}),a.addPoolingTo(r),t.exports=r},{125:125,162:162,25:25}],22:[function(e,t,n){"use strict";var r=e(10),o=r.injection.MUST_USE_PROPERTY,a=r.injection.HAS_BOOLEAN_VALUE,i=r.injection.HAS_NUMERIC_VALUE,s=r.injection.HAS_POSITIVE_NUMERIC_VALUE,u=r.injection.HAS_OVERLOADED_BOOLEAN_VALUE,l={isCustomAttribute:RegExp.prototype.test.bind(new RegExp("^(data|aria)-["+r.ATTRIBUTE_NAME_CHAR+"]*$")),Properties:{accept:0,acceptCharset:0,accessKey:0,action:0,allowFullScreen:a,allowTransparency:0,alt:0,async:a,autoComplete:0,autoPlay:a,capture:a,cellPadding:0,cellSpacing:0,charSet:0,challenge:0,checked:o|a,cite:0,classID:0,className:0,cols:s,colSpan:0,content:0,contentEditable:0,contextMenu:0,controls:a,coords:0,crossOrigin:0,data:0,dateTime:0,default:a,defer:a,dir:0,disabled:a,download:u,draggable:0,encType:0,form:0,formAction:0,formEncType:0,formMethod:0,formNoValidate:a,formTarget:0,frameBorder:0,headers:0,height:0,hidden:a,high:0,href:0,hrefLang:0,htmlFor:0,httpEquiv:0,icon:0,id:0,inputMode:0,integrity:0,is:0,keyParams:0,keyType:0,kind:0,label:0,lang:0,list:0,loop:a,low:0,manifest:0,marginHeight:0,marginWidth:0,max:0,maxLength:0,media:0,mediaGroup:0,method:0,min:0,minLength:0,multiple:o|a,muted:o|a,name:0,nonce:0,noValidate:a,open:a,optimum:0,pattern:0,placeholder:0,poster:0,preload:0,profile:0,radioGroup:0,readOnly:a,referrerPolicy:0,rel:0,required:a,reversed:a,role:0,rows:s,rowSpan:i,sandbox:0,scope:0,scoped:a,scrolling:0,seamless:a,selected:o|a,shape:0,size:s,sizes:0,span:s,spellCheck:0,src:0,srcDoc:0,srcLang:0,srcSet:0,start:i,step:0,style:0,summary:0,tabIndex:0,target:0,title:0,type:0,useMap:0,value:0,width:0,wmode:0,wrap:0,about:0,datatype:0,inlist:0,prefix:0,property:0,resource:0,typeof:0,vocab:0,autoCapitalize:0,autoCorrect:0,autoSave:0,color:0,itemProp:0,itemScope:a,itemType:0,itemID:0,itemRef:0,results:0,security:0,unselectable:0},DOMAttributeNames:{acceptCharset:"accept-charset",className:"class",htmlFor:"for",httpEquiv:"http-equiv"},DOMPropertyNames:{}};t.exports=l},{10:10}],23:[function(e,t,n){"use strict";function r(e){var t=/[=:]/g,n={"=":"=0",":":"=2"},r=(""+e).replace(t,function(e){return n[e]});return"$"+r}function o(e){var t=/(=0|=2)/g,n={"=0":"=","=2":":"},r="."===e[0]&&"$"===e[1]?e.substring(2):e.substring(1);return(""+r).replace(t,function(e){return n[e]})}var a={escape:r,unescape:o};t.exports=a},{}],24:[function(e,t,n){"use strict";function r(e){null!=e.checkedLink&&null!=e.valueLink?s("87"):void 0}function o(e){r(e),null!=e.value||null!=e.onChange?s("88"):void 0}function a(e){r(e),null!=e.checked||null!=e.onChange?s("89"):void 0}function i(e){if(e){var t=e.getName();if(t)return" Check the render method of `"+t+"`."}return""}var s=e(132),u=e(76),l=e(75),c=e(77),p=(e(154),e(161),{button:!0,checkbox:!0,image:!0,hidden:!0,radio:!0,reset:!0,submit:!0}),d={value:function(e,t,n){return!e[t]||p[e.type]||e.onChange||e.readOnly||e.disabled?null:new Error("You provided a `value` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultValue`. Otherwise, set either `onChange` or `readOnly`.")},checked:function(e,t,n){return!e[t]||e.onChange||e.readOnly||e.disabled?null:new Error("You provided a `checked` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultChecked`. Otherwise, set either `onChange` or `readOnly`.")},onChange:u.func},f={},h={checkPropTypes:function(e,t,n){for(var r in d){if(d.hasOwnProperty(r))var o=d[r](t,r,e,l.prop,null,c);o instanceof Error&&!(o.message in f)&&(f[o.message]=!0,i(n))}},getValue:function(e){return e.valueLink?(o(e),e.valueLink.value):e.value},getChecked:function(e){return e.checkedLink?(a(e),e.checkedLink.value):e.checked},executeOnChange:function(e,t){return e.valueLink?(o(e),e.valueLink.requestChange(t.target.value)):e.checkedLink?(a(e),e.checkedLink.requestChange(t.target.checked)):e.onChange?e.onChange.call(void 0,t):void 0}};t.exports=h},{132:132,154:154,161:161,75:75,76:76,77:77}],25:[function(e,t,n){"use strict";var r=e(132),o=(e(154),function(e){var t=this;if(t.instancePool.length){var n=t.instancePool.pop();return t.call(n,e),n}return new t(e)}),a=function(e,t){var n=this;if(n.instancePool.length){var r=n.instancePool.pop();return n.call(r,e,t),r}return new n(e,t)},i=function(e,t,n){var r=this;if(r.instancePool.length){var o=r.instancePool.pop();return r.call(o,e,t,n),o}return new r(e,t,n)},s=function(e,t,n,r){var o=this;if(o.instancePool.length){var a=o.instancePool.pop();return o.call(a,e,t,n,r),a}return new o(e,t,n,r)},u=function(e,t,n,r,o){var a=this;if(a.instancePool.length){var i=a.instancePool.pop();return a.call(i,e,t,n,r,o),i}return new a(e,t,n,r,o)},l=function(e){var t=this;e instanceof t?void 0:r("25"),e.destructor(),t.instancePool.length<t.poolSize&&t.instancePool.push(e)},c=10,p=o,d=function(e,t){var n=e;return n.instancePool=[],n.getPooled=t||p,n.poolSize||(n.poolSize=c),n.release=l,n},f={addPoolingTo:d,oneArgumentPooler:o,twoArgumentPooler:a,threeArgumentPooler:i,fourArgumentPooler:s,fiveArgumentPooler:u};t.exports=f},{132:132,154:154}],26:[function(e,t,n){"use strict";var r=e(162),o=e(29),a=e(31),i=e(78),s=e(30),u=e(43),l=e(56),c=e(76),p=e(89),d=e(130),f=(e(161),l.createElement),h=l.createFactory,m=l.cloneElement,v=r,g={Children:{map:o.map,forEach:o.forEach,count:o.count,toArray:o.toArray,only:d},Component:a,PureComponent:i,createElement:f,cloneElement:m,isValidElement:l.isValidElement,PropTypes:c,createClass:s.createClass,createFactory:h,createMixin:function(e){return e},DOM:u,version:p,__spread:v};t.exports=g},{130:130,161:161,162:162,29:29,30:30,31:31,43:43,56:56,76:76,78:78,89:89}],27:[function(e,t,n){"use strict";function r(e){return Object.prototype.hasOwnProperty.call(e,v)||(e[v]=h++,d[e[v]]={}),d[e[v]]}var o,a=e(162),i=e(16),s=e(18),u=e(59),l=e(107),c=e(126),p=e(128),d={},f=!1,h=0,m={
topAbort:"abort",topAnimationEnd:c("animationend")||"animationend",topAnimationIteration:c("animationiteration")||"animationiteration",topAnimationStart:c("animationstart")||"animationstart",topBlur:"blur",topCanPlay:"canplay",topCanPlayThrough:"canplaythrough",topChange:"change",topClick:"click",topCompositionEnd:"compositionend",topCompositionStart:"compositionstart",topCompositionUpdate:"compositionupdate",topContextMenu:"contextmenu",topCopy:"copy",topCut:"cut",topDoubleClick:"dblclick",topDrag:"drag",topDragEnd:"dragend",topDragEnter:"dragenter",topDragExit:"dragexit",topDragLeave:"dragleave",topDragOver:"dragover",topDragStart:"dragstart",topDrop:"drop",topDurationChange:"durationchange",topEmptied:"emptied",topEncrypted:"encrypted",topEnded:"ended",topError:"error",topFocus:"focus",topInput:"input",topKeyDown:"keydown",topKeyPress:"keypress",topKeyUp:"keyup",topLoadedData:"loadeddata",topLoadedMetadata:"loadedmetadata",topLoadStart:"loadstart",topMouseDown:"mousedown",topMouseMove:"mousemove",topMouseOut:"mouseout",topMouseOver:"mouseover",topMouseUp:"mouseup",topPaste:"paste",topPause:"pause",topPlay:"play",topPlaying:"playing",topProgress:"progress",topRateChange:"ratechange",topScroll:"scroll",topSeeked:"seeked",topSeeking:"seeking",topSelectionChange:"selectionchange",topStalled:"stalled",topSuspend:"suspend",topTextInput:"textInput",topTimeUpdate:"timeupdate",topTouchCancel:"touchcancel",topTouchEnd:"touchend",topTouchMove:"touchmove",topTouchStart:"touchstart",topTransitionEnd:c("transitionend")||"transitionend",topVolumeChange:"volumechange",topWaiting:"waiting",topWheel:"wheel"},v="_reactListenersID"+String(Math.random()).slice(2),g=a({},u,{ReactEventListener:null,injection:{injectReactEventListener:function(e){e.setHandleTopLevel(g.handleTopLevel),g.ReactEventListener=e}},setEnabled:function(e){g.ReactEventListener&&g.ReactEventListener.setEnabled(e)},isEnabled:function(){return!(!g.ReactEventListener||!g.ReactEventListener.isEnabled())},listenTo:function(e,t){for(var n=t,o=r(n),a=s.registrationNameDependencies[e],u=i.topLevelTypes,l=0;l<a.length;l++){var c=a[l];o.hasOwnProperty(c)&&o[c]||(c===u.topWheel?p("wheel")?g.ReactEventListener.trapBubbledEvent(u.topWheel,"wheel",n):p("mousewheel")?g.ReactEventListener.trapBubbledEvent(u.topWheel,"mousewheel",n):g.ReactEventListener.trapBubbledEvent(u.topWheel,"DOMMouseScroll",n):c===u.topScroll?p("scroll",!0)?g.ReactEventListener.trapCapturedEvent(u.topScroll,"scroll",n):g.ReactEventListener.trapBubbledEvent(u.topScroll,"scroll",g.ReactEventListener.WINDOW_HANDLE):c===u.topFocus||c===u.topBlur?(p("focus",!0)?(g.ReactEventListener.trapCapturedEvent(u.topFocus,"focus",n),g.ReactEventListener.trapCapturedEvent(u.topBlur,"blur",n)):p("focusin")&&(g.ReactEventListener.trapBubbledEvent(u.topFocus,"focusin",n),g.ReactEventListener.trapBubbledEvent(u.topBlur,"focusout",n)),o[u.topBlur]=!0,o[u.topFocus]=!0):m.hasOwnProperty(c)&&g.ReactEventListener.trapBubbledEvent(c,m[c],n),o[c]=!0)}},trapBubbledEvent:function(e,t,n){return g.ReactEventListener.trapBubbledEvent(e,t,n)},trapCapturedEvent:function(e,t,n){return g.ReactEventListener.trapCapturedEvent(e,t,n)},ensureScrollValueMonitoring:function(){if(void 0===o&&(o=document.createEvent&&"pageX"in document.createEvent("MouseEvent")),!o&&!f){var e=l.refreshScrollValues;g.ReactEventListener.monitorScrollValue(e),f=!0}}});t.exports=g},{107:107,126:126,128:128,16:16,162:162,18:18,59:59}],28:[function(e,t,n){(function(n){"use strict";function r(e,t,n,r){var o=void 0===e[n];null!=t&&o&&(e[n]=a(t,!0))}var o=e(80),a=e(127),i=(e(23),e(136)),s=e(137);e(161);"undefined"!=typeof n&&n.env,1;var u={instantiateChildren:function(e,t,n,o){if(null==e)return null;var a={};return s(e,r,a),a},updateChildren:function(e,t,n,r,s,u,l,c,p){if(t||e){var d,f;for(d in t)if(t.hasOwnProperty(d)){f=e&&e[d];var h=f&&f._currentElement,m=t[d];if(null!=f&&i(h,m))o.receiveComponent(f,m,s,c),t[d]=f;else{f&&(r[d]=o.getHostNode(f),o.unmountComponent(f,!1));var v=a(m,!0);t[d]=v;var g=o.mountComponent(v,s,u,l,c,p);n.push(g)}}for(d in e)!e.hasOwnProperty(d)||t&&t.hasOwnProperty(d)||(f=e[d],r[d]=o.getHostNode(f),o.unmountComponent(f,!1))}},unmountChildren:function(e,t){for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];o.unmountComponent(r,t)}}};t.exports=u}).call(this,void 0)},{127:127,136:136,137:137,161:161,23:23,80:80}],29:[function(e,t,n){"use strict";function r(e){return(""+e).replace(C,"$&/")}function o(e,t){this.func=e,this.context=t,this.count=0}function a(e,t,n){var r=e.func,o=e.context;r.call(o,t,e.count++)}function i(e,t,n){if(null==e)return e;var r=o.getPooled(t,n);g(e,a,r),o.release(r)}function s(e,t,n,r){this.result=e,this.keyPrefix=t,this.func=n,this.context=r,this.count=0}function u(e,t,n){var o=e.result,a=e.keyPrefix,i=e.func,s=e.context,u=i.call(s,t,e.count++);Array.isArray(u)?l(u,o,n,v.thatReturnsArgument):null!=u&&(m.isValidElement(u)&&(u=m.cloneAndReplaceKey(u,a+(!u.key||t&&t.key===u.key?"":r(u.key)+"/")+n)),o.push(u))}function l(e,t,n,o,a){var i="";null!=n&&(i=r(n)+"/");var l=s.getPooled(t,i,o,a);g(e,u,l),s.release(l)}function c(e,t,n){if(null==e)return e;var r=[];return l(e,r,null,t,n),r}function p(e,t,n){return null}function d(e,t){return g(e,p,null)}function f(e){var t=[];return l(e,t,null,v.thatReturnsArgument),t}var h=e(25),m=e(56),v=e(146),g=e(137),y=h.twoArgumentPooler,b=h.fourArgumentPooler,C=/\/+/g;o.prototype.destructor=function(){this.func=null,this.context=null,this.count=0},h.addPoolingTo(o,y),s.prototype.destructor=function(){this.result=null,this.keyPrefix=null,this.func=null,this.context=null,this.count=0},h.addPoolingTo(s,b);var _={forEach:i,map:c,mapIntoWithKeyPrefixInternal:l,count:d,toArray:f};t.exports=_},{137:137,146:146,25:25,56:56}],30:[function(e,t,n){"use strict";function r(e,t){var n=E.hasOwnProperty(t)?E[t]:null;T.hasOwnProperty(t)&&(n!==C.OVERRIDE_BASE?p("73",t):void 0),e&&(n!==C.DEFINE_MANY&&n!==C.DEFINE_MANY_MERGED?p("74",t):void 0)}function o(e,t){if(t){"function"==typeof t?p("75"):void 0,h.isValidElement(t)?p("76"):void 0;var n=e.prototype,o=n.__reactAutoBindPairs;t.hasOwnProperty(b)&&x.mixins(e,t.mixins);for(var a in t)if(t.hasOwnProperty(a)&&a!==b){var i=t[a],l=n.hasOwnProperty(a);if(r(l,a),x.hasOwnProperty(a))x[a](e,i);else{var c=E.hasOwnProperty(a),d="function"==typeof i,f=d&&!c&&!l&&t.autobind!==!1;if(f)o.push(a,i),n[a]=i;else if(l){var m=E[a];!c||m!==C.DEFINE_MANY_MERGED&&m!==C.DEFINE_MANY?p("77",m,a):void 0,m===C.DEFINE_MANY_MERGED?n[a]=s(n[a],i):m===C.DEFINE_MANY&&(n[a]=u(n[a],i))}else n[a]=i}}}}function a(e,t){if(t)for(var n in t){var r=t[n];if(t.hasOwnProperty(n)){var o=n in x;o?p("78",n):void 0;var a=n in e;a?p("79",n):void 0,e[n]=r}}}function i(e,t){e&&t&&"object"==typeof e&&"object"==typeof t?void 0:p("80");for(var n in t)t.hasOwnProperty(n)&&(void 0!==e[n]?p("81",n):void 0,e[n]=t[n]);return e}function s(e,t){return function(){var n=e.apply(this,arguments),r=t.apply(this,arguments);if(null==n)return r;if(null==r)return n;var o={};return i(o,n),i(o,r),o}}function u(e,t){return function(){e.apply(this,arguments),t.apply(this,arguments)}}function l(e,t){var n=t.bind(e);return n}function c(e){for(var t=e.__reactAutoBindPairs,n=0;n<t.length;n+=2){var r=t[n],o=t[n+1];e[r]=l(e,o)}}var p=e(132),d=e(162),f=e(31),h=e(56),m=(e(75),e(74),e(72)),v=e(147),g=(e(154),e(157)),y=e(158),b=(e(161),y({mixins:null})),C=g({DEFINE_ONCE:null,DEFINE_MANY:null,OVERRIDE_BASE:null,DEFINE_MANY_MERGED:null}),_=[],E={mixins:C.DEFINE_MANY,statics:C.DEFINE_MANY,propTypes:C.DEFINE_MANY,contextTypes:C.DEFINE_MANY,childContextTypes:C.DEFINE_MANY,getDefaultProps:C.DEFINE_MANY_MERGED,getInitialState:C.DEFINE_MANY_MERGED,getChildContext:C.DEFINE_MANY_MERGED,render:C.DEFINE_ONCE,componentWillMount:C.DEFINE_MANY,componentDidMount:C.DEFINE_MANY,componentWillReceiveProps:C.DEFINE_MANY,shouldComponentUpdate:C.DEFINE_ONCE,componentWillUpdate:C.DEFINE_MANY,componentDidUpdate:C.DEFINE_MANY,componentWillUnmount:C.DEFINE_MANY,updateComponent:C.OVERRIDE_BASE},x={displayName:function(e,t){e.displayName=t},mixins:function(e,t){if(t)for(var n=0;n<t.length;n++)o(e,t[n])},childContextTypes:function(e,t){e.childContextTypes=d({},e.childContextTypes,t)},contextTypes:function(e,t){e.contextTypes=d({},e.contextTypes,t)},getDefaultProps:function(e,t){e.getDefaultProps?e.getDefaultProps=s(e.getDefaultProps,t):e.getDefaultProps=t},propTypes:function(e,t){e.propTypes=d({},e.propTypes,t)},statics:function(e,t){a(e,t)},autobind:function(){}},T={replaceState:function(e,t){this.updater.enqueueReplaceState(this,e),t&&this.updater.enqueueCallback(this,t,"replaceState")},isMounted:function(){return this.updater.isMounted(this)}},N=function(){};d(N.prototype,f.prototype,T);var w={createClass:function(e){var t=function(e,n,r){this.__reactAutoBindPairs.length&&c(this),this.props=e,this.context=n,this.refs=v,this.updater=r||m,this.state=null;var o=this.getInitialState?this.getInitialState():null;"object"!=typeof o||Array.isArray(o)?p("82",t.displayName||"ReactCompositeComponent"):void 0,this.state=o};t.prototype=new N,t.prototype.constructor=t,t.prototype.__reactAutoBindPairs=[],_.forEach(o.bind(null,t)),o(t,e),t.getDefaultProps&&(t.defaultProps=t.getDefaultProps()),t.prototype.render?void 0:p("83");for(var n in E)t.prototype[n]||(t.prototype[n]=null);return t},injection:{injectMixin:function(e){_.push(e)}}};t.exports=w},{132:132,147:147,154:154,157:157,158:158,161:161,162:162,31:31,56:56,72:72,74:74,75:75}],31:[function(e,t,n){"use strict";function r(e,t,n){this.props=e,this.context=t,this.refs=i,this.updater=n||a}var o=e(132),a=e(72),i=(e(110),e(147));e(154),e(161);r.prototype.isReactComponent={},r.prototype.setState=function(e,t){"object"!=typeof e&&"function"!=typeof e&&null!=e?o("85"):void 0,this.updater.enqueueSetState(this,e),t&&this.updater.enqueueCallback(this,t,"setState")},r.prototype.forceUpdate=function(e){this.updater.enqueueForceUpdate(this),e&&this.updater.enqueueCallback(this,e,"forceUpdate")};t.exports=r},{110:110,132:132,147:147,154:154,161:161,72:72}],32:[function(e,t,n){"use strict";var r=e(7),o=e(45),a={processChildrenUpdates:o.dangerouslyProcessChildrenUpdates,replaceNodeWithMarkup:r.dangerouslyReplaceNodeWithMarkup};t.exports=a},{45:45,7:7}],33:[function(e,t,n){"use strict";var r=e(132),o=(e(154),!1),a={replaceNodeWithMarkup:null,processChildrenUpdates:null,injection:{injectEnvironment:function(e){o?r("104"):void 0,a.replaceNodeWithMarkup=e.replaceNodeWithMarkup,a.processChildrenUpdates=e.processChildrenUpdates,o=!0}}};t.exports=a},{132:132,154:154}],34:[function(e,t,n){"use strict";function r(e){}function o(e,t){}function a(e){return!(!e.prototype||!e.prototype.isReactComponent)}function i(e){return!(!e.prototype||!e.prototype.isPureReactComponent)}var s=e(132),u=e(162),l=e(33),c=e(35),p=e(56),d=e(58),f=e(65),h=(e(66),e(71)),m=(e(75),e(80)),v=e(111),g=e(147),y=(e(154),e(160)),b=e(136),C=(e(161),{ImpureClass:0,PureClass:1,StatelessFunctional:2});r.prototype.render=function(){var e=f.get(this)._currentElement.type,t=e(this.props,this.context,this.updater);return o(e,t),t};var _=1,E={construct:function(e){this._currentElement=e,this._rootNodeID=0,this._compositeType=null,this._instance=null,this._hostParent=null,this._hostContainerInfo=null,this._updateBatchNumber=null,this._pendingElement=null,this._pendingStateQueue=null,this._pendingReplaceState=!1,this._pendingForceUpdate=!1,this._renderedNodeType=null,this._renderedComponent=null,this._context=null,this._mountOrder=0,this._topLevelWrapper=null,this._pendingCallbacks=null,this._calledComponentWillUnmount=!1},mountComponent:function(e,t,n,u){this._context=u,this._mountOrder=_++,this._hostParent=t,this._hostContainerInfo=n;var l,c=this._currentElement.props,d=this._processContext(u),h=this._currentElement.type,m=e.getUpdateQueue(),v=a(h),y=this._constructComponent(v,c,d,m);v||null!=y&&null!=y.render?i(h)?this._compositeType=C.PureClass:this._compositeType=C.ImpureClass:(l=y,o(h,l),null===y||y===!1||p.isValidElement(y)?void 0:s("105",h.displayName||h.name||"Component"),y=new r(h),this._compositeType=C.StatelessFunctional),y.props=c,y.context=d,y.refs=g,y.updater=m,this._instance=y,f.set(y,this);var b=y.state;void 0===b&&(y.state=b=null),"object"!=typeof b||Array.isArray(b)?s("106",this.getName()||"ReactCompositeComponent"):void 0,this._pendingStateQueue=null,this._pendingReplaceState=!1,this._pendingForceUpdate=!1;var E;return E=y.unstable_handleError?this.performInitialMountWithErrorHandling(l,t,n,e,u):this.performInitialMount(l,t,n,e,u),y.componentDidMount&&e.getReactMountReady().enqueue(y.componentDidMount,y),E},_constructComponent:function(e,t,n,r){return this._constructComponentWithoutOwner(e,t,n,r)},_constructComponentWithoutOwner:function(e,t,n,r){var o,a=this._currentElement.type;return o=e?new a(t,n,r):a(t,n,r)},performInitialMountWithErrorHandling:function(e,t,n,r,o){var a,i=r.checkpoint();try{a=this.performInitialMount(e,t,n,r,o)}catch(s){r.rollback(i),this._instance.unstable_handleError(s),this._pendingStateQueue&&(this._instance.state=this._processPendingState(this._instance.props,this._instance.context)),i=r.checkpoint(),this._renderedComponent.unmountComponent(!0),r.rollback(i),a=this.performInitialMount(e,t,n,r,o)}return a},performInitialMount:function(e,t,n,r,o){var a=this._instance;a.componentWillMount&&(a.componentWillMount(),this._pendingStateQueue&&(a.state=this._processPendingState(a.props,a.context))),void 0===e&&(e=this._renderValidatedComponent());var i=h.getType(e);this._renderedNodeType=i;var s=this._instantiateReactComponent(e,i!==h.EMPTY);this._renderedComponent=s;var u=0,l=m.mountComponent(s,r,t,n,this._processChildContext(o),u);return l},getHostNode:function(){return m.getHostNode(this._renderedComponent)},unmountComponent:function(e){if(this._renderedComponent){var t=this._instance;if(t.componentWillUnmount&&!t._calledComponentWillUnmount)if(t._calledComponentWillUnmount=!0,e){var n=this.getName()+".componentWillUnmount()";d.invokeGuardedCallback(n,t.componentWillUnmount.bind(t))}else t.componentWillUnmount();this._renderedComponent&&(m.unmountComponent(this._renderedComponent,e),this._renderedNodeType=null,this._renderedComponent=null,this._instance=null),this._pendingStateQueue=null,this._pendingReplaceState=!1,this._pendingForceUpdate=!1,this._pendingCallbacks=null,this._pendingElement=null,this._context=null,this._rootNodeID=0,this._topLevelWrapper=null,f.remove(t)}},_maskContext:function(e){var t=this._currentElement.type,n=t.contextTypes;if(!n)return g;var r={};for(var o in n)r[o]=e[o];return r},_processContext:function(e){var t=this._maskContext(e);return t},_processChildContext:function(e){var t=this._currentElement.type,n=this._instance,r=n.getChildContext&&n.getChildContext();if(r){"object"!=typeof t.childContextTypes?s("107",this.getName()||"ReactCompositeComponent"):void 0;for(var o in r)o in t.childContextTypes?void 0:s("108",this.getName()||"ReactCompositeComponent",o);return u({},e,r)}return e},_checkContextTypes:function(e,t,n){v(e,t,n,this.getName(),null,this._debugID)},receiveComponent:function(e,t,n){var r=this._currentElement,o=this._context;this._pendingElement=null,this.updateComponent(t,r,e,o,n)},performUpdateIfNecessary:function(e){null!=this._pendingElement?m.receiveComponent(this,this._pendingElement,e,this._context):null!==this._pendingStateQueue||this._pendingForceUpdate?this.updateComponent(e,this._currentElement,this._currentElement,this._context,this._context):this._updateBatchNumber=null},updateComponent:function(e,t,n,r,o){var a=this._instance;null==a?s("136",this.getName()||"ReactCompositeComponent"):void 0;var i,u=!1;this._context===o?i=a.context:(i=this._processContext(o),u=!0);var l=t.props,c=n.props;t!==n&&(u=!0),u&&a.componentWillReceiveProps&&a.componentWillReceiveProps(c,i);var p=this._processPendingState(c,i),d=!0;this._pendingForceUpdate||(a.shouldComponentUpdate?d=a.shouldComponentUpdate(c,p,i):this._compositeType===C.PureClass&&(d=!y(l,c)||!y(a.state,p))),this._updateBatchNumber=null,d?(this._pendingForceUpdate=!1,this._performComponentUpdate(n,c,p,i,e,o)):(this._currentElement=n,this._context=o,a.props=c,a.state=p,a.context=i)},_processPendingState:function(e,t){var n=this._instance,r=this._pendingStateQueue,o=this._pendingReplaceState;if(this._pendingReplaceState=!1,this._pendingStateQueue=null,!r)return n.state;if(o&&1===r.length)return r[0];for(var a=u({},o?r[0]:n.state),i=o?1:0;i<r.length;i++){var s=r[i];u(a,"function"==typeof s?s.call(n,a,e,t):s)}return a},_performComponentUpdate:function(e,t,n,r,o,a){var i,s,u,l=this._instance,c=Boolean(l.componentDidUpdate);c&&(i=l.props,s=l.state,u=l.context),l.componentWillUpdate&&l.componentWillUpdate(t,n,r),this._currentElement=e,this._context=a,l.props=t,l.state=n,l.context=r,this._updateRenderedComponent(o,a),c&&o.getReactMountReady().enqueue(l.componentDidUpdate.bind(l,i,s,u),l)},_updateRenderedComponent:function(e,t){var n=this._renderedComponent,r=n._currentElement,o=this._renderValidatedComponent();if(b(r,o))m.receiveComponent(n,o,e,this._processChildContext(t));else{var a=m.getHostNode(n);m.unmountComponent(n,!1);var i=h.getType(o);this._renderedNodeType=i;var s=this._instantiateReactComponent(o,i!==h.EMPTY);this._renderedComponent=s;var u=0,l=m.mountComponent(s,e,this._hostParent,this._hostContainerInfo,this._processChildContext(t),u);this._replaceNodeWithMarkup(a,l,n)}},_replaceNodeWithMarkup:function(e,t,n){l.replaceNodeWithMarkup(e,t,n)},_renderValidatedComponentWithoutOwnerOrContext:function(){var e=this._instance,t=e.render();return t},_renderValidatedComponent:function(){var e;if(this._compositeType!==C.StatelessFunctional){c.current=this;try{e=this._renderValidatedComponentWithoutOwnerOrContext()}finally{c.current=null}}else e=this._renderValidatedComponentWithoutOwnerOrContext();return null===e||e===!1||p.isValidElement(e)?void 0:s("109",this.getName()||"ReactCompositeComponent"),e},attachRef:function(e,t){var n=this.getPublicInstance();null==n?s("110"):void 0;var r=t.getPublicInstance(),o=n.refs===g?n.refs={}:n.refs;o[e]=r},detachRef:function(e){var t=this.getPublicInstance().refs;delete t[e]},getName:function(){var e=this._currentElement.type,t=this._instance&&this._instance.constructor;return e.displayName||t&&t.displayName||e.name||t&&t.name||null},getPublicInstance:function(){var e=this._instance;return this._compositeType===C.StatelessFunctional?null:e},_instantiateReactComponent:null},x={Mixin:E};t.exports=x},{111:111,132:132,136:136,147:147,154:154,160:160,161:161,162:162,33:33,35:35,56:56,58:58,65:65,66:66,71:71,75:75,80:80}],35:[function(e,t,n){"use strict";var r={current:null};t.exports=r},{}],36:[function(e,t,n){"use strict";var r=e(40),o=e(55),a=e(68),i=e(80),s=e(88),u=e(89),l=e(115),c=e(122),p=e(133);e(161);o.inject();var d={findDOMNode:l,render:a.render,unmountComponentAtNode:a.unmountComponentAtNode,version:u,unstable_batchedUpdates:s.batchedUpdates,unstable_renderSubtreeIntoContainer:p};"undefined"!=typeof __REACT_DEVTOOLS_GLOBAL_HOOK__&&"function"==typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.inject&&__REACT_DEVTOOLS_GLOBAL_HOOK__.inject({ComponentTree:{getClosestInstanceFromNode:r.getClosestInstanceFromNode,getNodeFromInstance:function(e){return e._renderedComponent&&(e=c(e)),e?r.getNodeFromInstance(e):null}},Mount:a,Reconciler:i});t.exports=d},{115:115,122:122,133:133,161:161,40:40,55:55,68:68,80:80,88:88,89:89}],37:[function(e,t,n){"use strict";var r=e(14),o={getHostProps:r.getHostProps};t.exports=o},{14:14}],38:[function(e,t,n){"use strict";function r(e){if(e){var t=e._currentElement._owner||null;if(t){var n=t.getName();if(n)return" This DOM node was rendered by `"+n+"`."}}return""}function o(e,t){t&&($[e._tag]&&(null!=t.children||null!=t.dangerouslySetInnerHTML?m("137",e._tag,e._currentElement._owner?" Check the render method of "+e._currentElement._owner.getName()+".":""):void 0),null!=t.dangerouslySetInnerHTML&&(null!=t.children?m("60"):void 0,"object"==typeof t.dangerouslySetInnerHTML&&K in t.dangerouslySetInnerHTML?void 0:m("61")),null!=t.style&&"object"!=typeof t.style?m("62",r(e)):void 0)}function a(e,t,n,r){if(!(r instanceof A)){var o=e._hostContainerInfo,a=o._node&&o._node.nodeType===z,s=a?o._node:o._ownerDocument;B(t,s),r.getReactMountReady().enqueue(i,{inst:e,registrationName:t,listener:n})}}function i(){var e=this;T.putListener(e.inst,e.registrationName,e.listener)}function s(){var e=this;S.postMountWrapper(e)}function u(){var e=this;I.postMountWrapper(e)}function l(){var e=this;R.postMountWrapper(e)}function c(){var e=this;e._rootNodeID?void 0:m("63");var t=j(e);switch(t?void 0:m("64"),e._tag){case"iframe":case"object":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topLoad,"load",t)];break;case"video":case"audio":e._wrapperState.listeners=[];for(var n in X)X.hasOwnProperty(n)&&e._wrapperState.listeners.push(w.trapBubbledEvent(x.topLevelTypes[n],X[n],t));break;case"source":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topError,"error",t)];break;case"img":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topError,"error",t),w.trapBubbledEvent(x.topLevelTypes.topLoad,"load",t)];break;case"form":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topReset,"reset",t),w.trapBubbledEvent(x.topLevelTypes.topSubmit,"submit",t)];break;case"input":case"select":case"textarea":e._wrapperState.listeners=[w.trapBubbledEvent(x.topLevelTypes.topInvalid,"invalid",t)]}}function p(){O.postUpdateWrapper(this)}function d(e){ee.call(J,e)||(Z.test(e)?void 0:m("65",e),J[e]=!0)}function f(e,t){return e.indexOf("-")>=0||null!=t.is}function h(e){var t=e.type;d(t),this._currentElement=e,this._tag=t.toLowerCase(),this._namespaceURI=null,this._renderedChildren=null,this._previousStyle=null,this._previousStyleCopy=null,this._hostNode=null,this._hostParent=null,this._rootNodeID=0,this._domID=0,this._hostContainerInfo=null,this._wrapperState=null,this._topLevelWrapper=null,this._flags=0}var m=e(132),v=e(162),g=e(1),y=e(4),b=e(8),C=e(9),_=e(10),E=e(11),x=e(16),T=e(17),N=e(18),w=e(27),P=e(37),k=e(39),M=e(40),S=e(46),R=e(47),O=e(48),I=e(52),D=(e(66),e(69)),A=e(84),L=(e(146),e(114)),U=(e(154),e(128),e(158)),F=(e(160),e(138),e(161),k),V=T.deleteListener,j=M.getNodeFromInstance,B=w.listenTo,W=N.registrationNameModules,H={string:!0,number:!0},q=U({style:null}),K=U({__html:null}),Y={children:null,dangerouslySetInnerHTML:null,suppressContentEditableWarning:null},z=11,X={topAbort:"abort",topCanPlay:"canplay",topCanPlayThrough:"canplaythrough",topDurationChange:"durationchange",topEmptied:"emptied",topEncrypted:"encrypted",topEnded:"ended",topError:"error",topLoadedData:"loadeddata",topLoadedMetadata:"loadedmetadata",topLoadStart:"loadstart",topPause:"pause",topPlay:"play",topPlaying:"playing",topProgress:"progress",topRateChange:"ratechange",topSeeked:"seeked",topSeeking:"seeking",topStalled:"stalled",topSuspend:"suspend",topTimeUpdate:"timeupdate",topVolumeChange:"volumechange",topWaiting:"waiting"},G={area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0},Q={listing:!0,pre:!0,textarea:!0},$=v({menuitem:!0},G),Z=/^[a-zA-Z][a-zA-Z:_\.\-\d]*$/,J={},ee={}.hasOwnProperty,te=1;h.displayName="ReactDOMComponent",h.Mixin={mountComponent:function(e,t,n,r){this._rootNodeID=te++,this._domID=n._idCounter++,this._hostParent=t,this._hostContainerInfo=n;var a=this._currentElement.props;switch(this._tag){case"audio":case"form":case"iframe":case"img":case"link":case"object":case"source":case"video":this._wrapperState={listeners:null},e.getReactMountReady().enqueue(c,this);break;case"button":a=P.getHostProps(this,a,t);break;case"input":S.mountWrapper(this,a,t),a=S.getHostProps(this,a),e.getReactMountReady().enqueue(c,this);break;case"option":R.mountWrapper(this,a,t),a=R.getHostProps(this,a);break;case"select":O.mountWrapper(this,a,t),a=O.getHostProps(this,a),e.getReactMountReady().enqueue(c,this);break;case"textarea":I.mountWrapper(this,a,t),a=I.getHostProps(this,a),e.getReactMountReady().enqueue(c,this)}o(this,a);var i,p;null!=t?(i=t._namespaceURI,p=t._tag):n._tag&&(i=n._namespaceURI,p=n._tag),(null==i||i===C.svg&&"foreignobject"===p)&&(i=C.html),i===C.html&&("svg"===this._tag?i=C.svg:"math"===this._tag&&(i=C.mathml)),this._namespaceURI=i;var d;if(e.useCreateElement){var f,h=n._ownerDocument;if(i===C.html)if("script"===this._tag){var m=h.createElement("div"),v=this._currentElement.type;m.innerHTML="<"+v+"></"+v+">",f=m.removeChild(m.firstChild)}else f=a.is?h.createElement(this._currentElement.type,a.is):h.createElement(this._currentElement.type);else f=h.createElementNS(i,this._currentElement.type);M.precacheNode(this,f),this._flags|=F.hasCachedChildNodes,this._hostParent||E.setAttributeForRoot(f),this._updateDOMProperties(null,a,e);var y=b(f);this._createInitialChildren(e,a,r,y),d=y}else{var _=this._createOpenTagMarkupAndPutListeners(e,a),x=this._createContentMarkup(e,a,r);d=!x&&G[this._tag]?_+"/>":_+">"+x+"</"+this._currentElement.type+">"}switch(this._tag){case"input":e.getReactMountReady().enqueue(s,this),a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"textarea":e.getReactMountReady().enqueue(u,this),a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"select":a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"button":a.autoFocus&&e.getReactMountReady().enqueue(g.focusDOMComponent,this);break;case"option":e.getReactMountReady().enqueue(l,this)}return d},_createOpenTagMarkupAndPutListeners:function(e,t){var n="<"+this._currentElement.type;for(var r in t)if(t.hasOwnProperty(r)){var o=t[r];if(null!=o)if(W.hasOwnProperty(r))o&&a(this,r,o,e);else{r===q&&(o&&(o=this._previousStyleCopy=v({},t.style)),o=y.createMarkupForStyles(o,this));var i=null;null!=this._tag&&f(this._tag,t)?Y.hasOwnProperty(r)||(i=E.createMarkupForCustomAttribute(r,o)):i=E.createMarkupForProperty(r,o),i&&(n+=" "+i)}}return e.renderToStaticMarkup?n:(this._hostParent||(n+=" "+E.createMarkupForRoot()),n+=" "+E.createMarkupForID(this._domID))},_createContentMarkup:function(e,t,n){var r="",o=t.dangerouslySetInnerHTML;if(null!=o)null!=o.__html&&(r=o.__html);else{var a=H[typeof t.children]?t.children:null,i=null!=a?null:t.children;if(null!=a)r=L(a);else if(null!=i){var s=this.mountChildren(i,e,n);r=s.join("")}}return Q[this._tag]&&"\n"===r.charAt(0)?"\n"+r:r},_createInitialChildren:function(e,t,n,r){var o=t.dangerouslySetInnerHTML;if(null!=o)null!=o.__html&&b.queueHTML(r,o.__html);else{var a=H[typeof t.children]?t.children:null,i=null!=a?null:t.children;if(null!=a)b.queueText(r,a);else if(null!=i)for(var s=this.mountChildren(i,e,n),u=0;u<s.length;u++)b.queueChild(r,s[u])}},receiveComponent:function(e,t,n){var r=this._currentElement;this._currentElement=e,this.updateComponent(t,r,e,n)},updateComponent:function(e,t,n,r){var a=t.props,i=this._currentElement.props;switch(this._tag){case"button":a=P.getHostProps(this,a),i=P.getHostProps(this,i);break;case"input":a=S.getHostProps(this,a),i=S.getHostProps(this,i);break;case"option":a=R.getHostProps(this,a),i=R.getHostProps(this,i);break;case"select":a=O.getHostProps(this,a),i=O.getHostProps(this,i);break;case"textarea":a=I.getHostProps(this,a),i=I.getHostProps(this,i)}switch(o(this,i),this._updateDOMProperties(a,i,e),this._updateDOMChildren(a,i,e,r),this._tag){case"input":S.updateWrapper(this);break;case"textarea":I.updateWrapper(this);break;case"select":e.getReactMountReady().enqueue(p,this)}},_updateDOMProperties:function(e,t,n){var r,o,i;for(r in e)if(!t.hasOwnProperty(r)&&e.hasOwnProperty(r)&&null!=e[r])if(r===q){var s=this._previousStyleCopy;for(o in s)s.hasOwnProperty(o)&&(i=i||{},i[o]="");this._previousStyleCopy=null}else W.hasOwnProperty(r)?e[r]&&V(this,r):f(this._tag,e)?Y.hasOwnProperty(r)||E.deleteValueForAttribute(j(this),r):(_.properties[r]||_.isCustomAttribute(r))&&E.deleteValueForProperty(j(this),r);for(r in t){var u=t[r],l=r===q?this._previousStyleCopy:null!=e?e[r]:void 0;if(t.hasOwnProperty(r)&&u!==l&&(null!=u||null!=l))if(r===q)if(u?u=this._previousStyleCopy=v({},u):this._previousStyleCopy=null,l){for(o in l)!l.hasOwnProperty(o)||u&&u.hasOwnProperty(o)||(i=i||{},i[o]="");for(o in u)u.hasOwnProperty(o)&&l[o]!==u[o]&&(i=i||{},i[o]=u[o])}else i=u;else if(W.hasOwnProperty(r))u?a(this,r,u,n):l&&V(this,r);else if(f(this._tag,t))Y.hasOwnProperty(r)||E.setValueForAttribute(j(this),r,u);else if(_.properties[r]||_.isCustomAttribute(r)){var c=j(this);null!=u?E.setValueForProperty(c,r,u):E.deleteValueForProperty(c,r)}}i&&y.setValueForStyles(j(this),i,this)},_updateDOMChildren:function(e,t,n,r){var o=H[typeof e.children]?e.children:null,a=H[typeof t.children]?t.children:null,i=e.dangerouslySetInnerHTML&&e.dangerouslySetInnerHTML.__html,s=t.dangerouslySetInnerHTML&&t.dangerouslySetInnerHTML.__html,u=null!=o?null:e.children,l=null!=a?null:t.children,c=null!=o||null!=i,p=null!=a||null!=s;null!=u&&null==l?this.updateChildren(null,n,r):c&&!p&&this.updateTextContent(""),null!=a?o!==a&&this.updateTextContent(""+a):null!=s?i!==s&&this.updateMarkup(""+s):null!=l&&this.updateChildren(l,n,r)},getHostNode:function(){return j(this)},unmountComponent:function(e){switch(this._tag){case"audio":case"form":case"iframe":case"img":case"link":case"object":case"source":case"video":var t=this._wrapperState.listeners;if(t)for(var n=0;n<t.length;n++)t[n].remove();break;case"html":case"head":case"body":m("66",this._tag)}this.unmountChildren(e),M.uncacheNode(this),T.deleteAllListeners(this),this._rootNodeID=0,this._domID=0,this._wrapperState=null},getPublicInstance:function(){return j(this)}},v(h.prototype,h.Mixin,D.Mixin),t.exports=h},{1:1,10:10,11:11,114:114,128:128,132:132,138:138,146:146,154:154,158:158,16:16,160:160,161:161,162:162,17:17,18:18,27:27,37:37,39:39,4:4,40:40,46:46,47:47,48:48,52:52,66:66,69:69,8:8,84:84,9:9}],39:[function(e,t,n){"use strict";var r={hasCachedChildNodes:1};t.exports=r},{}],40:[function(e,t,n){"use strict";function r(e){for(var t;t=e._renderedComponent;)e=t;return e}function o(e,t){var n=r(e);n._hostNode=t,t[m]=n}function a(e){var t=e._hostNode;t&&(delete t[m],e._hostNode=null)}function i(e,t){if(!(e._flags&h.hasCachedChildNodes)){var n=e._renderedChildren,a=t.firstChild;e:for(var i in n)if(n.hasOwnProperty(i)){var s=n[i],u=r(s)._domID;if(0!==u){for(;null!==a;a=a.nextSibling)if(1===a.nodeType&&a.getAttribute(f)===String(u)||8===a.nodeType&&a.nodeValue===" react-text: "+u+" "||8===a.nodeType&&a.nodeValue===" react-empty: "+u+" "){o(s,a);continue e}c("32",u)}}e._flags|=h.hasCachedChildNodes}}function s(e){if(e[m])return e[m];for(var t=[];!e[m];){if(t.push(e),!e.parentNode)return null;e=e.parentNode}for(var n,r;e&&(r=e[m]);e=t.pop())n=r,t.length&&i(r,e);return n}function u(e){var t=s(e);return null!=t&&t._hostNode===e?t:null}function l(e){if(void 0===e._hostNode?c("33"):void 0,e._hostNode)return e._hostNode;for(var t=[];!e._hostNode;)t.push(e),e._hostParent?void 0:c("34"),e=e._hostParent;for(;t.length;e=t.pop())i(e,e._hostNode);return e._hostNode}var c=e(132),p=e(10),d=e(39),f=(e(154),p.ID_ATTRIBUTE_NAME),h=d,m="__reactInternalInstance$"+Math.random().toString(36).slice(2),v={getClosestInstanceFromNode:s,getInstanceFromNode:u,getNodeFromInstance:l,precacheChildNodes:i,precacheNode:o,uncacheNode:a};t.exports=v},{10:10,132:132,154:154,39:39}],41:[function(e,t,n){"use strict";function r(e,t){var n={_topLevelWrapper:e,_idCounter:1,_ownerDocument:t?t.nodeType===o?t:t.ownerDocument:null,_node:t,_tag:t?t.nodeName.toLowerCase():null,_namespaceURI:t?t.namespaceURI:null};return n}var o=(e(138),9);t.exports=r},{138:138}],42:[function(e,t,n){"use strict";var r=e(162),o=e(8),a=e(40),i=function(e){this._currentElement=null,this._hostNode=null,this._hostParent=null,this._hostContainerInfo=null,this._domID=0};r(i.prototype,{mountComponent:function(e,t,n,r){var i=n._idCounter++;this._domID=i,this._hostParent=t,this._hostContainerInfo=n;var s=" react-empty: "+this._domID+" ";if(e.useCreateElement){var u=n._ownerDocument,l=u.createComment(s);return a.precacheNode(this,l),
o(l)}return e.renderToStaticMarkup?"":"<!--"+s+"-->"},receiveComponent:function(){},getHostNode:function(){return a.getNodeFromInstance(this)},unmountComponent:function(){a.uncacheNode(this)}}),t.exports=i},{162:162,40:40,8:8}],43:[function(e,t,n){"use strict";var r=e(56),o=r.createFactory,a={a:o("a"),abbr:o("abbr"),address:o("address"),area:o("area"),article:o("article"),aside:o("aside"),audio:o("audio"),b:o("b"),base:o("base"),bdi:o("bdi"),bdo:o("bdo"),big:o("big"),blockquote:o("blockquote"),body:o("body"),br:o("br"),button:o("button"),canvas:o("canvas"),caption:o("caption"),cite:o("cite"),code:o("code"),col:o("col"),colgroup:o("colgroup"),data:o("data"),datalist:o("datalist"),dd:o("dd"),del:o("del"),details:o("details"),dfn:o("dfn"),dialog:o("dialog"),div:o("div"),dl:o("dl"),dt:o("dt"),em:o("em"),embed:o("embed"),fieldset:o("fieldset"),figcaption:o("figcaption"),figure:o("figure"),footer:o("footer"),form:o("form"),h1:o("h1"),h2:o("h2"),h3:o("h3"),h4:o("h4"),h5:o("h5"),h6:o("h6"),head:o("head"),header:o("header"),hgroup:o("hgroup"),hr:o("hr"),html:o("html"),i:o("i"),iframe:o("iframe"),img:o("img"),input:o("input"),ins:o("ins"),kbd:o("kbd"),keygen:o("keygen"),label:o("label"),legend:o("legend"),li:o("li"),link:o("link"),main:o("main"),map:o("map"),mark:o("mark"),menu:o("menu"),menuitem:o("menuitem"),meta:o("meta"),meter:o("meter"),nav:o("nav"),noscript:o("noscript"),object:o("object"),ol:o("ol"),optgroup:o("optgroup"),option:o("option"),output:o("output"),p:o("p"),param:o("param"),picture:o("picture"),pre:o("pre"),progress:o("progress"),q:o("q"),rp:o("rp"),rt:o("rt"),ruby:o("ruby"),s:o("s"),samp:o("samp"),script:o("script"),section:o("section"),select:o("select"),small:o("small"),source:o("source"),span:o("span"),strong:o("strong"),style:o("style"),sub:o("sub"),summary:o("summary"),sup:o("sup"),table:o("table"),tbody:o("tbody"),td:o("td"),textarea:o("textarea"),tfoot:o("tfoot"),th:o("th"),thead:o("thead"),time:o("time"),title:o("title"),tr:o("tr"),track:o("track"),u:o("u"),ul:o("ul"),var:o("var"),video:o("video"),wbr:o("wbr"),circle:o("circle"),clipPath:o("clipPath"),defs:o("defs"),ellipse:o("ellipse"),g:o("g"),image:o("image"),line:o("line"),linearGradient:o("linearGradient"),mask:o("mask"),path:o("path"),pattern:o("pattern"),polygon:o("polygon"),polyline:o("polyline"),radialGradient:o("radialGradient"),rect:o("rect"),stop:o("stop"),svg:o("svg"),text:o("text"),tspan:o("tspan")};t.exports=a},{56:56}],44:[function(e,t,n){"use strict";var r={useCreateElement:!0};t.exports=r},{}],45:[function(e,t,n){"use strict";var r=e(7),o=e(40),a={dangerouslyProcessChildrenUpdates:function(e,t){var n=o.getNodeFromInstance(e);r.processUpdates(n,t)}};t.exports=a},{40:40,7:7}],46:[function(e,t,n){"use strict";function r(){this._rootNodeID&&d.updateWrapper(this)}function o(e){var t=this._currentElement.props,n=l.executeOnChange(t,e);p.asap(r,this);var o=t.name;if("radio"===t.type&&null!=o){for(var i=c.getNodeFromInstance(this),s=i;s.parentNode;)s=s.parentNode;for(var u=s.querySelectorAll("input[name="+JSON.stringify(""+o)+'][type="radio"]'),d=0;d<u.length;d++){var f=u[d];if(f!==i&&f.form===i.form){var h=c.getInstanceFromNode(f);h?void 0:a("90"),p.asap(r,h)}}}return n}var a=e(132),i=e(162),s=e(14),u=e(11),l=e(24),c=e(40),p=e(88),d=(e(154),e(161),{getHostProps:function(e,t){var n=l.getValue(t),r=l.getChecked(t),o=i({type:void 0,step:void 0,min:void 0,max:void 0},s.getHostProps(e,t),{defaultChecked:void 0,defaultValue:void 0,value:null!=n?n:e._wrapperState.initialValue,checked:null!=r?r:e._wrapperState.initialChecked,onChange:e._wrapperState.onChange});return o},mountWrapper:function(e,t){var n=t.defaultValue;e._wrapperState={initialChecked:null!=t.checked?t.checked:t.defaultChecked,initialValue:null!=t.value?t.value:n,listeners:null,onChange:o.bind(e)}},updateWrapper:function(e){var t=e._currentElement.props,n=t.checked;null!=n&&u.setValueForProperty(c.getNodeFromInstance(e),"checked",n||!1);var r=c.getNodeFromInstance(e),o=l.getValue(t);if(null!=o){var a=""+o;a!==r.value&&(r.value=a)}else null==t.value&&null!=t.defaultValue&&(r.defaultValue=""+t.defaultValue),null==t.checked&&null!=t.defaultChecked&&(r.defaultChecked=!!t.defaultChecked)},postMountWrapper:function(e){var t=e._currentElement.props,n=c.getNodeFromInstance(e);switch(t.type){case"submit":case"reset":break;case"color":case"date":case"datetime":case"datetime-local":case"month":case"time":case"week":n.value="",n.value=n.defaultValue;break;default:n.value=n.value}var r=n.name;""!==r&&(n.name=""),n.defaultChecked=!n.defaultChecked,n.defaultChecked=!n.defaultChecked,""!==r&&(n.name=r)}});t.exports=d},{11:11,132:132,14:14,154:154,161:161,162:162,24:24,40:40,88:88}],47:[function(e,t,n){"use strict";function r(e){var t="";return a.forEach(e,function(e){null!=e&&("string"==typeof e||"number"==typeof e?t+=e:u||(u=!0))}),t}var o=e(162),a=e(29),i=e(40),s=e(48),u=(e(161),!1),l={mountWrapper:function(e,t,n){var o=null;if(null!=n){var a=n;"optgroup"===a._tag&&(a=a._hostParent),null!=a&&"select"===a._tag&&(o=s.getSelectValueContext(a))}var i=null;if(null!=o){var u;if(u=null!=t.value?t.value+"":r(t.children),i=!1,Array.isArray(o)){for(var l=0;l<o.length;l++)if(""+o[l]===u){i=!0;break}}else i=""+o===u}e._wrapperState={selected:i}},postMountWrapper:function(e){var t=e._currentElement.props;if(null!=t.value){var n=i.getNodeFromInstance(e);n.setAttribute("value",t.value)}},getHostProps:function(e,t){var n=o({selected:void 0,children:void 0},t);null!=e._wrapperState.selected&&(n.selected=e._wrapperState.selected);var a=r(t.children);return a&&(n.children=a),n}};t.exports=l},{161:161,162:162,29:29,40:40,48:48}],48:[function(e,t,n){"use strict";function r(){if(this._rootNodeID&&this._wrapperState.pendingUpdate){this._wrapperState.pendingUpdate=!1;var e=this._currentElement.props,t=u.getValue(e);null!=t&&o(this,Boolean(e.multiple),t)}}function o(e,t,n){var r,o,a=l.getNodeFromInstance(e).options;if(t){for(r={},o=0;o<n.length;o++)r[""+n[o]]=!0;for(o=0;o<a.length;o++){var i=r.hasOwnProperty(a[o].value);a[o].selected!==i&&(a[o].selected=i)}}else{for(r=""+n,o=0;o<a.length;o++)if(a[o].value===r)return void(a[o].selected=!0);a.length&&(a[0].selected=!0)}}function a(e){var t=this._currentElement.props,n=u.executeOnChange(t,e);return this._rootNodeID&&(this._wrapperState.pendingUpdate=!0),c.asap(r,this),n}var i=e(162),s=e(14),u=e(24),l=e(40),c=e(88),p=(e(161),!1),d={getHostProps:function(e,t){return i({},s.getHostProps(e,t),{onChange:e._wrapperState.onChange,value:void 0})},mountWrapper:function(e,t){var n=u.getValue(t);e._wrapperState={pendingUpdate:!1,initialValue:null!=n?n:t.defaultValue,listeners:null,onChange:a.bind(e),wasMultiple:Boolean(t.multiple)},void 0===t.value||void 0===t.defaultValue||p||(p=!0)},getSelectValueContext:function(e){return e._wrapperState.initialValue},postUpdateWrapper:function(e){var t=e._currentElement.props;e._wrapperState.initialValue=void 0;var n=e._wrapperState.wasMultiple;e._wrapperState.wasMultiple=Boolean(t.multiple);var r=u.getValue(t);null!=r?(e._wrapperState.pendingUpdate=!1,o(e,Boolean(t.multiple),r)):n!==Boolean(t.multiple)&&(null!=t.defaultValue?o(e,Boolean(t.multiple),t.defaultValue):o(e,Boolean(t.multiple),t.multiple?[]:""))}};t.exports=d},{14:14,161:161,162:162,24:24,40:40,88:88}],49:[function(e,t,n){"use strict";function r(e,t,n,r){return e===n&&t===r}function o(e){var t=document.selection,n=t.createRange(),r=n.text.length,o=n.duplicate();o.moveToElementText(e),o.setEndPoint("EndToStart",n);var a=o.text.length,i=a+r;return{start:a,end:i}}function a(e){var t=window.getSelection&&window.getSelection();if(!t||0===t.rangeCount)return null;var n=t.anchorNode,o=t.anchorOffset,a=t.focusNode,i=t.focusOffset,s=t.getRangeAt(0);try{s.startContainer.nodeType,s.endContainer.nodeType}catch(e){return null}var u=r(t.anchorNode,t.anchorOffset,t.focusNode,t.focusOffset),l=u?0:s.toString().length,c=s.cloneRange();c.selectNodeContents(e),c.setEnd(s.startContainer,s.startOffset);var p=r(c.startContainer,c.startOffset,c.endContainer,c.endOffset),d=p?0:c.toString().length,f=d+l,h=document.createRange();h.setStart(n,o),h.setEnd(a,i);var m=h.collapsed;return{start:m?f:d,end:m?d:f}}function i(e,t){var n,r,o=document.selection.createRange().duplicate();void 0===t.end?(n=t.start,r=n):t.start>t.end?(n=t.end,r=t.start):(n=t.start,r=t.end),o.moveToElementText(e),o.moveStart("character",n),o.setEndPoint("EndToStart",o),o.moveEnd("character",r-n),o.select()}function s(e,t){if(window.getSelection){var n=window.getSelection(),r=e[c()].length,o=Math.min(t.start,r),a=void 0===t.end?o:Math.min(t.end,r);if(!n.extend&&o>a){var i=a;a=o,o=i}var s=l(e,o),u=l(e,a);if(s&&u){var p=document.createRange();p.setStart(s.node,s.offset),n.removeAllRanges(),o>a?(n.addRange(p),n.extend(u.node,u.offset)):(p.setEnd(u.node,u.offset),n.addRange(p))}}}var u=e(140),l=e(124),c=e(125),p=u.canUseDOM&&"selection"in document&&!("getSelection"in window),d={getOffsets:p?o:a,setOffsets:p?i:s};t.exports=d},{124:124,125:125,140:140}],50:[function(e,t,n){"use strict";var r=e(55),o=e(83),a=e(89);r.inject();var i={renderToString:o.renderToString,renderToStaticMarkup:o.renderToStaticMarkup,version:a};t.exports=i},{55:55,83:83,89:89}],51:[function(e,t,n){"use strict";var r=e(132),o=e(162),a=e(7),i=e(8),s=e(40),u=e(114),l=(e(154),e(138),function(e){this._currentElement=e,this._stringText=""+e,this._hostNode=null,this._hostParent=null,this._domID=0,this._mountIndex=0,this._closingComment=null,this._commentNodes=null});o(l.prototype,{mountComponent:function(e,t,n,r){var o=n._idCounter++,a=" react-text: "+o+" ",l=" /react-text ";if(this._domID=o,this._hostParent=t,e.useCreateElement){var c=n._ownerDocument,p=c.createComment(a),d=c.createComment(l),f=i(c.createDocumentFragment());return i.queueChild(f,i(p)),this._stringText&&i.queueChild(f,i(c.createTextNode(this._stringText))),i.queueChild(f,i(d)),s.precacheNode(this,p),this._closingComment=d,f}var h=u(this._stringText);return e.renderToStaticMarkup?h:"<!--"+a+"-->"+h+"<!--"+l+"-->"},receiveComponent:function(e,t){if(e!==this._currentElement){this._currentElement=e;var n=""+e;if(n!==this._stringText){this._stringText=n;var r=this.getHostNode();a.replaceDelimitedText(r[0],r[1],n)}}},getHostNode:function(){var e=this._commentNodes;if(e)return e;if(!this._closingComment)for(var t=s.getNodeFromInstance(this),n=t.nextSibling;;){if(null==n?r("67",this._domID):void 0,8===n.nodeType&&" /react-text "===n.nodeValue){this._closingComment=n;break}n=n.nextSibling}return e=[this._hostNode,this._closingComment],this._commentNodes=e,e},unmountComponent:function(){this._closingComment=null,this._commentNodes=null,s.uncacheNode(this)}}),t.exports=l},{114:114,132:132,138:138,154:154,162:162,40:40,7:7,8:8}],52:[function(e,t,n){"use strict";function r(){this._rootNodeID&&p.updateWrapper(this)}function o(e){var t=this._currentElement.props,n=u.executeOnChange(t,e);return c.asap(r,this),n}var a=e(132),i=e(162),s=e(14),u=e(24),l=e(40),c=e(88),p=(e(154),e(161),{getHostProps:function(e,t){null!=t.dangerouslySetInnerHTML?a("91"):void 0;var n=i({},s.getHostProps(e,t),{value:void 0,defaultValue:void 0,children:""+e._wrapperState.initialValue,onChange:e._wrapperState.onChange});return n},mountWrapper:function(e,t){var n=u.getValue(t),r=n;if(null==n){var i=t.defaultValue,s=t.children;null!=s&&(null!=i?a("92"):void 0,Array.isArray(s)&&(s.length<=1?void 0:a("93"),s=s[0]),i=""+s),null==i&&(i=""),r=i}e._wrapperState={initialValue:""+r,listeners:null,onChange:o.bind(e)}},updateWrapper:function(e){var t=e._currentElement.props,n=l.getNodeFromInstance(e),r=u.getValue(t);if(null!=r){var o=""+r;o!==n.value&&(n.value=o),null==t.defaultValue&&(n.defaultValue=o)}null!=t.defaultValue&&(n.defaultValue=t.defaultValue)},postMountWrapper:function(e){var t=l.getNodeFromInstance(e);t.value=t.textContent}});t.exports=p},{132:132,14:14,154:154,161:161,162:162,24:24,40:40,88:88}],53:[function(e,t,n){"use strict";function r(e,t){"_hostNode"in e?void 0:u("33"),"_hostNode"in t?void 0:u("33");for(var n=0,r=e;r;r=r._hostParent)n++;for(var o=0,a=t;a;a=a._hostParent)o++;for(;n-o>0;)e=e._hostParent,n--;for(;o-n>0;)t=t._hostParent,o--;for(var i=n;i--;){if(e===t)return e;e=e._hostParent,t=t._hostParent}return null}function o(e,t){"_hostNode"in e?void 0:u("35"),"_hostNode"in t?void 0:u("35");for(;t;){if(t===e)return!0;t=t._hostParent}return!1}function a(e){return"_hostNode"in e?void 0:u("36"),e._hostParent}function i(e,t,n){for(var r=[];e;)r.push(e),e=e._hostParent;var o;for(o=r.length;o-- >0;)t(r[o],!1,n);for(o=0;o<r.length;o++)t(r[o],!0,n)}function s(e,t,n,o,a){for(var i=e&&t?r(e,t):null,s=[];e&&e!==i;)s.push(e),e=e._hostParent;for(var u=[];t&&t!==i;)u.push(t),t=t._hostParent;var l;for(l=0;l<s.length;l++)n(s[l],!0,o);for(l=u.length;l-- >0;)n(u[l],!1,a)}var u=e(132);e(154);t.exports={isAncestor:o,getLowestCommonAncestor:r,getParentInstance:a,traverseTwoPhase:i,traverseEnterLeave:s}},{132:132,154:154}],54:[function(e,t,n){"use strict";function r(){this.reinitializeTransaction()}var o=e(162),a=e(88),i=e(106),s=e(146),u={initialize:s,close:function(){d.isBatchingUpdates=!1}},l={initialize:s,close:a.flushBatchedUpdates.bind(a)},c=[l,u];o(r.prototype,i.Mixin,{getTransactionWrappers:function(){return c}});var p=new r,d={isBatchingUpdates:!1,batchedUpdates:function(e,t,n,r,o,a){var i=d.isBatchingUpdates;d.isBatchingUpdates=!0,i?e(t,n,r,o,a):p.perform(e,null,t,n,r,o,a)}};t.exports=d},{106:106,146:146,162:162,88:88}],55:[function(e,t,n){"use strict";function r(){E||(E=!0,g.EventEmitter.injectReactEventListener(v),g.EventPluginHub.injectEventPluginOrder(i),g.EventPluginUtils.injectComponentTree(p),g.EventPluginUtils.injectTreeTraversal(f),g.EventPluginHub.injectEventPluginsByName({SimpleEventPlugin:_,EnterLeaveEventPlugin:s,ChangeEventPlugin:a,SelectEventPlugin:C,BeforeInputEventPlugin:o}),g.HostComponent.injectGenericComponentClass(c),g.HostComponent.injectTextComponentClass(h),g.DOMProperty.injectDOMPropertyConfig(u),g.DOMProperty.injectDOMPropertyConfig(b),g.EmptyComponent.injectEmptyComponentFactory(function(e){return new d(e)}),g.Updates.injectReconcileTransaction(y),g.Updates.injectBatchingStrategy(m),g.Component.injectEnvironment(l))}var o=e(2),a=e(6),i=e(13),s=e(15),u=e(22),l=e(32),c=e(38),p=e(40),d=e(42),f=e(53),h=e(51),m=e(54),v=e(60),g=e(63),y=e(79),b=e(90),C=e(91),_=e(92),E=!1;t.exports={inject:r}},{13:13,15:15,2:2,22:22,32:32,38:38,40:40,42:42,51:51,53:53,54:54,6:6,60:60,63:63,79:79,90:90,91:91,92:92}],56:[function(e,t,n){"use strict";function r(e){return void 0!==e.ref}function o(e){return void 0!==e.key}var a=e(162),i=e(35),s=(e(161),e(110),Object.prototype.hasOwnProperty),u="function"==typeof Symbol&&Symbol.for&&Symbol.for("react.element")||60103,l={key:!0,ref:!0,__self:!0,__source:!0},c=function(e,t,n,r,o,a,i){var s={$$typeof:u,type:e,key:t,ref:n,props:i,_owner:a};return s};c.createElement=function(e,t,n){var a,u={},p=null,d=null,f=null,h=null;if(null!=t){r(t)&&(d=t.ref),o(t)&&(p=""+t.key),f=void 0===t.__self?null:t.__self,h=void 0===t.__source?null:t.__source;for(a in t)s.call(t,a)&&!l.hasOwnProperty(a)&&(u[a]=t[a])}var m=arguments.length-2;if(1===m)u.children=n;else if(m>1){for(var v=Array(m),g=0;g<m;g++)v[g]=arguments[g+2];u.children=v}if(e&&e.defaultProps){var y=e.defaultProps;for(a in y)void 0===u[a]&&(u[a]=y[a])}return c(e,p,d,f,h,i.current,u)},c.createFactory=function(e){var t=c.createElement.bind(null,e);return t.type=e,t},c.cloneAndReplaceKey=function(e,t){var n=c(e.type,t,e.ref,e._self,e._source,e._owner,e.props);return n},c.cloneElement=function(e,t,n){var u,p=a({},e.props),d=e.key,f=e.ref,h=e._self,m=e._source,v=e._owner;if(null!=t){r(t)&&(f=t.ref,v=i.current),o(t)&&(d=""+t.key);var g;e.type&&e.type.defaultProps&&(g=e.type.defaultProps);for(u in t)s.call(t,u)&&!l.hasOwnProperty(u)&&(void 0===t[u]&&void 0!==g?p[u]=g[u]:p[u]=t[u])}var y=arguments.length-2;if(1===y)p.children=n;else if(y>1){for(var b=Array(y),C=0;C<y;C++)b[C]=arguments[C+2];p.children=b}return c(e.type,d,f,h,m,v,p)},c.isValidElement=function(e){return"object"==typeof e&&null!==e&&e.$$typeof===u},c.REACT_ELEMENT_TYPE=u,t.exports=c},{110:110,161:161,162:162,35:35}],57:[function(e,t,n){"use strict";var r,o={injectEmptyComponentFactory:function(e){r=e}},a={create:function(e){return r(e)}};a.injection=o,t.exports=a},{}],58:[function(e,t,n){"use strict";function r(e,t,n,r){try{return t(n,r)}catch(e){return void(null===o&&(o=e))}}var o=null,a={invokeGuardedCallback:r,invokeGuardedCallbackWithCatch:r,rethrowCaughtError:function(){if(o){var e=o;throw o=null,e}}};t.exports=a},{}],59:[function(e,t,n){"use strict";function r(e){o.enqueueEvents(e),o.processEventQueue(!1)}var o=e(17),a={handleTopLevel:function(e,t,n,a){var i=o.extractEvents(e,t,n,a);r(i)}};t.exports=a},{17:17}],60:[function(e,t,n){"use strict";function r(e){for(;e._hostParent;)e=e._hostParent;var t=p.getNodeFromInstance(e),n=t.parentNode;return p.getClosestInstanceFromNode(n)}function o(e,t){this.topLevelType=e,this.nativeEvent=t,this.ancestors=[]}function a(e){var t=f(e.nativeEvent),n=p.getClosestInstanceFromNode(t),o=n;do e.ancestors.push(o),o=o&&r(o);while(o);for(var a=0;a<e.ancestors.length;a++)n=e.ancestors[a],m._handleTopLevel(e.topLevelType,n,e.nativeEvent,f(e.nativeEvent))}function i(e){var t=h(window);e(t)}var s=e(162),u=e(139),l=e(140),c=e(25),p=e(40),d=e(88),f=e(121),h=e(151);s(o.prototype,{destructor:function(){this.topLevelType=null,this.nativeEvent=null,this.ancestors.length=0}}),c.addPoolingTo(o,c.twoArgumentPooler);var m={_enabled:!0,_handleTopLevel:null,WINDOW_HANDLE:l.canUseDOM?window:null,setHandleTopLevel:function(e){m._handleTopLevel=e},setEnabled:function(e){m._enabled=!!e},isEnabled:function(){return m._enabled},trapBubbledEvent:function(e,t,n){var r=n;return r?u.listen(r,t,m.dispatchEvent.bind(null,e)):null},trapCapturedEvent:function(e,t,n){var r=n;return r?u.capture(r,t,m.dispatchEvent.bind(null,e)):null},monitorScrollValue:function(e){var t=i.bind(null,e);u.listen(window,"scroll",t)},dispatchEvent:function(e,t){if(m._enabled){var n=o.getPooled(e,t);try{d.batchedUpdates(a,n)}finally{o.release(n)}}}};t.exports=m},{121:121,139:139,140:140,151:151,162:162,25:25,40:40,88:88}],61:[function(e,t,n){"use strict";var r={logTopLevelRenders:!1};t.exports=r},{}],62:[function(e,t,n){"use strict";function r(e){return u?void 0:i("111",e.type),new u(e)}function o(e){return new c(e)}function a(e){return e instanceof c}var i=e(132),s=e(162),u=(e(154),null),l={},c=null,p={injectGenericComponentClass:function(e){u=e},injectTextComponentClass:function(e){c=e},injectComponentClasses:function(e){s(l,e)}},d={createInternalComponent:r,createInstanceForText:o,isTextComponent:a,injection:p};t.exports=d},{132:132,154:154,162:162}],63:[function(e,t,n){"use strict";var r=e(10),o=e(17),a=e(19),i=e(33),s=e(30),u=e(57),l=e(27),c=e(62),p=e(88),d={Component:i.injection,Class:s.injection,DOMProperty:r.injection,EmptyComponent:u.injection,EventPluginHub:o.injection,EventPluginUtils:a.injection,EventEmitter:l.injection,HostComponent:c.injection,Updates:p.injection};t.exports=d},{10:10,17:17,19:19,27:27,30:30,33:33,57:57,62:62,88:88}],64:[function(e,t,n){"use strict";function r(e){return a(document.documentElement,e)}var o=e(49),a=e(143),i=e(148),s=e(149),u={hasSelectionCapabilities:function(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&("input"===t&&"text"===e.type||"textarea"===t||"true"===e.contentEditable)},getSelectionInformation:function(){var e=s();return{focusedElem:e,selectionRange:u.hasSelectionCapabilities(e)?u.getSelection(e):null}},restoreSelection:function(e){var t=s(),n=e.focusedElem,o=e.selectionRange;t!==n&&r(n)&&(u.hasSelectionCapabilities(n)&&u.setSelection(n,o),i(n))},getSelection:function(e){var t;if("selectionStart"in e)t={start:e.selectionStart,end:e.selectionEnd};else if(document.selection&&e.nodeName&&"input"===e.nodeName.toLowerCase()){var n=document.selection.createRange();n.parentElement()===e&&(t={start:-n.moveStart("character",-e.value.length),end:-n.moveEnd("character",-e.value.length)})}else t=o.getOffsets(e);return t||{start:0,end:0}},setSelection:function(e,t){var n=t.start,r=t.end;if(void 0===r&&(r=n),"selectionStart"in e)e.selectionStart=n,e.selectionEnd=Math.min(r,e.value.length);else if(document.selection&&e.nodeName&&"input"===e.nodeName.toLowerCase()){var a=e.createTextRange();a.collapse(!0),a.moveStart("character",n),a.moveEnd("character",r-n),a.select()}else o.setOffsets(e,t)}};t.exports=u},{143:143,148:148,149:149,49:49}],65:[function(e,t,n){"use strict";var r={remove:function(e){e._reactInternalInstance=void 0},get:function(e){return e._reactInternalInstance},has:function(e){return void 0!==e._reactInternalInstance},set:function(e,t){e._reactInternalInstance=t}};t.exports=r},{}],66:[function(e,t,n){"use strict";var r=null;t.exports={debugTool:r}},{}],67:[function(e,t,n){"use strict";var r=e(109),o=/\/?>/,a=/^<\!\-\-/,i={CHECKSUM_ATTR_NAME:"data-react-checksum",addChecksumToMarkup:function(e){var t=r(e);return a.test(e)?e:e.replace(o," "+i.CHECKSUM_ATTR_NAME+'="'+t+'"$&')},canReuseMarkup:function(e,t){var n=t.getAttribute(i.CHECKSUM_ATTR_NAME);n=n&&parseInt(n,10);var o=r(e);return o===n}};t.exports=i},{109:109}],68:[function(e,t,n){"use strict";function r(e,t){for(var n=Math.min(e.length,t.length),r=0;r<n;r++)if(e.charAt(r)!==t.charAt(r))return r;return e.length===t.length?-1:n}function o(e){return e?e.nodeType===D?e.documentElement:e.firstChild:null}function a(e){return e.getAttribute&&e.getAttribute(R)||""}function i(e,t,n,r,o){var a;if(_.logTopLevelRenders){var i=e._currentElement.props,s=i.type;a="React mount: "+("string"==typeof s?s:s.displayName||s.name),console.time(a)}var u=T.mountComponent(e,n,null,y(e,t),o,0);a&&console.timeEnd(a),e._renderedComponent._topLevelWrapper=e,V._mountImageIntoNode(u,t,e,r,n)}function s(e,t,n,r){var o=w.ReactReconcileTransaction.getPooled(!n&&b.useCreateElement);o.perform(i,null,e,t,o,n,r),w.ReactReconcileTransaction.release(o)}function u(e,t,n){for(T.unmountComponent(e,n),t.nodeType===D&&(t=t.documentElement);t.lastChild;)t.removeChild(t.lastChild)}function l(e){var t=o(e);if(t){var n=g.getInstanceFromNode(t);return!(!n||!n._hostParent)}}function c(e){return!(!e||e.nodeType!==I&&e.nodeType!==D&&e.nodeType!==A)}function p(e){var t=o(e),n=t&&g.getInstanceFromNode(t);return n&&!n._hostParent?n:null}function d(e){var t=p(e);return t?t._hostContainerInfo._topLevelWrapper:null}var f=e(132),h=e(8),m=e(10),v=e(27),g=(e(35),e(40)),y=e(41),b=e(44),C=e(56),_=e(61),E=e(65),x=(e(66),e(67)),T=e(80),N=e(87),w=e(88),P=e(147),k=e(127),M=(e(154),e(134)),S=e(136),R=(e(161),m.ID_ATTRIBUTE_NAME),O=m.ROOT_ATTRIBUTE_NAME,I=1,D=9,A=11,L={},U=1,F=function(){this.rootID=U++};F.prototype.isReactComponent={},F.prototype.render=function(){return this.props};var V={TopLevelWrapper:F,_instancesByReactRootID:L,scrollMonitor:function(e,t){t()},_updateRootComponent:function(e,t,n,r,o){return V.scrollMonitor(r,function(){N.enqueueElementInternal(e,t,n),o&&N.enqueueCallbackInternal(e,o)}),e},_renderNewRootComponent:function(e,t,n,r){c(t)?void 0:f("37"),v.ensureScrollValueMonitoring();var o=k(e,!1);w.batchedUpdates(s,o,t,n,r);var a=o._instance.rootID;return L[a]=o,o},renderSubtreeIntoContainer:function(e,t,n,r){return null!=e&&E.has(e)?void 0:f("38"),V._renderSubtreeIntoContainer(e,t,n,r)},_renderSubtreeIntoContainer:function(e,t,n,r){N.validateCallback(r,"ReactDOM.render"),C.isValidElement(t)?void 0:f("39","string"==typeof t?" Instead of passing a string like 'div', pass React.createElement('div') or <div />.":"function"==typeof t?" Instead of passing a class like Foo, pass React.createElement(Foo) or <Foo />.":null!=t&&void 0!==t.props?" This may be caused by unintentionally loading two independent copies of React.":"");var i,s=C(F,null,null,null,null,null,t);if(e){var u=E.get(e);i=u._processChildContext(u._context)}else i=P;var c=d(n);if(c){var p=c._currentElement,h=p.props;if(S(h,t)){var m=c._renderedComponent.getPublicInstance(),v=r&&function(){r.call(m)};return V._updateRootComponent(c,s,i,n,v),m}V.unmountComponentAtNode(n)}var g=o(n),y=g&&!!a(g),b=l(n),_=y&&!c&&!b,x=V._renderNewRootComponent(s,n,_,i)._renderedComponent.getPublicInstance();return r&&r.call(x),x},render:function(e,t,n){return V._renderSubtreeIntoContainer(null,e,t,n)},unmountComponentAtNode:function(e){c(e)?void 0:f("40");var t=d(e);return t?(delete L[t._instance.rootID],w.batchedUpdates(u,t,e,!1),!0):(l(e),1===e.nodeType&&e.hasAttribute(O),!1)},_mountImageIntoNode:function(e,t,n,a,i){if(c(t)?void 0:f("41"),a){var s=o(t);if(x.canReuseMarkup(e,s))return void g.precacheNode(n,s);var u=s.getAttribute(x.CHECKSUM_ATTR_NAME);s.removeAttribute(x.CHECKSUM_ATTR_NAME);var l=s.outerHTML;s.setAttribute(x.CHECKSUM_ATTR_NAME,u);var p=e,d=r(p,l),m=" (client) "+p.substring(d-20,d+20)+"\n (server) "+l.substring(d-20,d+20);t.nodeType===D?f("42",m):void 0}if(t.nodeType===D?f("43"):void 0,i.useCreateElement){for(;t.lastChild;)t.removeChild(t.lastChild);h.insertTreeBefore(t,e,null)}else M(t,e),g.precacheNode(n,t.firstChild)}};t.exports=V},{10:10,127:127,132:132,134:134,136:136,147:147,154:154,161:161,27:27,35:35,40:40,41:41,44:44,56:56,61:61,65:65,66:66,67:67,8:8,80:80,87:87,88:88}],69:[function(e,t,n){"use strict";function r(e,t,n){return{type:d.INSERT_MARKUP,content:e,fromIndex:null,fromNode:null,toIndex:n,afterNode:t}}function o(e,t,n){return{type:d.MOVE_EXISTING,content:null,fromIndex:e._mountIndex,fromNode:f.getHostNode(e),toIndex:n,afterNode:t}}function a(e,t){return{type:d.REMOVE_NODE,content:null,fromIndex:e._mountIndex,fromNode:t,toIndex:null,afterNode:null}}function i(e){return{type:d.SET_MARKUP,content:e,fromIndex:null,fromNode:null,toIndex:null,afterNode:null}}function s(e){return{type:d.TEXT_CONTENT,content:e,fromIndex:null,fromNode:null,toIndex:null,afterNode:null}}function u(e,t){return t&&(e=e||[],e.push(t)),e}function l(e,t){p.processChildrenUpdates(e,t)}var c=e(132),p=e(33),d=(e(65),e(66),e(70)),f=(e(35),e(80)),h=e(28),m=(e(146),e(116)),v=(e(154),{Mixin:{_reconcilerInstantiateChildren:function(e,t,n){return h.instantiateChildren(e,t,n)},_reconcilerUpdateChildren:function(e,t,n,r,o,a){var i,s=0;return i=m(t,s),h.updateChildren(e,i,n,r,o,this,this._hostContainerInfo,a,s),i},mountChildren:function(e,t,n){var r=this._reconcilerInstantiateChildren(e,t,n);this._renderedChildren=r;var o=[],a=0;for(var i in r)if(r.hasOwnProperty(i)){var s=r[i],u=0,l=f.mountComponent(s,t,this,this._hostContainerInfo,n,u);s._mountIndex=a++,o.push(l)}return o},updateTextContent:function(e){var t=this._renderedChildren;h.unmountChildren(t,!1);for(var n in t)t.hasOwnProperty(n)&&c("118");var r=[s(e)];l(this,r)},updateMarkup:function(e){var t=this._renderedChildren;h.unmountChildren(t,!1);for(var n in t)t.hasOwnProperty(n)&&c("118");var r=[i(e)];l(this,r)},updateChildren:function(e,t,n){this._updateChildren(e,t,n)},_updateChildren:function(e,t,n){var r=this._renderedChildren,o={},a=[],i=this._reconcilerUpdateChildren(r,e,a,o,t,n);if(i||r){var s,c=null,p=0,d=0,h=0,m=null;for(s in i)if(i.hasOwnProperty(s)){var v=r&&r[s],g=i[s];v===g?(c=u(c,this.moveChild(v,m,p,d)),d=Math.max(v._mountIndex,d),v._mountIndex=p):(v&&(d=Math.max(v._mountIndex,d)),c=u(c,this._mountChildAtIndex(g,a[h],m,p,t,n)),h++),p++,m=f.getHostNode(g)}for(s in o)o.hasOwnProperty(s)&&(c=u(c,this._unmountChild(r[s],o[s])));c&&l(this,c),this._renderedChildren=i}},unmountChildren:function(e){var t=this._renderedChildren;h.unmountChildren(t,e),this._renderedChildren=null},moveChild:function(e,t,n,r){if(e._mountIndex<r)return o(e,t,n)},createChild:function(e,t,n){return r(n,t,e._mountIndex)},removeChild:function(e,t){return a(e,t)},_mountChildAtIndex:function(e,t,n,r,o,a){return e._mountIndex=r,this.createChild(e,n,t)},_unmountChild:function(e,t){var n=this.removeChild(e,t);return e._mountIndex=null,n}}});t.exports=v},{116:116,132:132,146:146,154:154,28:28,33:33,35:35,65:65,66:66,70:70,80:80}],70:[function(e,t,n){"use strict";var r=e(157),o=r({INSERT_MARKUP:null,MOVE_EXISTING:null,REMOVE_NODE:null,SET_MARKUP:null,TEXT_CONTENT:null});t.exports=o},{157:157}],71:[function(e,t,n){"use strict";var r=e(132),o=e(56),a=(e(154),{HOST:0,COMPOSITE:1,EMPTY:2,getType:function(e){return null===e||e===!1?a.EMPTY:o.isValidElement(e)?"function"==typeof e.type?a.COMPOSITE:a.HOST:void r("26",e)}});t.exports=a},{132:132,154:154,56:56}],72:[function(e,t,n){"use strict";function r(e,t){}var o=(e(161),{isMounted:function(e){return!1},enqueueCallback:function(e,t){},enqueueForceUpdate:function(e){r(e,"forceUpdate")},enqueueReplaceState:function(e,t){r(e,"replaceState")},enqueueSetState:function(e,t){r(e,"setState")}});t.exports=o},{161:161}],73:[function(e,t,n){"use strict";var r=e(132),o=(e(154),{isValidOwner:function(e){return!(!e||"function"!=typeof e.attachRef||"function"!=typeof e.detachRef)},addComponentAsRefTo:function(e,t,n){o.isValidOwner(n)?void 0:r("119"),n.attachRef(t,e)},removeComponentAsRefFrom:function(e,t,n){o.isValidOwner(n)?void 0:r("120");var a=n.getPublicInstance();a&&a.refs[t]===e.getPublicInstance()&&n.detachRef(t)}});t.exports=o},{132:132,154:154}],74:[function(e,t,n){"use strict";var r={};t.exports=r},{}],75:[function(e,t,n){"use strict";var r=e(157),o=r({prop:null,context:null,childContext:null});t.exports=o},{157:157}],76:[function(e,t,n){"use strict";function r(e,t){return e===t?0!==e||1/e===1/t:e!==e&&t!==t}function o(e){this.message=e,this.stack=""}function a(e){function t(t,n,r,a,i,s,u){if(a=a||w,s=s||r,null==n[r]){var l=E[i];return t?new o("Required "+l+" `"+s+"` was not specified in "+("`"+a+"`.")):null}return e(n,r,a,i,s)}var n=t.bind(null,!1);return n.isRequired=t.bind(null,!0),n}function i(e){function t(t,n,r,a,i,s){var u=t[n],l=y(u);if(l!==e){var c=E[a],p=b(u);return new o("Invalid "+c+" `"+i+"` of type "+("`"+p+"` supplied to `"+r+"`, expected ")+("`"+e+"`."))}return null}return a(t)}function s(){return a(T.thatReturns(null))}function u(e){function t(t,n,r,a,i){if("function"!=typeof e)return new o("Property `"+i+"` of component `"+r+"` has invalid PropType notation inside arrayOf.");var s=t[n];if(!Array.isArray(s)){var u=E[a],l=y(s);return new o("Invalid "+u+" `"+i+"` of type "+("`"+l+"` supplied to `"+r+"`, expected an array."))}for(var c=0;c<s.length;c++){var p=e(s,c,r,a,i+"["+c+"]",x);if(p instanceof Error)return p}return null}return a(t)}function l(){function e(e,t,n,r,a){var i=e[t];if(!_.isValidElement(i)){var s=E[r],u=y(i);return new o("Invalid "+s+" `"+a+"` of type "+("`"+u+"` supplied to `"+n+"`, expected a single ReactElement."))}return null}return a(e)}function c(e){function t(t,n,r,a,i){if(!(t[n]instanceof e)){var s=E[a],u=e.name||w,l=C(t[n]);return new o("Invalid "+s+" `"+i+"` of type "+("`"+l+"` supplied to `"+r+"`, expected ")+("instance of `"+u+"`."))}return null}return a(t)}function p(e){function t(t,n,a,i,s){for(var u=t[n],l=0;l<e.length;l++)if(r(u,e[l]))return null;var c=E[i],p=JSON.stringify(e);return new o("Invalid "+c+" `"+s+"` of value `"+u+"` "+("supplied to `"+a+"`, expected one of "+p+"."))}return Array.isArray(e)?a(t):T.thatReturnsNull}function d(e){function t(t,n,r,a,i){if("function"!=typeof e)return new o("Property `"+i+"` of component `"+r+"` has invalid PropType notation inside objectOf.");var s=t[n],u=y(s);if("object"!==u){var l=E[a];return new o("Invalid "+l+" `"+i+"` of type "+("`"+u+"` supplied to `"+r+"`, expected an object."))}for(var c in s)if(s.hasOwnProperty(c)){var p=e(s,c,r,a,i+"."+c,x);if(p instanceof Error)return p}return null}return a(t)}function f(e){function t(t,n,r,a,i){for(var s=0;s<e.length;s++){var u=e[s];if(null==u(t,n,r,a,i,x))return null}var l=E[a];return new o("Invalid "+l+" `"+i+"` supplied to "+("`"+r+"`."))}return Array.isArray(e)?a(t):T.thatReturnsNull}function h(){function e(e,t,n,r,a){if(!v(e[t])){var i=E[r];return new o("Invalid "+i+" `"+a+"` supplied to "+("`"+n+"`, expected a ReactNode."))}return null}return a(e)}function m(e){function t(t,n,r,a,i){var s=t[n],u=y(s);if("object"!==u){var l=E[a];return new o("Invalid "+l+" `"+i+"` of type `"+u+"` "+("supplied to `"+r+"`, expected `object`."));
}for(var c in e){var p=e[c];if(p){var d=p(s,c,r,a,i+"."+c,x);if(d)return d}}return null}return a(t)}function v(e){switch(typeof e){case"number":case"string":case"undefined":return!0;case"boolean":return!e;case"object":if(Array.isArray(e))return e.every(v);if(null===e||_.isValidElement(e))return!0;var t=N(e);if(!t)return!1;var n,r=t.call(e);if(t!==e.entries){for(;!(n=r.next()).done;)if(!v(n.value))return!1}else for(;!(n=r.next()).done;){var o=n.value;if(o&&!v(o[1]))return!1}return!0;default:return!1}}function g(e,t){return"symbol"===e||"Symbol"===t["@@toStringTag"]||"function"==typeof Symbol&&t instanceof Symbol}function y(e){var t=typeof e;return Array.isArray(e)?"array":e instanceof RegExp?"object":g(t,e)?"symbol":t}function b(e){var t=y(e);if("object"===t){if(e instanceof Date)return"date";if(e instanceof RegExp)return"regexp"}return t}function C(e){return e.constructor&&e.constructor.name?e.constructor.name:w}var _=e(56),E=e(74),x=e(77),T=e(146),N=e(123),w=(e(161),"<<anonymous>>"),P={array:i("array"),bool:i("boolean"),func:i("function"),number:i("number"),object:i("object"),string:i("string"),symbol:i("symbol"),any:s(),arrayOf:u,element:l(),instanceOf:c,node:h(),objectOf:d,oneOf:p,oneOfType:f,shape:m};o.prototype=Error.prototype,t.exports=P},{123:123,146:146,161:161,56:56,74:74,77:77}],77:[function(e,t,n){"use strict";var r="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED";t.exports=r},{}],78:[function(e,t,n){"use strict";function r(e,t,n){this.props=e,this.context=t,this.refs=u,this.updater=n||s}function o(){}var a=e(162),i=e(31),s=e(72),u=e(147);o.prototype=i.prototype,r.prototype=new o,r.prototype.constructor=r,a(r.prototype,i.prototype),r.prototype.isPureReactComponent=!0,t.exports=r},{147:147,162:162,31:31,72:72}],79:[function(e,t,n){"use strict";function r(e){this.reinitializeTransaction(),this.renderToStaticMarkup=!1,this.reactMountReady=a.getPooled(null),this.useCreateElement=e}var o=e(162),a=e(5),i=e(25),s=e(27),u=e(64),l=(e(66),e(106)),c=e(87),p={initialize:u.getSelectionInformation,close:u.restoreSelection},d={initialize:function(){var e=s.isEnabled();return s.setEnabled(!1),e},close:function(e){s.setEnabled(e)}},f={initialize:function(){this.reactMountReady.reset()},close:function(){this.reactMountReady.notifyAll()}},h=[p,d,f],m={getTransactionWrappers:function(){return h},getReactMountReady:function(){return this.reactMountReady},getUpdateQueue:function(){return c},checkpoint:function(){return this.reactMountReady.checkpoint()},rollback:function(e){this.reactMountReady.rollback(e)},destructor:function(){a.release(this.reactMountReady),this.reactMountReady=null}};o(r.prototype,l.Mixin,m),i.addPoolingTo(r),t.exports=r},{106:106,162:162,25:25,27:27,5:5,64:64,66:66,87:87}],80:[function(e,t,n){"use strict";function r(){o.attachRefs(this,this._currentElement)}var o=e(81),a=(e(66),e(161),{mountComponent:function(e,t,n,o,a,i){var s=e.mountComponent(t,n,o,a,i);return e._currentElement&&null!=e._currentElement.ref&&t.getReactMountReady().enqueue(r,e),s},getHostNode:function(e){return e.getHostNode()},unmountComponent:function(e,t){o.detachRefs(e,e._currentElement),e.unmountComponent(t)},receiveComponent:function(e,t,n,a){var i=e._currentElement;if(t!==i||a!==e._context){var s=o.shouldUpdateRefs(i,t);s&&o.detachRefs(e,i),e.receiveComponent(t,n,a),s&&e._currentElement&&null!=e._currentElement.ref&&n.getReactMountReady().enqueue(r,e)}},performUpdateIfNecessary:function(e,t,n){e._updateBatchNumber===n&&e.performUpdateIfNecessary(t)}});t.exports=a},{161:161,66:66,81:81}],81:[function(e,t,n){"use strict";function r(e,t,n){"function"==typeof e?e(t.getPublicInstance()):a.addComponentAsRefTo(t,e,n)}function o(e,t,n){"function"==typeof e?e(null):a.removeComponentAsRefFrom(t,e,n)}var a=e(73),i={};i.attachRefs=function(e,t){if(null!==t&&t!==!1){var n=t.ref;null!=n&&r(n,e,t._owner)}},i.shouldUpdateRefs=function(e,t){var n=null===e||e===!1,r=null===t||t===!1;return n||r||t.ref!==e.ref||"string"==typeof t.ref&&t._owner!==e._owner},i.detachRefs=function(e,t){if(null!==t&&t!==!1){var n=t.ref;null!=n&&o(n,e,t._owner)}},t.exports=i},{73:73}],82:[function(e,t,n){"use strict";var r={isBatchingUpdates:!1,batchedUpdates:function(e){}};t.exports=r},{}],83:[function(e,t,n){"use strict";function r(e,t){var n;try{return h.injection.injectBatchingStrategy(d),n=f.getPooled(t),g++,n.perform(function(){var r=v(e,!0),o=p.mountComponent(r,n,null,s(),m,0);return t||(o=c.addChecksumToMarkup(o)),o},null)}finally{g--,f.release(n),g||h.injection.injectBatchingStrategy(u)}}function o(e){return l.isValidElement(e)?void 0:i("46"),r(e,!1)}function a(e){return l.isValidElement(e)?void 0:i("47"),r(e,!0)}var i=e(132),s=e(41),u=e(54),l=e(56),c=(e(66),e(67)),p=e(80),d=e(82),f=e(84),h=e(88),m=e(147),v=e(127),g=(e(154),0);t.exports={renderToString:o,renderToStaticMarkup:a}},{127:127,132:132,147:147,154:154,41:41,54:54,56:56,66:66,67:67,80:80,82:82,84:84,88:88}],84:[function(e,t,n){"use strict";function r(e){this.reinitializeTransaction(),this.renderToStaticMarkup=e,this.useCreateElement=!1,this.updateQueue=new s(this)}var o=e(162),a=e(25),i=e(106),s=(e(66),e(85)),u=[],l={enqueue:function(){}},c={getTransactionWrappers:function(){return u},getReactMountReady:function(){return l},getUpdateQueue:function(){return this.updateQueue},destructor:function(){},checkpoint:function(){},rollback:function(){}};o(r.prototype,i.Mixin,c),a.addPoolingTo(r),t.exports=r},{106:106,162:162,25:25,66:66,85:85}],85:[function(e,t,n){"use strict";function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o(e,t){}var a=e(87),i=(e(106),e(161),function(){function e(t){r(this,e),this.transaction=t}return e.prototype.isMounted=function(e){return!1},e.prototype.enqueueCallback=function(e,t,n){this.transaction.isInTransaction()&&a.enqueueCallback(e,t,n)},e.prototype.enqueueForceUpdate=function(e){this.transaction.isInTransaction()?a.enqueueForceUpdate(e):o(e,"forceUpdate")},e.prototype.enqueueReplaceState=function(e,t){this.transaction.isInTransaction()?a.enqueueReplaceState(e,t):o(e,"replaceState")},e.prototype.enqueueSetState=function(e,t){this.transaction.isInTransaction()?a.enqueueSetState(e,t):o(e,"setState")},e}());t.exports=i},{106:106,161:161,87:87}],86:[function(e,t,n){"use strict";var r=e(162),o=e(36),a=e(50),i=e(26),s=r({__SECRET_DOM_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:o,__SECRET_DOM_SERVER_DO_NOT_USE_OR_YOU_WILL_BE_FIRED:a},i);t.exports=s},{162:162,26:26,36:36,50:50}],87:[function(e,t,n){"use strict";function r(e){u.enqueueUpdate(e)}function o(e){var t=typeof e;if("object"!==t)return t;var n=e.constructor&&e.constructor.name||t,r=Object.keys(e);return r.length>0&&r.length<20?n+" (keys: "+r.join(", ")+")":n}function a(e,t){var n=s.get(e);return n?n:null}var i=e(132),s=(e(35),e(65)),u=(e(66),e(88)),l=(e(154),e(161),{isMounted:function(e){var t=s.get(e);return!!t&&!!t._renderedComponent},enqueueCallback:function(e,t,n){l.validateCallback(t,n);var o=a(e);return o?(o._pendingCallbacks?o._pendingCallbacks.push(t):o._pendingCallbacks=[t],void r(o)):null},enqueueCallbackInternal:function(e,t){e._pendingCallbacks?e._pendingCallbacks.push(t):e._pendingCallbacks=[t],r(e)},enqueueForceUpdate:function(e){var t=a(e,"forceUpdate");t&&(t._pendingForceUpdate=!0,r(t))},enqueueReplaceState:function(e,t){var n=a(e,"replaceState");n&&(n._pendingStateQueue=[t],n._pendingReplaceState=!0,r(n))},enqueueSetState:function(e,t){var n=a(e,"setState");if(n){var o=n._pendingStateQueue||(n._pendingStateQueue=[]);o.push(t),r(n)}},enqueueElementInternal:function(e,t,n){e._pendingElement=t,e._context=n,r(e)},validateCallback:function(e,t){e&&"function"!=typeof e?i("122",t,o(e)):void 0}});t.exports=l},{132:132,154:154,161:161,35:35,65:65,66:66,88:88}],88:[function(e,t,n){"use strict";function r(){P.ReactReconcileTransaction&&_?void 0:c("123")}function o(){this.reinitializeTransaction(),this.dirtyComponentsLength=null,this.callbackQueue=d.getPooled(),this.reconcileTransaction=P.ReactReconcileTransaction.getPooled(!0)}function a(e,t,n,o,a,i){r(),_.batchedUpdates(e,t,n,o,a,i)}function i(e,t){return e._mountOrder-t._mountOrder}function s(e){var t=e.dirtyComponentsLength;t!==g.length?c("124",t,g.length):void 0,g.sort(i),y++;for(var n=0;n<t;n++){var r=g[n],o=r._pendingCallbacks;r._pendingCallbacks=null;var a;if(h.logTopLevelRenders){var s=r;r._currentElement.props===r._renderedComponent._currentElement&&(s=r._renderedComponent),a="React update: "+s.getName(),console.time(a)}if(m.performUpdateIfNecessary(r,e.reconcileTransaction,y),a&&console.timeEnd(a),o)for(var u=0;u<o.length;u++)e.callbackQueue.enqueue(o[u],r.getPublicInstance())}}function u(e){return r(),_.isBatchingUpdates?(g.push(e),void(null==e._updateBatchNumber&&(e._updateBatchNumber=y+1))):void _.batchedUpdates(u,e)}function l(e,t){_.isBatchingUpdates?void 0:c("125"),b.enqueue(e,t),C=!0}var c=e(132),p=e(162),d=e(5),f=e(25),h=e(61),m=e(80),v=e(106),g=(e(154),[]),y=0,b=d.getPooled(),C=!1,_=null,E={initialize:function(){this.dirtyComponentsLength=g.length},close:function(){this.dirtyComponentsLength!==g.length?(g.splice(0,this.dirtyComponentsLength),N()):g.length=0}},x={initialize:function(){this.callbackQueue.reset()},close:function(){this.callbackQueue.notifyAll()}},T=[E,x];p(o.prototype,v.Mixin,{getTransactionWrappers:function(){return T},destructor:function(){this.dirtyComponentsLength=null,d.release(this.callbackQueue),this.callbackQueue=null,P.ReactReconcileTransaction.release(this.reconcileTransaction),this.reconcileTransaction=null},perform:function(e,t,n){return v.Mixin.perform.call(this,this.reconcileTransaction.perform,this.reconcileTransaction,e,t,n)}}),f.addPoolingTo(o);var N=function(){for(;g.length||C;){if(g.length){var e=o.getPooled();e.perform(s,null,e),o.release(e)}if(C){C=!1;var t=b;b=d.getPooled(),t.notifyAll(),d.release(t)}}},w={injectReconcileTransaction:function(e){e?void 0:c("126"),P.ReactReconcileTransaction=e},injectBatchingStrategy:function(e){e?void 0:c("127"),"function"!=typeof e.batchedUpdates?c("128"):void 0,"boolean"!=typeof e.isBatchingUpdates?c("129"):void 0,_=e}},P={ReactReconcileTransaction:null,batchedUpdates:a,enqueueUpdate:u,flushBatchedUpdates:N,injection:w,asap:l};t.exports=P},{106:106,132:132,154:154,162:162,25:25,5:5,61:61,80:80}],89:[function(e,t,n){"use strict";t.exports="15.3.1"},{}],90:[function(e,t,n){"use strict";var r={xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace"},o={accentHeight:"accent-height",accumulate:0,additive:0,alignmentBaseline:"alignment-baseline",allowReorder:"allowReorder",alphabetic:0,amplitude:0,arabicForm:"arabic-form",ascent:0,attributeName:"attributeName",attributeType:"attributeType",autoReverse:"autoReverse",azimuth:0,baseFrequency:"baseFrequency",baseProfile:"baseProfile",baselineShift:"baseline-shift",bbox:0,begin:0,bias:0,by:0,calcMode:"calcMode",capHeight:"cap-height",clip:0,clipPath:"clip-path",clipRule:"clip-rule",clipPathUnits:"clipPathUnits",colorInterpolation:"color-interpolation",colorInterpolationFilters:"color-interpolation-filters",colorProfile:"color-profile",colorRendering:"color-rendering",contentScriptType:"contentScriptType",contentStyleType:"contentStyleType",cursor:0,cx:0,cy:0,d:0,decelerate:0,descent:0,diffuseConstant:"diffuseConstant",direction:0,display:0,divisor:0,dominantBaseline:"dominant-baseline",dur:0,dx:0,dy:0,edgeMode:"edgeMode",elevation:0,enableBackground:"enable-background",end:0,exponent:0,externalResourcesRequired:"externalResourcesRequired",fill:0,fillOpacity:"fill-opacity",fillRule:"fill-rule",filter:0,filterRes:"filterRes",filterUnits:"filterUnits",floodColor:"flood-color",floodOpacity:"flood-opacity",focusable:0,fontFamily:"font-family",fontSize:"font-size",fontSizeAdjust:"font-size-adjust",fontStretch:"font-stretch",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",format:0,from:0,fx:0,fy:0,g1:0,g2:0,glyphName:"glyph-name",glyphOrientationHorizontal:"glyph-orientation-horizontal",glyphOrientationVertical:"glyph-orientation-vertical",glyphRef:"glyphRef",gradientTransform:"gradientTransform",gradientUnits:"gradientUnits",hanging:0,horizAdvX:"horiz-adv-x",horizOriginX:"horiz-origin-x",ideographic:0,imageRendering:"image-rendering",in:0,in2:0,intercept:0,k:0,k1:0,k2:0,k3:0,k4:0,kernelMatrix:"kernelMatrix",kernelUnitLength:"kernelUnitLength",kerning:0,keyPoints:"keyPoints",keySplines:"keySplines",keyTimes:"keyTimes",lengthAdjust:"lengthAdjust",letterSpacing:"letter-spacing",lightingColor:"lighting-color",limitingConeAngle:"limitingConeAngle",local:0,markerEnd:"marker-end",markerMid:"marker-mid",markerStart:"marker-start",markerHeight:"markerHeight",markerUnits:"markerUnits",markerWidth:"markerWidth",mask:0,maskContentUnits:"maskContentUnits",maskUnits:"maskUnits",mathematical:0,mode:0,numOctaves:"numOctaves",offset:0,opacity:0,operator:0,order:0,orient:0,orientation:0,origin:0,overflow:0,overlinePosition:"overline-position",overlineThickness:"overline-thickness",paintOrder:"paint-order",panose1:"panose-1",pathLength:"pathLength",patternContentUnits:"patternContentUnits",patternTransform:"patternTransform",patternUnits:"patternUnits",pointerEvents:"pointer-events",points:0,pointsAtX:"pointsAtX",pointsAtY:"pointsAtY",pointsAtZ:"pointsAtZ",preserveAlpha:"preserveAlpha",preserveAspectRatio:"preserveAspectRatio",primitiveUnits:"primitiveUnits",r:0,radius:0,refX:"refX",refY:"refY",renderingIntent:"rendering-intent",repeatCount:"repeatCount",repeatDur:"repeatDur",requiredExtensions:"requiredExtensions",requiredFeatures:"requiredFeatures",restart:0,result:0,rotate:0,rx:0,ry:0,scale:0,seed:0,shapeRendering:"shape-rendering",slope:0,spacing:0,specularConstant:"specularConstant",specularExponent:"specularExponent",speed:0,spreadMethod:"spreadMethod",startOffset:"startOffset",stdDeviation:"stdDeviation",stemh:0,stemv:0,stitchTiles:"stitchTiles",stopColor:"stop-color",stopOpacity:"stop-opacity",strikethroughPosition:"strikethrough-position",strikethroughThickness:"strikethrough-thickness",string:0,stroke:0,strokeDasharray:"stroke-dasharray",strokeDashoffset:"stroke-dashoffset",strokeLinecap:"stroke-linecap",strokeLinejoin:"stroke-linejoin",strokeMiterlimit:"stroke-miterlimit",strokeOpacity:"stroke-opacity",strokeWidth:"stroke-width",surfaceScale:"surfaceScale",systemLanguage:"systemLanguage",tableValues:"tableValues",targetX:"targetX",targetY:"targetY",textAnchor:"text-anchor",textDecoration:"text-decoration",textRendering:"text-rendering",textLength:"textLength",to:0,transform:0,u1:0,u2:0,underlinePosition:"underline-position",underlineThickness:"underline-thickness",unicode:0,unicodeBidi:"unicode-bidi",unicodeRange:"unicode-range",unitsPerEm:"units-per-em",vAlphabetic:"v-alphabetic",vHanging:"v-hanging",vIdeographic:"v-ideographic",vMathematical:"v-mathematical",values:0,vectorEffect:"vector-effect",version:0,vertAdvY:"vert-adv-y",vertOriginX:"vert-origin-x",vertOriginY:"vert-origin-y",viewBox:"viewBox",viewTarget:"viewTarget",visibility:0,widths:0,wordSpacing:"word-spacing",writingMode:"writing-mode",x:0,xHeight:"x-height",x1:0,x2:0,xChannelSelector:"xChannelSelector",xlinkActuate:"xlink:actuate",xlinkArcrole:"xlink:arcrole",xlinkHref:"xlink:href",xlinkRole:"xlink:role",xlinkShow:"xlink:show",xlinkTitle:"xlink:title",xlinkType:"xlink:type",xmlBase:"xml:base",xmlns:0,xmlnsXlink:"xmlns:xlink",xmlLang:"xml:lang",xmlSpace:"xml:space",y:0,y1:0,y2:0,yChannelSelector:"yChannelSelector",z:0,zoomAndPan:"zoomAndPan"},a={Properties:{},DOMAttributeNamespaces:{xlinkActuate:r.xlink,xlinkArcrole:r.xlink,xlinkHref:r.xlink,xlinkRole:r.xlink,xlinkShow:r.xlink,xlinkTitle:r.xlink,xlinkType:r.xlink,xmlBase:r.xml,xmlLang:r.xml,xmlSpace:r.xml},DOMAttributeNames:{}};Object.keys(o).forEach(function(e){a.Properties[e]=0,o[e]&&(a.DOMAttributeNames[e]=o[e])}),t.exports=a},{}],91:[function(e,t,n){"use strict";function r(e){if("selectionStart"in e&&l.hasSelectionCapabilities(e))return{start:e.selectionStart,end:e.selectionEnd};if(window.getSelection){var t=window.getSelection();return{anchorNode:t.anchorNode,anchorOffset:t.anchorOffset,focusNode:t.focusNode,focusOffset:t.focusOffset}}if(document.selection){var n=document.selection.createRange();return{parentElement:n.parentElement(),text:n.text,top:n.boundingTop,left:n.boundingLeft}}}function o(e,t){if(_||null==y||y!==p())return null;var n=r(y);if(!C||!h(C,n)){C=n;var o=c.getPooled(g.select,b,e,t);return o.type="select",o.target=y,i.accumulateTwoPhaseDispatches(o),o}return null}var a=e(16),i=e(20),s=e(140),u=e(40),l=e(64),c=e(97),p=e(149),d=e(129),f=e(158),h=e(160),m=a.topLevelTypes,v=s.canUseDOM&&"documentMode"in document&&document.documentMode<=11,g={select:{phasedRegistrationNames:{bubbled:f({onSelect:null}),captured:f({onSelectCapture:null})},dependencies:[m.topBlur,m.topContextMenu,m.topFocus,m.topKeyDown,m.topMouseDown,m.topMouseUp,m.topSelectionChange]}},y=null,b=null,C=null,_=!1,E=!1,x=f({onSelect:null}),T={eventTypes:g,extractEvents:function(e,t,n,r){if(!E)return null;var a=t?u.getNodeFromInstance(t):window;switch(e){case m.topFocus:(d(a)||"true"===a.contentEditable)&&(y=a,b=t,C=null);break;case m.topBlur:y=null,b=null,C=null;break;case m.topMouseDown:_=!0;break;case m.topContextMenu:case m.topMouseUp:return _=!1,o(n,r);case m.topSelectionChange:if(v)break;case m.topKeyDown:case m.topKeyUp:return o(n,r)}return null},didPutListener:function(e,t,n){t===x&&(E=!0)}};t.exports=T},{129:129,140:140,149:149,158:158,16:16,160:160,20:20,40:40,64:64,97:97}],92:[function(e,t,n){"use strict";function r(e){return"."+e._rootNodeID}var o=e(132),a=e(16),i=e(139),s=e(20),u=e(40),l=e(93),c=e(94),p=e(97),d=e(98),f=e(100),h=e(101),m=e(96),v=e(102),g=e(103),y=e(104),b=e(105),C=e(146),_=e(118),E=(e(154),e(158)),x=a.topLevelTypes,T={abort:{phasedRegistrationNames:{bubbled:E({onAbort:!0}),captured:E({onAbortCapture:!0})}},animationEnd:{phasedRegistrationNames:{bubbled:E({onAnimationEnd:!0}),captured:E({onAnimationEndCapture:!0})}},animationIteration:{phasedRegistrationNames:{bubbled:E({onAnimationIteration:!0}),captured:E({onAnimationIterationCapture:!0})}},animationStart:{phasedRegistrationNames:{bubbled:E({onAnimationStart:!0}),captured:E({onAnimationStartCapture:!0})}},blur:{phasedRegistrationNames:{bubbled:E({onBlur:!0}),captured:E({onBlurCapture:!0})}},canPlay:{phasedRegistrationNames:{bubbled:E({onCanPlay:!0}),captured:E({onCanPlayCapture:!0})}},canPlayThrough:{phasedRegistrationNames:{bubbled:E({onCanPlayThrough:!0}),captured:E({onCanPlayThroughCapture:!0})}},click:{phasedRegistrationNames:{bubbled:E({onClick:!0}),captured:E({onClickCapture:!0})}},contextMenu:{phasedRegistrationNames:{bubbled:E({onContextMenu:!0}),captured:E({onContextMenuCapture:!0})}},copy:{phasedRegistrationNames:{bubbled:E({onCopy:!0}),captured:E({onCopyCapture:!0})}},cut:{phasedRegistrationNames:{bubbled:E({onCut:!0}),captured:E({onCutCapture:!0})}},doubleClick:{phasedRegistrationNames:{bubbled:E({onDoubleClick:!0}),captured:E({onDoubleClickCapture:!0})}},drag:{phasedRegistrationNames:{bubbled:E({onDrag:!0}),captured:E({onDragCapture:!0})}},dragEnd:{phasedRegistrationNames:{bubbled:E({onDragEnd:!0}),captured:E({onDragEndCapture:!0})}},dragEnter:{phasedRegistrationNames:{bubbled:E({onDragEnter:!0}),captured:E({onDragEnterCapture:!0})}},dragExit:{phasedRegistrationNames:{bubbled:E({onDragExit:!0}),captured:E({onDragExitCapture:!0})}},dragLeave:{phasedRegistrationNames:{bubbled:E({onDragLeave:!0}),captured:E({onDragLeaveCapture:!0})}},dragOver:{phasedRegistrationNames:{bubbled:E({onDragOver:!0}),captured:E({onDragOverCapture:!0})}},dragStart:{phasedRegistrationNames:{bubbled:E({onDragStart:!0}),captured:E({onDragStartCapture:!0})}},drop:{phasedRegistrationNames:{bubbled:E({onDrop:!0}),captured:E({onDropCapture:!0})}},durationChange:{phasedRegistrationNames:{bubbled:E({onDurationChange:!0}),captured:E({onDurationChangeCapture:!0})}},emptied:{phasedRegistrationNames:{bubbled:E({onEmptied:!0}),captured:E({onEmptiedCapture:!0})}},encrypted:{phasedRegistrationNames:{bubbled:E({onEncrypted:!0}),captured:E({onEncryptedCapture:!0})}},ended:{phasedRegistrationNames:{bubbled:E({onEnded:!0}),captured:E({onEndedCapture:!0})}},error:{phasedRegistrationNames:{bubbled:E({onError:!0}),captured:E({onErrorCapture:!0})}},focus:{phasedRegistrationNames:{bubbled:E({onFocus:!0}),captured:E({onFocusCapture:!0})}},input:{phasedRegistrationNames:{bubbled:E({onInput:!0}),captured:E({onInputCapture:!0})}},invalid:{phasedRegistrationNames:{bubbled:E({onInvalid:!0}),captured:E({onInvalidCapture:!0})}},keyDown:{phasedRegistrationNames:{bubbled:E({onKeyDown:!0}),captured:E({onKeyDownCapture:!0})}},keyPress:{phasedRegistrationNames:{bubbled:E({onKeyPress:!0}),captured:E({onKeyPressCapture:!0})}},keyUp:{phasedRegistrationNames:{bubbled:E({onKeyUp:!0}),captured:E({onKeyUpCapture:!0})}},load:{phasedRegistrationNames:{bubbled:E({onLoad:!0}),captured:E({onLoadCapture:!0})}},loadedData:{phasedRegistrationNames:{bubbled:E({onLoadedData:!0}),captured:E({onLoadedDataCapture:!0})}},loadedMetadata:{phasedRegistrationNames:{bubbled:E({onLoadedMetadata:!0}),captured:E({onLoadedMetadataCapture:!0})}},loadStart:{phasedRegistrationNames:{bubbled:E({onLoadStart:!0}),captured:E({onLoadStartCapture:!0})}},mouseDown:{phasedRegistrationNames:{bubbled:E({onMouseDown:!0}),captured:E({onMouseDownCapture:!0})}},mouseMove:{phasedRegistrationNames:{bubbled:E({onMouseMove:!0}),captured:E({onMouseMoveCapture:!0})}},mouseOut:{phasedRegistrationNames:{bubbled:E({onMouseOut:!0}),captured:E({onMouseOutCapture:!0})}},mouseOver:{phasedRegistrationNames:{bubbled:E({onMouseOver:!0}),captured:E({onMouseOverCapture:!0})}},mouseUp:{phasedRegistrationNames:{bubbled:E({onMouseUp:!0}),captured:E({onMouseUpCapture:!0})}},paste:{phasedRegistrationNames:{bubbled:E({onPaste:!0}),captured:E({onPasteCapture:!0})}},pause:{phasedRegistrationNames:{bubbled:E({onPause:!0}),captured:E({onPauseCapture:!0})}},play:{phasedRegistrationNames:{bubbled:E({onPlay:!0}),captured:E({onPlayCapture:!0})}},playing:{phasedRegistrationNames:{bubbled:E({onPlaying:!0}),captured:E({onPlayingCapture:!0})}},progress:{phasedRegistrationNames:{bubbled:E({onProgress:!0}),captured:E({onProgressCapture:!0})}},rateChange:{phasedRegistrationNames:{bubbled:E({onRateChange:!0}),captured:E({onRateChangeCapture:!0})}},reset:{phasedRegistrationNames:{bubbled:E({onReset:!0}),captured:E({onResetCapture:!0})}},scroll:{phasedRegistrationNames:{bubbled:E({onScroll:!0}),captured:E({onScrollCapture:!0})}},seeked:{phasedRegistrationNames:{bubbled:E({onSeeked:!0}),captured:E({onSeekedCapture:!0})}},seeking:{phasedRegistrationNames:{bubbled:E({onSeeking:!0}),captured:E({onSeekingCapture:!0})}},stalled:{phasedRegistrationNames:{bubbled:E({onStalled:!0}),captured:E({onStalledCapture:!0})}},submit:{phasedRegistrationNames:{bubbled:E({onSubmit:!0}),captured:E({onSubmitCapture:!0})}},suspend:{phasedRegistrationNames:{bubbled:E({onSuspend:!0}),captured:E({onSuspendCapture:!0})}},timeUpdate:{phasedRegistrationNames:{bubbled:E({onTimeUpdate:!0}),captured:E({onTimeUpdateCapture:!0})}},touchCancel:{phasedRegistrationNames:{bubbled:E({onTouchCancel:!0}),captured:E({onTouchCancelCapture:!0})}},touchEnd:{phasedRegistrationNames:{bubbled:E({onTouchEnd:!0}),captured:E({onTouchEndCapture:!0})}},touchMove:{phasedRegistrationNames:{bubbled:E({onTouchMove:!0}),captured:E({onTouchMoveCapture:!0})}},touchStart:{phasedRegistrationNames:{bubbled:E({onTouchStart:!0}),captured:E({onTouchStartCapture:!0})}},transitionEnd:{phasedRegistrationNames:{bubbled:E({onTransitionEnd:!0}),captured:E({onTransitionEndCapture:!0})}},volumeChange:{phasedRegistrationNames:{bubbled:E({onVolumeChange:!0}),captured:E({onVolumeChangeCapture:!0})}},waiting:{phasedRegistrationNames:{bubbled:E({onWaiting:!0}),captured:E({onWaitingCapture:!0})}},wheel:{phasedRegistrationNames:{bubbled:E({onWheel:!0}),captured:E({onWheelCapture:!0})}}},N={topAbort:T.abort,topAnimationEnd:T.animationEnd,topAnimationIteration:T.animationIteration,topAnimationStart:T.animationStart,topBlur:T.blur,topCanPlay:T.canPlay,topCanPlayThrough:T.canPlayThrough,topClick:T.click,topContextMenu:T.contextMenu,topCopy:T.copy,topCut:T.cut,topDoubleClick:T.doubleClick,topDrag:T.drag,topDragEnd:T.dragEnd,topDragEnter:T.dragEnter,topDragExit:T.dragExit,topDragLeave:T.dragLeave,topDragOver:T.dragOver,topDragStart:T.dragStart,topDrop:T.drop,topDurationChange:T.durationChange,topEmptied:T.emptied,topEncrypted:T.encrypted,topEnded:T.ended,topError:T.error,topFocus:T.focus,topInput:T.input,topInvalid:T.invalid,topKeyDown:T.keyDown,topKeyPress:T.keyPress,topKeyUp:T.keyUp,topLoad:T.load,topLoadedData:T.loadedData,topLoadedMetadata:T.loadedMetadata,topLoadStart:T.loadStart,topMouseDown:T.mouseDown,topMouseMove:T.mouseMove,topMouseOut:T.mouseOut,topMouseOver:T.mouseOver,topMouseUp:T.mouseUp,topPaste:T.paste,topPause:T.pause,topPlay:T.play,topPlaying:T.playing,topProgress:T.progress,topRateChange:T.rateChange,topReset:T.reset,topScroll:T.scroll,topSeeked:T.seeked,topSeeking:T.seeking,topStalled:T.stalled,topSubmit:T.submit,topSuspend:T.suspend,topTimeUpdate:T.timeUpdate,topTouchCancel:T.touchCancel,topTouchEnd:T.touchEnd,topTouchMove:T.touchMove,topTouchStart:T.touchStart,topTransitionEnd:T.transitionEnd,topVolumeChange:T.volumeChange,topWaiting:T.waiting,topWheel:T.wheel};for(var w in N)N[w].dependencies=[w];var P=E({onClick:null}),k={},M={eventTypes:T,extractEvents:function(e,t,n,r){var a=N[e];if(!a)return null;var i;switch(e){case x.topAbort:case x.topCanPlay:case x.topCanPlayThrough:case x.topDurationChange:case x.topEmptied:case x.topEncrypted:case x.topEnded:case x.topError:case x.topInput:case x.topInvalid:case x.topLoad:case x.topLoadedData:case x.topLoadedMetadata:case x.topLoadStart:case x.topPause:case x.topPlay:case x.topPlaying:case x.topProgress:case x.topRateChange:case x.topReset:case x.topSeeked:case x.topSeeking:case x.topStalled:case x.topSubmit:case x.topSuspend:case x.topTimeUpdate:case x.topVolumeChange:case x.topWaiting:i=p;break;case x.topKeyPress:if(0===_(n))return null;case x.topKeyDown:case x.topKeyUp:i=f;break;case x.topBlur:case x.topFocus:i=d;break;case x.topClick:if(2===n.button)return null;case x.topContextMenu:case x.topDoubleClick:case x.topMouseDown:case x.topMouseMove:case x.topMouseOut:case x.topMouseOver:case x.topMouseUp:i=h;break;case x.topDrag:case x.topDragEnd:case x.topDragEnter:case x.topDragExit:case x.topDragLeave:case x.topDragOver:case x.topDragStart:case x.topDrop:i=m;break;case x.topTouchCancel:case x.topTouchEnd:case x.topTouchMove:case x.topTouchStart:i=v;break;case x.topAnimationEnd:case x.topAnimationIteration:case x.topAnimationStart:i=l;break;case x.topTransitionEnd:i=g;break;case x.topScroll:i=y;break;case x.topWheel:i=b;break;case x.topCopy:case x.topCut:case x.topPaste:i=c}i?void 0:o("86",e);var u=i.getPooled(a,t,n,r);return s.accumulateTwoPhaseDispatches(u),u},didPutListener:function(e,t,n){if(t===P){var o=r(e),a=u.getNodeFromInstance(e);k[o]||(k[o]=i.listen(a,"click",C))}},willDeleteListener:function(e,t){if(t===P){var n=r(e);k[n].remove(),delete k[n]}}};t.exports=M},{100:100,101:101,102:102,103:103,104:104,105:105,118:118,132:132,139:139,146:146,154:154,158:158,16:16,20:20,40:40,93:93,94:94,96:96,97:97,98:98}],93:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={animationName:null,elapsedTime:null,pseudoElement:null};o.augmentClass(r,a),t.exports=r},{97:97}],94:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}};o.augmentClass(r,a),t.exports=r},{97:97}],95:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={data:null};o.augmentClass(r,a),t.exports=r},{97:97}],96:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(101),a={dataTransfer:null};o.augmentClass(r,a),t.exports=r},{101:101}],97:[function(e,t,n){"use strict";function r(e,t,n,r){this.dispatchConfig=e,this._targetInst=t,this.nativeEvent=n;var o=this.constructor.Interface;for(var a in o)if(o.hasOwnProperty(a)){var s=o[a];s?this[a]=s(n):"target"===a?this.target=r:this[a]=n[a]}var u=null!=n.defaultPrevented?n.defaultPrevented:n.returnValue===!1;return u?this.isDefaultPrevented=i.thatReturnsTrue:this.isDefaultPrevented=i.thatReturnsFalse,this.isPropagationStopped=i.thatReturnsFalse,this}var o=e(162),a=e(25),i=e(146),s=(e(161),"function"==typeof Proxy,["dispatchConfig","_targetInst","nativeEvent","isDefaultPrevented","isPropagationStopped","_dispatchListeners","_dispatchInstances"]),u={type:null,target:null,currentTarget:i.thatReturnsNull,eventPhase:null,bubbles:null,cancelable:null,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:null,isTrusted:null};o(r.prototype,{preventDefault:function(){this.defaultPrevented=!0;var e=this.nativeEvent;e&&(e.preventDefault?e.preventDefault():e.returnValue=!1,this.isDefaultPrevented=i.thatReturnsTrue)},stopPropagation:function(){var e=this.nativeEvent;e&&(e.stopPropagation?e.stopPropagation():"unknown"!=typeof e.cancelBubble&&(e.cancelBubble=!0),this.isPropagationStopped=i.thatReturnsTrue)},persist:function(){this.isPersistent=i.thatReturnsTrue},isPersistent:i.thatReturnsFalse,destructor:function(){var e=this.constructor.Interface;for(var t in e)this[t]=null;for(var n=0;n<s.length;n++)this[s[n]]=null}}),r.Interface=u,r.augmentClass=function(e,t){var n=this,r=function(){};r.prototype=n.prototype;var i=new r;o(i,e.prototype),e.prototype=i,e.prototype.constructor=e,e.Interface=o({},n.Interface,t),e.augmentClass=n.augmentClass,a.addPoolingTo(e,a.fourArgumentPooler)},a.addPoolingTo(r,a.fourArgumentPooler),t.exports=r},{146:146,161:161,162:162,25:25}],98:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a={relatedTarget:null};o.augmentClass(r,a),t.exports=r},{104:104}],99:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={data:null};o.augmentClass(r,a),t.exports=r},{97:97}],100:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a=e(118),i=e(119),s=e(120),u={key:i,location:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,repeat:null,locale:null,getModifierState:s,charCode:function(e){return"keypress"===e.type?a(e):0},keyCode:function(e){return"keydown"===e.type||"keyup"===e.type?e.keyCode:0},which:function(e){return"keypress"===e.type?a(e):"keydown"===e.type||"keyup"===e.type?e.keyCode:0}};o.augmentClass(r,u),t.exports=r},{104:104,118:118,119:119,120:120}],101:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a=e(107),i=e(120),s={screenX:null,screenY:null,clientX:null,clientY:null,ctrlKey:null,shiftKey:null,altKey:null,metaKey:null,getModifierState:i,button:function(e){var t=e.button;return"which"in e?t:2===t?2:4===t?1:0},buttons:null,relatedTarget:function(e){return e.relatedTarget||(e.fromElement===e.srcElement?e.toElement:e.fromElement)},pageX:function(e){return"pageX"in e?e.pageX:e.clientX+a.currentScrollLeft},pageY:function(e){return"pageY"in e?e.pageY:e.clientY+a.currentScrollTop}};o.augmentClass(r,s),t.exports=r},{104:104,107:107,120:120}],102:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(104),a=e(120),i={touches:null,targetTouches:null,changedTouches:null,altKey:null,metaKey:null,ctrlKey:null,shiftKey:null,getModifierState:a};o.augmentClass(r,i),t.exports=r},{104:104,120:120}],103:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a={propertyName:null,elapsedTime:null,pseudoElement:null};o.augmentClass(r,a),t.exports=r},{97:97}],104:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(97),a=e(121),i={view:function(e){if(e.view)return e.view;var t=a(e);if(t.window===t)return t;var n=t.ownerDocument;return n?n.defaultView||n.parentWindow:window},detail:function(e){return e.detail||0}};o.augmentClass(r,i),
t.exports=r},{121:121,97:97}],105:[function(e,t,n){"use strict";function r(e,t,n,r){return o.call(this,e,t,n,r)}var o=e(101),a={deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:null,deltaMode:null};o.augmentClass(r,a),t.exports=r},{101:101}],106:[function(e,t,n){"use strict";var r=e(132),o=(e(154),{reinitializeTransaction:function(){this.transactionWrappers=this.getTransactionWrappers(),this.wrapperInitData?this.wrapperInitData.length=0:this.wrapperInitData=[],this._isInTransaction=!1},_isInTransaction:!1,getTransactionWrappers:null,isInTransaction:function(){return!!this._isInTransaction},perform:function(e,t,n,o,a,i,s,u){this.isInTransaction()?r("27"):void 0;var l,c;try{this._isInTransaction=!0,l=!0,this.initializeAll(0),c=e.call(t,n,o,a,i,s,u),l=!1}finally{try{if(l)try{this.closeAll(0)}catch(e){}else this.closeAll(0)}finally{this._isInTransaction=!1}}return c},initializeAll:function(e){for(var t=this.transactionWrappers,n=e;n<t.length;n++){var r=t[n];try{this.wrapperInitData[n]=a.OBSERVED_ERROR,this.wrapperInitData[n]=r.initialize?r.initialize.call(this):null}finally{if(this.wrapperInitData[n]===a.OBSERVED_ERROR)try{this.initializeAll(n+1)}catch(e){}}}},closeAll:function(e){this.isInTransaction()?void 0:r("28");for(var t=this.transactionWrappers,n=e;n<t.length;n++){var o,i=t[n],s=this.wrapperInitData[n];try{o=!0,s!==a.OBSERVED_ERROR&&i.close&&i.close.call(this,s),o=!1}finally{if(o)try{this.closeAll(n+1)}catch(e){}}}this.wrapperInitData.length=0}}),a={Mixin:o,OBSERVED_ERROR:{}};t.exports=a},{132:132,154:154}],107:[function(e,t,n){"use strict";var r={currentScrollLeft:0,currentScrollTop:0,refreshScrollValues:function(e){r.currentScrollLeft=e.x,r.currentScrollTop=e.y}};t.exports=r},{}],108:[function(e,t,n){"use strict";function r(e,t){return null==t?o("30"):void 0,null==e?t:Array.isArray(e)?Array.isArray(t)?(e.push.apply(e,t),e):(e.push(t),e):Array.isArray(t)?[e].concat(t):[e,t]}var o=e(132);e(154);t.exports=r},{132:132,154:154}],109:[function(e,t,n){"use strict";function r(e){for(var t=1,n=0,r=0,a=e.length,i=a&-4;r<i;){for(var s=Math.min(r+4096,i);r<s;r+=4)n+=(t+=e.charCodeAt(r))+(t+=e.charCodeAt(r+1))+(t+=e.charCodeAt(r+2))+(t+=e.charCodeAt(r+3));t%=o,n%=o}for(;r<a;r++)n+=t+=e.charCodeAt(r);return t%=o,n%=o,t|n<<16}var o=65521;t.exports=r},{}],110:[function(e,t,n){"use strict";var r=!1;t.exports=r},{}],111:[function(e,t,n){(function(n){"use strict";function r(e,t,n,r,u,l){for(var c in e)if(e.hasOwnProperty(c)){var p;try{"function"!=typeof e[c]?o("84",r||"React class",a[n],c):void 0,p=e[c](t,c,r,n,null,i)}catch(e){p=e}p instanceof Error&&!(p.message in s)&&(s[p.message]=!0)}}var o=e(132),a=e(74),i=e(77);e(154),e(161);"undefined"!=typeof n&&n.env,1;var s={};t.exports=r}).call(this,void 0)},{132:132,154:154,161:161,74:74,77:77}],112:[function(e,t,n){"use strict";var r=function(e){return"undefined"!=typeof MSApp&&MSApp.execUnsafeLocalFunction?function(t,n,r,o){MSApp.execUnsafeLocalFunction(function(){return e(t,n,r,o)})}:e};t.exports=r},{}],113:[function(e,t,n){"use strict";function r(e,t,n){var r=null==t||"boolean"==typeof t||""===t;if(r)return"";var o=isNaN(t);return o||0===t||a.hasOwnProperty(e)&&a[e]?""+t:("string"==typeof t&&(t=t.trim()),t+"px")}var o=e(3),a=(e(161),o.isUnitlessNumber);t.exports=r},{161:161,3:3}],114:[function(e,t,n){"use strict";function r(e){var t=""+e,n=a.exec(t);if(!n)return t;var r,o="",i=0,s=0;for(i=n.index;i<t.length;i++){switch(t.charCodeAt(i)){case 34:r="&quot;";break;case 38:r="&amp;";break;case 39:r="&#x27;";break;case 60:r="&lt;";break;case 62:r="&gt;";break;default:continue}s!==i&&(o+=t.substring(s,i)),s=i+1,o+=r}return s!==i?o+t.substring(s,i):o}function o(e){return"boolean"==typeof e||"number"==typeof e?""+e:r(e)}var a=/["'&<>]/;t.exports=o},{}],115:[function(e,t,n){"use strict";function r(e){if(null==e)return null;if(1===e.nodeType)return e;var t=i.get(e);return t?(t=s(t),t?a.getNodeFromInstance(t):null):void("function"==typeof e.render?o("44"):o("45",Object.keys(e)))}var o=e(132),a=(e(35),e(40)),i=e(65),s=e(122);e(154),e(161);t.exports=r},{122:122,132:132,154:154,161:161,35:35,40:40,65:65}],116:[function(e,t,n){(function(n){"use strict";function r(e,t,n,r){if(e&&"object"==typeof e){var o=e,a=void 0===o[n];a&&null!=t&&(o[n]=t)}}function o(e,t){if(null==e)return e;var n={};return a(e,r,n),n}var a=(e(23),e(137));e(161);"undefined"!=typeof n&&n.env,t.exports=o}).call(this,void 0)},{137:137,161:161,23:23}],117:[function(e,t,n){"use strict";function r(e,t,n){Array.isArray(e)?e.forEach(t,n):e&&t.call(n,e)}t.exports=r},{}],118:[function(e,t,n){"use strict";function r(e){var t,n=e.keyCode;return"charCode"in e?(t=e.charCode,0===t&&13===n&&(t=13)):t=n,t>=32||13===t?t:0}t.exports=r},{}],119:[function(e,t,n){"use strict";function r(e){if(e.key){var t=a[e.key]||e.key;if("Unidentified"!==t)return t}if("keypress"===e.type){var n=o(e);return 13===n?"Enter":String.fromCharCode(n)}return"keydown"===e.type||"keyup"===e.type?i[e.keyCode]||"Unidentified":""}var o=e(118),a={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},i={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"};t.exports=r},{118:118}],120:[function(e,t,n){"use strict";function r(e){var t=this,n=t.nativeEvent;if(n.getModifierState)return n.getModifierState(e);var r=a[e];return!!r&&!!n[r]}function o(e){return r}var a={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};t.exports=o},{}],121:[function(e,t,n){"use strict";function r(e){var t=e.target||e.srcElement||window;return t.correspondingUseElement&&(t=t.correspondingUseElement),3===t.nodeType?t.parentNode:t}t.exports=r},{}],122:[function(e,t,n){"use strict";function r(e){for(var t;(t=e._renderedNodeType)===o.COMPOSITE;)e=e._renderedComponent;return t===o.HOST?e._renderedComponent:t===o.EMPTY?null:void 0}var o=e(71);t.exports=r},{71:71}],123:[function(e,t,n){"use strict";function r(e){var t=e&&(o&&e[o]||e[a]);if("function"==typeof t)return t}var o="function"==typeof Symbol&&Symbol.iterator,a="@@iterator";t.exports=r},{}],124:[function(e,t,n){"use strict";function r(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function o(e){for(;e;){if(e.nextSibling)return e.nextSibling;e=e.parentNode}}function a(e,t){for(var n=r(e),a=0,i=0;n;){if(3===n.nodeType){if(i=a+n.textContent.length,a<=t&&i>=t)return{node:n,offset:t-a};a=i}n=r(o(n))}}t.exports=a},{}],125:[function(e,t,n){"use strict";function r(){return!a&&o.canUseDOM&&(a="textContent"in document.documentElement?"textContent":"innerText"),a}var o=e(140),a=null;t.exports=r},{140:140}],126:[function(e,t,n){"use strict";function r(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n["ms"+e]="MS"+t,n["O"+e]="o"+t.toLowerCase(),n}function o(e){if(s[e])return s[e];if(!i[e])return e;var t=i[e];for(var n in t)if(t.hasOwnProperty(n)&&n in u)return s[e]=t[n];return""}var a=e(140),i={animationend:r("Animation","AnimationEnd"),animationiteration:r("Animation","AnimationIteration"),animationstart:r("Animation","AnimationStart"),transitionend:r("Transition","TransitionEnd")},s={},u={};a.canUseDOM&&(u=document.createElement("div").style,"AnimationEvent"in window||(delete i.animationend.animation,delete i.animationiteration.animation,delete i.animationstart.animation),"TransitionEvent"in window||delete i.transitionend.transition),t.exports=o},{140:140}],127:[function(e,t,n){"use strict";function r(e){if(e){var t=e.getName();if(t)return" Check the render method of `"+t+"`."}return""}function o(e){return"function"==typeof e&&"undefined"!=typeof e.prototype&&"function"==typeof e.prototype.mountComponent&&"function"==typeof e.prototype.receiveComponent}function a(e,t){var n;if(null===e||e===!1)n=l.create(a);else if("object"==typeof e){var s=e;!s||"function"!=typeof s.type&&"string"!=typeof s.type?i("130",null==s.type?s.type:typeof s.type,r(s._owner)):void 0,"string"==typeof s.type?n=c.createInternalComponent(s):o(s.type)?(n=new s.type(s),n.getHostNode||(n.getHostNode=n.getNativeNode)):n=new p(s)}else"string"==typeof e||"number"==typeof e?n=c.createInstanceForText(e):i("131",typeof e);return n._mountIndex=0,n._mountImage=null,n}var i=e(132),s=e(162),u=e(34),l=e(57),c=e(62),p=(e(154),e(161),function(e){this.construct(e)});s(p.prototype,u.Mixin,{_instantiateReactComponent:a});t.exports=a},{132:132,154:154,161:161,162:162,34:34,57:57,62:62}],128:[function(e,t,n){"use strict";function r(e,t){if(!a.canUseDOM||t&&!("addEventListener"in document))return!1;var n="on"+e,r=n in document;if(!r){var i=document.createElement("div");i.setAttribute(n,"return;"),r="function"==typeof i[n]}return!r&&o&&"wheel"===e&&(r=document.implementation.hasFeature("Events.wheel","3.0")),r}var o,a=e(140);a.canUseDOM&&(o=document.implementation&&document.implementation.hasFeature&&document.implementation.hasFeature("","")!==!0),t.exports=r},{140:140}],129:[function(e,t,n){"use strict";function r(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return"input"===t?!!o[e.type]:"textarea"===t}var o={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};t.exports=r},{}],130:[function(e,t,n){"use strict";function r(e){return a.isValidElement(e)?void 0:o("143"),e}var o=e(132),a=e(56);e(154);t.exports=r},{132:132,154:154,56:56}],131:[function(e,t,n){"use strict";function r(e){return'"'+o(e)+'"'}var o=e(114);t.exports=r},{114:114}],132:[function(e,t,n){"use strict";function r(e){for(var t=arguments.length-1,n="Minified React error #"+e+"; visit http://facebook.github.io/react/docs/error-decoder.html?invariant="+e,r=0;r<t;r++)n+="&args[]="+encodeURIComponent(arguments[r+1]);n+=" for the full message or use the non-minified dev environment for full errors and additional helpful warnings.";var o=new Error(n);throw o.name="Invariant Violation",o.framesToPop=1,o}t.exports=r},{}],133:[function(e,t,n){"use strict";var r=e(68);t.exports=r.renderSubtreeIntoContainer},{68:68}],134:[function(e,t,n){"use strict";var r,o=e(140),a=e(9),i=/^[ \r\n\t\f]/,s=/<(!--|link|noscript|meta|script|style)[ \r\n\t\f\/>]/,u=e(112),l=u(function(e,t){if(e.namespaceURI!==a.svg||"innerHTML"in e)e.innerHTML=t;else{r=r||document.createElement("div"),r.innerHTML="<svg>"+t+"</svg>";for(var n=r.firstChild.childNodes,o=0;o<n.length;o++)e.appendChild(n[o])}});if(o.canUseDOM){var c=document.createElement("div");c.innerHTML=" ",""===c.innerHTML&&(l=function(e,t){if(e.parentNode&&e.parentNode.replaceChild(e,e),i.test(t)||"<"===t[0]&&s.test(t)){e.innerHTML=String.fromCharCode(65279)+t;var n=e.firstChild;1===n.data.length?e.removeChild(n):n.deleteData(0,1)}else e.innerHTML=t}),c=null}t.exports=l},{112:112,140:140,9:9}],135:[function(e,t,n){"use strict";var r=e(140),o=e(114),a=e(134),i=function(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&3===n.nodeType)return void(n.nodeValue=t)}e.textContent=t};r.canUseDOM&&("textContent"in document.documentElement||(i=function(e,t){a(e,o(t))})),t.exports=i},{114:114,134:134,140:140}],136:[function(e,t,n){"use strict";function r(e,t){var n=null===e||e===!1,r=null===t||t===!1;if(n||r)return n===r;var o=typeof e,a=typeof t;return"string"===o||"number"===o?"string"===a||"number"===a:"object"===a&&e.type===t.type&&e.key===t.key}t.exports=r},{}],137:[function(e,t,n){"use strict";function r(e,t){return e&&"object"==typeof e&&null!=e.key?l.escape(e.key):t.toString(36)}function o(e,t,n,a){var d=typeof e;if("undefined"!==d&&"boolean"!==d||(e=null),null===e||"string"===d||"number"===d||s.isValidElement(e))return n(a,e,""===t?c+r(e,0):t),1;var f,h,m=0,v=""===t?c:t+p;if(Array.isArray(e))for(var g=0;g<e.length;g++)f=e[g],h=v+r(f,g),m+=o(f,h,n,a);else{var y=u(e);if(y){var b,C=y.call(e);if(y!==e.entries)for(var _=0;!(b=C.next()).done;)f=b.value,h=v+r(f,_++),m+=o(f,h,n,a);else for(;!(b=C.next()).done;){var E=b.value;E&&(f=E[1],h=v+l.escape(E[0])+p+r(f,0),m+=o(f,h,n,a))}}else if("object"===d){var x="",T=String(e);i("31","[object Object]"===T?"object with keys {"+Object.keys(e).join(", ")+"}":T,x)}}return m}function a(e,t,n){return null==e?0:o(e,"",t,n)}var i=e(132),s=(e(35),e(56)),u=e(123),l=(e(154),e(23)),c=(e(161),"."),p=":";t.exports=a},{123:123,132:132,154:154,161:161,23:23,35:35,56:56}],138:[function(e,t,n){"use strict";var r=(e(162),e(146)),o=(e(161),r);t.exports=o},{146:146,161:161,162:162}],139:[function(e,t,n){"use strict";var r=e(146),o={listen:function(e,t,n){return e.addEventListener?(e.addEventListener(t,n,!1),{remove:function(){e.removeEventListener(t,n,!1)}}):e.attachEvent?(e.attachEvent("on"+t,n),{remove:function(){e.detachEvent("on"+t,n)}}):void 0},capture:function(e,t,n){return e.addEventListener?(e.addEventListener(t,n,!0),{remove:function(){e.removeEventListener(t,n,!0)}}):{remove:r}},registerDefault:function(){}};t.exports=o},{146:146}],140:[function(e,t,n){"use strict";var r=!("undefined"==typeof window||!window.document||!window.document.createElement),o={canUseDOM:r,canUseWorkers:"undefined"!=typeof Worker,canUseEventListeners:r&&!(!window.addEventListener&&!window.attachEvent),canUseViewport:r&&!!window.screen,isInWorker:!r};t.exports=o},{}],141:[function(e,t,n){"use strict";function r(e){return e.replace(o,function(e,t){return t.toUpperCase()})}var o=/-(.)/g;t.exports=r},{}],142:[function(e,t,n){"use strict";function r(e){return o(e.replace(a,"ms-"))}var o=e(141),a=/^-ms-/;t.exports=r},{141:141}],143:[function(e,t,n){"use strict";function r(e,t){return!(!e||!t)&&(e===t||!o(e)&&(o(t)?r(e,t.parentNode):"contains"in e?e.contains(t):!!e.compareDocumentPosition&&!!(16&e.compareDocumentPosition(t))))}var o=e(156);t.exports=r},{156:156}],144:[function(e,t,n){"use strict";function r(e){var t=e.length;if(Array.isArray(e)||"object"!=typeof e&&"function"!=typeof e?i(!1):void 0,"number"!=typeof t?i(!1):void 0,0===t||t-1 in e?void 0:i(!1),"function"==typeof e.callee?i(!1):void 0,e.hasOwnProperty)try{return Array.prototype.slice.call(e)}catch(e){}for(var n=Array(t),r=0;r<t;r++)n[r]=e[r];return n}function o(e){return!!e&&("object"==typeof e||"function"==typeof e)&&"length"in e&&!("setInterval"in e)&&"number"!=typeof e.nodeType&&(Array.isArray(e)||"callee"in e||"item"in e)}function a(e){return o(e)?Array.isArray(e)?e.slice():r(e):[e]}var i=e(154);t.exports=a},{154:154}],145:[function(e,t,n){"use strict";function r(e){var t=e.match(c);return t&&t[1].toLowerCase()}function o(e,t){var n=l;l?void 0:u(!1);var o=r(e),a=o&&s(o);if(a){n.innerHTML=a[1]+e+a[2];for(var c=a[0];c--;)n=n.lastChild}else n.innerHTML=e;var p=n.getElementsByTagName("script");p.length&&(t?void 0:u(!1),i(p).forEach(t));for(var d=Array.from(n.childNodes);n.lastChild;)n.removeChild(n.lastChild);return d}var a=e(140),i=e(144),s=e(150),u=e(154),l=a.canUseDOM?document.createElement("div"):null,c=/^\s*<(\w+)/;t.exports=o},{140:140,144:144,150:150,154:154}],146:[function(e,t,n){"use strict";function r(e){return function(){return e}}var o=function(){};o.thatReturns=r,o.thatReturnsFalse=r(!1),o.thatReturnsTrue=r(!0),o.thatReturnsNull=r(null),o.thatReturnsThis=function(){return this},o.thatReturnsArgument=function(e){return e},t.exports=o},{}],147:[function(e,t,n){"use strict";var r={};t.exports=r},{}],148:[function(e,t,n){"use strict";function r(e){try{e.focus()}catch(e){}}t.exports=r},{}],149:[function(e,t,n){"use strict";function r(){if("undefined"==typeof document)return null;try{return document.activeElement||document.body}catch(e){return document.body}}t.exports=r},{}],150:[function(e,t,n){"use strict";function r(e){return i?void 0:a(!1),d.hasOwnProperty(e)||(e="*"),s.hasOwnProperty(e)||("*"===e?i.innerHTML="<link />":i.innerHTML="<"+e+"></"+e+">",s[e]=!i.firstChild),s[e]?d[e]:null}var o=e(140),a=e(154),i=o.canUseDOM?document.createElement("div"):null,s={},u=[1,'<select multiple="true">',"</select>"],l=[1,"<table>","</table>"],c=[3,"<table><tbody><tr>","</tr></tbody></table>"],p=[1,'<svg xmlns="http://www.w3.org/2000/svg">',"</svg>"],d={"*":[1,"?<div>","</div>"],area:[1,"<map>","</map>"],col:[2,"<table><tbody></tbody><colgroup>","</colgroup></table>"],legend:[1,"<fieldset>","</fieldset>"],param:[1,"<object>","</object>"],tr:[2,"<table><tbody>","</tbody></table>"],optgroup:u,option:u,caption:l,colgroup:l,tbody:l,tfoot:l,thead:l,td:c,th:c},f=["circle","clipPath","defs","ellipse","g","image","line","linearGradient","mask","path","pattern","polygon","polyline","radialGradient","rect","stop","text","tspan"];f.forEach(function(e){d[e]=p,s[e]=!0}),t.exports=r},{140:140,154:154}],151:[function(e,t,n){"use strict";function r(e){return e===window?{x:window.pageXOffset||document.documentElement.scrollLeft,y:window.pageYOffset||document.documentElement.scrollTop}:{x:e.scrollLeft,y:e.scrollTop}}t.exports=r},{}],152:[function(e,t,n){"use strict";function r(e){return e.replace(o,"-$1").toLowerCase()}var o=/([A-Z])/g;t.exports=r},{}],153:[function(e,t,n){"use strict";function r(e){return o(e).replace(a,"-ms-")}var o=e(152),a=/^ms-/;t.exports=r},{152:152}],154:[function(e,t,n){"use strict";function r(e,t,n,r,o,a,i,s){if(!e){var u;if(void 0===t)u=new Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var l=[n,r,o,a,i,s],c=0;u=new Error(t.replace(/%s/g,function(){return l[c++]})),u.name="Invariant Violation"}throw u.framesToPop=1,u}}t.exports=r},{}],155:[function(e,t,n){"use strict";function r(e){return!(!e||!("function"==typeof Node?e instanceof Node:"object"==typeof e&&"number"==typeof e.nodeType&&"string"==typeof e.nodeName))}t.exports=r},{}],156:[function(e,t,n){"use strict";function r(e){return o(e)&&3==e.nodeType}var o=e(155);t.exports=r},{155:155}],157:[function(e,t,n){"use strict";var r=e(154),o=function(e){var t,n={};e instanceof Object&&!Array.isArray(e)?void 0:r(!1);for(t in e)e.hasOwnProperty(t)&&(n[t]=t);return n};t.exports=o},{154:154}],158:[function(e,t,n){"use strict";var r=function(e){var t;for(t in e)if(e.hasOwnProperty(t))return t;return null};t.exports=r},{}],159:[function(e,t,n){"use strict";function r(e){var t={};return function(n){return t.hasOwnProperty(n)||(t[n]=e.call(this,n)),t[n]}}t.exports=r},{}],160:[function(e,t,n){"use strict";function r(e,t){return e===t?0!==e||1/e===1/t:e!==e&&t!==t}function o(e,t){if(r(e,t))return!0;if("object"!=typeof e||null===e||"object"!=typeof t||null===t)return!1;var n=Object.keys(e),o=Object.keys(t);if(n.length!==o.length)return!1;for(var i=0;i<n.length;i++)if(!a.call(t,n[i])||!r(e[n[i]],t[n[i]]))return!1;return!0}var a=Object.prototype.hasOwnProperty;t.exports=o},{}],161:[function(e,t,n){"use strict";var r=e(146),o=r;t.exports=o},{146:146}],162:[function(e,t,n){"use strict";function r(e){if(null===e||void 0===e)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(e)}function o(){try{if(!Object.assign)return!1;var e=new String("abc");if(e[5]="de","5"===Object.getOwnPropertyNames(e)[0])return!1;for(var t={},n=0;n<10;n++)t["_"+String.fromCharCode(n)]=n;var r=Object.getOwnPropertyNames(t).map(function(e){return t[e]});if("0123456789"!==r.join(""))return!1;var o={};return"abcdefghijklmnopqrst".split("").forEach(function(e){o[e]=e}),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},o)).join("")}catch(e){return!1}}var a=Object.prototype.hasOwnProperty,i=Object.prototype.propertyIsEnumerable;t.exports=o()?Object.assign:function(e,t){for(var n,o,s=r(e),u=1;u<arguments.length;u++){n=Object(arguments[u]);for(var l in n)a.call(n,l)&&(s[l]=n[l]);if(Object.getOwnPropertySymbols){o=Object.getOwnPropertySymbols(n);for(var c=0;c<o.length;c++)i.call(n,o[c])&&(s[o[c]]=n[o[c]])}}return s}},{}]},{},[86])(86)});
/**
 * ReactDOMServer v15.2.1
 *
 * Copyright 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 */
!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e(require("react"));else if("function"==typeof define&&define.amd)define(["react"],e);else{var f;f="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,f.ReactDOMServer=e(f.React)}}(function(e){return e.__SECRET_DOM_SERVER_DO_NOT_USE_OR_YOU_WILL_BE_FIRED});
/**
 * ReactDOM v15.2.1
 *
 * Copyright 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 */
!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e(require("react"));else if("function"==typeof define&&define.amd)define(["react"],e);else{var f;f="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:this,f.ReactDOM=e(f.React)}}(function(e){return e.__SECRET_DOM_DO_NOT_USE_OR_YOU_WILL_BE_FIRED});
!function(){function n(n){return n&&(n.ownerDocument||n.document||n).documentElement}function t(n){return n&&(n.ownerDocument&&n.ownerDocument.defaultView||n.document&&n||n.defaultView)}function e(n,t){return t>n?-1:n>t?1:n>=t?0:0/0}function r(n){return null===n?0/0:+n}function u(n){return!isNaN(n)}function i(n){return{left:function(t,e,r,u){for(arguments.length<3&&(r=0),arguments.length<4&&(u=t.length);u>r;){var i=r+u>>>1;n(t[i],e)<0?r=i+1:u=i}return r},right:function(t,e,r,u){for(arguments.length<3&&(r=0),arguments.length<4&&(u=t.length);u>r;){var i=r+u>>>1;n(t[i],e)>0?u=i:r=i+1}return r}}}function o(n){return n.length}function a(n){for(var t=1;n*t%1;)t*=10;return t}function c(n,t){for(var e in t)Object.defineProperty(n.prototype,e,{value:t[e],enumerable:!1})}function l(){this._=Object.create(null)}function s(n){return(n+="")===pa||n[0]===va?va+n:n}function f(n){return(n+="")[0]===va?n.slice(1):n}function h(n){return s(n)in this._}function g(n){return(n=s(n))in this._&&delete this._[n]}function p(){var n=[];for(var t in this._)n.push(f(t));return n}function v(){var n=0;for(var t in this._)++n;return n}function d(){for(var n in this._)return!1;return!0}function m(){this._=Object.create(null)}function y(n){return n}function M(n,t,e){return function(){var r=e.apply(t,arguments);return r===t?n:r}}function x(n,t){if(t in n)return t;t=t.charAt(0).toUpperCase()+t.slice(1);for(var e=0,r=da.length;r>e;++e){var u=da[e]+t;if(u in n)return u}}function b(){}function _(){}function w(n){function t(){for(var t,r=e,u=-1,i=r.length;++u<i;)(t=r[u].on)&&t.apply(this,arguments);return n}var e=[],r=new l;return t.on=function(t,u){var i,o=r.get(t);return arguments.length<2?o&&o.on:(o&&(o.on=null,e=e.slice(0,i=e.indexOf(o)).concat(e.slice(i+1)),r.remove(t)),u&&e.push(r.set(t,{on:u})),n)},t}function S(){ta.event.preventDefault()}function k(){for(var n,t=ta.event;n=t.sourceEvent;)t=n;return t}function E(n){for(var t=new _,e=0,r=arguments.length;++e<r;)t[arguments[e]]=w(t);return t.of=function(e,r){return function(u){try{var i=u.sourceEvent=ta.event;u.target=n,ta.event=u,t[u.type].apply(e,r)}finally{ta.event=i}}},t}function A(n){return ya(n,_a),n}function N(n){return"function"==typeof n?n:function(){return Ma(n,this)}}function C(n){return"function"==typeof n?n:function(){return xa(n,this)}}function z(n,t){function e(){this.removeAttribute(n)}function r(){this.removeAttributeNS(n.space,n.local)}function u(){this.setAttribute(n,t)}function i(){this.setAttributeNS(n.space,n.local,t)}function o(){var e=t.apply(this,arguments);null==e?this.removeAttribute(n):this.setAttribute(n,e)}function a(){var e=t.apply(this,arguments);null==e?this.removeAttributeNS(n.space,n.local):this.setAttributeNS(n.space,n.local,e)}return n=ta.ns.qualify(n),null==t?n.local?r:e:"function"==typeof t?n.local?a:o:n.local?i:u}function q(n){return n.trim().replace(/\s+/g," ")}function L(n){return new RegExp("(?:^|\\s+)"+ta.requote(n)+"(?:\\s+|$)","g")}function T(n){return(n+"").trim().split(/^|\s+/)}function R(n,t){function e(){for(var e=-1;++e<u;)n[e](this,t)}function r(){for(var e=-1,r=t.apply(this,arguments);++e<u;)n[e](this,r)}n=T(n).map(D);var u=n.length;return"function"==typeof t?r:e}function D(n){var t=L(n);return function(e,r){if(u=e.classList)return r?u.add(n):u.remove(n);var u=e.getAttribute("class")||"";r?(t.lastIndex=0,t.test(u)||e.setAttribute("class",q(u+" "+n))):e.setAttribute("class",q(u.replace(t," ")))}}function P(n,t,e){function r(){this.style.removeProperty(n)}function u(){this.style.setProperty(n,t,e)}function i(){var r=t.apply(this,arguments);null==r?this.style.removeProperty(n):this.style.setProperty(n,r,e)}return null==t?r:"function"==typeof t?i:u}function U(n,t){function e(){delete this[n]}function r(){this[n]=t}function u(){var e=t.apply(this,arguments);null==e?delete this[n]:this[n]=e}return null==t?e:"function"==typeof t?u:r}function j(n){function t(){var t=this.ownerDocument,e=this.namespaceURI;return e?t.createElementNS(e,n):t.createElement(n)}function e(){return this.ownerDocument.createElementNS(n.space,n.local)}return"function"==typeof n?n:(n=ta.ns.qualify(n)).local?e:t}function F(){var n=this.parentNode;n&&n.removeChild(this)}function H(n){return{__data__:n}}function O(n){return function(){return ba(this,n)}}function I(n){return arguments.length||(n=e),function(t,e){return t&&e?n(t.__data__,e.__data__):!t-!e}}function Y(n,t){for(var e=0,r=n.length;r>e;e++)for(var u,i=n[e],o=0,a=i.length;a>o;o++)(u=i[o])&&t(u,o,e);return n}function Z(n){return ya(n,Sa),n}function V(n){var t,e;return function(r,u,i){var o,a=n[i].update,c=a.length;for(i!=e&&(e=i,t=0),u>=t&&(t=u+1);!(o=a[t])&&++t<c;);return o}}function X(n,t,e){function r(){var t=this[o];t&&(this.removeEventListener(n,t,t.$),delete this[o])}function u(){var u=c(t,ra(arguments));r.call(this),this.addEventListener(n,this[o]=u,u.$=e),u._=t}function i(){var t,e=new RegExp("^__on([^.]+)"+ta.requote(n)+"$");for(var r in this)if(t=r.match(e)){var u=this[r];this.removeEventListener(t[1],u,u.$),delete this[r]}}var o="__on"+n,a=n.indexOf("."),c=$;a>0&&(n=n.slice(0,a));var l=ka.get(n);return l&&(n=l,c=B),a?t?u:r:t?b:i}function $(n,t){return function(e){var r=ta.event;ta.event=e,t[0]=this.__data__;try{n.apply(this,t)}finally{ta.event=r}}}function B(n,t){var e=$(n,t);return function(n){var t=this,r=n.relatedTarget;r&&(r===t||8&r.compareDocumentPosition(t))||e.call(t,n)}}function W(e){var r=".dragsuppress-"+ ++Aa,u="click"+r,i=ta.select(t(e)).on("touchmove"+r,S).on("dragstart"+r,S).on("selectstart"+r,S);if(null==Ea&&(Ea="onselectstart"in e?!1:x(e.style,"userSelect")),Ea){var o=n(e).style,a=o[Ea];o[Ea]="none"}return function(n){if(i.on(r,null),Ea&&(o[Ea]=a),n){var t=function(){i.on(u,null)};i.on(u,function(){S(),t()},!0),setTimeout(t,0)}}}function J(n,e){e.changedTouches&&(e=e.changedTouches[0]);var r=n.ownerSVGElement||n;if(r.createSVGPoint){var u=r.createSVGPoint();if(0>Na){var i=t(n);if(i.scrollX||i.scrollY){r=ta.select("body").append("svg").style({position:"absolute",top:0,left:0,margin:0,padding:0,border:"none"},"important");var o=r[0][0].getScreenCTM();Na=!(o.f||o.e),r.remove()}}return Na?(u.x=e.pageX,u.y=e.pageY):(u.x=e.clientX,u.y=e.clientY),u=u.matrixTransform(n.getScreenCTM().inverse()),[u.x,u.y]}var a=n.getBoundingClientRect();return[e.clientX-a.left-n.clientLeft,e.clientY-a.top-n.clientTop]}function G(){return ta.event.changedTouches[0].identifier}function K(n){return n>0?1:0>n?-1:0}function Q(n,t,e){return(t[0]-n[0])*(e[1]-n[1])-(t[1]-n[1])*(e[0]-n[0])}function nt(n){return n>1?0:-1>n?qa:Math.acos(n)}function tt(n){return n>1?Ra:-1>n?-Ra:Math.asin(n)}function et(n){return((n=Math.exp(n))-1/n)/2}function rt(n){return((n=Math.exp(n))+1/n)/2}function ut(n){return((n=Math.exp(2*n))-1)/(n+1)}function it(n){return(n=Math.sin(n/2))*n}function ot(){}function at(n,t,e){return this instanceof at?(this.h=+n,this.s=+t,void(this.l=+e)):arguments.length<2?n instanceof at?new at(n.h,n.s,n.l):bt(""+n,_t,at):new at(n,t,e)}function ct(n,t,e){function r(n){return n>360?n-=360:0>n&&(n+=360),60>n?i+(o-i)*n/60:180>n?o:240>n?i+(o-i)*(240-n)/60:i}function u(n){return Math.round(255*r(n))}var i,o;return n=isNaN(n)?0:(n%=360)<0?n+360:n,t=isNaN(t)?0:0>t?0:t>1?1:t,e=0>e?0:e>1?1:e,o=.5>=e?e*(1+t):e+t-e*t,i=2*e-o,new mt(u(n+120),u(n),u(n-120))}function lt(n,t,e){return this instanceof lt?(this.h=+n,this.c=+t,void(this.l=+e)):arguments.length<2?n instanceof lt?new lt(n.h,n.c,n.l):n instanceof ft?gt(n.l,n.a,n.b):gt((n=wt((n=ta.rgb(n)).r,n.g,n.b)).l,n.a,n.b):new lt(n,t,e)}function st(n,t,e){return isNaN(n)&&(n=0),isNaN(t)&&(t=0),new ft(e,Math.cos(n*=Da)*t,Math.sin(n)*t)}function ft(n,t,e){return this instanceof ft?(this.l=+n,this.a=+t,void(this.b=+e)):arguments.length<2?n instanceof ft?new ft(n.l,n.a,n.b):n instanceof lt?st(n.h,n.c,n.l):wt((n=mt(n)).r,n.g,n.b):new ft(n,t,e)}function ht(n,t,e){var r=(n+16)/116,u=r+t/500,i=r-e/200;return u=pt(u)*Xa,r=pt(r)*$a,i=pt(i)*Ba,new mt(dt(3.2404542*u-1.5371385*r-.4985314*i),dt(-.969266*u+1.8760108*r+.041556*i),dt(.0556434*u-.2040259*r+1.0572252*i))}function gt(n,t,e){return n>0?new lt(Math.atan2(e,t)*Pa,Math.sqrt(t*t+e*e),n):new lt(0/0,0/0,n)}function pt(n){return n>.206893034?n*n*n:(n-4/29)/7.787037}function vt(n){return n>.008856?Math.pow(n,1/3):7.787037*n+4/29}function dt(n){return Math.round(255*(.00304>=n?12.92*n:1.055*Math.pow(n,1/2.4)-.055))}function mt(n,t,e){return this instanceof mt?(this.r=~~n,this.g=~~t,void(this.b=~~e)):arguments.length<2?n instanceof mt?new mt(n.r,n.g,n.b):bt(""+n,mt,ct):new mt(n,t,e)}function yt(n){return new mt(n>>16,n>>8&255,255&n)}function Mt(n){return yt(n)+""}function xt(n){return 16>n?"0"+Math.max(0,n).toString(16):Math.min(255,n).toString(16)}function bt(n,t,e){var r,u,i,o=0,a=0,c=0;if(r=/([a-z]+)\((.*)\)/i.exec(n))switch(u=r[2].split(","),r[1]){case"hsl":return e(parseFloat(u[0]),parseFloat(u[1])/100,parseFloat(u[2])/100);case"rgb":return t(kt(u[0]),kt(u[1]),kt(u[2]))}return(i=Ga.get(n.toLowerCase()))?t(i.r,i.g,i.b):(null==n||"#"!==n.charAt(0)||isNaN(i=parseInt(n.slice(1),16))||(4===n.length?(o=(3840&i)>>4,o=o>>4|o,a=240&i,a=a>>4|a,c=15&i,c=c<<4|c):7===n.length&&(o=(16711680&i)>>16,a=(65280&i)>>8,c=255&i)),t(o,a,c))}function _t(n,t,e){var r,u,i=Math.min(n/=255,t/=255,e/=255),o=Math.max(n,t,e),a=o-i,c=(o+i)/2;return a?(u=.5>c?a/(o+i):a/(2-o-i),r=n==o?(t-e)/a+(e>t?6:0):t==o?(e-n)/a+2:(n-t)/a+4,r*=60):(r=0/0,u=c>0&&1>c?0:r),new at(r,u,c)}function wt(n,t,e){n=St(n),t=St(t),e=St(e);var r=vt((.4124564*n+.3575761*t+.1804375*e)/Xa),u=vt((.2126729*n+.7151522*t+.072175*e)/$a),i=vt((.0193339*n+.119192*t+.9503041*e)/Ba);return ft(116*u-16,500*(r-u),200*(u-i))}function St(n){return(n/=255)<=.04045?n/12.92:Math.pow((n+.055)/1.055,2.4)}function kt(n){var t=parseFloat(n);return"%"===n.charAt(n.length-1)?Math.round(2.55*t):t}function Et(n){return"function"==typeof n?n:function(){return n}}function At(n){return function(t,e,r){return 2===arguments.length&&"function"==typeof e&&(r=e,e=null),Nt(t,e,n,r)}}function Nt(n,t,e,r){function u(){var n,t=c.status;if(!t&&zt(c)||t>=200&&300>t||304===t){try{n=e.call(i,c)}catch(r){return void o.error.call(i,r)}o.load.call(i,n)}else o.error.call(i,c)}var i={},o=ta.dispatch("beforesend","progress","load","error"),a={},c=new XMLHttpRequest,l=null;return!this.XDomainRequest||"withCredentials"in c||!/^(http(s)?:)?\/\//.test(n)||(c=new XDomainRequest),"onload"in c?c.onload=c.onerror=u:c.onreadystatechange=function(){c.readyState>3&&u()},c.onprogress=function(n){var t=ta.event;ta.event=n;try{o.progress.call(i,c)}finally{ta.event=t}},i.header=function(n,t){return n=(n+"").toLowerCase(),arguments.length<2?a[n]:(null==t?delete a[n]:a[n]=t+"",i)},i.mimeType=function(n){return arguments.length?(t=null==n?null:n+"",i):t},i.responseType=function(n){return arguments.length?(l=n,i):l},i.response=function(n){return e=n,i},["get","post"].forEach(function(n){i[n]=function(){return i.send.apply(i,[n].concat(ra(arguments)))}}),i.send=function(e,r,u){if(2===arguments.length&&"function"==typeof r&&(u=r,r=null),c.open(e,n,!0),null==t||"accept"in a||(a.accept=t+",*/*"),c.setRequestHeader)for(var s in a)c.setRequestHeader(s,a[s]);return null!=t&&c.overrideMimeType&&c.overrideMimeType(t),null!=l&&(c.responseType=l),null!=u&&i.on("error",u).on("load",function(n){u(null,n)}),o.beforesend.call(i,c),c.send(null==r?null:r),i},i.abort=function(){return c.abort(),i},ta.rebind(i,o,"on"),null==r?i:i.get(Ct(r))}function Ct(n){return 1===n.length?function(t,e){n(null==t?e:null)}:n}function zt(n){var t=n.responseType;return t&&"text"!==t?n.response:n.responseText}function qt(){var n=Lt(),t=Tt()-n;t>24?(isFinite(t)&&(clearTimeout(tc),tc=setTimeout(qt,t)),nc=0):(nc=1,rc(qt))}function Lt(){var n=Date.now();for(ec=Ka;ec;)n>=ec.t&&(ec.f=ec.c(n-ec.t)),ec=ec.n;return n}function Tt(){for(var n,t=Ka,e=1/0;t;)t.f?t=n?n.n=t.n:Ka=t.n:(t.t<e&&(e=t.t),t=(n=t).n);return Qa=n,e}function Rt(n,t){return t-(n?Math.ceil(Math.log(n)/Math.LN10):1)}function Dt(n,t){var e=Math.pow(10,3*ga(8-t));return{scale:t>8?function(n){return n/e}:function(n){return n*e},symbol:n}}function Pt(n){var t=n.decimal,e=n.thousands,r=n.grouping,u=n.currency,i=r&&e?function(n,t){for(var u=n.length,i=[],o=0,a=r[0],c=0;u>0&&a>0&&(c+a+1>t&&(a=Math.max(1,t-c)),i.push(n.substring(u-=a,u+a)),!((c+=a+1)>t));)a=r[o=(o+1)%r.length];return i.reverse().join(e)}:y;return function(n){var e=ic.exec(n),r=e[1]||" ",o=e[2]||">",a=e[3]||"-",c=e[4]||"",l=e[5],s=+e[6],f=e[7],h=e[8],g=e[9],p=1,v="",d="",m=!1,y=!0;switch(h&&(h=+h.substring(1)),(l||"0"===r&&"="===o)&&(l=r="0",o="="),g){case"n":f=!0,g="g";break;case"%":p=100,d="%",g="f";break;case"p":p=100,d="%",g="r";break;case"b":case"o":case"x":case"X":"#"===c&&(v="0"+g.toLowerCase());case"c":y=!1;case"d":m=!0,h=0;break;case"s":p=-1,g="r"}"$"===c&&(v=u[0],d=u[1]),"r"!=g||h||(g="g"),null!=h&&("g"==g?h=Math.max(1,Math.min(21,h)):("e"==g||"f"==g)&&(h=Math.max(0,Math.min(20,h)))),g=oc.get(g)||Ut;var M=l&&f;return function(n){var e=d;if(m&&n%1)return"";var u=0>n||0===n&&0>1/n?(n=-n,"-"):"-"===a?"":a;if(0>p){var c=ta.formatPrefix(n,h);n=c.scale(n),e=c.symbol+d}else n*=p;n=g(n,h);var x,b,_=n.lastIndexOf(".");if(0>_){var w=y?n.lastIndexOf("e"):-1;0>w?(x=n,b=""):(x=n.substring(0,w),b=n.substring(w))}else x=n.substring(0,_),b=t+n.substring(_+1);!l&&f&&(x=i(x,1/0));var S=v.length+x.length+b.length+(M?0:u.length),k=s>S?new Array(S=s-S+1).join(r):"";return M&&(x=i(k+x,k.length?s-b.length:1/0)),u+=v,n=x+b,("<"===o?u+n+k:">"===o?k+u+n:"^"===o?k.substring(0,S>>=1)+u+n+k.substring(S):u+(M?n:k+n))+e}}}function Ut(n){return n+""}function jt(){this._=new Date(arguments.length>1?Date.UTC.apply(this,arguments):arguments[0])}function Ft(n,t,e){function r(t){var e=n(t),r=i(e,1);return r-t>t-e?e:r}function u(e){return t(e=n(new cc(e-1)),1),e}function i(n,e){return t(n=new cc(+n),e),n}function o(n,r,i){var o=u(n),a=[];if(i>1)for(;r>o;)e(o)%i||a.push(new Date(+o)),t(o,1);else for(;r>o;)a.push(new Date(+o)),t(o,1);return a}function a(n,t,e){try{cc=jt;var r=new jt;return r._=n,o(r,t,e)}finally{cc=Date}}n.floor=n,n.round=r,n.ceil=u,n.offset=i,n.range=o;var c=n.utc=Ht(n);return c.floor=c,c.round=Ht(r),c.ceil=Ht(u),c.offset=Ht(i),c.range=a,n}function Ht(n){return function(t,e){try{cc=jt;var r=new jt;return r._=t,n(r,e)._}finally{cc=Date}}}function Ot(n){function t(n){function t(t){for(var e,u,i,o=[],a=-1,c=0;++a<r;)37===n.charCodeAt(a)&&(o.push(n.slice(c,a)),null!=(u=sc[e=n.charAt(++a)])&&(e=n.charAt(++a)),(i=N[e])&&(e=i(t,null==u?"e"===e?" ":"0":u)),o.push(e),c=a+1);return o.push(n.slice(c,a)),o.join("")}var r=n.length;return t.parse=function(t){var r={y:1900,m:0,d:1,H:0,M:0,S:0,L:0,Z:null},u=e(r,n,t,0);if(u!=t.length)return null;"p"in r&&(r.H=r.H%12+12*r.p);var i=null!=r.Z&&cc!==jt,o=new(i?jt:cc);return"j"in r?o.setFullYear(r.y,0,r.j):"w"in r&&("W"in r||"U"in r)?(o.setFullYear(r.y,0,1),o.setFullYear(r.y,0,"W"in r?(r.w+6)%7+7*r.W-(o.getDay()+5)%7:r.w+7*r.U-(o.getDay()+6)%7)):o.setFullYear(r.y,r.m,r.d),o.setHours(r.H+(r.Z/100|0),r.M+r.Z%100,r.S,r.L),i?o._:o},t.toString=function(){return n},t}function e(n,t,e,r){for(var u,i,o,a=0,c=t.length,l=e.length;c>a;){if(r>=l)return-1;if(u=t.charCodeAt(a++),37===u){if(o=t.charAt(a++),i=C[o in sc?t.charAt(a++):o],!i||(r=i(n,e,r))<0)return-1}else if(u!=e.charCodeAt(r++))return-1}return r}function r(n,t,e){_.lastIndex=0;var r=_.exec(t.slice(e));return r?(n.w=w.get(r[0].toLowerCase()),e+r[0].length):-1}function u(n,t,e){x.lastIndex=0;var r=x.exec(t.slice(e));return r?(n.w=b.get(r[0].toLowerCase()),e+r[0].length):-1}function i(n,t,e){E.lastIndex=0;var r=E.exec(t.slice(e));return r?(n.m=A.get(r[0].toLowerCase()),e+r[0].length):-1}function o(n,t,e){S.lastIndex=0;var r=S.exec(t.slice(e));return r?(n.m=k.get(r[0].toLowerCase()),e+r[0].length):-1}function a(n,t,r){return e(n,N.c.toString(),t,r)}function c(n,t,r){return e(n,N.x.toString(),t,r)}function l(n,t,r){return e(n,N.X.toString(),t,r)}function s(n,t,e){var r=M.get(t.slice(e,e+=2).toLowerCase());return null==r?-1:(n.p=r,e)}var f=n.dateTime,h=n.date,g=n.time,p=n.periods,v=n.days,d=n.shortDays,m=n.months,y=n.shortMonths;t.utc=function(n){function e(n){try{cc=jt;var t=new cc;return t._=n,r(t)}finally{cc=Date}}var r=t(n);return e.parse=function(n){try{cc=jt;var t=r.parse(n);return t&&t._}finally{cc=Date}},e.toString=r.toString,e},t.multi=t.utc.multi=ae;var M=ta.map(),x=Yt(v),b=Zt(v),_=Yt(d),w=Zt(d),S=Yt(m),k=Zt(m),E=Yt(y),A=Zt(y);p.forEach(function(n,t){M.set(n.toLowerCase(),t)});var N={a:function(n){return d[n.getDay()]},A:function(n){return v[n.getDay()]},b:function(n){return y[n.getMonth()]},B:function(n){return m[n.getMonth()]},c:t(f),d:function(n,t){return It(n.getDate(),t,2)},e:function(n,t){return It(n.getDate(),t,2)},H:function(n,t){return It(n.getHours(),t,2)},I:function(n,t){return It(n.getHours()%12||12,t,2)},j:function(n,t){return It(1+ac.dayOfYear(n),t,3)},L:function(n,t){return It(n.getMilliseconds(),t,3)},m:function(n,t){return It(n.getMonth()+1,t,2)},M:function(n,t){return It(n.getMinutes(),t,2)},p:function(n){return p[+(n.getHours()>=12)]},S:function(n,t){return It(n.getSeconds(),t,2)},U:function(n,t){return It(ac.sundayOfYear(n),t,2)},w:function(n){return n.getDay()},W:function(n,t){return It(ac.mondayOfYear(n),t,2)},x:t(h),X:t(g),y:function(n,t){return It(n.getFullYear()%100,t,2)},Y:function(n,t){return It(n.getFullYear()%1e4,t,4)},Z:ie,"%":function(){return"%"}},C={a:r,A:u,b:i,B:o,c:a,d:Qt,e:Qt,H:te,I:te,j:ne,L:ue,m:Kt,M:ee,p:s,S:re,U:Xt,w:Vt,W:$t,x:c,X:l,y:Wt,Y:Bt,Z:Jt,"%":oe};return t}function It(n,t,e){var r=0>n?"-":"",u=(r?-n:n)+"",i=u.length;return r+(e>i?new Array(e-i+1).join(t)+u:u)}function Yt(n){return new RegExp("^(?:"+n.map(ta.requote).join("|")+")","i")}function Zt(n){for(var t=new l,e=-1,r=n.length;++e<r;)t.set(n[e].toLowerCase(),e);return t}function Vt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+1));return r?(n.w=+r[0],e+r[0].length):-1}function Xt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e));return r?(n.U=+r[0],e+r[0].length):-1}function $t(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e));return r?(n.W=+r[0],e+r[0].length):-1}function Bt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+4));return r?(n.y=+r[0],e+r[0].length):-1}function Wt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.y=Gt(+r[0]),e+r[0].length):-1}function Jt(n,t,e){return/^[+-]\d{4}$/.test(t=t.slice(e,e+5))?(n.Z=-t,e+5):-1}function Gt(n){return n+(n>68?1900:2e3)}function Kt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.m=r[0]-1,e+r[0].length):-1}function Qt(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.d=+r[0],e+r[0].length):-1}function ne(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+3));return r?(n.j=+r[0],e+r[0].length):-1}function te(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.H=+r[0],e+r[0].length):-1}function ee(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.M=+r[0],e+r[0].length):-1}function re(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+2));return r?(n.S=+r[0],e+r[0].length):-1}function ue(n,t,e){fc.lastIndex=0;var r=fc.exec(t.slice(e,e+3));return r?(n.L=+r[0],e+r[0].length):-1}function ie(n){var t=n.getTimezoneOffset(),e=t>0?"-":"+",r=ga(t)/60|0,u=ga(t)%60;return e+It(r,"0",2)+It(u,"0",2)}function oe(n,t,e){hc.lastIndex=0;var r=hc.exec(t.slice(e,e+1));return r?e+r[0].length:-1}function ae(n){for(var t=n.length,e=-1;++e<t;)n[e][0]=this(n[e][0]);return function(t){for(var e=0,r=n[e];!r[1](t);)r=n[++e];return r[0](t)}}function ce(){}function le(n,t,e){var r=e.s=n+t,u=r-n,i=r-u;e.t=n-i+(t-u)}function se(n,t){n&&dc.hasOwnProperty(n.type)&&dc[n.type](n,t)}function fe(n,t,e){var r,u=-1,i=n.length-e;for(t.lineStart();++u<i;)r=n[u],t.point(r[0],r[1],r[2]);t.lineEnd()}function he(n,t){var e=-1,r=n.length;for(t.polygonStart();++e<r;)fe(n[e],t,1);t.polygonEnd()}function ge(){function n(n,t){n*=Da,t=t*Da/2+qa/4;var e=n-r,o=e>=0?1:-1,a=o*e,c=Math.cos(t),l=Math.sin(t),s=i*l,f=u*c+s*Math.cos(a),h=s*o*Math.sin(a);yc.add(Math.atan2(h,f)),r=n,u=c,i=l}var t,e,r,u,i;Mc.point=function(o,a){Mc.point=n,r=(t=o)*Da,u=Math.cos(a=(e=a)*Da/2+qa/4),i=Math.sin(a)},Mc.lineEnd=function(){n(t,e)}}function pe(n){var t=n[0],e=n[1],r=Math.cos(e);return[r*Math.cos(t),r*Math.sin(t),Math.sin(e)]}function ve(n,t){return n[0]*t[0]+n[1]*t[1]+n[2]*t[2]}function de(n,t){return[n[1]*t[2]-n[2]*t[1],n[2]*t[0]-n[0]*t[2],n[0]*t[1]-n[1]*t[0]]}function me(n,t){n[0]+=t[0],n[1]+=t[1],n[2]+=t[2]}function ye(n,t){return[n[0]*t,n[1]*t,n[2]*t]}function Me(n){var t=Math.sqrt(n[0]*n[0]+n[1]*n[1]+n[2]*n[2]);n[0]/=t,n[1]/=t,n[2]/=t}function xe(n){return[Math.atan2(n[1],n[0]),tt(n[2])]}function be(n,t){return ga(n[0]-t[0])<Ca&&ga(n[1]-t[1])<Ca}function _e(n,t){n*=Da;var e=Math.cos(t*=Da);we(e*Math.cos(n),e*Math.sin(n),Math.sin(t))}function we(n,t,e){++xc,_c+=(n-_c)/xc,wc+=(t-wc)/xc,Sc+=(e-Sc)/xc}function Se(){function n(n,u){n*=Da;var i=Math.cos(u*=Da),o=i*Math.cos(n),a=i*Math.sin(n),c=Math.sin(u),l=Math.atan2(Math.sqrt((l=e*c-r*a)*l+(l=r*o-t*c)*l+(l=t*a-e*o)*l),t*o+e*a+r*c);bc+=l,kc+=l*(t+(t=o)),Ec+=l*(e+(e=a)),Ac+=l*(r+(r=c)),we(t,e,r)}var t,e,r;qc.point=function(u,i){u*=Da;var o=Math.cos(i*=Da);t=o*Math.cos(u),e=o*Math.sin(u),r=Math.sin(i),qc.point=n,we(t,e,r)}}function ke(){qc.point=_e}function Ee(){function n(n,t){n*=Da;var e=Math.cos(t*=Da),o=e*Math.cos(n),a=e*Math.sin(n),c=Math.sin(t),l=u*c-i*a,s=i*o-r*c,f=r*a-u*o,h=Math.sqrt(l*l+s*s+f*f),g=r*o+u*a+i*c,p=h&&-nt(g)/h,v=Math.atan2(h,g);Nc+=p*l,Cc+=p*s,zc+=p*f,bc+=v,kc+=v*(r+(r=o)),Ec+=v*(u+(u=a)),Ac+=v*(i+(i=c)),we(r,u,i)}var t,e,r,u,i;qc.point=function(o,a){t=o,e=a,qc.point=n,o*=Da;var c=Math.cos(a*=Da);r=c*Math.cos(o),u=c*Math.sin(o),i=Math.sin(a),we(r,u,i)},qc.lineEnd=function(){n(t,e),qc.lineEnd=ke,qc.point=_e}}function Ae(n,t){function e(e,r){return e=n(e,r),t(e[0],e[1])}return n.invert&&t.invert&&(e.invert=function(e,r){return e=t.invert(e,r),e&&n.invert(e[0],e[1])}),e}function Ne(){return!0}function Ce(n,t,e,r,u){var i=[],o=[];if(n.forEach(function(n){if(!((t=n.length-1)<=0)){var t,e=n[0],r=n[t];if(be(e,r)){u.lineStart();for(var a=0;t>a;++a)u.point((e=n[a])[0],e[1]);return void u.lineEnd()}var c=new qe(e,n,null,!0),l=new qe(e,null,c,!1);c.o=l,i.push(c),o.push(l),c=new qe(r,n,null,!1),l=new qe(r,null,c,!0),c.o=l,i.push(c),o.push(l)}}),o.sort(t),ze(i),ze(o),i.length){for(var a=0,c=e,l=o.length;l>a;++a)o[a].e=c=!c;for(var s,f,h=i[0];;){for(var g=h,p=!0;g.v;)if((g=g.n)===h)return;s=g.z,u.lineStart();do{if(g.v=g.o.v=!0,g.e){if(p)for(var a=0,l=s.length;l>a;++a)u.point((f=s[a])[0],f[1]);else r(g.x,g.n.x,1,u);g=g.n}else{if(p){s=g.p.z;for(var a=s.length-1;a>=0;--a)u.point((f=s[a])[0],f[1])}else r(g.x,g.p.x,-1,u);g=g.p}g=g.o,s=g.z,p=!p}while(!g.v);u.lineEnd()}}}function ze(n){if(t=n.length){for(var t,e,r=0,u=n[0];++r<t;)u.n=e=n[r],e.p=u,u=e;u.n=e=n[0],e.p=u}}function qe(n,t,e,r){this.x=n,this.z=t,this.o=e,this.e=r,this.v=!1,this.n=this.p=null}function Le(n,t,e,r){return function(u,i){function o(t,e){var r=u(t,e);n(t=r[0],e=r[1])&&i.point(t,e)}function a(n,t){var e=u(n,t);d.point(e[0],e[1])}function c(){y.point=a,d.lineStart()}function l(){y.point=o,d.lineEnd()}function s(n,t){v.push([n,t]);var e=u(n,t);x.point(e[0],e[1])}function f(){x.lineStart(),v=[]}function h(){s(v[0][0],v[0][1]),x.lineEnd();var n,t=x.clean(),e=M.buffer(),r=e.length;if(v.pop(),p.push(v),v=null,r)if(1&t){n=e[0];var u,r=n.length-1,o=-1;if(r>0){for(b||(i.polygonStart(),b=!0),i.lineStart();++o<r;)i.point((u=n[o])[0],u[1]);i.lineEnd()}}else r>1&&2&t&&e.push(e.pop().concat(e.shift())),g.push(e.filter(Te))}var g,p,v,d=t(i),m=u.invert(r[0],r[1]),y={point:o,lineStart:c,lineEnd:l,polygonStart:function(){y.point=s,y.lineStart=f,y.lineEnd=h,g=[],p=[]},polygonEnd:function(){y.point=o,y.lineStart=c,y.lineEnd=l,g=ta.merge(g);var n=Fe(m,p);g.length?(b||(i.polygonStart(),b=!0),Ce(g,De,n,e,i)):n&&(b||(i.polygonStart(),b=!0),i.lineStart(),e(null,null,1,i),i.lineEnd()),b&&(i.polygonEnd(),b=!1),g=p=null},sphere:function(){i.polygonStart(),i.lineStart(),e(null,null,1,i),i.lineEnd(),i.polygonEnd()}},M=Re(),x=t(M),b=!1;return y}}function Te(n){return n.length>1}function Re(){var n,t=[];return{lineStart:function(){t.push(n=[])},point:function(t,e){n.push([t,e])},lineEnd:b,buffer:function(){var e=t;return t=[],n=null,e},rejoin:function(){t.length>1&&t.push(t.pop().concat(t.shift()))}}}function De(n,t){return((n=n.x)[0]<0?n[1]-Ra-Ca:Ra-n[1])-((t=t.x)[0]<0?t[1]-Ra-Ca:Ra-t[1])}function Pe(n){var t,e=0/0,r=0/0,u=0/0;return{lineStart:function(){n.lineStart(),t=1},point:function(i,o){var a=i>0?qa:-qa,c=ga(i-e);ga(c-qa)<Ca?(n.point(e,r=(r+o)/2>0?Ra:-Ra),n.point(u,r),n.lineEnd(),n.lineStart(),n.point(a,r),n.point(i,r),t=0):u!==a&&c>=qa&&(ga(e-u)<Ca&&(e-=u*Ca),ga(i-a)<Ca&&(i-=a*Ca),r=Ue(e,r,i,o),n.point(u,r),n.lineEnd(),n.lineStart(),n.point(a,r),t=0),n.point(e=i,r=o),u=a},lineEnd:function(){n.lineEnd(),e=r=0/0},clean:function(){return 2-t}}}function Ue(n,t,e,r){var u,i,o=Math.sin(n-e);return ga(o)>Ca?Math.atan((Math.sin(t)*(i=Math.cos(r))*Math.sin(e)-Math.sin(r)*(u=Math.cos(t))*Math.sin(n))/(u*i*o)):(t+r)/2}function je(n,t,e,r){var u;if(null==n)u=e*Ra,r.point(-qa,u),r.point(0,u),r.point(qa,u),r.point(qa,0),r.point(qa,-u),r.point(0,-u),r.point(-qa,-u),r.point(-qa,0),r.point(-qa,u);else if(ga(n[0]-t[0])>Ca){var i=n[0]<t[0]?qa:-qa;u=e*i/2,r.point(-i,u),r.point(0,u),r.point(i,u)}else r.point(t[0],t[1])}function Fe(n,t){var e=n[0],r=n[1],u=[Math.sin(e),-Math.cos(e),0],i=0,o=0;yc.reset();for(var a=0,c=t.length;c>a;++a){var l=t[a],s=l.length;if(s)for(var f=l[0],h=f[0],g=f[1]/2+qa/4,p=Math.sin(g),v=Math.cos(g),d=1;;){d===s&&(d=0),n=l[d];var m=n[0],y=n[1]/2+qa/4,M=Math.sin(y),x=Math.cos(y),b=m-h,_=b>=0?1:-1,w=_*b,S=w>qa,k=p*M;if(yc.add(Math.atan2(k*_*Math.sin(w),v*x+k*Math.cos(w))),i+=S?b+_*La:b,S^h>=e^m>=e){var E=de(pe(f),pe(n));Me(E);var A=de(u,E);Me(A);var N=(S^b>=0?-1:1)*tt(A[2]);(r>N||r===N&&(E[0]||E[1]))&&(o+=S^b>=0?1:-1)}if(!d++)break;h=m,p=M,v=x,f=n}}return(-Ca>i||Ca>i&&0>yc)^1&o}function He(n){function t(n,t){return Math.cos(n)*Math.cos(t)>i}function e(n){var e,i,c,l,s;return{lineStart:function(){l=c=!1,s=1},point:function(f,h){var g,p=[f,h],v=t(f,h),d=o?v?0:u(f,h):v?u(f+(0>f?qa:-qa),h):0;if(!e&&(l=c=v)&&n.lineStart(),v!==c&&(g=r(e,p),(be(e,g)||be(p,g))&&(p[0]+=Ca,p[1]+=Ca,v=t(p[0],p[1]))),v!==c)s=0,v?(n.lineStart(),g=r(p,e),n.point(g[0],g[1])):(g=r(e,p),n.point(g[0],g[1]),n.lineEnd()),e=g;else if(a&&e&&o^v){var m;d&i||!(m=r(p,e,!0))||(s=0,o?(n.lineStart(),n.point(m[0][0],m[0][1]),n.point(m[1][0],m[1][1]),n.lineEnd()):(n.point(m[1][0],m[1][1]),n.lineEnd(),n.lineStart(),n.point(m[0][0],m[0][1])))}!v||e&&be(e,p)||n.point(p[0],p[1]),e=p,c=v,i=d},lineEnd:function(){c&&n.lineEnd(),e=null},clean:function(){return s|(l&&c)<<1}}}function r(n,t,e){var r=pe(n),u=pe(t),o=[1,0,0],a=de(r,u),c=ve(a,a),l=a[0],s=c-l*l;if(!s)return!e&&n;var f=i*c/s,h=-i*l/s,g=de(o,a),p=ye(o,f),v=ye(a,h);me(p,v);var d=g,m=ve(p,d),y=ve(d,d),M=m*m-y*(ve(p,p)-1);if(!(0>M)){var x=Math.sqrt(M),b=ye(d,(-m-x)/y);if(me(b,p),b=xe(b),!e)return b;var _,w=n[0],S=t[0],k=n[1],E=t[1];w>S&&(_=w,w=S,S=_);var A=S-w,N=ga(A-qa)<Ca,C=N||Ca>A;if(!N&&k>E&&(_=k,k=E,E=_),C?N?k+E>0^b[1]<(ga(b[0]-w)<Ca?k:E):k<=b[1]&&b[1]<=E:A>qa^(w<=b[0]&&b[0]<=S)){var z=ye(d,(-m+x)/y);return me(z,p),[b,xe(z)]}}}function u(t,e){var r=o?n:qa-n,u=0;return-r>t?u|=1:t>r&&(u|=2),-r>e?u|=4:e>r&&(u|=8),u}var i=Math.cos(n),o=i>0,a=ga(i)>Ca,c=gr(n,6*Da);return Le(t,e,c,o?[0,-n]:[-qa,n-qa])}function Oe(n,t,e,r){return function(u){var i,o=u.a,a=u.b,c=o.x,l=o.y,s=a.x,f=a.y,h=0,g=1,p=s-c,v=f-l;if(i=n-c,p||!(i>0)){if(i/=p,0>p){if(h>i)return;g>i&&(g=i)}else if(p>0){if(i>g)return;i>h&&(h=i)}if(i=e-c,p||!(0>i)){if(i/=p,0>p){if(i>g)return;i>h&&(h=i)}else if(p>0){if(h>i)return;g>i&&(g=i)}if(i=t-l,v||!(i>0)){if(i/=v,0>v){if(h>i)return;g>i&&(g=i)}else if(v>0){if(i>g)return;i>h&&(h=i)}if(i=r-l,v||!(0>i)){if(i/=v,0>v){if(i>g)return;i>h&&(h=i)}else if(v>0){if(h>i)return;g>i&&(g=i)}return h>0&&(u.a={x:c+h*p,y:l+h*v}),1>g&&(u.b={x:c+g*p,y:l+g*v}),u}}}}}}function Ie(n,t,e,r){function u(r,u){return ga(r[0]-n)<Ca?u>0?0:3:ga(r[0]-e)<Ca?u>0?2:1:ga(r[1]-t)<Ca?u>0?1:0:u>0?3:2}function i(n,t){return o(n.x,t.x)}function o(n,t){var e=u(n,1),r=u(t,1);return e!==r?e-r:0===e?t[1]-n[1]:1===e?n[0]-t[0]:2===e?n[1]-t[1]:t[0]-n[0]}return function(a){function c(n){for(var t=0,e=d.length,r=n[1],u=0;e>u;++u)for(var i,o=1,a=d[u],c=a.length,l=a[0];c>o;++o)i=a[o],l[1]<=r?i[1]>r&&Q(l,i,n)>0&&++t:i[1]<=r&&Q(l,i,n)<0&&--t,l=i;return 0!==t}function l(i,a,c,l){var s=0,f=0;if(null==i||(s=u(i,c))!==(f=u(a,c))||o(i,a)<0^c>0){do l.point(0===s||3===s?n:e,s>1?r:t);while((s=(s+c+4)%4)!==f)}else l.point(a[0],a[1])}function s(u,i){return u>=n&&e>=u&&i>=t&&r>=i}function f(n,t){s(n,t)&&a.point(n,t)}function h(){C.point=p,d&&d.push(m=[]),S=!0,w=!1,b=_=0/0}function g(){v&&(p(y,M),x&&w&&A.rejoin(),v.push(A.buffer())),C.point=f,w&&a.lineEnd()}function p(n,t){n=Math.max(-Tc,Math.min(Tc,n)),t=Math.max(-Tc,Math.min(Tc,t));var e=s(n,t);if(d&&m.push([n,t]),S)y=n,M=t,x=e,S=!1,e&&(a.lineStart(),a.point(n,t));else if(e&&w)a.point(n,t);else{var r={a:{x:b,y:_},b:{x:n,y:t}};N(r)?(w||(a.lineStart(),a.point(r.a.x,r.a.y)),a.point(r.b.x,r.b.y),e||a.lineEnd(),k=!1):e&&(a.lineStart(),a.point(n,t),k=!1)}b=n,_=t,w=e}var v,d,m,y,M,x,b,_,w,S,k,E=a,A=Re(),N=Oe(n,t,e,r),C={point:f,lineStart:h,lineEnd:g,polygonStart:function(){a=A,v=[],d=[],k=!0},polygonEnd:function(){a=E,v=ta.merge(v);var t=c([n,r]),e=k&&t,u=v.length;(e||u)&&(a.polygonStart(),e&&(a.lineStart(),l(null,null,1,a),a.lineEnd()),u&&Ce(v,i,t,l,a),a.polygonEnd()),v=d=m=null}};return C}}function Ye(n){var t=0,e=qa/3,r=ir(n),u=r(t,e);return u.parallels=function(n){return arguments.length?r(t=n[0]*qa/180,e=n[1]*qa/180):[t/qa*180,e/qa*180]},u}function Ze(n,t){function e(n,t){var e=Math.sqrt(i-2*u*Math.sin(t))/u;return[e*Math.sin(n*=u),o-e*Math.cos(n)]}var r=Math.sin(n),u=(r+Math.sin(t))/2,i=1+r*(2*u-r),o=Math.sqrt(i)/u;return e.invert=function(n,t){var e=o-t;return[Math.atan2(n,e)/u,tt((i-(n*n+e*e)*u*u)/(2*u))]},e}function Ve(){function n(n,t){Dc+=u*n-r*t,r=n,u=t}var t,e,r,u;Hc.point=function(i,o){Hc.point=n,t=r=i,e=u=o},Hc.lineEnd=function(){n(t,e)}}function Xe(n,t){Pc>n&&(Pc=n),n>jc&&(jc=n),Uc>t&&(Uc=t),t>Fc&&(Fc=t)}function $e(){function n(n,t){o.push("M",n,",",t,i)}function t(n,t){o.push("M",n,",",t),a.point=e}function e(n,t){o.push("L",n,",",t)}function r(){a.point=n}function u(){o.push("Z")}var i=Be(4.5),o=[],a={point:n,lineStart:function(){a.point=t},lineEnd:r,polygonStart:function(){a.lineEnd=u},polygonEnd:function(){a.lineEnd=r,a.point=n},pointRadius:function(n){return i=Be(n),a},result:function(){if(o.length){var n=o.join("");return o=[],n}}};return a}function Be(n){return"m0,"+n+"a"+n+","+n+" 0 1,1 0,"+-2*n+"a"+n+","+n+" 0 1,1 0,"+2*n+"z"}function We(n,t){_c+=n,wc+=t,++Sc}function Je(){function n(n,r){var u=n-t,i=r-e,o=Math.sqrt(u*u+i*i);kc+=o*(t+n)/2,Ec+=o*(e+r)/2,Ac+=o,We(t=n,e=r)}var t,e;Ic.point=function(r,u){Ic.point=n,We(t=r,e=u)}}function Ge(){Ic.point=We}function Ke(){function n(n,t){var e=n-r,i=t-u,o=Math.sqrt(e*e+i*i);kc+=o*(r+n)/2,Ec+=o*(u+t)/2,Ac+=o,o=u*n-r*t,Nc+=o*(r+n),Cc+=o*(u+t),zc+=3*o,We(r=n,u=t)}var t,e,r,u;Ic.point=function(i,o){Ic.point=n,We(t=r=i,e=u=o)},Ic.lineEnd=function(){n(t,e)}}function Qe(n){function t(t,e){n.moveTo(t+o,e),n.arc(t,e,o,0,La)}function e(t,e){n.moveTo(t,e),a.point=r}function r(t,e){n.lineTo(t,e)}function u(){a.point=t}function i(){n.closePath()}var o=4.5,a={point:t,lineStart:function(){a.point=e},lineEnd:u,polygonStart:function(){a.lineEnd=i},polygonEnd:function(){a.lineEnd=u,a.point=t},pointRadius:function(n){return o=n,a},result:b};return a}function nr(n){function t(n){return(a?r:e)(n)}function e(t){return rr(t,function(e,r){e=n(e,r),t.point(e[0],e[1])})}function r(t){function e(e,r){e=n(e,r),t.point(e[0],e[1])}function r(){M=0/0,S.point=i,t.lineStart()}function i(e,r){var i=pe([e,r]),o=n(e,r);u(M,x,y,b,_,w,M=o[0],x=o[1],y=e,b=i[0],_=i[1],w=i[2],a,t),t.point(M,x)}function o(){S.point=e,t.lineEnd()}function c(){r(),S.point=l,S.lineEnd=s}function l(n,t){i(f=n,h=t),g=M,p=x,v=b,d=_,m=w,S.point=i}function s(){u(M,x,y,b,_,w,g,p,f,v,d,m,a,t),S.lineEnd=o,o()}var f,h,g,p,v,d,m,y,M,x,b,_,w,S={point:e,lineStart:r,lineEnd:o,polygonStart:function(){t.polygonStart(),S.lineStart=c
},polygonEnd:function(){t.polygonEnd(),S.lineStart=r}};return S}function u(t,e,r,a,c,l,s,f,h,g,p,v,d,m){var y=s-t,M=f-e,x=y*y+M*M;if(x>4*i&&d--){var b=a+g,_=c+p,w=l+v,S=Math.sqrt(b*b+_*_+w*w),k=Math.asin(w/=S),E=ga(ga(w)-1)<Ca||ga(r-h)<Ca?(r+h)/2:Math.atan2(_,b),A=n(E,k),N=A[0],C=A[1],z=N-t,q=C-e,L=M*z-y*q;(L*L/x>i||ga((y*z+M*q)/x-.5)>.3||o>a*g+c*p+l*v)&&(u(t,e,r,a,c,l,N,C,E,b/=S,_/=S,w,d,m),m.point(N,C),u(N,C,E,b,_,w,s,f,h,g,p,v,d,m))}}var i=.5,o=Math.cos(30*Da),a=16;return t.precision=function(n){return arguments.length?(a=(i=n*n)>0&&16,t):Math.sqrt(i)},t}function tr(n){var t=nr(function(t,e){return n([t*Pa,e*Pa])});return function(n){return or(t(n))}}function er(n){this.stream=n}function rr(n,t){return{point:t,sphere:function(){n.sphere()},lineStart:function(){n.lineStart()},lineEnd:function(){n.lineEnd()},polygonStart:function(){n.polygonStart()},polygonEnd:function(){n.polygonEnd()}}}function ur(n){return ir(function(){return n})()}function ir(n){function t(n){return n=a(n[0]*Da,n[1]*Da),[n[0]*h+c,l-n[1]*h]}function e(n){return n=a.invert((n[0]-c)/h,(l-n[1])/h),n&&[n[0]*Pa,n[1]*Pa]}function r(){a=Ae(o=lr(m,M,x),i);var n=i(v,d);return c=g-n[0]*h,l=p+n[1]*h,u()}function u(){return s&&(s.valid=!1,s=null),t}var i,o,a,c,l,s,f=nr(function(n,t){return n=i(n,t),[n[0]*h+c,l-n[1]*h]}),h=150,g=480,p=250,v=0,d=0,m=0,M=0,x=0,b=Lc,_=y,w=null,S=null;return t.stream=function(n){return s&&(s.valid=!1),s=or(b(o,f(_(n)))),s.valid=!0,s},t.clipAngle=function(n){return arguments.length?(b=null==n?(w=n,Lc):He((w=+n)*Da),u()):w},t.clipExtent=function(n){return arguments.length?(S=n,_=n?Ie(n[0][0],n[0][1],n[1][0],n[1][1]):y,u()):S},t.scale=function(n){return arguments.length?(h=+n,r()):h},t.translate=function(n){return arguments.length?(g=+n[0],p=+n[1],r()):[g,p]},t.center=function(n){return arguments.length?(v=n[0]%360*Da,d=n[1]%360*Da,r()):[v*Pa,d*Pa]},t.rotate=function(n){return arguments.length?(m=n[0]%360*Da,M=n[1]%360*Da,x=n.length>2?n[2]%360*Da:0,r()):[m*Pa,M*Pa,x*Pa]},ta.rebind(t,f,"precision"),function(){return i=n.apply(this,arguments),t.invert=i.invert&&e,r()}}function or(n){return rr(n,function(t,e){n.point(t*Da,e*Da)})}function ar(n,t){return[n,t]}function cr(n,t){return[n>qa?n-La:-qa>n?n+La:n,t]}function lr(n,t,e){return n?t||e?Ae(fr(n),hr(t,e)):fr(n):t||e?hr(t,e):cr}function sr(n){return function(t,e){return t+=n,[t>qa?t-La:-qa>t?t+La:t,e]}}function fr(n){var t=sr(n);return t.invert=sr(-n),t}function hr(n,t){function e(n,t){var e=Math.cos(t),a=Math.cos(n)*e,c=Math.sin(n)*e,l=Math.sin(t),s=l*r+a*u;return[Math.atan2(c*i-s*o,a*r-l*u),tt(s*i+c*o)]}var r=Math.cos(n),u=Math.sin(n),i=Math.cos(t),o=Math.sin(t);return e.invert=function(n,t){var e=Math.cos(t),a=Math.cos(n)*e,c=Math.sin(n)*e,l=Math.sin(t),s=l*i-c*o;return[Math.atan2(c*i+l*o,a*r+s*u),tt(s*r-a*u)]},e}function gr(n,t){var e=Math.cos(n),r=Math.sin(n);return function(u,i,o,a){var c=o*t;null!=u?(u=pr(e,u),i=pr(e,i),(o>0?i>u:u>i)&&(u+=o*La)):(u=n+o*La,i=n-.5*c);for(var l,s=u;o>0?s>i:i>s;s-=c)a.point((l=xe([e,-r*Math.cos(s),-r*Math.sin(s)]))[0],l[1])}}function pr(n,t){var e=pe(t);e[0]-=n,Me(e);var r=nt(-e[1]);return((-e[2]<0?-r:r)+2*Math.PI-Ca)%(2*Math.PI)}function vr(n,t,e){var r=ta.range(n,t-Ca,e).concat(t);return function(n){return r.map(function(t){return[n,t]})}}function dr(n,t,e){var r=ta.range(n,t-Ca,e).concat(t);return function(n){return r.map(function(t){return[t,n]})}}function mr(n){return n.source}function yr(n){return n.target}function Mr(n,t,e,r){var u=Math.cos(t),i=Math.sin(t),o=Math.cos(r),a=Math.sin(r),c=u*Math.cos(n),l=u*Math.sin(n),s=o*Math.cos(e),f=o*Math.sin(e),h=2*Math.asin(Math.sqrt(it(r-t)+u*o*it(e-n))),g=1/Math.sin(h),p=h?function(n){var t=Math.sin(n*=h)*g,e=Math.sin(h-n)*g,r=e*c+t*s,u=e*l+t*f,o=e*i+t*a;return[Math.atan2(u,r)*Pa,Math.atan2(o,Math.sqrt(r*r+u*u))*Pa]}:function(){return[n*Pa,t*Pa]};return p.distance=h,p}function xr(){function n(n,u){var i=Math.sin(u*=Da),o=Math.cos(u),a=ga((n*=Da)-t),c=Math.cos(a);Yc+=Math.atan2(Math.sqrt((a=o*Math.sin(a))*a+(a=r*i-e*o*c)*a),e*i+r*o*c),t=n,e=i,r=o}var t,e,r;Zc.point=function(u,i){t=u*Da,e=Math.sin(i*=Da),r=Math.cos(i),Zc.point=n},Zc.lineEnd=function(){Zc.point=Zc.lineEnd=b}}function br(n,t){function e(t,e){var r=Math.cos(t),u=Math.cos(e),i=n(r*u);return[i*u*Math.sin(t),i*Math.sin(e)]}return e.invert=function(n,e){var r=Math.sqrt(n*n+e*e),u=t(r),i=Math.sin(u),o=Math.cos(u);return[Math.atan2(n*i,r*o),Math.asin(r&&e*i/r)]},e}function _r(n,t){function e(n,t){o>0?-Ra+Ca>t&&(t=-Ra+Ca):t>Ra-Ca&&(t=Ra-Ca);var e=o/Math.pow(u(t),i);return[e*Math.sin(i*n),o-e*Math.cos(i*n)]}var r=Math.cos(n),u=function(n){return Math.tan(qa/4+n/2)},i=n===t?Math.sin(n):Math.log(r/Math.cos(t))/Math.log(u(t)/u(n)),o=r*Math.pow(u(n),i)/i;return i?(e.invert=function(n,t){var e=o-t,r=K(i)*Math.sqrt(n*n+e*e);return[Math.atan2(n,e)/i,2*Math.atan(Math.pow(o/r,1/i))-Ra]},e):Sr}function wr(n,t){function e(n,t){var e=i-t;return[e*Math.sin(u*n),i-e*Math.cos(u*n)]}var r=Math.cos(n),u=n===t?Math.sin(n):(r-Math.cos(t))/(t-n),i=r/u+n;return ga(u)<Ca?ar:(e.invert=function(n,t){var e=i-t;return[Math.atan2(n,e)/u,i-K(u)*Math.sqrt(n*n+e*e)]},e)}function Sr(n,t){return[n,Math.log(Math.tan(qa/4+t/2))]}function kr(n){var t,e=ur(n),r=e.scale,u=e.translate,i=e.clipExtent;return e.scale=function(){var n=r.apply(e,arguments);return n===e?t?e.clipExtent(null):e:n},e.translate=function(){var n=u.apply(e,arguments);return n===e?t?e.clipExtent(null):e:n},e.clipExtent=function(n){var o=i.apply(e,arguments);if(o===e){if(t=null==n){var a=qa*r(),c=u();i([[c[0]-a,c[1]-a],[c[0]+a,c[1]+a]])}}else t&&(o=null);return o},e.clipExtent(null)}function Er(n,t){return[Math.log(Math.tan(qa/4+t/2)),-n]}function Ar(n){return n[0]}function Nr(n){return n[1]}function Cr(n){for(var t=n.length,e=[0,1],r=2,u=2;t>u;u++){for(;r>1&&Q(n[e[r-2]],n[e[r-1]],n[u])<=0;)--r;e[r++]=u}return e.slice(0,r)}function zr(n,t){return n[0]-t[0]||n[1]-t[1]}function qr(n,t,e){return(e[0]-t[0])*(n[1]-t[1])<(e[1]-t[1])*(n[0]-t[0])}function Lr(n,t,e,r){var u=n[0],i=e[0],o=t[0]-u,a=r[0]-i,c=n[1],l=e[1],s=t[1]-c,f=r[1]-l,h=(a*(c-l)-f*(u-i))/(f*o-a*s);return[u+h*o,c+h*s]}function Tr(n){var t=n[0],e=n[n.length-1];return!(t[0]-e[0]||t[1]-e[1])}function Rr(){tu(this),this.edge=this.site=this.circle=null}function Dr(n){var t=el.pop()||new Rr;return t.site=n,t}function Pr(n){Xr(n),Qc.remove(n),el.push(n),tu(n)}function Ur(n){var t=n.circle,e=t.x,r=t.cy,u={x:e,y:r},i=n.P,o=n.N,a=[n];Pr(n);for(var c=i;c.circle&&ga(e-c.circle.x)<Ca&&ga(r-c.circle.cy)<Ca;)i=c.P,a.unshift(c),Pr(c),c=i;a.unshift(c),Xr(c);for(var l=o;l.circle&&ga(e-l.circle.x)<Ca&&ga(r-l.circle.cy)<Ca;)o=l.N,a.push(l),Pr(l),l=o;a.push(l),Xr(l);var s,f=a.length;for(s=1;f>s;++s)l=a[s],c=a[s-1],Kr(l.edge,c.site,l.site,u);c=a[0],l=a[f-1],l.edge=Jr(c.site,l.site,null,u),Vr(c),Vr(l)}function jr(n){for(var t,e,r,u,i=n.x,o=n.y,a=Qc._;a;)if(r=Fr(a,o)-i,r>Ca)a=a.L;else{if(u=i-Hr(a,o),!(u>Ca)){r>-Ca?(t=a.P,e=a):u>-Ca?(t=a,e=a.N):t=e=a;break}if(!a.R){t=a;break}a=a.R}var c=Dr(n);if(Qc.insert(t,c),t||e){if(t===e)return Xr(t),e=Dr(t.site),Qc.insert(c,e),c.edge=e.edge=Jr(t.site,c.site),Vr(t),void Vr(e);if(!e)return void(c.edge=Jr(t.site,c.site));Xr(t),Xr(e);var l=t.site,s=l.x,f=l.y,h=n.x-s,g=n.y-f,p=e.site,v=p.x-s,d=p.y-f,m=2*(h*d-g*v),y=h*h+g*g,M=v*v+d*d,x={x:(d*y-g*M)/m+s,y:(h*M-v*y)/m+f};Kr(e.edge,l,p,x),c.edge=Jr(l,n,null,x),e.edge=Jr(n,p,null,x),Vr(t),Vr(e)}}function Fr(n,t){var e=n.site,r=e.x,u=e.y,i=u-t;if(!i)return r;var o=n.P;if(!o)return-1/0;e=o.site;var a=e.x,c=e.y,l=c-t;if(!l)return a;var s=a-r,f=1/i-1/l,h=s/l;return f?(-h+Math.sqrt(h*h-2*f*(s*s/(-2*l)-c+l/2+u-i/2)))/f+r:(r+a)/2}function Hr(n,t){var e=n.N;if(e)return Fr(e,t);var r=n.site;return r.y===t?r.x:1/0}function Or(n){this.site=n,this.edges=[]}function Ir(n){for(var t,e,r,u,i,o,a,c,l,s,f=n[0][0],h=n[1][0],g=n[0][1],p=n[1][1],v=Kc,d=v.length;d--;)if(i=v[d],i&&i.prepare())for(a=i.edges,c=a.length,o=0;c>o;)s=a[o].end(),r=s.x,u=s.y,l=a[++o%c].start(),t=l.x,e=l.y,(ga(r-t)>Ca||ga(u-e)>Ca)&&(a.splice(o,0,new Qr(Gr(i.site,s,ga(r-f)<Ca&&p-u>Ca?{x:f,y:ga(t-f)<Ca?e:p}:ga(u-p)<Ca&&h-r>Ca?{x:ga(e-p)<Ca?t:h,y:p}:ga(r-h)<Ca&&u-g>Ca?{x:h,y:ga(t-h)<Ca?e:g}:ga(u-g)<Ca&&r-f>Ca?{x:ga(e-g)<Ca?t:f,y:g}:null),i.site,null)),++c)}function Yr(n,t){return t.angle-n.angle}function Zr(){tu(this),this.x=this.y=this.arc=this.site=this.cy=null}function Vr(n){var t=n.P,e=n.N;if(t&&e){var r=t.site,u=n.site,i=e.site;if(r!==i){var o=u.x,a=u.y,c=r.x-o,l=r.y-a,s=i.x-o,f=i.y-a,h=2*(c*f-l*s);if(!(h>=-za)){var g=c*c+l*l,p=s*s+f*f,v=(f*g-l*p)/h,d=(c*p-s*g)/h,f=d+a,m=rl.pop()||new Zr;m.arc=n,m.site=u,m.x=v+o,m.y=f+Math.sqrt(v*v+d*d),m.cy=f,n.circle=m;for(var y=null,M=tl._;M;)if(m.y<M.y||m.y===M.y&&m.x<=M.x){if(!M.L){y=M.P;break}M=M.L}else{if(!M.R){y=M;break}M=M.R}tl.insert(y,m),y||(nl=m)}}}}function Xr(n){var t=n.circle;t&&(t.P||(nl=t.N),tl.remove(t),rl.push(t),tu(t),n.circle=null)}function $r(n){for(var t,e=Gc,r=Oe(n[0][0],n[0][1],n[1][0],n[1][1]),u=e.length;u--;)t=e[u],(!Br(t,n)||!r(t)||ga(t.a.x-t.b.x)<Ca&&ga(t.a.y-t.b.y)<Ca)&&(t.a=t.b=null,e.splice(u,1))}function Br(n,t){var e=n.b;if(e)return!0;var r,u,i=n.a,o=t[0][0],a=t[1][0],c=t[0][1],l=t[1][1],s=n.l,f=n.r,h=s.x,g=s.y,p=f.x,v=f.y,d=(h+p)/2,m=(g+v)/2;if(v===g){if(o>d||d>=a)return;if(h>p){if(i){if(i.y>=l)return}else i={x:d,y:c};e={x:d,y:l}}else{if(i){if(i.y<c)return}else i={x:d,y:l};e={x:d,y:c}}}else if(r=(h-p)/(v-g),u=m-r*d,-1>r||r>1)if(h>p){if(i){if(i.y>=l)return}else i={x:(c-u)/r,y:c};e={x:(l-u)/r,y:l}}else{if(i){if(i.y<c)return}else i={x:(l-u)/r,y:l};e={x:(c-u)/r,y:c}}else if(v>g){if(i){if(i.x>=a)return}else i={x:o,y:r*o+u};e={x:a,y:r*a+u}}else{if(i){if(i.x<o)return}else i={x:a,y:r*a+u};e={x:o,y:r*o+u}}return n.a=i,n.b=e,!0}function Wr(n,t){this.l=n,this.r=t,this.a=this.b=null}function Jr(n,t,e,r){var u=new Wr(n,t);return Gc.push(u),e&&Kr(u,n,t,e),r&&Kr(u,t,n,r),Kc[n.i].edges.push(new Qr(u,n,t)),Kc[t.i].edges.push(new Qr(u,t,n)),u}function Gr(n,t,e){var r=new Wr(n,null);return r.a=t,r.b=e,Gc.push(r),r}function Kr(n,t,e,r){n.a||n.b?n.l===e?n.b=r:n.a=r:(n.a=r,n.l=t,n.r=e)}function Qr(n,t,e){var r=n.a,u=n.b;this.edge=n,this.site=t,this.angle=e?Math.atan2(e.y-t.y,e.x-t.x):n.l===t?Math.atan2(u.x-r.x,r.y-u.y):Math.atan2(r.x-u.x,u.y-r.y)}function nu(){this._=null}function tu(n){n.U=n.C=n.L=n.R=n.P=n.N=null}function eu(n,t){var e=t,r=t.R,u=e.U;u?u.L===e?u.L=r:u.R=r:n._=r,r.U=u,e.U=r,e.R=r.L,e.R&&(e.R.U=e),r.L=e}function ru(n,t){var e=t,r=t.L,u=e.U;u?u.L===e?u.L=r:u.R=r:n._=r,r.U=u,e.U=r,e.L=r.R,e.L&&(e.L.U=e),r.R=e}function uu(n){for(;n.L;)n=n.L;return n}function iu(n,t){var e,r,u,i=n.sort(ou).pop();for(Gc=[],Kc=new Array(n.length),Qc=new nu,tl=new nu;;)if(u=nl,i&&(!u||i.y<u.y||i.y===u.y&&i.x<u.x))(i.x!==e||i.y!==r)&&(Kc[i.i]=new Or(i),jr(i),e=i.x,r=i.y),i=n.pop();else{if(!u)break;Ur(u.arc)}t&&($r(t),Ir(t));var o={cells:Kc,edges:Gc};return Qc=tl=Gc=Kc=null,o}function ou(n,t){return t.y-n.y||t.x-n.x}function au(n,t,e){return(n.x-e.x)*(t.y-n.y)-(n.x-t.x)*(e.y-n.y)}function cu(n){return n.x}function lu(n){return n.y}function su(){return{leaf:!0,nodes:[],point:null,x:null,y:null}}function fu(n,t,e,r,u,i){if(!n(t,e,r,u,i)){var o=.5*(e+u),a=.5*(r+i),c=t.nodes;c[0]&&fu(n,c[0],e,r,o,a),c[1]&&fu(n,c[1],o,r,u,a),c[2]&&fu(n,c[2],e,a,o,i),c[3]&&fu(n,c[3],o,a,u,i)}}function hu(n,t,e,r,u,i,o){var a,c=1/0;return function l(n,s,f,h,g){if(!(s>i||f>o||r>h||u>g)){if(p=n.point){var p,v=t-n.x,d=e-n.y,m=v*v+d*d;if(c>m){var y=Math.sqrt(c=m);r=t-y,u=e-y,i=t+y,o=e+y,a=p}}for(var M=n.nodes,x=.5*(s+h),b=.5*(f+g),_=t>=x,w=e>=b,S=w<<1|_,k=S+4;k>S;++S)if(n=M[3&S])switch(3&S){case 0:l(n,s,f,x,b);break;case 1:l(n,x,f,h,b);break;case 2:l(n,s,b,x,g);break;case 3:l(n,x,b,h,g)}}}(n,r,u,i,o),a}function gu(n,t){n=ta.rgb(n),t=ta.rgb(t);var e=n.r,r=n.g,u=n.b,i=t.r-e,o=t.g-r,a=t.b-u;return function(n){return"#"+xt(Math.round(e+i*n))+xt(Math.round(r+o*n))+xt(Math.round(u+a*n))}}function pu(n,t){var e,r={},u={};for(e in n)e in t?r[e]=mu(n[e],t[e]):u[e]=n[e];for(e in t)e in n||(u[e]=t[e]);return function(n){for(e in r)u[e]=r[e](n);return u}}function vu(n,t){return n=+n,t=+t,function(e){return n*(1-e)+t*e}}function du(n,t){var e,r,u,i=il.lastIndex=ol.lastIndex=0,o=-1,a=[],c=[];for(n+="",t+="";(e=il.exec(n))&&(r=ol.exec(t));)(u=r.index)>i&&(u=t.slice(i,u),a[o]?a[o]+=u:a[++o]=u),(e=e[0])===(r=r[0])?a[o]?a[o]+=r:a[++o]=r:(a[++o]=null,c.push({i:o,x:vu(e,r)})),i=ol.lastIndex;return i<t.length&&(u=t.slice(i),a[o]?a[o]+=u:a[++o]=u),a.length<2?c[0]?(t=c[0].x,function(n){return t(n)+""}):function(){return t}:(t=c.length,function(n){for(var e,r=0;t>r;++r)a[(e=c[r]).i]=e.x(n);return a.join("")})}function mu(n,t){for(var e,r=ta.interpolators.length;--r>=0&&!(e=ta.interpolators[r](n,t)););return e}function yu(n,t){var e,r=[],u=[],i=n.length,o=t.length,a=Math.min(n.length,t.length);for(e=0;a>e;++e)r.push(mu(n[e],t[e]));for(;i>e;++e)u[e]=n[e];for(;o>e;++e)u[e]=t[e];return function(n){for(e=0;a>e;++e)u[e]=r[e](n);return u}}function Mu(n){return function(t){return 0>=t?0:t>=1?1:n(t)}}function xu(n){return function(t){return 1-n(1-t)}}function bu(n){return function(t){return.5*(.5>t?n(2*t):2-n(2-2*t))}}function _u(n){return n*n}function wu(n){return n*n*n}function Su(n){if(0>=n)return 0;if(n>=1)return 1;var t=n*n,e=t*n;return 4*(.5>n?e:3*(n-t)+e-.75)}function ku(n){return function(t){return Math.pow(t,n)}}function Eu(n){return 1-Math.cos(n*Ra)}function Au(n){return Math.pow(2,10*(n-1))}function Nu(n){return 1-Math.sqrt(1-n*n)}function Cu(n,t){var e;return arguments.length<2&&(t=.45),arguments.length?e=t/La*Math.asin(1/n):(n=1,e=t/4),function(r){return 1+n*Math.pow(2,-10*r)*Math.sin((r-e)*La/t)}}function zu(n){return n||(n=1.70158),function(t){return t*t*((n+1)*t-n)}}function qu(n){return 1/2.75>n?7.5625*n*n:2/2.75>n?7.5625*(n-=1.5/2.75)*n+.75:2.5/2.75>n?7.5625*(n-=2.25/2.75)*n+.9375:7.5625*(n-=2.625/2.75)*n+.984375}function Lu(n,t){n=ta.hcl(n),t=ta.hcl(t);var e=n.h,r=n.c,u=n.l,i=t.h-e,o=t.c-r,a=t.l-u;return isNaN(o)&&(o=0,r=isNaN(r)?t.c:r),isNaN(i)?(i=0,e=isNaN(e)?t.h:e):i>180?i-=360:-180>i&&(i+=360),function(n){return st(e+i*n,r+o*n,u+a*n)+""}}function Tu(n,t){n=ta.hsl(n),t=ta.hsl(t);var e=n.h,r=n.s,u=n.l,i=t.h-e,o=t.s-r,a=t.l-u;return isNaN(o)&&(o=0,r=isNaN(r)?t.s:r),isNaN(i)?(i=0,e=isNaN(e)?t.h:e):i>180?i-=360:-180>i&&(i+=360),function(n){return ct(e+i*n,r+o*n,u+a*n)+""}}function Ru(n,t){n=ta.lab(n),t=ta.lab(t);var e=n.l,r=n.a,u=n.b,i=t.l-e,o=t.a-r,a=t.b-u;return function(n){return ht(e+i*n,r+o*n,u+a*n)+""}}function Du(n,t){return t-=n,function(e){return Math.round(n+t*e)}}function Pu(n){var t=[n.a,n.b],e=[n.c,n.d],r=ju(t),u=Uu(t,e),i=ju(Fu(e,t,-u))||0;t[0]*e[1]<e[0]*t[1]&&(t[0]*=-1,t[1]*=-1,r*=-1,u*=-1),this.rotate=(r?Math.atan2(t[1],t[0]):Math.atan2(-e[0],e[1]))*Pa,this.translate=[n.e,n.f],this.scale=[r,i],this.skew=i?Math.atan2(u,i)*Pa:0}function Uu(n,t){return n[0]*t[0]+n[1]*t[1]}function ju(n){var t=Math.sqrt(Uu(n,n));return t&&(n[0]/=t,n[1]/=t),t}function Fu(n,t,e){return n[0]+=e*t[0],n[1]+=e*t[1],n}function Hu(n,t){var e,r=[],u=[],i=ta.transform(n),o=ta.transform(t),a=i.translate,c=o.translate,l=i.rotate,s=o.rotate,f=i.skew,h=o.skew,g=i.scale,p=o.scale;return a[0]!=c[0]||a[1]!=c[1]?(r.push("translate(",null,",",null,")"),u.push({i:1,x:vu(a[0],c[0])},{i:3,x:vu(a[1],c[1])})):r.push(c[0]||c[1]?"translate("+c+")":""),l!=s?(l-s>180?s+=360:s-l>180&&(l+=360),u.push({i:r.push(r.pop()+"rotate(",null,")")-2,x:vu(l,s)})):s&&r.push(r.pop()+"rotate("+s+")"),f!=h?u.push({i:r.push(r.pop()+"skewX(",null,")")-2,x:vu(f,h)}):h&&r.push(r.pop()+"skewX("+h+")"),g[0]!=p[0]||g[1]!=p[1]?(e=r.push(r.pop()+"scale(",null,",",null,")"),u.push({i:e-4,x:vu(g[0],p[0])},{i:e-2,x:vu(g[1],p[1])})):(1!=p[0]||1!=p[1])&&r.push(r.pop()+"scale("+p+")"),e=u.length,function(n){for(var t,i=-1;++i<e;)r[(t=u[i]).i]=t.x(n);return r.join("")}}function Ou(n,t){return t=(t-=n=+n)||1/t,function(e){return(e-n)/t}}function Iu(n,t){return t=(t-=n=+n)||1/t,function(e){return Math.max(0,Math.min(1,(e-n)/t))}}function Yu(n){for(var t=n.source,e=n.target,r=Vu(t,e),u=[t];t!==r;)t=t.parent,u.push(t);for(var i=u.length;e!==r;)u.splice(i,0,e),e=e.parent;return u}function Zu(n){for(var t=[],e=n.parent;null!=e;)t.push(n),n=e,e=e.parent;return t.push(n),t}function Vu(n,t){if(n===t)return n;for(var e=Zu(n),r=Zu(t),u=e.pop(),i=r.pop(),o=null;u===i;)o=u,u=e.pop(),i=r.pop();return o}function Xu(n){n.fixed|=2}function $u(n){n.fixed&=-7}function Bu(n){n.fixed|=4,n.px=n.x,n.py=n.y}function Wu(n){n.fixed&=-5}function Ju(n,t,e){var r=0,u=0;if(n.charge=0,!n.leaf)for(var i,o=n.nodes,a=o.length,c=-1;++c<a;)i=o[c],null!=i&&(Ju(i,t,e),n.charge+=i.charge,r+=i.charge*i.cx,u+=i.charge*i.cy);if(n.point){n.leaf||(n.point.x+=Math.random()-.5,n.point.y+=Math.random()-.5);var l=t*e[n.point.index];n.charge+=n.pointCharge=l,r+=l*n.point.x,u+=l*n.point.y}n.cx=r/n.charge,n.cy=u/n.charge}function Gu(n,t){return ta.rebind(n,t,"sort","children","value"),n.nodes=n,n.links=ri,n}function Ku(n,t){for(var e=[n];null!=(n=e.pop());)if(t(n),(u=n.children)&&(r=u.length))for(var r,u;--r>=0;)e.push(u[r])}function Qu(n,t){for(var e=[n],r=[];null!=(n=e.pop());)if(r.push(n),(i=n.children)&&(u=i.length))for(var u,i,o=-1;++o<u;)e.push(i[o]);for(;null!=(n=r.pop());)t(n)}function ni(n){return n.children}function ti(n){return n.value}function ei(n,t){return t.value-n.value}function ri(n){return ta.merge(n.map(function(n){return(n.children||[]).map(function(t){return{source:n,target:t}})}))}function ui(n){return n.x}function ii(n){return n.y}function oi(n,t,e){n.y0=t,n.y=e}function ai(n){return ta.range(n.length)}function ci(n){for(var t=-1,e=n[0].length,r=[];++t<e;)r[t]=0;return r}function li(n){for(var t,e=1,r=0,u=n[0][1],i=n.length;i>e;++e)(t=n[e][1])>u&&(r=e,u=t);return r}function si(n){return n.reduce(fi,0)}function fi(n,t){return n+t[1]}function hi(n,t){return gi(n,Math.ceil(Math.log(t.length)/Math.LN2+1))}function gi(n,t){for(var e=-1,r=+n[0],u=(n[1]-r)/t,i=[];++e<=t;)i[e]=u*e+r;return i}function pi(n){return[ta.min(n),ta.max(n)]}function vi(n,t){return n.value-t.value}function di(n,t){var e=n._pack_next;n._pack_next=t,t._pack_prev=n,t._pack_next=e,e._pack_prev=t}function mi(n,t){n._pack_next=t,t._pack_prev=n}function yi(n,t){var e=t.x-n.x,r=t.y-n.y,u=n.r+t.r;return.999*u*u>e*e+r*r}function Mi(n){function t(n){s=Math.min(n.x-n.r,s),f=Math.max(n.x+n.r,f),h=Math.min(n.y-n.r,h),g=Math.max(n.y+n.r,g)}if((e=n.children)&&(l=e.length)){var e,r,u,i,o,a,c,l,s=1/0,f=-1/0,h=1/0,g=-1/0;if(e.forEach(xi),r=e[0],r.x=-r.r,r.y=0,t(r),l>1&&(u=e[1],u.x=u.r,u.y=0,t(u),l>2))for(i=e[2],wi(r,u,i),t(i),di(r,i),r._pack_prev=i,di(i,u),u=r._pack_next,o=3;l>o;o++){wi(r,u,i=e[o]);var p=0,v=1,d=1;for(a=u._pack_next;a!==u;a=a._pack_next,v++)if(yi(a,i)){p=1;break}if(1==p)for(c=r._pack_prev;c!==a._pack_prev&&!yi(c,i);c=c._pack_prev,d++);p?(d>v||v==d&&u.r<r.r?mi(r,u=a):mi(r=c,u),o--):(di(r,i),u=i,t(i))}var m=(s+f)/2,y=(h+g)/2,M=0;for(o=0;l>o;o++)i=e[o],i.x-=m,i.y-=y,M=Math.max(M,i.r+Math.sqrt(i.x*i.x+i.y*i.y));n.r=M,e.forEach(bi)}}function xi(n){n._pack_next=n._pack_prev=n}function bi(n){delete n._pack_next,delete n._pack_prev}function _i(n,t,e,r){var u=n.children;if(n.x=t+=r*n.x,n.y=e+=r*n.y,n.r*=r,u)for(var i=-1,o=u.length;++i<o;)_i(u[i],t,e,r)}function wi(n,t,e){var r=n.r+e.r,u=t.x-n.x,i=t.y-n.y;if(r&&(u||i)){var o=t.r+e.r,a=u*u+i*i;o*=o,r*=r;var c=.5+(r-o)/(2*a),l=Math.sqrt(Math.max(0,2*o*(r+a)-(r-=a)*r-o*o))/(2*a);e.x=n.x+c*u+l*i,e.y=n.y+c*i-l*u}else e.x=n.x+r,e.y=n.y}function Si(n,t){return n.parent==t.parent?1:2}function ki(n){var t=n.children;return t.length?t[0]:n.t}function Ei(n){var t,e=n.children;return(t=e.length)?e[t-1]:n.t}function Ai(n,t,e){var r=e/(t.i-n.i);t.c-=r,t.s+=e,n.c+=r,t.z+=e,t.m+=e}function Ni(n){for(var t,e=0,r=0,u=n.children,i=u.length;--i>=0;)t=u[i],t.z+=e,t.m+=e,e+=t.s+(r+=t.c)}function Ci(n,t,e){return n.a.parent===t.parent?n.a:e}function zi(n){return 1+ta.max(n,function(n){return n.y})}function qi(n){return n.reduce(function(n,t){return n+t.x},0)/n.length}function Li(n){var t=n.children;return t&&t.length?Li(t[0]):n}function Ti(n){var t,e=n.children;return e&&(t=e.length)?Ti(e[t-1]):n}function Ri(n){return{x:n.x,y:n.y,dx:n.dx,dy:n.dy}}function Di(n,t){var e=n.x+t[3],r=n.y+t[0],u=n.dx-t[1]-t[3],i=n.dy-t[0]-t[2];return 0>u&&(e+=u/2,u=0),0>i&&(r+=i/2,i=0),{x:e,y:r,dx:u,dy:i}}function Pi(n){var t=n[0],e=n[n.length-1];return e>t?[t,e]:[e,t]}function Ui(n){return n.rangeExtent?n.rangeExtent():Pi(n.range())}function ji(n,t,e,r){var u=e(n[0],n[1]),i=r(t[0],t[1]);return function(n){return i(u(n))}}function Fi(n,t){var e,r=0,u=n.length-1,i=n[r],o=n[u];return i>o&&(e=r,r=u,u=e,e=i,i=o,o=e),n[r]=t.floor(i),n[u]=t.ceil(o),n}function Hi(n){return n?{floor:function(t){return Math.floor(t/n)*n},ceil:function(t){return Math.ceil(t/n)*n}}:ml}function Oi(n,t,e,r){var u=[],i=[],o=0,a=Math.min(n.length,t.length)-1;for(n[a]<n[0]&&(n=n.slice().reverse(),t=t.slice().reverse());++o<=a;)u.push(e(n[o-1],n[o])),i.push(r(t[o-1],t[o]));return function(t){var e=ta.bisect(n,t,1,a)-1;return i[e](u[e](t))}}function Ii(n,t,e,r){function u(){var u=Math.min(n.length,t.length)>2?Oi:ji,c=r?Iu:Ou;return o=u(n,t,c,e),a=u(t,n,c,mu),i}function i(n){return o(n)}var o,a;return i.invert=function(n){return a(n)},i.domain=function(t){return arguments.length?(n=t.map(Number),u()):n},i.range=function(n){return arguments.length?(t=n,u()):t},i.rangeRound=function(n){return i.range(n).interpolate(Du)},i.clamp=function(n){return arguments.length?(r=n,u()):r},i.interpolate=function(n){return arguments.length?(e=n,u()):e},i.ticks=function(t){return Xi(n,t)},i.tickFormat=function(t,e){return $i(n,t,e)},i.nice=function(t){return Zi(n,t),u()},i.copy=function(){return Ii(n,t,e,r)},u()}function Yi(n,t){return ta.rebind(n,t,"range","rangeRound","interpolate","clamp")}function Zi(n,t){return Fi(n,Hi(Vi(n,t)[2]))}function Vi(n,t){null==t&&(t=10);var e=Pi(n),r=e[1]-e[0],u=Math.pow(10,Math.floor(Math.log(r/t)/Math.LN10)),i=t/r*u;return.15>=i?u*=10:.35>=i?u*=5:.75>=i&&(u*=2),e[0]=Math.ceil(e[0]/u)*u,e[1]=Math.floor(e[1]/u)*u+.5*u,e[2]=u,e}function Xi(n,t){return ta.range.apply(ta,Vi(n,t))}function $i(n,t,e){var r=Vi(n,t);if(e){var u=ic.exec(e);if(u.shift(),"s"===u[8]){var i=ta.formatPrefix(Math.max(ga(r[0]),ga(r[1])));return u[7]||(u[7]="."+Bi(i.scale(r[2]))),u[8]="f",e=ta.format(u.join("")),function(n){return e(i.scale(n))+i.symbol}}u[7]||(u[7]="."+Wi(u[8],r)),e=u.join("")}else e=",."+Bi(r[2])+"f";return ta.format(e)}function Bi(n){return-Math.floor(Math.log(n)/Math.LN10+.01)}function Wi(n,t){var e=Bi(t[2]);return n in yl?Math.abs(e-Bi(Math.max(ga(t[0]),ga(t[1]))))+ +("e"!==n):e-2*("%"===n)}function Ji(n,t,e,r){function u(n){return(e?Math.log(0>n?0:n):-Math.log(n>0?0:-n))/Math.log(t)}function i(n){return e?Math.pow(t,n):-Math.pow(t,-n)}function o(t){return n(u(t))}return o.invert=function(t){return i(n.invert(t))},o.domain=function(t){return arguments.length?(e=t[0]>=0,n.domain((r=t.map(Number)).map(u)),o):r},o.base=function(e){return arguments.length?(t=+e,n.domain(r.map(u)),o):t},o.nice=function(){var t=Fi(r.map(u),e?Math:xl);return n.domain(t),r=t.map(i),o},o.ticks=function(){var n=Pi(r),o=[],a=n[0],c=n[1],l=Math.floor(u(a)),s=Math.ceil(u(c)),f=t%1?2:t;if(isFinite(s-l)){if(e){for(;s>l;l++)for(var h=1;f>h;h++)o.push(i(l)*h);o.push(i(l))}else for(o.push(i(l));l++<s;)for(var h=f-1;h>0;h--)o.push(i(l)*h);for(l=0;o[l]<a;l++);for(s=o.length;o[s-1]>c;s--);o=o.slice(l,s)}return o},o.tickFormat=function(n,t){if(!arguments.length)return Ml;arguments.length<2?t=Ml:"function"!=typeof t&&(t=ta.format(t));var r,a=Math.max(.1,n/o.ticks().length),c=e?(r=1e-12,Math.ceil):(r=-1e-12,Math.floor);return function(n){return n/i(c(u(n)+r))<=a?t(n):""}},o.copy=function(){return Ji(n.copy(),t,e,r)},Yi(o,n)}function Gi(n,t,e){function r(t){return n(u(t))}var u=Ki(t),i=Ki(1/t);return r.invert=function(t){return i(n.invert(t))},r.domain=function(t){return arguments.length?(n.domain((e=t.map(Number)).map(u)),r):e},r.ticks=function(n){return Xi(e,n)},r.tickFormat=function(n,t){return $i(e,n,t)},r.nice=function(n){return r.domain(Zi(e,n))},r.exponent=function(o){return arguments.length?(u=Ki(t=o),i=Ki(1/t),n.domain(e.map(u)),r):t},r.copy=function(){return Gi(n.copy(),t,e)},Yi(r,n)}function Ki(n){return function(t){return 0>t?-Math.pow(-t,n):Math.pow(t,n)}}function Qi(n,t){function e(e){return i[((u.get(e)||("range"===t.t?u.set(e,n.push(e)):0/0))-1)%i.length]}function r(t,e){return ta.range(n.length).map(function(n){return t+e*n})}var u,i,o;return e.domain=function(r){if(!arguments.length)return n;n=[],u=new l;for(var i,o=-1,a=r.length;++o<a;)u.has(i=r[o])||u.set(i,n.push(i));return e[t.t].apply(e,t.a)},e.range=function(n){return arguments.length?(i=n,o=0,t={t:"range",a:arguments},e):i},e.rangePoints=function(u,a){arguments.length<2&&(a=0);var c=u[0],l=u[1],s=n.length<2?(c=(c+l)/2,0):(l-c)/(n.length-1+a);return i=r(c+s*a/2,s),o=0,t={t:"rangePoints",a:arguments},e},e.rangeRoundPoints=function(u,a){arguments.length<2&&(a=0);var c=u[0],l=u[1],s=n.length<2?(c=l=Math.round((c+l)/2),0):(l-c)/(n.length-1+a)|0;return i=r(c+Math.round(s*a/2+(l-c-(n.length-1+a)*s)/2),s),o=0,t={t:"rangeRoundPoints",a:arguments},e},e.rangeBands=function(u,a,c){arguments.length<2&&(a=0),arguments.length<3&&(c=a);var l=u[1]<u[0],s=u[l-0],f=u[1-l],h=(f-s)/(n.length-a+2*c);return i=r(s+h*c,h),l&&i.reverse(),o=h*(1-a),t={t:"rangeBands",a:arguments},e},e.rangeRoundBands=function(u,a,c){arguments.length<2&&(a=0),arguments.length<3&&(c=a);var l=u[1]<u[0],s=u[l-0],f=u[1-l],h=Math.floor((f-s)/(n.length-a+2*c));return i=r(s+Math.round((f-s-(n.length-a)*h)/2),h),l&&i.reverse(),o=Math.round(h*(1-a)),t={t:"rangeRoundBands",a:arguments},e},e.rangeBand=function(){return o},e.rangeExtent=function(){return Pi(t.a[0])},e.copy=function(){return Qi(n,t)},e.domain(n)}function no(n,t){function i(){var e=0,r=t.length;for(a=[];++e<r;)a[e-1]=ta.quantile(n,e/r);return o}function o(n){return isNaN(n=+n)?void 0:t[ta.bisect(a,n)]}var a;return o.domain=function(t){return arguments.length?(n=t.map(r).filter(u).sort(e),i()):n},o.range=function(n){return arguments.length?(t=n,i()):t},o.quantiles=function(){return a},o.invertExtent=function(e){return e=t.indexOf(e),0>e?[0/0,0/0]:[e>0?a[e-1]:n[0],e<a.length?a[e]:n[n.length-1]]},o.copy=function(){return no(n,t)},i()}function to(n,t,e){function r(t){return e[Math.max(0,Math.min(o,Math.floor(i*(t-n))))]}function u(){return i=e.length/(t-n),o=e.length-1,r}var i,o;return r.domain=function(e){return arguments.length?(n=+e[0],t=+e[e.length-1],u()):[n,t]},r.range=function(n){return arguments.length?(e=n,u()):e},r.invertExtent=function(t){return t=e.indexOf(t),t=0>t?0/0:t/i+n,[t,t+1/i]},r.copy=function(){return to(n,t,e)},u()}function eo(n,t){function e(e){return e>=e?t[ta.bisect(n,e)]:void 0}return e.domain=function(t){return arguments.length?(n=t,e):n},e.range=function(n){return arguments.length?(t=n,e):t},e.invertExtent=function(e){return e=t.indexOf(e),[n[e-1],n[e]]},e.copy=function(){return eo(n,t)},e}function ro(n){function t(n){return+n}return t.invert=t,t.domain=t.range=function(e){return arguments.length?(n=e.map(t),t):n},t.ticks=function(t){return Xi(n,t)},t.tickFormat=function(t,e){return $i(n,t,e)},t.copy=function(){return ro(n)},t}function uo(){return 0}function io(n){return n.innerRadius}function oo(n){return n.outerRadius}function ao(n){return n.startAngle}function co(n){return n.endAngle}function lo(n){return n&&n.padAngle}function so(n,t,e,r){return(n-e)*t-(t-r)*n>0?0:1}function fo(n,t,e,r,u){var i=n[0]-t[0],o=n[1]-t[1],a=(u?r:-r)/Math.sqrt(i*i+o*o),c=a*o,l=-a*i,s=n[0]+c,f=n[1]+l,h=t[0]+c,g=t[1]+l,p=(s+h)/2,v=(f+g)/2,d=h-s,m=g-f,y=d*d+m*m,M=e-r,x=s*g-h*f,b=(0>m?-1:1)*Math.sqrt(M*M*y-x*x),_=(x*m-d*b)/y,w=(-x*d-m*b)/y,S=(x*m+d*b)/y,k=(-x*d+m*b)/y,E=_-p,A=w-v,N=S-p,C=k-v;return E*E+A*A>N*N+C*C&&(_=S,w=k),[[_-c,w-l],[_*e/M,w*e/M]]}function ho(n){function t(t){function o(){l.push("M",i(n(s),a))}for(var c,l=[],s=[],f=-1,h=t.length,g=Et(e),p=Et(r);++f<h;)u.call(this,c=t[f],f)?s.push([+g.call(this,c,f),+p.call(this,c,f)]):s.length&&(o(),s=[]);return s.length&&o(),l.length?l.join(""):null}var e=Ar,r=Nr,u=Ne,i=go,o=i.key,a=.7;return t.x=function(n){return arguments.length?(e=n,t):e},t.y=function(n){return arguments.length?(r=n,t):r},t.defined=function(n){return arguments.length?(u=n,t):u},t.interpolate=function(n){return arguments.length?(o="function"==typeof n?i=n:(i=El.get(n)||go).key,t):o},t.tension=function(n){return arguments.length?(a=n,t):a},t}function go(n){return n.join("L")}function po(n){return go(n)+"Z"}function vo(n){for(var t=0,e=n.length,r=n[0],u=[r[0],",",r[1]];++t<e;)u.push("H",(r[0]+(r=n[t])[0])/2,"V",r[1]);return e>1&&u.push("H",r[0]),u.join("")}function mo(n){for(var t=0,e=n.length,r=n[0],u=[r[0],",",r[1]];++t<e;)u.push("V",(r=n[t])[1],"H",r[0]);return u.join("")}function yo(n){for(var t=0,e=n.length,r=n[0],u=[r[0],",",r[1]];++t<e;)u.push("H",(r=n[t])[0],"V",r[1]);return u.join("")}function Mo(n,t){return n.length<4?go(n):n[1]+_o(n.slice(1,-1),wo(n,t))}function xo(n,t){return n.length<3?go(n):n[0]+_o((n.push(n[0]),n),wo([n[n.length-2]].concat(n,[n[1]]),t))}function bo(n,t){return n.length<3?go(n):n[0]+_o(n,wo(n,t))}function _o(n,t){if(t.length<1||n.length!=t.length&&n.length!=t.length+2)return go(n);var e=n.length!=t.length,r="",u=n[0],i=n[1],o=t[0],a=o,c=1;if(e&&(r+="Q"+(i[0]-2*o[0]/3)+","+(i[1]-2*o[1]/3)+","+i[0]+","+i[1],u=n[1],c=2),t.length>1){a=t[1],i=n[c],c++,r+="C"+(u[0]+o[0])+","+(u[1]+o[1])+","+(i[0]-a[0])+","+(i[1]-a[1])+","+i[0]+","+i[1];for(var l=2;l<t.length;l++,c++)i=n[c],a=t[l],r+="S"+(i[0]-a[0])+","+(i[1]-a[1])+","+i[0]+","+i[1]}if(e){var s=n[c];r+="Q"+(i[0]+2*a[0]/3)+","+(i[1]+2*a[1]/3)+","+s[0]+","+s[1]}return r}function wo(n,t){for(var e,r=[],u=(1-t)/2,i=n[0],o=n[1],a=1,c=n.length;++a<c;)e=i,i=o,o=n[a],r.push([u*(o[0]-e[0]),u*(o[1]-e[1])]);return r}function So(n){if(n.length<3)return go(n);var t=1,e=n.length,r=n[0],u=r[0],i=r[1],o=[u,u,u,(r=n[1])[0]],a=[i,i,i,r[1]],c=[u,",",i,"L",No(Cl,o),",",No(Cl,a)];for(n.push(n[e-1]);++t<=e;)r=n[t],o.shift(),o.push(r[0]),a.shift(),a.push(r[1]),Co(c,o,a);return n.pop(),c.push("L",r),c.join("")}function ko(n){if(n.length<4)return go(n);for(var t,e=[],r=-1,u=n.length,i=[0],o=[0];++r<3;)t=n[r],i.push(t[0]),o.push(t[1]);for(e.push(No(Cl,i)+","+No(Cl,o)),--r;++r<u;)t=n[r],i.shift(),i.push(t[0]),o.shift(),o.push(t[1]),Co(e,i,o);return e.join("")}function Eo(n){for(var t,e,r=-1,u=n.length,i=u+4,o=[],a=[];++r<4;)e=n[r%u],o.push(e[0]),a.push(e[1]);for(t=[No(Cl,o),",",No(Cl,a)],--r;++r<i;)e=n[r%u],o.shift(),o.push(e[0]),a.shift(),a.push(e[1]),Co(t,o,a);return t.join("")}function Ao(n,t){var e=n.length-1;if(e)for(var r,u,i=n[0][0],o=n[0][1],a=n[e][0]-i,c=n[e][1]-o,l=-1;++l<=e;)r=n[l],u=l/e,r[0]=t*r[0]+(1-t)*(i+u*a),r[1]=t*r[1]+(1-t)*(o+u*c);return So(n)}function No(n,t){return n[0]*t[0]+n[1]*t[1]+n[2]*t[2]+n[3]*t[3]}function Co(n,t,e){n.push("C",No(Al,t),",",No(Al,e),",",No(Nl,t),",",No(Nl,e),",",No(Cl,t),",",No(Cl,e))}function zo(n,t){return(t[1]-n[1])/(t[0]-n[0])}function qo(n){for(var t=0,e=n.length-1,r=[],u=n[0],i=n[1],o=r[0]=zo(u,i);++t<e;)r[t]=(o+(o=zo(u=i,i=n[t+1])))/2;return r[t]=o,r}function Lo(n){for(var t,e,r,u,i=[],o=qo(n),a=-1,c=n.length-1;++a<c;)t=zo(n[a],n[a+1]),ga(t)<Ca?o[a]=o[a+1]=0:(e=o[a]/t,r=o[a+1]/t,u=e*e+r*r,u>9&&(u=3*t/Math.sqrt(u),o[a]=u*e,o[a+1]=u*r));for(a=-1;++a<=c;)u=(n[Math.min(c,a+1)][0]-n[Math.max(0,a-1)][0])/(6*(1+o[a]*o[a])),i.push([u||0,o[a]*u||0]);return i}function To(n){return n.length<3?go(n):n[0]+_o(n,Lo(n))}function Ro(n){for(var t,e,r,u=-1,i=n.length;++u<i;)t=n[u],e=t[0],r=t[1]-Ra,t[0]=e*Math.cos(r),t[1]=e*Math.sin(r);return n}function Do(n){function t(t){function c(){v.push("M",a(n(m),f),s,l(n(d.reverse()),f),"Z")}for(var h,g,p,v=[],d=[],m=[],y=-1,M=t.length,x=Et(e),b=Et(u),_=e===r?function(){return g}:Et(r),w=u===i?function(){return p}:Et(i);++y<M;)o.call(this,h=t[y],y)?(d.push([g=+x.call(this,h,y),p=+b.call(this,h,y)]),m.push([+_.call(this,h,y),+w.call(this,h,y)])):d.length&&(c(),d=[],m=[]);return d.length&&c(),v.length?v.join(""):null}var e=Ar,r=Ar,u=0,i=Nr,o=Ne,a=go,c=a.key,l=a,s="L",f=.7;return t.x=function(n){return arguments.length?(e=r=n,t):r},t.x0=function(n){return arguments.length?(e=n,t):e},t.x1=function(n){return arguments.length?(r=n,t):r
},t.y=function(n){return arguments.length?(u=i=n,t):i},t.y0=function(n){return arguments.length?(u=n,t):u},t.y1=function(n){return arguments.length?(i=n,t):i},t.defined=function(n){return arguments.length?(o=n,t):o},t.interpolate=function(n){return arguments.length?(c="function"==typeof n?a=n:(a=El.get(n)||go).key,l=a.reverse||a,s=a.closed?"M":"L",t):c},t.tension=function(n){return arguments.length?(f=n,t):f},t}function Po(n){return n.radius}function Uo(n){return[n.x,n.y]}function jo(n){return function(){var t=n.apply(this,arguments),e=t[0],r=t[1]-Ra;return[e*Math.cos(r),e*Math.sin(r)]}}function Fo(){return 64}function Ho(){return"circle"}function Oo(n){var t=Math.sqrt(n/qa);return"M0,"+t+"A"+t+","+t+" 0 1,1 0,"+-t+"A"+t+","+t+" 0 1,1 0,"+t+"Z"}function Io(n){return function(){var t,e;(t=this[n])&&(e=t[t.active])&&(--t.count?delete t[t.active]:delete this[n],t.active+=.5,e.event&&e.event.interrupt.call(this,this.__data__,e.index))}}function Yo(n,t,e){return ya(n,Pl),n.namespace=t,n.id=e,n}function Zo(n,t,e,r){var u=n.id,i=n.namespace;return Y(n,"function"==typeof e?function(n,o,a){n[i][u].tween.set(t,r(e.call(n,n.__data__,o,a)))}:(e=r(e),function(n){n[i][u].tween.set(t,e)}))}function Vo(n){return null==n&&(n=""),function(){this.textContent=n}}function Xo(n){return null==n?"__transition__":"__transition_"+n+"__"}function $o(n,t,e,r,u){var i=n[e]||(n[e]={active:0,count:0}),o=i[r];if(!o){var a=u.time;o=i[r]={tween:new l,time:a,delay:u.delay,duration:u.duration,ease:u.ease,index:t},u=null,++i.count,ta.timer(function(u){function c(e){if(i.active>r)return s();var u=i[i.active];u&&(--i.count,delete i[i.active],u.event&&u.event.interrupt.call(n,n.__data__,u.index)),i.active=r,o.event&&o.event.start.call(n,n.__data__,t),o.tween.forEach(function(e,r){(r=r.call(n,n.__data__,t))&&v.push(r)}),h=o.ease,f=o.duration,ta.timer(function(){return p.c=l(e||1)?Ne:l,1},0,a)}function l(e){if(i.active!==r)return 1;for(var u=e/f,a=h(u),c=v.length;c>0;)v[--c].call(n,a);return u>=1?(o.event&&o.event.end.call(n,n.__data__,t),s()):void 0}function s(){return--i.count?delete i[r]:delete n[e],1}var f,h,g=o.delay,p=ec,v=[];return p.t=g+a,u>=g?c(u-g):void(p.c=c)},0,a)}}function Bo(n,t,e){n.attr("transform",function(n){var r=t(n);return"translate("+(isFinite(r)?r:e(n))+",0)"})}function Wo(n,t,e){n.attr("transform",function(n){var r=t(n);return"translate(0,"+(isFinite(r)?r:e(n))+")"})}function Jo(n){return n.toISOString()}function Go(n,t,e){function r(t){return n(t)}function u(n,e){var r=n[1]-n[0],u=r/e,i=ta.bisect(Vl,u);return i==Vl.length?[t.year,Vi(n.map(function(n){return n/31536e6}),e)[2]]:i?t[u/Vl[i-1]<Vl[i]/u?i-1:i]:[Bl,Vi(n,e)[2]]}return r.invert=function(t){return Ko(n.invert(t))},r.domain=function(t){return arguments.length?(n.domain(t),r):n.domain().map(Ko)},r.nice=function(n,t){function e(e){return!isNaN(e)&&!n.range(e,Ko(+e+1),t).length}var i=r.domain(),o=Pi(i),a=null==n?u(o,10):"number"==typeof n&&u(o,n);return a&&(n=a[0],t=a[1]),r.domain(Fi(i,t>1?{floor:function(t){for(;e(t=n.floor(t));)t=Ko(t-1);return t},ceil:function(t){for(;e(t=n.ceil(t));)t=Ko(+t+1);return t}}:n))},r.ticks=function(n,t){var e=Pi(r.domain()),i=null==n?u(e,10):"number"==typeof n?u(e,n):!n.range&&[{range:n},t];return i&&(n=i[0],t=i[1]),n.range(e[0],Ko(+e[1]+1),1>t?1:t)},r.tickFormat=function(){return e},r.copy=function(){return Go(n.copy(),t,e)},Yi(r,n)}function Ko(n){return new Date(n)}function Qo(n){return JSON.parse(n.responseText)}function na(n){var t=ua.createRange();return t.selectNode(ua.body),t.createContextualFragment(n.responseText)}var ta={version:"3.5.5"},ea=[].slice,ra=function(n){return ea.call(n)},ua=this.document;if(ua)try{ra(ua.documentElement.childNodes)[0].nodeType}catch(ia){ra=function(n){for(var t=n.length,e=new Array(t);t--;)e[t]=n[t];return e}}if(Date.now||(Date.now=function(){return+new Date}),ua)try{ua.createElement("DIV").style.setProperty("opacity",0,"")}catch(oa){var aa=this.Element.prototype,ca=aa.setAttribute,la=aa.setAttributeNS,sa=this.CSSStyleDeclaration.prototype,fa=sa.setProperty;aa.setAttribute=function(n,t){ca.call(this,n,t+"")},aa.setAttributeNS=function(n,t,e){la.call(this,n,t,e+"")},sa.setProperty=function(n,t,e){fa.call(this,n,t+"",e)}}ta.ascending=e,ta.descending=function(n,t){return n>t?-1:t>n?1:t>=n?0:0/0},ta.min=function(n,t){var e,r,u=-1,i=n.length;if(1===arguments.length){for(;++u<i;)if(null!=(r=n[u])&&r>=r){e=r;break}for(;++u<i;)null!=(r=n[u])&&e>r&&(e=r)}else{for(;++u<i;)if(null!=(r=t.call(n,n[u],u))&&r>=r){e=r;break}for(;++u<i;)null!=(r=t.call(n,n[u],u))&&e>r&&(e=r)}return e},ta.max=function(n,t){var e,r,u=-1,i=n.length;if(1===arguments.length){for(;++u<i;)if(null!=(r=n[u])&&r>=r){e=r;break}for(;++u<i;)null!=(r=n[u])&&r>e&&(e=r)}else{for(;++u<i;)if(null!=(r=t.call(n,n[u],u))&&r>=r){e=r;break}for(;++u<i;)null!=(r=t.call(n,n[u],u))&&r>e&&(e=r)}return e},ta.extent=function(n,t){var e,r,u,i=-1,o=n.length;if(1===arguments.length){for(;++i<o;)if(null!=(r=n[i])&&r>=r){e=u=r;break}for(;++i<o;)null!=(r=n[i])&&(e>r&&(e=r),r>u&&(u=r))}else{for(;++i<o;)if(null!=(r=t.call(n,n[i],i))&&r>=r){e=u=r;break}for(;++i<o;)null!=(r=t.call(n,n[i],i))&&(e>r&&(e=r),r>u&&(u=r))}return[e,u]},ta.sum=function(n,t){var e,r=0,i=n.length,o=-1;if(1===arguments.length)for(;++o<i;)u(e=+n[o])&&(r+=e);else for(;++o<i;)u(e=+t.call(n,n[o],o))&&(r+=e);return r},ta.mean=function(n,t){var e,i=0,o=n.length,a=-1,c=o;if(1===arguments.length)for(;++a<o;)u(e=r(n[a]))?i+=e:--c;else for(;++a<o;)u(e=r(t.call(n,n[a],a)))?i+=e:--c;return c?i/c:void 0},ta.quantile=function(n,t){var e=(n.length-1)*t+1,r=Math.floor(e),u=+n[r-1],i=e-r;return i?u+i*(n[r]-u):u},ta.median=function(n,t){var i,o=[],a=n.length,c=-1;if(1===arguments.length)for(;++c<a;)u(i=r(n[c]))&&o.push(i);else for(;++c<a;)u(i=r(t.call(n,n[c],c)))&&o.push(i);return o.length?ta.quantile(o.sort(e),.5):void 0},ta.variance=function(n,t){var e,i,o=n.length,a=0,c=0,l=-1,s=0;if(1===arguments.length)for(;++l<o;)u(e=r(n[l]))&&(i=e-a,a+=i/++s,c+=i*(e-a));else for(;++l<o;)u(e=r(t.call(n,n[l],l)))&&(i=e-a,a+=i/++s,c+=i*(e-a));return s>1?c/(s-1):void 0},ta.deviation=function(){var n=ta.variance.apply(this,arguments);return n?Math.sqrt(n):n};var ha=i(e);ta.bisectLeft=ha.left,ta.bisect=ta.bisectRight=ha.right,ta.bisector=function(n){return i(1===n.length?function(t,r){return e(n(t),r)}:n)},ta.shuffle=function(n,t,e){(i=arguments.length)<3&&(e=n.length,2>i&&(t=0));for(var r,u,i=e-t;i;)u=Math.random()*i--|0,r=n[i+t],n[i+t]=n[u+t],n[u+t]=r;return n},ta.permute=function(n,t){for(var e=t.length,r=new Array(e);e--;)r[e]=n[t[e]];return r},ta.pairs=function(n){for(var t,e=0,r=n.length-1,u=n[0],i=new Array(0>r?0:r);r>e;)i[e]=[t=u,u=n[++e]];return i},ta.zip=function(){if(!(r=arguments.length))return[];for(var n=-1,t=ta.min(arguments,o),e=new Array(t);++n<t;)for(var r,u=-1,i=e[n]=new Array(r);++u<r;)i[u]=arguments[u][n];return e},ta.transpose=function(n){return ta.zip.apply(ta,n)},ta.keys=function(n){var t=[];for(var e in n)t.push(e);return t},ta.values=function(n){var t=[];for(var e in n)t.push(n[e]);return t},ta.entries=function(n){var t=[];for(var e in n)t.push({key:e,value:n[e]});return t},ta.merge=function(n){for(var t,e,r,u=n.length,i=-1,o=0;++i<u;)o+=n[i].length;for(e=new Array(o);--u>=0;)for(r=n[u],t=r.length;--t>=0;)e[--o]=r[t];return e};var ga=Math.abs;ta.range=function(n,t,e){if(arguments.length<3&&(e=1,arguments.length<2&&(t=n,n=0)),(t-n)/e===1/0)throw new Error("infinite range");var r,u=[],i=a(ga(e)),o=-1;if(n*=i,t*=i,e*=i,0>e)for(;(r=n+e*++o)>t;)u.push(r/i);else for(;(r=n+e*++o)<t;)u.push(r/i);return u},ta.map=function(n,t){var e=new l;if(n instanceof l)n.forEach(function(n,t){e.set(n,t)});else if(Array.isArray(n)){var r,u=-1,i=n.length;if(1===arguments.length)for(;++u<i;)e.set(u,n[u]);else for(;++u<i;)e.set(t.call(n,r=n[u],u),r)}else for(var o in n)e.set(o,n[o]);return e};var pa="__proto__",va="\x00";c(l,{has:h,get:function(n){return this._[s(n)]},set:function(n,t){return this._[s(n)]=t},remove:g,keys:p,values:function(){var n=[];for(var t in this._)n.push(this._[t]);return n},entries:function(){var n=[];for(var t in this._)n.push({key:f(t),value:this._[t]});return n},size:v,empty:d,forEach:function(n){for(var t in this._)n.call(this,f(t),this._[t])}}),ta.nest=function(){function n(t,o,a){if(a>=i.length)return r?r.call(u,o):e?o.sort(e):o;for(var c,s,f,h,g=-1,p=o.length,v=i[a++],d=new l;++g<p;)(h=d.get(c=v(s=o[g])))?h.push(s):d.set(c,[s]);return t?(s=t(),f=function(e,r){s.set(e,n(t,r,a))}):(s={},f=function(e,r){s[e]=n(t,r,a)}),d.forEach(f),s}function t(n,e){if(e>=i.length)return n;var r=[],u=o[e++];return n.forEach(function(n,u){r.push({key:n,values:t(u,e)})}),u?r.sort(function(n,t){return u(n.key,t.key)}):r}var e,r,u={},i=[],o=[];return u.map=function(t,e){return n(e,t,0)},u.entries=function(e){return t(n(ta.map,e,0),0)},u.key=function(n){return i.push(n),u},u.sortKeys=function(n){return o[i.length-1]=n,u},u.sortValues=function(n){return e=n,u},u.rollup=function(n){return r=n,u},u},ta.set=function(n){var t=new m;if(n)for(var e=0,r=n.length;r>e;++e)t.add(n[e]);return t},c(m,{has:h,add:function(n){return this._[s(n+="")]=!0,n},remove:g,values:p,size:v,empty:d,forEach:function(n){for(var t in this._)n.call(this,f(t))}}),ta.behavior={},ta.rebind=function(n,t){for(var e,r=1,u=arguments.length;++r<u;)n[e=arguments[r]]=M(n,t,t[e]);return n};var da=["webkit","ms","moz","Moz","o","O"];ta.dispatch=function(){for(var n=new _,t=-1,e=arguments.length;++t<e;)n[arguments[t]]=w(n);return n},_.prototype.on=function(n,t){var e=n.indexOf("."),r="";if(e>=0&&(r=n.slice(e+1),n=n.slice(0,e)),n)return arguments.length<2?this[n].on(r):this[n].on(r,t);if(2===arguments.length){if(null==t)for(n in this)this.hasOwnProperty(n)&&this[n].on(r,null);return this}},ta.event=null,ta.requote=function(n){return n.replace(ma,"\\$&")};var ma=/[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g,ya={}.__proto__?function(n,t){n.__proto__=t}:function(n,t){for(var e in t)n[e]=t[e]},Ma=function(n,t){return t.querySelector(n)},xa=function(n,t){return t.querySelectorAll(n)},ba=function(n,t){var e=n.matches||n[x(n,"matchesSelector")];return(ba=function(n,t){return e.call(n,t)})(n,t)};"function"==typeof Sizzle&&(Ma=function(n,t){return Sizzle(n,t)[0]||null},xa=Sizzle,ba=Sizzle.matchesSelector),ta.selection=function(){return ta.select(ua.documentElement)};var _a=ta.selection.prototype=[];_a.select=function(n){var t,e,r,u,i=[];n=N(n);for(var o=-1,a=this.length;++o<a;){i.push(t=[]),t.parentNode=(r=this[o]).parentNode;for(var c=-1,l=r.length;++c<l;)(u=r[c])?(t.push(e=n.call(u,u.__data__,c,o)),e&&"__data__"in u&&(e.__data__=u.__data__)):t.push(null)}return A(i)},_a.selectAll=function(n){var t,e,r=[];n=C(n);for(var u=-1,i=this.length;++u<i;)for(var o=this[u],a=-1,c=o.length;++a<c;)(e=o[a])&&(r.push(t=ra(n.call(e,e.__data__,a,u))),t.parentNode=e);return A(r)};var wa={svg:"http://www.w3.org/2000/svg",xhtml:"http://www.w3.org/1999/xhtml",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};ta.ns={prefix:wa,qualify:function(n){var t=n.indexOf(":"),e=n;return t>=0&&(e=n.slice(0,t),n=n.slice(t+1)),wa.hasOwnProperty(e)?{space:wa[e],local:n}:n}},_a.attr=function(n,t){if(arguments.length<2){if("string"==typeof n){var e=this.node();return n=ta.ns.qualify(n),n.local?e.getAttributeNS(n.space,n.local):e.getAttribute(n)}for(t in n)this.each(z(t,n[t]));return this}return this.each(z(n,t))},_a.classed=function(n,t){if(arguments.length<2){if("string"==typeof n){var e=this.node(),r=(n=T(n)).length,u=-1;if(t=e.classList){for(;++u<r;)if(!t.contains(n[u]))return!1}else for(t=e.getAttribute("class");++u<r;)if(!L(n[u]).test(t))return!1;return!0}for(t in n)this.each(R(t,n[t]));return this}return this.each(R(n,t))},_a.style=function(n,e,r){var u=arguments.length;if(3>u){if("string"!=typeof n){2>u&&(e="");for(r in n)this.each(P(r,n[r],e));return this}if(2>u){var i=this.node();return t(i).getComputedStyle(i,null).getPropertyValue(n)}r=""}return this.each(P(n,e,r))},_a.property=function(n,t){if(arguments.length<2){if("string"==typeof n)return this.node()[n];for(t in n)this.each(U(t,n[t]));return this}return this.each(U(n,t))},_a.text=function(n){return arguments.length?this.each("function"==typeof n?function(){var t=n.apply(this,arguments);this.textContent=null==t?"":t}:null==n?function(){this.textContent=""}:function(){this.textContent=n}):this.node().textContent},_a.html=function(n){return arguments.length?this.each("function"==typeof n?function(){var t=n.apply(this,arguments);this.innerHTML=null==t?"":t}:null==n?function(){this.innerHTML=""}:function(){this.innerHTML=n}):this.node().innerHTML},_a.append=function(n){return n=j(n),this.select(function(){return this.appendChild(n.apply(this,arguments))})},_a.insert=function(n,t){return n=j(n),t=N(t),this.select(function(){return this.insertBefore(n.apply(this,arguments),t.apply(this,arguments)||null)})},_a.remove=function(){return this.each(F)},_a.data=function(n,t){function e(n,e){var r,u,i,o=n.length,f=e.length,h=Math.min(o,f),g=new Array(f),p=new Array(f),v=new Array(o);if(t){var d,m=new l,y=new Array(o);for(r=-1;++r<o;)m.has(d=t.call(u=n[r],u.__data__,r))?v[r]=u:m.set(d,u),y[r]=d;for(r=-1;++r<f;)(u=m.get(d=t.call(e,i=e[r],r)))?u!==!0&&(g[r]=u,u.__data__=i):p[r]=H(i),m.set(d,!0);for(r=-1;++r<o;)m.get(y[r])!==!0&&(v[r]=n[r])}else{for(r=-1;++r<h;)u=n[r],i=e[r],u?(u.__data__=i,g[r]=u):p[r]=H(i);for(;f>r;++r)p[r]=H(e[r]);for(;o>r;++r)v[r]=n[r]}p.update=g,p.parentNode=g.parentNode=v.parentNode=n.parentNode,a.push(p),c.push(g),s.push(v)}var r,u,i=-1,o=this.length;if(!arguments.length){for(n=new Array(o=(r=this[0]).length);++i<o;)(u=r[i])&&(n[i]=u.__data__);return n}var a=Z([]),c=A([]),s=A([]);if("function"==typeof n)for(;++i<o;)e(r=this[i],n.call(r,r.parentNode.__data__,i));else for(;++i<o;)e(r=this[i],n);return c.enter=function(){return a},c.exit=function(){return s},c},_a.datum=function(n){return arguments.length?this.property("__data__",n):this.property("__data__")},_a.filter=function(n){var t,e,r,u=[];"function"!=typeof n&&(n=O(n));for(var i=0,o=this.length;o>i;i++){u.push(t=[]),t.parentNode=(e=this[i]).parentNode;for(var a=0,c=e.length;c>a;a++)(r=e[a])&&n.call(r,r.__data__,a,i)&&t.push(r)}return A(u)},_a.order=function(){for(var n=-1,t=this.length;++n<t;)for(var e,r=this[n],u=r.length-1,i=r[u];--u>=0;)(e=r[u])&&(i&&i!==e.nextSibling&&i.parentNode.insertBefore(e,i),i=e);return this},_a.sort=function(n){n=I.apply(this,arguments);for(var t=-1,e=this.length;++t<e;)this[t].sort(n);return this.order()},_a.each=function(n){return Y(this,function(t,e,r){n.call(t,t.__data__,e,r)})},_a.call=function(n){var t=ra(arguments);return n.apply(t[0]=this,t),this},_a.empty=function(){return!this.node()},_a.node=function(){for(var n=0,t=this.length;t>n;n++)for(var e=this[n],r=0,u=e.length;u>r;r++){var i=e[r];if(i)return i}return null},_a.size=function(){var n=0;return Y(this,function(){++n}),n};var Sa=[];ta.selection.enter=Z,ta.selection.enter.prototype=Sa,Sa.append=_a.append,Sa.empty=_a.empty,Sa.node=_a.node,Sa.call=_a.call,Sa.size=_a.size,Sa.select=function(n){for(var t,e,r,u,i,o=[],a=-1,c=this.length;++a<c;){r=(u=this[a]).update,o.push(t=[]),t.parentNode=u.parentNode;for(var l=-1,s=u.length;++l<s;)(i=u[l])?(t.push(r[l]=e=n.call(u.parentNode,i.__data__,l,a)),e.__data__=i.__data__):t.push(null)}return A(o)},Sa.insert=function(n,t){return arguments.length<2&&(t=V(this)),_a.insert.call(this,n,t)},ta.select=function(t){var e;return"string"==typeof t?(e=[Ma(t,ua)],e.parentNode=ua.documentElement):(e=[t],e.parentNode=n(t)),A([e])},ta.selectAll=function(n){var t;return"string"==typeof n?(t=ra(xa(n,ua)),t.parentNode=ua.documentElement):(t=n,t.parentNode=null),A([t])},_a.on=function(n,t,e){var r=arguments.length;if(3>r){if("string"!=typeof n){2>r&&(t=!1);for(e in n)this.each(X(e,n[e],t));return this}if(2>r)return(r=this.node()["__on"+n])&&r._;e=!1}return this.each(X(n,t,e))};var ka=ta.map({mouseenter:"mouseover",mouseleave:"mouseout"});ua&&ka.forEach(function(n){"on"+n in ua&&ka.remove(n)});var Ea,Aa=0;ta.mouse=function(n){return J(n,k())};var Na=this.navigator&&/WebKit/.test(this.navigator.userAgent)?-1:0;ta.touch=function(n,t,e){if(arguments.length<3&&(e=t,t=k().changedTouches),t)for(var r,u=0,i=t.length;i>u;++u)if((r=t[u]).identifier===e)return J(n,r)},ta.behavior.drag=function(){function n(){this.on("mousedown.drag",i).on("touchstart.drag",o)}function e(n,t,e,i,o){return function(){function a(){var n,e,r=t(h,v);r&&(n=r[0]-M[0],e=r[1]-M[1],p|=n|e,M=r,g({type:"drag",x:r[0]+l[0],y:r[1]+l[1],dx:n,dy:e}))}function c(){t(h,v)&&(m.on(i+d,null).on(o+d,null),y(p&&ta.event.target===f),g({type:"dragend"}))}var l,s=this,f=ta.event.target,h=s.parentNode,g=r.of(s,arguments),p=0,v=n(),d=".drag"+(null==v?"":"-"+v),m=ta.select(e(f)).on(i+d,a).on(o+d,c),y=W(f),M=t(h,v);u?(l=u.apply(s,arguments),l=[l.x-M[0],l.y-M[1]]):l=[0,0],g({type:"dragstart"})}}var r=E(n,"drag","dragstart","dragend"),u=null,i=e(b,ta.mouse,t,"mousemove","mouseup"),o=e(G,ta.touch,y,"touchmove","touchend");return n.origin=function(t){return arguments.length?(u=t,n):u},ta.rebind(n,r,"on")},ta.touches=function(n,t){return arguments.length<2&&(t=k().touches),t?ra(t).map(function(t){var e=J(n,t);return e.identifier=t.identifier,e}):[]};var Ca=1e-6,za=Ca*Ca,qa=Math.PI,La=2*qa,Ta=La-Ca,Ra=qa/2,Da=qa/180,Pa=180/qa,Ua=Math.SQRT2,ja=2,Fa=4;ta.interpolateZoom=function(n,t){function e(n){var t=n*y;if(m){var e=rt(v),o=i/(ja*h)*(e*ut(Ua*t+v)-et(v));return[r+o*l,u+o*s,i*e/rt(Ua*t+v)]}return[r+n*l,u+n*s,i*Math.exp(Ua*t)]}var r=n[0],u=n[1],i=n[2],o=t[0],a=t[1],c=t[2],l=o-r,s=a-u,f=l*l+s*s,h=Math.sqrt(f),g=(c*c-i*i+Fa*f)/(2*i*ja*h),p=(c*c-i*i-Fa*f)/(2*c*ja*h),v=Math.log(Math.sqrt(g*g+1)-g),d=Math.log(Math.sqrt(p*p+1)-p),m=d-v,y=(m||Math.log(c/i))/Ua;return e.duration=1e3*y,e},ta.behavior.zoom=function(){function n(n){n.on(q,f).on(Oa+".zoom",g).on("dblclick.zoom",p).on(R,h)}function e(n){return[(n[0]-k.x)/k.k,(n[1]-k.y)/k.k]}function r(n){return[n[0]*k.k+k.x,n[1]*k.k+k.y]}function u(n){k.k=Math.max(N[0],Math.min(N[1],n))}function i(n,t){t=r(t),k.x+=n[0]-t[0],k.y+=n[1]-t[1]}function o(t,e,r,o){t.__chart__={x:k.x,y:k.y,k:k.k},u(Math.pow(2,o)),i(d=e,r),t=ta.select(t),C>0&&(t=t.transition().duration(C)),t.call(n.event)}function a(){b&&b.domain(x.range().map(function(n){return(n-k.x)/k.k}).map(x.invert)),w&&w.domain(_.range().map(function(n){return(n-k.y)/k.k}).map(_.invert))}function c(n){z++||n({type:"zoomstart"})}function l(n){a(),n({type:"zoom",scale:k.k,translate:[k.x,k.y]})}function s(n){--z||n({type:"zoomend"}),d=null}function f(){function n(){f=1,i(ta.mouse(u),g),l(a)}function r(){h.on(L,null).on(T,null),p(f&&ta.event.target===o),s(a)}var u=this,o=ta.event.target,a=D.of(u,arguments),f=0,h=ta.select(t(u)).on(L,n).on(T,r),g=e(ta.mouse(u)),p=W(u);Dl.call(u),c(a)}function h(){function n(){var n=ta.touches(p);return g=k.k,n.forEach(function(n){n.identifier in d&&(d[n.identifier]=e(n))}),n}function t(){var t=ta.event.target;ta.select(t).on(x,r).on(b,a),_.push(t);for(var e=ta.event.changedTouches,u=0,i=e.length;i>u;++u)d[e[u].identifier]=null;var c=n(),l=Date.now();if(1===c.length){if(500>l-M){var s=c[0];o(p,s,d[s.identifier],Math.floor(Math.log(k.k)/Math.LN2)+1),S()}M=l}else if(c.length>1){var s=c[0],f=c[1],h=s[0]-f[0],g=s[1]-f[1];m=h*h+g*g}}function r(){var n,t,e,r,o=ta.touches(p);Dl.call(p);for(var a=0,c=o.length;c>a;++a,r=null)if(e=o[a],r=d[e.identifier]){if(t)break;n=e,t=r}if(r){var s=(s=e[0]-n[0])*s+(s=e[1]-n[1])*s,f=m&&Math.sqrt(s/m);n=[(n[0]+e[0])/2,(n[1]+e[1])/2],t=[(t[0]+r[0])/2,(t[1]+r[1])/2],u(f*g)}M=null,i(n,t),l(v)}function a(){if(ta.event.touches.length){for(var t=ta.event.changedTouches,e=0,r=t.length;r>e;++e)delete d[t[e].identifier];for(var u in d)return void n()}ta.selectAll(_).on(y,null),w.on(q,f).on(R,h),E(),s(v)}var g,p=this,v=D.of(p,arguments),d={},m=0,y=".zoom-"+ta.event.changedTouches[0].identifier,x="touchmove"+y,b="touchend"+y,_=[],w=ta.select(p),E=W(p);t(),c(v),w.on(q,null).on(R,t)}function g(){var n=D.of(this,arguments);y?clearTimeout(y):(v=e(d=m||ta.mouse(this)),Dl.call(this),c(n)),y=setTimeout(function(){y=null,s(n)},50),S(),u(Math.pow(2,.002*Ha())*k.k),i(d,v),l(n)}function p(){var n=ta.mouse(this),t=Math.log(k.k)/Math.LN2;o(this,n,e(n),ta.event.shiftKey?Math.ceil(t)-1:Math.floor(t)+1)}var v,d,m,y,M,x,b,_,w,k={x:0,y:0,k:1},A=[960,500],N=Ia,C=250,z=0,q="mousedown.zoom",L="mousemove.zoom",T="mouseup.zoom",R="touchstart.zoom",D=E(n,"zoomstart","zoom","zoomend");return Oa||(Oa="onwheel"in ua?(Ha=function(){return-ta.event.deltaY*(ta.event.deltaMode?120:1)},"wheel"):"onmousewheel"in ua?(Ha=function(){return ta.event.wheelDelta},"mousewheel"):(Ha=function(){return-ta.event.detail},"MozMousePixelScroll")),n.event=function(n){n.each(function(){var n=D.of(this,arguments),t=k;Tl?ta.select(this).transition().each("start.zoom",function(){k=this.__chart__||{x:0,y:0,k:1},c(n)}).tween("zoom:zoom",function(){var e=A[0],r=A[1],u=d?d[0]:e/2,i=d?d[1]:r/2,o=ta.interpolateZoom([(u-k.x)/k.k,(i-k.y)/k.k,e/k.k],[(u-t.x)/t.k,(i-t.y)/t.k,e/t.k]);return function(t){var r=o(t),a=e/r[2];this.__chart__=k={x:u-r[0]*a,y:i-r[1]*a,k:a},l(n)}}).each("interrupt.zoom",function(){s(n)}).each("end.zoom",function(){s(n)}):(this.__chart__=k,c(n),l(n),s(n))})},n.translate=function(t){return arguments.length?(k={x:+t[0],y:+t[1],k:k.k},a(),n):[k.x,k.y]},n.scale=function(t){return arguments.length?(k={x:k.x,y:k.y,k:+t},a(),n):k.k},n.scaleExtent=function(t){return arguments.length?(N=null==t?Ia:[+t[0],+t[1]],n):N},n.center=function(t){return arguments.length?(m=t&&[+t[0],+t[1]],n):m},n.size=function(t){return arguments.length?(A=t&&[+t[0],+t[1]],n):A},n.duration=function(t){return arguments.length?(C=+t,n):C},n.x=function(t){return arguments.length?(b=t,x=t.copy(),k={x:0,y:0,k:1},n):b},n.y=function(t){return arguments.length?(w=t,_=t.copy(),k={x:0,y:0,k:1},n):w},ta.rebind(n,D,"on")};var Ha,Oa,Ia=[0,1/0];ta.color=ot,ot.prototype.toString=function(){return this.rgb()+""},ta.hsl=at;var Ya=at.prototype=new ot;Ya.brighter=function(n){return n=Math.pow(.7,arguments.length?n:1),new at(this.h,this.s,this.l/n)},Ya.darker=function(n){return n=Math.pow(.7,arguments.length?n:1),new at(this.h,this.s,n*this.l)},Ya.rgb=function(){return ct(this.h,this.s,this.l)},ta.hcl=lt;var Za=lt.prototype=new ot;Za.brighter=function(n){return new lt(this.h,this.c,Math.min(100,this.l+Va*(arguments.length?n:1)))},Za.darker=function(n){return new lt(this.h,this.c,Math.max(0,this.l-Va*(arguments.length?n:1)))},Za.rgb=function(){return st(this.h,this.c,this.l).rgb()},ta.lab=ft;var Va=18,Xa=.95047,$a=1,Ba=1.08883,Wa=ft.prototype=new ot;Wa.brighter=function(n){return new ft(Math.min(100,this.l+Va*(arguments.length?n:1)),this.a,this.b)},Wa.darker=function(n){return new ft(Math.max(0,this.l-Va*(arguments.length?n:1)),this.a,this.b)},Wa.rgb=function(){return ht(this.l,this.a,this.b)},ta.rgb=mt;var Ja=mt.prototype=new ot;Ja.brighter=function(n){n=Math.pow(.7,arguments.length?n:1);var t=this.r,e=this.g,r=this.b,u=30;return t||e||r?(t&&u>t&&(t=u),e&&u>e&&(e=u),r&&u>r&&(r=u),new mt(Math.min(255,t/n),Math.min(255,e/n),Math.min(255,r/n))):new mt(u,u,u)},Ja.darker=function(n){return n=Math.pow(.7,arguments.length?n:1),new mt(n*this.r,n*this.g,n*this.b)},Ja.hsl=function(){return _t(this.r,this.g,this.b)},Ja.toString=function(){return"#"+xt(this.r)+xt(this.g)+xt(this.b)};var Ga=ta.map({aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074});Ga.forEach(function(n,t){Ga.set(n,yt(t))}),ta.functor=Et,ta.xhr=At(y),ta.dsv=function(n,t){function e(n,e,i){arguments.length<3&&(i=e,e=null);var o=Nt(n,t,null==e?r:u(e),i);return o.row=function(n){return arguments.length?o.response(null==(e=n)?r:u(n)):e},o}function r(n){return e.parse(n.responseText)}function u(n){return function(t){return e.parse(t.responseText,n)}}function i(t){return t.map(o).join(n)}function o(n){return a.test(n)?'"'+n.replace(/\"/g,'""')+'"':n}var a=new RegExp('["'+n+"\n]"),c=n.charCodeAt(0);return e.parse=function(n,t){var r;return e.parseRows(n,function(n,e){if(r)return r(n,e-1);var u=new Function("d","return {"+n.map(function(n,t){return JSON.stringify(n)+": d["+t+"]"}).join(",")+"}");r=t?function(n,e){return t(u(n),e)}:u})},e.parseRows=function(n,t){function e(){if(s>=l)return o;if(u)return u=!1,i;var t=s;if(34===n.charCodeAt(t)){for(var e=t;e++<l;)if(34===n.charCodeAt(e)){if(34!==n.charCodeAt(e+1))break;++e}s=e+2;var r=n.charCodeAt(e+1);return 13===r?(u=!0,10===n.charCodeAt(e+2)&&++s):10===r&&(u=!0),n.slice(t+1,e).replace(/""/g,'"')}for(;l>s;){var r=n.charCodeAt(s++),a=1;if(10===r)u=!0;else if(13===r)u=!0,10===n.charCodeAt(s)&&(++s,++a);else if(r!==c)continue;return n.slice(t,s-a)}return n.slice(t)}for(var r,u,i={},o={},a=[],l=n.length,s=0,f=0;(r=e())!==o;){for(var h=[];r!==i&&r!==o;)h.push(r),r=e();t&&null==(h=t(h,f++))||a.push(h)}return a},e.format=function(t){if(Array.isArray(t[0]))return e.formatRows(t);var r=new m,u=[];return t.forEach(function(n){for(var t in n)r.has(t)||u.push(r.add(t))}),[u.map(o).join(n)].concat(t.map(function(t){return u.map(function(n){return o(t[n])}).join(n)})).join("\n")},e.formatRows=function(n){return n.map(i).join("\n")},e},ta.csv=ta.dsv(",","text/csv"),ta.tsv=ta.dsv("	","text/tab-separated-values");var Ka,Qa,nc,tc,ec,rc=this[x(this,"requestAnimationFrame")]||function(n){setTimeout(n,17)};ta.timer=function(n,t,e){var r=arguments.length;2>r&&(t=0),3>r&&(e=Date.now());var u=e+t,i={c:n,t:u,f:!1,n:null};Qa?Qa.n=i:Ka=i,Qa=i,nc||(tc=clearTimeout(tc),nc=1,rc(qt))},ta.timer.flush=function(){Lt(),Tt()},ta.round=function(n,t){return t?Math.round(n*(t=Math.pow(10,t)))/t:Math.round(n)};var uc=["y","z","a","f","p","n","\xb5","m","","k","M","G","T","P","E","Z","Y"].map(Dt);ta.formatPrefix=function(n,t){var e=0;return n&&(0>n&&(n*=-1),t&&(n=ta.round(n,Rt(n,t))),e=1+Math.floor(1e-12+Math.log(n)/Math.LN10),e=Math.max(-24,Math.min(24,3*Math.floor((e-1)/3)))),uc[8+e/3]};var ic=/(?:([^{])?([<>=^]))?([+\- ])?([$#])?(0)?(\d+)?(,)?(\.-?\d+)?([a-z%])?/i,oc=ta.map({b:function(n){return n.toString(2)},c:function(n){return String.fromCharCode(n)},o:function(n){return n.toString(8)},x:function(n){return n.toString(16)},X:function(n){return n.toString(16).toUpperCase()},g:function(n,t){return n.toPrecision(t)},e:function(n,t){return n.toExponential(t)},f:function(n,t){return n.toFixed(t)},r:function(n,t){return(n=ta.round(n,Rt(n,t))).toFixed(Math.max(0,Math.min(20,Rt(n*(1+1e-15),t))))}}),ac=ta.time={},cc=Date;jt.prototype={getDate:function(){return this._.getUTCDate()},getDay:function(){return this._.getUTCDay()},getFullYear:function(){return this._.getUTCFullYear()},getHours:function(){return this._.getUTCHours()},getMilliseconds:function(){return this._.getUTCMilliseconds()},getMinutes:function(){return this._.getUTCMinutes()},getMonth:function(){return this._.getUTCMonth()},getSeconds:function(){return this._.getUTCSeconds()},getTime:function(){return this._.getTime()},getTimezoneOffset:function(){return 0},valueOf:function(){return this._.valueOf()},setDate:function(){lc.setUTCDate.apply(this._,arguments)},setDay:function(){lc.setUTCDay.apply(this._,arguments)},setFullYear:function(){lc.setUTCFullYear.apply(this._,arguments)},setHours:function(){lc.setUTCHours.apply(this._,arguments)},setMilliseconds:function(){lc.setUTCMilliseconds.apply(this._,arguments)},setMinutes:function(){lc.setUTCMinutes.apply(this._,arguments)},setMonth:function(){lc.setUTCMonth.apply(this._,arguments)},setSeconds:function(){lc.setUTCSeconds.apply(this._,arguments)},setTime:function(){lc.setTime.apply(this._,arguments)}};var lc=Date.prototype;ac.year=Ft(function(n){return n=ac.day(n),n.setMonth(0,1),n},function(n,t){n.setFullYear(n.getFullYear()+t)},function(n){return n.getFullYear()}),ac.years=ac.year.range,ac.years.utc=ac.year.utc.range,ac.day=Ft(function(n){var t=new cc(2e3,0);return t.setFullYear(n.getFullYear(),n.getMonth(),n.getDate()),t},function(n,t){n.setDate(n.getDate()+t)},function(n){return n.getDate()-1}),ac.days=ac.day.range,ac.days.utc=ac.day.utc.range,ac.dayOfYear=function(n){var t=ac.year(n);return Math.floor((n-t-6e4*(n.getTimezoneOffset()-t.getTimezoneOffset()))/864e5)},["sunday","monday","tuesday","wednesday","thursday","friday","saturday"].forEach(function(n,t){t=7-t;var e=ac[n]=Ft(function(n){return(n=ac.day(n)).setDate(n.getDate()-(n.getDay()+t)%7),n},function(n,t){n.setDate(n.getDate()+7*Math.floor(t))},function(n){var e=ac.year(n).getDay();return Math.floor((ac.dayOfYear(n)+(e+t)%7)/7)-(e!==t)});ac[n+"s"]=e.range,ac[n+"s"].utc=e.utc.range,ac[n+"OfYear"]=function(n){var e=ac.year(n).getDay();return Math.floor((ac.dayOfYear(n)+(e+t)%7)/7)}}),ac.week=ac.sunday,ac.weeks=ac.sunday.range,ac.weeks.utc=ac.sunday.utc.range,ac.weekOfYear=ac.sundayOfYear;var sc={"-":"",_:" ",0:"0"},fc=/^\s*\d+/,hc=/^%/;ta.locale=function(n){return{numberFormat:Pt(n),timeFormat:Ot(n)}};var gc=ta.locale({decimal:".",thousands:",",grouping:[3],currency:["$",""],dateTime:"%a %b %e %X %Y",date:"%m/%d/%Y",time:"%H:%M:%S",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});ta.format=gc.numberFormat,ta.geo={},ce.prototype={s:0,t:0,add:function(n){le(n,this.t,pc),le(pc.s,this.s,this),this.s?this.t+=pc.t:this.s=pc.t
},reset:function(){this.s=this.t=0},valueOf:function(){return this.s}};var pc=new ce;ta.geo.stream=function(n,t){n&&vc.hasOwnProperty(n.type)?vc[n.type](n,t):se(n,t)};var vc={Feature:function(n,t){se(n.geometry,t)},FeatureCollection:function(n,t){for(var e=n.features,r=-1,u=e.length;++r<u;)se(e[r].geometry,t)}},dc={Sphere:function(n,t){t.sphere()},Point:function(n,t){n=n.coordinates,t.point(n[0],n[1],n[2])},MultiPoint:function(n,t){for(var e=n.coordinates,r=-1,u=e.length;++r<u;)n=e[r],t.point(n[0],n[1],n[2])},LineString:function(n,t){fe(n.coordinates,t,0)},MultiLineString:function(n,t){for(var e=n.coordinates,r=-1,u=e.length;++r<u;)fe(e[r],t,0)},Polygon:function(n,t){he(n.coordinates,t)},MultiPolygon:function(n,t){for(var e=n.coordinates,r=-1,u=e.length;++r<u;)he(e[r],t)},GeometryCollection:function(n,t){for(var e=n.geometries,r=-1,u=e.length;++r<u;)se(e[r],t)}};ta.geo.area=function(n){return mc=0,ta.geo.stream(n,Mc),mc};var mc,yc=new ce,Mc={sphere:function(){mc+=4*qa},point:b,lineStart:b,lineEnd:b,polygonStart:function(){yc.reset(),Mc.lineStart=ge},polygonEnd:function(){var n=2*yc;mc+=0>n?4*qa+n:n,Mc.lineStart=Mc.lineEnd=Mc.point=b}};ta.geo.bounds=function(){function n(n,t){M.push(x=[s=n,h=n]),f>t&&(f=t),t>g&&(g=t)}function t(t,e){var r=pe([t*Da,e*Da]);if(m){var u=de(m,r),i=[u[1],-u[0],0],o=de(i,u);Me(o),o=xe(o);var c=t-p,l=c>0?1:-1,v=o[0]*Pa*l,d=ga(c)>180;if(d^(v>l*p&&l*t>v)){var y=o[1]*Pa;y>g&&(g=y)}else if(v=(v+360)%360-180,d^(v>l*p&&l*t>v)){var y=-o[1]*Pa;f>y&&(f=y)}else f>e&&(f=e),e>g&&(g=e);d?p>t?a(s,t)>a(s,h)&&(h=t):a(t,h)>a(s,h)&&(s=t):h>=s?(s>t&&(s=t),t>h&&(h=t)):t>p?a(s,t)>a(s,h)&&(h=t):a(t,h)>a(s,h)&&(s=t)}else n(t,e);m=r,p=t}function e(){b.point=t}function r(){x[0]=s,x[1]=h,b.point=n,m=null}function u(n,e){if(m){var r=n-p;y+=ga(r)>180?r+(r>0?360:-360):r}else v=n,d=e;Mc.point(n,e),t(n,e)}function i(){Mc.lineStart()}function o(){u(v,d),Mc.lineEnd(),ga(y)>Ca&&(s=-(h=180)),x[0]=s,x[1]=h,m=null}function a(n,t){return(t-=n)<0?t+360:t}function c(n,t){return n[0]-t[0]}function l(n,t){return t[0]<=t[1]?t[0]<=n&&n<=t[1]:n<t[0]||t[1]<n}var s,f,h,g,p,v,d,m,y,M,x,b={point:n,lineStart:e,lineEnd:r,polygonStart:function(){b.point=u,b.lineStart=i,b.lineEnd=o,y=0,Mc.polygonStart()},polygonEnd:function(){Mc.polygonEnd(),b.point=n,b.lineStart=e,b.lineEnd=r,0>yc?(s=-(h=180),f=-(g=90)):y>Ca?g=90:-Ca>y&&(f=-90),x[0]=s,x[1]=h}};return function(n){g=h=-(s=f=1/0),M=[],ta.geo.stream(n,b);var t=M.length;if(t){M.sort(c);for(var e,r=1,u=M[0],i=[u];t>r;++r)e=M[r],l(e[0],u)||l(e[1],u)?(a(u[0],e[1])>a(u[0],u[1])&&(u[1]=e[1]),a(e[0],u[1])>a(u[0],u[1])&&(u[0]=e[0])):i.push(u=e);for(var o,e,p=-1/0,t=i.length-1,r=0,u=i[t];t>=r;u=e,++r)e=i[r],(o=a(u[1],e[0]))>p&&(p=o,s=e[0],h=u[1])}return M=x=null,1/0===s||1/0===f?[[0/0,0/0],[0/0,0/0]]:[[s,f],[h,g]]}}(),ta.geo.centroid=function(n){xc=bc=_c=wc=Sc=kc=Ec=Ac=Nc=Cc=zc=0,ta.geo.stream(n,qc);var t=Nc,e=Cc,r=zc,u=t*t+e*e+r*r;return za>u&&(t=kc,e=Ec,r=Ac,Ca>bc&&(t=_c,e=wc,r=Sc),u=t*t+e*e+r*r,za>u)?[0/0,0/0]:[Math.atan2(e,t)*Pa,tt(r/Math.sqrt(u))*Pa]};var xc,bc,_c,wc,Sc,kc,Ec,Ac,Nc,Cc,zc,qc={sphere:b,point:_e,lineStart:Se,lineEnd:ke,polygonStart:function(){qc.lineStart=Ee},polygonEnd:function(){qc.lineStart=Se}},Lc=Le(Ne,Pe,je,[-qa,-qa/2]),Tc=1e9;ta.geo.clipExtent=function(){var n,t,e,r,u,i,o={stream:function(n){return u&&(u.valid=!1),u=i(n),u.valid=!0,u},extent:function(a){return arguments.length?(i=Ie(n=+a[0][0],t=+a[0][1],e=+a[1][0],r=+a[1][1]),u&&(u.valid=!1,u=null),o):[[n,t],[e,r]]}};return o.extent([[0,0],[960,500]])},(ta.geo.conicEqualArea=function(){return Ye(Ze)}).raw=Ze,ta.geo.albers=function(){return ta.geo.conicEqualArea().rotate([96,0]).center([-.6,38.7]).parallels([29.5,45.5]).scale(1070)},ta.geo.albersUsa=function(){function n(n){var i=n[0],o=n[1];return t=null,e(i,o),t||(r(i,o),t)||u(i,o),t}var t,e,r,u,i=ta.geo.albers(),o=ta.geo.conicEqualArea().rotate([154,0]).center([-2,58.5]).parallels([55,65]),a=ta.geo.conicEqualArea().rotate([157,0]).center([-3,19.9]).parallels([8,18]),c={point:function(n,e){t=[n,e]}};return n.invert=function(n){var t=i.scale(),e=i.translate(),r=(n[0]-e[0])/t,u=(n[1]-e[1])/t;return(u>=.12&&.234>u&&r>=-.425&&-.214>r?o:u>=.166&&.234>u&&r>=-.214&&-.115>r?a:i).invert(n)},n.stream=function(n){var t=i.stream(n),e=o.stream(n),r=a.stream(n);return{point:function(n,u){t.point(n,u),e.point(n,u),r.point(n,u)},sphere:function(){t.sphere(),e.sphere(),r.sphere()},lineStart:function(){t.lineStart(),e.lineStart(),r.lineStart()},lineEnd:function(){t.lineEnd(),e.lineEnd(),r.lineEnd()},polygonStart:function(){t.polygonStart(),e.polygonStart(),r.polygonStart()},polygonEnd:function(){t.polygonEnd(),e.polygonEnd(),r.polygonEnd()}}},n.precision=function(t){return arguments.length?(i.precision(t),o.precision(t),a.precision(t),n):i.precision()},n.scale=function(t){return arguments.length?(i.scale(t),o.scale(.35*t),a.scale(t),n.translate(i.translate())):i.scale()},n.translate=function(t){if(!arguments.length)return i.translate();var l=i.scale(),s=+t[0],f=+t[1];return e=i.translate(t).clipExtent([[s-.455*l,f-.238*l],[s+.455*l,f+.238*l]]).stream(c).point,r=o.translate([s-.307*l,f+.201*l]).clipExtent([[s-.425*l+Ca,f+.12*l+Ca],[s-.214*l-Ca,f+.234*l-Ca]]).stream(c).point,u=a.translate([s-.205*l,f+.212*l]).clipExtent([[s-.214*l+Ca,f+.166*l+Ca],[s-.115*l-Ca,f+.234*l-Ca]]).stream(c).point,n},n.scale(1070)};var Rc,Dc,Pc,Uc,jc,Fc,Hc={point:b,lineStart:b,lineEnd:b,polygonStart:function(){Dc=0,Hc.lineStart=Ve},polygonEnd:function(){Hc.lineStart=Hc.lineEnd=Hc.point=b,Rc+=ga(Dc/2)}},Oc={point:Xe,lineStart:b,lineEnd:b,polygonStart:b,polygonEnd:b},Ic={point:We,lineStart:Je,lineEnd:Ge,polygonStart:function(){Ic.lineStart=Ke},polygonEnd:function(){Ic.point=We,Ic.lineStart=Je,Ic.lineEnd=Ge}};ta.geo.path=function(){function n(n){return n&&("function"==typeof a&&i.pointRadius(+a.apply(this,arguments)),o&&o.valid||(o=u(i)),ta.geo.stream(n,o)),i.result()}function t(){return o=null,n}var e,r,u,i,o,a=4.5;return n.area=function(n){return Rc=0,ta.geo.stream(n,u(Hc)),Rc},n.centroid=function(n){return _c=wc=Sc=kc=Ec=Ac=Nc=Cc=zc=0,ta.geo.stream(n,u(Ic)),zc?[Nc/zc,Cc/zc]:Ac?[kc/Ac,Ec/Ac]:Sc?[_c/Sc,wc/Sc]:[0/0,0/0]},n.bounds=function(n){return jc=Fc=-(Pc=Uc=1/0),ta.geo.stream(n,u(Oc)),[[Pc,Uc],[jc,Fc]]},n.projection=function(n){return arguments.length?(u=(e=n)?n.stream||tr(n):y,t()):e},n.context=function(n){return arguments.length?(i=null==(r=n)?new $e:new Qe(n),"function"!=typeof a&&i.pointRadius(a),t()):r},n.pointRadius=function(t){return arguments.length?(a="function"==typeof t?t:(i.pointRadius(+t),+t),n):a},n.projection(ta.geo.albersUsa()).context(null)},ta.geo.transform=function(n){return{stream:function(t){var e=new er(t);for(var r in n)e[r]=n[r];return e}}},er.prototype={point:function(n,t){this.stream.point(n,t)},sphere:function(){this.stream.sphere()},lineStart:function(){this.stream.lineStart()},lineEnd:function(){this.stream.lineEnd()},polygonStart:function(){this.stream.polygonStart()},polygonEnd:function(){this.stream.polygonEnd()}},ta.geo.projection=ur,ta.geo.projectionMutator=ir,(ta.geo.equirectangular=function(){return ur(ar)}).raw=ar.invert=ar,ta.geo.rotation=function(n){function t(t){return t=n(t[0]*Da,t[1]*Da),t[0]*=Pa,t[1]*=Pa,t}return n=lr(n[0]%360*Da,n[1]*Da,n.length>2?n[2]*Da:0),t.invert=function(t){return t=n.invert(t[0]*Da,t[1]*Da),t[0]*=Pa,t[1]*=Pa,t},t},cr.invert=ar,ta.geo.circle=function(){function n(){var n="function"==typeof r?r.apply(this,arguments):r,t=lr(-n[0]*Da,-n[1]*Da,0).invert,u=[];return e(null,null,1,{point:function(n,e){u.push(n=t(n,e)),n[0]*=Pa,n[1]*=Pa}}),{type:"Polygon",coordinates:[u]}}var t,e,r=[0,0],u=6;return n.origin=function(t){return arguments.length?(r=t,n):r},n.angle=function(r){return arguments.length?(e=gr((t=+r)*Da,u*Da),n):t},n.precision=function(r){return arguments.length?(e=gr(t*Da,(u=+r)*Da),n):u},n.angle(90)},ta.geo.distance=function(n,t){var e,r=(t[0]-n[0])*Da,u=n[1]*Da,i=t[1]*Da,o=Math.sin(r),a=Math.cos(r),c=Math.sin(u),l=Math.cos(u),s=Math.sin(i),f=Math.cos(i);return Math.atan2(Math.sqrt((e=f*o)*e+(e=l*s-c*f*a)*e),c*s+l*f*a)},ta.geo.graticule=function(){function n(){return{type:"MultiLineString",coordinates:t()}}function t(){return ta.range(Math.ceil(i/d)*d,u,d).map(h).concat(ta.range(Math.ceil(l/m)*m,c,m).map(g)).concat(ta.range(Math.ceil(r/p)*p,e,p).filter(function(n){return ga(n%d)>Ca}).map(s)).concat(ta.range(Math.ceil(a/v)*v,o,v).filter(function(n){return ga(n%m)>Ca}).map(f))}var e,r,u,i,o,a,c,l,s,f,h,g,p=10,v=p,d=90,m=360,y=2.5;return n.lines=function(){return t().map(function(n){return{type:"LineString",coordinates:n}})},n.outline=function(){return{type:"Polygon",coordinates:[h(i).concat(g(c).slice(1),h(u).reverse().slice(1),g(l).reverse().slice(1))]}},n.extent=function(t){return arguments.length?n.majorExtent(t).minorExtent(t):n.minorExtent()},n.majorExtent=function(t){return arguments.length?(i=+t[0][0],u=+t[1][0],l=+t[0][1],c=+t[1][1],i>u&&(t=i,i=u,u=t),l>c&&(t=l,l=c,c=t),n.precision(y)):[[i,l],[u,c]]},n.minorExtent=function(t){return arguments.length?(r=+t[0][0],e=+t[1][0],a=+t[0][1],o=+t[1][1],r>e&&(t=r,r=e,e=t),a>o&&(t=a,a=o,o=t),n.precision(y)):[[r,a],[e,o]]},n.step=function(t){return arguments.length?n.majorStep(t).minorStep(t):n.minorStep()},n.majorStep=function(t){return arguments.length?(d=+t[0],m=+t[1],n):[d,m]},n.minorStep=function(t){return arguments.length?(p=+t[0],v=+t[1],n):[p,v]},n.precision=function(t){return arguments.length?(y=+t,s=vr(a,o,90),f=dr(r,e,y),h=vr(l,c,90),g=dr(i,u,y),n):y},n.majorExtent([[-180,-90+Ca],[180,90-Ca]]).minorExtent([[-180,-80-Ca],[180,80+Ca]])},ta.geo.greatArc=function(){function n(){return{type:"LineString",coordinates:[t||r.apply(this,arguments),e||u.apply(this,arguments)]}}var t,e,r=mr,u=yr;return n.distance=function(){return ta.geo.distance(t||r.apply(this,arguments),e||u.apply(this,arguments))},n.source=function(e){return arguments.length?(r=e,t="function"==typeof e?null:e,n):r},n.target=function(t){return arguments.length?(u=t,e="function"==typeof t?null:t,n):u},n.precision=function(){return arguments.length?n:0},n},ta.geo.interpolate=function(n,t){return Mr(n[0]*Da,n[1]*Da,t[0]*Da,t[1]*Da)},ta.geo.length=function(n){return Yc=0,ta.geo.stream(n,Zc),Yc};var Yc,Zc={sphere:b,point:b,lineStart:xr,lineEnd:b,polygonStart:b,polygonEnd:b},Vc=br(function(n){return Math.sqrt(2/(1+n))},function(n){return 2*Math.asin(n/2)});(ta.geo.azimuthalEqualArea=function(){return ur(Vc)}).raw=Vc;var Xc=br(function(n){var t=Math.acos(n);return t&&t/Math.sin(t)},y);(ta.geo.azimuthalEquidistant=function(){return ur(Xc)}).raw=Xc,(ta.geo.conicConformal=function(){return Ye(_r)}).raw=_r,(ta.geo.conicEquidistant=function(){return Ye(wr)}).raw=wr;var $c=br(function(n){return 1/n},Math.atan);(ta.geo.gnomonic=function(){return ur($c)}).raw=$c,Sr.invert=function(n,t){return[n,2*Math.atan(Math.exp(t))-Ra]},(ta.geo.mercator=function(){return kr(Sr)}).raw=Sr;var Bc=br(function(){return 1},Math.asin);(ta.geo.orthographic=function(){return ur(Bc)}).raw=Bc;var Wc=br(function(n){return 1/(1+n)},function(n){return 2*Math.atan(n)});(ta.geo.stereographic=function(){return ur(Wc)}).raw=Wc,Er.invert=function(n,t){return[-t,2*Math.atan(Math.exp(n))-Ra]},(ta.geo.transverseMercator=function(){var n=kr(Er),t=n.center,e=n.rotate;return n.center=function(n){return n?t([-n[1],n[0]]):(n=t(),[n[1],-n[0]])},n.rotate=function(n){return n?e([n[0],n[1],n.length>2?n[2]+90:90]):(n=e(),[n[0],n[1],n[2]-90])},e([0,0,90])}).raw=Er,ta.geom={},ta.geom.hull=function(n){function t(n){if(n.length<3)return[];var t,u=Et(e),i=Et(r),o=n.length,a=[],c=[];for(t=0;o>t;t++)a.push([+u.call(this,n[t],t),+i.call(this,n[t],t),t]);for(a.sort(zr),t=0;o>t;t++)c.push([a[t][0],-a[t][1]]);var l=Cr(a),s=Cr(c),f=s[0]===l[0],h=s[s.length-1]===l[l.length-1],g=[];for(t=l.length-1;t>=0;--t)g.push(n[a[l[t]][2]]);for(t=+f;t<s.length-h;++t)g.push(n[a[s[t]][2]]);return g}var e=Ar,r=Nr;return arguments.length?t(n):(t.x=function(n){return arguments.length?(e=n,t):e},t.y=function(n){return arguments.length?(r=n,t):r},t)},ta.geom.polygon=function(n){return ya(n,Jc),n};var Jc=ta.geom.polygon.prototype=[];Jc.area=function(){for(var n,t=-1,e=this.length,r=this[e-1],u=0;++t<e;)n=r,r=this[t],u+=n[1]*r[0]-n[0]*r[1];return.5*u},Jc.centroid=function(n){var t,e,r=-1,u=this.length,i=0,o=0,a=this[u-1];for(arguments.length||(n=-1/(6*this.area()));++r<u;)t=a,a=this[r],e=t[0]*a[1]-a[0]*t[1],i+=(t[0]+a[0])*e,o+=(t[1]+a[1])*e;return[i*n,o*n]},Jc.clip=function(n){for(var t,e,r,u,i,o,a=Tr(n),c=-1,l=this.length-Tr(this),s=this[l-1];++c<l;){for(t=n.slice(),n.length=0,u=this[c],i=t[(r=t.length-a)-1],e=-1;++e<r;)o=t[e],qr(o,s,u)?(qr(i,s,u)||n.push(Lr(i,o,s,u)),n.push(o)):qr(i,s,u)&&n.push(Lr(i,o,s,u)),i=o;a&&n.push(n[0]),s=u}return n};var Gc,Kc,Qc,nl,tl,el=[],rl=[];Or.prototype.prepare=function(){for(var n,t=this.edges,e=t.length;e--;)n=t[e].edge,n.b&&n.a||t.splice(e,1);return t.sort(Yr),t.length},Qr.prototype={start:function(){return this.edge.l===this.site?this.edge.a:this.edge.b},end:function(){return this.edge.l===this.site?this.edge.b:this.edge.a}},nu.prototype={insert:function(n,t){var e,r,u;if(n){if(t.P=n,t.N=n.N,n.N&&(n.N.P=t),n.N=t,n.R){for(n=n.R;n.L;)n=n.L;n.L=t}else n.R=t;e=n}else this._?(n=uu(this._),t.P=null,t.N=n,n.P=n.L=t,e=n):(t.P=t.N=null,this._=t,e=null);for(t.L=t.R=null,t.U=e,t.C=!0,n=t;e&&e.C;)r=e.U,e===r.L?(u=r.R,u&&u.C?(e.C=u.C=!1,r.C=!0,n=r):(n===e.R&&(eu(this,e),n=e,e=n.U),e.C=!1,r.C=!0,ru(this,r))):(u=r.L,u&&u.C?(e.C=u.C=!1,r.C=!0,n=r):(n===e.L&&(ru(this,e),n=e,e=n.U),e.C=!1,r.C=!0,eu(this,r))),e=n.U;this._.C=!1},remove:function(n){n.N&&(n.N.P=n.P),n.P&&(n.P.N=n.N),n.N=n.P=null;var t,e,r,u=n.U,i=n.L,o=n.R;if(e=i?o?uu(o):i:o,u?u.L===n?u.L=e:u.R=e:this._=e,i&&o?(r=e.C,e.C=n.C,e.L=i,i.U=e,e!==o?(u=e.U,e.U=n.U,n=e.R,u.L=n,e.R=o,o.U=e):(e.U=u,u=e,n=e.R)):(r=n.C,n=e),n&&(n.U=u),!r){if(n&&n.C)return void(n.C=!1);do{if(n===this._)break;if(n===u.L){if(t=u.R,t.C&&(t.C=!1,u.C=!0,eu(this,u),t=u.R),t.L&&t.L.C||t.R&&t.R.C){t.R&&t.R.C||(t.L.C=!1,t.C=!0,ru(this,t),t=u.R),t.C=u.C,u.C=t.R.C=!1,eu(this,u),n=this._;break}}else if(t=u.L,t.C&&(t.C=!1,u.C=!0,ru(this,u),t=u.L),t.L&&t.L.C||t.R&&t.R.C){t.L&&t.L.C||(t.R.C=!1,t.C=!0,eu(this,t),t=u.L),t.C=u.C,u.C=t.L.C=!1,ru(this,u),n=this._;break}t.C=!0,n=u,u=u.U}while(!n.C);n&&(n.C=!1)}}},ta.geom.voronoi=function(n){function t(n){var t=new Array(n.length),r=a[0][0],u=a[0][1],i=a[1][0],o=a[1][1];return iu(e(n),a).cells.forEach(function(e,a){var c=e.edges,l=e.site,s=t[a]=c.length?c.map(function(n){var t=n.start();return[t.x,t.y]}):l.x>=r&&l.x<=i&&l.y>=u&&l.y<=o?[[r,o],[i,o],[i,u],[r,u]]:[];s.point=n[a]}),t}function e(n){return n.map(function(n,t){return{x:Math.round(i(n,t)/Ca)*Ca,y:Math.round(o(n,t)/Ca)*Ca,i:t}})}var r=Ar,u=Nr,i=r,o=u,a=ul;return n?t(n):(t.links=function(n){return iu(e(n)).edges.filter(function(n){return n.l&&n.r}).map(function(t){return{source:n[t.l.i],target:n[t.r.i]}})},t.triangles=function(n){var t=[];return iu(e(n)).cells.forEach(function(e,r){for(var u,i,o=e.site,a=e.edges.sort(Yr),c=-1,l=a.length,s=a[l-1].edge,f=s.l===o?s.r:s.l;++c<l;)u=s,i=f,s=a[c].edge,f=s.l===o?s.r:s.l,r<i.i&&r<f.i&&au(o,i,f)<0&&t.push([n[r],n[i.i],n[f.i]])}),t},t.x=function(n){return arguments.length?(i=Et(r=n),t):r},t.y=function(n){return arguments.length?(o=Et(u=n),t):u},t.clipExtent=function(n){return arguments.length?(a=null==n?ul:n,t):a===ul?null:a},t.size=function(n){return arguments.length?t.clipExtent(n&&[[0,0],n]):a===ul?null:a&&a[1]},t)};var ul=[[-1e6,-1e6],[1e6,1e6]];ta.geom.delaunay=function(n){return ta.geom.voronoi().triangles(n)},ta.geom.quadtree=function(n,t,e,r,u){function i(n){function i(n,t,e,r,u,i,o,a){if(!isNaN(e)&&!isNaN(r))if(n.leaf){var c=n.x,s=n.y;if(null!=c)if(ga(c-e)+ga(s-r)<.01)l(n,t,e,r,u,i,o,a);else{var f=n.point;n.x=n.y=n.point=null,l(n,f,c,s,u,i,o,a),l(n,t,e,r,u,i,o,a)}else n.x=e,n.y=r,n.point=t}else l(n,t,e,r,u,i,o,a)}function l(n,t,e,r,u,o,a,c){var l=.5*(u+a),s=.5*(o+c),f=e>=l,h=r>=s,g=h<<1|f;n.leaf=!1,n=n.nodes[g]||(n.nodes[g]=su()),f?u=l:a=l,h?o=s:c=s,i(n,t,e,r,u,o,a,c)}var s,f,h,g,p,v,d,m,y,M=Et(a),x=Et(c);if(null!=t)v=t,d=e,m=r,y=u;else if(m=y=-(v=d=1/0),f=[],h=[],p=n.length,o)for(g=0;p>g;++g)s=n[g],s.x<v&&(v=s.x),s.y<d&&(d=s.y),s.x>m&&(m=s.x),s.y>y&&(y=s.y),f.push(s.x),h.push(s.y);else for(g=0;p>g;++g){var b=+M(s=n[g],g),_=+x(s,g);v>b&&(v=b),d>_&&(d=_),b>m&&(m=b),_>y&&(y=_),f.push(b),h.push(_)}var w=m-v,S=y-d;w>S?y=d+w:m=v+S;var k=su();if(k.add=function(n){i(k,n,+M(n,++g),+x(n,g),v,d,m,y)},k.visit=function(n){fu(n,k,v,d,m,y)},k.find=function(n){return hu(k,n[0],n[1],v,d,m,y)},g=-1,null==t){for(;++g<p;)i(k,n[g],f[g],h[g],v,d,m,y);--g}else n.forEach(k.add);return f=h=n=s=null,k}var o,a=Ar,c=Nr;return(o=arguments.length)?(a=cu,c=lu,3===o&&(u=e,r=t,e=t=0),i(n)):(i.x=function(n){return arguments.length?(a=n,i):a},i.y=function(n){return arguments.length?(c=n,i):c},i.extent=function(n){return arguments.length?(null==n?t=e=r=u=null:(t=+n[0][0],e=+n[0][1],r=+n[1][0],u=+n[1][1]),i):null==t?null:[[t,e],[r,u]]},i.size=function(n){return arguments.length?(null==n?t=e=r=u=null:(t=e=0,r=+n[0],u=+n[1]),i):null==t?null:[r-t,u-e]},i)},ta.interpolateRgb=gu,ta.interpolateObject=pu,ta.interpolateNumber=vu,ta.interpolateString=du;var il=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,ol=new RegExp(il.source,"g");ta.interpolate=mu,ta.interpolators=[function(n,t){var e=typeof t;return("string"===e?Ga.has(t)||/^(#|rgb\(|hsl\()/.test(t)?gu:du:t instanceof ot?gu:Array.isArray(t)?yu:"object"===e&&isNaN(t)?pu:vu)(n,t)}],ta.interpolateArray=yu;var al=function(){return y},cl=ta.map({linear:al,poly:ku,quad:function(){return _u},cubic:function(){return wu},sin:function(){return Eu},exp:function(){return Au},circle:function(){return Nu},elastic:Cu,back:zu,bounce:function(){return qu}}),ll=ta.map({"in":y,out:xu,"in-out":bu,"out-in":function(n){return bu(xu(n))}});ta.ease=function(n){var t=n.indexOf("-"),e=t>=0?n.slice(0,t):n,r=t>=0?n.slice(t+1):"in";return e=cl.get(e)||al,r=ll.get(r)||y,Mu(r(e.apply(null,ea.call(arguments,1))))},ta.interpolateHcl=Lu,ta.interpolateHsl=Tu,ta.interpolateLab=Ru,ta.interpolateRound=Du,ta.transform=function(n){var t=ua.createElementNS(ta.ns.prefix.svg,"g");return(ta.transform=function(n){if(null!=n){t.setAttribute("transform",n);var e=t.transform.baseVal.consolidate()}return new Pu(e?e.matrix:sl)})(n)},Pu.prototype.toString=function(){return"translate("+this.translate+")rotate("+this.rotate+")skewX("+this.skew+")scale("+this.scale+")"};var sl={a:1,b:0,c:0,d:1,e:0,f:0};ta.interpolateTransform=Hu,ta.layout={},ta.layout.bundle=function(){return function(n){for(var t=[],e=-1,r=n.length;++e<r;)t.push(Yu(n[e]));return t}},ta.layout.chord=function(){function n(){var n,l,f,h,g,p={},v=[],d=ta.range(i),m=[];for(e=[],r=[],n=0,h=-1;++h<i;){for(l=0,g=-1;++g<i;)l+=u[h][g];v.push(l),m.push(ta.range(i)),n+=l}for(o&&d.sort(function(n,t){return o(v[n],v[t])}),a&&m.forEach(function(n,t){n.sort(function(n,e){return a(u[t][n],u[t][e])})}),n=(La-s*i)/n,l=0,h=-1;++h<i;){for(f=l,g=-1;++g<i;){var y=d[h],M=m[y][g],x=u[y][M],b=l,_=l+=x*n;p[y+"-"+M]={index:y,subindex:M,startAngle:b,endAngle:_,value:x}}r[y]={index:y,startAngle:f,endAngle:l,value:(l-f)/n},l+=s}for(h=-1;++h<i;)for(g=h-1;++g<i;){var w=p[h+"-"+g],S=p[g+"-"+h];(w.value||S.value)&&e.push(w.value<S.value?{source:S,target:w}:{source:w,target:S})}c&&t()}function t(){e.sort(function(n,t){return c((n.source.value+n.target.value)/2,(t.source.value+t.target.value)/2)})}var e,r,u,i,o,a,c,l={},s=0;return l.matrix=function(n){return arguments.length?(i=(u=n)&&u.length,e=r=null,l):u},l.padding=function(n){return arguments.length?(s=n,e=r=null,l):s},l.sortGroups=function(n){return arguments.length?(o=n,e=r=null,l):o},l.sortSubgroups=function(n){return arguments.length?(a=n,e=null,l):a},l.sortChords=function(n){return arguments.length?(c=n,e&&t(),l):c},l.chords=function(){return e||n(),e},l.groups=function(){return r||n(),r},l},ta.layout.force=function(){function n(n){return function(t,e,r,u){if(t.point!==n){var i=t.cx-n.x,o=t.cy-n.y,a=u-e,c=i*i+o*o;if(c>a*a/d){if(p>c){var l=t.charge/c;n.px-=i*l,n.py-=o*l}return!0}if(t.point&&c&&p>c){var l=t.pointCharge/c;n.px-=i*l,n.py-=o*l}}return!t.charge}}function t(n){n.px=ta.event.x,n.py=ta.event.y,a.resume()}var e,r,u,i,o,a={},c=ta.dispatch("start","tick","end"),l=[1,1],s=.9,f=fl,h=hl,g=-30,p=gl,v=.1,d=.64,m=[],M=[];return a.tick=function(){if((r*=.99)<.005)return c.end({type:"end",alpha:r=0}),!0;var t,e,a,f,h,p,d,y,x,b=m.length,_=M.length;for(e=0;_>e;++e)a=M[e],f=a.source,h=a.target,y=h.x-f.x,x=h.y-f.y,(p=y*y+x*x)&&(p=r*i[e]*((p=Math.sqrt(p))-u[e])/p,y*=p,x*=p,h.x-=y*(d=f.weight/(h.weight+f.weight)),h.y-=x*d,f.x+=y*(d=1-d),f.y+=x*d);if((d=r*v)&&(y=l[0]/2,x=l[1]/2,e=-1,d))for(;++e<b;)a=m[e],a.x+=(y-a.x)*d,a.y+=(x-a.y)*d;if(g)for(Ju(t=ta.geom.quadtree(m),r,o),e=-1;++e<b;)(a=m[e]).fixed||t.visit(n(a));for(e=-1;++e<b;)a=m[e],a.fixed?(a.x=a.px,a.y=a.py):(a.x-=(a.px-(a.px=a.x))*s,a.y-=(a.py-(a.py=a.y))*s);c.tick({type:"tick",alpha:r})},a.nodes=function(n){return arguments.length?(m=n,a):m},a.links=function(n){return arguments.length?(M=n,a):M},a.size=function(n){return arguments.length?(l=n,a):l},a.linkDistance=function(n){return arguments.length?(f="function"==typeof n?n:+n,a):f},a.distance=a.linkDistance,a.linkStrength=function(n){return arguments.length?(h="function"==typeof n?n:+n,a):h},a.friction=function(n){return arguments.length?(s=+n,a):s},a.charge=function(n){return arguments.length?(g="function"==typeof n?n:+n,a):g},a.chargeDistance=function(n){return arguments.length?(p=n*n,a):Math.sqrt(p)},a.gravity=function(n){return arguments.length?(v=+n,a):v},a.theta=function(n){return arguments.length?(d=n*n,a):Math.sqrt(d)},a.alpha=function(n){return arguments.length?(n=+n,r?r=n>0?n:0:n>0&&(c.start({type:"start",alpha:r=n}),ta.timer(a.tick)),a):r},a.start=function(){function n(n,r){if(!e){for(e=new Array(c),a=0;c>a;++a)e[a]=[];for(a=0;s>a;++a){var u=M[a];e[u.source.index].push(u.target),e[u.target.index].push(u.source)}}for(var i,o=e[t],a=-1,l=o.length;++a<l;)if(!isNaN(i=o[a][n]))return i;return Math.random()*r}var t,e,r,c=m.length,s=M.length,p=l[0],v=l[1];for(t=0;c>t;++t)(r=m[t]).index=t,r.weight=0;for(t=0;s>t;++t)r=M[t],"number"==typeof r.source&&(r.source=m[r.source]),"number"==typeof r.target&&(r.target=m[r.target]),++r.source.weight,++r.target.weight;for(t=0;c>t;++t)r=m[t],isNaN(r.x)&&(r.x=n("x",p)),isNaN(r.y)&&(r.y=n("y",v)),isNaN(r.px)&&(r.px=r.x),isNaN(r.py)&&(r.py=r.y);if(u=[],"function"==typeof f)for(t=0;s>t;++t)u[t]=+f.call(this,M[t],t);else for(t=0;s>t;++t)u[t]=f;if(i=[],"function"==typeof h)for(t=0;s>t;++t)i[t]=+h.call(this,M[t],t);else for(t=0;s>t;++t)i[t]=h;if(o=[],"function"==typeof g)for(t=0;c>t;++t)o[t]=+g.call(this,m[t],t);else for(t=0;c>t;++t)o[t]=g;return a.resume()},a.resume=function(){return a.alpha(.1)},a.stop=function(){return a.alpha(0)},a.drag=function(){return e||(e=ta.behavior.drag().origin(y).on("dragstart.force",Xu).on("drag.force",t).on("dragend.force",$u)),arguments.length?void this.on("mouseover.force",Bu).on("mouseout.force",Wu).call(e):e},ta.rebind(a,c,"on")};var fl=20,hl=1,gl=1/0;ta.layout.hierarchy=function(){function n(u){var i,o=[u],a=[];for(u.depth=0;null!=(i=o.pop());)if(a.push(i),(l=e.call(n,i,i.depth))&&(c=l.length)){for(var c,l,s;--c>=0;)o.push(s=l[c]),s.parent=i,s.depth=i.depth+1;r&&(i.value=0),i.children=l}else r&&(i.value=+r.call(n,i,i.depth)||0),delete i.children;return Qu(u,function(n){var e,u;t&&(e=n.children)&&e.sort(t),r&&(u=n.parent)&&(u.value+=n.value)}),a}var t=ei,e=ni,r=ti;return n.sort=function(e){return arguments.length?(t=e,n):t},n.children=function(t){return arguments.length?(e=t,n):e},n.value=function(t){return arguments.length?(r=t,n):r},n.revalue=function(t){return r&&(Ku(t,function(n){n.children&&(n.value=0)}),Qu(t,function(t){var e;t.children||(t.value=+r.call(n,t,t.depth)||0),(e=t.parent)&&(e.value+=t.value)})),t},n},ta.layout.partition=function(){function n(t,e,r,u){var i=t.children;if(t.x=e,t.y=t.depth*u,t.dx=r,t.dy=u,i&&(o=i.length)){var o,a,c,l=-1;for(r=t.value?r/t.value:0;++l<o;)n(a=i[l],e,c=a.value*r,u),e+=c}}function t(n){var e=n.children,r=0;if(e&&(u=e.length))for(var u,i=-1;++i<u;)r=Math.max(r,t(e[i]));return 1+r}function e(e,i){var o=r.call(this,e,i);return n(o[0],0,u[0],u[1]/t(o[0])),o}var r=ta.layout.hierarchy(),u=[1,1];return e.size=function(n){return arguments.length?(u=n,e):u},Gu(e,r)},ta.layout.pie=function(){function n(o){var a,c=o.length,l=o.map(function(e,r){return+t.call(n,e,r)}),s=+("function"==typeof r?r.apply(this,arguments):r),f=("function"==typeof u?u.apply(this,arguments):u)-s,h=Math.min(Math.abs(f)/c,+("function"==typeof i?i.apply(this,arguments):i)),g=h*(0>f?-1:1),p=(f-c*g)/ta.sum(l),v=ta.range(c),d=[];return null!=e&&v.sort(e===pl?function(n,t){return l[t]-l[n]}:function(n,t){return e(o[n],o[t])}),v.forEach(function(n){d[n]={data:o[n],value:a=l[n],startAngle:s,endAngle:s+=a*p+g,padAngle:h}}),d}var t=Number,e=pl,r=0,u=La,i=0;return n.value=function(e){return arguments.length?(t=e,n):t},n.sort=function(t){return arguments.length?(e=t,n):e},n.startAngle=function(t){return arguments.length?(r=t,n):r},n.endAngle=function(t){return arguments.length?(u=t,n):u},n.padAngle=function(t){return arguments.length?(i=t,n):i},n};var pl={};ta.layout.stack=function(){function n(a,c){if(!(h=a.length))return a;var l=a.map(function(e,r){return t.call(n,e,r)}),s=l.map(function(t){return t.map(function(t,e){return[i.call(n,t,e),o.call(n,t,e)]})}),f=e.call(n,s,c);l=ta.permute(l,f),s=ta.permute(s,f);var h,g,p,v,d=r.call(n,s,c),m=l[0].length;for(p=0;m>p;++p)for(u.call(n,l[0][p],v=d[p],s[0][p][1]),g=1;h>g;++g)u.call(n,l[g][p],v+=s[g-1][p][1],s[g][p][1]);return a}var t=y,e=ai,r=ci,u=oi,i=ui,o=ii;return n.values=function(e){return arguments.length?(t=e,n):t},n.order=function(t){return arguments.length?(e="function"==typeof t?t:vl.get(t)||ai,n):e},n.offset=function(t){return arguments.length?(r="function"==typeof t?t:dl.get(t)||ci,n):r},n.x=function(t){return arguments.length?(i=t,n):i},n.y=function(t){return arguments.length?(o=t,n):o},n.out=function(t){return arguments.length?(u=t,n):u},n};var vl=ta.map({"inside-out":function(n){var t,e,r=n.length,u=n.map(li),i=n.map(si),o=ta.range(r).sort(function(n,t){return u[n]-u[t]}),a=0,c=0,l=[],s=[];for(t=0;r>t;++t)e=o[t],c>a?(a+=i[e],l.push(e)):(c+=i[e],s.push(e));return s.reverse().concat(l)},reverse:function(n){return ta.range(n.length).reverse()},"default":ai}),dl=ta.map({silhouette:function(n){var t,e,r,u=n.length,i=n[0].length,o=[],a=0,c=[];for(e=0;i>e;++e){for(t=0,r=0;u>t;t++)r+=n[t][e][1];r>a&&(a=r),o.push(r)}for(e=0;i>e;++e)c[e]=(a-o[e])/2;return c},wiggle:function(n){var t,e,r,u,i,o,a,c,l,s=n.length,f=n[0],h=f.length,g=[];for(g[0]=c=l=0,e=1;h>e;++e){for(t=0,u=0;s>t;++t)u+=n[t][e][1];for(t=0,i=0,a=f[e][0]-f[e-1][0];s>t;++t){for(r=0,o=(n[t][e][1]-n[t][e-1][1])/(2*a);t>r;++r)o+=(n[r][e][1]-n[r][e-1][1])/a;i+=o*n[t][e][1]}g[e]=c-=u?i/u*a:0,l>c&&(l=c)}for(e=0;h>e;++e)g[e]-=l;return g},expand:function(n){var t,e,r,u=n.length,i=n[0].length,o=1/u,a=[];for(e=0;i>e;++e){for(t=0,r=0;u>t;t++)r+=n[t][e][1];if(r)for(t=0;u>t;t++)n[t][e][1]/=r;else for(t=0;u>t;t++)n[t][e][1]=o}for(e=0;i>e;++e)a[e]=0;return a},zero:ci});ta.layout.histogram=function(){function n(n,i){for(var o,a,c=[],l=n.map(e,this),s=r.call(this,l,i),f=u.call(this,s,l,i),i=-1,h=l.length,g=f.length-1,p=t?1:1/h;++i<g;)o=c[i]=[],o.dx=f[i+1]-(o.x=f[i]),o.y=0;if(g>0)for(i=-1;++i<h;)a=l[i],a>=s[0]&&a<=s[1]&&(o=c[ta.bisect(f,a,1,g)-1],o.y+=p,o.push(n[i]));return c}var t=!0,e=Number,r=pi,u=hi;return n.value=function(t){return arguments.length?(e=t,n):e},n.range=function(t){return arguments.length?(r=Et(t),n):r},n.bins=function(t){return arguments.length?(u="number"==typeof t?function(n){return gi(n,t)}:Et(t),n):u},n.frequency=function(e){return arguments.length?(t=!!e,n):t},n},ta.layout.pack=function(){function n(n,i){var o=e.call(this,n,i),a=o[0],c=u[0],l=u[1],s=null==t?Math.sqrt:"function"==typeof t?t:function(){return t};if(a.x=a.y=0,Qu(a,function(n){n.r=+s(n.value)}),Qu(a,Mi),r){var f=r*(t?1:Math.max(2*a.r/c,2*a.r/l))/2;Qu(a,function(n){n.r+=f}),Qu(a,Mi),Qu(a,function(n){n.r-=f})}return _i(a,c/2,l/2,t?1:1/Math.max(2*a.r/c,2*a.r/l)),o}var t,e=ta.layout.hierarchy().sort(vi),r=0,u=[1,1];return n.size=function(t){return arguments.length?(u=t,n):u},n.radius=function(e){return arguments.length?(t=null==e||"function"==typeof e?e:+e,n):t},n.padding=function(t){return arguments.length?(r=+t,n):r},Gu(n,e)},ta.layout.tree=function(){function n(n,u){var s=o.call(this,n,u),f=s[0],h=t(f);if(Qu(h,e),h.parent.m=-h.z,Ku(h,r),l)Ku(f,i);else{var g=f,p=f,v=f;Ku(f,function(n){n.x<g.x&&(g=n),n.x>p.x&&(p=n),n.depth>v.depth&&(v=n)});var d=a(g,p)/2-g.x,m=c[0]/(p.x+a(p,g)/2+d),y=c[1]/(v.depth||1);Ku(f,function(n){n.x=(n.x+d)*m,n.y=n.depth*y})}return s}function t(n){for(var t,e={A:null,children:[n]},r=[e];null!=(t=r.pop());)for(var u,i=t.children,o=0,a=i.length;a>o;++o)r.push((i[o]=u={_:i[o],parent:t,children:(u=i[o].children)&&u.slice()||[],A:null,a:null,z:0,m:0,c:0,s:0,t:null,i:o}).a=u);return e.children[0]}function e(n){var t=n.children,e=n.parent.children,r=n.i?e[n.i-1]:null;if(t.length){Ni(n);var i=(t[0].z+t[t.length-1].z)/2;r?(n.z=r.z+a(n._,r._),n.m=n.z-i):n.z=i}else r&&(n.z=r.z+a(n._,r._));n.parent.A=u(n,r,n.parent.A||e[0])}function r(n){n._.x=n.z+n.parent.m,n.m+=n.parent.m}function u(n,t,e){if(t){for(var r,u=n,i=n,o=t,c=u.parent.children[0],l=u.m,s=i.m,f=o.m,h=c.m;o=Ei(o),u=ki(u),o&&u;)c=ki(c),i=Ei(i),i.a=n,r=o.z+f-u.z-l+a(o._,u._),r>0&&(Ai(Ci(o,n,e),n,r),l+=r,s+=r),f+=o.m,l+=u.m,h+=c.m,s+=i.m;o&&!Ei(i)&&(i.t=o,i.m+=f-s),u&&!ki(c)&&(c.t=u,c.m+=l-h,e=n)}return e}function i(n){n.x*=c[0],n.y=n.depth*c[1]}var o=ta.layout.hierarchy().sort(null).value(null),a=Si,c=[1,1],l=null;return n.separation=function(t){return arguments.length?(a=t,n):a},n.size=function(t){return arguments.length?(l=null==(c=t)?i:null,n):l?null:c},n.nodeSize=function(t){return arguments.length?(l=null==(c=t)?null:i,n):l?c:null},Gu(n,o)},ta.layout.cluster=function(){function n(n,i){var o,a=t.call(this,n,i),c=a[0],l=0;Qu(c,function(n){var t=n.children;t&&t.length?(n.x=qi(t),n.y=zi(t)):(n.x=o?l+=e(n,o):0,n.y=0,o=n)});var s=Li(c),f=Ti(c),h=s.x-e(s,f)/2,g=f.x+e(f,s)/2;return Qu(c,u?function(n){n.x=(n.x-c.x)*r[0],n.y=(c.y-n.y)*r[1]}:function(n){n.x=(n.x-h)/(g-h)*r[0],n.y=(1-(c.y?n.y/c.y:1))*r[1]}),a}var t=ta.layout.hierarchy().sort(null).value(null),e=Si,r=[1,1],u=!1;return n.separation=function(t){return arguments.length?(e=t,n):e},n.size=function(t){return arguments.length?(u=null==(r=t),n):u?null:r},n.nodeSize=function(t){return arguments.length?(u=null!=(r=t),n):u?r:null},Gu(n,t)},ta.layout.treemap=function(){function n(n,t){for(var e,r,u=-1,i=n.length;++u<i;)r=(e=n[u]).value*(0>t?0:t),e.area=isNaN(r)||0>=r?0:r}function t(e){var i=e.children;if(i&&i.length){var o,a,c,l=f(e),s=[],h=i.slice(),p=1/0,v="slice"===g?l.dx:"dice"===g?l.dy:"slice-dice"===g?1&e.depth?l.dy:l.dx:Math.min(l.dx,l.dy);for(n(h,l.dx*l.dy/e.value),s.area=0;(c=h.length)>0;)s.push(o=h[c-1]),s.area+=o.area,"squarify"!==g||(a=r(s,v))<=p?(h.pop(),p=a):(s.area-=s.pop().area,u(s,v,l,!1),v=Math.min(l.dx,l.dy),s.length=s.area=0,p=1/0);s.length&&(u(s,v,l,!0),s.length=s.area=0),i.forEach(t)}}function e(t){var r=t.children;if(r&&r.length){var i,o=f(t),a=r.slice(),c=[];for(n(a,o.dx*o.dy/t.value),c.area=0;i=a.pop();)c.push(i),c.area+=i.area,null!=i.z&&(u(c,i.z?o.dx:o.dy,o,!a.length),c.length=c.area=0);r.forEach(e)}}function r(n,t){for(var e,r=n.area,u=0,i=1/0,o=-1,a=n.length;++o<a;)(e=n[o].area)&&(i>e&&(i=e),e>u&&(u=e));return r*=r,t*=t,r?Math.max(t*u*p/r,r/(t*i*p)):1/0}function u(n,t,e,r){var u,i=-1,o=n.length,a=e.x,l=e.y,s=t?c(n.area/t):0;if(t==e.dx){for((r||s>e.dy)&&(s=e.dy);++i<o;)u=n[i],u.x=a,u.y=l,u.dy=s,a+=u.dx=Math.min(e.x+e.dx-a,s?c(u.area/s):0);u.z=!0,u.dx+=e.x+e.dx-a,e.y+=s,e.dy-=s}else{for((r||s>e.dx)&&(s=e.dx);++i<o;)u=n[i],u.x=a,u.y=l,u.dx=s,l+=u.dy=Math.min(e.y+e.dy-l,s?c(u.area/s):0);u.z=!1,u.dy+=e.y+e.dy-l,e.x+=s,e.dx-=s}}function i(r){var u=o||a(r),i=u[0];return i.x=0,i.y=0,i.dx=l[0],i.dy=l[1],o&&a.revalue(i),n([i],i.dx*i.dy/i.value),(o?e:t)(i),h&&(o=u),u}var o,a=ta.layout.hierarchy(),c=Math.round,l=[1,1],s=null,f=Ri,h=!1,g="squarify",p=.5*(1+Math.sqrt(5));
return i.size=function(n){return arguments.length?(l=n,i):l},i.padding=function(n){function t(t){var e=n.call(i,t,t.depth);return null==e?Ri(t):Di(t,"number"==typeof e?[e,e,e,e]:e)}function e(t){return Di(t,n)}if(!arguments.length)return s;var r;return f=null==(s=n)?Ri:"function"==(r=typeof n)?t:"number"===r?(n=[n,n,n,n],e):e,i},i.round=function(n){return arguments.length?(c=n?Math.round:Number,i):c!=Number},i.sticky=function(n){return arguments.length?(h=n,o=null,i):h},i.ratio=function(n){return arguments.length?(p=n,i):p},i.mode=function(n){return arguments.length?(g=n+"",i):g},Gu(i,a)},ta.random={normal:function(n,t){var e=arguments.length;return 2>e&&(t=1),1>e&&(n=0),function(){var e,r,u;do e=2*Math.random()-1,r=2*Math.random()-1,u=e*e+r*r;while(!u||u>1);return n+t*e*Math.sqrt(-2*Math.log(u)/u)}},logNormal:function(){var n=ta.random.normal.apply(ta,arguments);return function(){return Math.exp(n())}},bates:function(n){var t=ta.random.irwinHall(n);return function(){return t()/n}},irwinHall:function(n){return function(){for(var t=0,e=0;n>e;e++)t+=Math.random();return t}}},ta.scale={};var ml={floor:y,ceil:y};ta.scale.linear=function(){return Ii([0,1],[0,1],mu,!1)};var yl={s:1,g:1,p:1,r:1,e:1};ta.scale.log=function(){return Ji(ta.scale.linear().domain([0,1]),10,!0,[1,10])};var Ml=ta.format(".0e"),xl={floor:function(n){return-Math.ceil(-n)},ceil:function(n){return-Math.floor(-n)}};ta.scale.pow=function(){return Gi(ta.scale.linear(),1,[0,1])},ta.scale.sqrt=function(){return ta.scale.pow().exponent(.5)},ta.scale.ordinal=function(){return Qi([],{t:"range",a:[[]]})},ta.scale.category10=function(){return ta.scale.ordinal().range(bl)},ta.scale.category20=function(){return ta.scale.ordinal().range(_l)},ta.scale.category20b=function(){return ta.scale.ordinal().range(wl)},ta.scale.category20c=function(){return ta.scale.ordinal().range(Sl)};var bl=[2062260,16744206,2924588,14034728,9725885,9197131,14907330,8355711,12369186,1556175].map(Mt),_l=[2062260,11454440,16744206,16759672,2924588,10018698,14034728,16750742,9725885,12955861,9197131,12885140,14907330,16234194,8355711,13092807,12369186,14408589,1556175,10410725].map(Mt),wl=[3750777,5395619,7040719,10264286,6519097,9216594,11915115,13556636,9202993,12426809,15186514,15190932,8666169,11356490,14049643,15177372,8077683,10834324,13528509,14589654].map(Mt),Sl=[3244733,7057110,10406625,13032431,15095053,16616764,16625259,16634018,3253076,7652470,10607003,13101504,7695281,10394312,12369372,14342891,6513507,9868950,12434877,14277081].map(Mt);ta.scale.quantile=function(){return no([],[])},ta.scale.quantize=function(){return to(0,1,[0,1])},ta.scale.threshold=function(){return eo([.5],[0,1])},ta.scale.identity=function(){return ro([0,1])},ta.svg={},ta.svg.arc=function(){function n(){var n=Math.max(0,+e.apply(this,arguments)),l=Math.max(0,+r.apply(this,arguments)),s=o.apply(this,arguments)-Ra,f=a.apply(this,arguments)-Ra,h=Math.abs(f-s),g=s>f?0:1;if(n>l&&(p=l,l=n,n=p),h>=Ta)return t(l,g)+(n?t(n,1-g):"")+"Z";var p,v,d,m,y,M,x,b,_,w,S,k,E=0,A=0,N=[];if((m=(+c.apply(this,arguments)||0)/2)&&(d=i===kl?Math.sqrt(n*n+l*l):+i.apply(this,arguments),g||(A*=-1),l&&(A=tt(d/l*Math.sin(m))),n&&(E=tt(d/n*Math.sin(m)))),l){y=l*Math.cos(s+A),M=l*Math.sin(s+A),x=l*Math.cos(f-A),b=l*Math.sin(f-A);var C=Math.abs(f-s-2*A)<=qa?0:1;if(A&&so(y,M,x,b)===g^C){var z=(s+f)/2;y=l*Math.cos(z),M=l*Math.sin(z),x=b=null}}else y=M=0;if(n){_=n*Math.cos(f-E),w=n*Math.sin(f-E),S=n*Math.cos(s+E),k=n*Math.sin(s+E);var q=Math.abs(s-f+2*E)<=qa?0:1;if(E&&so(_,w,S,k)===1-g^q){var L=(s+f)/2;_=n*Math.cos(L),w=n*Math.sin(L),S=k=null}}else _=w=0;if((p=Math.min(Math.abs(l-n)/2,+u.apply(this,arguments)))>.001){v=l>n^g?0:1;var T=null==S?[_,w]:null==x?[y,M]:Lr([y,M],[S,k],[x,b],[_,w]),R=y-T[0],D=M-T[1],P=x-T[0],U=b-T[1],j=1/Math.sin(Math.acos((R*P+D*U)/(Math.sqrt(R*R+D*D)*Math.sqrt(P*P+U*U)))/2),F=Math.sqrt(T[0]*T[0]+T[1]*T[1]);if(null!=x){var H=Math.min(p,(l-F)/(j+1)),O=fo(null==S?[_,w]:[S,k],[y,M],l,H,g),I=fo([x,b],[_,w],l,H,g);p===H?N.push("M",O[0],"A",H,",",H," 0 0,",v," ",O[1],"A",l,",",l," 0 ",1-g^so(O[1][0],O[1][1],I[1][0],I[1][1]),",",g," ",I[1],"A",H,",",H," 0 0,",v," ",I[0]):N.push("M",O[0],"A",H,",",H," 0 1,",v," ",I[0])}else N.push("M",y,",",M);if(null!=S){var Y=Math.min(p,(n-F)/(j-1)),Z=fo([y,M],[S,k],n,-Y,g),V=fo([_,w],null==x?[y,M]:[x,b],n,-Y,g);p===Y?N.push("L",V[0],"A",Y,",",Y," 0 0,",v," ",V[1],"A",n,",",n," 0 ",g^so(V[1][0],V[1][1],Z[1][0],Z[1][1]),",",1-g," ",Z[1],"A",Y,",",Y," 0 0,",v," ",Z[0]):N.push("L",V[0],"A",Y,",",Y," 0 0,",v," ",Z[0])}else N.push("L",_,",",w)}else N.push("M",y,",",M),null!=x&&N.push("A",l,",",l," 0 ",C,",",g," ",x,",",b),N.push("L",_,",",w),null!=S&&N.push("A",n,",",n," 0 ",q,",",1-g," ",S,",",k);return N.push("Z"),N.join("")}function t(n,t){return"M0,"+n+"A"+n+","+n+" 0 1,"+t+" 0,"+-n+"A"+n+","+n+" 0 1,"+t+" 0,"+n}var e=io,r=oo,u=uo,i=kl,o=ao,a=co,c=lo;return n.innerRadius=function(t){return arguments.length?(e=Et(t),n):e},n.outerRadius=function(t){return arguments.length?(r=Et(t),n):r},n.cornerRadius=function(t){return arguments.length?(u=Et(t),n):u},n.padRadius=function(t){return arguments.length?(i=t==kl?kl:Et(t),n):i},n.startAngle=function(t){return arguments.length?(o=Et(t),n):o},n.endAngle=function(t){return arguments.length?(a=Et(t),n):a},n.padAngle=function(t){return arguments.length?(c=Et(t),n):c},n.centroid=function(){var n=(+e.apply(this,arguments)+ +r.apply(this,arguments))/2,t=(+o.apply(this,arguments)+ +a.apply(this,arguments))/2-Ra;return[Math.cos(t)*n,Math.sin(t)*n]},n};var kl="auto";ta.svg.line=function(){return ho(y)};var El=ta.map({linear:go,"linear-closed":po,step:vo,"step-before":mo,"step-after":yo,basis:So,"basis-open":ko,"basis-closed":Eo,bundle:Ao,cardinal:bo,"cardinal-open":Mo,"cardinal-closed":xo,monotone:To});El.forEach(function(n,t){t.key=n,t.closed=/-closed$/.test(n)});var Al=[0,2/3,1/3,0],Nl=[0,1/3,2/3,0],Cl=[0,1/6,2/3,1/6];ta.svg.line.radial=function(){var n=ho(Ro);return n.radius=n.x,delete n.x,n.angle=n.y,delete n.y,n},mo.reverse=yo,yo.reverse=mo,ta.svg.area=function(){return Do(y)},ta.svg.area.radial=function(){var n=Do(Ro);return n.radius=n.x,delete n.x,n.innerRadius=n.x0,delete n.x0,n.outerRadius=n.x1,delete n.x1,n.angle=n.y,delete n.y,n.startAngle=n.y0,delete n.y0,n.endAngle=n.y1,delete n.y1,n},ta.svg.chord=function(){function n(n,a){var c=t(this,i,n,a),l=t(this,o,n,a);return"M"+c.p0+r(c.r,c.p1,c.a1-c.a0)+(e(c,l)?u(c.r,c.p1,c.r,c.p0):u(c.r,c.p1,l.r,l.p0)+r(l.r,l.p1,l.a1-l.a0)+u(l.r,l.p1,c.r,c.p0))+"Z"}function t(n,t,e,r){var u=t.call(n,e,r),i=a.call(n,u,r),o=c.call(n,u,r)-Ra,s=l.call(n,u,r)-Ra;return{r:i,a0:o,a1:s,p0:[i*Math.cos(o),i*Math.sin(o)],p1:[i*Math.cos(s),i*Math.sin(s)]}}function e(n,t){return n.a0==t.a0&&n.a1==t.a1}function r(n,t,e){return"A"+n+","+n+" 0 "+ +(e>qa)+",1 "+t}function u(n,t,e,r){return"Q 0,0 "+r}var i=mr,o=yr,a=Po,c=ao,l=co;return n.radius=function(t){return arguments.length?(a=Et(t),n):a},n.source=function(t){return arguments.length?(i=Et(t),n):i},n.target=function(t){return arguments.length?(o=Et(t),n):o},n.startAngle=function(t){return arguments.length?(c=Et(t),n):c},n.endAngle=function(t){return arguments.length?(l=Et(t),n):l},n},ta.svg.diagonal=function(){function n(n,u){var i=t.call(this,n,u),o=e.call(this,n,u),a=(i.y+o.y)/2,c=[i,{x:i.x,y:a},{x:o.x,y:a},o];return c=c.map(r),"M"+c[0]+"C"+c[1]+" "+c[2]+" "+c[3]}var t=mr,e=yr,r=Uo;return n.source=function(e){return arguments.length?(t=Et(e),n):t},n.target=function(t){return arguments.length?(e=Et(t),n):e},n.projection=function(t){return arguments.length?(r=t,n):r},n},ta.svg.diagonal.radial=function(){var n=ta.svg.diagonal(),t=Uo,e=n.projection;return n.projection=function(n){return arguments.length?e(jo(t=n)):t},n},ta.svg.symbol=function(){function n(n,r){return(zl.get(t.call(this,n,r))||Oo)(e.call(this,n,r))}var t=Ho,e=Fo;return n.type=function(e){return arguments.length?(t=Et(e),n):t},n.size=function(t){return arguments.length?(e=Et(t),n):e},n};var zl=ta.map({circle:Oo,cross:function(n){var t=Math.sqrt(n/5)/2;return"M"+-3*t+","+-t+"H"+-t+"V"+-3*t+"H"+t+"V"+-t+"H"+3*t+"V"+t+"H"+t+"V"+3*t+"H"+-t+"V"+t+"H"+-3*t+"Z"},diamond:function(n){var t=Math.sqrt(n/(2*Ll)),e=t*Ll;return"M0,"+-t+"L"+e+",0 0,"+t+" "+-e+",0Z"},square:function(n){var t=Math.sqrt(n)/2;return"M"+-t+","+-t+"L"+t+","+-t+" "+t+","+t+" "+-t+","+t+"Z"},"triangle-down":function(n){var t=Math.sqrt(n/ql),e=t*ql/2;return"M0,"+e+"L"+t+","+-e+" "+-t+","+-e+"Z"},"triangle-up":function(n){var t=Math.sqrt(n/ql),e=t*ql/2;return"M0,"+-e+"L"+t+","+e+" "+-t+","+e+"Z"}});ta.svg.symbolTypes=zl.keys();var ql=Math.sqrt(3),Ll=Math.tan(30*Da);_a.transition=function(n){for(var t,e,r=Tl||++Ul,u=Xo(n),i=[],o=Rl||{time:Date.now(),ease:Su,delay:0,duration:250},a=-1,c=this.length;++a<c;){i.push(t=[]);for(var l=this[a],s=-1,f=l.length;++s<f;)(e=l[s])&&$o(e,s,u,r,o),t.push(e)}return Yo(i,u,r)},_a.interrupt=function(n){return this.each(null==n?Dl:Io(Xo(n)))};var Tl,Rl,Dl=Io(Xo()),Pl=[],Ul=0;Pl.call=_a.call,Pl.empty=_a.empty,Pl.node=_a.node,Pl.size=_a.size,ta.transition=function(n,t){return n&&n.transition?Tl?n.transition(t):n:ta.selection().transition(n)},ta.transition.prototype=Pl,Pl.select=function(n){var t,e,r,u=this.id,i=this.namespace,o=[];n=N(n);for(var a=-1,c=this.length;++a<c;){o.push(t=[]);for(var l=this[a],s=-1,f=l.length;++s<f;)(r=l[s])&&(e=n.call(r,r.__data__,s,a))?("__data__"in r&&(e.__data__=r.__data__),$o(e,s,i,u,r[i][u]),t.push(e)):t.push(null)}return Yo(o,i,u)},Pl.selectAll=function(n){var t,e,r,u,i,o=this.id,a=this.namespace,c=[];n=C(n);for(var l=-1,s=this.length;++l<s;)for(var f=this[l],h=-1,g=f.length;++h<g;)if(r=f[h]){i=r[a][o],e=n.call(r,r.__data__,h,l),c.push(t=[]);for(var p=-1,v=e.length;++p<v;)(u=e[p])&&$o(u,p,a,o,i),t.push(u)}return Yo(c,a,o)},Pl.filter=function(n){var t,e,r,u=[];"function"!=typeof n&&(n=O(n));for(var i=0,o=this.length;o>i;i++){u.push(t=[]);for(var e=this[i],a=0,c=e.length;c>a;a++)(r=e[a])&&n.call(r,r.__data__,a,i)&&t.push(r)}return Yo(u,this.namespace,this.id)},Pl.tween=function(n,t){var e=this.id,r=this.namespace;return arguments.length<2?this.node()[r][e].tween.get(n):Y(this,null==t?function(t){t[r][e].tween.remove(n)}:function(u){u[r][e].tween.set(n,t)})},Pl.attr=function(n,t){function e(){this.removeAttribute(a)}function r(){this.removeAttributeNS(a.space,a.local)}function u(n){return null==n?e:(n+="",function(){var t,e=this.getAttribute(a);return e!==n&&(t=o(e,n),function(n){this.setAttribute(a,t(n))})})}function i(n){return null==n?r:(n+="",function(){var t,e=this.getAttributeNS(a.space,a.local);return e!==n&&(t=o(e,n),function(n){this.setAttributeNS(a.space,a.local,t(n))})})}if(arguments.length<2){for(t in n)this.attr(t,n[t]);return this}var o="transform"==n?Hu:mu,a=ta.ns.qualify(n);return Zo(this,"attr."+n,t,a.local?i:u)},Pl.attrTween=function(n,t){function e(n,e){var r=t.call(this,n,e,this.getAttribute(u));return r&&function(n){this.setAttribute(u,r(n))}}function r(n,e){var r=t.call(this,n,e,this.getAttributeNS(u.space,u.local));return r&&function(n){this.setAttributeNS(u.space,u.local,r(n))}}var u=ta.ns.qualify(n);return this.tween("attr."+n,u.local?r:e)},Pl.style=function(n,e,r){function u(){this.style.removeProperty(n)}function i(e){return null==e?u:(e+="",function(){var u,i=t(this).getComputedStyle(this,null).getPropertyValue(n);return i!==e&&(u=mu(i,e),function(t){this.style.setProperty(n,u(t),r)})})}var o=arguments.length;if(3>o){if("string"!=typeof n){2>o&&(e="");for(r in n)this.style(r,n[r],e);return this}r=""}return Zo(this,"style."+n,e,i)},Pl.styleTween=function(n,e,r){function u(u,i){var o=e.call(this,u,i,t(this).getComputedStyle(this,null).getPropertyValue(n));return o&&function(t){this.style.setProperty(n,o(t),r)}}return arguments.length<3&&(r=""),this.tween("style."+n,u)},Pl.text=function(n){return Zo(this,"text",n,Vo)},Pl.remove=function(){var n=this.namespace;return this.each("end.transition",function(){var t;this[n].count<2&&(t=this.parentNode)&&t.removeChild(this)})},Pl.ease=function(n){var t=this.id,e=this.namespace;return arguments.length<1?this.node()[e][t].ease:("function"!=typeof n&&(n=ta.ease.apply(ta,arguments)),Y(this,function(r){r[e][t].ease=n}))},Pl.delay=function(n){var t=this.id,e=this.namespace;return arguments.length<1?this.node()[e][t].delay:Y(this,"function"==typeof n?function(r,u,i){r[e][t].delay=+n.call(r,r.__data__,u,i)}:(n=+n,function(r){r[e][t].delay=n}))},Pl.duration=function(n){var t=this.id,e=this.namespace;return arguments.length<1?this.node()[e][t].duration:Y(this,"function"==typeof n?function(r,u,i){r[e][t].duration=Math.max(1,n.call(r,r.__data__,u,i))}:(n=Math.max(1,n),function(r){r[e][t].duration=n}))},Pl.each=function(n,t){var e=this.id,r=this.namespace;if(arguments.length<2){var u=Rl,i=Tl;try{Tl=e,Y(this,function(t,u,i){Rl=t[r][e],n.call(t,t.__data__,u,i)})}finally{Rl=u,Tl=i}}else Y(this,function(u){var i=u[r][e];(i.event||(i.event=ta.dispatch("start","end","interrupt"))).on(n,t)});return this},Pl.transition=function(){for(var n,t,e,r,u=this.id,i=++Ul,o=this.namespace,a=[],c=0,l=this.length;l>c;c++){a.push(n=[]);for(var t=this[c],s=0,f=t.length;f>s;s++)(e=t[s])&&(r=e[o][u],$o(e,s,o,i,{time:r.time,ease:r.ease,delay:r.delay+r.duration,duration:r.duration})),n.push(e)}return Yo(a,o,i)},ta.svg.axis=function(){function n(n){n.each(function(){var n,l=ta.select(this),s=this.__chart__||e,f=this.__chart__=e.copy(),h=null==c?f.ticks?f.ticks.apply(f,a):f.domain():c,g=null==t?f.tickFormat?f.tickFormat.apply(f,a):y:t,p=l.selectAll(".tick").data(h,f),v=p.enter().insert("g",".domain").attr("class","tick").style("opacity",Ca),d=ta.transition(p.exit()).style("opacity",Ca).remove(),m=ta.transition(p.order()).style("opacity",1),M=Math.max(u,0)+o,x=Ui(f),b=l.selectAll(".domain").data([0]),_=(b.enter().append("path").attr("class","domain"),ta.transition(b));v.append("line"),v.append("text");var w,S,k,E,A=v.select("line"),N=m.select("line"),C=p.select("text").text(g),z=v.select("text"),q=m.select("text"),L="top"===r||"left"===r?-1:1;if("bottom"===r||"top"===r?(n=Bo,w="x",k="y",S="x2",E="y2",C.attr("dy",0>L?"0em":".71em").style("text-anchor","middle"),_.attr("d","M"+x[0]+","+L*i+"V0H"+x[1]+"V"+L*i)):(n=Wo,w="y",k="x",S="y2",E="x2",C.attr("dy",".32em").style("text-anchor",0>L?"end":"start"),_.attr("d","M"+L*i+","+x[0]+"H0V"+x[1]+"H"+L*i)),A.attr(E,L*u),z.attr(k,L*M),N.attr(S,0).attr(E,L*u),q.attr(w,0).attr(k,L*M),f.rangeBand){var T=f,R=T.rangeBand()/2;s=f=function(n){return T(n)+R}}else s.rangeBand?s=f:d.call(n,f,s);v.call(n,s,f),m.call(n,f,f)})}var t,e=ta.scale.linear(),r=jl,u=6,i=6,o=3,a=[10],c=null;return n.scale=function(t){return arguments.length?(e=t,n):e},n.orient=function(t){return arguments.length?(r=t in Fl?t+"":jl,n):r},n.ticks=function(){return arguments.length?(a=arguments,n):a},n.tickValues=function(t){return arguments.length?(c=t,n):c},n.tickFormat=function(e){return arguments.length?(t=e,n):t},n.tickSize=function(t){var e=arguments.length;return e?(u=+t,i=+arguments[e-1],n):u},n.innerTickSize=function(t){return arguments.length?(u=+t,n):u},n.outerTickSize=function(t){return arguments.length?(i=+t,n):i},n.tickPadding=function(t){return arguments.length?(o=+t,n):o},n.tickSubdivide=function(){return arguments.length&&n},n};var jl="bottom",Fl={top:1,right:1,bottom:1,left:1};ta.svg.brush=function(){function n(t){t.each(function(){var t=ta.select(this).style("pointer-events","all").style("-webkit-tap-highlight-color","rgba(0,0,0,0)").on("mousedown.brush",i).on("touchstart.brush",i),o=t.selectAll(".background").data([0]);o.enter().append("rect").attr("class","background").style("visibility","hidden").style("cursor","crosshair"),t.selectAll(".extent").data([0]).enter().append("rect").attr("class","extent").style("cursor","move");var a=t.selectAll(".resize").data(v,y);a.exit().remove(),a.enter().append("g").attr("class",function(n){return"resize "+n}).style("cursor",function(n){return Hl[n]}).append("rect").attr("x",function(n){return/[ew]$/.test(n)?-3:null}).attr("y",function(n){return/^[ns]/.test(n)?-3:null}).attr("width",6).attr("height",6).style("visibility","hidden"),a.style("display",n.empty()?"none":null);var c,f=ta.transition(t),h=ta.transition(o);l&&(c=Ui(l),h.attr("x",c[0]).attr("width",c[1]-c[0]),r(f)),s&&(c=Ui(s),h.attr("y",c[0]).attr("height",c[1]-c[0]),u(f)),e(f)})}function e(n){n.selectAll(".resize").attr("transform",function(n){return"translate("+f[+/e$/.test(n)]+","+h[+/^s/.test(n)]+")"})}function r(n){n.select(".extent").attr("x",f[0]),n.selectAll(".extent,.n>rect,.s>rect").attr("width",f[1]-f[0])}function u(n){n.select(".extent").attr("y",h[0]),n.selectAll(".extent,.e>rect,.w>rect").attr("height",h[1]-h[0])}function i(){function i(){32==ta.event.keyCode&&(C||(M=null,q[0]-=f[1],q[1]-=h[1],C=2),S())}function v(){32==ta.event.keyCode&&2==C&&(q[0]+=f[1],q[1]+=h[1],C=0,S())}function d(){var n=ta.mouse(b),t=!1;x&&(n[0]+=x[0],n[1]+=x[1]),C||(ta.event.altKey?(M||(M=[(f[0]+f[1])/2,(h[0]+h[1])/2]),q[0]=f[+(n[0]<M[0])],q[1]=h[+(n[1]<M[1])]):M=null),A&&m(n,l,0)&&(r(k),t=!0),N&&m(n,s,1)&&(u(k),t=!0),t&&(e(k),w({type:"brush",mode:C?"move":"resize"}))}function m(n,t,e){var r,u,i=Ui(t),c=i[0],l=i[1],s=q[e],v=e?h:f,d=v[1]-v[0];return C&&(c-=s,l-=d+s),r=(e?p:g)?Math.max(c,Math.min(l,n[e])):n[e],C?u=(r+=s)+d:(M&&(s=Math.max(c,Math.min(l,2*M[e]-r))),r>s?(u=r,r=s):u=s),v[0]!=r||v[1]!=u?(e?a=null:o=null,v[0]=r,v[1]=u,!0):void 0}function y(){d(),k.style("pointer-events","all").selectAll(".resize").style("display",n.empty()?"none":null),ta.select("body").style("cursor",null),L.on("mousemove.brush",null).on("mouseup.brush",null).on("touchmove.brush",null).on("touchend.brush",null).on("keydown.brush",null).on("keyup.brush",null),z(),w({type:"brushend"})}var M,x,b=this,_=ta.select(ta.event.target),w=c.of(b,arguments),k=ta.select(b),E=_.datum(),A=!/^(n|s)$/.test(E)&&l,N=!/^(e|w)$/.test(E)&&s,C=_.classed("extent"),z=W(b),q=ta.mouse(b),L=ta.select(t(b)).on("keydown.brush",i).on("keyup.brush",v);if(ta.event.changedTouches?L.on("touchmove.brush",d).on("touchend.brush",y):L.on("mousemove.brush",d).on("mouseup.brush",y),k.interrupt().selectAll("*").interrupt(),C)q[0]=f[0]-q[0],q[1]=h[0]-q[1];else if(E){var T=+/w$/.test(E),R=+/^n/.test(E);x=[f[1-T]-q[0],h[1-R]-q[1]],q[0]=f[T],q[1]=h[R]}else ta.event.altKey&&(M=q.slice());k.style("pointer-events","none").selectAll(".resize").style("display",null),ta.select("body").style("cursor",_.style("cursor")),w({type:"brushstart"}),d()}var o,a,c=E(n,"brushstart","brush","brushend"),l=null,s=null,f=[0,0],h=[0,0],g=!0,p=!0,v=Ol[0];return n.event=function(n){n.each(function(){var n=c.of(this,arguments),t={x:f,y:h,i:o,j:a},e=this.__chart__||t;this.__chart__=t,Tl?ta.select(this).transition().each("start.brush",function(){o=e.i,a=e.j,f=e.x,h=e.y,n({type:"brushstart"})}).tween("brush:brush",function(){var e=yu(f,t.x),r=yu(h,t.y);return o=a=null,function(u){f=t.x=e(u),h=t.y=r(u),n({type:"brush",mode:"resize"})}}).each("end.brush",function(){o=t.i,a=t.j,n({type:"brush",mode:"resize"}),n({type:"brushend"})}):(n({type:"brushstart"}),n({type:"brush",mode:"resize"}),n({type:"brushend"}))})},n.x=function(t){return arguments.length?(l=t,v=Ol[!l<<1|!s],n):l},n.y=function(t){return arguments.length?(s=t,v=Ol[!l<<1|!s],n):s},n.clamp=function(t){return arguments.length?(l&&s?(g=!!t[0],p=!!t[1]):l?g=!!t:s&&(p=!!t),n):l&&s?[g,p]:l?g:s?p:null},n.extent=function(t){var e,r,u,i,c;return arguments.length?(l&&(e=t[0],r=t[1],s&&(e=e[0],r=r[0]),o=[e,r],l.invert&&(e=l(e),r=l(r)),e>r&&(c=e,e=r,r=c),(e!=f[0]||r!=f[1])&&(f=[e,r])),s&&(u=t[0],i=t[1],l&&(u=u[1],i=i[1]),a=[u,i],s.invert&&(u=s(u),i=s(i)),u>i&&(c=u,u=i,i=c),(u!=h[0]||i!=h[1])&&(h=[u,i])),n):(l&&(o?(e=o[0],r=o[1]):(e=f[0],r=f[1],l.invert&&(e=l.invert(e),r=l.invert(r)),e>r&&(c=e,e=r,r=c))),s&&(a?(u=a[0],i=a[1]):(u=h[0],i=h[1],s.invert&&(u=s.invert(u),i=s.invert(i)),u>i&&(c=u,u=i,i=c))),l&&s?[[e,u],[r,i]]:l?[e,r]:s&&[u,i])},n.clear=function(){return n.empty()||(f=[0,0],h=[0,0],o=a=null),n},n.empty=function(){return!!l&&f[0]==f[1]||!!s&&h[0]==h[1]},ta.rebind(n,c,"on")};var Hl={n:"ns-resize",e:"ew-resize",s:"ns-resize",w:"ew-resize",nw:"nwse-resize",ne:"nesw-resize",se:"nwse-resize",sw:"nesw-resize"},Ol=[["n","e","s","w","nw","ne","se","sw"],["e","w"],["n","s"],[]],Il=ac.format=gc.timeFormat,Yl=Il.utc,Zl=Yl("%Y-%m-%dT%H:%M:%S.%LZ");Il.iso=Date.prototype.toISOString&&+new Date("2000-01-01T00:00:00.000Z")?Jo:Zl,Jo.parse=function(n){var t=new Date(n);return isNaN(t)?null:t},Jo.toString=Zl.toString,ac.second=Ft(function(n){return new cc(1e3*Math.floor(n/1e3))},function(n,t){n.setTime(n.getTime()+1e3*Math.floor(t))},function(n){return n.getSeconds()}),ac.seconds=ac.second.range,ac.seconds.utc=ac.second.utc.range,ac.minute=Ft(function(n){return new cc(6e4*Math.floor(n/6e4))},function(n,t){n.setTime(n.getTime()+6e4*Math.floor(t))},function(n){return n.getMinutes()}),ac.minutes=ac.minute.range,ac.minutes.utc=ac.minute.utc.range,ac.hour=Ft(function(n){var t=n.getTimezoneOffset()/60;return new cc(36e5*(Math.floor(n/36e5-t)+t))},function(n,t){n.setTime(n.getTime()+36e5*Math.floor(t))},function(n){return n.getHours()}),ac.hours=ac.hour.range,ac.hours.utc=ac.hour.utc.range,ac.month=Ft(function(n){return n=ac.day(n),n.setDate(1),n},function(n,t){n.setMonth(n.getMonth()+t)},function(n){return n.getMonth()}),ac.months=ac.month.range,ac.months.utc=ac.month.utc.range;var Vl=[1e3,5e3,15e3,3e4,6e4,3e5,9e5,18e5,36e5,108e5,216e5,432e5,864e5,1728e5,6048e5,2592e6,7776e6,31536e6],Xl=[[ac.second,1],[ac.second,5],[ac.second,15],[ac.second,30],[ac.minute,1],[ac.minute,5],[ac.minute,15],[ac.minute,30],[ac.hour,1],[ac.hour,3],[ac.hour,6],[ac.hour,12],[ac.day,1],[ac.day,2],[ac.week,1],[ac.month,1],[ac.month,3],[ac.year,1]],$l=Il.multi([[".%L",function(n){return n.getMilliseconds()}],[":%S",function(n){return n.getSeconds()}],["%I:%M",function(n){return n.getMinutes()}],["%I %p",function(n){return n.getHours()}],["%a %d",function(n){return n.getDay()&&1!=n.getDate()}],["%b %d",function(n){return 1!=n.getDate()}],["%B",function(n){return n.getMonth()}],["%Y",Ne]]),Bl={range:function(n,t,e){return ta.range(Math.ceil(n/e)*e,+t,e).map(Ko)},floor:y,ceil:y};Xl.year=ac.year,ac.scale=function(){return Go(ta.scale.linear(),Xl,$l)};var Wl=Xl.map(function(n){return[n[0].utc,n[1]]}),Jl=Yl.multi([[".%L",function(n){return n.getUTCMilliseconds()}],[":%S",function(n){return n.getUTCSeconds()}],["%I:%M",function(n){return n.getUTCMinutes()}],["%I %p",function(n){return n.getUTCHours()}],["%a %d",function(n){return n.getUTCDay()&&1!=n.getUTCDate()}],["%b %d",function(n){return 1!=n.getUTCDate()}],["%B",function(n){return n.getUTCMonth()}],["%Y",Ne]]);Wl.year=ac.year.utc,ac.scale.utc=function(){return Go(ta.scale.linear(),Wl,Jl)},ta.text=At(function(n){return n.responseText}),ta.json=function(n,t){return Nt(n,"application/json",Qo,t)},ta.html=function(n,t){return Nt(n,"text/html",na,t)},ta.xml=At(function(n){return n.responseXML}),"function"==typeof define&&define.amd?define(ta):"object"==typeof module&&module.exports&&(module.exports=ta),this.d3=ta}();
!function(a){"use strict";function b(a){this.owner=a}function c(a,b){if(Object.create)b.prototype=Object.create(a.prototype);else{var c=function(){};c.prototype=a.prototype,b.prototype=new c}return b.prototype.constructor=b,b}function d(a){var b=this.internal=new e(this);b.loadConfig(a),b.beforeInit(a),b.init(),b.afterInit(a),function c(a,b,d){Object.keys(a).forEach(function(e){b[e]=a[e].bind(d),Object.keys(a[e]).length>0&&c(a[e],b[e],d)})}(h,this,this)}function e(b){var c=this;c.d3=a.d3?a.d3:"undefined"!=typeof require?require("d3"):void 0,c.api=b,c.config=c.getDefaultConfig(),c.data={},c.cache={},c.axes={}}function f(a){b.call(this,a)}function g(a,b){function c(a,b){a.attr("transform",function(a){return"translate("+Math.ceil(b(a)+u)+", 0)"})}function d(a,b){a.attr("transform",function(a){return"translate(0,"+Math.ceil(b(a))+")"})}function e(a){var b=a[0],c=a[a.length-1];return c>b?[b,c]:[c,b]}function f(a){var b,c,d=[];if(a.ticks)return a.ticks.apply(a,n);for(c=a.domain(),b=Math.ceil(c[0]);b<c[1];b++)d.push(b);return d.length>0&&d[0]>0&&d.unshift(d[0]-(d[1]-d[0])),d}function g(){var a,c=p.copy();return b.isCategory&&(a=p.domain(),c.domain([a[0],a[1]-1])),c}function h(a){var b=m?m(a):a;return"undefined"!=typeof b?b:""}function i(a){if(A)return A;var b={h:11.5,w:5.5};return a.select("text").text(h).each(function(a){var c=this.getBoundingClientRect(),d=h(a),e=c.height,f=d?c.width/d.length:void 0;e&&f&&(b.h=e,b.w=f)}).text(""),A=b,b}function j(c){return b.withoutTransition?c:a.transition(c)}function k(m){m.each(function(){function m(a,c){function d(a,b){f=void 0;for(var h=1;h<b.length;h++)if(" "===b.charAt(h)&&(f=h),e=b.substr(0,h+1),g=U.w*e.length,g>c)return d(a.concat(b.substr(0,f?f:h)),b.slice(f?f+1:h));return a.concat(b)}var e,f,g,i=h(a),j=[];return"[object Array]"===Object.prototype.toString.call(i)?i:((!c||0>=c)&&(c=X?95:b.isCategory?Math.ceil(F(G[1])-F(G[0]))-12:110),d(j,i+""))}function n(a,b){var c=U.h;return 0===b&&(c="left"===q||"right"===q?-((V[a.index]-1)*(U.h/2)-3):".71em"),c}function v(a){var b=p(a)+(o?0:u);return L[0]<b&&b<L[1]?r:0}function w(a){return a?a>0?"start":"end":"middle"}function x(a){return a?"rotate("+a+")":""}function y(a){return a?8*Math.sin(Math.PI*(a/180)):0}function z(a){return a?11.5-2.5*(a/15)*(a>0?1:-1):W}var A,B,C,D=k.g=a.select(this),E=this.__chart__||p,F=this.__chart__=g(),G=t?t:f(F),H=D.selectAll(".tick").data(G,F),I=H.enter().insert("g",".domain").attr("class","tick").style("opacity",1e-6),J=H.exit().remove(),K=j(H).style("opacity",1),L=p.rangeExtent?p.rangeExtent():e(p.range()),M=D.selectAll(".domain").data([0]),N=(M.enter().append("path").attr("class","domain"),j(M));I.append("line"),I.append("text");var O=I.select("line"),P=K.select("line"),Q=I.select("text"),R=K.select("text");b.isCategory?(u=Math.ceil((F(1)-F(0))/2),B=o?0:u,C=o?u:0):u=B=0;var S,T,U=i(D.select(".tick")),V=[],W=Math.max(r,0)+s,X="left"===q||"right"===q;S=H.select("text"),T=S.selectAll("tspan").data(function(a,c){var d=b.tickMultiline?m(a,b.tickWidth):[].concat(h(a));return V[c]=d.length,d.map(function(a){return{index:c,splitted:a}})}),T.enter().append("tspan"),T.exit().remove(),T.text(function(a){return a.splitted});var Y=b.tickTextRotate;switch(q){case"bottom":A=c,O.attr("y2",r),Q.attr("y",W),P.attr("x1",B).attr("x2",B).attr("y2",v),R.attr("x",0).attr("y",z(Y)).style("text-anchor",w(Y)).attr("transform",x(Y)),T.attr("x",0).attr("dy",n).attr("dx",y(Y)),N.attr("d","M"+L[0]+","+l+"V0H"+L[1]+"V"+l);break;case"top":A=c,O.attr("y2",-r),Q.attr("y",-W),P.attr("x2",0).attr("y2",-r),R.attr("x",0).attr("y",-W),S.style("text-anchor","middle"),T.attr("x",0).attr("dy","0em"),N.attr("d","M"+L[0]+","+-l+"V0H"+L[1]+"V"+-l);break;case"left":A=d,O.attr("x2",-r),Q.attr("x",-W),P.attr("x2",-r).attr("y1",C).attr("y2",C),R.attr("x",-W).attr("y",u),S.style("text-anchor","end"),T.attr("x",-W).attr("dy",n),N.attr("d","M"+-l+","+L[0]+"H0V"+L[1]+"H"+-l);break;case"right":A=d,O.attr("x2",r),Q.attr("x",W),P.attr("x2",r).attr("y2",0),R.attr("x",W).attr("y",0),S.style("text-anchor","start"),T.attr("x",W).attr("dy",n),N.attr("d","M"+l+","+L[0]+"H0V"+L[1]+"H"+l)}if(F.rangeBand){var Z=F,$=Z.rangeBand()/2;E=F=function(a){return Z(a)+$}}else E.rangeBand?E=F:J.call(A,F);I.call(A,E),K.call(A,F)})}var l,m,n,o,p=a.scale.linear(),q="bottom",r=6,s=3,t=null,u=0,v=!0;return b=b||{},l=b.withOuterTick?6:0,k.scale=function(a){return arguments.length?(p=a,k):p},k.orient=function(a){return arguments.length?(q=a in{top:1,right:1,bottom:1,left:1}?a+"":"bottom",k):q},k.tickFormat=function(a){return arguments.length?(m=a,k):m},k.tickCentered=function(a){return arguments.length?(o=a,k):o},k.tickOffset=function(){return u},k.tickInterval=function(){var a,c;return b.isCategory?a=2*u:(c=k.g.select("path.domain").node().getTotalLength()-2*l,a=c/k.g.selectAll("line").size()),a===1/0?0:a},k.ticks=function(){return arguments.length?(n=arguments,k):n},k.tickCulling=function(a){return arguments.length?(v=a,k):v},k.tickValues=function(a){if("function"==typeof a)t=function(){return a(p.domain())};else{if(!arguments.length)return t;t=a}return k},k}var h,i,j,k={version:"0.4.11"};k.generate=function(a){return new d(a)},k.chart={fn:d.prototype,internal:{fn:e.prototype,axis:{fn:f.prototype}}},h=k.chart.fn,i=k.chart.internal.fn,j=k.chart.internal.axis.fn,i.beforeInit=function(){},i.afterInit=function(){},i.init=function(){var a=this,b=a.config;if(a.initParams(),b.data_url)a.convertUrlToData(b.data_url,b.data_mimeType,b.data_headers,b.data_keys,a.initWithData);else if(b.data_json)a.initWithData(a.convertJsonToData(b.data_json,b.data_keys));else if(b.data_rows)a.initWithData(a.convertRowsToData(b.data_rows));else{if(!b.data_columns)throw Error("url or json or rows or columns is required.");a.initWithData(a.convertColumnsToData(b.data_columns))}},i.initParams=function(){var a=this,b=a.d3,c=a.config;a.clipId="c3-"+ +new Date+"-clip",a.clipIdForXAxis=a.clipId+"-xaxis",a.clipIdForYAxis=a.clipId+"-yaxis",a.clipIdForGrid=a.clipId+"-grid",a.clipIdForSubchart=a.clipId+"-subchart",a.clipPath=a.getClipPath(a.clipId),a.clipPathForXAxis=a.getClipPath(a.clipIdForXAxis),a.clipPathForYAxis=a.getClipPath(a.clipIdForYAxis),a.clipPathForGrid=a.getClipPath(a.clipIdForGrid),a.clipPathForSubchart=a.getClipPath(a.clipIdForSubchart),a.dragStart=null,a.dragging=!1,a.flowing=!1,a.cancelClick=!1,a.mouseover=!1,a.transiting=!1,a.color=a.generateColor(),a.levelColor=a.generateLevelColor(),a.dataTimeFormat=c.data_xLocaltime?b.time.format:b.time.format.utc,a.axisTimeFormat=c.axis_x_localtime?b.time.format:b.time.format.utc,a.defaultAxisTimeFormat=a.axisTimeFormat.multi([[".%L",function(a){return a.getMilliseconds()}],[":%S",function(a){return a.getSeconds()}],["%I:%M",function(a){return a.getMinutes()}],["%I %p",function(a){return a.getHours()}],["%-m/%-d",function(a){return a.getDay()&&1!==a.getDate()}],["%-m/%-d",function(a){return 1!==a.getDate()}],["%-m/%-d",function(a){return a.getMonth()}],["%Y/%-m/%-d",function(){return!0}]]),a.hiddenTargetIds=[],a.hiddenLegendIds=[],a.focusedTargetIds=[],a.defocusedTargetIds=[],a.xOrient=c.axis_rotated?"left":"bottom",a.yOrient=c.axis_rotated?c.axis_y_inner?"top":"bottom":c.axis_y_inner?"right":"left",a.y2Orient=c.axis_rotated?c.axis_y2_inner?"bottom":"top":c.axis_y2_inner?"left":"right",a.subXOrient=c.axis_rotated?"left":"bottom",a.isLegendRight="right"===c.legend_position,a.isLegendInset="inset"===c.legend_position,a.isLegendTop="top-left"===c.legend_inset_anchor||"top-right"===c.legend_inset_anchor,a.isLegendLeft="top-left"===c.legend_inset_anchor||"bottom-left"===c.legend_inset_anchor,a.legendStep=0,a.legendItemWidth=0,a.legendItemHeight=0,a.currentMaxTickWidths={x:0,y:0,y2:0},a.rotated_padding_left=30,a.rotated_padding_right=c.axis_rotated&&!c.axis_x_show?0:30,a.rotated_padding_top=5,a.withoutFadeIn={},a.intervalForObserveInserted=void 0,a.axes.subx=b.selectAll([])},i.initChartElements=function(){this.initBar&&this.initBar(),this.initLine&&this.initLine(),this.initArc&&this.initArc(),this.initGauge&&this.initGauge(),this.initText&&this.initText()},i.initWithData=function(a){var b,c,d=this,e=d.d3,g=d.config,h=!0;d.axis=new f(d),d.initPie&&d.initPie(),d.initBrush&&d.initBrush(),d.initZoom&&d.initZoom(),g.bindto?"function"==typeof g.bindto.node?d.selectChart=g.bindto:d.selectChart=e.select(g.bindto):d.selectChart=e.selectAll([]),d.selectChart.empty()&&(d.selectChart=e.select(document.createElement("div")).style("opacity",0),d.observeInserted(d.selectChart),h=!1),d.selectChart.html("").classed("c3",!0),d.data.xs={},d.data.targets=d.convertDataToTargets(a),g.data_filter&&(d.data.targets=d.data.targets.filter(g.data_filter)),g.data_hide&&d.addHiddenTargetIds(g.data_hide===!0?d.mapToIds(d.data.targets):g.data_hide),g.legend_hide&&d.addHiddenLegendIds(g.legend_hide===!0?d.mapToIds(d.data.targets):g.legend_hide),d.hasType("gauge")&&(g.legend_show=!1),d.updateSizes(),d.updateScales(),d.x.domain(e.extent(d.getXDomain(d.data.targets))),d.y.domain(d.getYDomain(d.data.targets,"y")),d.y2.domain(d.getYDomain(d.data.targets,"y2")),d.subX.domain(d.x.domain()),d.subY.domain(d.y.domain()),d.subY2.domain(d.y2.domain()),d.orgXDomain=d.x.domain(),d.brush&&d.brush.scale(d.subX),g.zoom_enabled&&d.zoom.scale(d.x),d.svg=d.selectChart.append("svg").style("overflow","hidden").on("mouseenter",function(){return g.onmouseover.call(d)}).on("mouseleave",function(){return g.onmouseout.call(d)}),d.config.svg_classname&&d.svg.attr("class",d.config.svg_classname),b=d.svg.append("defs"),d.clipChart=d.appendClip(b,d.clipId),d.clipXAxis=d.appendClip(b,d.clipIdForXAxis),d.clipYAxis=d.appendClip(b,d.clipIdForYAxis),d.clipGrid=d.appendClip(b,d.clipIdForGrid),d.clipSubchart=d.appendClip(b,d.clipIdForSubchart),d.updateSvgSize(),c=d.main=d.svg.append("g").attr("transform",d.getTranslate("main")),d.initSubchart&&d.initSubchart(),d.initTooltip&&d.initTooltip(),d.initLegend&&d.initLegend(),d.initTitle&&d.initTitle(),c.append("text").attr("class",l.text+" "+l.empty).attr("text-anchor","middle").attr("dominant-baseline","middle"),d.initRegion(),d.initGrid(),c.append("g").attr("clip-path",d.clipPath).attr("class",l.chart),g.grid_lines_front&&d.initGridLines(),d.initEventRect(),d.initChartElements(),c.insert("rect",g.zoom_privileged?null:"g."+l.regions).attr("class",l.zoomRect).attr("width",d.width).attr("height",d.height).style("opacity",0).on("dblclick.zoom",null),g.axis_x_extent&&d.brush.extent(d.getDefaultExtent()),d.axis.init(),d.updateTargets(d.data.targets),h&&(d.updateDimension(),d.config.oninit.call(d),d.redraw({withTransition:!1,withTransform:!0,withUpdateXDomain:!0,withUpdateOrgXDomain:!0,withTransitionForAxis:!1})),d.bindResize(),d.api.element=d.selectChart.node()},i.smoothLines=function(a,b){var c=this;"grid"===b&&a.each(function(){var a=c.d3.select(this),b=a.attr("x1"),d=a.attr("x2"),e=a.attr("y1"),f=a.attr("y2");a.attr({x1:Math.ceil(b),x2:Math.ceil(d),y1:Math.ceil(e),y2:Math.ceil(f)})})},i.updateSizes=function(){var a=this,b=a.config,c=a.legend?a.getLegendHeight():0,d=a.legend?a.getLegendWidth():0,e=a.isLegendRight||a.isLegendInset?0:c,f=a.hasArcType(),g=b.axis_rotated||f?0:a.getHorizontalAxisHeight("x"),h=b.subchart_show&&!f?b.subchart_size_height+g:0;a.currentWidth=a.getCurrentWidth(),a.currentHeight=a.getCurrentHeight(),a.margin=b.axis_rotated?{top:a.getHorizontalAxisHeight("y2")+a.getCurrentPaddingTop(),right:f?0:a.getCurrentPaddingRight(),bottom:a.getHorizontalAxisHeight("y")+e+a.getCurrentPaddingBottom(),left:h+(f?0:a.getCurrentPaddingLeft())}:{top:4+a.getCurrentPaddingTop(),right:f?0:a.getCurrentPaddingRight(),bottom:g+h+e+a.getCurrentPaddingBottom(),left:f?0:a.getCurrentPaddingLeft()},a.margin2=b.axis_rotated?{top:a.margin.top,right:NaN,bottom:20+e,left:a.rotated_padding_left}:{top:a.currentHeight-h-e,right:NaN,bottom:g+e,left:a.margin.left},a.margin3={top:0,right:NaN,bottom:0,left:0},a.updateSizeForLegend&&a.updateSizeForLegend(c,d),a.width=a.currentWidth-a.margin.left-a.margin.right,a.height=a.currentHeight-a.margin.top-a.margin.bottom,a.width<0&&(a.width=0),a.height<0&&(a.height=0),a.width2=b.axis_rotated?a.margin.left-a.rotated_padding_left-a.rotated_padding_right:a.width,a.height2=b.axis_rotated?a.height:a.currentHeight-a.margin2.top-a.margin2.bottom,a.width2<0&&(a.width2=0),a.height2<0&&(a.height2=0),a.arcWidth=a.width-(a.isLegendRight?d+10:0),a.arcHeight=a.height-(a.isLegendRight?0:10),a.hasType("gauge")&&!b.gauge_fullCircle&&(a.arcHeight+=a.height-a.getGaugeLabelHeight()),a.updateRadius&&a.updateRadius(),a.isLegendRight&&f&&(a.margin3.left=a.arcWidth/2+1.1*a.radiusExpanded)},i.updateTargets=function(a){var b=this;b.updateTargetsForText(a),b.updateTargetsForBar(a),b.updateTargetsForLine(a),b.hasArcType()&&b.updateTargetsForArc&&b.updateTargetsForArc(a),b.updateTargetsForSubchart&&b.updateTargetsForSubchart(a),b.showTargets()},i.showTargets=function(){var a=this;a.svg.selectAll("."+l.target).filter(function(b){return a.isTargetToShow(b.id)}).transition().duration(a.config.transition_duration).style("opacity",1)},i.redraw=function(a,b){var c,d,e,f,g,h,i,j,k,m,n,o,p,q,r,s,t,u,v,x,y,z,A,B,C,D,E,F,G,H=this,I=H.main,J=H.d3,K=H.config,L=H.getShapeIndices(H.isAreaType),M=H.getShapeIndices(H.isBarType),N=H.getShapeIndices(H.isLineType),O=H.hasArcType(),P=H.filterTargetsToShow(H.data.targets),Q=H.xv.bind(H);if(a=a||{},c=w(a,"withY",!0),d=w(a,"withSubchart",!0),e=w(a,"withTransition",!0),h=w(a,"withTransform",!1),i=w(a,"withUpdateXDomain",!1),j=w(a,"withUpdateOrgXDomain",!1),k=w(a,"withTrimXDomain",!0),p=w(a,"withUpdateXAxis",i),m=w(a,"withLegend",!1),n=w(a,"withEventRect",!0),o=w(a,"withDimension",!0),f=w(a,"withTransitionForExit",e),g=w(a,"withTransitionForAxis",e),v=e?K.transition_duration:0,x=f?v:0,y=g?v:0,b=b||H.axis.generateTransitions(y),m&&K.legend_show?H.updateLegend(H.mapToIds(H.data.targets),a,b):o&&H.updateDimension(!0),H.isCategorized()&&0===P.length&&H.x.domain([0,H.axes.x.selectAll(".tick").size()]),P.length?(H.updateXDomain(P,i,j,k),K.axis_x_tick_values||(B=H.axis.updateXAxisTickValues(P))):(H.xAxis.tickValues([]),H.subXAxis.tickValues([])),K.zoom_rescale&&!a.flow&&(E=H.x.orgDomain()),H.y.domain(H.getYDomain(P,"y",E)),H.y2.domain(H.getYDomain(P,"y2",E)),!K.axis_y_tick_values&&K.axis_y_tick_count&&H.yAxis.tickValues(H.axis.generateTickValues(H.y.domain(),K.axis_y_tick_count)),!K.axis_y2_tick_values&&K.axis_y2_tick_count&&H.y2Axis.tickValues(H.axis.generateTickValues(H.y2.domain(),K.axis_y2_tick_count)),H.axis.redraw(b,O),H.axis.updateLabels(e),(i||p)&&P.length)if(K.axis_x_tick_culling&&B){for(C=1;C<B.length;C++)if(B.length/C<K.axis_x_tick_culling_max){D=C;break}H.svg.selectAll("."+l.axisX+" .tick text").each(function(a){var b=B.indexOf(a);b>=0&&J.select(this).style("display",b%D?"none":"block")})}else H.svg.selectAll("."+l.axisX+" .tick text").style("display","block");q=H.generateDrawArea?H.generateDrawArea(L,!1):void 0,r=H.generateDrawBar?H.generateDrawBar(M):void 0,s=H.generateDrawLine?H.generateDrawLine(N,!1):void 0,t=H.generateXYForText(L,M,N,!0),u=H.generateXYForText(L,M,N,!1),c&&(H.subY.domain(H.getYDomain(P,"y")),H.subY2.domain(H.getYDomain(P,"y2"))),H.updateXgridFocus(),I.select("text."+l.text+"."+l.empty).attr("x",H.width/2).attr("y",H.height/2).text(K.data_empty_label_text).transition().style("opacity",P.length?0:1),H.updateGrid(v),H.updateRegion(v),H.updateBar(x),H.updateLine(x),H.updateArea(x),H.updateCircle(),H.hasDataLabel()&&H.updateText(x),H.redrawTitle&&H.redrawTitle(),H.redrawArc&&H.redrawArc(v,x,h),H.redrawSubchart&&H.redrawSubchart(d,b,v,x,L,M,N),I.selectAll("."+l.selectedCircles).filter(H.isBarType.bind(H)).selectAll("circle").remove(),K.interaction_enabled&&!a.flow&&n&&(H.redrawEventRect(),H.updateZoom&&H.updateZoom()),H.updateCircleY(),F=(H.config.axis_rotated?H.circleY:H.circleX).bind(H),G=(H.config.axis_rotated?H.circleX:H.circleY).bind(H),a.flow&&(A=H.generateFlow({targets:P,flow:a.flow,duration:a.flow.duration,drawBar:r,drawLine:s,drawArea:q,cx:F,cy:G,xv:Q,xForText:t,yForText:u})),(v||A)&&H.isTabVisible()?J.transition().duration(v).each(function(){var b=[];[H.redrawBar(r,!0),H.redrawLine(s,!0),H.redrawArea(q,!0),H.redrawCircle(F,G,!0),H.redrawText(t,u,a.flow,!0),H.redrawRegion(!0),H.redrawGrid(!0)].forEach(function(a){a.forEach(function(a){b.push(a)})}),z=H.generateWait(),b.forEach(function(a){z.add(a)})}).call(z,function(){A&&A(),K.onrendered&&K.onrendered.call(H)}):(H.redrawBar(r),H.redrawLine(s),H.redrawArea(q),H.redrawCircle(F,G),H.redrawText(t,u,a.flow),H.redrawRegion(),H.redrawGrid(),K.onrendered&&K.onrendered.call(H)),H.mapToIds(H.data.targets).forEach(function(a){H.withoutFadeIn[a]=!0})},i.updateAndRedraw=function(a){var b,c=this,d=c.config;a=a||{},a.withTransition=w(a,"withTransition",!0),a.withTransform=w(a,"withTransform",!1),a.withLegend=w(a,"withLegend",!1),a.withUpdateXDomain=!0,a.withUpdateOrgXDomain=!0,a.withTransitionForExit=!1,a.withTransitionForTransform=w(a,"withTransitionForTransform",a.withTransition),c.updateSizes(),a.withLegend&&d.legend_show||(b=c.axis.generateTransitions(a.withTransitionForAxis?d.transition_duration:0),c.updateScales(),c.updateSvgSize(),c.transformAll(a.withTransitionForTransform,b)),c.redraw(a,b)},i.redrawWithoutRescale=function(){this.redraw({withY:!1,withSubchart:!1,withEventRect:!1,withTransitionForAxis:!1})},i.isTimeSeries=function(){return"timeseries"===this.config.axis_x_type},i.isCategorized=function(){return this.config.axis_x_type.indexOf("categor")>=0},i.isCustomX=function(){var a=this,b=a.config;return!a.isTimeSeries()&&(b.data_x||v(b.data_xs))},i.isTimeSeriesY=function(){return"timeseries"===this.config.axis_y_type},i.getTranslate=function(a){var b,c,d=this,e=d.config;return"main"===a?(b=s(d.margin.left),c=s(d.margin.top)):"context"===a?(b=s(d.margin2.left),c=s(d.margin2.top)):"legend"===a?(b=d.margin3.left,c=d.margin3.top):"x"===a?(b=0,c=e.axis_rotated?0:d.height):"y"===a?(b=0,c=e.axis_rotated?d.height:0):"y2"===a?(b=e.axis_rotated?0:d.width,c=e.axis_rotated?1:0):"subx"===a?(b=0,c=e.axis_rotated?0:d.height2):"arc"===a&&(b=d.arcWidth/2,c=d.arcHeight/2),"translate("+b+","+c+")"},i.initialOpacity=function(a){return null!==a.value&&this.withoutFadeIn[a.id]?1:0},i.initialOpacityForCircle=function(a){return null!==a.value&&this.withoutFadeIn[a.id]?this.opacityForCircle(a):0},i.opacityForCircle=function(a){var b=this.config.point_show?1:0;return m(a.value)?this.isScatterType(a)?.5:b:0},i.opacityForText=function(){return this.hasDataLabel()?1:0},i.xx=function(a){return a?this.x(a.x):null},i.xv=function(a){var b=this,c=a.value;return b.isTimeSeries()?c=b.parseDate(a.value):b.isCategorized()&&"string"==typeof a.value&&(c=b.config.axis_x_categories.indexOf(a.value)),Math.ceil(b.x(c))},i.yv=function(a){var b=this,c=a.axis&&"y2"===a.axis?b.y2:b.y;return Math.ceil(c(a.value))},i.subxx=function(a){return a?this.subX(a.x):null},i.transformMain=function(a,b){var c,d,e,f=this;b&&b.axisX?c=b.axisX:(c=f.main.select("."+l.axisX),a&&(c=c.transition())),b&&b.axisY?d=b.axisY:(d=f.main.select("."+l.axisY),a&&(d=d.transition())),b&&b.axisY2?e=b.axisY2:(e=f.main.select("."+l.axisY2),a&&(e=e.transition())),(a?f.main.transition():f.main).attr("transform",f.getTranslate("main")),c.attr("transform",f.getTranslate("x")),d.attr("transform",f.getTranslate("y")),e.attr("transform",f.getTranslate("y2")),f.main.select("."+l.chartArcs).attr("transform",f.getTranslate("arc"))},i.transformAll=function(a,b){var c=this;c.transformMain(a,b),c.config.subchart_show&&c.transformContext(a,b),c.legend&&c.transformLegend(a)},i.updateSvgSize=function(){var a=this,b=a.svg.select(".c3-brush .background");a.svg.attr("width",a.currentWidth).attr("height",a.currentHeight),a.svg.selectAll(["#"+a.clipId,"#"+a.clipIdForGrid]).select("rect").attr("width",a.width).attr("height",a.height),a.svg.select("#"+a.clipIdForXAxis).select("rect").attr("x",a.getXAxisClipX.bind(a)).attr("y",a.getXAxisClipY.bind(a)).attr("width",a.getXAxisClipWidth.bind(a)).attr("height",a.getXAxisClipHeight.bind(a)),a.svg.select("#"+a.clipIdForYAxis).select("rect").attr("x",a.getYAxisClipX.bind(a)).attr("y",a.getYAxisClipY.bind(a)).attr("width",a.getYAxisClipWidth.bind(a)).attr("height",a.getYAxisClipHeight.bind(a)),a.svg.select("#"+a.clipIdForSubchart).select("rect").attr("width",a.width).attr("height",b.size()?b.attr("height"):0),a.svg.select("."+l.zoomRect).attr("width",a.width).attr("height",a.height),a.selectChart.style("max-height",a.currentHeight+"px")},i.updateDimension=function(a){var b=this;a||(b.config.axis_rotated?(b.axes.x.call(b.xAxis),b.axes.subx.call(b.subXAxis)):(b.axes.y.call(b.yAxis),b.axes.y2.call(b.y2Axis))),b.updateSizes(),b.updateScales(),b.updateSvgSize(),b.transformAll(!1)},i.observeInserted=function(b){var c,d=this;return"undefined"==typeof MutationObserver?void a.console.error("MutationObserver not defined."):(c=new MutationObserver(function(e){e.forEach(function(e){"childList"===e.type&&e.previousSibling&&(c.disconnect(),d.intervalForObserveInserted=a.setInterval(function(){b.node().parentNode&&(a.clearInterval(d.intervalForObserveInserted),d.updateDimension(),d.brush&&d.brush.update(),d.config.oninit.call(d),d.redraw({withTransform:!0,withUpdateXDomain:!0,withUpdateOrgXDomain:!0,withTransition:!1,withTransitionForTransform:!1,withLegend:!0}),b.transition().style("opacity",1))},10))})}),void c.observe(b.node(),{attributes:!0,childList:!0,characterData:!0}))},i.bindResize=function(){var b=this,c=b.config;if(b.resizeFunction=b.generateResize(),b.resizeFunction.add(function(){c.onresize.call(b)}),c.resize_auto&&b.resizeFunction.add(function(){void 0!==b.resizeTimeout&&a.clearTimeout(b.resizeTimeout),b.resizeTimeout=a.setTimeout(function(){delete b.resizeTimeout,b.api.flush()},100)}),b.resizeFunction.add(function(){c.onresized.call(b)}),a.attachEvent)a.attachEvent("onresize",b.resizeFunction);else if(a.addEventListener)a.addEventListener("resize",b.resizeFunction,!1);else{var d=a.onresize;d?d.add&&d.remove||(d=b.generateResize(),d.add(a.onresize)):d=b.generateResize(),d.add(b.resizeFunction),a.onresize=d}},i.generateResize=function(){function a(){b.forEach(function(a){a()})}var b=[];return a.add=function(a){b.push(a)},a.remove=function(a){for(var c=0;c<b.length;c++)if(b[c]===a){b.splice(c,1);break}},a},i.endall=function(a,b){var c=0;a.each(function(){++c}).each("end",function(){--c||b.apply(this,arguments)})},i.generateWait=function(){var a=[],b=function(b,c){var d=setInterval(function(){var b=0;a.forEach(function(a){if(a.empty())return void(b+=1);try{a.transition()}catch(c){b+=1}}),b===a.length&&(clearInterval(d),c&&c())},10)};return b.add=function(b){a.push(b)},b},i.parseDate=function(b){var c,d=this;return b instanceof Date?c=b:"string"==typeof b?c=d.dataTimeFormat(d.config.data_xFormat).parse(b):"number"!=typeof b||isNaN(b)||(c=new Date(+b)),c&&!isNaN(+c)||a.console.error("Failed to parse x '"+b+"' to Date object"),c},i.isTabVisible=function(){var a;return"undefined"!=typeof document.hidden?a="hidden":"undefined"!=typeof document.mozHidden?a="mozHidden":"undefined"!=typeof document.msHidden?a="msHidden":"undefined"!=typeof document.webkitHidden&&(a="webkitHidden"),!document[a]},i.getDefaultConfig=function(){var a={bindto:"#chart",svg_classname:void 0,size_width:void 0,size_height:void 0,padding_left:void 0,padding_right:void 0,padding_top:void 0,padding_bottom:void 0,resize_auto:!0,zoom_enabled:!1,zoom_extent:void 0,zoom_privileged:!1,zoom_rescale:!1,zoom_onzoom:function(){},zoom_onzoomstart:function(){},zoom_onzoomend:function(){},zoom_x_min:void 0,zoom_x_max:void 0,interaction_brighten:!0,interaction_enabled:!0,onmouseover:function(){},onmouseout:function(){},onresize:function(){},onresized:function(){},oninit:function(){},onrendered:function(){},transition_duration:350,data_x:void 0,data_xs:{},data_xFormat:"%Y-%m-%d",data_xLocaltime:!0,data_xSort:!0,data_idConverter:function(a){return a},data_names:{},data_classes:{},data_groups:[],data_axes:{},data_type:void 0,data_types:{},data_labels:{},data_order:"desc",data_regions:{},data_color:void 0,data_colors:{},data_hide:!1,data_filter:void 0,data_selection_enabled:!1,data_selection_grouped:!1,data_selection_isselectable:function(){return!0},data_selection_multiple:!0,data_selection_draggable:!1,data_onclick:function(){},data_onmouseover:function(){},data_onmouseout:function(){},data_onselected:function(){},data_onunselected:function(){},data_url:void 0,data_headers:void 0,data_json:void 0,data_rows:void 0,data_columns:void 0,data_mimeType:void 0,data_keys:void 0,data_empty_label_text:"",subchart_show:!1,subchart_size_height:60,subchart_axis_x_show:!0,subchart_onbrush:function(){},color_pattern:[],color_threshold:{},legend_show:!0,legend_hide:!1,legend_position:"bottom",legend_inset_anchor:"top-left",legend_inset_x:10,legend_inset_y:0,legend_inset_step:void 0,legend_item_onclick:void 0,legend_item_onmouseover:void 0,legend_item_onmouseout:void 0,legend_equally:!1,legend_padding:0,legend_item_tile_width:10,legend_item_tile_height:10,axis_rotated:!1,axis_x_show:!0,axis_x_type:"indexed",axis_x_localtime:!0,axis_x_categories:[],axis_x_tick_centered:!1,axis_x_tick_format:void 0,axis_x_tick_culling:{},axis_x_tick_culling_max:10,axis_x_tick_count:void 0,axis_x_tick_fit:!0,axis_x_tick_values:null,axis_x_tick_rotate:0,axis_x_tick_outer:!0,axis_x_tick_multiline:!0,axis_x_tick_width:null,axis_x_max:void 0,axis_x_min:void 0,axis_x_padding:{},axis_x_height:void 0,axis_x_extent:void 0,axis_x_label:{},axis_y_show:!0,axis_y_type:void 0,axis_y_max:void 0,axis_y_min:void 0,axis_y_inverted:!1,axis_y_center:void 0,axis_y_inner:void 0,axis_y_label:{},axis_y_tick_format:void 0,axis_y_tick_outer:!0,axis_y_tick_values:null,axis_y_tick_rotate:0,axis_y_tick_count:void 0,axis_y_tick_time_value:void 0,axis_y_tick_time_interval:void 0,axis_y_padding:{},axis_y_default:void 0,axis_y2_show:!1,axis_y2_max:void 0,axis_y2_min:void 0,axis_y2_inverted:!1,axis_y2_center:void 0,axis_y2_inner:void 0,axis_y2_label:{},axis_y2_tick_format:void 0,axis_y2_tick_outer:!0,axis_y2_tick_values:null,axis_y2_tick_count:void 0,axis_y2_padding:{},axis_y2_default:void 0,grid_x_show:!1,grid_x_type:"tick",grid_x_lines:[],grid_y_show:!1,grid_y_lines:[],grid_y_ticks:10,grid_focus_show:!0,grid_lines_front:!0,point_show:!0,point_r:2.5,point_sensitivity:10,point_focus_expand_enabled:!0,point_focus_expand_r:void 0,point_select_r:void 0,line_connectNull:!1,line_step_type:"step",bar_width:void 0,bar_width_ratio:.6,bar_width_max:void 0,bar_zerobased:!0,area_zerobased:!0,area_above:!1,pie_label_show:!0,pie_label_format:void 0,pie_label_threshold:.05,pie_label_ratio:void 0,pie_expand:{},pie_expand_duration:50,gauge_fullCircle:!1,gauge_label_show:!0,gauge_label_format:void 0,gauge_min:0,gauge_max:100,gauge_startingAngle:-1*Math.PI/2,gauge_units:void 0,gauge_width:void 0,gauge_expand:{},gauge_expand_duration:50,donut_label_show:!0,donut_label_format:void 0,donut_label_threshold:.05,donut_label_ratio:void 0,donut_width:void 0,donut_title:"",donut_expand:{},donut_expand_duration:50,spline_interpolation_type:"cardinal",regions:[],tooltip_show:!0,tooltip_grouped:!0,tooltip_format_title:void 0,tooltip_format_name:void 0,tooltip_format_value:void 0,tooltip_position:void 0,tooltip_contents:function(a,b,c,d){return this.getTooltipContent?this.getTooltipContent(a,b,c,d):""},tooltip_init_show:!1,tooltip_init_x:0,tooltip_init_position:{top:"0px",left:"50px"},tooltip_onshow:function(){},tooltip_onhide:function(){},title_text:void 0,title_padding:{top:0,right:0,bottom:0,left:0},title_position:"top-center"};return Object.keys(this.additionalConfig).forEach(function(b){a[b]=this.additionalConfig[b]},this),a},i.additionalConfig={},i.loadConfig=function(a){function b(){var a=d.shift();return a&&c&&"object"==typeof c&&a in c?(c=c[a],b()):a?void 0:c}var c,d,e,f=this.config;Object.keys(f).forEach(function(g){c=a,d=g.split("_"),e=b(),q(e)&&(f[g]=e)})},i.getScale=function(a,b,c){return(c?this.d3.time.scale():this.d3.scale.linear()).range([a,b])},i.getX=function(a,b,c,d){var e,f=this,g=f.getScale(a,b,f.isTimeSeries()),h=c?g.domain(c):g;f.isCategorized()?(d=d||function(){return 0},g=function(a,b){var c=h(a)+d(a);return b?c:Math.ceil(c)}):g=function(a,b){var c=h(a);return b?c:Math.ceil(c)};for(e in h)g[e]=h[e];return g.orgDomain=function(){return h.domain()},f.isCategorized()&&(g.domain=function(a){return arguments.length?(h.domain(a),g):(a=this.orgDomain(),[a[0],a[1]+1])}),g},i.getY=function(a,b,c){var d=this.getScale(a,b,this.isTimeSeriesY());return c&&d.domain(c),d},i.getYScale=function(a){return"y2"===this.axis.getId(a)?this.y2:this.y},i.getSubYScale=function(a){return"y2"===this.axis.getId(a)?this.subY2:this.subY},i.updateScales=function(){var a=this,b=a.config,c=!a.x;a.xMin=b.axis_rotated?1:0,a.xMax=b.axis_rotated?a.height:a.width,a.yMin=b.axis_rotated?0:a.height,a.yMax=b.axis_rotated?a.width:1,a.subXMin=a.xMin,a.subXMax=a.xMax,a.subYMin=b.axis_rotated?0:a.height2,a.subYMax=b.axis_rotated?a.width2:1,a.x=a.getX(a.xMin,a.xMax,c?void 0:a.x.orgDomain(),function(){return a.xAxis.tickOffset()}),a.y=a.getY(a.yMin,a.yMax,c?b.axis_y_default:a.y.domain()),a.y2=a.getY(a.yMin,a.yMax,c?b.axis_y2_default:a.y2.domain()),a.subX=a.getX(a.xMin,a.xMax,a.orgXDomain,function(b){return b%1?0:a.subXAxis.tickOffset()}),a.subY=a.getY(a.subYMin,a.subYMax,c?b.axis_y_default:a.subY.domain()),a.subY2=a.getY(a.subYMin,a.subYMax,c?b.axis_y2_default:a.subY2.domain()),a.xAxisTickFormat=a.axis.getXAxisTickFormat(),a.xAxisTickValues=a.axis.getXAxisTickValues(),a.yAxisTickValues=a.axis.getYAxisTickValues(),a.y2AxisTickValues=a.axis.getY2AxisTickValues(),a.xAxis=a.axis.getXAxis(a.x,a.xOrient,a.xAxisTickFormat,a.xAxisTickValues,b.axis_x_tick_outer),a.subXAxis=a.axis.getXAxis(a.subX,a.subXOrient,a.xAxisTickFormat,a.xAxisTickValues,b.axis_x_tick_outer),a.yAxis=a.axis.getYAxis(a.y,a.yOrient,b.axis_y_tick_format,a.yAxisTickValues,b.axis_y_tick_outer),a.y2Axis=a.axis.getYAxis(a.y2,a.y2Orient,b.axis_y2_tick_format,a.y2AxisTickValues,b.axis_y2_tick_outer),c||(a.brush&&a.brush.scale(a.subX),b.zoom_enabled&&a.zoom.scale(a.x)),a.updateArc&&a.updateArc()},i.getYDomainMin=function(a){var b,c,d,e,f,g,h=this,i=h.config,j=h.mapToIds(a),k=h.getValuesAsIdKeyed(a);if(i.data_groups.length>0)for(g=h.hasNegativeValueInTargets(a),b=0;b<i.data_groups.length;b++)if(e=i.data_groups[b].filter(function(a){return j.indexOf(a)>=0}),0!==e.length)for(d=e[0],g&&k[d]&&k[d].forEach(function(a,b){k[d][b]=0>a?a:0}),c=1;c<e.length;c++)f=e[c],k[f]&&k[f].forEach(function(a,b){h.axis.getId(f)!==h.axis.getId(d)||!k[d]||g&&+a>0||(k[d][b]+=+a)});return h.d3.min(Object.keys(k).map(function(a){return h.d3.min(k[a])}))},i.getYDomainMax=function(a){var b,c,d,e,f,g,h=this,i=h.config,j=h.mapToIds(a),k=h.getValuesAsIdKeyed(a);if(i.data_groups.length>0)for(g=h.hasPositiveValueInTargets(a),b=0;b<i.data_groups.length;b++)if(e=i.data_groups[b].filter(function(a){return j.indexOf(a)>=0}),0!==e.length)for(d=e[0],g&&k[d]&&k[d].forEach(function(a,b){k[d][b]=a>0?a:0}),c=1;c<e.length;c++)f=e[c],k[f]&&k[f].forEach(function(a,b){h.axis.getId(f)!==h.axis.getId(d)||!k[d]||g&&0>+a||(k[d][b]+=+a)});return h.d3.max(Object.keys(k).map(function(a){return h.d3.max(k[a])}))},i.getYDomain=function(a,b,c){var d,e,f,g,h,i,j,k,l,n,o,p=this,q=p.config,r=a.filter(function(a){return p.axis.getId(a.id)===b}),s=c?p.filterByXDomain(r,c):r,u="y2"===b?q.axis_y2_min:q.axis_y_min,w="y2"===b?q.axis_y2_max:q.axis_y_max,x=p.getYDomainMin(s),y=p.getYDomainMax(s),z="y2"===b?q.axis_y2_center:q.axis_y_center,A=p.hasType("bar",s)&&q.bar_zerobased||p.hasType("area",s)&&q.area_zerobased,B="y2"===b?q.axis_y2_inverted:q.axis_y_inverted,C=p.hasDataLabel()&&q.axis_rotated,D=p.hasDataLabel()&&!q.axis_rotated;return x=m(u)?u:m(w)?w>x?x:w-10:x,y=m(w)?w:m(u)?y>u?y:u+10:y,0===s.length?"y2"===b?p.y2.domain():p.y.domain():(isNaN(x)&&(x=0),isNaN(y)&&(y=x),x===y&&(0>x?y=0:x=0),n=x>=0&&y>=0,o=0>=x&&0>=y,(m(u)&&n||m(w)&&o)&&(A=!1),A&&(n&&(x=0),o&&(y=0)),e=Math.abs(y-x),f=g=h=.1*e,"undefined"!=typeof z&&(i=Math.max(Math.abs(x),Math.abs(y)),y=z+i,x=z-i),C?(j=p.getDataLabelLength(x,y,"width"),k=t(p.y.range()),l=[j[0]/k,j[1]/k],
g+=e*(l[1]/(1-l[0]-l[1])),h+=e*(l[0]/(1-l[0]-l[1]))):D&&(j=p.getDataLabelLength(x,y,"height"),g+=p.axis.convertPixelsToAxisPadding(j[1],e),h+=p.axis.convertPixelsToAxisPadding(j[0],e)),"y"===b&&v(q.axis_y_padding)&&(g=p.axis.getPadding(q.axis_y_padding,"top",g,e),h=p.axis.getPadding(q.axis_y_padding,"bottom",h,e)),"y2"===b&&v(q.axis_y2_padding)&&(g=p.axis.getPadding(q.axis_y2_padding,"top",g,e),h=p.axis.getPadding(q.axis_y2_padding,"bottom",h,e)),A&&(n&&(h=x),o&&(g=-y)),d=[x-h,y+g],B?d.reverse():d)},i.getXDomainMin=function(a){var b=this,c=b.config;return q(c.axis_x_min)?b.isTimeSeries()?this.parseDate(c.axis_x_min):c.axis_x_min:b.d3.min(a,function(a){return b.d3.min(a.values,function(a){return a.x})})},i.getXDomainMax=function(a){var b=this,c=b.config;return q(c.axis_x_max)?b.isTimeSeries()?this.parseDate(c.axis_x_max):c.axis_x_max:b.d3.max(a,function(a){return b.d3.max(a.values,function(a){return a.x})})},i.getXDomainPadding=function(a){var b,c,d,e,f=this,g=f.config,h=a[1]-a[0];return f.isCategorized()?c=0:f.hasType("bar")?(b=f.getMaxDataCount(),c=b>1?h/(b-1)/2:.5):c=.01*h,"object"==typeof g.axis_x_padding&&v(g.axis_x_padding)?(d=m(g.axis_x_padding.left)?g.axis_x_padding.left:c,e=m(g.axis_x_padding.right)?g.axis_x_padding.right:c):d=e="number"==typeof g.axis_x_padding?g.axis_x_padding:c,{left:d,right:e}},i.getXDomain=function(a){var b=this,c=[b.getXDomainMin(a),b.getXDomainMax(a)],d=c[0],e=c[1],f=b.getXDomainPadding(c),g=0,h=0;return d-e!==0||b.isCategorized()||(b.isTimeSeries()?(d=new Date(.5*d.getTime()),e=new Date(1.5*e.getTime())):(d=0===d?1:.5*d,e=0===e?-1:1.5*e)),(d||0===d)&&(g=b.isTimeSeries()?new Date(d.getTime()-f.left):d-f.left),(e||0===e)&&(h=b.isTimeSeries()?new Date(e.getTime()+f.right):e+f.right),[g,h]},i.updateXDomain=function(a,b,c,d,e){var f=this,g=f.config;return c&&(f.x.domain(e?e:f.d3.extent(f.getXDomain(a))),f.orgXDomain=f.x.domain(),g.zoom_enabled&&f.zoom.scale(f.x).updateScaleExtent(),f.subX.domain(f.x.domain()),f.brush&&f.brush.scale(f.subX)),b&&(f.x.domain(e?e:!f.brush||f.brush.empty()?f.orgXDomain:f.brush.extent()),g.zoom_enabled&&f.zoom.scale(f.x).updateScaleExtent()),d&&f.x.domain(f.trimXDomain(f.x.orgDomain())),f.x.domain()},i.trimXDomain=function(a){var b=this.getZoomDomain(),c=b[0],d=b[1];return a[0]<=c&&(a[1]=+a[1]+(c-a[0]),a[0]=c),d<=a[1]&&(a[0]=+a[0]-(a[1]-d),a[1]=d),a},i.isX=function(a){var b=this,c=b.config;return c.data_x&&a===c.data_x||v(c.data_xs)&&x(c.data_xs,a)},i.isNotX=function(a){return!this.isX(a)},i.getXKey=function(a){var b=this,c=b.config;return c.data_x?c.data_x:v(c.data_xs)?c.data_xs[a]:null},i.getXValuesOfXKey=function(a,b){var c,d=this,e=b&&v(b)?d.mapToIds(b):[];return e.forEach(function(b){d.getXKey(b)===a&&(c=d.data.xs[b])}),c},i.getIndexByX=function(a){var b=this,c=b.filterByX(b.data.targets,a);return c.length?c[0].index:null},i.getXValue=function(a,b){var c=this;return a in c.data.xs&&c.data.xs[a]&&m(c.data.xs[a][b])?c.data.xs[a][b]:b},i.getOtherTargetXs=function(){var a=this,b=Object.keys(a.data.xs);return b.length?a.data.xs[b[0]]:null},i.getOtherTargetX=function(a){var b=this.getOtherTargetXs();return b&&a<b.length?b[a]:null},i.addXs=function(a){var b=this;Object.keys(a).forEach(function(c){b.config.data_xs[c]=a[c]})},i.hasMultipleX=function(a){return this.d3.set(Object.keys(a).map(function(b){return a[b]})).size()>1},i.isMultipleX=function(){return v(this.config.data_xs)||!this.config.data_xSort||this.hasType("scatter")},i.addName=function(a){var b,c=this;return a&&(b=c.config.data_names[a.id],a.name=void 0!==b?b:a.id),a},i.getValueOnIndex=function(a,b){var c=a.filter(function(a){return a.index===b});return c.length?c[0]:null},i.updateTargetX=function(a,b){var c=this;a.forEach(function(a){a.values.forEach(function(d,e){d.x=c.generateTargetX(b[e],a.id,e)}),c.data.xs[a.id]=b})},i.updateTargetXs=function(a,b){var c=this;a.forEach(function(a){b[a.id]&&c.updateTargetX([a],b[a.id])})},i.generateTargetX=function(a,b,c){var d,e=this;return d=e.isTimeSeries()?a?e.parseDate(a):e.parseDate(e.getXValue(b,c)):e.isCustomX()&&!e.isCategorized()?m(a)?+a:e.getXValue(b,c):c},i.cloneTarget=function(a){return{id:a.id,id_org:a.id_org,values:a.values.map(function(a){return{x:a.x,value:a.value,id:a.id}})}},i.updateXs=function(){var a=this;a.data.targets.length&&(a.xs=[],a.data.targets[0].values.forEach(function(b){a.xs[b.index]=b.x}))},i.getPrevX=function(a){var b=this.xs[a-1];return"undefined"!=typeof b?b:null},i.getNextX=function(a){var b=this.xs[a+1];return"undefined"!=typeof b?b:null},i.getMaxDataCount=function(){var a=this;return a.d3.max(a.data.targets,function(a){return a.values.length})},i.getMaxDataCountTarget=function(a){var b,c=a.length,d=0;return c>1?a.forEach(function(a){a.values.length>d&&(b=a,d=a.values.length)}):b=c?a[0]:null,b},i.getEdgeX=function(a){var b=this;return a.length?[b.d3.min(a,function(a){return a.values[0].x}),b.d3.max(a,function(a){return a.values[a.values.length-1].x})]:[0,0]},i.mapToIds=function(a){return a.map(function(a){return a.id})},i.mapToTargetIds=function(a){var b=this;return a?[].concat(a):b.mapToIds(b.data.targets)},i.hasTarget=function(a,b){var c,d=this.mapToIds(a);for(c=0;c<d.length;c++)if(d[c]===b)return!0;return!1},i.isTargetToShow=function(a){return this.hiddenTargetIds.indexOf(a)<0},i.isLegendToShow=function(a){return this.hiddenLegendIds.indexOf(a)<0},i.filterTargetsToShow=function(a){var b=this;return a.filter(function(a){return b.isTargetToShow(a.id)})},i.mapTargetsToUniqueXs=function(a){var b=this,c=b.d3.set(b.d3.merge(a.map(function(a){return a.values.map(function(a){return+a.x})}))).values();return c=b.isTimeSeries()?c.map(function(a){return new Date(+a)}):c.map(function(a){return+a}),c.sort(function(a,b){return b>a?-1:a>b?1:a>=b?0:NaN})},i.addHiddenTargetIds=function(a){this.hiddenTargetIds=this.hiddenTargetIds.concat(a)},i.removeHiddenTargetIds=function(a){this.hiddenTargetIds=this.hiddenTargetIds.filter(function(b){return a.indexOf(b)<0})},i.addHiddenLegendIds=function(a){this.hiddenLegendIds=this.hiddenLegendIds.concat(a)},i.removeHiddenLegendIds=function(a){this.hiddenLegendIds=this.hiddenLegendIds.filter(function(b){return a.indexOf(b)<0})},i.getValuesAsIdKeyed=function(a){var b={};return a.forEach(function(a){b[a.id]=[],a.values.forEach(function(c){b[a.id].push(c.value)})}),b},i.checkValueInTargets=function(a,b){var c,d,e,f=Object.keys(a);for(c=0;c<f.length;c++)for(e=a[f[c]].values,d=0;d<e.length;d++)if(b(e[d].value))return!0;return!1},i.hasNegativeValueInTargets=function(a){return this.checkValueInTargets(a,function(a){return 0>a})},i.hasPositiveValueInTargets=function(a){return this.checkValueInTargets(a,function(a){return a>0})},i.isOrderDesc=function(){var a=this.config;return"string"==typeof a.data_order&&"desc"===a.data_order.toLowerCase()},i.isOrderAsc=function(){var a=this.config;return"string"==typeof a.data_order&&"asc"===a.data_order.toLowerCase()},i.orderTargets=function(a){var b=this,c=b.config,d=b.isOrderAsc(),e=b.isOrderDesc();return d||e?a.sort(function(a,b){var c=function(a,b){return a+Math.abs(b.value)},e=a.values.reduce(c,0),f=b.values.reduce(c,0);return d?f-e:e-f}):n(c.data_order)&&a.sort(c.data_order),a},i.filterByX=function(a,b){return this.d3.merge(a.map(function(a){return a.values})).filter(function(a){return a.x-b===0})},i.filterRemoveNull=function(a){return a.filter(function(a){return m(a.value)})},i.filterByXDomain=function(a,b){return a.map(function(a){return{id:a.id,id_org:a.id_org,values:a.values.filter(function(a){return b[0]<=a.x&&a.x<=b[1]})}})},i.hasDataLabel=function(){var a=this.config;return"boolean"==typeof a.data_labels&&a.data_labels?!0:!("object"!=typeof a.data_labels||!v(a.data_labels))},i.getDataLabelLength=function(a,b,c){var d=this,e=[0,0],f=1.3;return d.selectChart.select("svg").selectAll(".dummy").data([a,b]).enter().append("text").text(function(a){return d.dataLabelFormat(a.id)(a)}).each(function(a,b){e[b]=this.getBoundingClientRect()[c]*f}).remove(),e},i.isNoneArc=function(a){return this.hasTarget(this.data.targets,a.id)},i.isArc=function(a){return"data"in a&&this.hasTarget(this.data.targets,a.data.id)},i.findSameXOfValues=function(a,b){var c,d=a[b].x,e=[];for(c=b-1;c>=0&&d===a[c].x;c--)e.push(a[c]);for(c=b;c<a.length&&d===a[c].x;c++)e.push(a[c]);return e},i.findClosestFromTargets=function(a,b){var c,d=this;return c=a.map(function(a){return d.findClosest(a.values,b)}),d.findClosest(c,b)},i.findClosest=function(a,b){var c,d=this,e=d.config.point_sensitivity;return a.filter(function(a){return a&&d.isBarType(a.id)}).forEach(function(a){var b=d.main.select("."+l.bars+d.getTargetSelectorSuffix(a.id)+" ."+l.bar+"-"+a.index).node();!c&&d.isWithinBar(b)&&(c=a)}),a.filter(function(a){return a&&!d.isBarType(a.id)}).forEach(function(a){var f=d.dist(a,b);e>f&&(e=f,c=a)}),c},i.dist=function(a,b){var c=this,d=c.config,e=d.axis_rotated?1:0,f=d.axis_rotated?0:1,g=c.circleY(a,a.index),h=c.x(a.x);return Math.sqrt(Math.pow(h-b[e],2)+Math.pow(g-b[f],2))},i.convertValuesToStep=function(a){var b,c=[].concat(a);if(!this.isCategorized())return a;for(b=a.length+1;b>0;b--)c[b]=c[b-1];return c[0]={x:c[0].x-1,value:c[0].value,id:c[0].id},c[a.length+1]={x:c[a.length].x+1,value:c[a.length].value,id:c[a.length].id},c},i.updateDataAttributes=function(a,b){var c=this,d=c.config,e=d["data_"+a];return"undefined"==typeof b?e:(Object.keys(b).forEach(function(a){e[a]=b[a]}),c.redraw({withLegend:!0}),e)},i.convertUrlToData=function(a,b,c,d,e){var f=this,g=b?b:"csv",h=f.d3.xhr(a);c&&Object.keys(c).forEach(function(a){h.header(a,c[a])}),h.get(function(a,b){var c;if(!b)throw new Error(a.responseURL+" "+a.status+" ("+a.statusText+")");c="json"===g?f.convertJsonToData(JSON.parse(b.response),d):"tsv"===g?f.convertTsvToData(b.response):f.convertCsvToData(b.response),e.call(f,c)})},i.convertXsvToData=function(a,b){var c,d=b.parseRows(a);return 1===d.length?(c=[{}],d[0].forEach(function(a){c[0][a]=null})):c=b.parse(a),c},i.convertCsvToData=function(a){return this.convertXsvToData(a,this.d3.csv)},i.convertTsvToData=function(a){return this.convertXsvToData(a,this.d3.tsv)},i.convertJsonToData=function(a,b){var c,d,e=this,f=[];return b?(b.x?(c=b.value.concat(b.x),e.config.data_x=b.x):c=b.value,f.push(c),a.forEach(function(a){var b=[];c.forEach(function(c){var d=e.findValueInJson(a,c);p(d)&&(d=null),b.push(d)}),f.push(b)}),d=e.convertRowsToData(f)):(Object.keys(a).forEach(function(b){f.push([b].concat(a[b]))}),d=e.convertColumnsToData(f)),d},i.findValueInJson=function(a,b){b=b.replace(/\[(\w+)\]/g,".$1"),b=b.replace(/^\./,"");for(var c=b.split("."),d=0;d<c.length;++d){var e=c[d];if(!(e in a))return;a=a[e]}return a},i.convertRowsToData=function(a){var b,c,d=a[0],e={},f=[];for(b=1;b<a.length;b++){for(e={},c=0;c<a[b].length;c++){if(p(a[b][c]))throw new Error("Source data is missing a component at ("+b+","+c+")!");e[d[c]]=a[b][c]}f.push(e)}return f},i.convertColumnsToData=function(a){var b,c,d,e=[];for(b=0;b<a.length;b++)for(d=a[b][0],c=1;c<a[b].length;c++){if(p(e[c-1])&&(e[c-1]={}),p(a[b][c]))throw new Error("Source data is missing a component at ("+b+","+c+")!");e[c-1][d]=a[b][c]}return e},i.convertDataToTargets=function(a,b){var c,d=this,e=d.config,f=d.d3.keys(a[0]).filter(d.isNotX,d),g=d.d3.keys(a[0]).filter(d.isX,d);return f.forEach(function(c){var f=d.getXKey(c);d.isCustomX()||d.isTimeSeries()?g.indexOf(f)>=0?d.data.xs[c]=(b&&d.data.xs[c]?d.data.xs[c]:[]).concat(a.map(function(a){return a[f]}).filter(m).map(function(a,b){return d.generateTargetX(a,c,b)})):e.data_x?d.data.xs[c]=d.getOtherTargetXs():v(e.data_xs)&&(d.data.xs[c]=d.getXValuesOfXKey(f,d.data.targets)):d.data.xs[c]=a.map(function(a,b){return b})}),f.forEach(function(a){if(!d.data.xs[a])throw new Error('x is not defined for id = "'+a+'".')}),c=f.map(function(b,c){var f=e.data_idConverter(b);return{id:f,id_org:b,values:a.map(function(a,g){var h,i=d.getXKey(b),j=a[i],k=null===a[b]||isNaN(a[b])?null:+a[b];return d.isCustomX()&&d.isCategorized()&&0===c&&!p(j)?(0===c&&0===g&&(e.axis_x_categories=[]),h=e.axis_x_categories.indexOf(j),-1===h&&(h=e.axis_x_categories.length,e.axis_x_categories.push(j))):h=d.generateTargetX(j,b,g),(p(a[b])||d.data.xs[b].length<=g)&&(h=void 0),{x:h,value:k,id:f}}).filter(function(a){return q(a.x)})}}),c.forEach(function(a){var b;e.data_xSort&&(a.values=a.values.sort(function(a,b){var c=a.x||0===a.x?a.x:1/0,d=b.x||0===b.x?b.x:1/0;return c-d})),b=0,a.values.forEach(function(a){a.index=b++}),d.data.xs[a.id].sort(function(a,b){return a-b})}),d.hasNegativeValue=d.hasNegativeValueInTargets(c),d.hasPositiveValue=d.hasPositiveValueInTargets(c),e.data_type&&d.setTargetType(d.mapToIds(c).filter(function(a){return!(a in e.data_types)}),e.data_type),c.forEach(function(a){d.addCache(a.id_org,a)}),c},i.load=function(a,b){var c=this;a&&(b.filter&&(a=a.filter(b.filter)),(b.type||b.types)&&a.forEach(function(a){var d=b.types&&b.types[a.id]?b.types[a.id]:b.type;c.setTargetType(a.id,d)}),c.data.targets.forEach(function(b){for(var c=0;c<a.length;c++)if(b.id===a[c].id){b.values=a[c].values,a.splice(c,1);break}}),c.data.targets=c.data.targets.concat(a)),c.updateTargets(c.data.targets),c.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0}),b.done&&b.done()},i.loadFromArgs=function(a){var b=this;a.data?b.load(b.convertDataToTargets(a.data),a):a.url?b.convertUrlToData(a.url,a.mimeType,a.headers,a.keys,function(c){b.load(b.convertDataToTargets(c),a)}):a.json?b.load(b.convertDataToTargets(b.convertJsonToData(a.json,a.keys)),a):a.rows?b.load(b.convertDataToTargets(b.convertRowsToData(a.rows)),a):a.columns?b.load(b.convertDataToTargets(b.convertColumnsToData(a.columns)),a):b.load(null,a)},i.unload=function(a,b){var c=this;return b||(b=function(){}),a=a.filter(function(a){return c.hasTarget(c.data.targets,a)}),a&&0!==a.length?(c.svg.selectAll(a.map(function(a){return c.selectorTarget(a)})).transition().style("opacity",0).remove().call(c.endall,b),void a.forEach(function(a){c.withoutFadeIn[a]=!1,c.legend&&c.legend.selectAll("."+l.legendItem+c.getTargetSelectorSuffix(a)).remove(),c.data.targets=c.data.targets.filter(function(b){return b.id!==a})})):void b()},i.categoryName=function(a){var b=this.config;return a<b.axis_x_categories.length?b.axis_x_categories[a]:a},i.initEventRect=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.eventRects).style("fill-opacity",0)},i.redrawEventRect=function(){var a,b,c=this,d=c.config,e=c.isMultipleX(),f=c.main.select("."+l.eventRects).style("cursor",d.zoom_enabled?d.axis_rotated?"ns-resize":"ew-resize":null).classed(l.eventRectsMultiple,e).classed(l.eventRectsSingle,!e);f.selectAll("."+l.eventRect).remove(),c.eventRect=f.selectAll("."+l.eventRect),e?(a=c.eventRect.data([0]),c.generateEventRectsForMultipleXs(a.enter()),c.updateEventRect(a)):(b=c.getMaxDataCountTarget(c.data.targets),f.datum(b?b.values:[]),c.eventRect=f.selectAll("."+l.eventRect),a=c.eventRect.data(function(a){return a}),c.generateEventRectsForSingleX(a.enter()),c.updateEventRect(a),a.exit().remove())},i.updateEventRect=function(a){var b,c,d,e,f,g,h=this,i=h.config;a=a||h.eventRect.data(function(a){return a}),h.isMultipleX()?(b=0,c=0,d=h.width,e=h.height):(!h.isCustomX()&&!h.isTimeSeries()||h.isCategorized()?(f=h.getEventRectWidth(),g=function(a){return h.x(a.x)-f/2}):(h.updateXs(),f=function(a){var b=h.getPrevX(a.index),c=h.getNextX(a.index);return null===b&&null===c?i.axis_rotated?h.height:h.width:(null===b&&(b=h.x.domain()[0]),null===c&&(c=h.x.domain()[1]),Math.max(0,(h.x(c)-h.x(b))/2))},g=function(a){var b=h.getPrevX(a.index),c=h.getNextX(a.index),d=h.data.xs[a.id][a.index];return null===b&&null===c?0:(null===b&&(b=h.x.domain()[0]),(h.x(d)+h.x(b))/2)}),b=i.axis_rotated?0:g,c=i.axis_rotated?g:0,d=i.axis_rotated?h.width:f,e=i.axis_rotated?f:h.height),a.attr("class",h.classEvent.bind(h)).attr("x",b).attr("y",c).attr("width",d).attr("height",e)},i.generateEventRectsForSingleX=function(a){var b=this,c=b.d3,d=b.config;a.append("rect").attr("class",b.classEvent.bind(b)).style("cursor",d.data_selection_enabled&&d.data_selection_grouped?"pointer":null).on("mouseover",function(a){var c=a.index;b.dragging||b.flowing||b.hasArcType()||(d.point_focus_expand_enabled&&b.expandCircles(c,null,!0),b.expandBars(c,null,!0),b.main.selectAll("."+l.shape+"-"+c).each(function(a){d.data_onmouseover.call(b.api,a)}))}).on("mouseout",function(a){var c=a.index;b.config&&(b.hasArcType()||(b.hideXGridFocus(),b.hideTooltip(),b.unexpandCircles(),b.unexpandBars(),b.main.selectAll("."+l.shape+"-"+c).each(function(a){d.data_onmouseout.call(b.api,a)})))}).on("mousemove",function(a){var e,f=a.index,g=b.svg.select("."+l.eventRect+"-"+f);b.dragging||b.flowing||b.hasArcType()||(b.isStepType(a)&&"step-after"===b.config.line_step_type&&c.mouse(this)[0]<b.x(b.getXValue(a.id,f))&&(f-=1),e=b.filterTargetsToShow(b.data.targets).map(function(a){return b.addName(b.getValueOnIndex(a.values,f))}),d.tooltip_grouped&&(b.showTooltip(e,this),b.showXGridFocus(e)),(!d.tooltip_grouped||d.data_selection_enabled&&!d.data_selection_grouped)&&b.main.selectAll("."+l.shape+"-"+f).each(function(){c.select(this).classed(l.EXPANDED,!0),d.data_selection_enabled&&g.style("cursor",d.data_selection_grouped?"pointer":null),d.tooltip_grouped||(b.hideXGridFocus(),b.hideTooltip(),d.data_selection_grouped||(b.unexpandCircles(f),b.unexpandBars(f)))}).filter(function(a){return b.isWithinShape(this,a)}).each(function(a){d.data_selection_enabled&&(d.data_selection_grouped||d.data_selection_isselectable(a))&&g.style("cursor","pointer"),d.tooltip_grouped||(b.showTooltip([a],this),b.showXGridFocus([a]),d.point_focus_expand_enabled&&b.expandCircles(f,a.id,!0),b.expandBars(f,a.id,!0))}))}).on("click",function(a){var e=a.index;if(!b.hasArcType()&&b.toggleShape){if(b.cancelClick)return void(b.cancelClick=!1);b.isStepType(a)&&"step-after"===d.line_step_type&&c.mouse(this)[0]<b.x(b.getXValue(a.id,e))&&(e-=1),b.main.selectAll("."+l.shape+"-"+e).each(function(a){(d.data_selection_grouped||b.isWithinShape(this,a))&&(b.toggleShape(this,a,e),b.config.data_onclick.call(b.api,a,this))})}}).call(d.data_selection_draggable&&b.drag?c.behavior.drag().origin(Object).on("drag",function(){b.drag(c.mouse(this))}).on("dragstart",function(){b.dragstart(c.mouse(this))}).on("dragend",function(){b.dragend()}):function(){})},i.generateEventRectsForMultipleXs=function(a){function b(){c.svg.select("."+l.eventRect).style("cursor",null),c.hideXGridFocus(),c.hideTooltip(),c.unexpandCircles(),c.unexpandBars()}var c=this,d=c.d3,e=c.config;a.append("rect").attr("x",0).attr("y",0).attr("width",c.width).attr("height",c.height).attr("class",l.eventRect).on("mouseout",function(){c.config&&(c.hasArcType()||b())}).on("mousemove",function(){var a,f,g,h,i=c.filterTargetsToShow(c.data.targets);if(!c.dragging&&!c.hasArcType(i)){if(a=d.mouse(this),f=c.findClosestFromTargets(i,a),!c.mouseover||f&&f.id===c.mouseover.id||(e.data_onmouseout.call(c.api,c.mouseover),c.mouseover=void 0),!f)return void b();g=c.isScatterType(f)||!e.tooltip_grouped?[f]:c.filterByX(i,f.x),h=g.map(function(a){return c.addName(a)}),c.showTooltip(h,this),e.point_focus_expand_enabled&&c.expandCircles(f.index,f.id,!0),c.expandBars(f.index,f.id,!0),c.showXGridFocus(h),(c.isBarType(f.id)||c.dist(f,a)<e.point_sensitivity)&&(c.svg.select("."+l.eventRect).style("cursor","pointer"),c.mouseover||(e.data_onmouseover.call(c.api,f),c.mouseover=f))}}).on("click",function(){var a,b,f=c.filterTargetsToShow(c.data.targets);c.hasArcType(f)||(a=d.mouse(this),b=c.findClosestFromTargets(f,a),b&&(c.isBarType(b.id)||c.dist(b,a)<e.point_sensitivity)&&c.main.selectAll("."+l.shapes+c.getTargetSelectorSuffix(b.id)).selectAll("."+l.shape+"-"+b.index).each(function(){(e.data_selection_grouped||c.isWithinShape(this,b))&&(c.toggleShape(this,b,b.index),c.config.data_onclick.call(c.api,b,this))}))}).call(e.data_selection_draggable&&c.drag?d.behavior.drag().origin(Object).on("drag",function(){c.drag(d.mouse(this))}).on("dragstart",function(){c.dragstart(d.mouse(this))}).on("dragend",function(){c.dragend()}):function(){})},i.dispatchEvent=function(b,c,d){var e=this,f="."+l.eventRect+(e.isMultipleX()?"":"-"+c),g=e.main.select(f).node(),h=g.getBoundingClientRect(),i=h.left+(d?d[0]:0),j=h.top+(d?d[1]:0),k=document.createEvent("MouseEvents");k.initMouseEvent(b,!0,!0,a,0,i,j,i,j,!1,!1,!1,!1,0,null),g.dispatchEvent(k)},i.getCurrentWidth=function(){var a=this,b=a.config;return b.size_width?b.size_width:a.getParentWidth()},i.getCurrentHeight=function(){var a=this,b=a.config,c=b.size_height?b.size_height:a.getParentHeight();return c>0?c:320/(a.hasType("gauge")&&!b.gauge_fullCircle?2:1)},i.getCurrentPaddingTop=function(){var a=this,b=a.config,c=m(b.padding_top)?b.padding_top:0;return a.title&&a.title.node()&&(c+=a.getTitlePadding()),c},i.getCurrentPaddingBottom=function(){var a=this.config;return m(a.padding_bottom)?a.padding_bottom:0},i.getCurrentPaddingLeft=function(a){var b=this,c=b.config;return m(c.padding_left)?c.padding_left:c.axis_rotated?c.axis_x_show?Math.max(r(b.getAxisWidthByAxisId("x",a)),40):1:!c.axis_y_show||c.axis_y_inner?b.axis.getYAxisLabelPosition().isOuter?30:1:r(b.getAxisWidthByAxisId("y",a))},i.getCurrentPaddingRight=function(){var a=this,b=a.config,c=10,d=a.isLegendRight?a.getLegendWidth()+20:0;return m(b.padding_right)?b.padding_right+1:b.axis_rotated?c+d:!b.axis_y2_show||b.axis_y2_inner?2+d+(a.axis.getY2AxisLabelPosition().isOuter?20:0):r(a.getAxisWidthByAxisId("y2"))+d},i.getParentRectValue=function(a){for(var b,c=this.selectChart.node();c&&"BODY"!==c.tagName;){try{b=c.getBoundingClientRect()[a]}catch(d){"width"===a&&(b=c.offsetWidth)}if(b)break;c=c.parentNode}return b},i.getParentWidth=function(){return this.getParentRectValue("width")},i.getParentHeight=function(){var a=this.selectChart.style("height");return a.indexOf("px")>0?+a.replace("px",""):0},i.getSvgLeft=function(a){var b=this,c=b.config,d=c.axis_rotated||!c.axis_rotated&&!c.axis_y_inner,e=c.axis_rotated?l.axisX:l.axisY,f=b.main.select("."+e).node(),g=f&&d?f.getBoundingClientRect():{right:0},h=b.selectChart.node().getBoundingClientRect(),i=b.hasArcType(),j=g.right-h.left-(i?0:b.getCurrentPaddingLeft(a));return j>0?j:0},i.getAxisWidthByAxisId=function(a,b){var c=this,d=c.axis.getLabelPositionById(a);return c.axis.getMaxTickWidth(a,b)+(d.isInner?20:40)},i.getHorizontalAxisHeight=function(a){var b=this,c=b.config,d=30;return"x"!==a||c.axis_x_show?"x"===a&&c.axis_x_height?c.axis_x_height:"y"!==a||c.axis_y_show?"y2"!==a||c.axis_y2_show?("x"===a&&!c.axis_rotated&&c.axis_x_tick_rotate&&(d=30+b.axis.getMaxTickWidth(a)*Math.cos(Math.PI*(90-c.axis_x_tick_rotate)/180)),"y"===a&&c.axis_rotated&&c.axis_y_tick_rotate&&(d=30+b.axis.getMaxTickWidth(a)*Math.cos(Math.PI*(90-c.axis_y_tick_rotate)/180)),d+(b.axis.getLabelPositionById(a).isInner?0:10)+("y2"===a?-10:0)):b.rotated_padding_top:!c.legend_show||b.isLegendRight||b.isLegendInset?1:10:8},i.getEventRectWidth=function(){return Math.max(0,this.xAxis.tickInterval())},i.getShapeIndices=function(a){var b,c,d=this,e=d.config,f={},g=0;return d.filterTargetsToShow(d.data.targets.filter(a,d)).forEach(function(a){for(b=0;b<e.data_groups.length;b++)if(!(e.data_groups[b].indexOf(a.id)<0))for(c=0;c<e.data_groups[b].length;c++)if(e.data_groups[b][c]in f){f[a.id]=f[e.data_groups[b][c]];break}p(f[a.id])&&(f[a.id]=g++)}),f.__max__=g-1,f},i.getShapeX=function(a,b,c,d){var e=this,f=d?e.subX:e.x;return function(d){var e=d.id in c?c[d.id]:0;return d.x||0===d.x?f(d.x)-a*(b/2-e):0}},i.getShapeY=function(a){var b=this;return function(c){var d=a?b.getSubYScale(c.id):b.getYScale(c.id);return d(c.value)}},i.getShapeOffset=function(a,b,c){var d=this,e=d.orderTargets(d.filterTargetsToShow(d.data.targets.filter(a,d))),f=e.map(function(a){return a.id});return function(a,g){var h=c?d.getSubYScale(a.id):d.getYScale(a.id),i=h(0),j=i;return e.forEach(function(c){var e=d.isStepType(a)?d.convertValuesToStep(c.values):c.values;c.id!==a.id&&b[c.id]===b[a.id]&&f.indexOf(c.id)<f.indexOf(a.id)&&("undefined"!=typeof e[g]&&+e[g].x===+a.x||(g=-1,e.forEach(function(b,c){b.x===a.x&&(g=c)})),g in e&&e[g].value*a.value>=0&&(j+=h(e[g].value)-i))}),j}},i.isWithinShape=function(a,b){var c,d=this,e=d.d3.select(a);return d.isTargetToShow(b.id)?"circle"===a.nodeName?c=d.isStepType(b)?d.isWithinStep(a,d.getYScale(b.id)(b.value)):d.isWithinCircle(a,1.5*d.pointSelectR(b)):"path"===a.nodeName&&(c=e.classed(l.bar)?d.isWithinBar(a):!0):c=!1,c},i.getInterpolate=function(a){var b=this,c=b.isInterpolationType(b.config.spline_interpolation_type)?b.config.spline_interpolation_type:"cardinal";return b.isSplineType(a)?c:b.isStepType(a)?b.config.line_step_type:"linear"},i.initLine=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.chartLines)},i.updateTargetsForLine=function(a){var b,c,d=this,e=d.config,f=d.classChartLine.bind(d),g=d.classLines.bind(d),h=d.classAreas.bind(d),i=d.classCircles.bind(d),j=d.classFocus.bind(d);b=d.main.select("."+l.chartLines).selectAll("."+l.chartLine).data(a).attr("class",function(a){return f(a)+j(a)}),c=b.enter().append("g").attr("class",f).style("opacity",0).style("pointer-events","none"),c.append("g").attr("class",g),c.append("g").attr("class",h),c.append("g").attr("class",function(a){return d.generateClass(l.selectedCircles,a.id)}),c.append("g").attr("class",i).style("cursor",function(a){return e.data_selection_isselectable(a)?"pointer":null}),a.forEach(function(a){d.main.selectAll("."+l.selectedCircles+d.getTargetSelectorSuffix(a.id)).selectAll("."+l.selectedCircle).each(function(b){b.value=a.values[b.index].value})})},i.updateLine=function(a){var b=this;b.mainLine=b.main.selectAll("."+l.lines).selectAll("."+l.line).data(b.lineData.bind(b)),b.mainLine.enter().append("path").attr("class",b.classLine.bind(b)).style("stroke",b.color),b.mainLine.style("opacity",b.initialOpacity.bind(b)).style("shape-rendering",function(a){return b.isStepType(a)?"crispEdges":""}).attr("transform",null),b.mainLine.exit().transition().duration(a).style("opacity",0).remove()},i.redrawLine=function(a,b){return[(b?this.mainLine.transition(Math.random().toString()):this.mainLine).attr("d",a).style("stroke",this.color).style("opacity",1)]},i.generateDrawLine=function(a,b){var c=this,d=c.config,e=c.d3.svg.line(),f=c.generateGetLinePoints(a,b),g=b?c.getSubYScale:c.getYScale,h=function(a){return(b?c.subxx:c.xx).call(c,a)},i=function(a,b){return d.data_groups.length>0?f(a,b)[0][1]:g.call(c,a.id)(a.value)};return e=d.axis_rotated?e.x(i).y(h):e.x(h).y(i),d.line_connectNull||(e=e.defined(function(a){return null!=a.value})),function(a){var f,h=d.line_connectNull?c.filterRemoveNull(a.values):a.values,i=b?c.x:c.subX,j=g.call(c,a.id),k=0,l=0;return c.isLineType(a)?d.data_regions[a.id]?f=c.lineWithRegions(h,i,j,d.data_regions[a.id]):(c.isStepType(a)&&(h=c.convertValuesToStep(h)),f=e.interpolate(c.getInterpolate(a))(h)):(h[0]&&(k=i(h[0].x),l=j(h[0].value)),f=d.axis_rotated?"M "+l+" "+k:"M "+k+" "+l),f?f:"M 0 0"}},i.generateGetLinePoints=function(a,b){var c=this,d=c.config,e=a.__max__+1,f=c.getShapeX(0,e,a,!!b),g=c.getShapeY(!!b),h=c.getShapeOffset(c.isLineType,a,!!b),i=b?c.getSubYScale:c.getYScale;return function(a,b){var e=i.call(c,a.id)(0),j=h(a,b)||e,k=f(a),l=g(a);return d.axis_rotated&&(0<a.value&&e>l||a.value<0&&l>e)&&(l=e),[[k,l-(e-j)],[k,l-(e-j)],[k,l-(e-j)],[k,l-(e-j)]]}},i.lineWithRegions=function(a,b,c,d){function e(a,b){var c;for(c=0;c<b.length;c++)if(b[c].start<a&&a<=b[c].end)return!0;return!1}function f(a){return"M"+a[0][0]+" "+a[0][1]+" "+a[1][0]+" "+a[1][1]}var g,h,i,j,k,l,m,n,o,r,s,t,u=this,v=u.config,w=-1,x="M",y=u.isCategorized()?.5:0,z=[];if(q(d))for(g=0;g<d.length;g++)z[g]={},p(d[g].start)?z[g].start=a[0].x:z[g].start=u.isTimeSeries()?u.parseDate(d[g].start):d[g].start,p(d[g].end)?z[g].end=a[a.length-1].x:z[g].end=u.isTimeSeries()?u.parseDate(d[g].end):d[g].end;for(s=v.axis_rotated?function(a){return c(a.value)}:function(a){return b(a.x)},t=v.axis_rotated?function(a){return b(a.x)}:function(a){return c(a.value)},i=u.isTimeSeries()?function(a,d,e,g){var h,i=a.x.getTime(),j=d.x-a.x,l=new Date(i+j*e),m=new Date(i+j*(e+g));return h=v.axis_rotated?[[c(k(e)),b(l)],[c(k(e+g)),b(m)]]:[[b(l),c(k(e))],[b(m),c(k(e+g))]],f(h)}:function(a,d,e,g){var h;return h=v.axis_rotated?[[c(k(e),!0),b(j(e))],[c(k(e+g),!0),b(j(e+g))]]:[[b(j(e),!0),c(k(e))],[b(j(e+g),!0),c(k(e+g))]],f(h)},g=0;g<a.length;g++){if(p(z)||!e(a[g].x,z))x+=" "+s(a[g])+" "+t(a[g]);else for(j=u.getScale(a[g-1].x+y,a[g].x+y,u.isTimeSeries()),k=u.getScale(a[g-1].value,a[g].value),l=b(a[g].x)-b(a[g-1].x),m=c(a[g].value)-c(a[g-1].value),n=Math.sqrt(Math.pow(l,2)+Math.pow(m,2)),o=2/n,r=2*o,h=o;1>=h;h+=r)x+=i(a[g-1],a[g],h,o);w=a[g].x}return x},i.updateArea=function(a){var b=this,c=b.d3;b.mainArea=b.main.selectAll("."+l.areas).selectAll("."+l.area).data(b.lineData.bind(b)),b.mainArea.enter().append("path").attr("class",b.classArea.bind(b)).style("fill",b.color).style("opacity",function(){return b.orgAreaOpacity=+c.select(this).style("opacity"),0}),b.mainArea.style("opacity",b.orgAreaOpacity),b.mainArea.exit().transition().duration(a).style("opacity",0).remove()},i.redrawArea=function(a,b){return[(b?this.mainArea.transition(Math.random().toString()):this.mainArea).attr("d",a).style("fill",this.color).style("opacity",this.orgAreaOpacity)]},i.generateDrawArea=function(a,b){var c=this,d=c.config,e=c.d3.svg.area(),f=c.generateGetAreaPoints(a,b),g=b?c.getSubYScale:c.getYScale,h=function(a){return(b?c.subxx:c.xx).call(c,a)},i=function(a,b){return d.data_groups.length>0?f(a,b)[0][1]:g.call(c,a.id)(c.getAreaBaseValue(a.id))},j=function(a,b){return d.data_groups.length>0?f(a,b)[1][1]:g.call(c,a.id)(a.value)};return e=d.axis_rotated?e.x0(i).x1(j).y(h):e.x(h).y0(d.area_above?0:i).y1(j),d.line_connectNull||(e=e.defined(function(a){return null!==a.value})),function(a){var b,f=d.line_connectNull?c.filterRemoveNull(a.values):a.values,g=0,h=0;return c.isAreaType(a)?(c.isStepType(a)&&(f=c.convertValuesToStep(f)),b=e.interpolate(c.getInterpolate(a))(f)):(f[0]&&(g=c.x(f[0].x),h=c.getYScale(a.id)(f[0].value)),b=d.axis_rotated?"M "+h+" "+g:"M "+g+" "+h),b?b:"M 0 0"}},i.getAreaBaseValue=function(){return 0},i.generateGetAreaPoints=function(a,b){var c=this,d=c.config,e=a.__max__+1,f=c.getShapeX(0,e,a,!!b),g=c.getShapeY(!!b),h=c.getShapeOffset(c.isAreaType,a,!!b),i=b?c.getSubYScale:c.getYScale;return function(a,b){var e=i.call(c,a.id)(0),j=h(a,b)||e,k=f(a),l=g(a);return d.axis_rotated&&(0<a.value&&e>l||a.value<0&&l>e)&&(l=e),[[k,j],[k,l-(e-j)],[k,l-(e-j)],[k,j]]}},i.updateCircle=function(){var a=this;a.mainCircle=a.main.selectAll("."+l.circles).selectAll("."+l.circle).data(a.lineOrScatterData.bind(a)),a.mainCircle.enter().append("circle").attr("class",a.classCircle.bind(a)).attr("r",a.pointR.bind(a)).style("fill",a.color),a.mainCircle.style("opacity",a.initialOpacityForCircle.bind(a)),a.mainCircle.exit().remove()},i.redrawCircle=function(a,b,c){var d=this.main.selectAll("."+l.selectedCircle);return[(c?this.mainCircle.transition(Math.random().toString()):this.mainCircle).style("opacity",this.opacityForCircle.bind(this)).style("fill",this.color).attr("cx",a).attr("cy",b),(c?d.transition(Math.random().toString()):d).attr("cx",a).attr("cy",b)]},i.circleX=function(a){return a.x||0===a.x?this.x(a.x):null},i.updateCircleY=function(){var a,b,c=this;c.config.data_groups.length>0?(a=c.getShapeIndices(c.isLineType),b=c.generateGetLinePoints(a),c.circleY=function(a,c){return b(a,c)[0][1]}):c.circleY=function(a){return c.getYScale(a.id)(a.value)}},i.getCircles=function(a,b){var c=this;return(b?c.main.selectAll("."+l.circles+c.getTargetSelectorSuffix(b)):c.main).selectAll("."+l.circle+(m(a)?"-"+a:""))},i.expandCircles=function(a,b,c){var d=this,e=d.pointExpandedR.bind(d);c&&d.unexpandCircles(),d.getCircles(a,b).classed(l.EXPANDED,!0).attr("r",e)},i.unexpandCircles=function(a){var b=this,c=b.pointR.bind(b);b.getCircles(a).filter(function(){return b.d3.select(this).classed(l.EXPANDED)}).classed(l.EXPANDED,!1).attr("r",c)},i.pointR=function(a){var b=this,c=b.config;return b.isStepType(a)?0:n(c.point_r)?c.point_r(a):c.point_r;
},i.pointExpandedR=function(a){var b=this,c=b.config;return c.point_focus_expand_enabled?c.point_focus_expand_r?c.point_focus_expand_r:1.75*b.pointR(a):b.pointR(a)},i.pointSelectR=function(a){var b=this,c=b.config;return n(c.point_select_r)?c.point_select_r(a):c.point_select_r?c.point_select_r:4*b.pointR(a)},i.isWithinCircle=function(a,b){var c=this.d3,d=c.mouse(a),e=c.select(a),f=+e.attr("cx"),g=+e.attr("cy");return Math.sqrt(Math.pow(f-d[0],2)+Math.pow(g-d[1],2))<b},i.isWithinStep=function(a,b){return Math.abs(b-this.d3.mouse(a)[1])<30},i.initBar=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.chartBars)},i.updateTargetsForBar=function(a){var b,c,d=this,e=d.config,f=d.classChartBar.bind(d),g=d.classBars.bind(d),h=d.classFocus.bind(d);b=d.main.select("."+l.chartBars).selectAll("."+l.chartBar).data(a).attr("class",function(a){return f(a)+h(a)}),c=b.enter().append("g").attr("class",f).style("opacity",0).style("pointer-events","none"),c.append("g").attr("class",g).style("cursor",function(a){return e.data_selection_isselectable(a)?"pointer":null})},i.updateBar=function(a){var b=this,c=b.barData.bind(b),d=b.classBar.bind(b),e=b.initialOpacity.bind(b),f=function(a){return b.color(a.id)};b.mainBar=b.main.selectAll("."+l.bars).selectAll("."+l.bar).data(c),b.mainBar.enter().append("path").attr("class",d).style("stroke",f).style("fill",f),b.mainBar.style("opacity",e),b.mainBar.exit().transition().duration(a).style("opacity",0).remove()},i.redrawBar=function(a,b){return[(b?this.mainBar.transition(Math.random().toString()):this.mainBar).attr("d",a).style("fill",this.color).style("opacity",1)]},i.getBarW=function(a,b){var c=this,d=c.config,e="number"==typeof d.bar_width?d.bar_width:b?a.tickInterval()*d.bar_width_ratio/b:0;return d.bar_width_max&&e>d.bar_width_max?d.bar_width_max:e},i.getBars=function(a,b){var c=this;return(b?c.main.selectAll("."+l.bars+c.getTargetSelectorSuffix(b)):c.main).selectAll("."+l.bar+(m(a)?"-"+a:""))},i.expandBars=function(a,b,c){var d=this;c&&d.unexpandBars(),d.getBars(a,b).classed(l.EXPANDED,!0)},i.unexpandBars=function(a){var b=this;b.getBars(a).classed(l.EXPANDED,!1)},i.generateDrawBar=function(a,b){var c=this,d=c.config,e=c.generateGetBarPoints(a,b);return function(a,b){var c=e(a,b),f=d.axis_rotated?1:0,g=d.axis_rotated?0:1,h="M "+c[0][f]+","+c[0][g]+" L"+c[1][f]+","+c[1][g]+" L"+c[2][f]+","+c[2][g]+" L"+c[3][f]+","+c[3][g]+" z";return h}},i.generateGetBarPoints=function(a,b){var c=this,d=b?c.subXAxis:c.xAxis,e=a.__max__+1,f=c.getBarW(d,e),g=c.getShapeX(f,e,a,!!b),h=c.getShapeY(!!b),i=c.getShapeOffset(c.isBarType,a,!!b),j=b?c.getSubYScale:c.getYScale;return function(a,b){var d=j.call(c,a.id)(0),e=i(a,b)||d,k=g(a),l=h(a);return c.config.axis_rotated&&(0<a.value&&d>l||a.value<0&&l>d)&&(l=d),[[k,e],[k,l-(d-e)],[k+f,l-(d-e)],[k+f,e]]}},i.isWithinBar=function(a){var b=this.d3.mouse(a),c=a.getBoundingClientRect(),d=a.pathSegList.getItem(0),e=a.pathSegList.getItem(1),f=Math.min(d.x,e.x),g=Math.min(d.y,e.y),h=c.width,i=c.height,j=2,k=f-j,l=f+h+j,m=g+i+j,n=g-j;return k<b[0]&&b[0]<l&&n<b[1]&&b[1]<m},i.initText=function(){var a=this;a.main.select("."+l.chart).append("g").attr("class",l.chartTexts),a.mainText=a.d3.selectAll([])},i.updateTargetsForText=function(a){var b,c,d=this,e=d.classChartText.bind(d),f=d.classTexts.bind(d),g=d.classFocus.bind(d);b=d.main.select("."+l.chartTexts).selectAll("."+l.chartText).data(a).attr("class",function(a){return e(a)+g(a)}),c=b.enter().append("g").attr("class",e).style("opacity",0).style("pointer-events","none"),c.append("g").attr("class",f)},i.updateText=function(a){var b=this,c=b.config,d=b.barOrLineData.bind(b),e=b.classText.bind(b);b.mainText=b.main.selectAll("."+l.texts).selectAll("."+l.text).data(d),b.mainText.enter().append("text").attr("class",e).attr("text-anchor",function(a){return c.axis_rotated?a.value<0?"end":"start":"middle"}).style("stroke","none").style("fill",function(a){return b.color(a)}).style("fill-opacity",0),b.mainText.text(function(a,c,d){return b.dataLabelFormat(a.id)(a.value,a.id,c,d)}),b.mainText.exit().transition().duration(a).style("fill-opacity",0).remove()},i.redrawText=function(a,b,c,d){return[(d?this.mainText.transition():this.mainText).attr("x",a).attr("y",b).style("fill",this.color).style("fill-opacity",c?0:this.opacityForText.bind(this))]},i.getTextRect=function(a,b,c){var d,e=this.d3.select("body").append("div").classed("c3",!0),f=e.append("svg").style("visibility","hidden").style("position","fixed").style("top",0).style("left",0),g=this.d3.select(c).style("font");return f.selectAll(".dummy").data([a]).enter().append("text").classed(b?b:"",!0).style("font",g).text(a).each(function(){d=this.getBoundingClientRect()}),e.remove(),d},i.generateXYForText=function(a,b,c,d){var e=this,f=e.generateGetAreaPoints(a,!1),g=e.generateGetBarPoints(b,!1),h=e.generateGetLinePoints(c,!1),i=d?e.getXForText:e.getYForText;return function(a,b){var c=e.isAreaType(a)?f:e.isBarType(a)?g:h;return i.call(e,c(a,b),a,this)}},i.getXForText=function(a,b,c){var d,e,f=this,g=c.getBoundingClientRect();return f.config.axis_rotated?(e=f.isBarType(b)?4:6,d=a[2][1]+e*(b.value<0?-1:1)):d=f.hasType("bar")?(a[2][0]+a[0][0])/2:a[0][0],null===b.value&&(d>f.width?d=f.width-g.width:0>d&&(d=4)),d},i.getYForText=function(a,b,c){var d,e=this,f=c.getBoundingClientRect();return e.config.axis_rotated?d=(a[0][0]+a[2][0]+.6*f.height)/2:(d=a[2][1],b.value<0||0===b.value&&!e.hasPositiveValue?(d+=f.height,e.isBarType(b)&&e.isSafari()?d-=3:!e.isBarType(b)&&e.isChrome()&&(d+=3)):d+=e.isBarType(b)?-3:-6),null!==b.value||e.config.axis_rotated||(d<f.height?d=f.height:d>this.height&&(d=this.height-4)),d},i.setTargetType=function(a,b){var c=this,d=c.config;c.mapToTargetIds(a).forEach(function(a){c.withoutFadeIn[a]=b===d.data_types[a],d.data_types[a]=b}),a||(d.data_type=b)},i.hasType=function(a,b){var c=this,d=c.config.data_types,e=!1;return b=b||c.data.targets,b&&b.length?b.forEach(function(b){var c=d[b.id];(c&&c.indexOf(a)>=0||!c&&"line"===a)&&(e=!0)}):Object.keys(d).length?Object.keys(d).forEach(function(b){d[b]===a&&(e=!0)}):e=c.config.data_type===a,e},i.hasArcType=function(a){return this.hasType("pie",a)||this.hasType("donut",a)||this.hasType("gauge",a)},i.isLineType=function(a){var b=this.config,c=o(a)?a:a.id;return!b.data_types[c]||["line","spline","area","area-spline","step","area-step"].indexOf(b.data_types[c])>=0},i.isStepType=function(a){var b=o(a)?a:a.id;return["step","area-step"].indexOf(this.config.data_types[b])>=0},i.isSplineType=function(a){var b=o(a)?a:a.id;return["spline","area-spline"].indexOf(this.config.data_types[b])>=0},i.isAreaType=function(a){var b=o(a)?a:a.id;return["area","area-spline","area-step"].indexOf(this.config.data_types[b])>=0},i.isBarType=function(a){var b=o(a)?a:a.id;return"bar"===this.config.data_types[b]},i.isScatterType=function(a){var b=o(a)?a:a.id;return"scatter"===this.config.data_types[b]},i.isPieType=function(a){var b=o(a)?a:a.id;return"pie"===this.config.data_types[b]},i.isGaugeType=function(a){var b=o(a)?a:a.id;return"gauge"===this.config.data_types[b]},i.isDonutType=function(a){var b=o(a)?a:a.id;return"donut"===this.config.data_types[b]},i.isArcType=function(a){return this.isPieType(a)||this.isDonutType(a)||this.isGaugeType(a)},i.lineData=function(a){return this.isLineType(a)?[a]:[]},i.arcData=function(a){return this.isArcType(a.data)?[a]:[]},i.barData=function(a){return this.isBarType(a)?a.values:[]},i.lineOrScatterData=function(a){return this.isLineType(a)||this.isScatterType(a)?a.values:[]},i.barOrLineData=function(a){return this.isBarType(a)||this.isLineType(a)?a.values:[]},i.isInterpolationType=function(a){return["linear","linear-closed","basis","basis-open","basis-closed","bundle","cardinal","cardinal-open","cardinal-closed","monotone"].indexOf(a)>=0},i.initGrid=function(){var a=this,b=a.config,c=a.d3;a.grid=a.main.append("g").attr("clip-path",a.clipPathForGrid).attr("class",l.grid),b.grid_x_show&&a.grid.append("g").attr("class",l.xgrids),b.grid_y_show&&a.grid.append("g").attr("class",l.ygrids),b.grid_focus_show&&a.grid.append("g").attr("class",l.xgridFocus).append("line").attr("class",l.xgridFocus),a.xgrid=c.selectAll([]),b.grid_lines_front||a.initGridLines()},i.initGridLines=function(){var a=this,b=a.d3;a.gridLines=a.main.append("g").attr("clip-path",a.clipPathForGrid).attr("class",l.grid+" "+l.gridLines),a.gridLines.append("g").attr("class",l.xgridLines),a.gridLines.append("g").attr("class",l.ygridLines),a.xgridLines=b.selectAll([])},i.updateXGrid=function(a){var b=this,c=b.config,d=b.d3,e=b.generateGridData(c.grid_x_type,b.x),f=b.isCategorized()?b.xAxis.tickOffset():0;b.xgridAttr=c.axis_rotated?{x1:0,x2:b.width,y1:function(a){return b.x(a)-f},y2:function(a){return b.x(a)-f}}:{x1:function(a){return b.x(a)+f},x2:function(a){return b.x(a)+f},y1:0,y2:b.height},b.xgrid=b.main.select("."+l.xgrids).selectAll("."+l.xgrid).data(e),b.xgrid.enter().append("line").attr("class",l.xgrid),a||b.xgrid.attr(b.xgridAttr).style("opacity",function(){return+d.select(this).attr(c.axis_rotated?"y1":"x1")===(c.axis_rotated?b.height:0)?0:1}),b.xgrid.exit().remove()},i.updateYGrid=function(){var a=this,b=a.config,c=a.yAxis.tickValues()||a.y.ticks(b.grid_y_ticks);a.ygrid=a.main.select("."+l.ygrids).selectAll("."+l.ygrid).data(c),a.ygrid.enter().append("line").attr("class",l.ygrid),a.ygrid.attr("x1",b.axis_rotated?a.y:0).attr("x2",b.axis_rotated?a.y:a.width).attr("y1",b.axis_rotated?0:a.y).attr("y2",b.axis_rotated?a.height:a.y),a.ygrid.exit().remove(),a.smoothLines(a.ygrid,"grid")},i.gridTextAnchor=function(a){return a.position?a.position:"end"},i.gridTextDx=function(a){return"start"===a.position?4:"middle"===a.position?0:-4},i.xGridTextX=function(a){return"start"===a.position?-this.height:"middle"===a.position?-this.height/2:0},i.yGridTextX=function(a){return"start"===a.position?0:"middle"===a.position?this.width/2:this.width},i.updateGrid=function(a){var b,c,d,e=this,f=e.main,g=e.config;e.grid.style("visibility",e.hasArcType()?"hidden":"visible"),f.select("line."+l.xgridFocus).style("visibility","hidden"),g.grid_x_show&&e.updateXGrid(),e.xgridLines=f.select("."+l.xgridLines).selectAll("."+l.xgridLine).data(g.grid_x_lines),b=e.xgridLines.enter().append("g").attr("class",function(a){return l.xgridLine+(a["class"]?" "+a["class"]:"")}),b.append("line").style("opacity",0),b.append("text").attr("text-anchor",e.gridTextAnchor).attr("transform",g.axis_rotated?"":"rotate(-90)").attr("dx",e.gridTextDx).attr("dy",-5).style("opacity",0),e.xgridLines.exit().transition().duration(a).style("opacity",0).remove(),g.grid_y_show&&e.updateYGrid(),e.ygridLines=f.select("."+l.ygridLines).selectAll("."+l.ygridLine).data(g.grid_y_lines),c=e.ygridLines.enter().append("g").attr("class",function(a){return l.ygridLine+(a["class"]?" "+a["class"]:"")}),c.append("line").style("opacity",0),c.append("text").attr("text-anchor",e.gridTextAnchor).attr("transform",g.axis_rotated?"rotate(-90)":"").attr("dx",e.gridTextDx).attr("dy",-5).style("opacity",0),d=e.yv.bind(e),e.ygridLines.select("line").transition().duration(a).attr("x1",g.axis_rotated?d:0).attr("x2",g.axis_rotated?d:e.width).attr("y1",g.axis_rotated?0:d).attr("y2",g.axis_rotated?e.height:d).style("opacity",1),e.ygridLines.select("text").transition().duration(a).attr("x",g.axis_rotated?e.xGridTextX.bind(e):e.yGridTextX.bind(e)).attr("y",d).text(function(a){return a.text}).style("opacity",1),e.ygridLines.exit().transition().duration(a).style("opacity",0).remove()},i.redrawGrid=function(a){var b=this,c=b.config,d=b.xv.bind(b),e=b.xgridLines.select("line"),f=b.xgridLines.select("text");return[(a?e.transition():e).attr("x1",c.axis_rotated?0:d).attr("x2",c.axis_rotated?b.width:d).attr("y1",c.axis_rotated?d:0).attr("y2",c.axis_rotated?d:b.height).style("opacity",1),(a?f.transition():f).attr("x",c.axis_rotated?b.yGridTextX.bind(b):b.xGridTextX.bind(b)).attr("y",d).text(function(a){return a.text}).style("opacity",1)]},i.showXGridFocus=function(a){var b=this,c=b.config,d=a.filter(function(a){return a&&m(a.value)}),e=b.main.selectAll("line."+l.xgridFocus),f=b.xx.bind(b);c.tooltip_show&&(b.hasType("scatter")||b.hasArcType()||(e.style("visibility","visible").data([d[0]]).attr(c.axis_rotated?"y1":"x1",f).attr(c.axis_rotated?"y2":"x2",f),b.smoothLines(e,"grid")))},i.hideXGridFocus=function(){this.main.select("line."+l.xgridFocus).style("visibility","hidden")},i.updateXgridFocus=function(){var a=this,b=a.config;a.main.select("line."+l.xgridFocus).attr("x1",b.axis_rotated?0:-10).attr("x2",b.axis_rotated?a.width:-10).attr("y1",b.axis_rotated?-10:0).attr("y2",b.axis_rotated?-10:a.height)},i.generateGridData=function(a,b){var c,d,e,f,g=this,h=[],i=g.main.select("."+l.axisX).selectAll(".tick").size();if("year"===a)for(c=g.getXDomain(),d=c[0].getFullYear(),e=c[1].getFullYear(),f=d;e>=f;f++)h.push(new Date(f+"-01-01 00:00:00"));else h=b.ticks(10),h.length>i&&(h=h.filter(function(a){return(""+a).indexOf(".")<0}));return h},i.getGridFilterToRemove=function(a){return a?function(b){var c=!1;return[].concat(a).forEach(function(a){("value"in a&&b.value===a.value||"class"in a&&b["class"]===a["class"])&&(c=!0)}),c}:function(){return!0}},i.removeGridLines=function(a,b){var c=this,d=c.config,e=c.getGridFilterToRemove(a),f=function(a){return!e(a)},g=b?l.xgridLines:l.ygridLines,h=b?l.xgridLine:l.ygridLine;c.main.select("."+g).selectAll("."+h).filter(e).transition().duration(d.transition_duration).style("opacity",0).remove(),b?d.grid_x_lines=d.grid_x_lines.filter(f):d.grid_y_lines=d.grid_y_lines.filter(f)},i.initTooltip=function(){var a,b=this,c=b.config;if(b.tooltip=b.selectChart.style("position","relative").append("div").attr("class",l.tooltipContainer).style("position","absolute").style("pointer-events","none").style("display","none"),c.tooltip_init_show){if(b.isTimeSeries()&&o(c.tooltip_init_x)){for(c.tooltip_init_x=b.parseDate(c.tooltip_init_x),a=0;a<b.data.targets[0].values.length&&b.data.targets[0].values[a].x-c.tooltip_init_x!==0;a++);c.tooltip_init_x=a}b.tooltip.html(c.tooltip_contents.call(b,b.data.targets.map(function(a){return b.addName(a.values[c.tooltip_init_x])}),b.axis.getXAxisTickFormat(),b.getYFormat(b.hasArcType()),b.color)),b.tooltip.style("top",c.tooltip_init_position.top).style("left",c.tooltip_init_position.left).style("display","block")}},i.getTooltipContent=function(a,b,c,d){var e,f,g,h,i,j,k=this,l=k.config,m=l.tooltip_format_title||b,n=l.tooltip_format_name||function(a){return a},o=l.tooltip_format_value||c,p=k.isOrderAsc();if(0===l.data_groups.length)a.sort(function(a,b){var c=a?a.value:null,d=b?b.value:null;return p?c-d:d-c});else{var q=k.orderTargets(k.data.targets).map(function(a){return a.id});a.sort(function(a,b){var c=a?a.value:null,d=b?b.value:null;return c>0&&d>0&&(c=a?q.indexOf(a.id):null,d=b?q.indexOf(b.id):null),p?c-d:d-c})}for(f=0;f<a.length;f++)if(a[f]&&(a[f].value||0===a[f].value)&&(e||(g=y(m?m(a[f].x):a[f].x),e="<table class='"+k.CLASS.tooltip+"'>"+(g||0===g?"<tr><th colspan='2'>"+g+"</th></tr>":"")),h=y(o(a[f].value,a[f].ratio,a[f].id,a[f].index,a)),void 0!==h)){if(null===a[f].name)continue;i=y(n(a[f].name,a[f].ratio,a[f].id,a[f].index)),j=k.levelColor?k.levelColor(a[f].value):d(a[f].id),e+="<tr class='"+k.CLASS.tooltipName+"-"+k.getTargetSelectorSuffix(a[f].id)+"'>",e+="<td class='name'><span style='background-color:"+j+"'></span>"+i+"</td>",e+="<td class='value'>"+h+"</td>",e+="</tr>"}return e+"</table>"},i.tooltipPosition=function(a,b,c,d){var e,f,g,h,i,j=this,k=j.config,l=j.d3,m=j.hasArcType(),n=l.mouse(d);return m?(f=(j.width-(j.isLegendRight?j.getLegendWidth():0))/2+n[0],h=j.height/2+n[1]+20):(e=j.getSvgLeft(!0),k.axis_rotated?(f=e+n[0]+100,g=f+b,i=j.currentWidth-j.getCurrentPaddingRight(),h=j.x(a[0].x)+20):(f=e+j.getCurrentPaddingLeft(!0)+j.x(a[0].x)+20,g=f+b,i=e+j.currentWidth-j.getCurrentPaddingRight(),h=n[1]+15),g>i&&(f-=g-i+20),h+c>j.currentHeight&&(h-=c+30)),0>h&&(h=0),{top:h,left:f}},i.showTooltip=function(a,b){var c,d,e,f=this,g=f.config,h=f.hasArcType(),j=a.filter(function(a){return a&&m(a.value)}),k=g.tooltip_position||i.tooltipPosition;0!==j.length&&g.tooltip_show&&(f.tooltip.html(g.tooltip_contents.call(f,a,f.axis.getXAxisTickFormat(),f.getYFormat(h),f.color)).style("display","block"),c=f.tooltip.property("offsetWidth"),d=f.tooltip.property("offsetHeight"),e=k.call(this,j,c,d,b),f.tooltip.style("top",e.top+"px").style("left",e.left+"px"))},i.hideTooltip=function(){this.tooltip.style("display","none")},i.initLegend=function(){var a=this;return a.legendItemTextBox={},a.legendHasRendered=!1,a.legend=a.svg.append("g").attr("transform",a.getTranslate("legend")),a.config.legend_show?void a.updateLegendWithDefaults():(a.legend.style("visibility","hidden"),void(a.hiddenLegendIds=a.mapToIds(a.data.targets)))},i.updateLegendWithDefaults=function(){var a=this;a.updateLegend(a.mapToIds(a.data.targets),{withTransform:!1,withTransitionForTransform:!1,withTransition:!1})},i.updateSizeForLegend=function(a,b){var c=this,d=c.config,e={top:c.isLegendTop?c.getCurrentPaddingTop()+d.legend_inset_y+5.5:c.currentHeight-a-c.getCurrentPaddingBottom()-d.legend_inset_y,left:c.isLegendLeft?c.getCurrentPaddingLeft()+d.legend_inset_x+.5:c.currentWidth-b-c.getCurrentPaddingRight()-d.legend_inset_x+.5};c.margin3={top:c.isLegendRight?0:c.isLegendInset?e.top:c.currentHeight-a,right:NaN,bottom:0,left:c.isLegendRight?c.currentWidth-b:c.isLegendInset?e.left:0}},i.transformLegend=function(a){var b=this;(a?b.legend.transition():b.legend).attr("transform",b.getTranslate("legend"))},i.updateLegendStep=function(a){this.legendStep=a},i.updateLegendItemWidth=function(a){this.legendItemWidth=a},i.updateLegendItemHeight=function(a){this.legendItemHeight=a},i.getLegendWidth=function(){var a=this;return a.config.legend_show?a.isLegendRight||a.isLegendInset?a.legendItemWidth*(a.legendStep+1):a.currentWidth:0},i.getLegendHeight=function(){var a=this,b=0;return a.config.legend_show&&(b=a.isLegendRight?a.currentHeight:Math.max(20,a.legendItemHeight)*(a.legendStep+1)),b},i.opacityForLegend=function(a){return a.classed(l.legendItemHidden)?null:1},i.opacityForUnfocusedLegend=function(a){return a.classed(l.legendItemHidden)?null:.3},i.toggleFocusLegend=function(a,b){var c=this;a=c.mapToTargetIds(a),c.legend.selectAll("."+l.legendItem).filter(function(b){return a.indexOf(b)>=0}).classed(l.legendItemFocused,b).transition().duration(100).style("opacity",function(){var a=b?c.opacityForLegend:c.opacityForUnfocusedLegend;return a.call(c,c.d3.select(this))})},i.revertLegend=function(){var a=this,b=a.d3;a.legend.selectAll("."+l.legendItem).classed(l.legendItemFocused,!1).transition().duration(100).style("opacity",function(){return a.opacityForLegend(b.select(this))})},i.showLegend=function(a){var b=this,c=b.config;c.legend_show||(c.legend_show=!0,b.legend.style("visibility","visible"),b.legendHasRendered||b.updateLegendWithDefaults()),b.removeHiddenLegendIds(a),b.legend.selectAll(b.selectorLegends(a)).style("visibility","visible").transition().style("opacity",function(){return b.opacityForLegend(b.d3.select(this))})},i.hideLegend=function(a){var b=this,c=b.config;c.legend_show&&u(a)&&(c.legend_show=!1,b.legend.style("visibility","hidden")),b.addHiddenLegendIds(a),b.legend.selectAll(b.selectorLegends(a)).style("opacity",0).style("visibility","hidden")},i.clearLegendItemTextBoxCache=function(){this.legendItemTextBox={}},i.updateLegend=function(a,b,c){function d(a,b){return y.legendItemTextBox[b]||(y.legendItemTextBox[b]=y.getTextRect(a.textContent,l.legendItem,a)),y.legendItemTextBox[b]}function e(b,c,e){function f(a,b){b||(g=(o-G-n)/2,E>g&&(g=(o-n)/2,G=0,M++)),L[a]=M,K[M]=y.isLegendInset?10:g,H[a]=G,G+=n}var g,h,i=0===e,j=e===a.length-1,k=d(b,c),l=k.width+F+(!j||y.isLegendRight||y.isLegendInset?B:0)+z.legend_padding,m=k.height+A,n=y.isLegendRight||y.isLegendInset?m:l,o=y.isLegendRight||y.isLegendInset?y.getLegendHeight():y.getLegendWidth();return i&&(G=0,M=0,C=0,D=0),z.legend_show&&!y.isLegendToShow(c)?void(I[c]=J[c]=L[c]=H[c]=0):(I[c]=l,J[c]=m,(!C||l>=C)&&(C=l),(!D||m>=D)&&(D=m),h=y.isLegendRight||y.isLegendInset?D:C,void(z.legend_equally?(Object.keys(I).forEach(function(a){I[a]=C}),Object.keys(J).forEach(function(a){J[a]=D}),g=(o-h*a.length)/2,E>g?(G=0,M=0,a.forEach(function(a){f(a)})):f(c,!0)):f(c)))}var f,g,h,i,j,k,m,n,o,p,r,s,t,u,v,x,y=this,z=y.config,A=4,B=10,C=0,D=0,E=10,F=z.legend_item_tile_width+5,G=0,H={},I={},J={},K=[0],L={},M=0;a=a.filter(function(a){return!q(z.data_names[a])||null!==z.data_names[a]}),b=b||{},r=w(b,"withTransition",!0),s=w(b,"withTransitionForTransform",!0),y.isLegendInset&&(M=z.legend_inset_step?z.legend_inset_step:a.length,y.updateLegendStep(M)),y.isLegendRight?(f=function(a){return C*L[a]},i=function(a){return K[L[a]]+H[a]}):y.isLegendInset?(f=function(a){return C*L[a]+10},i=function(a){return K[L[a]]+H[a]}):(f=function(a){return K[L[a]]+H[a]},i=function(a){return D*L[a]}),g=function(a,b){return f(a,b)+4+z.legend_item_tile_width},j=function(a,b){return i(a,b)+9},h=function(a,b){return f(a,b)},k=function(a,b){return i(a,b)-5},m=function(a,b){return f(a,b)-2},n=function(a,b){return f(a,b)-2+z.legend_item_tile_width},o=function(a,b){return i(a,b)+4},p=y.legend.selectAll("."+l.legendItem).data(a).enter().append("g").attr("class",function(a){return y.generateClass(l.legendItem,a)}).style("visibility",function(a){return y.isLegendToShow(a)?"visible":"hidden"}).style("cursor","pointer").on("click",function(a){z.legend_item_onclick?z.legend_item_onclick.call(y,a):y.d3.event.altKey?(y.api.hide(),y.api.show(a)):(y.api.toggle(a),y.isTargetToShow(a)?y.api.focus(a):y.api.revert())}).on("mouseover",function(a){z.legend_item_onmouseover?z.legend_item_onmouseover.call(y,a):(y.d3.select(this).classed(l.legendItemFocused,!0),!y.transiting&&y.isTargetToShow(a)&&y.api.focus(a))}).on("mouseout",function(a){z.legend_item_onmouseout?z.legend_item_onmouseout.call(y,a):(y.d3.select(this).classed(l.legendItemFocused,!1),y.api.revert())}),p.append("text").text(function(a){return q(z.data_names[a])?z.data_names[a]:a}).each(function(a,b){e(this,a,b)}).style("pointer-events","none").attr("x",y.isLegendRight||y.isLegendInset?g:-200).attr("y",y.isLegendRight||y.isLegendInset?-200:j),p.append("rect").attr("class",l.legendItemEvent).style("fill-opacity",0).attr("x",y.isLegendRight||y.isLegendInset?h:-200).attr("y",y.isLegendRight||y.isLegendInset?-200:k),p.append("line").attr("class",l.legendItemTile).style("stroke",y.color).style("pointer-events","none").attr("x1",y.isLegendRight||y.isLegendInset?m:-200).attr("y1",y.isLegendRight||y.isLegendInset?-200:o).attr("x2",y.isLegendRight||y.isLegendInset?n:-200).attr("y2",y.isLegendRight||y.isLegendInset?-200:o).attr("stroke-width",z.legend_item_tile_height),x=y.legend.select("."+l.legendBackground+" rect"),y.isLegendInset&&C>0&&0===x.size()&&(x=y.legend.insert("g","."+l.legendItem).attr("class",l.legendBackground).append("rect")),t=y.legend.selectAll("text").data(a).text(function(a){return q(z.data_names[a])?z.data_names[a]:a}).each(function(a,b){e(this,a,b)}),(r?t.transition():t).attr("x",g).attr("y",j),u=y.legend.selectAll("rect."+l.legendItemEvent).data(a),(r?u.transition():u).attr("width",function(a){return I[a]}).attr("height",function(a){return J[a]}).attr("x",h).attr("y",k),v=y.legend.selectAll("line."+l.legendItemTile).data(a),(r?v.transition():v).style("stroke",y.color).attr("x1",m).attr("y1",o).attr("x2",n).attr("y2",o),x&&(r?x.transition():x).attr("height",y.getLegendHeight()-12).attr("width",C*(M+1)+10),y.legend.selectAll("."+l.legendItem).classed(l.legendItemHidden,function(a){return!y.isTargetToShow(a)}),y.updateLegendItemWidth(C),y.updateLegendItemHeight(D),y.updateLegendStep(M),y.updateSizes(),y.updateScales(),y.updateSvgSize(),y.transformAll(s,c),y.legendHasRendered=!0},i.initTitle=function(){var a=this;a.title=a.svg.append("text").text(a.config.title_text).attr("class",a.CLASS.title)},i.redrawTitle=function(){var a=this;a.title.attr("x",a.xForTitle.bind(a)).attr("y",a.yForTitle.bind(a))},i.xForTitle=function(){var a,b=this,c=b.config,d=c.title_position||"left";return a=d.indexOf("right")>=0?b.currentWidth-b.getTextRect(b.title.node().textContent,b.CLASS.title,b.title.node()).width-c.title_padding.right:d.indexOf("center")>=0?(b.currentWidth-b.getTextRect(b.title.node().textContent,b.CLASS.title,b.title.node()).width)/2:c.title_padding.left},i.yForTitle=function(){var a=this;return a.config.title_padding.top+a.getTextRect(a.title.node().textContent,a.CLASS.title,a.title.node()).height},i.getTitlePadding=function(){var a=this;return a.yForTitle()+a.config.title_padding.bottom},c(b,f),f.prototype.init=function(){var a=this.owner,b=a.config,c=a.main;a.axes.x=c.append("g").attr("class",l.axis+" "+l.axisX).attr("clip-path",a.clipPathForXAxis).attr("transform",a.getTranslate("x")).style("visibility",b.axis_x_show?"visible":"hidden"),a.axes.x.append("text").attr("class",l.axisXLabel).attr("transform",b.axis_rotated?"rotate(-90)":"").style("text-anchor",this.textAnchorForXAxisLabel.bind(this)),a.axes.y=c.append("g").attr("class",l.axis+" "+l.axisY).attr("clip-path",b.axis_y_inner?"":a.clipPathForYAxis).attr("transform",a.getTranslate("y")).style("visibility",b.axis_y_show?"visible":"hidden"),a.axes.y.append("text").attr("class",l.axisYLabel).attr("transform",b.axis_rotated?"":"rotate(-90)").style("text-anchor",this.textAnchorForYAxisLabel.bind(this)),a.axes.y2=c.append("g").attr("class",l.axis+" "+l.axisY2).attr("transform",a.getTranslate("y2")).style("visibility",b.axis_y2_show?"visible":"hidden"),a.axes.y2.append("text").attr("class",l.axisY2Label).attr("transform",b.axis_rotated?"":"rotate(-90)").style("text-anchor",this.textAnchorForY2AxisLabel.bind(this))},f.prototype.getXAxis=function(a,b,c,d,e,f,h){var i=this.owner,j=i.config,k={isCategory:i.isCategorized(),withOuterTick:e,tickMultiline:j.axis_x_tick_multiline,tickWidth:j.axis_x_tick_width,tickTextRotate:h?0:j.axis_x_tick_rotate,withoutTransition:f},l=g(i.d3,k).scale(a).orient(b);return i.isTimeSeries()&&d&&"function"!=typeof d&&(d=d.map(function(a){return i.parseDate(a)})),l.tickFormat(c).tickValues(d),i.isCategorized()&&(l.tickCentered(j.axis_x_tick_centered),u(j.axis_x_tick_culling)&&(j.axis_x_tick_culling=!1)),l},f.prototype.updateXAxisTickValues=function(a,b){var c,d=this.owner,e=d.config;return(e.axis_x_tick_fit||e.axis_x_tick_count)&&(c=this.generateTickValues(d.mapTargetsToUniqueXs(a),e.axis_x_tick_count,d.isTimeSeries())),b?b.tickValues(c):(d.xAxis.tickValues(c),d.subXAxis.tickValues(c)),c},f.prototype.getYAxis=function(a,b,c,d,e,f,h){var i=this.owner,j=i.config,k={withOuterTick:e,withoutTransition:f,tickTextRotate:h?0:j.axis_y_tick_rotate},l=g(i.d3,k).scale(a).orient(b).tickFormat(c);return i.isTimeSeriesY()?l.ticks(i.d3.time[j.axis_y_tick_time_value],j.axis_y_tick_time_interval):l.tickValues(d),l},f.prototype.getId=function(a){var b=this.owner.config;return a in b.data_axes?b.data_axes[a]:"y"},f.prototype.getXAxisTickFormat=function(){var a=this.owner,b=a.config,c=a.isTimeSeries()?a.defaultAxisTimeFormat:a.isCategorized()?a.categoryName:function(a){return 0>a?a.toFixed(0):a};return b.axis_x_tick_format&&(n(b.axis_x_tick_format)?c=b.axis_x_tick_format:a.isTimeSeries()&&(c=function(c){return c?a.axisTimeFormat(b.axis_x_tick_format)(c):""})),n(c)?function(b){return c.call(a,b)}:c},f.prototype.getTickValues=function(a,b){return a?a:b?b.tickValues():void 0},f.prototype.getXAxisTickValues=function(){return this.getTickValues(this.owner.config.axis_x_tick_values,this.owner.xAxis)},f.prototype.getYAxisTickValues=function(){return this.getTickValues(this.owner.config.axis_y_tick_values,this.owner.yAxis)},f.prototype.getY2AxisTickValues=function(){return this.getTickValues(this.owner.config.axis_y2_tick_values,this.owner.y2Axis)},f.prototype.getLabelOptionByAxisId=function(a){var b,c=this.owner,d=c.config;return"y"===a?b=d.axis_y_label:"y2"===a?b=d.axis_y2_label:"x"===a&&(b=d.axis_x_label),b},f.prototype.getLabelText=function(a){var b=this.getLabelOptionByAxisId(a);return o(b)?b:b?b.text:null},f.prototype.setLabelText=function(a,b){var c=this.owner,d=c.config,e=this.getLabelOptionByAxisId(a);o(e)?"y"===a?d.axis_y_label=b:"y2"===a?d.axis_y2_label=b:"x"===a&&(d.axis_x_label=b):e&&(e.text=b)},f.prototype.getLabelPosition=function(a,b){var c=this.getLabelOptionByAxisId(a),d=c&&"object"==typeof c&&c.position?c.position:b;return{isInner:d.indexOf("inner")>=0,isOuter:d.indexOf("outer")>=0,isLeft:d.indexOf("left")>=0,isCenter:d.indexOf("center")>=0,isRight:d.indexOf("right")>=0,isTop:d.indexOf("top")>=0,isMiddle:d.indexOf("middle")>=0,isBottom:d.indexOf("bottom")>=0}},f.prototype.getXAxisLabelPosition=function(){return this.getLabelPosition("x",this.owner.config.axis_rotated?"inner-top":"inner-right")},f.prototype.getYAxisLabelPosition=function(){return this.getLabelPosition("y",this.owner.config.axis_rotated?"inner-right":"inner-top")},f.prototype.getY2AxisLabelPosition=function(){return this.getLabelPosition("y2",this.owner.config.axis_rotated?"inner-right":"inner-top")},f.prototype.getLabelPositionById=function(a){return"y2"===a?this.getY2AxisLabelPosition():"y"===a?this.getYAxisLabelPosition():this.getXAxisLabelPosition()},f.prototype.textForXAxisLabel=function(){return this.getLabelText("x")},f.prototype.textForYAxisLabel=function(){return this.getLabelText("y")},f.prototype.textForY2AxisLabel=function(){return this.getLabelText("y2")},f.prototype.xForAxisLabel=function(a,b){var c=this.owner;return a?b.isLeft?0:b.isCenter?c.width/2:c.width:b.isBottom?-c.height:b.isMiddle?-c.height/2:0},f.prototype.dxForAxisLabel=function(a,b){return a?b.isLeft?"0.5em":b.isRight?"-0.5em":"0":b.isTop?"-0.5em":b.isBottom?"0.5em":"0"},f.prototype.textAnchorForAxisLabel=function(a,b){return a?b.isLeft?"start":b.isCenter?"middle":"end":b.isBottom?"start":b.isMiddle?"middle":"end"},f.prototype.xForXAxisLabel=function(){return this.xForAxisLabel(!this.owner.config.axis_rotated,this.getXAxisLabelPosition())},f.prototype.xForYAxisLabel=function(){return this.xForAxisLabel(this.owner.config.axis_rotated,this.getYAxisLabelPosition())},f.prototype.xForY2AxisLabel=function(){return this.xForAxisLabel(this.owner.config.axis_rotated,this.getY2AxisLabelPosition())},f.prototype.dxForXAxisLabel=function(){return this.dxForAxisLabel(!this.owner.config.axis_rotated,this.getXAxisLabelPosition())},f.prototype.dxForYAxisLabel=function(){return this.dxForAxisLabel(this.owner.config.axis_rotated,this.getYAxisLabelPosition())},f.prototype.dxForY2AxisLabel=function(){return this.dxForAxisLabel(this.owner.config.axis_rotated,this.getY2AxisLabelPosition())},f.prototype.dyForXAxisLabel=function(){var a=this.owner,b=a.config,c=this.getXAxisLabelPosition();return b.axis_rotated?c.isInner?"1.2em":-25-this.getMaxTickWidth("x"):c.isInner?"-0.5em":b.axis_x_height?b.axis_x_height-10:"3em"},f.prototype.dyForYAxisLabel=function(){var a=this.owner,b=this.getYAxisLabelPosition();return a.config.axis_rotated?b.isInner?"-0.5em":"3em":b.isInner?"1.2em":-10-(a.config.axis_y_inner?0:this.getMaxTickWidth("y")+10)},f.prototype.dyForY2AxisLabel=function(){var a=this.owner,b=this.getY2AxisLabelPosition();return a.config.axis_rotated?b.isInner?"1.2em":"-2.2em":b.isInner?"-0.5em":15+(a.config.axis_y2_inner?0:this.getMaxTickWidth("y2")+15)},f.prototype.textAnchorForXAxisLabel=function(){var a=this.owner;return this.textAnchorForAxisLabel(!a.config.axis_rotated,this.getXAxisLabelPosition())},f.prototype.textAnchorForYAxisLabel=function(){var a=this.owner;return this.textAnchorForAxisLabel(a.config.axis_rotated,this.getYAxisLabelPosition())},f.prototype.textAnchorForY2AxisLabel=function(){var a=this.owner;return this.textAnchorForAxisLabel(a.config.axis_rotated,this.getY2AxisLabelPosition())},f.prototype.getMaxTickWidth=function(a,b){var c,d,e,f,g,h=this.owner,i=h.config,j=0;return b&&h.currentMaxTickWidths[a]?h.currentMaxTickWidths[a]:(h.svg&&(c=h.filterTargetsToShow(h.data.targets),"y"===a?(d=h.y.copy().domain(h.getYDomain(c,"y")),e=this.getYAxis(d,h.yOrient,i.axis_y_tick_format,h.yAxisTickValues,!1,!0,!0)):"y2"===a?(d=h.y2.copy().domain(h.getYDomain(c,"y2")),
e=this.getYAxis(d,h.y2Orient,i.axis_y2_tick_format,h.y2AxisTickValues,!1,!0,!0)):(d=h.x.copy().domain(h.getXDomain(c)),e=this.getXAxis(d,h.xOrient,h.xAxisTickFormat,h.xAxisTickValues,!1,!0,!0),this.updateXAxisTickValues(c,e)),f=h.d3.select("body").append("div").classed("c3",!0),g=f.append("svg").style("visibility","hidden").style("position","fixed").style("top",0).style("left",0),g.append("g").call(e).each(function(){h.d3.select(this).selectAll("text").each(function(){var a=this.getBoundingClientRect();j<a.width&&(j=a.width)}),f.remove()})),h.currentMaxTickWidths[a]=0>=j?h.currentMaxTickWidths[a]:j,h.currentMaxTickWidths[a])},f.prototype.updateLabels=function(a){var b=this.owner,c=b.main.select("."+l.axisX+" ."+l.axisXLabel),d=b.main.select("."+l.axisY+" ."+l.axisYLabel),e=b.main.select("."+l.axisY2+" ."+l.axisY2Label);(a?c.transition():c).attr("x",this.xForXAxisLabel.bind(this)).attr("dx",this.dxForXAxisLabel.bind(this)).attr("dy",this.dyForXAxisLabel.bind(this)).text(this.textForXAxisLabel.bind(this)),(a?d.transition():d).attr("x",this.xForYAxisLabel.bind(this)).attr("dx",this.dxForYAxisLabel.bind(this)).attr("dy",this.dyForYAxisLabel.bind(this)).text(this.textForYAxisLabel.bind(this)),(a?e.transition():e).attr("x",this.xForY2AxisLabel.bind(this)).attr("dx",this.dxForY2AxisLabel.bind(this)).attr("dy",this.dyForY2AxisLabel.bind(this)).text(this.textForY2AxisLabel.bind(this))},f.prototype.getPadding=function(a,b,c,d){var e="number"==typeof a?a:a[b];return m(e)?"ratio"===a.unit?a[b]*d:this.convertPixelsToAxisPadding(e,d):c},f.prototype.convertPixelsToAxisPadding=function(a,b){var c=this.owner,d=c.config.axis_rotated?c.width:c.height;return b*(a/d)},f.prototype.generateTickValues=function(a,b,c){var d,e,f,g,h,i,j,k=a;if(b)if(d=n(b)?b():b,1===d)k=[a[0]];else if(2===d)k=[a[0],a[a.length-1]];else if(d>2){for(g=d-2,e=a[0],f=a[a.length-1],h=(f-e)/(g+1),k=[e],i=0;g>i;i++)j=+e+h*(i+1),k.push(c?new Date(j):j);k.push(f)}return c||(k=k.sort(function(a,b){return a-b})),k},f.prototype.generateTransitions=function(a){var b=this.owner,c=b.axes;return{axisX:a?c.x.transition().duration(a):c.x,axisY:a?c.y.transition().duration(a):c.y,axisY2:a?c.y2.transition().duration(a):c.y2,axisSubX:a?c.subx.transition().duration(a):c.subx}},f.prototype.redraw=function(a,b){var c=this.owner;c.axes.x.style("opacity",b?0:1),c.axes.y.style("opacity",b?0:1),c.axes.y2.style("opacity",b?0:1),c.axes.subx.style("opacity",b?0:1),a.axisX.call(c.xAxis),a.axisY.call(c.yAxis),a.axisY2.call(c.y2Axis),a.axisSubX.call(c.subXAxis)},i.getClipPath=function(b){var c=a.navigator.appVersion.toLowerCase().indexOf("msie 9.")>=0;return"url("+(c?"":document.URL.split("#")[0])+"#"+b+")"},i.appendClip=function(a,b){return a.append("clipPath").attr("id",b).append("rect")},i.getAxisClipX=function(a){var b=Math.max(30,this.margin.left);return a?-(1+b):-(b-1)},i.getAxisClipY=function(a){return a?-20:-this.margin.top},i.getXAxisClipX=function(){var a=this;return a.getAxisClipX(!a.config.axis_rotated)},i.getXAxisClipY=function(){var a=this;return a.getAxisClipY(!a.config.axis_rotated)},i.getYAxisClipX=function(){var a=this;return a.config.axis_y_inner?-1:a.getAxisClipX(a.config.axis_rotated)},i.getYAxisClipY=function(){var a=this;return a.getAxisClipY(a.config.axis_rotated)},i.getAxisClipWidth=function(a){var b=this,c=Math.max(30,b.margin.left),d=Math.max(30,b.margin.right);return a?b.width+2+c+d:b.margin.left+20},i.getAxisClipHeight=function(a){return(a?this.margin.bottom:this.margin.top+this.height)+20},i.getXAxisClipWidth=function(){var a=this;return a.getAxisClipWidth(!a.config.axis_rotated)},i.getXAxisClipHeight=function(){var a=this;return a.getAxisClipHeight(!a.config.axis_rotated)},i.getYAxisClipWidth=function(){var a=this;return a.getAxisClipWidth(a.config.axis_rotated)+(a.config.axis_y_inner?20:0)},i.getYAxisClipHeight=function(){var a=this;return a.getAxisClipHeight(a.config.axis_rotated)},i.initPie=function(){var a=this,b=a.d3,c=a.config;a.pie=b.layout.pie().value(function(a){return a.values.reduce(function(a,b){return a+b.value},0)}),c.data_order||a.pie.sort(null)},i.updateRadius=function(){var a=this,b=a.config,c=b.gauge_width||b.donut_width;a.radiusExpanded=Math.min(a.arcWidth,a.arcHeight)/2,a.radius=.95*a.radiusExpanded,a.innerRadiusRatio=c?(a.radius-c)/a.radius:.6,a.innerRadius=a.hasType("donut")||a.hasType("gauge")?a.radius*a.innerRadiusRatio:0},i.updateArc=function(){var a=this;a.svgArc=a.getSvgArc(),a.svgArcExpanded=a.getSvgArcExpanded(),a.svgArcExpandedSub=a.getSvgArcExpanded(.98)},i.updateAngle=function(a){var b,c,d,e,f=this,g=f.config,h=!1,i=0;return g?(f.pie(f.filterTargetsToShow(f.data.targets)).forEach(function(b){h||b.data.id!==a.data.id||(h=!0,a=b,a.index=i),i++}),isNaN(a.startAngle)&&(a.startAngle=0),isNaN(a.endAngle)&&(a.endAngle=a.startAngle),f.isGaugeType(a.data)&&(b=g.gauge_min,c=g.gauge_max,d=Math.PI*(g.gauge_fullCircle?2:1)/(c-b),e=a.value<b?0:a.value<c?a.value-b:c-b,a.startAngle=g.gauge_startingAngle,a.endAngle=a.startAngle+d*e),h?a:null):null},i.getSvgArc=function(){var a=this,b=a.d3.svg.arc().outerRadius(a.radius).innerRadius(a.innerRadius),c=function(c,d){var e;return d?b(c):(e=a.updateAngle(c),e?b(e):"M 0 0")};return c.centroid=b.centroid,c},i.getSvgArcExpanded=function(a){var b=this,c=b.d3.svg.arc().outerRadius(b.radiusExpanded*(a?a:1)).innerRadius(b.innerRadius);return function(a){var d=b.updateAngle(a);return d?c(d):"M 0 0"}},i.getArc=function(a,b,c){return c||this.isArcType(a.data)?this.svgArc(a,b):"M 0 0"},i.transformForArcLabel=function(a){var b,c,d,e,f,g=this,h=g.config,i=g.updateAngle(a),j="";return i&&!g.hasType("gauge")&&(b=this.svgArc.centroid(i),c=isNaN(b[0])?0:b[0],d=isNaN(b[1])?0:b[1],e=Math.sqrt(c*c+d*d),f=g.hasType("donut")&&h.donut_label_ratio?n(h.donut_label_ratio)?h.donut_label_ratio(a,g.radius,e):h.donut_label_ratio:g.hasType("pie")&&h.pie_label_ratio?n(h.pie_label_ratio)?h.pie_label_ratio(a,g.radius,e):h.pie_label_ratio:g.radius&&e?(36/g.radius>.375?1.175-36/g.radius:.8)*g.radius/e:0,j="translate("+c*f+","+d*f+")"),j},i.getArcRatio=function(a){var b=this,c=b.config,d=Math.PI*(b.hasType("gauge")&&!c.gauge_fullCircle?1:2);return a?(a.endAngle-a.startAngle)/d:null},i.convertToArcData=function(a){return this.addName({id:a.data.id,value:a.value,ratio:this.getArcRatio(a),index:a.index})},i.textForArcLabel=function(a){var b,c,d,e,f,g=this;return g.shouldShowArcLabel()?(b=g.updateAngle(a),c=b?b.value:null,d=g.getArcRatio(b),e=a.data.id,g.hasType("gauge")||g.meetsArcLabelThreshold(d)?(f=g.getArcLabelFormat(),f?f(c,d,e):g.defaultArcValueFormat(c,d)):""):""},i.expandArc=function(b){var c,d=this;return d.transiting?void(c=a.setInterval(function(){d.transiting||(a.clearInterval(c),d.legend.selectAll(".c3-legend-item-focused").size()>0&&d.expandArc(b))},10)):(b=d.mapToTargetIds(b),void d.svg.selectAll(d.selectorTargets(b,"."+l.chartArc)).each(function(a){d.shouldExpand(a.data.id)&&d.d3.select(this).selectAll("path").transition().duration(d.expandDuration(a.data.id)).attr("d",d.svgArcExpanded).transition().duration(2*d.expandDuration(a.data.id)).attr("d",d.svgArcExpandedSub).each(function(a){d.isDonutType(a.data)})}))},i.unexpandArc=function(a){var b=this;b.transiting||(a=b.mapToTargetIds(a),b.svg.selectAll(b.selectorTargets(a,"."+l.chartArc)).selectAll("path").transition().duration(function(a){return b.expandDuration(a.data.id)}).attr("d",b.svgArc),b.svg.selectAll("."+l.arc).style("opacity",1))},i.expandDuration=function(a){var b=this,c=b.config;return b.isDonutType(a)?c.donut_expand_duration:b.isGaugeType(a)?c.gauge_expand_duration:b.isPieType(a)?c.pie_expand_duration:50},i.shouldExpand=function(a){var b=this,c=b.config;return b.isDonutType(a)&&c.donut_expand||b.isGaugeType(a)&&c.gauge_expand||b.isPieType(a)&&c.pie_expand},i.shouldShowArcLabel=function(){var a=this,b=a.config,c=!0;return a.hasType("donut")?c=b.donut_label_show:a.hasType("pie")&&(c=b.pie_label_show),c},i.meetsArcLabelThreshold=function(a){var b=this,c=b.config,d=b.hasType("donut")?c.donut_label_threshold:c.pie_label_threshold;return a>=d},i.getArcLabelFormat=function(){var a=this,b=a.config,c=b.pie_label_format;return a.hasType("gauge")?c=b.gauge_label_format:a.hasType("donut")&&(c=b.donut_label_format),c},i.getArcTitle=function(){var a=this;return a.hasType("donut")?a.config.donut_title:""},i.updateTargetsForArc=function(a){var b,c,d=this,e=d.main,f=d.classChartArc.bind(d),g=d.classArcs.bind(d),h=d.classFocus.bind(d);b=e.select("."+l.chartArcs).selectAll("."+l.chartArc).data(d.pie(a)).attr("class",function(a){return f(a)+h(a.data)}),c=b.enter().append("g").attr("class",f),c.append("g").attr("class",g),c.append("text").attr("dy",d.hasType("gauge")?"-.1em":".35em").style("opacity",0).style("text-anchor","middle").style("pointer-events","none")},i.initArc=function(){var a=this;a.arcs=a.main.select("."+l.chart).append("g").attr("class",l.chartArcs).attr("transform",a.getTranslate("arc")),a.arcs.append("text").attr("class",l.chartArcsTitle).style("text-anchor","middle").text(a.getArcTitle())},i.redrawArc=function(a,b,c){var d,e=this,f=e.d3,g=e.config,h=e.main;d=h.selectAll("."+l.arcs).selectAll("."+l.arc).data(e.arcData.bind(e)),d.enter().append("path").attr("class",e.classArc.bind(e)).style("fill",function(a){return e.color(a.data)}).style("cursor",function(a){return g.interaction_enabled&&g.data_selection_isselectable(a)?"pointer":null}).style("opacity",0).each(function(a){e.isGaugeType(a.data)&&(a.startAngle=a.endAngle=g.gauge_startingAngle),this._current=a}),d.attr("transform",function(a){return!e.isGaugeType(a.data)&&c?"scale(0)":""}).style("opacity",function(a){return a===this._current?0:1}).on("mouseover",g.interaction_enabled?function(a){var b,c;e.transiting||(b=e.updateAngle(a),b&&(c=e.convertToArcData(b),e.expandArc(b.data.id),e.api.focus(b.data.id),e.toggleFocusLegend(b.data.id,!0),e.config.data_onmouseover(c,this)))}:null).on("mousemove",g.interaction_enabled?function(a){var b,c,d=e.updateAngle(a);d&&(b=e.convertToArcData(d),c=[b],e.showTooltip(c,this))}:null).on("mouseout",g.interaction_enabled?function(a){var b,c;e.transiting||(b=e.updateAngle(a),b&&(c=e.convertToArcData(b),e.unexpandArc(b.data.id),e.api.revert(),e.revertLegend(),e.hideTooltip(),e.config.data_onmouseout(c,this)))}:null).on("click",g.interaction_enabled?function(a,b){var c,d=e.updateAngle(a);d&&(c=e.convertToArcData(d),e.toggleShape&&e.toggleShape(this,c,b),e.config.data_onclick.call(e.api,c,this))}:null).each(function(){e.transiting=!0}).transition().duration(a).attrTween("d",function(a){var b,c=e.updateAngle(a);return c?(isNaN(this._current.startAngle)&&(this._current.startAngle=0),isNaN(this._current.endAngle)&&(this._current.endAngle=this._current.startAngle),b=f.interpolate(this._current,c),this._current=b(0),function(c){var d=b(c);return d.data=a.data,e.getArc(d,!0)}):function(){return"M 0 0"}}).attr("transform",c?"scale(1)":"").style("fill",function(a){return e.levelColor?e.levelColor(a.data.values[0].value):e.color(a.data.id)}).style("opacity",1).call(e.endall,function(){e.transiting=!1}),d.exit().transition().duration(b).style("opacity",0).remove(),h.selectAll("."+l.chartArc).select("text").style("opacity",0).attr("class",function(a){return e.isGaugeType(a.data)?l.gaugeValue:""}).text(e.textForArcLabel.bind(e)).attr("transform",e.transformForArcLabel.bind(e)).style("font-size",function(a){return e.isGaugeType(a.data)?Math.round(e.radius/5)+"px":""}).transition().duration(a).style("opacity",function(a){return e.isTargetToShow(a.data.id)&&e.isArcType(a.data)?1:0}),h.select("."+l.chartArcsTitle).style("opacity",e.hasType("donut")||e.hasType("gauge")?1:0),e.hasType("gauge")&&(e.arcs.select("."+l.chartArcsBackground).attr("d",function(){var a={data:[{value:g.gauge_max}],startAngle:g.gauge_startingAngle,endAngle:-1*g.gauge_startingAngle};return e.getArc(a,!0,!0)}),e.arcs.select("."+l.chartArcsGaugeUnit).attr("dy",".75em").text(g.gauge_label_show?g.gauge_units:""),e.arcs.select("."+l.chartArcsGaugeMin).attr("dx",-1*(e.innerRadius+(e.radius-e.innerRadius)/(g.gauge_fullCircle?1:2))+"px").attr("dy","1.2em").text(g.gauge_label_show?g.gauge_min:""),e.arcs.select("."+l.chartArcsGaugeMax).attr("dx",e.innerRadius+(e.radius-e.innerRadius)/(g.gauge_fullCircle?1:2)+"px").attr("dy","1.2em").text(g.gauge_label_show?g.gauge_max:""))},i.initGauge=function(){var a=this.arcs;this.hasType("gauge")&&(a.append("path").attr("class",l.chartArcsBackground),a.append("text").attr("class",l.chartArcsGaugeUnit).style("text-anchor","middle").style("pointer-events","none"),a.append("text").attr("class",l.chartArcsGaugeMin).style("text-anchor","middle").style("pointer-events","none"),a.append("text").attr("class",l.chartArcsGaugeMax).style("text-anchor","middle").style("pointer-events","none"))},i.getGaugeLabelHeight=function(){return this.config.gauge_label_show?20:0},i.initRegion=function(){var a=this;a.region=a.main.append("g").attr("clip-path",a.clipPath).attr("class",l.regions)},i.updateRegion=function(a){var b=this,c=b.config;b.region.style("visibility",b.hasArcType()?"hidden":"visible"),b.mainRegion=b.main.select("."+l.regions).selectAll("."+l.region).data(c.regions),b.mainRegion.enter().append("g").append("rect").style("fill-opacity",0),b.mainRegion.attr("class",b.classRegion.bind(b)),b.mainRegion.exit().transition().duration(a).style("opacity",0).remove()},i.redrawRegion=function(a){var b=this,c=b.mainRegion.selectAll("rect").each(function(){var a=b.d3.select(this.parentNode).datum();b.d3.select(this).datum(a)}),d=b.regionX.bind(b),e=b.regionY.bind(b),f=b.regionWidth.bind(b),g=b.regionHeight.bind(b);return[(a?c.transition():c).attr("x",d).attr("y",e).attr("width",f).attr("height",g).style("fill-opacity",function(a){return m(a.opacity)?a.opacity:.1})]},i.regionX=function(a){var b,c=this,d=c.config,e="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated&&"start"in a?e(a.start):0:d.axis_rotated?0:"start"in a?c.x(c.isTimeSeries()?c.parseDate(a.start):a.start):0},i.regionY=function(a){var b,c=this,d=c.config,e="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated?0:"end"in a?e(a.end):0:d.axis_rotated&&"start"in a?c.x(c.isTimeSeries()?c.parseDate(a.start):a.start):0},i.regionWidth=function(a){var b,c=this,d=c.config,e=c.regionX(a),f="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated&&"end"in a?f(a.end):c.width:d.axis_rotated?c.width:"end"in a?c.x(c.isTimeSeries()?c.parseDate(a.end):a.end):c.width,e>b?0:b-e},i.regionHeight=function(a){var b,c=this,d=c.config,e=this.regionY(a),f="y"===a.axis?c.y:c.y2;return b="y"===a.axis||"y2"===a.axis?d.axis_rotated?c.height:"start"in a?f(a.start):c.height:d.axis_rotated&&"end"in a?c.x(c.isTimeSeries()?c.parseDate(a.end):a.end):c.height,e>b?0:b-e},i.isRegionOnX=function(a){return!a.axis||"x"===a.axis},i.drag=function(a){var b,c,d,e,f,g,h,i,j=this,k=j.config,m=j.main,n=j.d3;j.hasArcType()||k.data_selection_enabled&&(k.zoom_enabled&&!j.zoom.altDomain||k.data_selection_multiple&&(b=j.dragStart[0],c=j.dragStart[1],d=a[0],e=a[1],f=Math.min(b,d),g=Math.max(b,d),h=k.data_selection_grouped?j.margin.top:Math.min(c,e),i=k.data_selection_grouped?j.height:Math.max(c,e),m.select("."+l.dragarea).attr("x",f).attr("y",h).attr("width",g-f).attr("height",i-h),m.selectAll("."+l.shapes).selectAll("."+l.shape).filter(function(a){return k.data_selection_isselectable(a)}).each(function(a,b){var c,d,e,k,m,o,p=n.select(this),q=p.classed(l.SELECTED),r=p.classed(l.INCLUDED),s=!1;if(p.classed(l.circle))c=1*p.attr("cx"),d=1*p.attr("cy"),m=j.togglePoint,s=c>f&&g>c&&d>h&&i>d;else{if(!p.classed(l.bar))return;o=z(this),c=o.x,d=o.y,e=o.width,k=o.height,m=j.togglePath,s=!(c>g||f>c+e||d>i||h>d+k)}s^r&&(p.classed(l.INCLUDED,!r),p.classed(l.SELECTED,!q),m.call(j,!q,p,a,b))})))},i.dragstart=function(a){var b=this,c=b.config;b.hasArcType()||c.data_selection_enabled&&(b.dragStart=a,b.main.select("."+l.chart).append("rect").attr("class",l.dragarea).style("opacity",.1),b.dragging=!0)},i.dragend=function(){var a=this,b=a.config;a.hasArcType()||b.data_selection_enabled&&(a.main.select("."+l.dragarea).transition().duration(100).style("opacity",0).remove(),a.main.selectAll("."+l.shape).classed(l.INCLUDED,!1),a.dragging=!1)},i.selectPoint=function(a,b,c){var d=this,e=d.config,f=(e.axis_rotated?d.circleY:d.circleX).bind(d),g=(e.axis_rotated?d.circleX:d.circleY).bind(d),h=d.pointSelectR.bind(d);e.data_onselected.call(d.api,b,a.node()),d.main.select("."+l.selectedCircles+d.getTargetSelectorSuffix(b.id)).selectAll("."+l.selectedCircle+"-"+c).data([b]).enter().append("circle").attr("class",function(){return d.generateClass(l.selectedCircle,c)}).attr("cx",f).attr("cy",g).attr("stroke",function(){return d.color(b)}).attr("r",function(a){return 1.4*d.pointSelectR(a)}).transition().duration(100).attr("r",h)},i.unselectPoint=function(a,b,c){var d=this;d.config.data_onunselected.call(d.api,b,a.node()),d.main.select("."+l.selectedCircles+d.getTargetSelectorSuffix(b.id)).selectAll("."+l.selectedCircle+"-"+c).transition().duration(100).attr("r",0).remove()},i.togglePoint=function(a,b,c,d){a?this.selectPoint(b,c,d):this.unselectPoint(b,c,d)},i.selectPath=function(a,b){var c=this;c.config.data_onselected.call(c,b,a.node()),c.config.interaction_brighten&&a.transition().duration(100).style("fill",function(){return c.d3.rgb(c.color(b)).brighter(.75)})},i.unselectPath=function(a,b){var c=this;c.config.data_onunselected.call(c,b,a.node()),c.config.interaction_brighten&&a.transition().duration(100).style("fill",function(){return c.color(b)})},i.togglePath=function(a,b,c,d){a?this.selectPath(b,c,d):this.unselectPath(b,c,d)},i.getToggle=function(a,b){var c,d=this;return"circle"===a.nodeName?c=d.isStepType(b)?function(){}:d.togglePoint:"path"===a.nodeName&&(c=d.togglePath),c},i.toggleShape=function(a,b,c){var d=this,e=d.d3,f=d.config,g=e.select(a),h=g.classed(l.SELECTED),i=d.getToggle(a,b).bind(d);f.data_selection_enabled&&f.data_selection_isselectable(b)&&(f.data_selection_multiple||d.main.selectAll("."+l.shapes+(f.data_selection_grouped?d.getTargetSelectorSuffix(b.id):"")).selectAll("."+l.shape).each(function(a,b){var c=e.select(this);c.classed(l.SELECTED)&&i(!1,c.classed(l.SELECTED,!1),a,b)}),g.classed(l.SELECTED,!h),i(!h,g,b,c))},i.initBrush=function(){var a=this,b=a.d3;a.brush=b.svg.brush().on("brush",function(){a.redrawForBrush()}),a.brush.update=function(){return a.context&&a.context.select("."+l.brush).call(this),this},a.brush.scale=function(b){return a.config.axis_rotated?this.y(b):this.x(b)}},i.initSubchart=function(){var a=this,b=a.config,c=a.context=a.svg.append("g").attr("transform",a.getTranslate("context")),d=b.subchart_show?"visible":"hidden";c.style("visibility",d),c.append("g").attr("clip-path",a.clipPathForSubchart).attr("class",l.chart),c.select("."+l.chart).append("g").attr("class",l.chartBars),c.select("."+l.chart).append("g").attr("class",l.chartLines),c.append("g").attr("clip-path",a.clipPath).attr("class",l.brush).call(a.brush),a.axes.subx=c.append("g").attr("class",l.axisX).attr("transform",a.getTranslate("subx")).attr("clip-path",b.axis_rotated?"":a.clipPathForXAxis).style("visibility",b.subchart_axis_x_show?d:"hidden")},i.updateTargetsForSubchart=function(a){var b,c,d,e,f=this,g=f.context,h=f.config,i=f.classChartBar.bind(f),j=f.classBars.bind(f),k=f.classChartLine.bind(f),m=f.classLines.bind(f),n=f.classAreas.bind(f);h.subchart_show&&(e=g.select("."+l.chartBars).selectAll("."+l.chartBar).data(a).attr("class",i),d=e.enter().append("g").style("opacity",0).attr("class",i),d.append("g").attr("class",j),c=g.select("."+l.chartLines).selectAll("."+l.chartLine).data(a).attr("class",k),b=c.enter().append("g").style("opacity",0).attr("class",k),b.append("g").attr("class",m),b.append("g").attr("class",n),g.selectAll("."+l.brush+" rect").attr(h.axis_rotated?"width":"height",h.axis_rotated?f.width2:f.height2))},i.updateBarForSubchart=function(a){var b=this;b.contextBar=b.context.selectAll("."+l.bars).selectAll("."+l.bar).data(b.barData.bind(b)),b.contextBar.enter().append("path").attr("class",b.classBar.bind(b)).style("stroke","none").style("fill",b.color),b.contextBar.style("opacity",b.initialOpacity.bind(b)),b.contextBar.exit().transition().duration(a).style("opacity",0).remove()},i.redrawBarForSubchart=function(a,b,c){(b?this.contextBar.transition(Math.random().toString()).duration(c):this.contextBar).attr("d",a).style("opacity",1)},i.updateLineForSubchart=function(a){var b=this;b.contextLine=b.context.selectAll("."+l.lines).selectAll("."+l.line).data(b.lineData.bind(b)),b.contextLine.enter().append("path").attr("class",b.classLine.bind(b)).style("stroke",b.color),b.contextLine.style("opacity",b.initialOpacity.bind(b)),b.contextLine.exit().transition().duration(a).style("opacity",0).remove()},i.redrawLineForSubchart=function(a,b,c){(b?this.contextLine.transition(Math.random().toString()).duration(c):this.contextLine).attr("d",a).style("opacity",1)},i.updateAreaForSubchart=function(a){var b=this,c=b.d3;b.contextArea=b.context.selectAll("."+l.areas).selectAll("."+l.area).data(b.lineData.bind(b)),b.contextArea.enter().append("path").attr("class",b.classArea.bind(b)).style("fill",b.color).style("opacity",function(){return b.orgAreaOpacity=+c.select(this).style("opacity"),0}),b.contextArea.style("opacity",0),b.contextArea.exit().transition().duration(a).style("opacity",0).remove()},i.redrawAreaForSubchart=function(a,b,c){(b?this.contextArea.transition(Math.random().toString()).duration(c):this.contextArea).attr("d",a).style("fill",this.color).style("opacity",this.orgAreaOpacity)},i.redrawSubchart=function(a,b,c,d,e,f,g){var h,i,j,k=this,l=k.d3,m=k.config;k.context.style("visibility",m.subchart_show?"visible":"hidden"),m.subchart_show&&(l.event&&"zoom"===l.event.type&&k.brush.extent(k.x.orgDomain()).update(),a&&(k.brush.empty()||k.brush.extent(k.x.orgDomain()).update(),h=k.generateDrawArea(e,!0),i=k.generateDrawBar(f,!0),j=k.generateDrawLine(g,!0),k.updateBarForSubchart(c),k.updateLineForSubchart(c),k.updateAreaForSubchart(c),k.redrawBarForSubchart(i,c,c),k.redrawLineForSubchart(j,c,c),k.redrawAreaForSubchart(h,c,c)))},i.redrawForBrush=function(){var a=this,b=a.x;a.redraw({withTransition:!1,withY:a.config.zoom_rescale,withSubchart:!1,withUpdateXDomain:!0,withDimension:!1}),a.config.subchart_onbrush.call(a.api,b.orgDomain())},i.transformContext=function(a,b){var c,d=this;b&&b.axisSubX?c=b.axisSubX:(c=d.context.select("."+l.axisX),a&&(c=c.transition())),d.context.attr("transform",d.getTranslate("context")),c.attr("transform",d.getTranslate("subx"))},i.getDefaultExtent=function(){var a=this,b=a.config,c=n(b.axis_x_extent)?b.axis_x_extent(a.getXDomain(a.data.targets)):b.axis_x_extent;return a.isTimeSeries()&&(c=[a.parseDate(c[0]),a.parseDate(c[1])]),c},i.initZoom=function(){var a,b=this,c=b.d3,d=b.config;b.zoom=c.behavior.zoom().on("zoomstart",function(){a=c.event.sourceEvent,b.zoom.altDomain=c.event.sourceEvent.altKey?b.x.orgDomain():null,d.zoom_onzoomstart.call(b.api,c.event.sourceEvent)}).on("zoom",function(){b.redrawForZoom.call(b)}).on("zoomend",function(){var e=c.event.sourceEvent;e&&a.clientX===e.clientX&&a.clientY===e.clientY||(b.redrawEventRect(),b.updateZoom(),d.zoom_onzoomend.call(b.api,b.x.orgDomain()))}),b.zoom.scale=function(a){return d.axis_rotated?this.y(a):this.x(a)},b.zoom.orgScaleExtent=function(){var a=d.zoom_extent?d.zoom_extent:[1,10];return[a[0],Math.max(b.getMaxDataCount()/a[1],a[1])]},b.zoom.updateScaleExtent=function(){var a=t(b.x.orgDomain())/t(b.getZoomDomain()),c=this.orgScaleExtent();return this.scaleExtent([c[0]*a,c[1]*a]),this}},i.getZoomDomain=function(){var a=this,b=a.config,c=a.d3,d=c.min([a.orgXDomain[0],b.zoom_x_min]),e=c.max([a.orgXDomain[1],b.zoom_x_max]);return[d,e]},i.updateZoom=function(){var a=this,b=a.config.zoom_enabled?a.zoom:function(){};a.main.select("."+l.zoomRect).call(b).on("dblclick.zoom",null),a.main.selectAll("."+l.eventRect).call(b).on("dblclick.zoom",null)},i.redrawForZoom=function(){var a=this,b=a.d3,c=a.config,d=a.zoom,e=a.x;if(c.zoom_enabled&&0!==a.filterTargetsToShow(a.data.targets).length){if("mousemove"===b.event.sourceEvent.type&&d.altDomain)return e.domain(d.altDomain),void d.scale(e).updateScaleExtent();a.isCategorized()&&e.orgDomain()[0]===a.orgXDomain[0]&&e.domain([a.orgXDomain[0]-1e-10,e.orgDomain()[1]]),a.redraw({withTransition:!1,withY:c.zoom_rescale,withSubchart:!1,withEventRect:!1,withDimension:!1}),"mousemove"===b.event.sourceEvent.type&&(a.cancelClick=!0),c.zoom_onzoom.call(a.api,e.orgDomain())}},i.generateColor=function(){var a=this,b=a.config,c=a.d3,d=b.data_colors,e=v(b.color_pattern)?b.color_pattern:c.scale.category10().range(),f=b.data_color,g=[];return function(a){var b,c=a.id||a.data&&a.data.id||a;return d[c]instanceof Function?b=d[c](a):d[c]?b=d[c]:(g.indexOf(c)<0&&g.push(c),b=e[g.indexOf(c)%e.length],d[c]=b),f instanceof Function?f(b,a):b}},i.generateLevelColor=function(){var a=this,b=a.config,c=b.color_pattern,d=b.color_threshold,e="value"===d.unit,f=d.values&&d.values.length?d.values:[],g=d.max||100;return v(b.color_threshold)?function(a){var b,d,h=c[c.length-1];for(b=0;b<f.length;b++)if(d=e?a:100*a/g,d<f[b]){h=c[b];break}return h}:null},i.getYFormat=function(a){var b=this,c=a&&!b.hasType("gauge")?b.defaultArcValueFormat:b.yFormat,d=a&&!b.hasType("gauge")?b.defaultArcValueFormat:b.y2Format;return function(a,e,f){var g="y2"===b.axis.getId(f)?d:c;return g.call(b,a,e)}},i.yFormat=function(a){var b=this,c=b.config,d=c.axis_y_tick_format?c.axis_y_tick_format:b.defaultValueFormat;return d(a)},i.y2Format=function(a){var b=this,c=b.config,d=c.axis_y2_tick_format?c.axis_y2_tick_format:b.defaultValueFormat;return d(a)},i.defaultValueFormat=function(a){return m(a)?+a:""},i.defaultArcValueFormat=function(a,b){return(100*b).toFixed(1)+"%"},i.dataLabelFormat=function(a){var b,c=this,d=c.config.data_labels,e=function(a){return m(a)?+a:""};return b="function"==typeof d.format?d.format:"object"==typeof d.format?d.format[a]?d.format[a]===!0?e:d.format[a]:function(){return""}:e},i.hasCaches=function(a){for(var b=0;b<a.length;b++)if(!(a[b]in this.cache))return!1;return!0},i.addCache=function(a,b){this.cache[a]=this.cloneTarget(b)},i.getCaches=function(a){var b,c=[];for(b=0;b<a.length;b++)a[b]in this.cache&&c.push(this.cloneTarget(this.cache[a[b]]));return c};var l=i.CLASS={target:"c3-target",chart:"c3-chart",chartLine:"c3-chart-line",chartLines:"c3-chart-lines",chartBar:"c3-chart-bar",chartBars:"c3-chart-bars",chartText:"c3-chart-text",chartTexts:"c3-chart-texts",chartArc:"c3-chart-arc",chartArcs:"c3-chart-arcs",chartArcsTitle:"c3-chart-arcs-title",chartArcsBackground:"c3-chart-arcs-background",chartArcsGaugeUnit:"c3-chart-arcs-gauge-unit",chartArcsGaugeMax:"c3-chart-arcs-gauge-max",chartArcsGaugeMin:"c3-chart-arcs-gauge-min",selectedCircle:"c3-selected-circle",selectedCircles:"c3-selected-circles",eventRect:"c3-event-rect",eventRects:"c3-event-rects",eventRectsSingle:"c3-event-rects-single",eventRectsMultiple:"c3-event-rects-multiple",zoomRect:"c3-zoom-rect",brush:"c3-brush",focused:"c3-focused",defocused:"c3-defocused",region:"c3-region",regions:"c3-regions",title:"c3-title",tooltipContainer:"c3-tooltip-container",tooltip:"c3-tooltip",tooltipName:"c3-tooltip-name",shape:"c3-shape",shapes:"c3-shapes",line:"c3-line",lines:"c3-lines",bar:"c3-bar",bars:"c3-bars",circle:"c3-circle",circles:"c3-circles",arc:"c3-arc",arcs:"c3-arcs",area:"c3-area",areas:"c3-areas",empty:"c3-empty",text:"c3-text",texts:"c3-texts",gaugeValue:"c3-gauge-value",grid:"c3-grid",gridLines:"c3-grid-lines",xgrid:"c3-xgrid",xgrids:"c3-xgrids",xgridLine:"c3-xgrid-line",xgridLines:"c3-xgrid-lines",xgridFocus:"c3-xgrid-focus",ygrid:"c3-ygrid",ygrids:"c3-ygrids",ygridLine:"c3-ygrid-line",ygridLines:"c3-ygrid-lines",axis:"c3-axis",axisX:"c3-axis-x",axisXLabel:"c3-axis-x-label",axisY:"c3-axis-y",axisYLabel:"c3-axis-y-label",axisY2:"c3-axis-y2",axisY2Label:"c3-axis-y2-label",legendBackground:"c3-legend-background",legendItem:"c3-legend-item",legendItemEvent:"c3-legend-item-event",legendItemTile:"c3-legend-item-tile",legendItemHidden:"c3-legend-item-hidden",legendItemFocused:"c3-legend-item-focused",dragarea:"c3-dragarea",EXPANDED:"_expanded_",SELECTED:"_selected_",INCLUDED:"_included_"};i.generateClass=function(a,b){return" "+a+" "+a+this.getTargetSelectorSuffix(b)},i.classText=function(a){return this.generateClass(l.text,a.index)},i.classTexts=function(a){return this.generateClass(l.texts,a.id)},i.classShape=function(a){return this.generateClass(l.shape,a.index)},i.classShapes=function(a){return this.generateClass(l.shapes,a.id)},i.classLine=function(a){return this.classShape(a)+this.generateClass(l.line,a.id)},i.classLines=function(a){return this.classShapes(a)+this.generateClass(l.lines,a.id)},i.classCircle=function(a){return this.classShape(a)+this.generateClass(l.circle,a.index)},i.classCircles=function(a){return this.classShapes(a)+this.generateClass(l.circles,a.id)},i.classBar=function(a){return this.classShape(a)+this.generateClass(l.bar,a.index)},i.classBars=function(a){return this.classShapes(a)+this.generateClass(l.bars,a.id)},i.classArc=function(a){return this.classShape(a.data)+this.generateClass(l.arc,a.data.id)},i.classArcs=function(a){return this.classShapes(a.data)+this.generateClass(l.arcs,a.data.id)},i.classArea=function(a){return this.classShape(a)+this.generateClass(l.area,a.id)},i.classAreas=function(a){return this.classShapes(a)+this.generateClass(l.areas,a.id)},i.classRegion=function(a,b){return this.generateClass(l.region,b)+" "+("class"in a?a["class"]:"")},i.classEvent=function(a){return this.generateClass(l.eventRect,a.index)},i.classTarget=function(a){var b=this,c=b.config.data_classes[a],d="";return c&&(d=" "+l.target+"-"+c),b.generateClass(l.target,a)+d},i.classFocus=function(a){return this.classFocused(a)+this.classDefocused(a)},i.classFocused=function(a){return" "+(this.focusedTargetIds.indexOf(a.id)>=0?l.focused:"")},i.classDefocused=function(a){return" "+(this.defocusedTargetIds.indexOf(a.id)>=0?l.defocused:"")},i.classChartText=function(a){return l.chartText+this.classTarget(a.id)},i.classChartLine=function(a){return l.chartLine+this.classTarget(a.id)},i.classChartBar=function(a){return l.chartBar+this.classTarget(a.id)},i.classChartArc=function(a){return l.chartArc+this.classTarget(a.data.id)},i.getTargetSelectorSuffix=function(a){return a||0===a?("-"+a).replace(/[\s?!@#$%^&*()_=+,.<>'":;\[\]\/|~`{}\\]/g,"-"):""},i.selectorTarget=function(a,b){return(b||"")+"."+l.target+this.getTargetSelectorSuffix(a)},i.selectorTargets=function(a,b){var c=this;return a=a||[],a.length?a.map(function(a){return c.selectorTarget(a,b)}):null},i.selectorLegend=function(a){return"."+l.legendItem+this.getTargetSelectorSuffix(a)},i.selectorLegends=function(a){var b=this;return a&&a.length?a.map(function(a){return b.selectorLegend(a)}):null};var m=i.isValue=function(a){return a||0===a},n=i.isFunction=function(a){return"function"==typeof a},o=i.isString=function(a){return"string"==typeof a},p=i.isUndefined=function(a){return"undefined"==typeof a},q=i.isDefined=function(a){return"undefined"!=typeof a},r=i.ceil10=function(a){return 10*Math.ceil(a/10)},s=i.asHalfPixel=function(a){return Math.ceil(a)+.5},t=i.diffDomain=function(a){return a[1]-a[0]},u=i.isEmpty=function(a){return"undefined"==typeof a||null===a||o(a)&&0===a.length||"object"==typeof a&&0===Object.keys(a).length},v=i.notEmpty=function(a){return!i.isEmpty(a)},w=i.getOption=function(a,b,c){return q(a[b])?a[b]:c},x=i.hasValue=function(a,b){var c=!1;return Object.keys(a).forEach(function(d){a[d]===b&&(c=!0)}),c},y=i.sanitise=function(a){return"string"==typeof a?a.replace(/</g,"&lt;").replace(/>/g,"&gt;"):a},z=i.getPathBox=function(a){var b=a.getBoundingClientRect(),c=[a.pathSegList.getItem(0),a.pathSegList.getItem(1)],d=c[0].x,e=Math.min(c[0].y,c[1].y);return{x:d,y:e,width:b.width,height:b.height}};h.focus=function(a){var b,c=this.internal;a=c.mapToTargetIds(a),b=c.svg.selectAll(c.selectorTargets(a.filter(c.isTargetToShow,c))),this.revert(),this.defocus(),b.classed(l.focused,!0).classed(l.defocused,!1),
c.hasArcType()&&c.expandArc(a),c.toggleFocusLegend(a,!0),c.focusedTargetIds=a,c.defocusedTargetIds=c.defocusedTargetIds.filter(function(b){return a.indexOf(b)<0})},h.defocus=function(a){var b,c=this.internal;a=c.mapToTargetIds(a),b=c.svg.selectAll(c.selectorTargets(a.filter(c.isTargetToShow,c))),b.classed(l.focused,!1).classed(l.defocused,!0),c.hasArcType()&&c.unexpandArc(a),c.toggleFocusLegend(a,!1),c.focusedTargetIds=c.focusedTargetIds.filter(function(b){return a.indexOf(b)<0}),c.defocusedTargetIds=a},h.revert=function(a){var b,c=this.internal;a=c.mapToTargetIds(a),b=c.svg.selectAll(c.selectorTargets(a)),b.classed(l.focused,!1).classed(l.defocused,!1),c.hasArcType()&&c.unexpandArc(a),c.config.legend_show&&(c.showLegend(a.filter(c.isLegendToShow.bind(c))),c.legend.selectAll(c.selectorLegends(a)).filter(function(){return c.d3.select(this).classed(l.legendItemFocused)}).classed(l.legendItemFocused,!1)),c.focusedTargetIds=[],c.defocusedTargetIds=[]},h.show=function(a,b){var c,d=this.internal;a=d.mapToTargetIds(a),b=b||{},d.removeHiddenTargetIds(a),c=d.svg.selectAll(d.selectorTargets(a)),c.transition().style("opacity",1,"important").call(d.endall,function(){c.style("opacity",null).style("opacity",1)}),b.withLegend&&d.showLegend(a),d.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0})},h.hide=function(a,b){var c,d=this.internal;a=d.mapToTargetIds(a),b=b||{},d.addHiddenTargetIds(a),c=d.svg.selectAll(d.selectorTargets(a)),c.transition().style("opacity",0,"important").call(d.endall,function(){c.style("opacity",null).style("opacity",0)}),b.withLegend&&d.hideLegend(a),d.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0})},h.toggle=function(a,b){var c=this,d=this.internal;d.mapToTargetIds(a).forEach(function(a){d.isTargetToShow(a)?c.hide(a,b):c.show(a,b)})},h.zoom=function(a){var b=this.internal;return a&&(b.isTimeSeries()&&(a=a.map(function(a){return b.parseDate(a)})),b.brush.extent(a),b.redraw({withUpdateXDomain:!0,withY:b.config.zoom_rescale}),b.config.zoom_onzoom.call(this,b.x.orgDomain())),b.brush.extent()},h.zoom.enable=function(a){var b=this.internal;b.config.zoom_enabled=a,b.updateAndRedraw()},h.unzoom=function(){var a=this.internal;a.brush.clear().update(),a.redraw({withUpdateXDomain:!0})},h.zoom.max=function(a){var b=this.internal,c=b.config,d=b.d3;return 0===a||a?void(c.zoom_x_max=d.max([b.orgXDomain[1],a])):c.zoom_x_max},h.zoom.min=function(a){var b=this.internal,c=b.config,d=b.d3;return 0===a||a?void(c.zoom_x_min=d.min([b.orgXDomain[0],a])):c.zoom_x_min},h.zoom.range=function(a){return arguments.length?(q(a.max)&&this.domain.max(a.max),void(q(a.min)&&this.domain.min(a.min))):{max:this.domain.max(),min:this.domain.min()}},h.load=function(a){var b=this.internal,c=b.config;return a.xs&&b.addXs(a.xs),"names"in a&&h.data.names.bind(this)(a.names),"classes"in a&&Object.keys(a.classes).forEach(function(b){c.data_classes[b]=a.classes[b]}),"categories"in a&&b.isCategorized()&&(c.axis_x_categories=a.categories),"axes"in a&&Object.keys(a.axes).forEach(function(b){c.data_axes[b]=a.axes[b]}),"colors"in a&&Object.keys(a.colors).forEach(function(b){c.data_colors[b]=a.colors[b]}),"cacheIds"in a&&b.hasCaches(a.cacheIds)?void b.load(b.getCaches(a.cacheIds),a.done):void("unload"in a?b.unload(b.mapToTargetIds("boolean"==typeof a.unload&&a.unload?null:a.unload),function(){b.loadFromArgs(a)}):b.loadFromArgs(a))},h.unload=function(a){var b=this.internal;a=a||{},a instanceof Array?a={ids:a}:"string"==typeof a&&(a={ids:[a]}),b.unload(b.mapToTargetIds(a.ids),function(){b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0,withLegend:!0}),a.done&&a.done()})},h.flow=function(a){var b,c,d,e,f,g,h,i,j=this.internal,k=[],l=j.getMaxDataCount(),n=0,o=0;if(a.json)c=j.convertJsonToData(a.json,a.keys);else if(a.rows)c=j.convertRowsToData(a.rows);else{if(!a.columns)return;c=j.convertColumnsToData(a.columns)}b=j.convertDataToTargets(c,!0),j.data.targets.forEach(function(a){var c,d,e=!1;for(c=0;c<b.length;c++)if(a.id===b[c].id){for(e=!0,a.values[a.values.length-1]&&(o=a.values[a.values.length-1].index+1),n=b[c].values.length,d=0;n>d;d++)b[c].values[d].index=o+d,j.isTimeSeries()||(b[c].values[d].x=o+d);a.values=a.values.concat(b[c].values),b.splice(c,1);break}e||k.push(a.id)}),j.data.targets.forEach(function(a){var b,c;for(b=0;b<k.length;b++)if(a.id===k[b])for(o=a.values[a.values.length-1].index+1,c=0;n>c;c++)a.values.push({id:a.id,index:o+c,x:j.isTimeSeries()?j.getOtherTargetX(o+c):o+c,value:null})}),j.data.targets.length&&b.forEach(function(a){var b,c=[];for(b=j.data.targets[0].values[0].index;o>b;b++)c.push({id:a.id,index:b,x:j.isTimeSeries()?j.getOtherTargetX(b):b,value:null});a.values.forEach(function(a){a.index+=o,j.isTimeSeries()||(a.x+=o)}),a.values=c.concat(a.values)}),j.data.targets=j.data.targets.concat(b),d=j.getMaxDataCount(),f=j.data.targets[0],g=f.values[0],q(a.to)?(n=0,i=j.isTimeSeries()?j.parseDate(a.to):a.to,f.values.forEach(function(a){a.x<i&&n++})):q(a.length)&&(n=a.length),l?1===l&&j.isTimeSeries()&&(h=(f.values[f.values.length-1].x-g.x)/2,e=[new Date(+g.x-h),new Date(+g.x+h)],j.updateXDomain(null,!0,!0,!1,e)):(h=j.isTimeSeries()?f.values.length>1?f.values[f.values.length-1].x-g.x:g.x-j.getXDomain(j.data.targets)[0]:1,e=[g.x-h,g.x],j.updateXDomain(null,!0,!0,!1,e)),j.updateTargets(j.data.targets),j.redraw({flow:{index:g.index,length:n,duration:m(a.duration)?a.duration:j.config.transition_duration,done:a.done,orgDataCount:l},withLegend:!0,withTransition:l>1,withTrimXDomain:!1,withUpdateXAxis:!0})},i.generateFlow=function(a){var b=this,c=b.config,d=b.d3;return function(){var e,f,g,h=a.targets,i=a.flow,j=a.drawBar,k=a.drawLine,m=a.drawArea,n=a.cx,o=a.cy,p=a.xv,q=a.xForText,r=a.yForText,s=a.duration,u=1,v=i.index,w=i.length,x=b.getValueOnIndex(b.data.targets[0].values,v),y=b.getValueOnIndex(b.data.targets[0].values,v+w),z=b.x.domain(),A=i.duration||s,B=i.done||function(){},C=b.generateWait(),D=b.xgrid||d.selectAll([]),E=b.xgridLines||d.selectAll([]),F=b.mainRegion||d.selectAll([]),G=b.mainText||d.selectAll([]),H=b.mainBar||d.selectAll([]),I=b.mainLine||d.selectAll([]),J=b.mainArea||d.selectAll([]),K=b.mainCircle||d.selectAll([]);b.flowing=!0,b.data.targets.forEach(function(a){a.values.splice(0,w)}),g=b.updateXDomain(h,!0,!0),b.updateXGrid&&b.updateXGrid(!0),i.orgDataCount?e=1===i.orgDataCount||(x&&x.x)===(y&&y.x)?b.x(z[0])-b.x(g[0]):b.isTimeSeries()?b.x(z[0])-b.x(g[0]):b.x(x.x)-b.x(y.x):1!==b.data.targets[0].values.length?e=b.x(z[0])-b.x(g[0]):b.isTimeSeries()?(x=b.getValueOnIndex(b.data.targets[0].values,0),y=b.getValueOnIndex(b.data.targets[0].values,b.data.targets[0].values.length-1),e=b.x(x.x)-b.x(y.x)):e=t(g)/2,u=t(z)/t(g),f="translate("+e+",0) scale("+u+",1)",b.hideXGridFocus(),d.transition().ease("linear").duration(A).each(function(){C.add(b.axes.x.transition().call(b.xAxis)),C.add(H.transition().attr("transform",f)),C.add(I.transition().attr("transform",f)),C.add(J.transition().attr("transform",f)),C.add(K.transition().attr("transform",f)),C.add(G.transition().attr("transform",f)),C.add(F.filter(b.isRegionOnX).transition().attr("transform",f)),C.add(D.transition().attr("transform",f)),C.add(E.transition().attr("transform",f))}).call(C,function(){var a,d=[],e=[],f=[];if(w){for(a=0;w>a;a++)d.push("."+l.shape+"-"+(v+a)),e.push("."+l.text+"-"+(v+a)),f.push("."+l.eventRect+"-"+(v+a));b.svg.selectAll("."+l.shapes).selectAll(d).remove(),b.svg.selectAll("."+l.texts).selectAll(e).remove(),b.svg.selectAll("."+l.eventRects).selectAll(f).remove(),b.svg.select("."+l.xgrid).remove()}D.attr("transform",null).attr(b.xgridAttr),E.attr("transform",null),E.select("line").attr("x1",c.axis_rotated?0:p).attr("x2",c.axis_rotated?b.width:p),E.select("text").attr("x",c.axis_rotated?b.width:0).attr("y",p),H.attr("transform",null).attr("d",j),I.attr("transform",null).attr("d",k),J.attr("transform",null).attr("d",m),K.attr("transform",null).attr("cx",n).attr("cy",o),G.attr("transform",null).attr("x",q).attr("y",r).style("fill-opacity",b.opacityForText.bind(b)),F.attr("transform",null),F.select("rect").filter(b.isRegionOnX).attr("x",b.regionX.bind(b)).attr("width",b.regionWidth.bind(b)),c.interaction_enabled&&b.redrawEventRect(),B(),b.flowing=!1})}},h.selected=function(a){var b=this.internal,c=b.d3;return c.merge(b.main.selectAll("."+l.shapes+b.getTargetSelectorSuffix(a)).selectAll("."+l.shape).filter(function(){return c.select(this).classed(l.SELECTED)}).map(function(a){return a.map(function(a){var b=a.__data__;return b.data?b.data:b})}))},h.select=function(a,b,c){var d=this.internal,e=d.d3,f=d.config;f.data_selection_enabled&&d.main.selectAll("."+l.shapes).selectAll("."+l.shape).each(function(g,h){var i=e.select(this),j=g.data?g.data.id:g.id,k=d.getToggle(this,g).bind(d),m=f.data_selection_grouped||!a||a.indexOf(j)>=0,n=!b||b.indexOf(h)>=0,o=i.classed(l.SELECTED);i.classed(l.line)||i.classed(l.area)||(m&&n?f.data_selection_isselectable(g)&&!o&&k(!0,i.classed(l.SELECTED,!0),g,h):q(c)&&c&&o&&k(!1,i.classed(l.SELECTED,!1),g,h))})},h.unselect=function(a,b){var c=this.internal,d=c.d3,e=c.config;e.data_selection_enabled&&c.main.selectAll("."+l.shapes).selectAll("."+l.shape).each(function(f,g){var h=d.select(this),i=f.data?f.data.id:f.id,j=c.getToggle(this,f).bind(c),k=e.data_selection_grouped||!a||a.indexOf(i)>=0,m=!b||b.indexOf(g)>=0,n=h.classed(l.SELECTED);h.classed(l.line)||h.classed(l.area)||k&&m&&e.data_selection_isselectable(f)&&n&&j(!1,h.classed(l.SELECTED,!1),f,g)})},h.transform=function(a,b){var c=this.internal,d=["pie","donut"].indexOf(a)>=0?{withTransform:!0}:null;c.transformTo(b,a,d)},i.transformTo=function(a,b,c){var d=this,e=!d.hasArcType(),f=c||{withTransitionForAxis:e};f.withTransitionForTransform=!1,d.transiting=!1,d.setTargetType(a,b),d.updateTargets(d.data.targets),d.updateAndRedraw(f)},h.groups=function(a){var b=this.internal,c=b.config;return p(a)?c.data_groups:(c.data_groups=a,b.redraw(),c.data_groups)},h.xgrids=function(a){var b=this.internal,c=b.config;return a?(c.grid_x_lines=a,b.redrawWithoutRescale(),c.grid_x_lines):c.grid_x_lines},h.xgrids.add=function(a){var b=this.internal;return this.xgrids(b.config.grid_x_lines.concat(a?a:[]))},h.xgrids.remove=function(a){var b=this.internal;b.removeGridLines(a,!0)},h.ygrids=function(a){var b=this.internal,c=b.config;return a?(c.grid_y_lines=a,b.redrawWithoutRescale(),c.grid_y_lines):c.grid_y_lines},h.ygrids.add=function(a){var b=this.internal;return this.ygrids(b.config.grid_y_lines.concat(a?a:[]))},h.ygrids.remove=function(a){var b=this.internal;b.removeGridLines(a,!1)},h.regions=function(a){var b=this.internal,c=b.config;return a?(c.regions=a,b.redrawWithoutRescale(),c.regions):c.regions},h.regions.add=function(a){var b=this.internal,c=b.config;return a?(c.regions=c.regions.concat(a),b.redrawWithoutRescale(),c.regions):c.regions},h.regions.remove=function(a){var b,c,d,e=this.internal,f=e.config;return a=a||{},b=e.getOption(a,"duration",f.transition_duration),c=e.getOption(a,"classes",[l.region]),d=e.main.select("."+l.regions).selectAll(c.map(function(a){return"."+a})),(b?d.transition().duration(b):d).style("opacity",0).remove(),f.regions=f.regions.filter(function(a){var b=!1;return a["class"]?(a["class"].split(" ").forEach(function(a){c.indexOf(a)>=0&&(b=!0)}),!b):!0}),f.regions},h.data=function(a){var b=this.internal.data.targets;return"undefined"==typeof a?b:b.filter(function(b){return[].concat(a).indexOf(b.id)>=0})},h.data.shown=function(a){return this.internal.filterTargetsToShow(this.data(a))},h.data.values=function(a){var b,c=null;return a&&(b=this.data(a),c=b[0]?b[0].values.map(function(a){return a.value}):null),c},h.data.names=function(a){return this.internal.clearLegendItemTextBoxCache(),this.internal.updateDataAttributes("names",a)},h.data.colors=function(a){return this.internal.updateDataAttributes("colors",a)},h.data.axes=function(a){return this.internal.updateDataAttributes("axes",a)},h.category=function(a,b){var c=this.internal,d=c.config;return arguments.length>1&&(d.axis_x_categories[a]=b,c.redraw()),d.axis_x_categories[a]},h.categories=function(a){var b=this.internal,c=b.config;return arguments.length?(c.axis_x_categories=a,b.redraw(),c.axis_x_categories):c.axis_x_categories},h.color=function(a){var b=this.internal;return b.color(a)},h.x=function(a){var b=this.internal;return arguments.length&&(b.updateTargetX(b.data.targets,a),b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})),b.data.xs},h.xs=function(a){var b=this.internal;return arguments.length&&(b.updateTargetXs(b.data.targets,a),b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})),b.data.xs},h.axis=function(){},h.axis.labels=function(a){var b=this.internal;arguments.length&&(Object.keys(a).forEach(function(c){b.axis.setLabelText(c,a[c])}),b.axis.updateLabels())},h.axis.max=function(a){var b=this.internal,c=b.config;return arguments.length?("object"==typeof a?(m(a.x)&&(c.axis_x_max=a.x),m(a.y)&&(c.axis_y_max=a.y),m(a.y2)&&(c.axis_y2_max=a.y2)):c.axis_y_max=c.axis_y2_max=a,void b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})):{x:c.axis_x_max,y:c.axis_y_max,y2:c.axis_y2_max}},h.axis.min=function(a){var b=this.internal,c=b.config;return arguments.length?("object"==typeof a?(m(a.x)&&(c.axis_x_min=a.x),m(a.y)&&(c.axis_y_min=a.y),m(a.y2)&&(c.axis_y2_min=a.y2)):c.axis_y_min=c.axis_y2_min=a,void b.redraw({withUpdateOrgXDomain:!0,withUpdateXDomain:!0})):{x:c.axis_x_min,y:c.axis_y_min,y2:c.axis_y2_min}},h.axis.range=function(a){return arguments.length?(q(a.max)&&this.axis.max(a.max),void(q(a.min)&&this.axis.min(a.min))):{max:this.axis.max(),min:this.axis.min()}},h.legend=function(){},h.legend.show=function(a){var b=this.internal;b.showLegend(b.mapToTargetIds(a)),b.updateAndRedraw({withLegend:!0})},h.legend.hide=function(a){var b=this.internal;b.hideLegend(b.mapToTargetIds(a)),b.updateAndRedraw({withLegend:!0})},h.resize=function(a){var b=this.internal,c=b.config;c.size_width=a?a.width:null,c.size_height=a?a.height:null,this.flush()},h.flush=function(){var a=this.internal;a.updateAndRedraw({withLegend:!0,withTransition:!1,withTransitionForTransform:!1})},h.destroy=function(){var b=this.internal;if(a.clearInterval(b.intervalForObserveInserted),void 0!==b.resizeTimeout&&a.clearTimeout(b.resizeTimeout),a.detachEvent)a.detachEvent("onresize",b.resizeFunction);else if(a.removeEventListener)a.removeEventListener("resize",b.resizeFunction);else{var c=a.onresize;c&&c.add&&c.remove&&c.remove(b.resizeFunction)}return b.selectChart.classed("c3",!1).html(""),Object.keys(b).forEach(function(a){b[a]=null}),null},h.tooltip=function(){},h.tooltip.show=function(a){var b,c,d=this.internal;a.mouse&&(c=a.mouse),a.data?d.isMultipleX()?(c=[d.x(a.data.x),d.getYScale(a.data.id)(a.data.value)],b=null):b=m(a.data.index)?a.data.index:d.getIndexByX(a.data.x):"undefined"!=typeof a.x?b=d.getIndexByX(a.x):"undefined"!=typeof a.index&&(b=a.index),d.dispatchEvent("mouseover",b,c),d.dispatchEvent("mousemove",b,c),d.config.tooltip_onshow.call(d,a.data)},h.tooltip.hide=function(){this.internal.dispatchEvent("mouseout",0),this.internal.config.tooltip_onhide.call(this)};var A;i.isSafari=function(){var b=a.navigator.userAgent;return b.indexOf("Safari")>=0&&b.indexOf("Chrome")<0},i.isChrome=function(){var b=a.navigator.userAgent;return b.indexOf("Chrome")>=0},Function.prototype.bind||(Function.prototype.bind=function(a){if("function"!=typeof this)throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");var b=Array.prototype.slice.call(arguments,1),c=this,d=function(){},e=function(){return c.apply(this instanceof d?this:a,b.concat(Array.prototype.slice.call(arguments)))};return d.prototype=this.prototype,e.prototype=new d,e}),function(){"SVGPathSeg"in a||(a.SVGPathSeg=function(a,b,c){this.pathSegType=a,this.pathSegTypeAsLetter=b,this._owningPathSegList=c},SVGPathSeg.PATHSEG_UNKNOWN=0,SVGPathSeg.PATHSEG_CLOSEPATH=1,SVGPathSeg.PATHSEG_MOVETO_ABS=2,SVGPathSeg.PATHSEG_MOVETO_REL=3,SVGPathSeg.PATHSEG_LINETO_ABS=4,SVGPathSeg.PATHSEG_LINETO_REL=5,SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS=6,SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL=7,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS=8,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL=9,SVGPathSeg.PATHSEG_ARC_ABS=10,SVGPathSeg.PATHSEG_ARC_REL=11,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS=12,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL=13,SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS=14,SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL=15,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS=16,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL=17,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS=18,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL=19,SVGPathSeg.prototype._segmentChanged=function(){this._owningPathSegList&&this._owningPathSegList.segmentChanged(this)},a.SVGPathSegClosePath=function(a){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CLOSEPATH,"z",a)},SVGPathSegClosePath.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegClosePath.prototype.toString=function(){return"[object SVGPathSegClosePath]"},SVGPathSegClosePath.prototype._asPathString=function(){return this.pathSegTypeAsLetter},SVGPathSegClosePath.prototype.clone=function(){return new SVGPathSegClosePath(void 0)},a.SVGPathSegMovetoAbs=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_MOVETO_ABS,"M",a),this._x=b,this._y=c},SVGPathSegMovetoAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegMovetoAbs.prototype.toString=function(){return"[object SVGPathSegMovetoAbs]"},SVGPathSegMovetoAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegMovetoAbs.prototype.clone=function(){return new SVGPathSegMovetoAbs(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegMovetoAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegMovetoAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegMovetoRel=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_MOVETO_REL,"m",a),this._x=b,this._y=c},SVGPathSegMovetoRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegMovetoRel.prototype.toString=function(){return"[object SVGPathSegMovetoRel]"},SVGPathSegMovetoRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegMovetoRel.prototype.clone=function(){return new SVGPathSegMovetoRel(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegMovetoRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegMovetoRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoAbs=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_ABS,"L",a),this._x=b,this._y=c},SVGPathSegLinetoAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoAbs.prototype.toString=function(){return"[object SVGPathSegLinetoAbs]"},SVGPathSegLinetoAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegLinetoAbs.prototype.clone=function(){return new SVGPathSegLinetoAbs(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegLinetoAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegLinetoAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoRel=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_REL,"l",a),this._x=b,this._y=c},SVGPathSegLinetoRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoRel.prototype.toString=function(){return"[object SVGPathSegLinetoRel]"},SVGPathSegLinetoRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegLinetoRel.prototype.clone=function(){return new SVGPathSegLinetoRel(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegLinetoRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegLinetoRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicAbs=function(a,b,c,d,e,f,g){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS,"C",a),this._x=b,this._y=c,this._x1=d,this._y1=e,this._x2=f,this._y2=g},SVGPathSegCurvetoCubicAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicAbs]"},SVGPathSegCurvetoCubicAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicAbs.prototype.clone=function(){return new SVGPathSegCurvetoCubicAbs(void 0,this._x,this._y,this._x1,this._y1,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicAbs.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicRel=function(a,b,c,d,e,f,g){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL,"c",a),this._x=b,this._y=c,this._x1=d,this._y1=e,this._x2=f,this._y2=g},SVGPathSegCurvetoCubicRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicRel.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicRel]"},SVGPathSegCurvetoCubicRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicRel.prototype.clone=function(){return new SVGPathSegCurvetoCubicRel(void 0,this._x,this._y,this._x1,this._y1,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicRel.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticAbs=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS,"Q",a),this._x=b,this._y=c,this._x1=d,this._y1=e},SVGPathSegCurvetoQuadraticAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticAbs]"},SVGPathSegCurvetoQuadraticAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticAbs.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticAbs(void 0,this._x,this._y,this._x1,this._y1)},Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticAbs.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticRel=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL,"q",a),this._x=b,this._y=c,this._x1=d,this._y1=e},SVGPathSegCurvetoQuadraticRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticRel.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticRel]"},SVGPathSegCurvetoQuadraticRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x1+" "+this._y1+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticRel.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticRel(void 0,this._x,this._y,this._x1,this._y1)},Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"x1",{get:function(){return this._x1},set:function(a){this._x1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticRel.prototype,"y1",{get:function(){return this._y1},set:function(a){this._y1=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegArcAbs=function(a,b,c,d,e,f,g,h){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_ARC_ABS,"A",a),this._x=b,this._y=c,this._r1=d,this._r2=e,this._angle=f,this._largeArcFlag=g,this._sweepFlag=h},SVGPathSegArcAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegArcAbs.prototype.toString=function(){return"[object SVGPathSegArcAbs]"},SVGPathSegArcAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._r1+" "+this._r2+" "+this._angle+" "+(this._largeArcFlag?"1":"0")+" "+(this._sweepFlag?"1":"0")+" "+this._x+" "+this._y},SVGPathSegArcAbs.prototype.clone=function(){return new SVGPathSegArcAbs(void 0,this._x,this._y,this._r1,this._r2,this._angle,this._largeArcFlag,this._sweepFlag)},Object.defineProperty(SVGPathSegArcAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"r1",{get:function(){return this._r1},set:function(a){this._r1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"r2",{get:function(){return this._r2},set:function(a){this._r2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"angle",{get:function(){return this._angle},set:function(a){this._angle=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"largeArcFlag",{get:function(){return this._largeArcFlag},set:function(a){this._largeArcFlag=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcAbs.prototype,"sweepFlag",{get:function(){return this._sweepFlag},set:function(a){this._sweepFlag=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegArcRel=function(a,b,c,d,e,f,g,h){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_ARC_REL,"a",a),this._x=b,this._y=c,this._r1=d,this._r2=e,this._angle=f,this._largeArcFlag=g,this._sweepFlag=h},SVGPathSegArcRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegArcRel.prototype.toString=function(){return"[object SVGPathSegArcRel]"},SVGPathSegArcRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._r1+" "+this._r2+" "+this._angle+" "+(this._largeArcFlag?"1":"0")+" "+(this._sweepFlag?"1":"0")+" "+this._x+" "+this._y},SVGPathSegArcRel.prototype.clone=function(){return new SVGPathSegArcRel(void 0,this._x,this._y,this._r1,this._r2,this._angle,this._largeArcFlag,this._sweepFlag)},Object.defineProperty(SVGPathSegArcRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"r1",{get:function(){return this._r1},set:function(a){this._r1=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"r2",{get:function(){return this._r2},set:function(a){this._r2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"angle",{get:function(){return this._angle},set:function(a){this._angle=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"largeArcFlag",{get:function(){return this._largeArcFlag},set:function(a){this._largeArcFlag=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegArcRel.prototype,"sweepFlag",{get:function(){return this._sweepFlag},set:function(a){this._sweepFlag=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoHorizontalAbs=function(a,b){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS,"H",a),this._x=b},SVGPathSegLinetoHorizontalAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoHorizontalAbs.prototype.toString=function(){return"[object SVGPathSegLinetoHorizontalAbs]"},SVGPathSegLinetoHorizontalAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x},SVGPathSegLinetoHorizontalAbs.prototype.clone=function(){return new SVGPathSegLinetoHorizontalAbs(void 0,this._x)},Object.defineProperty(SVGPathSegLinetoHorizontalAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoHorizontalRel=function(a,b){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL,"h",a),this._x=b},SVGPathSegLinetoHorizontalRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoHorizontalRel.prototype.toString=function(){return"[object SVGPathSegLinetoHorizontalRel]"},SVGPathSegLinetoHorizontalRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x},SVGPathSegLinetoHorizontalRel.prototype.clone=function(){return new SVGPathSegLinetoHorizontalRel(void 0,this._x)},Object.defineProperty(SVGPathSegLinetoHorizontalRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoVerticalAbs=function(a,b){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS,"V",a),this._y=b},SVGPathSegLinetoVerticalAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoVerticalAbs.prototype.toString=function(){return"[object SVGPathSegLinetoVerticalAbs]"},SVGPathSegLinetoVerticalAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._y},SVGPathSegLinetoVerticalAbs.prototype.clone=function(){return new SVGPathSegLinetoVerticalAbs(void 0,this._y)},Object.defineProperty(SVGPathSegLinetoVerticalAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegLinetoVerticalRel=function(a,b){
SVGPathSeg.call(this,SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL,"v",a),this._y=b},SVGPathSegLinetoVerticalRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegLinetoVerticalRel.prototype.toString=function(){return"[object SVGPathSegLinetoVerticalRel]"},SVGPathSegLinetoVerticalRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._y},SVGPathSegLinetoVerticalRel.prototype.clone=function(){return new SVGPathSegLinetoVerticalRel(void 0,this._y)},Object.defineProperty(SVGPathSegLinetoVerticalRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicSmoothAbs=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS,"S",a),this._x=b,this._y=c,this._x2=d,this._y2=e},SVGPathSegCurvetoCubicSmoothAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicSmoothAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicSmoothAbs]"},SVGPathSegCurvetoCubicSmoothAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicSmoothAbs.prototype.clone=function(){return new SVGPathSegCurvetoCubicSmoothAbs(void 0,this._x,this._y,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothAbs.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoCubicSmoothRel=function(a,b,c,d,e){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL,"s",a),this._x=b,this._y=c,this._x2=d,this._y2=e},SVGPathSegCurvetoCubicSmoothRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoCubicSmoothRel.prototype.toString=function(){return"[object SVGPathSegCurvetoCubicSmoothRel]"},SVGPathSegCurvetoCubicSmoothRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x2+" "+this._y2+" "+this._x+" "+this._y},SVGPathSegCurvetoCubicSmoothRel.prototype.clone=function(){return new SVGPathSegCurvetoCubicSmoothRel(void 0,this._x,this._y,this._x2,this._y2)},Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"x2",{get:function(){return this._x2},set:function(a){this._x2=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoCubicSmoothRel.prototype,"y2",{get:function(){return this._y2},set:function(a){this._y2=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticSmoothAbs=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS,"T",a),this._x=b,this._y=c},SVGPathSegCurvetoQuadraticSmoothAbs.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticSmoothAbs.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticSmoothAbs]"},SVGPathSegCurvetoQuadraticSmoothAbs.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticSmoothAbs.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticSmoothAbs(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothAbs.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothAbs.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),a.SVGPathSegCurvetoQuadraticSmoothRel=function(a,b,c){SVGPathSeg.call(this,SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL,"t",a),this._x=b,this._y=c},SVGPathSegCurvetoQuadraticSmoothRel.prototype=Object.create(SVGPathSeg.prototype),SVGPathSegCurvetoQuadraticSmoothRel.prototype.toString=function(){return"[object SVGPathSegCurvetoQuadraticSmoothRel]"},SVGPathSegCurvetoQuadraticSmoothRel.prototype._asPathString=function(){return this.pathSegTypeAsLetter+" "+this._x+" "+this._y},SVGPathSegCurvetoQuadraticSmoothRel.prototype.clone=function(){return new SVGPathSegCurvetoQuadraticSmoothRel(void 0,this._x,this._y)},Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothRel.prototype,"x",{get:function(){return this._x},set:function(a){this._x=a,this._segmentChanged()},enumerable:!0}),Object.defineProperty(SVGPathSegCurvetoQuadraticSmoothRel.prototype,"y",{get:function(){return this._y},set:function(a){this._y=a,this._segmentChanged()},enumerable:!0}),SVGPathElement.prototype.createSVGPathSegClosePath=function(){return new SVGPathSegClosePath(void 0)},SVGPathElement.prototype.createSVGPathSegMovetoAbs=function(a,b){return new SVGPathSegMovetoAbs(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegMovetoRel=function(a,b){return new SVGPathSegMovetoRel(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegLinetoAbs=function(a,b){return new SVGPathSegLinetoAbs(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegLinetoRel=function(a,b){return new SVGPathSegLinetoRel(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicAbs=function(a,b,c,d,e,f){return new SVGPathSegCurvetoCubicAbs(void 0,a,b,c,d,e,f)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicRel=function(a,b,c,d,e,f){return new SVGPathSegCurvetoCubicRel(void 0,a,b,c,d,e,f)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticAbs=function(a,b,c,d){return new SVGPathSegCurvetoQuadraticAbs(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticRel=function(a,b,c,d){return new SVGPathSegCurvetoQuadraticRel(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegArcAbs=function(a,b,c,d,e,f,g){return new SVGPathSegArcAbs(void 0,a,b,c,d,e,f,g)},SVGPathElement.prototype.createSVGPathSegArcRel=function(a,b,c,d,e,f,g){return new SVGPathSegArcRel(void 0,a,b,c,d,e,f,g)},SVGPathElement.prototype.createSVGPathSegLinetoHorizontalAbs=function(a){return new SVGPathSegLinetoHorizontalAbs(void 0,a)},SVGPathElement.prototype.createSVGPathSegLinetoHorizontalRel=function(a){return new SVGPathSegLinetoHorizontalRel(void 0,a)},SVGPathElement.prototype.createSVGPathSegLinetoVerticalAbs=function(a){return new SVGPathSegLinetoVerticalAbs(void 0,a)},SVGPathElement.prototype.createSVGPathSegLinetoVerticalRel=function(a){return new SVGPathSegLinetoVerticalRel(void 0,a)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicSmoothAbs=function(a,b,c,d){return new SVGPathSegCurvetoCubicSmoothAbs(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegCurvetoCubicSmoothRel=function(a,b,c,d){return new SVGPathSegCurvetoCubicSmoothRel(void 0,a,b,c,d)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticSmoothAbs=function(a,b){return new SVGPathSegCurvetoQuadraticSmoothAbs(void 0,a,b)},SVGPathElement.prototype.createSVGPathSegCurvetoQuadraticSmoothRel=function(a,b){return new SVGPathSegCurvetoQuadraticSmoothRel(void 0,a,b)}),"SVGPathSegList"in a||(a.SVGPathSegList=function(a){this._pathElement=a,this._list=this._parsePath(this._pathElement.getAttribute("d")),this._mutationObserverConfig={attributes:!0,attributeFilter:["d"]},this._pathElementMutationObserver=new MutationObserver(this._updateListFromPathMutations.bind(this)),this._pathElementMutationObserver.observe(this._pathElement,this._mutationObserverConfig)},Object.defineProperty(SVGPathSegList.prototype,"numberOfItems",{get:function(){return this._checkPathSynchronizedToList(),this._list.length},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"pathSegList",{get:function(){return this._pathSegList||(this._pathSegList=new SVGPathSegList(this)),this._pathSegList},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"normalizedPathSegList",{get:function(){return this.pathSegList},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"animatedPathSegList",{get:function(){return this.pathSegList},enumerable:!0}),Object.defineProperty(SVGPathElement.prototype,"animatedNormalizedPathSegList",{get:function(){return this.pathSegList},enumerable:!0}),SVGPathSegList.prototype._checkPathSynchronizedToList=function(){this._updateListFromPathMutations(this._pathElementMutationObserver.takeRecords())},SVGPathSegList.prototype._updateListFromPathMutations=function(a){if(this._pathElement){var b=!1;a.forEach(function(a){"d"==a.attributeName&&(b=!0)}),b&&(this._list=this._parsePath(this._pathElement.getAttribute("d")))}},SVGPathSegList.prototype._writeListToPath=function(){this._pathElementMutationObserver.disconnect(),this._pathElement.setAttribute("d",SVGPathSegList._pathSegArrayAsString(this._list)),this._pathElementMutationObserver.observe(this._pathElement,this._mutationObserverConfig)},SVGPathSegList.prototype.segmentChanged=function(a){this._writeListToPath()},SVGPathSegList.prototype.clear=function(){this._checkPathSynchronizedToList(),this._list.forEach(function(a){a._owningPathSegList=null}),this._list=[],this._writeListToPath()},SVGPathSegList.prototype.initialize=function(a){return this._checkPathSynchronizedToList(),this._list=[a],a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList.prototype._checkValidIndex=function(a){if(isNaN(a)||0>a||a>=this.numberOfItems)throw"INDEX_SIZE_ERR"},SVGPathSegList.prototype.getItem=function(a){return this._checkPathSynchronizedToList(),this._checkValidIndex(a),this._list[a]},SVGPathSegList.prototype.insertItemBefore=function(a,b){return this._checkPathSynchronizedToList(),b>this.numberOfItems&&(b=this.numberOfItems),a._owningPathSegList&&(a=a.clone()),this._list.splice(b,0,a),a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList.prototype.replaceItem=function(a,b){return this._checkPathSynchronizedToList(),a._owningPathSegList&&(a=a.clone()),this._checkValidIndex(b),this._list[b]=a,a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList.prototype.removeItem=function(a){this._checkPathSynchronizedToList(),this._checkValidIndex(a);var b=this._list[a];return this._list.splice(a,1),this._writeListToPath(),b},SVGPathSegList.prototype.appendItem=function(a){return this._checkPathSynchronizedToList(),a._owningPathSegList&&(a=a.clone()),this._list.push(a),a._owningPathSegList=this,this._writeListToPath(),a},SVGPathSegList._pathSegArrayAsString=function(a){var b="",c=!0;return a.forEach(function(a){c?(c=!1,b+=a._asPathString()):b+=" "+a._asPathString()}),b},SVGPathSegList.prototype._parsePath=function(a){if(!a||0==a.length)return[];var b=this,c=function(){this.pathSegList=[]};c.prototype.appendSegment=function(a){this.pathSegList.push(a)};var d=function(a){this._string=a,this._currentIndex=0,this._endIndex=this._string.length,this._previousCommand=SVGPathSeg.PATHSEG_UNKNOWN,this._skipOptionalSpaces()};d.prototype._isCurrentSpace=function(){var a=this._string[this._currentIndex];return" ">=a&&(" "==a||"\n"==a||"	"==a||"\r"==a||"\f"==a)},d.prototype._skipOptionalSpaces=function(){for(;this._currentIndex<this._endIndex&&this._isCurrentSpace();)this._currentIndex++;return this._currentIndex<this._endIndex},d.prototype._skipOptionalSpacesOrDelimiter=function(){return this._currentIndex<this._endIndex&&!this._isCurrentSpace()&&","!=this._string.charAt(this._currentIndex)?!1:(this._skipOptionalSpaces()&&this._currentIndex<this._endIndex&&","==this._string.charAt(this._currentIndex)&&(this._currentIndex++,this._skipOptionalSpaces()),this._currentIndex<this._endIndex)},d.prototype.hasMoreData=function(){return this._currentIndex<this._endIndex},d.prototype.peekSegmentType=function(){var a=this._string[this._currentIndex];return this._pathSegTypeFromChar(a)},d.prototype._pathSegTypeFromChar=function(a){switch(a){case"Z":case"z":return SVGPathSeg.PATHSEG_CLOSEPATH;case"M":return SVGPathSeg.PATHSEG_MOVETO_ABS;case"m":return SVGPathSeg.PATHSEG_MOVETO_REL;case"L":return SVGPathSeg.PATHSEG_LINETO_ABS;case"l":return SVGPathSeg.PATHSEG_LINETO_REL;case"C":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS;case"c":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL;case"Q":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS;case"q":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL;case"A":return SVGPathSeg.PATHSEG_ARC_ABS;case"a":return SVGPathSeg.PATHSEG_ARC_REL;case"H":return SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS;case"h":return SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL;case"V":return SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS;case"v":return SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL;case"S":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS;case"s":return SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL;case"T":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS;case"t":return SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL;default:return SVGPathSeg.PATHSEG_UNKNOWN}},d.prototype._nextCommandHelper=function(a,b){return("+"==a||"-"==a||"."==a||a>="0"&&"9">=a)&&b!=SVGPathSeg.PATHSEG_CLOSEPATH?b==SVGPathSeg.PATHSEG_MOVETO_ABS?SVGPathSeg.PATHSEG_LINETO_ABS:b==SVGPathSeg.PATHSEG_MOVETO_REL?SVGPathSeg.PATHSEG_LINETO_REL:b:SVGPathSeg.PATHSEG_UNKNOWN},d.prototype.initialCommandIsMoveTo=function(){if(!this.hasMoreData())return!0;var a=this.peekSegmentType();return a==SVGPathSeg.PATHSEG_MOVETO_ABS||a==SVGPathSeg.PATHSEG_MOVETO_REL},d.prototype._parseNumber=function(){var a=0,b=0,c=1,d=0,e=1,f=1,g=this._currentIndex;if(this._skipOptionalSpaces(),this._currentIndex<this._endIndex&&"+"==this._string.charAt(this._currentIndex)?this._currentIndex++:this._currentIndex<this._endIndex&&"-"==this._string.charAt(this._currentIndex)&&(this._currentIndex++,e=-1),!(this._currentIndex==this._endIndex||(this._string.charAt(this._currentIndex)<"0"||this._string.charAt(this._currentIndex)>"9")&&"."!=this._string.charAt(this._currentIndex))){for(var h=this._currentIndex;this._currentIndex<this._endIndex&&this._string.charAt(this._currentIndex)>="0"&&this._string.charAt(this._currentIndex)<="9";)this._currentIndex++;if(this._currentIndex!=h)for(var i=this._currentIndex-1,j=1;i>=h;)b+=j*(this._string.charAt(i--)-"0"),j*=10;if(this._currentIndex<this._endIndex&&"."==this._string.charAt(this._currentIndex)){if(this._currentIndex++,this._currentIndex>=this._endIndex||this._string.charAt(this._currentIndex)<"0"||this._string.charAt(this._currentIndex)>"9")return;for(;this._currentIndex<this._endIndex&&this._string.charAt(this._currentIndex)>="0"&&this._string.charAt(this._currentIndex)<="9";)d+=(this._string.charAt(this._currentIndex++)-"0")*(c*=.1)}if(this._currentIndex!=g&&this._currentIndex+1<this._endIndex&&("e"==this._string.charAt(this._currentIndex)||"E"==this._string.charAt(this._currentIndex))&&"x"!=this._string.charAt(this._currentIndex+1)&&"m"!=this._string.charAt(this._currentIndex+1)){if(this._currentIndex++,"+"==this._string.charAt(this._currentIndex)?this._currentIndex++:"-"==this._string.charAt(this._currentIndex)&&(this._currentIndex++,f=-1),this._currentIndex>=this._endIndex||this._string.charAt(this._currentIndex)<"0"||this._string.charAt(this._currentIndex)>"9")return;for(;this._currentIndex<this._endIndex&&this._string.charAt(this._currentIndex)>="0"&&this._string.charAt(this._currentIndex)<="9";)a*=10,a+=this._string.charAt(this._currentIndex)-"0",this._currentIndex++}var k=b+d;if(k*=e,a&&(k*=Math.pow(10,f*a)),g!=this._currentIndex)return this._skipOptionalSpacesOrDelimiter(),k}},d.prototype._parseArcFlag=function(){if(!(this._currentIndex>=this._endIndex)){var a=!1,b=this._string.charAt(this._currentIndex++);if("0"==b)a=!1;else{if("1"!=b)return;a=!0}return this._skipOptionalSpacesOrDelimiter(),a}},d.prototype.parseSegment=function(){var a=this._string[this._currentIndex],c=this._pathSegTypeFromChar(a);if(c==SVGPathSeg.PATHSEG_UNKNOWN){if(this._previousCommand==SVGPathSeg.PATHSEG_UNKNOWN)return null;if(c=this._nextCommandHelper(a,this._previousCommand),c==SVGPathSeg.PATHSEG_UNKNOWN)return null}else this._currentIndex++;switch(this._previousCommand=c,c){case SVGPathSeg.PATHSEG_MOVETO_REL:return new SVGPathSegMovetoRel(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_MOVETO_ABS:return new SVGPathSegMovetoAbs(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_REL:return new SVGPathSegLinetoRel(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_ABS:return new SVGPathSegLinetoAbs(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_REL:return new SVGPathSegLinetoHorizontalRel(b,this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_HORIZONTAL_ABS:return new SVGPathSegLinetoHorizontalAbs(b,this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_VERTICAL_REL:return new SVGPathSegLinetoVerticalRel(b,this._parseNumber());case SVGPathSeg.PATHSEG_LINETO_VERTICAL_ABS:return new SVGPathSegLinetoVerticalAbs(b,this._parseNumber());case SVGPathSeg.PATHSEG_CLOSEPATH:return this._skipOptionalSpaces(),new SVGPathSegClosePath(b);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_REL:var d={x1:this._parseNumber(),y1:this._parseNumber(),x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicRel(b,d.x,d.y,d.x1,d.y1,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_ABS:var d={x1:this._parseNumber(),y1:this._parseNumber(),x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicAbs(b,d.x,d.y,d.x1,d.y1,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_REL:var d={x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicSmoothRel(b,d.x,d.y,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:var d={x2:this._parseNumber(),y2:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoCubicSmoothAbs(b,d.x,d.y,d.x2,d.y2);case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_REL:var d={x1:this._parseNumber(),y1:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoQuadraticRel(b,d.x,d.y,d.x1,d.y1);case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_ABS:var d={x1:this._parseNumber(),y1:this._parseNumber(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegCurvetoQuadraticAbs(b,d.x,d.y,d.x1,d.y1);case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:return new SVGPathSegCurvetoQuadraticSmoothRel(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:return new SVGPathSegCurvetoQuadraticSmoothAbs(b,this._parseNumber(),this._parseNumber());case SVGPathSeg.PATHSEG_ARC_REL:var d={x1:this._parseNumber(),y1:this._parseNumber(),arcAngle:this._parseNumber(),arcLarge:this._parseArcFlag(),arcSweep:this._parseArcFlag(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegArcRel(b,d.x,d.y,d.x1,d.y1,d.arcAngle,d.arcLarge,d.arcSweep);case SVGPathSeg.PATHSEG_ARC_ABS:var d={x1:this._parseNumber(),y1:this._parseNumber(),arcAngle:this._parseNumber(),arcLarge:this._parseArcFlag(),arcSweep:this._parseArcFlag(),x:this._parseNumber(),y:this._parseNumber()};return new SVGPathSegArcAbs(b,d.x,d.y,d.x1,d.y1,d.arcAngle,d.arcLarge,d.arcSweep);default:throw"Unknown path seg type."}};var e=new c,f=new d(a);if(!f.initialCommandIsMoveTo())return[];for(;f.hasMoreData();){var g=f.parseSegment();if(!g)return[];e.appendSegment(g)}return e.pathSegList})}(),"function"==typeof define&&define.amd?define("c3",["d3"],function(){return k}):"undefined"!=typeof exports&&"undefined"!=typeof module?module.exports=k:a.c3=k}(window);
;(function(){
var f,ba=ba||{},ca=this;function da(a){return void 0!==a}function ea(a){return"string"==typeof a}function fa(a){return"number"==typeof a}function ha(a,b){var c=a.split("."),d=ca;c[0]in d||!d.execScript||d.execScript("var "+c[0]);for(var e;c.length&&(e=c.shift());)!c.length&&da(b)?d[e]=b:d=d[e]&&d[e]!==Object.prototype[e]?d[e]:d[e]={}}function ia(a){a=a.split(".");for(var b=ca,c;c=a.shift();)if(null!=b[c])b=b[c];else return null;return b}function ja(){}
function ka(a){a.wo=void 0;a.ua=function(){return a.wo?a.wo:a.wo=new a}}
function n(a){var b=typeof a;if("object"==b)if(a){if(a instanceof Array)return"array";if(a instanceof Object)return b;var c=Object.prototype.toString.call(a);if("[object Window]"==c)return"object";if("[object Array]"==c||"number"==typeof a.length&&"undefined"!=typeof a.splice&&"undefined"!=typeof a.propertyIsEnumerable&&!a.propertyIsEnumerable("splice"))return"array";if("[object Function]"==c||"undefined"!=typeof a.call&&"undefined"!=typeof a.propertyIsEnumerable&&!a.propertyIsEnumerable("call"))return"function"}else return"null";
else if("function"==b&&"undefined"==typeof a.call)return"object";return b}function la(a){return"array"==n(a)}function ma(a){var b=n(a);return"array"==b||"object"==b&&"number"==typeof a.length}function na(a){return"function"==n(a)}function oa(a){var b=typeof a;return"object"==b&&null!=a||"function"==b}function pa(a){return a[qa]||(a[qa]=++sa)}var qa="closure_uid_"+(1E9*Math.random()>>>0),sa=0;function ta(a,b,c){return a.call.apply(a.bind,arguments)}
function ua(a,b,c){if(!a)throw Error();if(2<arguments.length){var d=Array.prototype.slice.call(arguments,2);return function(){var c=Array.prototype.slice.call(arguments);Array.prototype.unshift.apply(c,d);return a.apply(b,c)}}return function(){return a.apply(b,arguments)}}function q(a,b,c){q=Function.prototype.bind&&-1!=Function.prototype.bind.toString().indexOf("native code")?ta:ua;return q.apply(null,arguments)}
function va(a,b){var c=Array.prototype.slice.call(arguments,1);return function(){var b=c.slice();b.push.apply(b,arguments);return a.apply(this,b)}}var wa=Date.now||function(){return+new Date};function r(a,b){function c(){}c.prototype=b.prototype;a.m=b.prototype;a.prototype=new c;a.prototype.constructor=a;a.base=function(a,c,g){for(var d=Array(arguments.length-2),e=2;e<arguments.length;e++)d[e-2]=arguments[e];return b.prototype[c].apply(a,d)}};function xa(a,b){this.Ig=a;this.Wi=b;this.xc=q(this.xc,this);this.filters={}}xa.prototype.register=function(a,b){return this.filters[a]=b};xa.prototype.xc=function(a){var b,c,d;var e=this.filters;var g=[];for(d in e){var h=e[d];g.push(function(){var e;var g=a.querySelectorAll(d);var m=[];var p=0;for(e=g.length;p<e;p++){b=g[p];try{m.push(this.Ig(h,b))}catch(u){c=u,m.push(this.Wi.log(c.message,String(c.stack)))}}return m}.call(this))}return g};var ya;function za(a,b){var c=a.length-b.length;return 0<=c&&a.indexOf(b,c)==c}function Aa(a){return/^[\s\xa0]*$/.test(a)}function Ba(a){return Aa(null==a?"":String(a))}function Ca(a){return a.replace(/[\t\r\n ]+/g," ").replace(/^[\t\r\n ]+|[\t\r\n ]+$/g,"")}var Da=String.prototype.trim?function(a){return a.trim()}:function(a){return a.replace(/^[\s\xa0]+|[\s\xa0]+$/g,"")};function Ea(a){return decodeURIComponent(a.replace(/\+/g," "))}
function Fa(a,b){return a.replace(/(\r\n|\r|\n)/g,b?"\x3cbr /\x3e":"\x3cbr\x3e")}function Ga(a){if(!Ha.test(a))return a;-1!=a.indexOf("\x26")&&(a=a.replace(Ia,"\x26amp;"));-1!=a.indexOf("\x3c")&&(a=a.replace(Ja,"\x26lt;"));-1!=a.indexOf("\x3e")&&(a=a.replace(Ka,"\x26gt;"));-1!=a.indexOf('"')&&(a=a.replace(La,"\x26quot;"));-1!=a.indexOf("'")&&(a=a.replace(Ma,"\x26#39;"));-1!=a.indexOf("\x00")&&(a=a.replace(Na,"\x26#0;"));return a}var Ia=/&/g,Ja=/</g,Ka=/>/g,La=/"/g,Ma=/'/g,Na=/\x00/g,Ha=/[\x00&<>"']/;
function Oa(a){return Pa(a,"\x26")?"document"in ca?Qa(a):Ra(a):a}function Qa(a){var b={"\x26amp;":"\x26","\x26lt;":"\x3c","\x26gt;":"\x3e","\x26quot;":'"'};var c=ca.document.createElement("div");return a.replace(Sa,function(a,e){var d=b[a];if(d)return d;if("#"==e.charAt(0)){var h=Number("0"+e.substr(1));isNaN(h)||(d=String.fromCharCode(h))}d||(c.innerHTML=a+" ",d=c.firstChild.nodeValue.slice(0,-1));return b[a]=d})}
function Ra(a){return a.replace(/&([^;]+);/g,function(a,c){switch(c){case "amp":return"\x26";case "lt":return"\x3c";case "gt":return"\x3e";case "quot":return'"';default:if("#"==c.charAt(0)){var b=Number("0"+c.substr(1));if(!isNaN(b))return String.fromCharCode(b)}return a}})}var Sa=/&([^;\s<&]+);?/g;function Pa(a,b){return-1!=a.indexOf(b)}function Ta(a){return String(a).replace(/([-()\[\]{}+?*.$\^|,:#<!\\])/g,"\\$1").replace(/\x08/g,"\\x08")}
var Ua=String.prototype.repeat?function(a,b){return a.repeat(b)}:function(a,b){return Array(b+1).join(a)};function Va(a,b){var c=da(void 0)?a.toFixed(void 0):String(a),d=c.indexOf(".");-1==d&&(d=c.length);return Ua("0",Math.max(0,b-d))+c}function Wa(a,b){return a<b?-1:a>b?1:0}function Xa(a){return String(a).replace(/\-([a-z])/g,function(a,c){return c.toUpperCase()})}function Ya(a){return String(a).replace(/([A-Z])/g,"-$1").toLowerCase()}
function $a(a){var b=ea(void 0)?Ta(void 0):"\\s";return a.replace(new RegExp("(^"+(b?"|["+b+"]+":"")+")([a-z])","g"),function(a,b,e){return b+e.toUpperCase()})};function ab(a){return a}function bb(a,b){var c=arguments,d=c.length;return function(){var a;d&&(a=c[d-1].apply(this,arguments));for(var b=d-2;0<=b;b--)a=c[b].call(this,a);return a}};var cb=Array.prototype.indexOf?function(a,b,c){return Array.prototype.indexOf.call(a,b,c)}:function(a,b,c){c=null==c?0:0>c?Math.max(0,a.length+c):c;if(ea(a))return ea(b)&&1==b.length?a.indexOf(b,c):-1;for(;c<a.length;c++)if(c in a&&a[c]===b)return c;return-1},db=Array.prototype.forEach?function(a,b,c){Array.prototype.forEach.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=ea(a)?a.split(""):a,g=0;g<d;g++)g in e&&b.call(c,e[g],g,a)},eb=Array.prototype.filter?function(a,b,c){return Array.prototype.filter.call(a,
b,c)}:function(a,b,c){for(var d=a.length,e=[],g=0,h=ea(a)?a.split(""):a,k=0;k<d;k++)if(k in h){var l=h[k];b.call(c,l,k,a)&&(e[g++]=l)}return e},fb=Array.prototype.map?function(a,b,c){return Array.prototype.map.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=Array(d),g=ea(a)?a.split(""):a,h=0;h<d;h++)h in g&&(e[h]=b.call(c,g[h],h,a));return e},hb=Array.prototype.reduce?function(a,b,c,d){d&&(b=q(b,d));return Array.prototype.reduce.call(a,b,c)}:function(a,b,c,d){var e=c;db(a,function(c,h){e=b.call(d,
e,c,h,a)});return e},ib=Array.prototype.some?function(a,b,c){return Array.prototype.some.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=ea(a)?a.split(""):a,g=0;g<d;g++)if(g in e&&b.call(c,e[g],g,a))return!0;return!1},jb=Array.prototype.every?function(a,b,c){return Array.prototype.every.call(a,b,c)}:function(a,b,c){for(var d=a.length,e=ea(a)?a.split(""):a,g=0;g<d;g++)if(g in e&&!b.call(c,e[g],g,a))return!1;return!0};function kb(a,b){var c=lb(a,b);return 0>c?null:ea(a)?a.charAt(c):a[c]}
function lb(a,b){for(var c=a.length,d=ea(a)?a.split(""):a,e=0;e<c;e++)if(e in d&&b.call(void 0,d[e],e,a))return e;return-1}function mb(a,b){return 0<=cb(a,b)}function nb(a,b){var c=cb(a,b),d;(d=0<=c)&&Array.prototype.splice.call(a,c,1);return d}function ob(a,b){var c=lb(a,b);return 0<=c?(Array.prototype.splice.call(a,c,1),!0):!1}function pb(a){return Array.prototype.concat.apply([],arguments)}function qb(a){var b=a.length;if(0<b){for(var c=Array(b),d=0;d<b;d++)c[d]=a[d];return c}return[]}
function rb(a,b,c,d){Array.prototype.splice.apply(a,sb(arguments,1))}function sb(a,b,c){return 2>=arguments.length?Array.prototype.slice.call(a,b):Array.prototype.slice.call(a,b,c)}function ub(a){for(var b={},c=0,d=0;d<a.length;){var e=a[d++];var g=e;g=oa(g)?"o"+pa(g):(typeof g).charAt(0)+g;Object.prototype.hasOwnProperty.call(b,g)||(b[g]=!0,a[c++]=e)}a.length=c}function vb(a,b){var c=[],d=0,e=a;void 0!==b&&(d=a,e=b);if(0>1*(e-d))return[];for(;d<e;d+=1)c.push(d);return c}
function wb(a){if(!arguments.length)return[];for(var b=[],c=arguments[0].length,d=1;d<arguments.length;d++)arguments[d].length<c&&(c=arguments[d].length);for(d=0;d<c;d++){for(var e=[],g=0;g<arguments.length;g++)e.push(arguments[g][d]);b.push(e)}return b};var xb;a:{var yb=ca.navigator;if(yb){var Bb=yb.userAgent;if(Bb){xb=Bb;break a}}xb=""}function Cb(a){return Pa(xb,a)};function Db(a,b,c){for(var d in a)b.call(c,a[d],d,a)}function Eb(a,b){for(var c in a)if(b.call(void 0,a[c],c,a))return!0;return!1}function Fb(a){var b=0,c;for(c in a)b++;return b}function Gb(a){var b=[],c=0,d;for(d in a)b[c++]=a[d];return b}function Hb(a){var b=[],c=0,d;for(d in a)b[c++]=d;return b}function Jb(a,b){return null!==a&&b in a}function Kb(){var a=Lb,b;for(b in a)return!1;return!0}function Mb(a,b){b in a&&delete a[b]}
function Nb(a,b,c){if(null!==a&&b in a)throw Error('The object already contains the key "'+b+'"');a[b]=c}function Ob(a,b,c){return null!==a&&b in a?a[b]:c}var Pb="constructor hasOwnProperty isPrototypeOf propertyIsEnumerable toLocaleString toString valueOf".split(" ");function Qb(a,b){for(var c,d,e=1;e<arguments.length;e++){d=arguments[e];for(c in d)a[c]=d[c];for(var g=0;g<Pb.length;g++)c=Pb[g],Object.prototype.hasOwnProperty.call(d,c)&&(a[c]=d[c])}}
function Rb(a){var b=arguments.length;if(1==b&&la(arguments[0]))return Rb.apply(null,arguments[0]);for(var c={},d=0;d<b;d++)c[arguments[d]]=!0;return c};function Sb(){return Cb("Safari")&&!(Tb()||Cb("Coast")||Cb("Opera")||Cb("Edge")||Cb("Silk")||Cb("Android"))}function Tb(){return(Cb("Chrome")||Cb("CriOS"))&&!Cb("Edge")};function Ub(a,b,c){var d=a;b&&(d=q(a,b));d=Ub.ux(d);na(ca.setImmediate)&&(c||Ub.qx())?ca.setImmediate(d):(Ub.ot||(Ub.ot=Ub.Lv()),Ub.ot(d))}Ub.qx=function(){return ca.Window&&ca.Window.prototype&&!Cb("Edge")&&ca.Window.prototype.setImmediate==ca.setImmediate?!1:!0};
Ub.Lv=function(){var a=ca.MessageChannel;"undefined"===typeof a&&"undefined"!==typeof window&&window.postMessage&&window.addEventListener&&!Cb("Presto")&&(a=function(){var a=document.createElement("IFRAME");a.style.display="none";a.src="";document.documentElement.appendChild(a);var b=a.contentWindow,a=b.document;a.open();a.write("");a.close();var c="callImmediate"+Math.random(),d="file:"==b.location.protocol?"*":b.location.protocol+"//"+b.location.host,a=q(function(a){if(("*"==d||a.origin==d)&&a.data==
c)this.port1.onmessage()},this);b.addEventListener("message",a,!1);this.port1={};this.port2={postMessage:function(){b.postMessage(c,d)}}});if("undefined"!==typeof a&&!Cb("Trident")&&!Cb("MSIE")){var b=new a,c={},d=c;b.port1.onmessage=function(){if(da(c.next)){c=c.next;var a=c.Yq;c.Yq=null;a()}};return function(a){d.next={Yq:a};d=d.next;b.port2.postMessage(0)}}return"undefined"!==typeof document&&"onreadystatechange"in document.createElement("SCRIPT")?function(a){var b=document.createElement("SCRIPT");
b.onreadystatechange=function(){b.onreadystatechange=null;b.parentNode.removeChild(b);b=null;a();a=null};document.documentElement.appendChild(b)}:function(a){ca.setTimeout(a,0)}};Ub.ux=ab;function Vb(){0!=Yb&&(Zb[pa(this)]=this);this.Re=this.Re;this.mg=this.mg}var Yb=0,Zb={};Vb.prototype.Re=!1;Vb.prototype.ib=function(){if(!this.Re&&(this.Re=!0,this.V(),0!=Yb)){var a=pa(this);delete Zb[a]}};function $b(a,b){a.Re?da(void 0)?b.call(void 0):b():(a.mg||(a.mg=[]),a.mg.push(da(void 0)?q(b,void 0):b))}Vb.prototype.V=function(){if(this.mg)for(;this.mg.length;)this.mg.shift()()};function ac(a){a&&"function"==typeof a.ib&&a.ib()};function bc(){return Cb("iPhone")&&!Cb("iPod")&&!Cb("iPad")}function cc(){bc()||Cb("iPad")||Cb("iPod")};function dc(a){dc[" "](a);return a}dc[" "]=ja;function ec(a,b){try{return dc(a[b]),!0}catch(c){}return!1}function fc(a,b,c){return Object.prototype.hasOwnProperty.call(a,b)?a[b]:a[b]=c(b)};var gc=Cb("Opera"),t=Cb("Trident")||Cb("MSIE"),hc=Cb("Edge"),ic=hc||t,jc=Cb("Gecko")&&!(Pa(xb.toLowerCase(),"webkit")&&!Cb("Edge"))&&!(Cb("Trident")||Cb("MSIE"))&&!Cb("Edge"),kc=Pa(xb.toLowerCase(),"webkit")&&!Cb("Edge");kc&&Cb("Mobile");var lc=Cb("Macintosh");Cb("Windows");Cb("Linux")||Cb("CrOS");var mc=ca.navigator||null;mc&&Pa(mc.appVersion||"","X11");Cb("Android");bc();Cb("iPad");Cb("iPod");cc();function nc(){var a=ca.document;return a?a.documentMode:void 0}var oc;
a:{var qc="",rc=function(){var a=xb;if(jc)return/rv\:([^\);]+)(\)|;)/.exec(a);if(hc)return/Edge\/([\d\.]+)/.exec(a);if(t)return/\b(?:MSIE|rv)[: ]([^\);]+)(\)|;)/.exec(a);if(kc)return/WebKit\/(\S+)/.exec(a);if(gc)return/(?:Version)[ \/]?(\S+)/.exec(a)}();rc&&(qc=rc?rc[1]:"");if(t){var sc=nc();if(null!=sc&&sc>parseFloat(qc)){oc=String(sc);break a}}oc=qc}var tc={};
function uc(a){return fc(tc,a,function(){for(var b=0,c=Da(String(oc)).split("."),d=Da(String(a)).split("."),e=Math.max(c.length,d.length),g=0;0==b&&g<e;g++){var h=c[g]||"",k=d[g]||"";do{h=/(\d*)(\D*)(.*)/.exec(h)||["","","",""];k=/(\d*)(\D*)(.*)/.exec(k)||["","","",""];if(0==h[0].length&&0==k[0].length)break;b=Wa(0==h[1].length?0:parseInt(h[1],10),0==k[1].length?0:parseInt(k[1],10))||Wa(0==h[2].length,0==k[2].length)||Wa(h[2],k[2]);h=h[3];k=k[3]}while(0==b)}return 0<=b})}
function vc(a){return Number(wc)>=a}var xc;var yc=ca.document;xc=yc&&t?nc()||("CSS1Compat"==yc.compatMode?parseInt(oc,10):5):void 0;var wc=xc;var zc=!t||vc(9),Ac=!t||vc(9),Bc=t&&!uc("9");!kc||uc("528");jc&&uc("1.9b")||t&&uc("8")||gc&&uc("9.5")||kc&&uc("528");jc&&!uc("8")||t&&uc("9");function Cc(a,b){this.type=a;this.currentTarget=this.target=b;this.defaultPrevented=this.qd=!1;this.Xi=!0}Cc.prototype.stopPropagation=function(){this.qd=!0};Cc.prototype.preventDefault=function(){this.defaultPrevented=!0;this.Xi=!1};function Dc(a){a.preventDefault()};function Ec(a){return kc?"webkit"+a:gc?"o"+a.toLowerCase():a.toLowerCase()}
var Fc={oq:"click",Rz:"rightclick",Yx:"dblclick",Vy:"mousedown",az:"mouseup",$y:"mouseover",Zy:"mouseout",Yy:"mousemove",Wy:"mouseenter",Xy:"mouseleave",Wz:"selectionchange",Xz:"selectstart",CA:"wheel",Iy:"keypress",Hy:"keydown",tq:"keyup",Kx:"blur",xy:"focus",Zx:"deactivate",yy:t?"focusin":"DOMFocusIn",zy:t?"focusout":"DOMFocusOut",Ox:"change",Oz:"reset",Vz:"select",iA:"submit",Fy:"input",Kz:"propertychange",oy:"dragstart",jy:"drag",ly:"dragenter",ny:"dragover",my:"dragleave",py:"drop",ky:"dragend",
qA:"touchstart",pA:"touchmove",oA:"touchend",nA:"touchcancel",Jx:"beforeunload",Tx:"consolemessage",Ux:"contextmenu",$x:"devicemotion",ay:"deviceorientation",dy:"DOMContentLoaded",ty:"error",Dy:"help",Jy:"load",Ry:"losecapture",uz:"orientationchange",Mz:"readystatechange",Pz:"resize",Sz:"scroll",tA:"unload",Mx:"canplay",Nx:"canplaythrough",qy:"durationchange",ry:"emptied",sy:"ended",My:"loadeddata",Ny:"loadedmetadata",yz:"pause",zz:"play",Az:"playing",Lz:"ratechange",Tz:"seeked",Uz:"seeking",eA:"stalled",
jA:"suspend",mA:"timeupdate",AA:"volumechange",BA:"waiting",dA:"sourceopen",cA:"sourceended",bA:"sourceclosed",wx:"abort",vA:"update",yA:"updatestart",wA:"updateend",Cy:"hashchange",vz:"pagehide",wz:"pageshow",Jz:"popstate",Wx:"copy",xz:"paste",Xx:"cut",Ex:"beforecopy",Fx:"beforecut",Hx:"beforepaste",tz:"online",sz:"offline",Ty:"message",Sx:"connect",Gy:"install",yx:"activate",wy:"fetch",Ay:"foreignfetch",Uy:"messageerror",fA:"statechange",xA:"updatefound",Vx:"controllerchange",Dx:Ec("AnimationStart"),
Bx:Ec("AnimationEnd"),Cx:Ec("AnimationIteration"),rA:Ec("TransitionEnd"),Cz:"pointerdown",Iz:"pointerup",Bz:"pointercancel",Fz:"pointermove",Hz:"pointerover",Gz:"pointerout",Dz:"pointerenter",Ez:"pointerleave",By:"gotpointercapture",Sy:"lostpointercapture",bz:"MSGestureChange",cz:"MSGestureEnd",dz:"MSGestureHold",ez:"MSGestureStart",fz:"MSGestureTap",gz:"MSGotPointerCapture",hz:"MSInertiaStart",iz:"MSLostPointerCapture",jz:"MSPointerCancel",kz:"MSPointerDown",lz:"MSPointerEnter",mz:"MSPointerHover",
nz:"MSPointerLeave",oz:"MSPointerMove",pz:"MSPointerOut",qz:"MSPointerOver",rz:"MSPointerUp",kA:"text",lA:t?"textinput":"textInput",Qx:"compositionstart",Rx:"compositionupdate",Px:"compositionend",Gx:"beforeinput",uy:"exit",Ky:"loadabort",Ly:"loadcommit",Oy:"loadredirect",Py:"loadstart",Qy:"loadstop",Qz:"responsive",Yz:"sizechanged",uA:"unresponsive",zA:"visibilitychange",hA:"storage",iy:"DOMSubtreeModified",ey:"DOMNodeInserted",gy:"DOMNodeRemoved",hy:"DOMNodeRemovedFromDocument",fy:"DOMNodeInsertedIntoDocument",
by:"DOMAttrModified",cy:"DOMCharacterDataModified",Ix:"beforeprint",zx:"afterprint"};function Gc(a,b){Cc.call(this,a?a.type:"");this.relatedTarget=this.currentTarget=this.target=null;this.button=this.screenY=this.screenX=this.clientY=this.clientX=this.offsetY=this.offsetX=0;this.key="";this.charCode=this.keyCode=0;this.metaKey=this.shiftKey=this.altKey=this.ctrlKey=!1;this.state=null;this.hp=!1;this.bd=null;a&&this.init(a,b)}r(Gc,Cc);var Hc=[1,4,2];
Gc.prototype.init=function(a,b){var c=this.type=a.type,d=a.changedTouches?a.changedTouches[0]:null;this.target=a.target||a.srcElement;this.currentTarget=b;var e=a.relatedTarget;e?jc&&(ec(e,"nodeName")||(e=null)):"mouseover"==c?e=a.fromElement:"mouseout"==c&&(e=a.toElement);this.relatedTarget=e;null===d?(this.offsetX=kc||void 0!==a.offsetX?a.offsetX:a.layerX,this.offsetY=kc||void 0!==a.offsetY?a.offsetY:a.layerY,this.clientX=void 0!==a.clientX?a.clientX:a.pageX,this.clientY=void 0!==a.clientY?a.clientY:
a.pageY,this.screenX=a.screenX||0,this.screenY=a.screenY||0):(this.clientX=void 0!==d.clientX?d.clientX:d.pageX,this.clientY=void 0!==d.clientY?d.clientY:d.pageY,this.screenX=d.screenX||0,this.screenY=d.screenY||0);this.button=a.button;this.keyCode=a.keyCode||0;this.key=a.key||"";this.charCode=a.charCode||("keypress"==c?a.keyCode:0);this.ctrlKey=a.ctrlKey;this.altKey=a.altKey;this.shiftKey=a.shiftKey;this.metaKey=a.metaKey;this.hp=lc?a.metaKey:a.ctrlKey;this.state=a.state;this.bd=a;a.defaultPrevented&&
this.preventDefault()};function Jc(a){return(zc?0==a.bd.button:"click"==a.type?!0:!!(a.bd.button&Hc[0]))&&!(kc&&lc&&a.ctrlKey)}Gc.prototype.stopPropagation=function(){Gc.m.stopPropagation.call(this);this.bd.stopPropagation?this.bd.stopPropagation():this.bd.cancelBubble=!0};Gc.prototype.preventDefault=function(){Gc.m.preventDefault.call(this);var a=this.bd;if(a.preventDefault)a.preventDefault();else if(a.returnValue=!1,Bc)try{if(a.ctrlKey||112<=a.keyCode&&123>=a.keyCode)a.keyCode=-1}catch(b){}};var Kc="closure_listenable_"+(1E6*Math.random()|0);function Lc(a){return!(!a||!a[Kc])}var Mc=0;function Nc(a,b,c,d,e){this.listener=a;this.Xk=null;this.src=b;this.type=c;this.capture=!!d;this.cb=e;this.key=++Mc;this.Xc=this.Cj=!1}function Oc(a){a.Xc=!0;a.listener=null;a.Xk=null;a.src=null;a.cb=null};function Pc(a){this.src=a;this.Na={};this.oj=0}f=Pc.prototype;f.add=function(a,b,c,d,e){var g=a.toString();a=this.Na[g];a||(a=this.Na[g]=[],this.oj++);var h=Qc(a,b,d,e);-1<h?(b=a[h],c||(b.Cj=!1)):(b=new Nc(b,this.src,g,!!d,e),b.Cj=c,a.push(b));return b};f.remove=function(a,b,c,d){a=a.toString();if(!(a in this.Na))return!1;var e=this.Na[a];b=Qc(e,b,c,d);return-1<b?(Oc(e[b]),Array.prototype.splice.call(e,b,1),0==e.length&&(delete this.Na[a],this.oj--),!0):!1};
function Rc(a,b){var c=b.type;if(!(c in a.Na))return!1;var d=nb(a.Na[c],b);d&&(Oc(b),0==a.Na[c].length&&(delete a.Na[c],a.oj--));return d}f.sd=function(a){a=a&&a.toString();var b=0,c;for(c in this.Na)if(!a||c==a){for(var d=this.Na[c],e=0;e<d.length;e++)++b,Oc(d[e]);delete this.Na[c];this.oj--}return b};f.si=function(a,b,c,d){a=this.Na[a.toString()];var e=-1;a&&(e=Qc(a,b,c,d));return-1<e?a[e]:null};
f.hasListener=function(a,b){var c=da(a),d=c?a.toString():"",e=da(b);return Eb(this.Na,function(a){for(var g=0;g<a.length;++g)if(!(c&&a[g].type!=d||e&&a[g].capture!=b))return!0;return!1})};function Qc(a,b,c,d){for(var e=0;e<a.length;++e){var g=a[e];if(!g.Xc&&g.listener==b&&g.capture==!!c&&g.cb==d)return e}return-1};var Sc="closure_lm_"+(1E6*Math.random()|0),Tc={},Uc=0;function v(a,b,c,d,e){if(la(b)){for(var g=0;g<b.length;g++)v(a,b[g],c,d,e);return null}c=Xc(c);return Lc(a)?a.A(b,c,d,e):Yc(a,b,c,!1,d,e)}
function Yc(a,b,c,d,e,g){if(!b)throw Error("Invalid event type");var h=!!e,k=Zc(a);k||(a[Sc]=k=new Pc(a));c=k.add(b,c,d,e,g);if(c.Xk)return c;d=$c();c.Xk=d;d.src=a;d.listener=c;if(a.addEventListener)a.addEventListener(b.toString(),d,h);else if(a.attachEvent)a.attachEvent(ad(b.toString()),d);else throw Error("addEventListener and attachEvent are unavailable.");Uc++;return c}
function $c(){var a=bd,b=Ac?function(c){return a.call(b.src,b.listener,c)}:function(c){c=a.call(b.src,b.listener,c);if(!c)return c};return b}function cd(a,b,c,d,e){if(la(b)){for(var g=0;g<b.length;g++)cd(a,b[g],c,d,e);return null}c=Xc(c);return Lc(a)?a.No(b,c,d,e):Yc(a,b,c,!0,d,e)}function dd(a,b,c,d,e){if(la(b))for(var g=0;g<b.length;g++)dd(a,b[g],c,d,e);else c=Xc(c),Lc(a)?a.Ba(b,c,d,e):a&&(a=Zc(a))&&(b=a.si(b,c,!!d,e))&&ed(b)}
function ed(a){if(fa(a)||!a||a.Xc)return!1;var b=a.src;if(Lc(b))return Rc(b.Tc,a);var c=a.type,d=a.Xk;b.removeEventListener?b.removeEventListener(c,d,a.capture):b.detachEvent&&b.detachEvent(ad(c),d);Uc--;(c=Zc(b))?(Rc(c,a),0==c.oj&&(c.src=null,b[Sc]=null)):Oc(a);return!0}function fd(a){if(a)if(Lc(a))a.Tc&&a.Tc.sd(void 0);else if(a=Zc(a)){var b=0,c;for(c in a.Na)for(var d=a.Na[c].concat(),e=0;e<d.length;++e)ed(d[e])&&++b}}function ad(a){return a in Tc?Tc[a]:Tc[a]="on"+a}
function gd(a,b,c,d){return Lc(a)?hd(a,b,c,d):id(a,b,c,d)}function id(a,b,c,d){var e=!0;if(a=Zc(a))if(b=a.Na[b.toString()])for(b=b.concat(),a=0;a<b.length;a++){var g=b[a];g&&g.capture==c&&!g.Xc&&(g=jd(g,d),e=e&&!1!==g)}return e}function jd(a,b){var c=a.listener,d=a.cb||a.src;a.Cj&&ed(a);return c.call(d,b)}
function bd(a,b){if(a.Xc)return!0;if(!Ac){var c=b||ia("window.event"),d=new Gc(c,this),e=!0;if(!(0>c.keyCode||void 0!=c.returnValue)){a:{var g=!1;if(0==c.keyCode)try{c.keyCode=-1;break a}catch(l){g=!0}if(g||void 0==c.returnValue)c.returnValue=!0}c=[];for(g=d.currentTarget;g;g=g.parentNode)c.push(g);for(var h=a.type,k=c.length-1;!d.qd&&0<=k;k--)d.currentTarget=c[k],g=id(c[k],h,!0,d),e=e&&g;for(k=0;!d.qd&&k<c.length;k++)d.currentTarget=c[k],g=id(c[k],h,!1,d),e=e&&g}return e}return jd(a,new Gc(b,this))}
function Zc(a){a=a[Sc];return a instanceof Pc?a:null}var kd="__closure_events_fn_"+(1E9*Math.random()>>>0);function Xc(a){if(na(a))return a;a[kd]||(a[kd]=function(b){return a.handleEvent(b)});return a[kd]};function md(){Vb.call(this);this.Tc=new Pc(this);this.Nu=this;this.ep=null}r(md,Vb);md.prototype[Kc]=!0;f=md.prototype;f.mk=function(){return this.ep};f.nl=function(a){this.ep=a};f.addEventListener=function(a,b,c,d){v(this,a,b,c,d)};f.removeEventListener=function(a,b,c,d){dd(this,a,b,c,d)};
f.dispatchEvent=function(a){var b,c=this.mk();if(c)for(b=[];c;c=c.mk())b.push(c);var c=this.Nu,d=a.type||a;if(ea(a))a=new Cc(a,c);else if(a instanceof Cc)a.target=a.target||c;else{var e=a;a=new Cc(d,c);Qb(a,e)}var e=!0;if(b)for(var g=b.length-1;!a.qd&&0<=g;g--){var h=a.currentTarget=b[g];e=hd(h,d,!0,a)&&e}a.qd||(h=a.currentTarget=c,e=hd(h,d,!0,a)&&e,a.qd||(e=hd(h,d,!1,a)&&e));if(b)for(g=0;!a.qd&&g<b.length;g++)h=a.currentTarget=b[g],e=hd(h,d,!1,a)&&e;return e};
f.V=function(){md.m.V.call(this);this.Tc&&this.Tc.sd(void 0);this.ep=null};f.A=function(a,b,c,d){return this.Tc.add(String(a),b,!1,c,d)};f.No=function(a,b,c,d){return this.Tc.add(String(a),b,!0,c,d)};f.Ba=function(a,b,c,d){return this.Tc.remove(String(a),b,c,d)};function hd(a,b,c,d){b=a.Tc.Na[String(b)];if(!b)return!0;b=b.concat();for(var e=!0,g=0;g<b.length;++g){var h=b[g];if(h&&!h.Xc&&h.capture==c){var k=h.listener,l=h.cb||h.src;h.Cj&&Rc(a.Tc,h);e=!1!==k.call(l,d)&&e}}return e&&0!=d.Xi}
f.si=function(a,b,c,d){return this.Tc.si(String(a),b,c,d)};f.hasListener=function(a,b){return this.Tc.hasListener(da(a)?String(a):void 0,b)};function nd(a,b){md.call(this);this.eg=a||1;this.Ah=b||ca;this.ym=q(this.mx,this);this.Ko=wa()}r(nd,md);f=nd.prototype;f.enabled=!1;f.wa=null;f.setInterval=function(a){this.eg=a;this.wa&&this.enabled?(this.stop(),this.start()):this.wa&&this.stop()};
f.mx=function(){if(this.enabled){var a=wa()-this.Ko;0<a&&a<.8*this.eg?this.wa=this.Ah.setTimeout(this.ym,this.eg-a):(this.wa&&(this.Ah.clearTimeout(this.wa),this.wa=null),this.dispatchEvent("tick"),this.enabled&&(this.wa=this.Ah.setTimeout(this.ym,this.eg),this.Ko=wa()))}};f.start=function(){this.enabled=!0;this.wa||(this.wa=this.Ah.setTimeout(this.ym,this.eg),this.Ko=wa())};f.stop=function(){this.enabled=!1;this.wa&&(this.Ah.clearTimeout(this.wa),this.wa=null)};
f.V=function(){nd.m.V.call(this);this.stop();delete this.Ah};function od(a,b,c){if(na(a))c&&(a=q(a,c));else if(a&&"function"==typeof a.handleEvent)a=q(a.handleEvent,a);else throw Error("Invalid listener argument");return 2147483647<Number(b)?-1:ca.setTimeout(a,b||0)};function pd(a){return/^\s*$/.test(a)?!1:/^[\],:{}\s\u2028\u2029]*$/.test(a.replace(/\\["\\\/bfnrtu]/g,"@").replace(/(?:"[^"\\\n\r\u2028\u2029\x00-\x08\x0a-\x1f]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)[\s\u2028\u2029]*(?=:|,|]|}|$)/g,"]").replace(/(?:^|:|,)(?:[\s\u2028\u2029]*\[)+/g,""))}function qd(a){a=String(a);if(pd(a))try{return eval("("+a+")")}catch(b){}throw Error("Invalid JSON string: "+a);}function rd(a){var b=[];sd(new td,a,b);return b.join("")}function td(){this.cl=void 0}
function sd(a,b,c){if(null==b)c.push("null");else{if("object"==typeof b){if(la(b)){var d=b;b=d.length;c.push("[");for(var e="",g=0;g<b;g++)c.push(e),e=d[g],sd(a,a.cl?a.cl.call(d,String(g),e):e,c),e=",";c.push("]");return}if(b instanceof String||b instanceof Number||b instanceof Boolean)b=b.valueOf();else{c.push("{");g="";for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&(e=b[d],"function"!=typeof e&&(c.push(g),ud(d,c),c.push(":"),sd(a,a.cl?a.cl.call(b,d,e):e,c),g=","));c.push("}");return}}switch(typeof b){case "string":ud(b,
c);break;case "number":c.push(isFinite(b)&&!isNaN(b)?String(b):"null");break;case "boolean":c.push(String(b));break;case "function":c.push("null");break;default:throw Error("Unknown type: "+typeof b);}}}var vd={'"':'\\"',"\\":"\\\\","/":"\\/","\b":"\\b","\f":"\\f","\n":"\\n","\r":"\\r","\t":"\\t","\x0B":"\\u000b"},wd=/\uffff/.test("")?/[\\\"\x00-\x1f\x7f-\uffff]/g:/[\\\"\x00-\x1f\x7f-\xff]/g;
function ud(a,b){b.push('"',a.replace(wd,function(a){var b=vd[a];b||(b="\\u"+(a.charCodeAt(0)|65536).toString(16).substr(1),vd[a]=b);return b}),'"')};function xd(a){var b=Error();if(Error.captureStackTrace)Error.captureStackTrace(b,a||xd),b=String(b.stack);else{try{throw b;}catch(c){b=c}b=(b=b.stack)?String(b):null}b||(b=yd(a||arguments.callee.caller,[]));return b}
function yd(a,b){var c=[];if(mb(b,a))c.push("[...circular reference...]");else if(a&&50>b.length){c.push(zd(a)+"(");for(var d=a.arguments,e=0;d&&e<d.length;e++){0<e&&c.push(", ");var g=d[e];switch(typeof g){case "object":g=g?"object":"null";break;case "string":break;case "number":g=String(g);break;case "boolean":g=g?"true":"false";break;case "function":g=(g=zd(g))?g:"[fn]";break;default:g=typeof g}40<g.length&&(g=g.substr(0,40)+"...");c.push(g)}b.push(a);c.push(")\n");try{c.push(yd(a.caller,b))}catch(h){c.push("[exception trying to get caller]\n")}}else a?
c.push("[...long stack...]"):c.push("[end]");return c.join("")}function zd(a){if(Ad[a])return Ad[a];a=String(a);if(!Ad[a]){var b=/function ([^\(]+)/.exec(a);Ad[a]=b?b[1]:"[Anonymous]"}return Ad[a]}var Ad={};function Bd(){}Bd.prototype.Vq=null;function Cd(a){var b;(b=a.Vq)||(b={},Dd(a)&&(b[0]=!0,b[1]=!0),b=a.Vq=b);return b};var Ed;function Fd(){}r(Fd,Bd);function Gd(a){return(a=Dd(a))?new ActiveXObject(a):new XMLHttpRequest}function Dd(a){if(!a.Ur&&"undefined"==typeof XMLHttpRequest&&"undefined"!=typeof ActiveXObject){for(var b=["MSXML2.XMLHTTP.6.0","MSXML2.XMLHTTP.3.0","MSXML2.XMLHTTP","Microsoft.XMLHTTP"],c=0;c<b.length;c++){var d=b[c];try{return new ActiveXObject(d),a.Ur=d}catch(e){}}throw Error("Could not create ActiveXObject. ActiveX might be disabled, or MSXML might not be installed");}return a.Ur}Ed=new Fd;function Hd(a){return a.Gd&&"function"==typeof a.Gd?a.Gd():ma(a)||ea(a)?a.length:Fb(a)}function Id(a){if(a.Ib&&"function"==typeof a.Ib)return a.Ib();if(ea(a))return a.split("");if(ma(a)){for(var b=[],c=a.length,d=0;d<c;d++)b.push(a[d]);return b}return Gb(a)}function Jd(a){if(a.uc&&"function"==typeof a.uc)return a.uc();if(!a.Ib||"function"!=typeof a.Ib){if(ma(a)||ea(a)){var b=[];a=a.length;for(var c=0;c<a;c++)b.push(c);return b}return Hb(a)}}
function Kd(a,b,c){if(a.forEach&&"function"==typeof a.forEach)a.forEach(b,c);else if(ma(a)||ea(a))db(a,b,c);else for(var d=Jd(a),e=Id(a),g=e.length,h=0;h<g;h++)b.call(c,e[h],d&&d[h],a)}function Md(a,b){if("function"==typeof a.every)return a.every(b,void 0);if(ma(a)||ea(a))return jb(a,b,void 0);for(var c=Jd(a),d=Id(a),e=d.length,g=0;g<e;g++)if(!b.call(void 0,d[g],c&&c[g],a))return!1;return!0};var Nd="StopIteration"in ca?ca.StopIteration:{message:"StopIteration",stack:""};function Od(){}Od.prototype.next=function(){throw Nd;};function Pd(a,b){this.ob={};this.Ga=[];this.La=0;var c=arguments.length;if(1<c){if(c%2)throw Error("Uneven number of arguments");for(var d=0;d<c;d+=2)this.set(arguments[d],arguments[d+1])}else a&&this.addAll(a)}f=Pd.prototype;f.Gd=function(){return this.La};f.Ib=function(){Qd(this);for(var a=[],b=0;b<this.Ga.length;b++)a.push(this.ob[this.Ga[b]]);return a};f.uc=function(){Qd(this);return this.Ga.concat()};f.ke=function(a){return Rd(this.ob,a)};
f.di=function(a){for(var b=0;b<this.Ga.length;b++){var c=this.Ga[b];if(Rd(this.ob,c)&&this.ob[c]==a)return!0}return!1};f.tb=function(a,b){if(this===a)return!0;if(this.La!=a.Gd())return!1;var c=b||Sd;Qd(this);for(var d,e=0;d=this.Ga[e];e++)if(!c(this.get(d),a.get(d)))return!1;return!0};function Sd(a,b){return a===b}f.isEmpty=function(){return 0==this.La};f.clear=function(){this.ob={};this.La=this.Ga.length=0};
f.remove=function(a){return Rd(this.ob,a)?(delete this.ob[a],this.La--,this.Ga.length>2*this.La&&Qd(this),!0):!1};function Qd(a){var b,c;if(a.La!=a.Ga.length){for(b=c=0;c<a.Ga.length;){var d=a.Ga[c];Rd(a.ob,d)&&(a.Ga[b++]=d);c++}a.Ga.length=b}if(a.La!=a.Ga.length){var e={};for(b=c=0;c<a.Ga.length;)d=a.Ga[c],Rd(e,d)||(a.Ga[b++]=d,e[d]=1),c++;a.Ga.length=b}}f.get=function(a,b){return Rd(this.ob,a)?this.ob[a]:b};f.set=function(a,b){Rd(this.ob,a)||(this.La++,this.Ga.push(a));this.ob[a]=b};
f.addAll=function(a){if(a instanceof Pd){var b=a.uc();a=a.Ib()}else b=Hb(a),a=Gb(a);for(var c=0;c<b.length;c++)this.set(b[c],a[c])};f.forEach=function(a,b){for(var c=this.uc(),d=0;d<c.length;d++){var e=c[d],g=this.get(e);a.call(b,g,e,this)}};f.clone=function(){return new Pd(this)};f.transpose=function(){for(var a=new Pd,b=0;b<this.Ga.length;b++){var c=this.Ga[b];a.set(this.ob[c],c)}return a};function Rd(a,b){return Object.prototype.hasOwnProperty.call(a,b)};var Td=/^(?:([^:/?#.]+):)?(?:\/\/(?:([^/?#]*)@)?([^/#?]*?)(?::([0-9]+))?(?=[/#?]|$))?([^?#]+)?(?:\?([^#]*))?(?:#([\s\S]*))?$/;function Ud(a,b){if(a)for(var c=a.split("\x26"),d=0;d<c.length;d++){var e=c[d].indexOf("\x3d"),g=null;if(0<=e){var h=c[d].substring(0,e);g=c[d].substring(e+1)}else h=c[d];b(h,g?Ea(g):"")}};function Vd(a){md.call(this);this.headers=new Pd;this.Nl=a||null;this.zb=!1;this.Ml=this.U=null;this.gg="";this.cg=this.ro=this.Ak=this.jn=!1;this.Tp=0;this.Cl=null;this.ct=Wd;this.aq=this.Tw=this.Wt=!1}r(Vd,md);var Wd="",Xd=/^https?$/i,Yd=["POST","PUT"];f=Vd.prototype;
f.send=function(a,b,c,d){if(this.U)throw Error("[goog.net.XhrIo] Object is active with another request\x3d"+this.gg+"; newUri\x3d"+a);b=b?b.toUpperCase():"GET";this.gg=a;this.jn=!1;this.zb=!0;this.U=this.Nl?Gd(this.Nl):Gd(Ed);this.Ml=this.Nl?Cd(this.Nl):Cd(Ed);this.U.onreadystatechange=q(this.Es,this);this.Tw&&"onprogress"in this.U&&(this.U.onprogress=q(function(a){this.Ds(a,!0)},this),this.U.upload&&(this.U.upload.onprogress=q(this.Ds,this)));try{this.ro=!0,this.U.open(b,String(a),!0),this.ro=!1}catch(g){Zd(this);
return}a=c||"";var e=this.headers.clone();d&&Kd(d,function(a,b){e.set(b,a)});d=kb(e.uc(),$d);c=ca.FormData&&a instanceof ca.FormData;!mb(Yd,b)||d||c||e.set("Content-Type","application/x-www-form-urlencoded;charset\x3dutf-8");e.forEach(function(a,b){this.U.setRequestHeader(b,a)},this);this.ct&&(this.U.responseType=this.ct);"withCredentials"in this.U&&this.U.withCredentials!==this.Wt&&(this.U.withCredentials=this.Wt);try{ae(this),0<this.Tp&&((this.aq=be(this.U))?(this.U.timeout=this.Tp,this.U.ontimeout=
q(this.Lt,this)):this.Cl=od(this.Lt,this.Tp,this)),this.Ak=!0,this.U.send(a),this.Ak=!1}catch(g){Zd(this)}};function be(a){return t&&uc(9)&&fa(a.timeout)&&da(a.ontimeout)}function $d(a){return"content-type"==a.toLowerCase()}f.Lt=function(){"undefined"!=typeof ba&&this.U&&(this.dispatchEvent("timeout"),this.abort(8))};function Zd(a){a.zb=!1;a.U&&(a.cg=!0,a.U.abort(),a.cg=!1);ce(a);de(a)}function ce(a){a.jn||(a.jn=!0,a.dispatchEvent("complete"),a.dispatchEvent("error"))}
f.abort=function(){this.U&&this.zb&&(this.zb=!1,this.cg=!0,this.U.abort(),this.cg=!1,this.dispatchEvent("complete"),this.dispatchEvent("abort"),de(this))};f.V=function(){this.U&&(this.zb&&(this.zb=!1,this.cg=!0,this.U.abort(),this.cg=!1),de(this,!0));Vd.m.V.call(this)};f.Es=function(){this.Re||(this.ro||this.Ak||this.cg?ee(this):this.Ew())};f.Ew=function(){ee(this)};
function ee(a){if(a.zb&&"undefined"!=typeof ba&&(!a.Ml[1]||4!=(a.U?a.U.readyState:0)||2!=fe(a)))if(a.Ak&&4==(a.U?a.U.readyState:0))od(a.Es,0,a);else if(a.dispatchEvent("readystatechange"),a.Dk()){a.zb=!1;try{a.Zr()?(a.dispatchEvent("complete"),a.dispatchEvent("success")):ce(a)}finally{de(a)}}}f.Ds=function(a,b){this.dispatchEvent(ge(a,"progress"));this.dispatchEvent(ge(a,b?"downloadprogress":"uploadprogress"))};
function ge(a,b){return{type:b,lengthComputable:a.lengthComputable,loaded:a.loaded,total:a.total}}function de(a,b){if(a.U){ae(a);var c=a.U,d=a.Ml[0]?ja:null;a.U=null;a.Ml=null;b||a.dispatchEvent("ready");try{c.onreadystatechange=d}catch(e){}}}function ae(a){a.U&&a.aq&&(a.U.ontimeout=null);fa(a.Cl)&&(ca.clearTimeout(a.Cl),a.Cl=null)}f.fg=function(){return!!this.U};f.Dk=function(){return 4==(this.U?this.U.readyState:0)};
f.Zr=function(){var a=fe(this);a:switch(a){case 200:case 201:case 202:case 204:case 206:case 304:case 1223:var b=!0;break a;default:b=!1}if(!b){if(a=0===a)a=String(this.gg).match(Td)[1]||null,!a&&ca.self&&ca.self.location&&(a=ca.self.location.protocol,a=a.substr(0,a.length-1)),a=!Xd.test(a?a.toLowerCase():"");b=a}return b};function fe(a){try{return 2<(a.U?a.U.readyState:0)?a.U.status:-1}catch(b){return-1}}f.ti=function(){try{return this.U?this.U.responseText:""}catch(a){return""}};
f.Mc=function(a){if(this.U){var b=this.U.responseText;a&&0==b.indexOf(a)&&(b=b.substring(a.length));return qd(b)}};f.getResponseHeader=function(a){if(this.U&&this.Dk())return a=this.U.getResponseHeader(a),null===a?void 0:a};f.getAllResponseHeaders=function(){return this.U&&this.Dk()?this.U.getAllResponseHeaders():""};
function ie(a){var b={};a=a.getAllResponseHeaders().split("\r\n");for(var c=0;c<a.length;c++)if(!Aa(a[c])){var d=2;for(var e=a[c].split(": "),g=[];0<d&&e.length;)g.push(e.shift()),d--;e.length&&g.push(e.join(": "));d=g;b[d[0]]=b[d[0]]?b[d[0]]+(", "+d[1]):d[1]}return b};function je(a){null==a&&(a={});this.ka=a.ka;this.ig=a.ig;this.rf=a.rf;null==this.ka&&(this.ka=!0);null==this.ig&&(this.ig=!0);null==this.rf&&(this.rf=!0)};function ke(a){var b=this;ke.m.constructor.call(this,a);this.ls=[];this.qc=new je;this.A("complete",function(){return b.dispatchEvent(le)})}r(ke,Vd);var le="done";ke.prototype.send=function(a,b,c,d){var e;this.dispatchEvent("send");var g=this.ls;var h=0;for(e=g.length;h<e;h++){var k=g[h];k.apply(d)}return ke.m.send.call(this,a,b,c,d)};function me(a,b,c,d){this.ka=a;this.Uu=b;this.jx=c;this.l=d;this.Uf=q(this.Uf,this);this.handleError=q(this.handleError,this);this.Hn=q(this.Hn,this);this.uj=this.Hq=0;this.pm=null;this.Yt=[];this.navigator=ne(this.l).navigator;this.ss=oe(this);this.bt=[];t||v(ne(this.l),pe,this.Hn)}var pe="beforeunload";f=me.prototype;f.Hn=function(){var a;var b=this.Yt;var c=0;for(a=b.length;c<a;c++){var d=b[c];d.abort()}};
f.ic=function(a){null==a&&(a=new je);a=this.Em(a);a.A("error",this.handleError);a.A("complete",this.Uf);return a};f.handleError=function(a){qe(this,a);var b=a.target;if(this.xo(b)){var c=b.Mc();a=a.target.qc;a.ka&&this.ka.Ij();return re(this,c,a,fe(b))}a.target.qc.ka&&(c=oe(this),0===fe(b)&&!1===c&&!1!==this.ss?this.ka.yw():500<=fe(b)&&this.ka.error(fe(b)),this.ss=c)};function oe(a){var b=null;null!=a.navigator&&null!=a.navigator.onLine&&(b=a.navigator.onLine?!0:!1);return b}
f.Uf=function(a){qe(this,a);var b=a.target;if(204!==fe(b)){a=a.target.qc;try{var c=b.Mc();a.ka&&this.ka.Ij();return re(this,c,a,fe(b))}catch(d){}}};function re(a,b,c,d){c.ka&&a.ka.apply(b,d);if(c.rf)return a.jx.apply(b)}
f.Em=function(a){var b=this;var c=new ke;c.qc=a;this.Yt.push(c);c.Nc=this.Hq;this.Hq++;c.ls.push(this.Uu);c.A("send",function(c){a=c.target.qc;0===b.uj&&a.ka&&a.ig&&(c=b.ka.ig(),b.pm=c.pop());return b.uj++});c.A(le,function(){b.uj--;if(0===b.uj&&null!=b.pm)return b.pm.destroy()});return c};f.xo=function(a){a=a.getResponseHeader("Content-Type");return null!=a&&-1!==a.indexOf("json")};function se(a,b){a.bt.push(b)}
function qe(a,b){var c;var d=a.bt;var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];e.push(h.apply(b))}};var te=!t||vc(9),ue=!jc&&!t||t&&vc(9)||jc&&uc("1.9.1"),ve=t&&!uc("9"),we=t||gc||kc;var xe={area:!0,base:!0,br:!0,col:!0,command:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0};function ye(){this.yl="";this.Fu=ze}ye.prototype.af=!0;ye.prototype.ve=function(){return this.yl};ye.prototype.toString=function(){return"Const{"+this.yl+"}"};function Ae(a){return a instanceof ye&&a.constructor===ye&&a.Fu===ze?a.yl:"type_error:Const"}var ze={};function Be(a){var b=new ye;b.yl=a;return b}Be("");function Ce(){this.lp="";this.Bu=De}Ce.prototype.af=!0;var De={};Ce.prototype.ve=function(){return this.lp};Ce.prototype.bh=function(a){this.lp=a;return this};var Ee=(new Ce).bh(""),Fe=/^([-,."'%_!# a-zA-Z0-9]+|(?:rgb|hsl)a?\([0-9.%, ]+\))$/;function Ge(){this.mp="";this.Iu=He}Ge.prototype.af=!0;Ge.prototype.ve=function(){return this.mp};Ge.prototype.po=!0;Ge.prototype.te=function(){return 1};function Ie(a){if(a instanceof Ge&&a.constructor===Ge&&a.Iu===He)return a.mp;n(a);return"type_error:TrustedResourceUrl"}var He={};function Ke(a){var b=new Ge;b.mp=a;return b};function Le(){this.kf="";this.Cu=Me}Le.prototype.af=!0;Le.prototype.ve=function(){return this.kf};Le.prototype.po=!0;Le.prototype.te=function(){return 1};function Ne(a){if(a instanceof Le&&a.constructor===Le&&a.Cu===Me)return a.kf;n(a);return"type_error:SafeUrl"}var Oe=/^(?:(?:https?|mailto|ftp):|[^:/?#]*(?:[/?#]|$))/i;function Pe(a){if(a instanceof Le)return a;a=a.af?a.ve():String(a);Oe.test(a)||(a="about:invalid#zClosurez");return Qe(a)}var Me={};function Qe(a){var b=new Le;b.kf=a;return b}Qe("about:blank");function Re(){this.kf="";this.Au=Se;this.en=null}Re.prototype.po=!0;Re.prototype.te=function(){return this.en};Re.prototype.af=!0;Re.prototype.ve=function(){return this.kf};function Te(a){if(a instanceof Re&&a.constructor===Re&&a.Au===Se)return a.kf;n(a);return"type_error:SafeHtml"}function Ue(a){if(a instanceof Re)return a;var b=null;a.po&&(b=a.te());return Ve(Ga(a.af?a.ve():String(a)),b)}
function We(a){if(a instanceof Re)return a;a=Ue(a);var b=Te(a);b=Fa(b.replace(/  /g," \x26#160;"),void 0);return Ve(b,a.te())}var Xe=/^[a-zA-Z0-9-]+$/,Ye={action:!0,cite:!0,data:!0,formaction:!0,href:!0,manifest:!0,poster:!0,src:!0},Ze={APPLET:!0,BASE:!0,EMBED:!0,IFRAME:!0,LINK:!0,MATH:!0,META:!0,OBJECT:!0,SCRIPT:!0,STYLE:!0,SVG:!0,TEMPLATE:!0};
function $e(a,b,c){var d=String(a);if(!Xe.test(d))throw Error("Invalid tag name \x3c"+d+"\x3e.");if(d.toUpperCase()in Ze)throw Error("Tag name \x3c"+d+"\x3e is not allowed for SafeHtml.");return af(String(a),b,c)}function bf(a){function b(a){la(a)?db(a,b):(a=Ue(a),d+=Te(a),a=a.te(),0==c?c=a:0!=a&&c!=a&&(c=null))}var c=0,d="";db(arguments,b);return Ve(d,c)}var Se={};function Ve(a,b){return(new Re).bh(a,b)}Re.prototype.bh=function(a,b){this.kf=a;this.en=b;return this};
function af(a,b,c){var d=null,e="";if(b)for(E in b){if(!Xe.test(E))throw Error('Invalid attribute name "'+E+'".');var g=b[E];if(null!=g){var h=a;var k=E;var l=g;if(l instanceof ye)l=Ae(l);else if("style"==k.toLowerCase()){g=void 0;h=l;if(!oa(h))throw Error('The "style" attribute requires goog.html.SafeStyle or map of style properties, '+typeof h+" given: "+h);if(!(h instanceof Ce)){l="";for(g in h){if(!/^[-_a-zA-Z0-9]+$/.test(g))throw Error("Name allows only [-_a-zA-Z0-9], got: "+g);var m=h[g];if(null!=
m){if(m instanceof ye)m=Ae(m);else if(Fe.test(m)){for(var p=!0,u=!0,A=0;A<m.length;A++){var D=m.charAt(A);"'"==D&&u?p=!p:'"'==D&&p&&(u=!u)}p&&u||(m="zClosurez")}else m="zClosurez";l+=g+":"+m+";"}}h=l?(new Ce).bh(l):Ee}h instanceof Ce&&h.constructor===Ce&&h.Bu===De?g=h.lp:(n(h),g="type_error:SafeStyle");l=g}else{if(/^on/i.test(k))throw Error('Attribute "'+k+'" requires goog.string.Const value, "'+l+'" given.');if(k.toLowerCase()in Ye)if(l instanceof Ge)l=Ie(l);else if(l instanceof Le)l=Ne(l);else if(ea(l))l=
Pe(l).ve();else throw Error('Attribute "'+k+'" on tag "'+h+'" requires goog.html.SafeUrl, goog.string.Const, or string, value "'+l+'" given.');}l.af&&(l=l.ve());k=k+'\x3d"'+Ga(String(l))+'"';e+=" "+k}}var E="\x3c"+a+e;null!=c?la(c)||(c=[c]):c=[];!0===xe[a.toLowerCase()]?E+="\x3e":(d=bf(c),E+="\x3e"+Te(d)+"\x3c/"+a+"\x3e",d=d.te());(a=b&&b.dir)&&(d=/^(ltr|rtl|auto)$/i.test(a)?0:null);return Ve(E,d)}Ve("\x3c!DOCTYPE html\x3e",0);var cf=Ve("",0);Ve("\x3cbr\x3e",0);function df(a,b){this.x=da(a)?a:0;this.y=da(b)?b:0}f=df.prototype;f.clone=function(){return new df(this.x,this.y)};f.tb=function(a){return a instanceof df&&ef(this,a)};function ef(a,b){return a==b?!0:a&&b?a.x==b.x&&a.y==b.y:!1}function ff(a,b){return new df(a.x-b.x,a.y-b.y)}f.ceil=function(){this.x=Math.ceil(this.x);this.y=Math.ceil(this.y);return this};f.floor=function(){this.x=Math.floor(this.x);this.y=Math.floor(this.y);return this};
f.round=function(){this.x=Math.round(this.x);this.y=Math.round(this.y);return this};f.translate=function(a,b){a instanceof df?(this.x+=a.x,this.y+=a.y):(this.x+=Number(a),fa(b)&&(this.y+=b));return this};f.scale=function(a,b){var c=fa(b)?b:a;this.x*=a;this.y*=c;return this};function gf(a,b){this.width=a;this.height=b}f=gf.prototype;f.clone=function(){return new gf(this.width,this.height)};f.area=function(){return this.width*this.height};f.isEmpty=function(){return!this.area()};f.ceil=function(){this.width=Math.ceil(this.width);this.height=Math.ceil(this.height);return this};f.floor=function(){this.width=Math.floor(this.width);this.height=Math.floor(this.height);return this};f.round=function(){this.width=Math.round(this.width);this.height=Math.round(this.height);return this};
f.scale=function(a,b){var c=fa(b)?b:a;this.width*=a;this.height*=c;return this};function hf(a){return a?new jf(kf(a)):ya||(ya=new jf)}function lf(a,b){return ea(b)?a.getElementById(b):b}function mf(a,b){return(b||document).getElementsByTagName(String(a))}function nf(a,b){var c=b||document;return(c.getElementsByClassName?c.getElementsByClassName(a)[0]:c.querySelectorAll&&c.querySelector?c.querySelector("."+a):of(document,"*",a,b)[0])||null}
function of(a,b,c,d){a=d||a;var e=b&&"*"!=b?String(b).toUpperCase():"";if(a.querySelectorAll&&a.querySelector&&(e||c))return a.querySelectorAll(e+(c?"."+c:""));if(c&&a.getElementsByClassName){d=a.getElementsByClassName(c);if(e){a={};for(var g=b=0,h;h=d[g];g++)e==h.nodeName&&(a[b++]=h);a.length=b;return a}return d}d=a.getElementsByTagName(e||"*");if(c){a={};for(g=b=0;h=d[g];g++)e=h.className,"function"==typeof e.split&&mb(e.split(/\s+/),c)&&(a[b++]=h);a.length=b;return a}return d}
function qf(a,b){Db(b,function(b,d){b&&b.af&&(b=b.ve());"style"==d?a.style.cssText=b:"class"==d?a.className=b:"for"==d?a.htmlFor=b:rf.hasOwnProperty(d)?a.setAttribute(rf[d],b):0==d.lastIndexOf("aria-",0)||0==d.lastIndexOf("data-",0)?a.setAttribute(d,b):a[d]=b})}var rf={cellpadding:"cellPadding",cellspacing:"cellSpacing",colspan:"colSpan",frameborder:"frameBorder",height:"height",maxlength:"maxLength",nonce:"nonce",role:"role",rowspan:"rowSpan",type:"type",usemap:"useMap",valign:"vAlign",width:"width"};
function sf(a){a=a.document;a=tf(a)?a.documentElement:a.body;return new gf(a.clientWidth,a.clientHeight)}function uf(a){var b=vf(a);a=a.parentWindow||a.defaultView;return t&&uc("10")&&a.pageYOffset!=b.scrollTop?new df(b.scrollLeft,b.scrollTop):new df(a.pageXOffset||b.scrollLeft,a.pageYOffset||b.scrollTop)}function vf(a){return a.scrollingElement?a.scrollingElement:!kc&&tf(a)?a.documentElement:a.body||a.documentElement}function wf(a){return a?a.parentWindow||a.defaultView:window}
function xf(a,b,c){return yf(document,arguments)}function yf(a,b){var c=String(b[0]),d=b[1];if(!te&&d&&(d.name||d.type)){c=["\x3c",c];d.name&&c.push(' name\x3d"',Ga(d.name),'"');if(d.type){c.push(' type\x3d"',Ga(d.type),'"');var e={};Qb(e,d);delete e.type;d=e}c.push("\x3e");c=c.join("")}c=a.createElement(c);d&&(ea(d)?c.className=d:la(d)?c.className=d.join(" "):qf(c,d));2<b.length&&zf(a,c,b,2);return c}
function zf(a,b,c,d){function e(c){c&&b.appendChild(ea(c)?a.createTextNode(c):c)}for(;d<c.length;d++){var g=c[d];!ma(g)||oa(g)&&0<g.nodeType?e(g):db(Af(g)?qb(g):g,e)}}function tf(a){return"CSS1Compat"==a.compatMode}function Bf(a,b){zf(kf(a),a,arguments,1)}function Cf(a){for(var b;b=a.firstChild;)a.removeChild(b)}function Df(a,b){b.parentNode&&b.parentNode.insertBefore(a,b)}function Ef(a,b){b.parentNode&&b.parentNode.insertBefore(a,b.nextSibling)}
function Ff(a){return a&&a.parentNode?a.parentNode.removeChild(a):null}function Gf(a,b){for(;a&&1!=a.nodeType;)a=b?a.nextSibling:a.previousSibling;return a}function Hf(a){return oa(a)&&1==a.nodeType}function If(a,b){if(!a||!b)return!1;if(a.contains&&1==b.nodeType)return a==b||a.contains(b);if("undefined"!=typeof a.compareDocumentPosition)return a==b||!!(a.compareDocumentPosition(b)&16);for(;b&&a!=b;)b=b.parentNode;return b==a}function kf(a){return 9==a.nodeType?a:a.ownerDocument||a.document}
function Jf(a){return a.contentDocument||a.contentWindow.document}function Kf(a,b){if("textContent"in a)a.textContent=b;else if(3==a.nodeType)a.data=b;else if(a.firstChild&&3==a.firstChild.nodeType){for(;a.lastChild!=a.firstChild;)a.removeChild(a.lastChild);a.firstChild.data=b}else Cf(a),a.appendChild(kf(a).createTextNode(String(b)))}var Lf={SCRIPT:1,STYLE:1,HEAD:1,IFRAME:1,OBJECT:1},Mf={IMG:" ",BR:"\n"};function Nf(a){return Of(a)&&Pf(a)}
function Qf(a,b){b?a.tabIndex=0:(a.tabIndex=-1,a.removeAttribute("tabIndex"))}function Of(a){return t&&!uc("9")?(a=a.getAttributeNode("tabindex"),null!=a&&a.specified):a.hasAttribute("tabindex")}function Pf(a){a=a.tabIndex;return fa(a)&&0<=a&&32768>a}
function Rf(a){if(ve&&null!==a&&"innerText"in a)a=a.innerText.replace(/(\r\n|\r|\n)/g,"\n");else{var b=[];Sf(a,b,!0);a=b.join("")}a=a.replace(/ \xAD /g," ").replace(/\xAD/g,"");a=a.replace(/\u200B/g,"");ve||(a=a.replace(/ +/g," "));" "!=a&&(a=a.replace(/^\s*/,""));return a}function Tf(a){var b=[];Sf(a,b,!1);return b.join("")}
function Sf(a,b,c){if(!(a.nodeName in Lf))if(3==a.nodeType)c?b.push(String(a.nodeValue).replace(/(\r\n|\r|\n)/g,"")):b.push(a.nodeValue);else if(a.nodeName in Mf)b.push(Mf[a.nodeName]);else for(a=a.firstChild;a;)Sf(a,b,c),a=a.nextSibling}function Af(a){if(a&&"number"==typeof a.length){if(oa(a))return"function"==typeof a.item||"string"==typeof a.item;if(na(a))return"function"==typeof a.item}return!1}function Uf(a){try{return a&&a.activeElement}catch(b){}return null}
function jf(a){this.R=a||ca.document||document}f=jf.prototype;f.I=hf;f.o=function(a){return lf(this.R,a)};f.getElementsByTagName=function(a,b){return(b||this.R).getElementsByTagName(String(a))};f.xn=function(a,b){var c=b||this.R,d=c||document;return d.querySelectorAll&&d.querySelector?d.querySelectorAll("."+a):of(document,"*",a,c)};f.jk=function(a,b){return nf(a,b||this.R)};f.ex=qf;f.D=function(a,b,c){return yf(this.R,arguments)};f.createElement=function(a){return this.R.createElement(String(a))};
f.createTextNode=function(a){return this.R.createTextNode(String(a))};function ne(a){a=a.R;return a.parentWindow||a.defaultView}f.sn=function(a){return Uf(a||this.R)};f.appendChild=function(a,b){a.appendChild(b)};f.append=Bf;f.canHaveChildren=function(a){if(1!=a.nodeType)return!1;switch(a.tagName){case "APPLET":case "AREA":case "BASE":case "BR":case "COL":case "COMMAND":case "EMBED":case "FRAME":case "HR":case "IMG":case "INPUT":case "IFRAME":case "ISINDEX":case "KEYGEN":case "LINK":case "NOFRAMES":case "NOSCRIPT":case "META":case "OBJECT":case "PARAM":case "SCRIPT":case "SOURCE":case "STYLE":case "TRACK":case "WBR":return!1}return!0};
f.sh=Cf;f.vo=Df;f.Ki=Ef;function Vf(a,b){a.insertBefore(b,a.childNodes[0]||null)}f.removeNode=Ff;function Wf(a,b){var c=b.parentNode;c&&c.replaceChild(a,b)}f.se=function(a){return ue&&void 0!=a.children?a.children:eb(a.childNodes,function(a){return 1==a.nodeType})};function Xf(a){return da(a.nextElementSibling)?a.nextElementSibling:Gf(a.nextSibling,!0)}
function Yf(a){var b;if(we&&!(t&&uc("9")&&!uc("10")&&ca.SVGElement&&a instanceof ca.SVGElement)&&(b=a.parentElement))return b;b=a.parentNode;return Hf(b)?b:null}f.contains=If;f.Yc=Kf;
f.ye=function(a){var b;(b="A"==a.tagName||"INPUT"==a.tagName||"TEXTAREA"==a.tagName||"SELECT"==a.tagName||"BUTTON"==a.tagName?!a.disabled&&(!Of(a)||Pf(a)):Nf(a))&&t?(a=!na(a.getBoundingClientRect)||t&&null==a.parentElement?{height:a.offsetHeight,width:a.offsetWidth}:a.getBoundingClientRect(),a=null!=a&&0<a.height&&0<a.width):a=b;return a};f.Vg=Rf;function Zf(a,b,c,d){this.top=a;this.right=b;this.bottom=c;this.left=d}f=Zf.prototype;f.Jr=function(){return this.bottom-this.top};f.clone=function(){return new Zf(this.top,this.right,this.bottom,this.left)};f.contains=function(a){return this&&a?a instanceof Zf?a.left>=this.left&&a.right<=this.right&&a.top>=this.top&&a.bottom<=this.bottom:a.x>=this.left&&a.x<=this.right&&a.y>=this.top&&a.y<=this.bottom:!1};
f.expand=function(a,b,c,d){oa(a)?(this.top-=a.top,this.right+=a.right,this.bottom+=a.bottom,this.left-=a.left):(this.top-=a,this.right+=Number(b),this.bottom+=Number(c),this.left-=Number(d));return this};f.ceil=function(){this.top=Math.ceil(this.top);this.right=Math.ceil(this.right);this.bottom=Math.ceil(this.bottom);this.left=Math.ceil(this.left);return this};
f.floor=function(){this.top=Math.floor(this.top);this.right=Math.floor(this.right);this.bottom=Math.floor(this.bottom);this.left=Math.floor(this.left);return this};f.round=function(){this.top=Math.round(this.top);this.right=Math.round(this.right);this.bottom=Math.round(this.bottom);this.left=Math.round(this.left);return this};f.translate=function(a,b){a instanceof df?(this.left+=a.x,this.right+=a.x,this.top+=a.y,this.bottom+=a.y):(this.left+=a,this.right+=a,fa(b)&&(this.top+=b,this.bottom+=b));return this};
f.scale=function(a,b){var c=fa(b)?b:a;this.left*=a;this.right*=a;this.top*=c;this.bottom*=c;return this};function $f(a,b,c,d){this.left=a;this.top=b;this.width=c;this.height=d}f=$f.prototype;f.clone=function(){return new $f(this.left,this.top,this.width,this.height)};f.Wr=function(a){var b=Math.max(this.left,a.left),c=Math.min(this.left+this.width,a.left+a.width);if(b<=c){var d=Math.max(this.top,a.top);a=Math.min(this.top+this.height,a.top+a.height);if(d<=a)return this.left=b,this.top=d,this.width=c-b,this.height=a-d,!0}return!1};
f.contains=function(a){return a instanceof df?a.x>=this.left&&a.x<=this.left+this.width&&a.y>=this.top&&a.y<=this.top+this.height:this.left<=a.left&&this.left+this.width>=a.left+a.width&&this.top<=a.top&&this.top+this.height>=a.top+a.height};f.ceil=function(){this.left=Math.ceil(this.left);this.top=Math.ceil(this.top);this.width=Math.ceil(this.width);this.height=Math.ceil(this.height);return this};
f.floor=function(){this.left=Math.floor(this.left);this.top=Math.floor(this.top);this.width=Math.floor(this.width);this.height=Math.floor(this.height);return this};f.round=function(){this.left=Math.round(this.left);this.top=Math.round(this.top);this.width=Math.round(this.width);this.height=Math.round(this.height);return this};f.translate=function(a,b){a instanceof df?(this.left+=a.x,this.top+=a.y):(this.left+=a,fa(b)&&(this.top+=b));return this};
f.scale=function(a,b){var c=fa(b)?b:a;this.left*=a;this.width*=a;this.top*=c;this.height*=c;return this};function ag(a,b,c){if(ea(b))(b=bg(a,b))&&(a.style[b]=c);else for(var d in b){c=a;var e=b[d],g=bg(c,d);g&&(c.style[g]=e)}}var cg={};function bg(a,b){var c=cg[b];if(!c){var d=Xa(b),c=d;void 0===a.style[d]&&(d=(kc?"Webkit":jc?"Moz":t?"ms":gc?"O":null)+$a(d),void 0!==a.style[d]&&(c=d));cg[b]=c}return c}function dg(a,b){var c=kf(a);return c.defaultView&&c.defaultView.getComputedStyle&&(c=c.defaultView.getComputedStyle(a,null))?c[b]||c.getPropertyValue(b)||"":""}
function eg(a,b){return dg(a,b)||(a.currentStyle?a.currentStyle[b]:null)||a.style&&a.style[b]}function fg(a){return eg(a,"position")}function gg(a,b,c){if(b instanceof df){var d=b.x;b=b.y}else d=b,b=c;a.style.left=hg(d,!1);a.style.top=hg(b,!1)}function ig(a){a=a?kf(a):document;return!t||vc(9)||tf(hf(a).R)?a.documentElement:a.body}
function jg(a){try{var b=a.getBoundingClientRect()}catch(c){return{left:0,top:0,right:0,bottom:0}}t&&a.ownerDocument.body&&(a=a.ownerDocument,b.left-=a.documentElement.clientLeft+a.body.clientLeft,b.top-=a.documentElement.clientTop+a.body.clientTop);return b}
function kg(a){if(t&&!vc(8))return a.offsetParent;var b=kf(a),c=eg(a,"position"),d="fixed"==c||"absolute"==c;for(a=a.parentNode;a&&a!=b;a=a.parentNode)if(11==a.nodeType&&a.host&&(a=a.host),c=eg(a,"position"),d=d&&"static"==c&&a!=b.documentElement&&a!=b.body,!d&&(a.scrollWidth>a.clientWidth||a.scrollHeight>a.clientHeight||"fixed"==c||"absolute"==c||"relative"==c))return a;return null}
function lg(a){for(var b=new Zf(0,Infinity,Infinity,0),c=hf(a),d=c.R.body,e=c.R.documentElement,g=vf(c.R);a=kg(a);)if(!(t&&0==a.clientWidth||kc&&0==a.clientHeight&&a==d)&&a!=d&&a!=e&&"visible"!=eg(a,"overflow")){var h=mg(a),k=new df(a.clientLeft,a.clientTop);h.x+=k.x;h.y+=k.y;b.top=Math.max(b.top,h.y);b.right=Math.min(b.right,h.x+a.clientWidth);b.bottom=Math.min(b.bottom,h.y+a.clientHeight);b.left=Math.max(b.left,h.x)}d=g.scrollLeft;g=g.scrollTop;b.left=Math.max(b.left,d);b.top=Math.max(b.top,g);
c=sf(ne(c)||window);b.right=Math.min(b.right,d+c.width);b.bottom=Math.min(b.bottom,g+c.height);return 0<=b.top&&0<=b.left&&b.bottom>b.top&&b.right>b.left?b:null}
function ng(a,b){var c=b||vf(document),d=c||vf(document);var e=mg(a);var g=mg(d),h=qg(d);if(d==vf(document)){var k=e.x-d.scrollLeft;e=e.y-d.scrollTop;t&&!vc(10)&&(k+=h.left,e+=h.top)}else k=e.x-g.x-h.left,e=e.y-g.y-h.top;var h=rg(a),g=d.clientHeight-h.height,l=d.scrollLeft,m=d.scrollTop,l=l+Math.min(k,Math.max(k-(d.clientWidth-h.width),0)),m=m+Math.min(e,Math.max(e-g,0));k=new df(l,m);c.scrollLeft=k.x;c.scrollTop=k.y}
function mg(a){var b=kf(a),c=new df(0,0),d=ig(b);if(a==d)return c;a=jg(a);b=uf(hf(b).R);c.x=a.left+b.x;c.y=a.top+b.y;return c}function sg(a,b,c){if(b instanceof gf)c=b.height,b=b.width;else if(void 0==c)throw Error("missing height argument");a.style.width=hg(b,!0);a.style.height=hg(c,!0)}function hg(a,b){"number"==typeof a&&(a=(b?Math.round(a):a)+"px");return a}
function tg(a){var b=rg;if("none"!=eg(a,"display"))return b(a);var c=a.style,d=c.display,e=c.visibility,g=c.position;c.visibility="hidden";c.position="absolute";c.display="inline";a=b(a);c.display=d;c.position=g;c.visibility=e;return a}function rg(a){var b=a.offsetWidth,c=a.offsetHeight,d=kc&&!b&&!c;return da(b)&&!d||!a.getBoundingClientRect?new gf(b,c):(a=jg(a),new gf(a.right-a.left,a.bottom-a.top))}
function ug(a,b){var c=a.style;"opacity"in c?c.opacity=b:"MozOpacity"in c?c.MozOpacity=b:"filter"in c&&(c.filter=""===b?"":"alpha(opacity\x3d"+100*Number(b)+")")}function w(a,b){a.style.display=b?"":"none"}function vg(a){return"rtl"==eg(a,"direction")}var wg=jc?"MozUserSelect":kc||hc?"WebkitUserSelect":null;
function xg(a,b,c){c=c?null:a.getElementsByTagName("*");if(wg){if(b=b?"none":"",a.style&&(a.style[wg]=b),c){a=0;for(var d;d=c[a];a++)d.style&&(d.style[wg]=b)}}else if(t||gc)if(b=b?"on":"",a.setAttribute("unselectable",b),c)for(a=0;d=c[a];a++)d.setAttribute("unselectable",b)}
function yg(a,b){if(/^\d+px?$/.test(b))return parseInt(b,10);var c=a.style.left,d=a.runtimeStyle.left;a.runtimeStyle.left=a.currentStyle.left;a.style.left=b;var e=a.style.pixelLeft;a.style.left=c;a.runtimeStyle.left=d;return+e}function zg(a,b){var c=a.currentStyle?a.currentStyle[b]:null;return c?yg(a,c):0}var Ag={thin:2,medium:4,thick:6};
function Bg(a,b){if("none"==(a.currentStyle?a.currentStyle[b+"Style"]:null))return 0;var c=a.currentStyle?a.currentStyle[b+"Width"]:null;return c in Ag?Ag[c]:yg(a,c)}function qg(a){if(t&&!vc(9)){var b=Bg(a,"borderLeft");var c=Bg(a,"borderRight");var d=Bg(a,"borderTop");a=Bg(a,"borderBottom");return new Zf(d,c,a,b)}b=dg(a,"borderLeftWidth");c=dg(a,"borderRightWidth");d=dg(a,"borderTopWidth");a=dg(a,"borderBottomWidth");return new Zf(parseFloat(d),parseFloat(c),parseFloat(a),parseFloat(b))};function Cg(a){Cg.m.constructor.call(this,a)}r(Cg,jf);f=Cg.prototype;f.G=function(a,b,c){var d;null==b&&(b=null);null==c&&(c=null);b=this.D(a,b);if(ea(c))this.Yc(b,c);else if(null!=c&&null!=c.nodeType)c.appendChild(b);else if(la(c)){var e=0;for(d=c.length;e<d;e++)a=c[e],b.appendChild(a)}return b};f.show=function(a){return w(a,!0)};f.hide=function(a){return w(a,!1)};f.jo=function(a){return null!=a.children&&0<a.children.length};
function Dg(a,b,c){null==c&&(c=null);null==c&&(c=a.R);return c.querySelector(b)}f.all=function(a,b){null==b&&(b=null);null==b&&(b=this.R);return b.querySelectorAll(a)};function Eg(a,b,c){null==c&&(c=null);return a.jk(b,c)}function Fg(a,b,c){null==c&&(c=null);return a.xn(b,c)}function Gg(a,b,c,d){var e;c=String(c);var g=a.all("option",b);var h=[];b=0;for(e=g.length;b<e;b++)if(a=g[b],a.value===c)if(d(a))break;else h.push(void 0);else h.push(void 0);return h}
f.select=function(a,b){return Gg(this,a,b,function(a){return a.selected=!0})};f.unselect=function(a,b){return Gg(this,a,b,function(a){return a.selected=!1})};function Hg(a){var b;if(t)for(b=[];a.hasChildNodes();)b.push(a.removeChild(a.lastChild));else a.innerHTML=""};function Ig(){md.call(this);this.ma=Jg;this.endTime=this.startTime=null}r(Ig,md);var Jg=0;Ig.prototype.ze=function(){this.Cd("begin")};Ig.prototype.hf=function(){this.Cd("end")};Ig.prototype.Cd=function(a){this.dispatchEvent(a)};function Kg(a,b,c){Vb.call(this);this.Oo=a;this.eg=b||0;this.vb=c;this.Yu=q(this.Av,this)}r(Kg,Vb);f=Kg.prototype;f.Nc=0;f.V=function(){Kg.m.V.call(this);this.stop();delete this.Oo;delete this.vb};f.start=function(a){this.stop();this.Nc=od(this.Yu,da(a)?a:this.eg)};f.stop=function(){this.fg()&&ca.clearTimeout(this.Nc);this.Nc=0};f.fg=function(){return 0!=this.Nc};f.Av=function(){this.Nc=0;this.Oo&&this.Oo.call(this.vb)};var Lb={},Lg=null;function Mg(a){a=pa(a);delete Lb[a];Kb()&&Lg&&Lg.stop()}function Ng(){Lg||(Lg=new Kg(function(){Og()},20));var a=Lg;a.fg()||a.start()}function Og(){var a=wa();Db(Lb,function(b){b.Wj(a)});Kb()||Ng()};function Pg(a,b,c,d){Ig.call(this);if(!la(a)||!la(b))throw Error("Start and end parameters must be arrays");if(a.length!=b.length)throw Error("Start and end points must be the same length");this.gj=a;this.Dv=b;this.duration=c;this.Iq=d;this.coords=[];this.Kl=!1;this.progress=0}r(Pg,Ig);f=Pg.prototype;
f.play=function(a){if(a||this.ma==Jg)this.progress=0,this.coords=this.gj;else if(1==this.ma)return!1;Mg(this);this.startTime=a=wa();-1==this.ma&&(this.startTime-=this.duration*this.progress);this.endTime=this.startTime+this.duration;this.progress||this.ze();this.Cd("play");-1==this.ma&&this.Cd("resume");this.ma=1;var b=pa(this);b in Lb||(Lb[b]=this);Ng();this.Wj(a);return!0};f.stop=function(a){Mg(this);this.ma=Jg;a&&(this.progress=1);Qg(this,this.progress);this.Cd("stop");this.hf()};
f.pause=function(){1==this.ma&&(Mg(this),this.ma=-1,this.Cd("pause"))};f.V=function(){this.ma==Jg||this.stop(!1);this.Cd("destroy");Pg.m.V.call(this)};f.destroy=function(){this.ib()};f.Wj=function(a){a<this.startTime&&(this.endTime=a+this.endTime-this.startTime,this.startTime=a);this.progress=(a-this.startTime)/(this.endTime-this.startTime);1<this.progress&&(this.progress=1);Qg(this,this.progress);1==this.progress?(this.ma=Jg,Mg(this),this.Cd("finish"),this.hf()):1==this.ma&&this.Zo()};
function Qg(a,b){na(a.Iq)&&(b=a.Iq(b));a.coords=Array(a.gj.length);for(var c=0;c<a.gj.length;c++)a.coords[c]=(a.Dv[c]-a.gj[c])*b+a.gj[c]}f.Zo=function(){this.Cd("animate")};f.Cd=function(a){this.dispatchEvent(new Rg(a,this))};function Rg(a,b){Cc.call(this,a);this.coords=b.coords;this.x=b.coords[0];this.y=b.coords[1];this.z=b.coords[2];this.duration=b.duration;this.progress=b.progress;this.state=b.ma}r(Rg,Cc);function Sg(a,b,c,d,e){Pg.call(this,b,c,d,e);this.element=a}r(Sg,Pg);f=Sg.prototype;f.Il=ja;f.Fk=function(){da(this.Ld)||(this.Ld=vg(this.element));return this.Ld};f.Zo=function(){this.Il();Sg.m.Zo.call(this)};f.hf=function(){this.Il();Sg.m.hf.call(this)};f.ze=function(){this.Il();Sg.m.ze.call(this)};function Tg(a,b,c,d,e){fa(b)&&(b=[b]);fa(c)&&(c=[c]);Sg.call(this,a,b,c,d,e);if(1!=b.length||1!=c.length)throw Error("Start and end points must be 1D");this.Kk=Ug}r(Tg,Sg);var Vg=1/1024,Ug=-1;f=Tg.prototype;
f.Il=function(){var a=this.coords[0];Math.abs(a-this.Kk)>=Vg&&(ug(this.element,a),this.Kk=a)};f.ze=function(){this.Kk=Ug;Tg.m.ze.call(this)};f.hf=function(){this.Kk=Ug;Tg.m.hf.call(this)};f.show=function(){this.element.style.display=""};f.hide=function(){this.element.style.display="none"};function Wg(a,b,c){Tg.call(this,a,1,0,b,c)}r(Wg,Tg);Wg.prototype.ze=function(){this.show();Wg.m.ze.call(this)};Wg.prototype.hf=function(){this.hide();Wg.m.hf.call(this)};
function Xg(a,b,c){Tg.call(this,a,0,1,b,c)}r(Xg,Tg);Xg.prototype.ze=function(){this.show();Xg.m.ze.call(this)};function Yg(a){a=a.className;return ea(a)&&a.match(/\S+/g)||[]}function Zg(a,b){var c=Yg(a),d=sb(arguments,1),e=c.length+d.length;$g(c,d);a.className=c.join(" ");return c.length==e}function ah(a,b){var c=Yg(a),d=sb(arguments,1),e=bh(c,d);a.className=e.join(" ");return e.length==c.length-d.length}function $g(a,b){for(var c=0;c<b.length;c++)mb(a,b[c])||a.push(b[c])}function bh(a,b){return eb(a,function(a){return!mb(b,a)})}
function ch(a,b,c){var d=Yg(a);ea(b)?nb(d,b):la(b)&&(d=bh(d,b));ea(c)&&!mb(d,c)?d.push(c):la(c)&&$g(d,c);a.className=d.join(" ")}function dh(a,b){return mb(Yg(a),b)}function eh(a,b){var c=!dh(a,b);c?Zg(a,b):ah(a,b);return c};function fh(a){Vb.call(this);this.vb=a;this.Ga={}}r(fh,Vb);var gh=[];f=fh.prototype;f.A=function(a,b,c,d){la(b)||(b&&(gh[0]=b.toString()),b=gh);for(var e=0;e<b.length;e++){var g=v(a,b[e],c||this.handleEvent,d||!1,this.vb||this);if(!g)break;this.Ga[g.key]=g}return this};f.No=function(a,b,c,d){return hh(this,a,b,c,d)};function hh(a,b,c,d,e,g){if(la(c))for(var h=0;h<c.length;h++)hh(a,b,c[h],d,e,g);else{b=cd(b,c,d||a.handleEvent,e,g||a.vb||a);if(!b)return a;a.Ga[b.key]=b}return a}
f.Ba=function(a,b,c,d,e){if(la(b))for(var g=0;g<b.length;g++)this.Ba(a,b[g],c,d,e);else c=c||this.handleEvent,e=e||this.vb||this,c=Xc(c),d=!!d,b=Lc(a)?a.si(b,c,d,e):a?(a=Zc(a))?a.si(b,c,d,e):null:null,b&&(ed(b),delete this.Ga[b.key]);return this};f.sd=function(){Db(this.Ga,function(a,b){this.Ga.hasOwnProperty(b)&&ed(a)},this);this.Ga={}};f.V=function(){fh.m.V.call(this);this.sd()};f.handleEvent=function(){throw Error("EventHandler.handleEvent not implemented");};function ih(){}ka(ih);ih.prototype.vw=0;function jh(a){return":"+(a.vw++).toString(36)};function kh(a){md.call(this);this.S=a||hf();this.Ld=lh;this.Nc=null;this.la=!1;this.F=null;this.Xe=void 0;this.ad=this.jc=this.Zb=this.Sa=null;this.Tt=!1}r(kh,md);kh.prototype.jw=ih.ua();var lh=null;
function mh(a,b){switch(a){case 1:return b?"disable":"enable";case 2:return b?"highlight":"unhighlight";case 4:return b?"activate":"deactivate";case 8:return b?"select":"unselect";case 16:return b?"check":"uncheck";case 32:return b?"focus":"blur";case 64:return b?"open":"close"}throw Error("Invalid component state");}function nh(a){return a.Nc||(a.Nc=jh(a.jw))}function oh(a,b){a.Zb&&a.Zb.ad&&(Mb(a.Zb.ad,a.Nc),Nb(a.Zb.ad,b,a));a.Nc=b}f=kh.prototype;f.o=function(){return this.F};
f.xn=function(a){return this.F?this.S.xn(a,this.F):[]};f.jk=function(a){return this.F?this.S.jk(a,this.F):null};f.Va=function(){this.Xe||(this.Xe=new fh(this));return this.Xe};function ph(a,b){if(a==b)throw Error("Unable to set parent component");if(b&&a.Zb&&a.Nc&&qh(a.Zb,a.Nc)&&a.Zb!=b)throw Error("Unable to set parent component");a.Zb=b;kh.m.nl.call(a,b)}f.getParent=function(){return this.Zb};f.nl=function(a){if(this.Zb&&this.Zb!=a)throw Error("Method not supported");kh.m.nl.call(this,a)};f.I=function(){return this.S};
f.D=function(){this.F=this.S.createElement("DIV")};f.render=function(a){this.Vi(a)};f.qg=function(a){this.Vi(a.parentNode,a)};f.Vi=function(a,b){if(this.la)throw Error("Component already rendered");this.F||this.D();a?a.insertBefore(this.F,b||null):this.S.R.body.appendChild(this.F);this.Zb&&!this.Zb.la||this.Aa()};
f.J=function(a){if(this.la)throw Error("Component already rendered");if(a&&this.rb(a)){this.Tt=!0;var b=kf(a);this.S&&this.S.R==b||(this.S=hf(a));this.Ca(a);this.Aa()}else throw Error("Invalid element to decorate");};f.rb=function(){return!0};f.Ca=function(a){this.F=a};f.Aa=function(){this.la=!0;rh(this,function(a){!a.la&&a.o()&&a.Aa()})};f.lb=function(){rh(this,function(a){a.la&&a.lb()});this.Xe&&this.Xe.sd();this.la=!1};
f.V=function(){this.la&&this.lb();this.Xe&&(this.Xe.ib(),delete this.Xe);rh(this,function(a){a.ib()});!this.Tt&&this.F&&Ff(this.F);this.Zb=this.Sa=this.F=this.ad=this.jc=null;kh.m.V.call(this)};f.$a=function(a,b){this.sm(a,sh(this),b)};
f.sm=function(a,b,c){if(a.la&&(c||!this.la))throw Error("Component already rendered");if(0>b||b>sh(this))throw Error("Child component index out of bounds");this.ad&&this.jc||(this.ad={},this.jc=[]);if(a.getParent()==this){var d=nh(a);this.ad[d]=a;nb(this.jc,a)}else Nb(this.ad,nh(a),a);ph(a,this);rb(this.jc,b,0,a);a.la&&this.la&&a.getParent()==this?(c=this.ub(),b=c.childNodes[b]||null,b!=a.o()&&c.insertBefore(a.o(),b)):c?(this.F||this.D(),b=th(this,b+1),a.Vi(this.ub(),b?b.F:null)):this.la&&!a.la&&
a.F&&a.F.parentNode&&1==a.F.parentNode.nodeType&&a.Aa()};f.ub=function(){return this.F};f.Fk=function(){null==this.Ld&&(this.Ld=vg(this.la?this.F:this.S.R.body));return this.Ld};f.yh=function(a){if(this.la)throw Error("Component already rendered");this.Ld=a};f.jo=function(){return!!this.jc&&0!=this.jc.length};function sh(a){return a.jc?a.jc.length:0}function uh(a){var b=[];rh(a,function(a){b.push(nh(a))});return b}function qh(a,b){return a.ad&&b?Ob(a.ad,b)||null:null}
function th(a,b){return a.jc?a.jc[b]||null:null}function rh(a,b,c){a.jc&&db(a.jc,b,c)}function vh(a,b){return a.jc&&b?cb(a.jc,b):-1}f.removeChild=function(a,b){if(a){var c=ea(a)?a:nh(a);a=qh(this,c);c&&a&&(Mb(this.ad,c),nb(this.jc,a),b&&(a.lb(),a.F&&Ff(a.F)),ph(a,null))}if(!a)throw Error("Child is not in parent component");return a};f.sh=function(a){for(var b=[];this.jo();)b.push(this.removeChild(th(this,0),a));return b};function wh(){this.tag="div";this.Ad=[]}wh.prototype.D=function(a){return a.I().G(this.tag,this.xd(a),a.qe())};wh.prototype.fd=function(a){return this.Ad.concat(a.sr)};wh.prototype.xd=function(a){return{id:nh(a),"class":this.fd(a).join(" ")}};ka(wh);function xh(a){null==a&&(a={});null==a.l&&(a.l=new Cg);this.l=a.l;xh.m.constructor.call(this,a.l);this.K=a.K;this.caption=a.caption;null==this.K&&(this.K=wh.ua());this.sr=[];this.rg=!0;null==this.caption&&(this.caption="")}r(xh,kh);f=xh.prototype;f.addClass=function(a){return this.sr.push(a)};f.qe=function(){return this.caption};f.nt=function(a){this.caption=a};f.D=function(){var a;if(!this.la&&(null!=this.K?this.F=a=this.K.D(this):xh.m.D.call(this),this.rg))return rh(this,function(b){b.D();return a.appendChild(b.o())})};
f.render=function(a){var b=this;null==a&&(a=null);return this.lf(function(c){return null!=a?a.insertBefore(c,null):b.l.R.body.appendChild(c)})};f.qg=function(a){var b=this;return this.lf(function(c){return null!=(null!=a?a.parentNode:void 0)?a.parentNode.insertBefore(c,a):b.l.R.body.appendChild(c)})};f.sp=function(a){var b=this;return this.lf(function(c){return b.l.Ki(c,a)})};f.Vi=function(a,b){var c=this;null==b&&(b=null);return this.lf(function(d){return null!=a?a.insertBefore(d,b):c.l.R.body.appendChild(d)})};
f.lf=function(a){if(this.la)throw Error("Component already rendered");null==this.o()&&this.D();a(this.o());return this.Aa()};f.Aa=function(){xh.m.Aa.call(this);return this.xf()};f.xf=function(){};f.destroy=function(){this.lb();return this.l.removeNode(this.o())};function yh(a,b){this.Ad=a;this.$u=null!=b?b:"×"}r(yh,wh);yh.prototype.D=function(a){return a.I().G("div",{"class":[this.Ad,"btn-icon"].join(" ")},this.$u)};function zh(a){null==a&&(a={});null==a.K&&(a.K=new yh(a.Cf));zh.m.constructor.call(this,a);this.addClass(a.Cf)}r(zh,xh);zh.prototype.xf=function(){var a=this;v(this.o(),"click",function(b){b.preventDefault();return a.dispatchEvent("action")})};zh.prototype.rb=function(a){return null!=a&&dh(a,this.Cf)};zh.prototype.Ca=function(a){this.F=a};function Ah(a){null==a&&(a={});null==a.Cf&&(a.Cf=Bh);Ah.m.constructor.call(this,a);null!=a.Xc&&(this.Xc=a.Xc)}r(Ah,zh);var Bh="goog-icon-remove close";function Ch(){Ch.m.constructor.call(this);this.Ad.push(Dh);this.Ad.push(Eh)}r(Ch,wh);var Fh={info:"alert-info",success:"alert-success",warn:"alert-warning",error:"alert-danger"},Dh="flash-msg",Eh="alert";Ch.prototype.J=function(a){var b;Zg(a,Eh);var c=[];for(b in Fh){var d=Fh[b];dh(a,b)?c.push(Zg(a,d)):c.push(void 0)}return c};Ch.prototype.D=function(a){var b=a.I();var c=Ch.m.D.call(this,a);var d=b.G("span","msg-text",c);b.Yc(d,a.Rk);return c};Ch.prototype.mark=function(a,b){return Zg(a,Fh[b])};ka(Ch);function Gh(a){null==a.K&&(a.K=Ch.ua());Gh.m.constructor.call(this,a);this.Rk=a.Rk;this.severity=a.severity;this.ek=a.ek;this.Og=a.Og;this.timeout=a.timeout;null==this.Gv&&(this.Gv=!0);null==this.Og&&(this.Og=!0);null==this.timeout&&(this.timeout=1E4)}r(Gh,xh);Gh.prototype.D=function(){Gh.m.D.call(this);this.Og&&Hh(this,this.o());this.ek&&Ih(this,this.o());var a=this.o();a=null!=this.severity?this.K.mark(a,this.severity):void 0;return a};Gh.prototype.rb=function(a){return null!=a&&dh(a,Dh)};
function Hh(a,b){var c=new Ah({l:a.l});c.A("action",function(){return a.destroy()});c.D();b.appendChild(c.o());c.Aa()}Gh.prototype.Ca=function(a){Gh.m.Ca.call(this,a);this.K.J(a);dh(a,"error")||dh(a,Fh["alert-danger"])||Hh(this,a);if(!dh(a,"error")&&!dh(a,"ho-hide"))return Ih(this,a)};function Ih(a,b){return setTimeout(function(){var c=new Wg(b,1E3);c.A("end",function(){return a.destroy()});return c.play()},a.timeout)};function Jh(){Jh.m.constructor.call(this);this.Ad.push(Kh)}r(Jh,wh);var Kh="flash";ka(Jh);function Lh(a){null==a.K&&(a.K=Jh.ua());this.Mp=[];Lh.m.constructor.call(this,a)}r(Lh,xh);f=Lh.prototype;f.kx=function(a,b,c){null==b&&(b=void 0);null==c&&(c=!0);return this.Lh(a,"success",b,c)};f.Lh=function(a,b,c,d,e){var g;null==b&&(b="info");null==c&&(c=void 0);null==d&&(d=!0);null==e&&(e=1E4);void 0===c&&(c="error"!==b);var h=la(a)?a:[a];a=[];var k=0;for(g=h.length;k<g;k++){var l=h[k];l=new Gh({l:this.l,Rk:l,severity:b,ek:c,Og:d,timeout:e});this.Mp.push(l);a.push(l);this.$a(l)}return a};
f.$a=function(a){return a.render(this.o())};function Mh(a,b){return a.Lh(b,"error")}f.rb=function(a){return null!=a&&dh(a,Kh)};f.Ca=function(a){var b;Lh.m.Ca.call(this,a);var c=this.l.se(a);var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];a=new Gh({l:this.l});d.push(a.J(g))}return d};f.D=function(){Lh.m.D.call(this);return this.Ca(this.o())};f.Ij=function(){var a;var b=this.Mp;var c=0;for(a=b.length;c<a;c++){var d=b[c];d.destroy()}return this.Mp=[]};Cb("Firefox");var Nh=bc()||Cb("iPod"),Oh=Cb("iPad");!Cb("Android")||Tb()||Cb("Firefox")||Cb("Opera")||Cb("Silk");Tb();Sb()&&cc();var Ph=!t&&!Sb();function Qh(a,b,c){Ph&&a.dataset?a.dataset[b]=c:a.setAttribute("data-"+Ya(b),c)}function Rh(a,b){return Ph&&a.dataset?b in a.dataset?a.dataset[b]:null:a.getAttribute("data-"+Ya(b))}function Sh(a,b){Ph&&a.dataset?Th(a,b)&&delete a.dataset[b]:a.removeAttribute("data-"+Ya(b))}function Th(a,b){return Ph&&a.dataset?b in a.dataset:a.hasAttribute?a.hasAttribute("data-"+Ya(b)):!!a.getAttribute("data-"+Ya(b))};function Uh(a,b,c,d,e,g){if(!(t||hc||kc&&uc("525")))return!0;if(lc&&e)return Vh(a);if(e&&!d)return!1;fa(b)&&(b=Wh(b));e=17==b||18==b||lc&&91==b;if((!c||lc)&&e||lc&&16==b&&(d||g))return!1;if((kc||hc)&&d&&c)switch(a){case 220:case 219:case 221:case 192:case 186:case 189:case 187:case 188:case 190:case 191:case 192:case 222:return!1}if(t&&d&&b==a)return!1;switch(a){case 13:return!0;case 27:return!(kc||hc)}return Vh(a)}
function Vh(a){if(48<=a&&57>=a||96<=a&&106>=a||65<=a&&90>=a||(kc||hc)&&0==a)return!0;switch(a){case 32:case 43:case 63:case 64:case 107:case 109:case 110:case 111:case 186:case 59:case 189:case 187:case 61:case 188:case 190:case 191:case 192:case 222:case 219:case 220:case 221:return!0;default:return!1}}function Wh(a){if(jc)a=Xh(a);else if(lc&&kc)switch(a){case 93:a=91}return a}
function Xh(a){switch(a){case 61:return 187;case 59:return 186;case 173:return 189;case 224:return 91;case 0:return 224;default:return a}};function Yh(a,b){md.call(this);a&&this.zf(a,b)}r(Yh,md);f=Yh.prototype;f.F=null;f.Hk=null;f.Fo=null;f.Ik=null;f.Oc=-1;f.df=-1;f.um=!1;
var Zh={3:13,12:144,63232:38,63233:40,63234:37,63235:39,63236:112,63237:113,63238:114,63239:115,63240:116,63241:117,63242:118,63243:119,63244:120,63245:121,63246:122,63247:123,63248:44,63272:46,63273:36,63275:35,63276:33,63277:34,63289:144,63302:45},ai={Up:38,Down:40,Left:37,Right:39,Enter:13,F1:112,F2:113,F3:114,F4:115,F5:116,F6:117,F7:118,F8:119,F9:120,F10:121,F11:122,F12:123,"U+007F":46,Home:36,End:35,PageUp:33,PageDown:34,Insert:45},bi=t||hc||kc&&uc("525"),ci=lc&&jc;f=Yh.prototype;
f.Sv=function(a){if(kc||hc)if(17==this.Oc&&!a.ctrlKey||18==this.Oc&&!a.altKey||lc&&91==this.Oc&&!a.metaKey)this.df=this.Oc=-1;-1==this.Oc&&(a.ctrlKey&&17!=a.keyCode?this.Oc=17:a.altKey&&18!=a.keyCode?this.Oc=18:a.metaKey&&91!=a.keyCode&&(this.Oc=91));bi&&!Uh(a.keyCode,this.Oc,a.shiftKey,a.ctrlKey,a.altKey,a.metaKey)?this.handleEvent(a):(this.df=Wh(a.keyCode),ci&&(this.um=a.altKey))};f.Uv=function(a){this.df=this.Oc=-1;this.um=a.altKey};
f.handleEvent=function(a){var b=a.bd,c=b.altKey;if(t&&"keypress"==a.type){var d=this.df;var e=13!=d&&27!=d?b.keyCode:0}else(kc||hc)&&"keypress"==a.type?(d=this.df,e=0<=b.charCode&&63232>b.charCode&&Vh(d)?b.charCode:0):gc&&!kc?(d=this.df,e=Vh(d)?b.keyCode:0):(d=b.keyCode||this.df,e=b.charCode||0,ci&&(c=this.um),lc&&63==e&&224==d&&(d=191));var g=d=Wh(d);d?63232<=d&&d in Zh?g=Zh[d]:25==d&&a.shiftKey&&(g=9):b.keyIdentifier&&b.keyIdentifier in ai&&(g=ai[b.keyIdentifier]);a=g==this.Oc;this.Oc=g;b=new di(g,
e,a,b);b.altKey=c;this.dispatchEvent(b)};f.o=function(){return this.F};f.zf=function(a,b){this.Ik&&this.detach();this.F=a;this.Hk=v(this.F,"keypress",this,b);this.Fo=v(this.F,"keydown",this.Sv,b,this);this.Ik=v(this.F,"keyup",this.Uv,b,this)};f.detach=function(){this.Hk&&(ed(this.Hk),ed(this.Fo),ed(this.Ik),this.Ik=this.Fo=this.Hk=null);this.F=null;this.df=this.Oc=-1};f.V=function(){Yh.m.V.call(this);this.detach()};
function di(a,b,c,d){Gc.call(this,d);this.type="key";this.keyCode=a;this.charCode=b;this.repeat=c}r(di,Gc);var ei;Rb("A AREA BUTTON HEAD INPUT LINK MENU META OPTGROUP OPTION PROGRESS STYLE SELECT SOURCE TEXTAREA TITLE TRACK".split(" "));function fi(a,b){b?a.setAttribute("role",b):a.removeAttribute("role")}
function gi(a,b,c){la(c)&&(c=c.join(" "));var d="aria-"+b;""===c||void 0==c?(ei||(ei={atomic:!1,autocomplete:"none",dropeffect:"none",haspopup:!1,live:"off",multiline:!1,multiselectable:!1,orientation:"vertical",readonly:!1,relevant:"additions text",required:!1,sort:"none",busy:!1,disabled:!1,hidden:!1,invalid:"false"}),c=ei,b in c?a.setAttribute(d,c[b]):a.removeAttribute(d)):a.setAttribute(d,c)}function hi(a,b){var c="";b&&(c=b.id);gi(a,"activedescendant",c)};function ii(a){if(a.classList)return a.classList;a=a.className;return ea(a)&&a.match(/\S+/g)||[]}function ji(a,b){return a.classList?a.classList.contains(b):mb(ii(a),b)}function ki(a,b){a.classList?a.classList.add(b):ji(a,b)||(a.className+=0<a.className.length?" "+b:b)}function li(a,b){if(a.classList)db(b,function(b){ki(a,b)});else{var c={};db(ii(a),function(a){c[a]=!0});db(b,function(a){c[a]=!0});a.className="";for(var d in c)a.className+=0<a.className.length?" "+d:d}}
function mi(a,b){a.classList?a.classList.remove(b):ji(a,b)&&(a.className=eb(ii(a),function(a){return a!=b}).join(" "))}function ni(a,b){a.classList?db(b,function(b){mi(a,b)}):a.className=eb(ii(a),function(a){return!mb(b,a)}).join(" ")};function oi(){}var pi;ka(oi);var qi={button:"pressed",checkbox:"checked",menuitem:"selected",menuitemcheckbox:"checked",menuitemradio:"checked",radio:"checked",tab:"selected",treeitem:"selected"};f=oi.prototype;f.Qf=function(){};f.D=function(a){return a.I().D("DIV",this.fd(a).join(" "),a.Fd())};f.ub=function(a){return a};f.ji=function(a,b,c){if(a=a.o?a.o():a){var d=[b];t&&!uc("7")&&(d=ri(ii(a),b),d.push(b));(c?li:ni)(a,d)}};f.rb=function(){return!0};
f.J=function(a,b){b.id&&oh(a,b.id);var c=this.ub(b);c&&c.firstChild?si(a,c.firstChild.nextSibling?qb(c.childNodes):c.firstChild):a.ab=null;var d=0,e=this.ta(),g=this.ta(),h=!1,k=!1,l=!1,m=qb(ii(b));db(m,function(a){h||a!=e?k||a!=g?d|=this.nk(a):k=!0:(h=!0,g==e&&(k=!0));1==this.nk(a)&&Nf(c)&&Qf(c,!1)},this);a.ma=d;h||(m.push(e),g==e&&(k=!0));k||m.push(g);var p=a.cd;p&&m.push.apply(m,p);if(t&&!uc("7")){var u=ri(m);0<u.length&&(m.push.apply(m,u),l=!0)}if(!h||!k||p||l)b.className=m.join(" ");return b};
f.dg=function(a){a.Fk()&&this.yh(a.o(),!0);a.isEnabled()&&this.He(a,a.va())};function ti(a,b,c){if(a=c||a.Qf())c=b.getAttribute("role")||null,a!=c&&fi(b,a)}function ui(a,b,c){var d=b.Pq;null!=d&&a.Ap(c,d);b.va()||gi(c,"hidden",!b.va());b.isEnabled()||a.Od(c,1,!b.isEnabled());vi(b,8)&&a.Od(c,8,b.dh());vi(b,16)&&a.Od(c,16,!!(b.ma&16));vi(b,64)&&a.Od(c,64,b.cf())}f.Ap=function(a,b){gi(a,"label",b)};f.vh=function(a,b){xg(a,!b,!t&&!gc)};f.yh=function(a,b){this.ji(a,this.ta()+"-rtl",b)};
f.ye=function(a){var b;return vi(a,32)&&(b=a.Hb())?Nf(b):!1};f.He=function(a,b){var c;if(vi(a,32)&&(c=a.Hb())){if(!b&&a.ma&32){try{c.blur()}catch(d){}a.ma&32&&a.handleBlur(null)}Nf(c)!=b&&Qf(c,b)}};f.$=function(a,b){w(a,b);a&&gi(a,"hidden",!b)};f.setState=function(a,b,c){var d=a.o();if(d){var e=this.ri(b);e&&this.ji(a,e,c);this.Od(d,b,c)}};
f.Od=function(a,b,c){pi||(pi={1:"disabled",8:"selected",16:"checked",64:"expanded"});b=pi[b];var d=a.getAttribute("role")||null;d&&(d=qi[d]||b,b="checked"==b||"selected"==b?d:b);b&&gi(a,b,c)};f.oc=function(a,b){var c=this.ub(a);if(c&&(Cf(c),b))if(ea(b))Kf(c,b);else{var d=function(a){if(a){var b=kf(c);c.appendChild(ea(a)?b.createTextNode(a):a)}};la(b)?db(b,d):!ma(b)||"nodeType"in b?d(b):db(qb(b),d)}};f.Hb=function(a){return a.o()};f.ta=function(){return"goog-control"};
f.fd=function(a){var b=this.ta(),c=[b],d=this.ta();d!=b&&c.push(d);b=a.getState();for(d=[];b;){var e=b&-b;d.push(this.ri(e));b&=~e}c.push.apply(c,d);(a=a.cd)&&c.push.apply(c,a);t&&!uc("7")&&c.push.apply(c,ri(c));return c};function ri(a,b){var c=[];b&&(a=pb(a,[b]));db([],function(d){!jb(d,va(mb,a))||b&&!mb(d,b)||c.push(d.join("_"))});return c}f.ri=function(a){this.Gj||wi(this);return this.Gj[a]};
f.nk=function(a){if(!this.Ht){this.Gj||wi(this);var b=this.Gj,c={},d;for(d in b)c[b[d]]=d;this.Ht=c}a=parseInt(this.Ht[a],10);return isNaN(a)?0:a};function wi(a){var b=a.ta();Pa(b.replace(/\xa0|\s/g," ")," ");a.Gj={1:b+"-disabled",2:b+"-hover",4:b+"-active",8:b+"-selected",16:b+"-checked",32:b+"-focused",64:b+"-open"}};function xi(a,b){if(!a)throw Error("Invalid class name "+a);if(!na(b))throw Error("Invalid decorator function "+b);yi[a]=b}var zi={},yi={};function Ai(a,b,c){kh.call(this,c);if(!b){b=this.constructor;for(var d;b;){d=pa(b);if(d=zi[d])break;b=b.m?b.m.constructor:null}b=d?na(d.ua)?d.ua():new d:null}this.W=b;this.ab=da(a)?a:null;this.Pq=null}r(Ai,kh);f=Ai.prototype;f.ab=null;f.ma=0;f.jj=39;f.Ph=255;f.hj=0;f.Oa=!0;f.cd=null;f.Wn=!0;f.Mq=!1;f.kp=null;function Bi(a,b){a.la&&b!=a.Wn&&Ci(a,b);a.Wn=b}f.Hb=function(){return this.W.Hb(this)};f.kk=function(){return this.Yb||(this.Yb=new Yh)};
function Di(a,b){b&&(a.cd?mb(a.cd,b)||a.cd.push(b):a.cd=[b],a.W.ji(a,b,!0))}f.ji=function(a,b){b?Di(this,a):a&&this.cd&&nb(this.cd,a)&&(0==this.cd.length&&(this.cd=null),this.W.ji(this,a,!1))};f.D=function(){var a=this.W.D(this);this.F=a;ti(this.W,a,this.ue());this.Ck()||this.W.vh(a,!1);this.va()||this.W.$(a,!1)};f.ue=function(){return this.kp};f.Ap=function(a){this.Pq=a;var b=this.o();b&&this.W.Ap(b,a)};f.ub=function(){return this.W.ub(this.o())};f.rb=function(a){return this.W.rb(a)};
f.Ca=function(a){this.F=a=this.W.J(this,a);ti(this.W,a,this.ue());this.Ck()||this.W.vh(a,!1);this.Oa="none"!=a.style.display};f.Aa=function(){Ai.m.Aa.call(this);ui(this.W,this,this.F);this.W.dg(this);if(this.jj&-2&&(this.Wn&&Ci(this,!0),vi(this,32))){var a=this.Hb();if(a){var b=this.kk();b.zf(a);this.Va().A(b,"key",this.gd).A(a,"focus",this.handleFocus).A(a,"blur",this.handleBlur)}}};
function Ci(a,b){var c=a.Va(),d=a.o();b?(c.A(d,"mouseover",a.Wg).A(d,"mousedown",a.xe).A(d,"mouseup",a.Xg).A(d,"mouseout",a.Xn),a.wi!=ja&&c.A(d,"contextmenu",a.wi),t&&(uc(9)||c.A(d,"dblclick",a.Lr),a.Fi||(a.Fi=new Ei(a),$b(a,va(ac,a.Fi))))):(c.Ba(d,"mouseover",a.Wg).Ba(d,"mousedown",a.xe).Ba(d,"mouseup",a.Xg).Ba(d,"mouseout",a.Xn),a.wi!=ja&&c.Ba(d,"contextmenu",a.wi),t&&(uc(9)||c.Ba(d,"dblclick",a.Lr),ac(a.Fi),a.Fi=null))}
f.lb=function(){Ai.m.lb.call(this);this.Yb&&this.Yb.detach();this.va()&&this.isEnabled()&&this.W.He(this,!1)};f.V=function(){Ai.m.V.call(this);this.Yb&&(this.Yb.ib(),delete this.Yb);delete this.W;this.Fi=this.cd=this.ab=null};f.Fd=function(){return this.ab};f.oc=function(a){this.W.oc(this.o(),a);this.ab=a};function si(a,b){a.ab=b}f.qe=function(){var a=this.Fd();if(!a)return"";a=ea(a)?a:la(a)?fb(a,Tf).join(""):Rf(a);return Ca(a)};f.nt=function(a){this.oc(a)};
f.yh=function(a){Ai.m.yh.call(this,a);var b=this.o();b&&this.W.yh(b,a)};f.Ck=function(){return this.Mq};f.vh=function(a){this.Mq=a;var b=this.o();b&&this.W.vh(b,a)};f.va=function(){return this.Oa};f.$=function(a,b){if(b||this.Oa!=a&&this.dispatchEvent(a?"show":"hide")){var c=this.o();c&&this.W.$(c,a);this.isEnabled()&&this.W.He(this,a);this.Oa=a;return!0}return!1};f.isEnabled=function(){return!(this.ma&1)};
f.pa=function(a){var b=this.getParent();b&&"function"==typeof b.isEnabled&&!b.isEnabled()||!Fi(this,1,!a)||(a||(this.setActive(!1),this.ud(!1)),this.va()&&this.W.He(this,a),this.setState(1,!a,!0))};f.ud=function(a){Fi(this,2,a)&&this.setState(2,a)};f.fg=function(){return!!(this.ma&4)};f.setActive=function(a){Fi(this,4,a)&&this.setState(4,a)};f.dh=function(){return!!(this.ma&8)};f.setSelected=function(a){Fi(this,8,a)&&this.setState(8,a)};f.cf=function(){return!!(this.ma&64)};
f.setOpen=function(a){Fi(this,64,a)&&this.setState(64,a)};f.getState=function(){return this.ma};f.setState=function(a,b,c){c||1!=a?vi(this,a)&&b!=!!(this.ma&a)&&(this.W.setState(this,a,b),this.ma=b?this.ma|a:this.ma&~a):this.pa(!b)};function vi(a,b){return!!(a.jj&b)}f.pc=function(a,b){if(this.la&&this.ma&a&&!b)throw Error("Component already rendered");!b&&this.ma&a&&this.setState(a,!1);this.jj=b?this.jj|a:this.jj&~a};function Gi(a,b){return!!(a.Ph&b)&&vi(a,b)}
function Fi(a,b,c){return vi(a,b)&&!!(a.ma&b)!=c&&(!(a.hj&b)||a.dispatchEvent(mh(b,c)))&&!a.Re}f.Wg=function(a){!Hi(a,this.o())&&this.dispatchEvent("enter")&&this.isEnabled()&&Gi(this,2)&&this.ud(!0)};f.Xn=function(a){!Hi(a,this.o())&&this.dispatchEvent("leave")&&(Gi(this,4)&&this.setActive(!1),Gi(this,2)&&this.ud(!1))};f.wi=ja;function Hi(a,b){return!!a.relatedTarget&&If(b,a.relatedTarget)}
f.xe=function(a){this.isEnabled()&&(Gi(this,2)&&this.ud(!0),Jc(a)&&(Gi(this,4)&&this.setActive(!0),this.W&&this.W.ye(this)&&this.Hb().focus()));!this.Ck()&&Jc(a)&&a.preventDefault()};f.Xg=function(a){this.isEnabled()&&(Gi(this,2)&&this.ud(!0),this.fg()&&this.nh(a)&&Gi(this,4)&&this.setActive(!1))};f.Lr=function(a){this.isEnabled()&&this.nh(a)};
f.nh=function(a){if(Gi(this,16)){var b=!(this.ma&16);Fi(this,16,b)&&this.setState(16,b)}Gi(this,8)&&this.setSelected(!0);Gi(this,64)&&this.setOpen(!this.cf());b=new Cc("action",this);a&&(b.altKey=a.altKey,b.ctrlKey=a.ctrlKey,b.metaKey=a.metaKey,b.shiftKey=a.shiftKey,b.hp=a.hp);return this.dispatchEvent(b)};f.handleFocus=function(){Gi(this,32)&&Fi(this,32,!0)&&this.setState(32,!0)};f.handleBlur=function(){Gi(this,4)&&this.setActive(!1);Gi(this,32)&&Fi(this,32,!1)&&this.setState(32,!1)};
f.gd=function(a){return this.va()&&this.isEnabled()&&this.we(a)?(a.preventDefault(),a.stopPropagation(),!0):!1};f.we=function(a){return 13==a.keyCode&&this.nh(a)};if(!na(Ai))throw Error("Invalid component class "+Ai);if(!na(oi))throw Error("Invalid renderer class "+oi);var Ii=pa(Ai);zi[Ii]=oi;xi("goog-control",function(){return new Ai(null)});
function Ei(a){Vb.call(this);this.Uj=a;this.Jj=!1;this.vb=new fh(this);$b(this,va(ac,this.vb));a=this.Uj.F;this.vb.A(a,"mousedown",this.xi).A(a,"mouseup",this.Xv).A(a,"click",this.vi)}r(Ei,Vb);var Ji=!t||vc(9);Ei.prototype.xi=function(){this.Jj=!1};Ei.prototype.Xv=function(){this.Jj=!0};
function Ki(a,b){if(!Ji)return a.button=0,a.type=b,a;var c=document.createEvent("MouseEvents");c.initMouseEvent(b,a.bubbles,a.cancelable,a.view||null,a.detail,a.screenX,a.screenY,a.clientX,a.clientY,a.ctrlKey,a.altKey,a.shiftKey,a.metaKey,0,a.relatedTarget||null);return c}Ei.prototype.vi=function(a){if(this.Jj)this.Jj=!1;else{var b=a.bd,c=b.button,d=b.type,e=Ki(b,"mousedown");this.Uj.xe(new Gc(e,a.currentTarget));e=Ki(b,"mouseup");this.Uj.Xg(new Gc(e,a.currentTarget));Ji||(b.button=c,b.type=d)}};
Ei.prototype.V=function(){this.Uj=null;Ei.m.V.call(this)};function Li(){Li.m.constructor.call(this)}r(Li,oi);Li.prototype.J=function(a,b){Li.m.J.call(this,a,b);a.oc(b.value);return b};Li.prototype.rb=function(a){return"INPUT"===a.tagName};Li.prototype.D=function(a){return a.I().G("input",{type:"text","class":this.fd(a).join(" "),placeholder:a.placeholder})};ka(Li);function Mi(a,b,c){null==b&&(b=Li.ua());null==c&&(c=null);this.Ze=q(this.Ze,this);Mi.m.constructor.call(this,a,b,c);Bi(this,!1);this.vh(!0);null==a&&this.oc("");this.pj=null;this.placeholder=""}r(Mi,Ai);function Ni(a,b){a.placeholder=b;null==a.placeholder&&(a.placeholder="");a.o()&&a.o().setAttribute("placeholder",String(a.placeholder))}f=Mi.prototype;f.$=function(a,b){null==b&&(b=!1);this.pj&&w(this.pj,a);return Mi.m.$.call(this,a,b)};
f.qa=function(a){a=String(a);this.oc(a);return this.o().value=a};f.oc=function(a){this.ab=a;this.dispatchEvent(new Cc("VALUE_CHANGE",this))};f.ba=function(){return this.o().value};f.clear=function(){return this.qa("")};f.qg=function(a){Mi.m.qg.call(this,a);return this.ni()};f.render=function(a){Mi.m.render.call(this,a);this.ni();v(this.o(),"keyup",this.Ze)};f.Ze=function(a){var b;return null!=(null!=a?null!=(b=a.target)?b.value:void 0:void 0)?this.oc(a.target.value):this.oc(this.o().value)};
f.ni=function(){this.Ek(this.o())||(this.pj=this.o(),this.F=this.Ek(this.pj)?this.pj:this.S.se(this.o())[0])};f.Ek=function(a){return"INPUT"===a.tagName};f.Ok=function(a){null==a&&(a=!0);return this.o().required=a?"required":void 0};f.destroy=function(){this.lb();return this.l.removeNode(this.o())};function Oi(a,b){this.type=a;return this.currentTarget=this.target=b||null}f=Oi.prototype;f.qd=!1;f.defaultPrevented=!1;f.Xi=!0;f.stopPropagation=function(){this.qd=!0};f.preventDefault=function(){this.defaultPrevented=!0;this.Xi=!1};function Pi(a){this.l=a;this.Vc=q(this.Vc,this);Pi.m.constructor.call(this,null,null,this.l);this.he=new Ah({l:this.l});this.he.A("action",this.Vc)}r(Pi,Mi);Pi.prototype.render=function(a){this.je=a;Pi.m.render.call(this,this.je);this.he.render(this.je);w(this.he.o(),!1);this.F=this.o()};
Pi.prototype.Vc=function(){this.qa("");w(this.he.o(),!1);var a=new Oi("change",this.o()),b;a.Xi=!0;var c=[];for(b=a.target;b;)c.push(b),b=b.parentNode;for(b=c.length-1;0<=b&&!a.qd;)gd(c[b],a.type,!0,new Gc(a,c[b])),b--;for(b=0;b<c.length&&!a.qd;)gd(c[b],a.type,!1,new Gc(a,c[b])),b++;return this.dispatchEvent(new Cc("clear",this))};function Qi(a){if(!a.o().getAttribute("readonly"))return w(a.he.o(),!0)}
Pi.prototype.Ze=function(a){Pi.m.Ze.call(this,a);if(this.ba())return Qi(this);this.dispatchEvent(new Cc("clear",this));return w(this.he.o(),!1)};var Ri={Wl:["př. n. l.","n. l."],rq:["př. n. l.","n. l."],vq:"1 2 3 4 5 6 7 8 9 10 11 12".split(" "),Aq:"1 2 3 4 5 6 7 8 9 10 11 12".split(" "),Zl:"ledna února března dubna května června července srpna září října listopadu prosince".split(" "),Eg:"leden únor březen duben květen červen červenec srpen září říjen listopad prosinec".split(" "),hm:"led úno bře dub kvě čvn čvc srp zář říj lis pro".split(" "),km:"led úno bře dub kvě čvn čvc srp zář říj lis pro".split(" "),mm:"neděle pondělí úterý středa čtvrtek pátek sobota".split(" "),
Cq:"neděle pondělí úterý středa čtvrtek pátek sobota".split(" "),jm:"ne po út st čt pá so".split(" "),lm:"ne po út st čt pá so".split(" "),pu:"NPÚSČPS".split(""),Bq:"NPÚSČPS".split(""),im:["Q1","Q2","Q3","Q4"],cm:["1. čtvrtletí","2. čtvrtletí","3. čtvrtletí","4. čtvrtletí"],Ol:["dop.","odp."],Cg:["EEEE d. MMMM y","d. MMMM y","d. M. y","dd.MM.yy"],Jh:["H:mm:ss zzzz","H:mm:ss z","H:mm:ss","H:mm"],Vl:["{1} {0}","{1} {0}","{1} {0}","{1} {0}"],rj:0,nm:[5,6],sj:3},Si={Wl:["BC","AD"],rq:["Before Christ",
"Anno Domini"],vq:"JFMAMJJASOND".split(""),Aq:"JFMAMJJASOND".split(""),Zl:"January February March April May June July August September October November December".split(" "),Eg:"January February March April May June July August September October November December".split(" "),hm:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" "),km:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" "),mm:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),Cq:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),
jm:"Sun Mon Tue Wed Thu Fri Sat".split(" "),lm:"Sun Mon Tue Wed Thu Fri Sat".split(" "),pu:"SMTWTFS".split(""),Bq:"SMTWTFS".split(""),im:["Q1","Q2","Q3","Q4"],cm:["1st quarter","2nd quarter","3rd quarter","4th quarter"],Ol:["AM","PM"],Cg:["EEEE, MMMM d, y","MMMM d, y","MMM d, y","M/d/yy"],Jh:["h:mm:ss a zzzz","h:mm:ss a z","h:mm:ss a","h:mm a"],Vl:["{1} 'at' {0}","{1} 'at' {0}","{1}, {0}","{1}, {0}"],rj:6,nm:[5,6],sj:5},Ti=Si,Ti=Si;function Ui(a,b){switch(b){case 1:return 0!=a%4||0==a%100&&0!=a%400?28:29;case 5:case 8:case 10:case 3:return 30}return 31}function Vi(a,b,c,d,e){a=new Date(a,b,c);d=da(d)?d:3;e=e||0;b=((a.getDay()+6)%7-e+7)%7;e=a.valueOf()+864E5*((d-e+7)%7-b);d=(new Date((new Date(e)).getFullYear(),0,1)).valueOf();return Math.floor(Math.round((e-d)/864E5)/7)+1}
function Wi(a,b,c,d,e,g){ea(a)?(this.years=a==Xi?b:0,this.months=a==Yi?b:0,this.days=a==Zi?b:0,this.hours=a==$i?b:0,this.minutes=a==aj?b:0,this.seconds=a==bj?b:0):(this.years=a||0,this.months=b||0,this.days=c||0,this.hours=d||0,this.minutes=e||0,this.seconds=g||0)}
Wi.prototype.toIsoString=function(a){var b=Math.min(this.years,this.months,this.days,this.hours,this.minutes,this.seconds),c=Math.max(this.years,this.months,this.days,this.hours,this.minutes,this.seconds);if(0>b&&0<c)return null;if(!a&&0==b&&0==c)return"PT0S";c=[];0>b&&c.push("-");c.push("P");(this.years||a)&&c.push(Math.abs(this.years)+"Y");(this.months||a)&&c.push(Math.abs(this.months)+"M");(this.days||a)&&c.push(Math.abs(this.days)+"D");if(this.hours||this.minutes||this.seconds||a)c.push("T"),
(this.hours||a)&&c.push(Math.abs(this.hours)+"H"),(this.minutes||a)&&c.push(Math.abs(this.minutes)+"M"),(this.seconds||a)&&c.push(Math.abs(this.seconds)+"S");return c.join("")};Wi.prototype.tb=function(a){return a.years==this.years&&a.months==this.months&&a.days==this.days&&a.hours==this.hours&&a.minutes==this.minutes&&a.seconds==this.seconds};Wi.prototype.clone=function(){return new Wi(this.years,this.months,this.days,this.hours,this.minutes,this.seconds)};
var Xi="y",Yi="m",Zi="d",$i="h",aj="n",bj="s";Wi.prototype.Xb=function(){return 0==this.years&&0==this.months&&0==this.days&&0==this.hours&&0==this.minutes&&0==this.seconds};Wi.prototype.add=function(a){this.years+=a.years;this.months+=a.months;this.days+=a.days;this.hours+=a.hours;this.minutes+=a.minutes;this.seconds+=a.seconds};
function cj(a,b,c){fa(a)?(this.date=dj(a,b||0,c||1),ej(this,c||1)):oa(a)?(this.date=dj(a.getFullYear(),a.getMonth(),a.getDate()),ej(this,a.getDate())):(this.date=new Date(wa()),a=this.date.getDate(),this.date.setHours(0),this.date.setMinutes(0),this.date.setSeconds(0),this.date.setMilliseconds(0),ej(this,a))}function dj(a,b,c){b=new Date(a,b,c);0<=a&&100>a&&b.setFullYear(b.getFullYear()-1900);return b}f=cj.prototype;f.ed=Ti.rj;f.Pf=Ti.sj;
f.clone=function(){var a=new cj(this.date);a.ed=this.ed;a.Pf=this.Pf;return a};f.getFullYear=function(){return this.date.getFullYear()};f.getYear=function(){return this.getFullYear()};f.getMonth=function(){return this.date.getMonth()};f.getDate=function(){return this.date.getDate()};f.getTime=function(){return this.date.getTime()};f.getDay=function(){return this.date.getDay()};f.getUTCFullYear=function(){return this.date.getUTCFullYear()};f.getUTCMonth=function(){return this.date.getUTCMonth()};
f.getUTCDate=function(){return this.date.getUTCDate()};f.getUTCDay=function(){return this.date.getDay()};f.getUTCHours=function(){return this.date.getUTCHours()};f.getUTCMinutes=function(){return this.date.getUTCMinutes()};function gj(a){return Ui(a.getFullYear(),a.getMonth())}f.getTimezoneOffset=function(){return this.date.getTimezoneOffset()};function hj(a){a=a.getTimezoneOffset();if(0==a)a="Z";else{var b=Math.abs(a)/60,c=Math.floor(b),b=60*(b-c);a=(0<a?"-":"+")+Va(c,2)+":"+Va(b,2)}return a}
f.set=function(a){this.date=new Date(a.getFullYear(),a.getMonth(),a.getDate())};f.setFullYear=function(a){this.date.setFullYear(a)};f.setYear=function(a){this.setFullYear(a)};f.setMonth=function(a){this.date.setMonth(a)};f.setDate=function(a){this.date.setDate(a)};f.setTime=function(a){this.date.setTime(a)};f.setUTCFullYear=function(a){this.date.setUTCFullYear(a)};f.setUTCMonth=function(a){this.date.setUTCMonth(a)};f.setUTCDate=function(a){this.date.setUTCDate(a)};
f.add=function(a){if(a.years||a.months){var b=this.getMonth()+a.months+12*a.years,c=this.getYear()+Math.floor(b/12),b=b%12;0>b&&(b+=12);var d=Math.min(Ui(c,b),this.getDate());this.setDate(1);this.setFullYear(c);this.setMonth(b);this.setDate(d)}a.days&&(b=new Date(this.getYear(),this.getMonth(),this.getDate(),12),a=new Date(b.getTime()+864E5*a.days),this.setDate(1),this.setFullYear(a.getFullYear()),this.setMonth(a.getMonth()),this.setDate(a.getDate()),ej(this,a.getDate()))};
f.toIsoString=function(a,b){return[this.getFullYear(),Va(this.getMonth()+1,2),Va(this.getDate(),2)].join(a?"-":"")+(b?hj(this):"")};f.tb=function(a){return!(!a||this.getYear()!=a.getYear()||this.getMonth()!=a.getMonth()||this.getDate()!=a.getDate())};f.toString=function(){return this.toIsoString()};function ej(a,b){if(a.getDate()!=b){var c=a.getDate()<b?1:-1;a.date.setUTCHours(a.date.getUTCHours()+c)}}f.valueOf=function(){return this.date.valueOf()};
function ij(a,b,c,d,e,g,h){this.date=fa(a)?new Date(a,b||0,c||1,d||0,e||0,g||0,h||0):new Date(a&&a.getTime?a.getTime():wa())}r(ij,cj);f=ij.prototype;f.getHours=function(){return this.date.getHours()};f.getMinutes=function(){return this.date.getMinutes()};f.getSeconds=function(){return this.date.getSeconds()};f.getMilliseconds=function(){return this.date.getMilliseconds()};f.getUTCDay=function(){return this.date.getUTCDay()};f.getUTCHours=function(){return this.date.getUTCHours()};
f.getUTCMinutes=function(){return this.date.getUTCMinutes()};f.getUTCSeconds=function(){return this.date.getUTCSeconds()};f.getUTCMilliseconds=function(){return this.date.getUTCMilliseconds()};f.setHours=function(a){this.date.setHours(a)};f.setMinutes=function(a){this.date.setMinutes(a)};f.setSeconds=function(a){this.date.setSeconds(a)};f.setMilliseconds=function(a){this.date.setMilliseconds(a)};f.setUTCHours=function(a){this.date.setUTCHours(a)};f.setUTCMinutes=function(a){this.date.setUTCMinutes(a)};
f.setUTCSeconds=function(a){this.date.setUTCSeconds(a)};f.setUTCMilliseconds=function(a){this.date.setUTCMilliseconds(a)};f.add=function(a){cj.prototype.add.call(this,a);a.hours&&this.setUTCHours(this.date.getUTCHours()+a.hours);a.minutes&&this.setUTCMinutes(this.date.getUTCMinutes()+a.minutes);a.seconds&&this.setUTCSeconds(this.date.getUTCSeconds()+a.seconds)};
f.toIsoString=function(a,b){var c=cj.prototype.toIsoString.call(this,a);return a?c+" "+Va(this.getHours(),2)+":"+Va(this.getMinutes(),2)+":"+Va(this.getSeconds(),2)+(b?hj(this):""):c+"T"+Va(this.getHours(),2)+Va(this.getMinutes(),2)+Va(this.getSeconds(),2)+(b?hj(this):"")};f.tb=function(a){return this.getTime()==a.getTime()};f.toString=function(){return this.toIsoString()};f.clone=function(){var a=new ij(this.date);a.ed=this.ed;a.Pf=this.Pf;return a};function jj(){}
function kj(a){if("number"==typeof a){var b=new jj;b.wl=a;var c=a;if(0==c)c="Etc/GMT";else{var d=["Etc/GMT",0>c?"-":"+"];c=Math.abs(c);d.push(Math.floor(c/60)%100);c%=60;0!=c&&d.push(":",Va(c,2));c=d.join("")}b.Sp=c;c=a;0==c?c="UTC":(d=["UTC",0>c?"+":"-"],c=Math.abs(c),d.push(Math.floor(c/60)%100),c%=60,0!=c&&d.push(":",c),c=d.join(""));a=lj(a);b.Yp=[c,c];b.yg={gA:a,Dq:a};b.Fl=[];return b}b=new jj;b.Sp=a.id;b.wl=-a.std_offset;b.Yp=a.names;b.yg=a.names_ext;b.Fl=a.transitions;return b}
function lj(a){var b=["GMT"];b.push(0>=a?"+":"-");a=Math.abs(a);b.push(Va(Math.floor(a/60)%100,2),":",Va(a%60,2));return b.join("")}function mj(a,b){for(var c=Date.UTC(b.getUTCFullYear(),b.getUTCMonth(),b.getUTCDate(),b.getUTCHours(),b.getUTCMinutes())/36E5,d=0;d<a.Fl.length&&c>=a.Fl[d];)d+=2;return 0==d?0:a.Fl[d-1]};function nj(a,b){this.Xa=[];this.da=b||Ti;"number"==typeof a?this.zj(a):this.Nh(a)}var oj=[/^\'(?:[^\']|\'\')*(\'|$)/,/^(?:G+|y+|M+|k+|S+|E+|a+|h+|K+|H+|c+|L+|Q+|d+|m+|s+|v+|V+|w+|z+|Z+)/,/^[^\'GyMkSEahKHcLQdmsvVwzZ]+/];function pj(a){return a.getHours?a.getHours():0}
nj.prototype.Nh=function(a){for(qj&&(a=a.replace(/\u200f/g,""));a;){for(var b=a,c=0;c<oj.length;++c){var d=a.match(oj[c]);if(d){var e=d[0];a=a.substring(e.length);0==c&&("''"==e?e="'":(e=e.substring(1,"'"==d[1]?e.length-1:e.length),e=e.replace(/\'\'/g,"'")));this.Xa.push({text:e,type:c});break}}if(b===a)throw Error("Malformed pattern part: "+a);}};
nj.prototype.format=function(a,b){if(!a)throw Error("The date to format must be non-null.");var c=b?6E4*(a.getTimezoneOffset()-(b.wl-mj(b,a))):0,d=c?new Date(a.getTime()+c):a,e=d;b&&d.getTimezoneOffset()!=a.getTimezoneOffset()&&(d=new Date(d.getTime()+6E4*(d.getTimezoneOffset()-a.getTimezoneOffset())),e=new Date(a.getTime()+(c+(0<c?-864E5:864E5))));for(var c=[],g=0;g<this.Xa.length;++g){var h=this.Xa[g].text;1==this.Xa[g].type?c.push(rj(this,h,a,d,e,b)):c.push(h)}return c.join("")};
nj.prototype.zj=function(a){if(4>a)var b=this.da.Cg[a];else if(8>a)b=this.da.Jh[a-4];else if(12>a)b=this.da.Vl[a-8],b=b.replace("{1}",this.da.Cg[a-8]),b=b.replace("{0}",this.da.Jh[a-8]);else{this.zj(10);return}this.Nh(b)};function sj(a,b){var c=String(b);var d=a.da||Ti;if(void 0!==d.om){for(var e=[],g=0;g<c.length;g++){var h=c.charCodeAt(g);e.push(48<=h&&57>=h?String.fromCharCode(d.om+h-48):c.charAt(g))}c=e.join("")}return c}var qj=!1;
function tj(a){if(!(a.getHours&&a.getSeconds&&a.getMinutes))throw Error("The date to format has no time (probably a goog.date.Date). Use Date or goog.date.DateTime, or use a pattern without time fields.");}
function rj(a,b,c,d,e,g){var h=b.length;switch(b.charAt(0)){case "G":return c=0<d.getFullYear()?1:0,4<=h?a.da.rq[c]:a.da.Wl[c];case "y":return c=d.getFullYear(),0>c&&(c=-c),2==h&&(c%=100),sj(a,Va(c,h));case "M":a:switch(c=d.getMonth(),h){case 5:h=a.da.vq[c];break a;case 4:h=a.da.Zl[c];break a;case 3:h=a.da.hm[c];break a;default:h=sj(a,Va(c+1,h))}return h;case "k":return tj(e),c=pj(e)||24,sj(a,Va(c,h));case "S":return sj(a,(e.getTime()%1E3/1E3).toFixed(Math.min(3,h)).substr(2)+(3<h?Va(0,h-3):""));
case "E":return c=d.getDay(),4<=h?a.da.mm[c]:a.da.jm[c];case "a":return tj(e),h=pj(e),a.da.Ol[12<=h&&24>h?1:0];case "h":return tj(e),c=pj(e)%12||12,sj(a,Va(c,h));case "K":return tj(e),c=pj(e)%12,sj(a,Va(c,h));case "H":return tj(e),c=pj(e),sj(a,Va(c,h));case "c":a:switch(c=d.getDay(),h){case 5:h=a.da.Bq[c];break a;case 4:h=a.da.Cq[c];break a;case 3:h=a.da.lm[c];break a;default:h=sj(a,Va(c,1))}return h;case "L":a:switch(c=d.getMonth(),h){case 5:h=a.da.Aq[c];break a;case 4:h=a.da.Eg[c];break a;case 3:h=
a.da.km[c];break a;default:h=sj(a,Va(c+1,h))}return h;case "Q":return c=Math.floor(d.getMonth()/3),4>h?a.da.im[c]:a.da.cm[c];case "d":return sj(a,Va(d.getDate(),h));case "m":return tj(e),sj(a,Va(e.getMinutes(),h));case "s":return tj(e),sj(a,Va(e.getSeconds(),h));case "v":return h=g||kj(c.getTimezoneOffset()),h.Sp;case "V":return a=g||kj(c.getTimezoneOffset()),2>=h?a.Sp:0<mj(a,c)?da(a.yg.eu)?a.yg.eu:a.yg.DST_GENERIC_LOCATION:da(a.yg.Dq)?a.yg.Dq:a.yg.STD_GENERIC_LOCATION;case "w":return c=Vi(e.getFullYear(),
e.getMonth(),e.getDate(),a.da.sj,a.da.rj),sj(a,Va(c,h));case "z":return a=g||kj(c.getTimezoneOffset()),4>h?a.Yp[0<mj(a,c)?2:0]:a.Yp[0<mj(a,c)?3:1];case "Z":return b=g||kj(c.getTimezoneOffset()),4>h?(h=-(b.wl-mj(b,c)),a=[0>h?"-":"+"],h=Math.abs(h),a.push(Va(Math.floor(h/60)%100,2),Va(h%60,2)),h=a.join("")):h=sj(a,lj(b.wl-mj(b,c))),h;default:return""}};function uj(a,b){this.Xa=[];this.da=b||Ti;"number"==typeof a?this.zj(a):this.Nh(a)}
uj.prototype.Nh=function(a){for(var b=!1,c="",d=0;d<a.length;d++){var e=a.charAt(d);if(" "==e)for(0<c.length&&(this.Xa.push({text:c,count:0,Gg:!1}),c=""),this.Xa.push({text:" ",count:0,Gg:!1});d<a.length-1&&" "==a.charAt(d+1);)d++;else if(b)"'"==e?d+1<a.length&&"'"==a.charAt(d+1)?(c+="'",d++):b=!1:c+=e;else if(0<=vj.indexOf(e)){0<c.length&&(this.Xa.push({text:c,count:0,Gg:!1}),c="");var g=a.charAt(d);for(var h=d+1;h<a.length&&a.charAt(h)==g;)h++;g=h-d;this.Xa.push({text:e,count:g,Gg:!1});d+=g-1}else"'"==
e?d+1<a.length&&"'"==a.charAt(d+1)?(c+="'",d++):b=!0:c+=e}0<c.length&&this.Xa.push({text:c,count:0,Gg:!1});a=!1;for(b=0;b<this.Xa.length;b++)wj(this.Xa[b])?!a&&b+1<this.Xa.length&&wj(this.Xa[b+1])&&(a=!0,this.Xa[b].Gg=!0):a=!1};uj.prototype.zj=function(a){11<a&&(a=10);if(4>a)var b=this.da.Cg[a];else 8>a?b=this.da.Jh[a-4]:(b=this.da.Vl[a-8],b=b.replace("{1}",this.da.Cg[a-8]),b=b.replace("{0}",this.da.Jh[a-8]));this.Nh(b)};uj.prototype.parse=function(a,b,c){return xj(this,a,b,c||0,!1)};
function xj(a,b,c,d,e){for(var g=new yj,h=[d],k=-1,l=0,m=0,p=0;p<a.Xa.length;p++)if(0<a.Xa[p].count)if(0>k&&a.Xa[p].Gg&&(k=p,l=d,m=0),0<=k){var u=a.Xa[p].count;if(p==k&&(u-=m,m++,0==u))return 0;zj(a,b,h,a.Xa[p],u,g)||(p=k-1,h[0]=l)}else{if(k=-1,!zj(a,b,h,a.Xa[p],0,g))return 0}else{k=-1;if(" "==a.Xa[p].text.charAt(0)){if(u=h[0],Aj(b,h),h[0]>u)continue}else if(b.indexOf(a.Xa[p].text,h[0])==h[0]){h[0]+=a.Xa[p].text.length;continue}return 0}a:if(void 0!=g.zr&&void 0!=g.year&&0==g.zr&&0<g.year&&(g.year=
-(g.year-1)),void 0!=g.year&&c.setFullYear(g.year),a=c.getDate(),c.setDate(1),void 0!=g.month&&c.setMonth(g.month),void 0!=g.day?c.setDate(g.day):(b=Ui(c.getFullYear(),c.getMonth()),c.setDate(a>b?b:a)),na(c.setHours)&&(void 0==g.hours&&(g.hours=c.getHours()),void 0!=g.Nq&&0<g.Nq&&12>g.hours&&(g.hours+=12),c.setHours(g.hours)),na(c.setMinutes)&&void 0!=g.minutes&&c.setMinutes(g.minutes),na(c.setSeconds)&&void 0!=g.seconds&&c.setSeconds(g.seconds),na(c.setMilliseconds)&&void 0!=g.milliseconds&&c.setMilliseconds(g.milliseconds),
e&&(void 0!=g.year&&g.year!=c.getFullYear()||void 0!=g.month&&g.month!=c.getMonth()||void 0!=g.day&&g.day!=c.getDate()||24<=g.hours||60<=g.minutes||60<=g.seconds||1E3<=g.milliseconds))c=!1;else{void 0!=g.Zp&&c.setTime(c.getTime()+6E4*(g.Zp-c.getTimezoneOffset()));g.Qu&&(e=new Date,e.setFullYear(e.getFullYear()-80),c.getTime()<e.getTime()&&c.setFullYear(e.getFullYear()+100));if(void 0!=g.Ym)if(void 0==g.day)g=(7+g.Ym-c.getDay())%7,3<g&&(g-=7),e=c.getMonth(),c.setDate(c.getDate()+g),c.getMonth()!=e&&
c.setDate(c.getDate()+(0<g?-7:7));else if(g.Ym!=c.getDay()){c=!1;break a}c=!0}return c?h[0]-d:0}var vj="GyMdkHmsSEDahKzZvQL";function wj(a){if(0>=a.count)return!1;var b="MydhHmsSDkK".indexOf(a.text.charAt(0));return 0<b||0==b&&3>a.count}function Aj(a,b){var c=a.substring(b[0]).match(/^\s+/);c&&(b[0]+=c[0].length)}
function zj(a,b,c,d,e,g){Aj(b,c);var h=c[0],k=d.text.charAt(0),l=-1;if(wj(d))if(0<e){if(h+e>b.length)return!1;l=Bj(a,b.substring(0,h+e),c)}else l=Bj(a,b,c);switch(k){case "G":return l=Cj(b,c,a.da.Wl),0<=l&&(g.zr=l),!0;case "M":case "L":a:{d=l;if(0>d){d=Cj(b,c,a.da.Zl.concat(a.da.Eg).concat(a.da.hm).concat(a.da.km));if(0>d){g=!1;break a}g.month=d%12}else g.month=d-1;g=!0}return g;case "E":return d=Cj(b,c,a.da.mm),0>d&&(d=Cj(b,c,a.da.jm)),0>d?g=!1:(g.Ym=d,g=!0),g;case "a":return l=Cj(b,c,a.da.Ol),0<=
l&&(g.Nq=l),!0;case "y":a:{if(0>l){var m=b.charAt(c[0]);if("+"!=m&&"-"!=m){g=!1;break a}c[0]++;l=Bj(a,b,c);if(0>l){g=!1;break a}"-"==m&&(l=-l)}m||2!=c[0]-h||2!=d.count?g.year=l:(a=l,b=(new Date).getFullYear()-80,c=b%100,g.Qu=a==c,a+=100*Math.floor(b/100)+(a<c?100:0),g.year=a);g=!0}return g;case "Q":return 0>l?(d=Cj(b,c,a.da.cm),0>d&&(d=Cj(b,c,a.da.im)),0>d?g=!1:(g.month=3*d,g.day=1,g=!0)):g=!1,g;case "d":return 0<=l&&(g.day=l),!0;case "S":return a=c[0]-h,g.milliseconds=3>a?l*Math.pow(10,3-a):Math.round(l/
Math.pow(10,a-3)),!0;case "h":12==l&&(l=0);case "K":case "H":case "k":return 0<=l&&(g.hours=l),!0;case "m":return 0<=l&&(g.minutes=l),!0;case "s":return 0<=l&&(g.seconds=l),!0;case "z":case "Z":case "v":b.indexOf("GMT",c[0])==c[0]&&(c[0]+=3);a:if(c[0]>=b.length)g.Zp=0,g=!0;else{d=1;switch(b.charAt(c[0])){case "-":d=-1;case "+":c[0]++}h=c[0];l=Bj(a,b,c);if(0>l)g=!1;else{if(c[0]<b.length&&":"==b.charAt(c[0])){m=60*l;c[0]++;l=Bj(a,b,c);if(0>l){g=!1;break a}m+=l}else m=l,m=24>m&&2>=c[0]-h?60*m:m%100+
m/100*60;g.Zp=-(m*d);g=!0}}return g;default:return!1}}function Bj(a,b,c){if(a.da.om){for(var d=[],e=c[0];e<b.length;e++){var g=b.charCodeAt(e)-a.da.om;d.push(0<=g&&9>=g?String.fromCharCode(g+48):b.charAt(e))}b=d.join("")}else b=b.substring(c[0]);a=b.match(/^\d+/);if(!a)return-1;c[0]+=a[0].length;return parseInt(a[0],10)}
function Cj(a,b,c){var d=0,e=-1;a=a.substring(b[0]).toLowerCase();for(var g=0;g<c.length;g++){var h=c[g].length;h>d&&0==a.indexOf(c[g].toLowerCase())&&(e=g,d=h)}0<=e&&(b[0]+=d);return e}function yj(){};function Dj(a,b){this.Pp=a;this.Te=b}var Ej=new cj(0,0,1),Fj=new cj(9999,11,31);Dj.prototype.getStartDate=function(){return this.Pp};Dj.prototype.contains=function(a){return a.valueOf()>=this.Pp.valueOf()&&a.valueOf()<=this.Te.valueOf()};Dj.prototype.iterator=function(){return new Gj(this)};function Hj(a,b){var c=a.clone();c.add(new Wi(Zi,b));return c}function Ij(a,b){var c=a.clone();c.setDate(1);c.add(new Wi(Yi,b));return c}function Jj(a){a=Kj(a);a=Hj(a,-1);return new Dj(a,a.clone())}
function Lj(a){a=Kj(a);return new Dj(a,a.clone())}function Mj(a){a=Kj(a);var b=Hj(a,-1);return new Dj(Hj(a,-7),b)}function Nj(a){a=Kj(a);return new Dj(Ij(a,-1),Hj(Ij(a,0),-1))}function Oj(a){a=Kj(a);var b=(a.getDay()+6)%7;var c=a.ed;a=Hj(a,-(b>=c?b-c:b+(7-c)));b=Hj(a,6);b=new Dj(a,b);a=Hj(b.getStartDate(),-7);b=Hj(b.Te,-7);return new Dj(a,b)}function Kj(a){return a?a.clone():new cj}function Gj(a){this.Wo=a.getStartDate().clone();this.Te=Number(a.Te.toIsoString())}r(Gj,Od);
Gj.prototype.next=function(){if(Number(this.Wo.toIsoString())>this.Te)throw Nd;var a=this.Wo.clone();this.Wo.add(new Wi(Zi,1));return a};var Pj={Eq:"y",Lu:"y G",Mu:"LLLL y",Fq:"LLLL y",ku:"d. M.",lu:"dd. MMMM",nu:"d. M.",mu:"d. MMMM",ou:"d. M. y",Ju:"EEE d. M.",Ku:"EEE d. M. y",cu:"d."},Qj={Eq:"y",Lu:"y G",Mu:"MMM y",Fq:"MMMM y",ku:"MMM d",lu:"MMMM dd",nu:"M/d",mu:"MMMM d",ou:"MMM d, y",Ju:"EEE, MMM d",Ku:"EEE, MMM d, y",cu:"d"},Rj=Qj,Rj=Qj;function Sj(a,b){this.Wu=a;this.S=b||hf()}Sj.prototype.I=function(){return this.S};Sj.prototype.X=function(){return this.Wu};Sj.prototype.$k=function(a,b){var c=this.I().createElement("TD");c.colSpan=b?2:3;c.className=this.X()+"-today-cont";Tj(this,c,"Today",this.X()+"-today-btn");a.appendChild(c);c=this.I().createElement("TD");c.colSpan=b?4:3;a.appendChild(c);c=this.I().createElement("TD");c.colSpan=2;c.className=this.X()+"-none-cont";Tj(this,c,"None",this.X()+"-none-btn");a.appendChild(c)};
function Tj(a,b,c,d){var e=[a.X()+"-btn"];d&&e.push(d);d=a.I().createElement("BUTTON");d.className=e.join(" ");d.appendChild(a.I().createTextNode(c));b.appendChild(d)};function Uj(a,b,c,d){kh.call(this,c);this.zc=b||Ti;this.sx=this.zc.lm;this.gw=new nj("d",this.zc);this.ew=new nj("dd",this.zc);this.iw=new nj("w",this.zc);this.fw=new nj("M d",this.zc);this.Ei=new nj(Rj.Eq||"y",this.zc);this.hw=new nj(Rj.Fq||"MMMM y",this.zc);this.W=d||new Sj(this.X(),this.I());this.bb=new cj(a);this.bb.Pf=this.zc.sj;this.bb.ed=this.zc.rj;this.Qa=this.bb.clone();this.Qa.setDate(1);this.Ll="      ".split(" ");this.Ll[this.zc.nm[0]]=this.X()+"-wkend-start";this.Ll[this.zc.nm[1]]=this.X()+
"-wkend-end";this.fh={};this.Tf=[]}r(Uj,kh);f=Uj.prototype;f.wt=!0;f.ix=!0;f.bq=new Dj(Ej,Fj);f.Fv=!0;f.fj=!0;f.At=!0;f.Mh=!0;f.sl=!0;f.Np=!1;f.vr=null;f.nw=!1;f.ak=null;f.$j=null;f.Zj=null;f.Zu=ih.ua();f.X=function(){return"goog-date-picker"};function Vj(a){w(a.ck,a.sl);w(a.bk,a.Mh);w(a.It,a.sl||a.Mh)}f.Rs=function(){this.Qa.add(new Wi(Yi,-1));Wj(this);Xj(this)};f.ts=function(){this.Qa.add(new Wi(Yi,1));Wj(this);Xj(this)};f.Sw=function(){this.Qa.add(new Wi(Xi,-1));Wj(this);Xj(this)};
f.xw=function(){this.Qa.add(new Wi(Xi,1));Wj(this);Xj(this)};f.kt=function(){this.setDate(new cj)};f.jt=function(){this.Mh&&this.setDate(null)};f.getDate=function(){return this.bb&&this.bb.clone()};f.setDate=function(a){Yj(this,a,!0)};
function Yj(a,b,c){var d=b==a.bb||b&&a.bb&&b.getFullYear()==a.bb.getFullYear()&&b.getMonth()==a.bb.getMonth(),e=b==a.bb||d&&b.getDate()==a.bb.getDate();a.bb=b&&new cj(b);b&&(a.Qa.set(a.bb),a.Qa.setFullYear(a.bb.getFullYear()),a.Qa.setDate(1));Wj(a);c&&a.dispatchEvent(new Zj("select",a,a.bb));e||a.dispatchEvent(new Zj("change",a,a.bb));d||Xj(a)}
function ak(a){if(a.ak){for(var b=a.ak;b.firstChild;)b.removeChild(b.firstChild);var c=a.W,d=a.fj,e=a.zc.Cg[0].toLowerCase();if(a.Np)e=c.I().createElement("TD"),e.colSpan=d?1:2,Tj(c,e,"«",c.X()+"-previousMonth"),b.appendChild(e),e=c.I().createElement("TD"),e.colSpan=d?6:5,e.className=c.X()+"-monthyear",b.appendChild(e),e=c.I().createElement("TD"),Tj(c,e,"»",c.X()+"-nextMonth"),b.appendChild(e);else{d=c.I().createElement("TD");d.colSpan=5;Tj(c,d,"«",c.X()+"-previousMonth");Tj(c,d,"",c.X()+"-month");
Tj(c,d,"»",c.X()+"-nextMonth");var g=c.I().createElement("TD");g.colSpan=3;Tj(c,g,"«",c.X()+"-previousYear");Tj(c,g,"",c.X()+"-year");Tj(c,g,"»",c.X()+"-nextYear");e.indexOf("y")<e.indexOf("m")?(b.appendChild(g),b.appendChild(d)):(b.appendChild(d),b.appendChild(g))}if(a.Np){bk(a,b,a.X()+"-previousMonth",a.Rs);if(c=nf(a.X()+"-previousMonth",b))gi(c,"hidden",!0),c.tabIndex=-1;bk(a,b,a.X()+"-nextMonth",a.ts);if(c=nf(a.X()+"-nextMonth",b))gi(c,"hidden",!0),c.tabIndex=-1;a.$j=nf(a.X()+"-monthyear",b)}else bk(a,
b,a.X()+"-previousMonth",a.Rs),bk(a,b,a.X()+"-nextMonth",a.ts),bk(a,b,a.X()+"-month",a.hx),bk(a,b,a.X()+"-previousYear",a.Sw),bk(a,b,a.X()+"-nextYear",a.xw),bk(a,b,a.X()+"-year",a.Kp),a.Jf=nf(a.X()+"-month",b),a.Kf=hf().jk(a.X()+"-year",b)}}function bk(a,b,c,d){b=nf(c,b);a.Va().A(b,"click",function(a){a.preventDefault();d.call(this,a)})}
function ck(a){if(a.Zj){var b=a.Zj;Cf(b);a.W.$k(b,a.fj);bk(a,b,a.X()+"-today-btn",a.kt);bk(a,b,a.X()+"-none-btn",a.jt);a.ck=nf(a.X()+"-today-btn",b);a.bk=nf(a.X()+"-none-btn",b);Vj(a)}}
f.Ca=function(a){Uj.m.Ca.call(this,a);ki(a,this.X());var b=this.S.createElement("TABLE"),c=this.S.createElement("THEAD"),d=this.S.createElement("TBODY"),e=this.S.createElement("TFOOT");fi(d,"grid");d.tabIndex=0;this.Rp=d;this.It=e;var g=this.S.createElement("TR");g.className=this.X()+"-head";this.ak=g;ak(this);c.appendChild(g);this.Kc=[];for(var h=0;7>h;h++){g=this.S.createElement("TR");this.Kc[h]=[];for(var k=0;8>k;k++){var l=this.S.createElement(0==k||0==h?"th":"td");0!=k&&0!=h||k==h||(l.className=
0==k?this.X()+"-week":this.X()+"-wday",fi(l,0==k?"rowheader":"columnheader"));g.appendChild(l);this.Kc[h][k]=l}d.appendChild(g)}g=this.S.createElement("TR");g.className=this.X()+"-foot";this.Zj=g;ck(this);e.appendChild(g);b.cellSpacing="0";b.cellPadding="0";b.appendChild(c);b.appendChild(d);b.appendChild(e);a.appendChild(b);if(this.o()){if(this.At)for(b=0;7>b;b++)Kf(this.Kc[0][b+1],this.sx[((b+this.Qa.ed+7)%7+1)%7]);w(this.Kc[0][0].parentElement||this.Kc[0][0].parentNode,this.At)}Wj(this);a.tabIndex=
0};f.D=function(){Uj.m.D.call(this);this.Ca(this.o())};f.Aa=function(){Uj.m.Aa.call(this);var a=this.Va();a.A(this.Rp,"click",this.Pv);a.A(dk(this,this.o()),"key",this.Qv)};f.lb=function(){Uj.m.lb.call(this);this.Qg();for(var a in this.fh)this.fh[a].ib();this.fh={}};f.create=Uj.prototype.J;f.V=function(){Uj.m.V.call(this);this.bk=this.ck=this.Kf=this.$j=this.Jf=this.Zj=this.ak=this.It=this.Rp=this.Kc=null};
f.Pv=function(a){if("TD"==a.target.tagName){var b,c=-2,d=-2;for(b=a.target;b;b=b.previousSibling,c++);for(b=a.target.parentNode;b;b=b.previousSibling,d++);a=this.Tf[d][c];this.bq.contains(a)&&this.setDate(a.clone())}};
f.Qv=function(a){switch(a.keyCode){case 33:a.preventDefault();var b=-1;break;case 34:a.preventDefault();b=1;break;case 37:a.preventDefault();var c=-1;break;case 39:a.preventDefault();c=1;break;case 38:a.preventDefault();c=-7;break;case 40:a.preventDefault();c=7;break;case 36:a.preventDefault(),this.kt();case 46:a.preventDefault();this.jt();break;case 13:case 32:a.preventDefault(),Yj(this,this.bb,!0);default:return}this.bb?(a=this.bb.clone(),a.add(new Wi(0,b,c))):(a=this.Qa.clone(),a.setDate(1));this.bq.contains(a)&&
Yj(this,a,!1)};f.hx=function(a){a.stopPropagation();a=[];for(var b=0;12>b;b++)a.push(this.zc.Eg[b]);ek(this,this.Jf,a,this.Wv,this.zc.Eg[this.Qa.getMonth()])};f.Kp=function(a){a.stopPropagation();a=[];for(var b=this.Qa.getFullYear(),c=this.Qa.clone(),d=-5;5>=d;d++)c.setFullYear(b+d),a.push(this.Ei.format(c));ek(this,this.Kf,a,this.io,this.Ei.format(this.Qa))};f.Wv=function(a){a=Number(a.getAttribute("itemIndex"));this.Qa.setMonth(a);Wj(this);this.Jf.focus&&this.Jf.focus()};
f.io=function(a){if(3==a.firstChild.nodeType){a=Number(a.getAttribute("itemIndex"));var b=this.Qa.getFullYear();this.Qa.setFullYear(b+a-5);Wj(this)}this.Kf.focus()};
function ek(a,b,c,d,e){a.Qg();var g=a.S.createElement("DIV");g.className=a.X()+"-menu";a.lh=null;for(var h=a.S.createElement("UL"),k=0;k<c.length;k++){var l=a.S.D("LI",null,c[k]);l.setAttribute("itemIndex",k);c[k]==e&&(a.lh=l);h.appendChild(l)}g.appendChild(h);g.style.left=b.offsetLeft+b.parentNode.offsetLeft+"px";g.style.top=b.offsetTop+"px";g.style.width=b.clientWidth+"px";a.Jf.parentNode.appendChild(g);a.jg=g;a.lh||(a.lh=h.firstChild);a.lh.className=a.X()+"-menu-selected";a.To=d;b=a.Va();b.A(a.jg,
"click",a.Mr);b.A(dk(a,a.jg),"key",a.Nr);b.A(a.S.R,"click",a.Qg);g.tabIndex=0;g.focus()}f.Mr=function(a){a.stopPropagation();this.Qg();this.To&&this.To(a.target)};
f.Nr=function(a){a.stopPropagation();var b=this.lh;switch(a.keyCode){case 35:a.preventDefault();var c=b.parentNode.lastChild;break;case 36:a.preventDefault();c=b.parentNode.firstChild;break;case 38:a.preventDefault();c=b.previousSibling;break;case 40:a.preventDefault();c=b.nextSibling;break;case 13:case 9:case 0:a.preventDefault(),this.Qg(),this.To(b)}c&&c!=b&&(b.className="",c.className=this.X()+"-menu-selected",this.lh=c)};
f.Qg=function(){if(this.jg){var a=this.Va();a.Ba(this.jg,"click",this.Mr);a.Ba(dk(this,this.jg),"key",this.Nr);a.Ba(this.S.R,"click",this.Qg);Ff(this.jg);delete this.jg}};
function Wj(a){if(a.o()){var b=a.Qa.clone();b.setDate(1);a.$j&&Kf(a.$j,a.hw.format(b));a.Jf&&Kf(a.Jf,a.zc.Eg[b.getMonth()]);a.Kf&&Kf(a.Kf,a.Ei.format(b));var c=((b.getDay()+6)%7-b.ed+7)%7;var d=gj(b);b.add(new Wi(Yi,-1));b.setDate(gj(b)-(c-1));a.wt&&!a.Fv&&33>d+c&&b.add(new Wi(Zi,-7));c=new Wi(Zi,1);a.Tf=[];for(d=0;6>d;d++){a.Tf[d]=[];for(var e=0;7>e;e++){a.Tf[d][e]=b.clone();var g=b.getFullYear();b.add(c);0==b.getMonth()&&1==b.getDate()&&g++;b.setFullYear(g)}}if(a.o())for(b=a.Qa.getMonth(),e=new cj,
c=e.getFullYear(),d=e.getMonth(),e=e.getDate(),g=0;6>g;g++){a.fj?(Kf(a.Kc[g+1][0],a.iw.format(a.Tf[g][0])),a.Kc[g+1][0].className=a.X()+"-week"):(Kf(a.Kc[g+1][0],""),a.Kc[g+1][0].className="");for(var h=0;7>h;h++){var k=a.Tf[g][h],l=a.Kc[g+1][h+1];l.id||(l.id=jh(a.Zu));fi(l,"gridcell");var m=a.fw.format(k);gi(l,"label",m);m=[a.X()+"-date"];a.bq.contains(k)||m.push(a.X()+"-unavailable-date");if(a.ix||k.getMonth()==b){k.getMonth()!=b&&m.push(a.X()+"-other-month");var p=(h+a.Qa.ed+7)%7;a.Ll[p]&&m.push(a.Ll[p]);
k.getDate()==e&&k.getMonth()==d&&k.getFullYear()==c&&m.push(a.X()+"-today");a.bb&&k.getDate()==a.bb.getDate()&&k.getMonth()==a.bb.getMonth()&&k.getFullYear()==a.bb.getFullYear()&&(m.push(a.X()+"-selected"),gi(a.Rp,"activedescendant",l.id));a.vr&&(p=a.vr(k))&&m.push(p);k=a.nw?a.ew.format(k):a.gw.format(k);Kf(l,k)}else Kf(l,"");l.className=m.join(" ")}4<=g&&w(a.Kc[g+1][0].parentElement||a.Kc[g+1][0].parentNode,a.Tf[g][0].getMonth()==b||a.wt)}}}
function Xj(a){var b=new Zj("changeActiveMonth",a,a.Qa.clone());a.dispatchEvent(b)}function dk(a,b){var c=pa(b);c in a.fh||(a.fh[c]=new Yh(b));return a.fh[c]}function Zj(a,b,c){Cc.call(this,a,b);this.date=c}r(Zj,Cc);function fk(a,b){md.call(this);this.vb=new fh(this);this.Cp(a||null);b&&(this.Ch=b)}r(fk,md);f=fk.prototype;f.F=null;f.Vu=!0;f.Rq=null;f.Sq=null;f.eh=!1;f.fx=!1;f.Io=-1;f.cw=!1;f.Cv=!0;f.Ch="toggle_display";f.o=function(){return this.F};f.Cp=function(a){if(this.eh)throw Error("Can not change this state of the popup while showing.");this.F=a};f.Va=function(){return this.vb};f.va=function(){return this.eh};f.$=function(a){this.ej&&this.ej.stop();this.Di&&this.Di.stop();a?this.Lp():this.$g()};f.yc=ja;
f.Lp=function(){if(!this.eh&&this.$o()){if(!this.F)throw Error("Caller must call setElement before trying to show the popup");this.yc();var a=kf(this.F);this.cw&&this.vb.A(a,"keydown",this.Aw,!0);if(this.Vu)if(this.vb.A(a,"mousedown",this.ws,!0),t){try{var b=a.activeElement}catch(d){}for(;b&&"IFRAME"==b.nodeName;){try{var c=Jf(b)}catch(d){break}a=c;b=a.activeElement}this.vb.A(a,"mousedown",this.ws,!0);this.vb.A(a,"deactivate",this.vs)}else this.vb.A(a,"blur",this.vs);"toggle_display"==this.Ch?(this.F.style.visibility=
"visible",w(this.F,!0)):"move_offscreen"==this.Ch&&this.yc();this.eh=!0;this.Io=wa();this.ej?(cd(this.ej,"end",this.ng,!1,this),this.ej.play()):this.ng()}};f.$g=function(a){if(!this.eh||!this.dispatchEvent({type:"beforehide",target:a}))return!1;this.vb&&this.vb.sd();this.eh=!1;wa();this.Di?(cd(this.Di,"end",va(this.or,a),!1,this),this.Di.play()):this.or(a);return!0};
f.or=function(a){"toggle_display"==this.Ch?this.fx?od(this.Rr,0,this):this.Rr():"move_offscreen"==this.Ch&&(this.F.style.top="-10000px");this.jf(a)};f.Rr=function(){this.F.style.visibility="hidden";w(this.F,!1)};f.$o=function(){return this.dispatchEvent("beforeshow")};f.ng=function(){this.dispatchEvent("show")};f.jf=function(a){this.dispatchEvent({type:"hide",target:a})};f.ws=function(a){a=a.target;If(this.F,a)||gk(this,a)||this.Sq&&!If(this.Sq,a)||150>wa()-this.Io||this.$g(a)};
f.Aw=function(a){27==a.keyCode&&this.$g(a.target)&&(a.preventDefault(),a.stopPropagation())};f.vs=function(a){if(this.Cv){var b=kf(this.F);if("undefined"!=typeof document.activeElement){if(a=b.activeElement,!a||If(this.F,a)||"BODY"==a.tagName)return}else if(a.target!=b)return;150>wa()-this.Io||this.$g()}};function gk(a,b){return ib(a.Rq||[],function(a){return b===a||If(a,b)})}f.V=function(){fk.m.V.call(this);this.vb.ib();ac(this.ej);ac(this.Di);delete this.F;delete this.vb;delete this.Rq};function hk(a,b,c,d,e,g,h,k){var l;if(l=c.offsetParent){var m="HTML"==l.tagName||"BODY"==l.tagName;if(!m||"static"!=fg(l)){var p=mg(l);m||(m=(m=vg(l))&&jc?-l.scrollLeft:!m||ic&&uc("8")||"visible"==eg(l,"overflowX")?l.scrollLeft:l.scrollWidth-l.clientWidth-l.scrollLeft,p=ff(p,new df(m,l.scrollTop)))}}l=p||new df;p=mg(a);m=tg(a);p=new $f(p.x,p.y,m.width,m.height);(m=lg(a))&&p.Wr(new $f(m.left,m.top,m.right-m.left,m.bottom-m.top));var m=hf(a),u=hf(c);if(m.R!=u.R){var A=m.R.body;var u=ne(u),D=new df(0,
0),E=wf(kf(A));if(ec(E,"parent")){var J=A;do{if(E==u)var M=mg(J);else M=jg(J),M=new df(M.left,M.top);D.x+=M.x;D.y+=M.y}while(E&&E!=u&&E!=E.parent&&(J=E.frameElement)&&(E=E.parent))}A=ff(D,mg(A));!t||vc(9)||tf(m.R)||(A=ff(A,uf(m.R)));p.left+=A.x;p.top+=A.y}a=ik(a,b);b=p.left;a&4?b+=p.width:a&2&&(b+=p.width/2);b=new df(b,p.top+(a&1?p.height:0));b=ff(b,l);e&&(b.x+=(a&4?-1:1)*e.x,b.y+=(a&1?-1:1)*e.y);var O;h&&(O=lg(c))&&(O.top-=l.y,O.right-=l.x,O.bottom-=l.y,O.left-=l.x);return jk(b,c,d,g,O,h,k)}
function jk(a,b,c,d,e,g,h){a=a.clone();var k=ik(b,c);c=tg(b);h=h?h.clone():c.clone();a=a.clone();h=h.clone();var l=0;if(d||0!=k)k&4?a.x-=h.width+(d?d.right:0):k&2?a.x-=h.width/2:d&&(a.x+=d.left),k&1?a.y-=h.height+(d?d.bottom:0):d&&(a.y+=d.top);if(g){if(e){d=a;k=h;l=0;65==(g&65)&&(d.x<e.left||d.x>=e.right)&&(g&=-2);132==(g&132)&&(d.y<e.top||d.y>=e.bottom)&&(g&=-5);d.x<e.left&&g&1&&(d.x=e.left,l|=1);if(g&16){var m=d.x;d.x<e.left&&(d.x=e.left,l|=4);d.x+k.width>e.right&&(k.width=Math.min(e.right-d.x,
m+k.width-e.left),k.width=Math.max(k.width,0),l|=4)}d.x+k.width>e.right&&g&1&&(d.x=Math.max(e.right-k.width,e.left),l|=1);g&2&&(l|=(d.x<e.left?16:0)|(d.x+k.width>e.right?32:0));d.y<e.top&&g&4&&(d.y=e.top,l|=2);g&32&&(m=d.y,d.y<e.top&&(d.y=e.top,l|=8),d.y+k.height>e.bottom&&(k.height=Math.min(e.bottom-d.y,m+k.height-e.top),k.height=Math.max(k.height,0),l|=8));d.y+k.height>e.bottom&&g&4&&(d.y=Math.max(e.bottom-k.height,e.top),l|=2);g&8&&(l|=(d.y<e.top?64:0)|(d.y+k.height>e.bottom?128:0));e=l}else e=
256;l=e}g=new $f(0,0,0,0);g.left=a.x;g.top=a.y;g.width=h.width;g.height=h.height;e=l;if(e&496)return e;gg(b,new df(g.left,g.top));h=new gf(g.width,g.height);c==h||c&&h&&c.width==h.width&&c.height==h.height||(c=h,a=tf(hf(kf(b)).R),!t||uc("10")||a&&uc("8")?(b=b.style,jc?b.MozBoxSizing="border-box":kc?b.WebkitBoxSizing="border-box":b.boxSizing="border-box",b.width=Math.max(c.width,0)+"px",b.height=Math.max(c.height,0)+"px"):(h=b.style,a?(t?(k=zg(b,"paddingLeft"),d=zg(b,"paddingRight"),g=zg(b,"paddingTop"),
a=zg(b,"paddingBottom"),a=new Zf(g,d,a,k)):(k=dg(b,"paddingLeft"),d=dg(b,"paddingRight"),g=dg(b,"paddingTop"),a=dg(b,"paddingBottom"),a=new Zf(parseFloat(g),parseFloat(d),parseFloat(a),parseFloat(k))),b=qg(b),h.pixelWidth=c.width-b.left-a.left-a.right-b.right,h.pixelHeight=c.height-b.top-a.top-a.bottom-b.bottom):(h.pixelWidth=c.width,h.pixelHeight=c.height)));return e}function ik(a,b){return(b&8&&vg(a)?b^4:b)&-9};function kk(){}kk.prototype.yc=function(){};function lk(a,b,c){this.element=a;this.qr=b;this.Kw=c}r(lk,kk);lk.prototype.yc=function(a,b,c){hk(this.element,this.qr,a,b,void 0,c,this.Kw)};function mk(a,b){this.Nw=8;this.ip=b||void 0;fk.call(this,a)}r(mk,fk);mk.prototype.setPosition=function(a){this.ip=a||void 0;this.va()&&this.yc()};mk.prototype.yc=function(){if(this.ip){var a=!this.va()&&"move_offscreen"!=this.Ch,b=this.o();a&&(b.style.visibility="hidden",w(b,!0));this.ip.yc(b,this.Nw,this.QA);a&&w(b,!1)}};function nk(a,b){kh.call(this,b);this.sc=a||new Uj}r(nk,kh);f=nk.prototype;f.sc=null;f.Kd=null;f.Lk=null;f.yf=!0;f.D=function(){nk.m.D.call(this);this.o().className="goog-popupdatepicker";this.Kd=new mk(this.o());this.Kd.nl(this)};f.va=function(){return this.Kd?this.Kd.va():!1};f.Aa=function(){nk.m.Aa.call(this);if(!this.sc.la){var a=this.o();a.style.visibility="hidden";w(a,!1);this.sc.J(a)}this.Va().A(this.sc,"change",this.Pi).A(this.sc,"select",this.ap)};
f.V=function(){nk.m.V.call(this);this.Kd&&(this.Kd.ib(),this.Kd=null);this.sc.ib();this.Lk=this.sc=null};f.rb=function(){return!1};f.vn=function(){return this.sc};f.getDate=function(){return this.sc.getDate()};f.setDate=function(a){this.sc.setDate(a)};f.zf=function(a){this.Va().A(a,"mousedown",this.xt)};f.detach=function(a){this.Va().Ba(a,"mousedown",this.xt)};f.mt=function(a){this.yf=a};
f.xt=function(a){this.Lk=a=a.currentTarget;this.Kd.setPosition(new lk(a,9,197));this.Va().Ba(this.sc,"change",this.Pi).Ba(this.sc,"select",this.ap);this.sc.setDate(null);this.dispatchEvent("show");this.Kd.$(!0);this.yf&&this.o().focus();this.Va().A(this.sc,"change",this.Pi).A(this.sc,"select",this.ap)};f.ko=function(){this.Kd.$(!1);this.yf&&this.Lk&&this.Lk.focus()};f.ap=function(a){this.ko();this.dispatchEvent(a)};f.Pi=function(a){this.dispatchEvent(a)};function ok(a,b,c,d){kh.call(this,d);this.Xm=a;this.ur=b;this.pb=new nk(c,d);this.$a(this.pb);this.pb.mt(!1)}r(ok,kh);f=ok.prototype;f.Xm=null;f.ur=null;f.pb=null;f.Os=null;f.vn=function(){return this.pb.vn()};f.getDate=function(){var a=pk(this),b=this.pb.getDate();a&&b?a.tb(b)||this.pb.setDate(a):this.pb.setDate(null);return a};f.setDate=function(a){this.pb.setDate(a)};f.ll=function(a){var b=this.o();b.Ho?b.Ho.qa(a):b.value=a};
function pk(a){var b=a.o();b=b.Ho?b.Ho.ba():b.value;if(b=Da(b)){var c=new ij;if(0<xj(a.ur,b,c,0,!0))return c}return null}f.D=function(){this.F=this.I().D("INPUT",{type:"text"});this.pb.D()};f.Aa=function(){(this.Os||this.I().R.body).appendChild(this.pb.o());ok.m.Aa.call(this);var a=this.o();this.pb.zf(a);this.pb.setDate(pk(this));a=this.Va();a.A(this.pb,"change",this.Pi);a.A(this.pb,"show",this.Dw)};f.lb=function(){ok.m.lb.call(this);var a=this.o();this.pb.detach(a);this.pb.lb();Ff(this.pb.o())};
f.Ca=function(a){ok.m.Ca.call(this,a);this.pb.D()};f.V=function(){ok.m.V.call(this);this.pb.ib();this.Os=this.pb=null};f.ko=function(){this.pb.ko()};f.Dw=function(){var a=pk(this);this.setDate(a);a&&(a=this.vn().getDate(),this.ll(a?this.Xm.format(a):""))};f.Pi=function(a){a=a.date;this.ll(a?this.Xm.format(a):"")};function qk(a){if(na(document.createEvent)){var b=document.createEvent("HTMLEvents");b.initEvent("change",!1,!0);a.dispatchEvent(b)}else a.fireEvent("onchange")};function rk(a,b,c,d){rk.m.constructor.call(this,a,b,c,d);this.Eh=null}r(rk,Uj);rk.prototype.Kp=function(a){var b,c=this;if(null==this.Eh)rk.m.Kp.call(this,a);else{a.stopPropagation();var d=this.Qa.clone();var e=function(a){d.setFullYear(a);return c.Ei.format(d)};return ek(this,this.Kf,function(){var a;var c=vb(this.Eh[0],this.Eh[1]);var d=[];var l=0;for(a=c.length;l<a;l++)b=c[l],d.push(e(b));return d}.call(this),this.io,this.Ei.format(this.Qa))}};
function sk(a,b,c){return null!=a.Eh?a.Eh[0]+c:b+c-5}rk.prototype.io=function(a){3===a.firstChild.nodeType&&(this.Qa.setFullYear(sk(this,this.Qa.getFullYear(),Number(a.getAttribute("itemIndex")))),Wj(this));return this.Kf.focus()};function tk(a,b,c,d){tk.m.constructor.call(this,a,b,c,d)}r(tk,ok);var uk={5:/[0-9][0-9]:[0-9][0-9]/,8:/[0-9][0-9]:[0-9][0-9]:[0-9][0-9]/};tk.prototype.ll=function(a){return tk.m.ll.call(this,vk(this,a))};function vk(a,b){var c;if(c=a.o().value){var d=c.split(" ").pop();c=uk[d.length];if(null!=c&&c.test(d))return[b,d].join(" ")}return b};function wk(a,b,c){this.l=a;this.pattern=null!=b?b:"yyyy'-'MM'-'dd";this.ec=c;this.o=q(this.o,this);this.In=q(this.In,this);wk.m.constructor.call(this,this.l);null==this.ec&&(this.ec={});null==(a=this.ec).yt&&(a.yt=!1);null==(a=this.ec).Lq&&(a.Lq=!1);null==(a=this.ec).zt&&(a.zt=!1);null==(a=this.ec).Rt&&(a.Rt=!1)}r(wk,Pi);
wk.prototype.J=function(a){this.G=a;this.G.type="text";var b=this.pattern;var c=Rh(this.G,"pattern");null!=c&&(b=String(c));c=b;b=new nj(c);var d=new uj(c);c=new rk;if(a=Rh(a,"yearRange")){if(a=qd(a),!la(a))throw Error("data-year-range must be an Array");}else a=null;if(null!=a&&2>a.length)throw Error("Given unexpected range values, expected two given: "+a.length);c.Eh=a;c.sl=this.ec.yt;c.ck&&Vj(c);c.Mh=this.ec.Lq;c.bk&&Vj(c);c.fj=this.ec.zt;ak(c);ck(c);Wj(c);c.Np=this.ec.Rt;ak(c);Wj(c);b=new tk(b,
d,c);b.A("change",this.In);this.he=new Ah({l:this.l});this.he.A("action",this.Vc);this.he.render(Yf(this.G));b.J(this.G);return b};wk.prototype.In=function(a){qk(a.currentTarget.o());return this.Ze(a)};wk.prototype.o=function(){return this.G};function xk(a,b){return v(a,"click",function(){eh(a,"collapsed");var c=Rh(a,"target");c=Eg(b,String(c));return eh(c,"in")})};function yk(a,b){this.oe=this.zg=this.mf="";this.qh=null;this.pi=this.pd="";this.Wc=this.Bo=!1;if(a instanceof yk){this.Wc=da(b)?b:a.Wc;zk(this,a.mf);var c=a.zg;Ak(this);this.zg=c;c=a.oe;Ak(this);this.oe=c;Bk(this,a.qh);Ck(this,a.pd);Dk(this,a.qb.clone());this.wh(a.pi)}else if(a&&(c=String(a).match(Td))){this.Wc=!!b;zk(this,c[1]||"",!0);var d=c[2]||"";Ak(this);this.zg=Ek(d);d=c[3]||"";Ak(this);this.oe=Ek(d,!0);Bk(this,c[4]);Ck(this,c[5]||"",!0);Dk(this,c[6]||"",!0);this.wh(c[7]||"",!0)}else this.Wc=
!!b,this.qb=new Fk(null,0,this.Wc)}f=yk.prototype;f.toString=function(){var a=[],b=this.mf;b&&a.push(Gk(b,Hk,!0),":");var c=this.oe;if(c||"file"==b)a.push("//"),(b=this.zg)&&a.push(Gk(b,Hk,!0),"@"),a.push(encodeURIComponent(String(c)).replace(/%25([0-9a-fA-F]{2})/g,"%$1")),c=this.qh,null!=c&&a.push(":",String(c));if(c=this.pd)this.oe&&"/"!=c.charAt(0)&&a.push("/"),a.push(Gk(c,"/"==c.charAt(0)?Ik:Jk,!0));(c=this.qb.toString())&&a.push("?",c);(c=this.pi)&&a.push("#",Gk(c,Kk));return a.join("")};
f.resolve=function(a){var b=this.clone(),c=!!a.mf;c?zk(b,a.mf):c=!!a.zg;if(c){var d=a.zg;Ak(b);b.zg=d}else c=!!a.oe;c?(d=a.oe,Ak(b),b.oe=d):c=null!=a.qh;d=a.pd;if(c)Bk(b,a.qh);else if(c=!!a.pd){if("/"!=d.charAt(0))if(this.oe&&!this.pd)d="/"+d;else{var e=b.pd.lastIndexOf("/");-1!=e&&(d=b.pd.substr(0,e+1)+d)}e=d;if(".."==e||"."==e)d="";else if(Pa(e,"./")||Pa(e,"/.")){for(var d=0==e.lastIndexOf("/",0),e=e.split("/"),g=[],h=0;h<e.length;){var k=e[h++];"."==k?d&&h==e.length&&g.push(""):".."==k?((1<g.length||
1==g.length&&""!=g[0])&&g.pop(),d&&h==e.length&&g.push("")):(g.push(k),d=!0)}d=g.join("/")}else d=e}c?Ck(b,d):c=""!==a.qb.toString();c?Dk(b,a.qb.clone()):c=!!a.pi;c&&b.wh(a.pi);return b};f.clone=function(){return new yk(this)};function zk(a,b,c){Ak(a);a.mf=c?Ek(b,!0):b;a.mf&&(a.mf=a.mf.replace(/:$/,""))}function Bk(a,b){Ak(a);if(b){b=Number(b);if(isNaN(b)||0>b)throw Error("Bad port number "+b);a.qh=b}else a.qh=null}function Ck(a,b,c){Ak(a);a.pd=c?Ek(b,!0):b}
function Dk(a,b,c){Ak(a);b instanceof Fk?(a.qb=b,a.qb.Ep(a.Wc)):(c||(b=Gk(b,Lk)),a.qb=new Fk(b,0,a.Wc));return a}f.qt=function(a,b){return Dk(this,a,b)};f.vc=function(){return this.qb.toString()};function Mk(a,b,c){Ak(a);a.qb.set(b,c)}f.wh=function(a,b){Ak(this);this.pi=b?Ek(a):a;return this};f.removeParameter=function(a){Ak(this);this.qb.remove(a);return this};f.st=function(a){this.Bo=a;return this};f.Yr=function(){return this.Bo};
function Ak(a){if(a.Bo)throw Error("Tried to modify a read-only Uri");}f.Ep=function(a){this.Wc=a;this.qb&&this.qb.Ep(a);return this};function Ek(a,b){return a?b?decodeURI(a.replace(/%25/g,"%2525")):decodeURIComponent(a):""}function Gk(a,b,c){return ea(a)?(a=encodeURI(a).replace(b,Nk),c&&(a=a.replace(/%25([0-9a-fA-F]{2})/g,"%$1")),a):null}function Nk(a){a=a.charCodeAt(0);return"%"+(a>>4&15).toString(16)+(a&15).toString(16)}var Hk=/[#\/\?@]/g,Jk=/[\#\?:]/g,Ik=/[\#\?]/g,Lk=/[\#\?@]/g,Kk=/#/g;
function Fk(a,b,c){this.La=this.fb=null;this.Lc=a||null;this.Wc=!!c}function Ok(a){a.fb||(a.fb=new Pd,a.La=0,a.Lc&&Ud(a.Lc,function(b,c){a.add(Ea(b),c)}))}f=Fk.prototype;f.Gd=function(){Ok(this);return this.La};f.add=function(a,b){Ok(this);this.Lc=null;a=Pk(this,a);var c=this.fb.get(a);c||this.fb.set(a,c=[]);c.push(b);this.La+=1;return this};f.remove=function(a){Ok(this);a=Pk(this,a);return this.fb.ke(a)?(this.Lc=null,this.La-=this.fb.get(a).length,this.fb.remove(a)):!1};
f.clear=function(){this.fb=this.Lc=null;this.La=0};f.isEmpty=function(){Ok(this);return 0==this.La};f.ke=function(a){Ok(this);a=Pk(this,a);return this.fb.ke(a)};f.di=function(a){var b=this.Ib();return mb(b,a)};f.forEach=function(a,b){Ok(this);this.fb.forEach(function(c,d){db(c,function(c){a.call(b,c,d,this)},this)},this)};f.uc=function(){Ok(this);for(var a=this.fb.Ib(),b=this.fb.uc(),c=[],d=0;d<b.length;d++)for(var e=a[d],g=0;g<e.length;g++)c.push(b[d]);return c};
f.Ib=function(a){Ok(this);var b=[];if(ea(a))this.ke(a)&&(b=pb(b,this.fb.get(Pk(this,a))));else{a=this.fb.Ib();for(var c=0;c<a.length;c++)b=pb(b,a[c])}return b};f.set=function(a,b){Ok(this);this.Lc=null;a=Pk(this,a);this.ke(a)&&(this.La-=this.fb.get(a).length);this.fb.set(a,[b]);this.La+=1;return this};f.get=function(a,b){var c=a?this.Ib(a):[];return 0<c.length?String(c[0]):b};
f.toString=function(){if(this.Lc)return this.Lc;if(!this.fb)return"";for(var a=[],b=this.fb.uc(),c=0;c<b.length;c++)for(var d=b[c],e=encodeURIComponent(String(d)),d=this.Ib(d),g=0;g<d.length;g++){var h=e;""!==d[g]&&(h+="\x3d"+encodeURIComponent(String(d[g])));a.push(h)}return this.Lc=a.join("\x26")};f.clone=function(){var a=new Fk;a.Lc=this.Lc;this.fb&&(a.fb=this.fb.clone(),a.La=this.La);return a};function Pk(a,b){var c=String(b);a.Wc&&(c=c.toLowerCase());return c}
f.Ep=function(a){a&&!this.Wc&&(Ok(this),this.Lc=null,this.fb.forEach(function(a,c){var b=c.toLowerCase();c!=b&&(this.remove(c),this.remove(b),0<a.length&&(this.Lc=null,this.fb.set(Pk(this,b),qb(a)),this.La+=a.length))},this));this.Wc=a};f.extend=function(a){for(var b=0;b<arguments.length;b++)Kd(arguments[b],function(a,b){this.add(b,a)},this)};function Qk(){this.Ws=wa()}new Qk;Qk.prototype.set=function(a){this.Ws=a};Qk.prototype.reset=function(){this.set(wa())};Qk.prototype.get=function(){return this.Ws};function Rk(){md.call(this);this.Vo="closure_frame"+Sk++;this.oo=[];Tk[this.Vo]=this}var Uk;r(Rk,md);var Tk={},Sk=0;function Vk(a,b){var c=hf(a);Kd(b,function(b,e){la(b)||(b=[b]);db(b,function(b){b=c.D("INPUT",{type:"hidden",name:e,value:b});a.appendChild(b)})})}f=Rk.prototype;f.Ua=null;f.Wa=null;f.ag=null;f.ww=0;f.zb=!1;f.ci=!1;f.Al=!1;f.gg=null;f.Jk=null;f.$f=null;f.Gi=!1;
f.send=function(a,b,c,d){if(this.zb)throw Error("[goog.net.IframeIo] Unable to send, already active.");this.gg=a=new yk(a);b=b?b.toUpperCase():"GET";c&&(Ak(a),Mk(a,"zx",Math.floor(2147483648*Math.random()).toString(36)+Math.abs(Math.floor(2147483648*Math.random())^wa()).toString(36)));Uk||(Uk=xf("FORM"),Uk.acceptCharset="utf-8",c=Uk.style,c.position="absolute",c.visibility="hidden",c.top=c.left="-10px",c.width=c.height="10px",c.overflow="hidden",document.body.appendChild(Uk));this.Ua=Uk;"GET"==b&&
Vk(this.Ua,a.qb);d&&Vk(this.Ua,d);this.Ua.action=a.toString();this.Ua.method=b;Wk(this);Xk(this)};function Yk(a,b){if(a.zb)throw Error("[goog.net.IframeIo] Unable to send, already active.");var c=new yk(b.action);a.gg=c;a.Ua=b;a.Ua.action=c.toString();Wk(a)}f.abort=function(){if(this.zb){var a=Zk(this);fd(a);this.Al=this.zb=this.ci=!1;this.dispatchEvent("abort");$k(this)}};
f.V=function(){this.zb&&this.abort();Rk.m.V.call(this);this.Wa&&al(this);Xk(this);delete this.Ar;this.gg=this.Jk=this.Ua=null;delete Tk[this.Vo]};f.Dk=function(){return this.ci};f.Zr=function(){return this.Al};f.fg=function(){return this.zb};f.ti=function(){return this.Jk};f.Mc=function(){return qd(this.Jk)};
function Wk(a){a.zb=!0;a.ci=!1;a.ag=a.Vo+"_"+(a.ww++).toString(36);var b={name:a.ag,id:a.ag};t&&7>Number(oc)&&(b.src='javascript:""');a.Wa=hf(a.Ua).D("IFRAME",b);b=a.Wa.style;b.visibility="hidden";b.width=b.height="10px";b.display="none";kc?b.marginTop=b.marginLeft="-10px":(b.position="absolute",b.top=b.left="-10px");if(t&&!uc("11")){a.Ua.target=a.ag||"";hf(a.Ua).R.body.appendChild(a.Wa);a.Gi||v(a.Wa,"readystatechange",a.bp,!1,a);try{a.kn=!1,a.Ua.submit()}catch(W){a.Gi||dd(a.Wa,"readystatechange",
a.bp,!1,a),bl(a)}}else{hf(a.Ua).R.body.appendChild(a.Wa);var b=a.ag+"_inner",c=Jf(a.Wa);if(document.baseURI){var d=Ga(b);Be("Short HTML snippet, input escaped, safe URL, for performance");d='\x3chead\x3e\x3cbase href\x3d"'+Ga(document.baseURI)+'"\x3e\x3c/head\x3e\x3cbody\x3e\x3ciframe id\x3d"'+d+'" name\x3d"'+d+'"\x3e\x3c/iframe\x3e';d=Ve(d,null)}else d=Ga(b),Be("Short HTML snippet, input escaped, for performance"),d=Ve('\x3cbody\x3e\x3ciframe id\x3d"'+d+'" name\x3d"'+d+'"\x3e\x3c/iframe\x3e',null);
gc&&!kc?c.documentElement.innerHTML=Te(d):c.write(Te(d));a.Gi||v(c.getElementById(b),"load",a.Tk,!1,a);var e=mf("TEXTAREA",a.Ua);d=0;for(var g=e.length;d<g;d++){var h=e[d].value;Tf(e[d])!=h&&(Kf(e[d],h),e[d].value=h)}e=c.importNode(a.Ua,!0);e.target=b;e.action=a.Ua.action;c.body.appendChild(e);var h=mf("SELECT",a.Ua),k=mf("SELECT",e);d=0;for(g=h.length;d<g;d++)for(var l=mf("OPTION",h[d]),m=mf("OPTION",k[d]),p=0,u=l.length;p<u;p++)m[p].selected=l[p].selected;h=mf("INPUT",a.Ua);k=mf("INPUT",e);d=0;
for(g=h.length;d<g;d++)if("file"==h[d].type&&h[d].value!=k[d].value){a.Ua.target=b;e=a.Ua;break}try{a.kn=!1,e.submit(),c.close(),jc&&od(a.Kt,250,a)}catch(W){d=W;try{var A=ia("window.location.href");if(ea(d))var D={message:d,name:"Unknown error",lineNumber:"Not available",fileName:A,stack:"Not available"};else{var g=!1;try{var E=d.lineNumber||d.line||"Not available"}catch(fj){E="Not available",g=!0}try{var J=d.fileName||d.filename||d.sourceURL||ca.$googDebugFname||A}catch(fj){J="Not available",g=!0}D=
!g&&d.lineNumber&&d.fileName&&d.stack&&d.message&&d.name?d:{message:d.message||"Not available",name:d.name||"UnknownError",lineNumber:E,fileName:J,stack:d.stack||"Not available"}}var M=D.fileName;null!=M||(M="");if(/^https?:\/\//i.test(M)){var O=Pe(M);Be("view-source scheme plus HTTP/HTTPS URL");var aa="view-source:"+Ne(O);var ra=Qe(aa)}else{var Za=Be("sanitizedviewsrc");ra=Qe(Ae(Za))}var Vc=bf(We("Message: "+D.message+"\nUrl: "),$e("a",{href:ra,target:"_new"},D.fileName),We("\nLine: "+D.lineNumber+
"\n\nBrowser stack:\n"+D.stack+"-\x3e [end]\n\nJS stack traversal:\n"+xd(void 0)+"-\x3e "))}catch(fj){Vc=We("Exception trying to expose exception! You win, we lose. "+fj)}Te(Vc);a.Gi||dd(c.getElementById(b),"load",a.Tk,!1,a);c.close();bl(a)}}}f.bp=function(){if("complete"==this.Wa.readyState){dd(this.Wa,"readystatechange",this.bp,!1,this);try{var a=Jf(this.Wa);if(t&&"about:blank"==a.location&&!navigator.onLine){bl(this);return}}catch(b){bl(this);return}cl(this,a)}};
f.Tk=function(){if(!gc||kc||"about:blank"!=(this.Wa?Jf(Zk(this)):null).location){dd(Zk(this),"load",this.Tk,!1,this);try{cl(this,this.Wa?Jf(Zk(this)):null)}catch(a){bl(this)}}};function cl(a,b){a.ci=!0;a.zb=!1;try{var c=b.body;a.Jk=c.textContent||c.innerText}catch(e){var d=1}d||"function"!=typeof a.Ar||(c=a.Ar(b))&&(d=4);d?bl(a):(a.Al=!0,a.dispatchEvent("complete"),a.dispatchEvent("success"),$k(a))}
function bl(a){a.kn||(a.Al=!1,a.zb=!1,a.ci=!0,a.dispatchEvent("complete"),a.dispatchEvent("error"),$k(a),a.kn=!0)}function $k(a){al(a);Xk(a);a.Ua=null;a.dispatchEvent("ready")}function al(a){var b=a.Wa;b&&(b.onreadystatechange=null,b.onload=null,b.onerror=null,a.oo.push(b));a.$f&&(ca.clearTimeout(a.$f),a.$f=null);jc||gc&&!kc?a.$f=od(a.xr,2E3,a):a.xr();a.Wa=null;a.ag=null}f.xr=function(){this.$f&&(ca.clearTimeout(this.$f),this.$f=null);for(;0!=this.oo.length;){var a=this.oo.pop();Ff(a)}};
function Xk(a){a.Ua&&a.Ua==Uk&&Cf(a.Ua)}function Zk(a){return a.Wa?t&&!uc("11")?a.Wa:Jf(a.Wa).getElementById(a.ag+"_inner"):null}f.Kt=function(){if(this.zb){var a=this.Wa?Jf(Zk(this)):null;a&&!ec(a,"documentUri")?(this.Gi||dd(Zk(this),"load",this.Tk,!1,this),bl(this)):od(this.Kt,250,this)}};function dl(){}function el(a,b,c){var d,e;var g=null!=(d=a._rest_links)?null!=(e=d[b])?e.url:void 0:void 0;if(!g)throw Error("Given object is not a model");var h;if(h=!c,0<=cb(a._rest_links[b].methods,h))throw Error("Given model does not support "+c+" method");return g};function fl(a){var b;var c=this.Gq=a;for(b in c)a=c[b],this[b]=a}fl.prototype.toString=function(){if(null!=this._autocomplete_value)var a=this._autocomplete_value;else if(this._obj_name)a=this._obj_name;else a:{a=this.Gq;for(var b in a){a=a[b];break a}a=void 0}return a};function gl(){}gl.prototype.ic=function(a){if(la(a)){var b;var c=[];var d=0;for(b=a.length;d<b;d++){var e=a[d];c.push(this.Th(e))}a=c}else a=this.Th(a);return a};gl.prototype.Th=function(a){return new fl(a)};function hl(a,b){this.name=a;this.descending=null!=b?b:!1}hl.prototype.getName=function(){return this.name};hl.prototype.Aj=function(){return this.descending=!1};hl.prototype.dn=function(){return this.descending=!0};hl.prototype.toString=function(){return(this.descending?"-":"")+this.name};function il(){this.Ma=[]}f=il.prototype;f.add=function(a){return this.Ma.push(a)};f.remove=function(a){return ob(this.Ma,function(b){return b.getName()===a})};f.Aj=function(a){var b;(b=jl(this,a))&&b.Aj();if(b)return b.Aj();b=new hl(a,!1);return this.add(b)};f.dn=function(a){var b;if(b=jl(this,a))return b.dn();b=new hl(a,!0);return this.add(b)};function jl(a,b){return kb(a.Ma,function(a){return a.getName()===b})}f.toString=function(){return this.Ma.join(",")};function kl(a,b,c,d){this.name=a;this.Be=b;this.value=c;this.zo=d;null==this.zo&&(this.zo=!1)}f=kl.prototype;f.getName=function(){return this.name};f.We=function(){return this.Be};f.ba=function(){return this.value};f.lk=function(){return za(this.name,this.Be)?this.name:[this.name,this.Be].join("__")};f.qa=function(a){this.value=a};function ll(a){return null==a||0===a.length};function ml(a){null==a&&(a="");this.vl=new il;null==this.base&&(this.base=10);null==this.offset&&(this.offset=null);null==this.cursor&&(this.cursor=null);this.uri=new yk(a);this.Nf=["id"];this.zp=nl;this.accept="application/json";this.filters=new Fk;var b;a={};var c=this.uri.qb.uc();var d=0;for(b=c.length;d<b;d++){var e=c[d];a[e]=this.uri.qb.get(e);this.uri.removeParameter(e)}this.hi=a;this.el=!1;this}var nl="RAW";f=ml.prototype;f.rx=function(){return this.zp="VERBOSE"};
f.raw=function(){return this.zp=nl};f.Qm=function(){return this.uri.toString()};function ol(a,b){var c;a.filters.remove(b.getName());var d=a.uri.qb.uc();var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];var k=b.getName();0==h.lastIndexOf(k,0)?e.push(a.uri.removeParameter(h)):e.push(void 0)}}f.filter=function(a){a.ba()?(ol(this,a),this.filters.add(a.getName(),a),Mk(this.uri,a.lk(),a.ba())):ol(this,a);return this};f.Ve=function(a){return this.filters.Ib(a)};
function pl(a,b){var c=a.Ve(b.getName());c=b.zo?kb(c,function(a){return a.We()===b.We()}):0==c.length?null:c[0];if(!c&&(null==b.ba()||""===b.ba()))return!1;if(!c)return!0;var d=b.ba();return c.ba()===d&&c.We()===b.We()||void 0===c.ba()&&""===d?!1:!0}f.each=function(a){var b;var c=this.uri.qb;var d=c.uc();var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(a(h,c.Ib(h)))}return e};
f.sort=function(a,b){switch(b){case 1:return this.vl.Aj(a);case 2:return this.vl.dn(a);case 3:return this.vl.remove(a);default:throw Error("Given invalid sort direction "+b);}};function ql(a,b,c){this.Qb=a;this.context=null!=b?b:"";this.qc=null!=c?c:new je;this.headers={};this.headers["Content-Type"]=rl;this.headers.Accept=rl;this.ki=new dl;this.zd=new gl}var sl={Hu:"X-Total",qj:"X-Base",su:"X-Offset",ru:"X-Next-Offset",xu:"X-Prev-Offset",tu:"X-Order",yu:"X-Referer",vy:"X-Extra-Fields",fu:"X-Fields",Du:"X-Serialization-Format",Nz:"X-Requested-With",qu:"X-Next-Cursor",qq:"X-Cursor",zu:"X-Request-Count"},rl="application/json";
function tl(a,b){a.headers["X-Serialization-Format"]=b}function ul(a,b,c){null==c&&(c=null);null!=c&&cd(b,"error",function(){return a.xo(b)?c(b.Mc()):c(void 0)})}
ql.prototype.find=function(a,b,c,d){var e=this;null==c&&(c=null);null==d&&(d=null);var g=this.Qb.ic(this.qc);cd(g,"success",function(){if(null!=b){var a={total:parseInt(g.getResponseHeader(sl.Hu),10),lg:parseInt(g.getResponseHeader(sl.ru)),pg:parseInt(g.getResponseHeader(sl.xu)),Pc:g.getResponseHeader(sl.qu)};return b(e.zd.ic(g.Mc()),a)}});ul(this,g,c);c=null;null!=d&&(c={},Qb(c,this.headers),null!=d.base&&(c[sl.qj]=d.base),null!=d.offset&&(c[sl.su]=d.offset),c[sl.tu]=d.vl.toString(),null!=d.Vw&&
(c[sl.yu]=d.Vw),0<d.Nf.length&&(c[sl.fu]=d.Nf.join(",")),c[sl.Du]=d.zp,null!=d.cursor&&(c[sl.qq]=d.cursor),c.Accept=d.accept,d.el&&(c[sl.zu]=d.el));return this.send(vl(this,a,"api-"),"GET",g,null,c)};function vl(a,b,c){oa(b)?(a=[el(b,c+a.context,"GET")],a.push(b.id),b=a.join("/")):b=String(b);return b}
ql.prototype.get=function(a,b,c){var d=this;null==c&&(c=null);var e=this.Qb.ic(this.qc);cd(e,"success",function(){return b(d.zd.ic(e.Mc()))});ul(this,e,c);return this.send(vl(this,a,"api-resource-"),"GET",e)};ql.prototype["delete"]=function(a,b,c){null==b&&(b=null);null==c&&(c=null);var d=this.Qb.ic(this.qc);cd(d,"success",function(){if(null!=b)return b()});ul(this,d,c);a=el(a,"api-resource-"+this.context,"DELETE");return this.send(a,"DELETE",d)};f=ql.prototype;
f.create=function(a,b,c){null==b&&(b=null);null==c&&(c=null);var d=this.Qb.ic(this.qc);cd(d,"success",function(){if(null!=b)return b(d.Mc())});ul(this,d,c);c=el(a,"api-"+this.context,"POST");return this.send(c,"POST",d,rd(a))};f.update=function(a,b,c){null==b&&(b=null);null==c&&(c=null);var d=this.Qb.ic(this.qc);cd(d,"success",function(){if(null!=b)return b(d.Mc())});ul(this,d,c);c=el(a,"api-resource-"+this.context,"PUT");return this.send(""+c+"/"+a.id,"PUT",d,rd(a))};
f.request=function(a,b,c,d,e,g,h){var k=this;null==c&&(c={});null==d&&(d=null);null==e&&(e=null);null==g&&(g=!1);null==h&&(h={});var l=this.Qb.ic(this.qc);cd(l,"success",function(){if(d)return d(g?k.zd.ic(l.Mc()):wl(l))});ul(this,l,e);Qb(h,this.headers);return this.send(a,b,l,c,h)};function wl(a){var b=null;try{b=a.Mc()}catch(c){b=a.ti()}return b}
function xl(a,b,c){var d;null==d&&(d=null);var e=a.Qb.ic(a.qc);cd(e,"success",function(){var a=wl(e);return c(a)});ul(a,e,d);return a.send(b,"GET",e,null,{Accept:"text/html,application/json"})}f.send=function(a,b,c,d,e){null==c&&(c=null);null==d&&(d=null);null==e&&(e=null);oa(d)&&(d=rd(d),kb(Hb(e),this.uv)||(e["Content-Type"]=rl));null==c&&(c=this.Qb.ic(this.qc));return c.send(a,b,d,null!=e?e:this.headers)};f.uv=function(a){return"content-type"==a.toLowerCase()};
function yl(a,b,c,d,e){var g=a.Qb.ic(a.qc);g.No("success",function(){var b;if(null!=g.getResponseHeader("Location"))return a.reload(g.getResponseHeader("Location"));if(zl(g,"text/html"))return e(g.ti());try{return d(null!=(b=g.Mc())?b:{})}catch(k){return d({})}});ul(a,g,e);return a.send(b,"POST",g,c,{Accept:"text/html,application/json","Content-Type":"application/x-www-form-urlencoded","X-Requested-With":"XMLHttpRequest"})}f.reload=function(a){ne(this.Qb.l).location.assign(a);return ne(this.Qb.l).location.reload(!0)};
function Al(a,b,c,d){var e=new Rk;Yk(e,b);e.A("success",function(b){b=b.target.Mc();if(null!=b.location)return a.reload(b.location);c(b);return re(a.Qb,b,a.qc,200)});return e.A("error",function(a){return d(a.target.ti())})}f.xo=function(a){return zl(a,"application/json")};function zl(a,b){return a.getResponseHeader("Content-Type")?-1!==a.getResponseHeader("Content-Type").indexOf(b):!1};function Bl(a,b,c){md.call(this);this.target=a;this.handle=b||a;this.Mk=c||new $f(NaN,NaN,NaN,NaN);this.R=kf(a);this.Ub=new fh(this);$b(this,va(ac,this.Ub));this.deltaY=this.deltaX=this.Gt=this.Ft=this.screenY=this.screenX=this.clientY=this.clientX=0;this.Tb=!0;this.If=!1;this.Qs=!0;this.Tr=0;this.Kl=this.kw=!1;v(this.handle,["touchstart","mousedown"],this.Dt,!1,this)}r(Bl,md);var Cl=ca.document&&ca.document.documentElement&&!!ca.document.documentElement.setCapture&&!!ca.document.releaseCapture;
f=Bl.prototype;f.Va=function(){return this.Ub};f.pa=function(a){this.Tb=a};f.V=function(){Bl.m.V.call(this);dd(this.handle,["touchstart","mousedown"],this.Dt,!1,this);this.Ub.sd();Cl&&this.R.releaseCapture();this.handle=this.target=null};function Dl(a){da(a.Ld)||(a.Ld=vg(a.target));return a.Ld}
f.Dt=function(a){var b="mousedown"==a.type;if(!this.Tb||this.If||b&&!Jc(a))this.dispatchEvent("earlycancel");else{if(0==this.Tr)if(this.dispatchEvent(new El("start",this,a.clientX,a.clientY)))this.If=!0,this.Qs&&b&&a.preventDefault();else return;else this.Qs&&b&&a.preventDefault();var b=this.R,c=b.documentElement,d=!Cl;this.Ub.A(b,["touchmove","mousemove"],this.Yv,d);this.Ub.A(b,["touchend","mouseup"],this.dk,d);Cl?(c.setCapture(!1),this.Ub.A(c,"losecapture",this.dk)):this.Ub.A(wf(b),"blur",this.dk);
t&&this.kw&&this.Ub.A(b,"dragstart",Dc);this.$w&&this.Ub.A(this.$w,"scroll",this.Fw,d);this.clientX=this.Ft=a.clientX;this.clientY=this.Gt=a.clientY;this.screenX=a.screenX;this.screenY=a.screenY;this.Kl?(a=this.target,c=a.offsetLeft,d=a.offsetParent,d||"fixed"!=fg(a)||(d=kf(a).documentElement),d?(jc?(b=qg(d),c+=b.left):vc(8)&&!vc(9)&&(b=qg(d),c-=b.left),a=vg(d)?d.clientWidth-(c+a.offsetWidth):c):a=c):a=this.target.offsetLeft;this.deltaX=a;this.deltaY=this.target.offsetTop;this.cp=uf(hf(this.R).R)}};
f.dk=function(a){this.Ub.sd();Cl&&this.R.releaseCapture();this.If?(this.If=!1,this.dispatchEvent(new El("end",this,a.clientX,a.clientY,0,Fl(this,this.deltaX),Gl(this,this.deltaY)))):this.dispatchEvent("earlycancel")};
f.Yv=function(a){if(this.Tb){var b=(this.Kl&&Dl(this)?-1:1)*(a.clientX-this.clientX),c=a.clientY-this.clientY;this.clientX=a.clientX;this.clientY=a.clientY;this.screenX=a.screenX;this.screenY=a.screenY;if(!this.If){var d=this.Ft-this.clientX,e=this.Gt-this.clientY;if(d*d+e*e>this.Tr)if(this.dispatchEvent(new El("start",this,a.clientX,a.clientY)))this.If=!0;else{this.Re||this.dk(a);return}}c=Hl(this,b,c);b=c.x;c=c.y;this.If&&this.dispatchEvent(new El("beforedrag",this,a.clientX,a.clientY,0,b,c))&&
(Il(this,a,b,c),a.preventDefault())}};function Hl(a,b,c){var d=uf(hf(a.R).R);b+=d.x-a.cp.x;c+=d.y-a.cp.y;a.cp=d;a.deltaX+=b;a.deltaY+=c;return new df(Fl(a,a.deltaX),Gl(a,a.deltaY))}f.Fw=function(a){var b=Hl(this,0,0);a.clientX=this.clientX;a.clientY=this.clientY;Il(this,a,b.x,b.y)};function Il(a,b,c,d){a.Kl&&Dl(a)?a.target.style.right=c+"px":a.target.style.left=c+"px";a.target.style.top=d+"px";a.dispatchEvent(new El("drag",a,b.clientX,b.clientY,0,c,d))}
function Fl(a,b){var c=a.Mk,d=isNaN(c.left)?null:c.left,c=isNaN(c.width)?0:c.width;return Math.min(null!=d?d+c:Infinity,Math.max(null!=d?d:-Infinity,b))}function Gl(a,b){var c=a.Mk,d=isNaN(c.top)?null:c.top,c=isNaN(c.height)?0:c.height;return Math.min(null!=d?d+c:Infinity,Math.max(null!=d?d:-Infinity,b))}function El(a,b,c,d,e,g,h){Cc.call(this,a);this.clientX=c;this.clientY=d;this.left=da(g)?g:b.deltaX;this.top=da(h)?h:b.deltaY}r(El,Cc);(function(){for(var a=["ms","moz","webkit","o"],b=0,c;c=a[b]&&!ca.requestAnimationFrame;++b)ca.requestAnimationFrame=ca[c+"RequestAnimationFrame"],ca.cancelAnimationFrame=ca[c+"CancelAnimationFrame"]||ca[c+"CancelRequestAnimationFrame"];if(!ca.requestAnimationFrame){var d=0;ca.requestAnimationFrame=function(a){var b=(new Date).getTime(),c=Math.max(0,16-(b-d));d=b+c;return ca.setTimeout(function(){a(b+c)},c)};ca.cancelAnimationFrame||(ca.cancelAnimationFrame=function(a){clearTimeout(a)})}})();
var Jl=[[],[]],Kl=0,Ll=!1,Ml=0;function Nl(a,b){var c=Ml++,d={rw:{id:c,fn:a.measure,context:b},uw:{id:c,fn:a.tw,context:b},state:{},Ab:void 0,Gk:!1};return function(){0<arguments.length?(d.Ab||(d.Ab=[]),d.Ab.length=0,d.Ab.push.apply(d.Ab,arguments),d.Ab.push(d.state)):d.Ab&&0!=d.Ab.length?(d.Ab[0]=d.state,d.Ab.length=1):d.Ab=[d.state];d.Gk||(d.Gk=!0,Jl[Kl].push(d));Ll||(Ll=!0,window.requestAnimationFrame(Ol))}}
function Ol(){Ll=!1;var a=Jl[Kl],b=a.length;Kl=(Kl+1)%2;for(var c,d=0;d<b;++d){c=a[d];var e=c.rw;c.Gk=!1;e.fn&&e.fn.apply(e.context,c.Ab)}for(d=0;d<b;++d)c=a[d],e=c.uw,c.Gk=!1,e.fn&&e.fn.apply(e.context,c.Ab),c.state={};a.length=0};var Pl=t?Ke(Ae(Be('javascript:""'))):Ke(Ae(Be("about:blank"))),Ql=Ie(Pl),Rl=t?Ke(Ae(Be('javascript:""'))):Ke(Ae(Be("javascript:undefined")));Ie(Rl);function Vl(a){md.call(this);this.F=a;a=t?"focusout":"blur";this.lw=v(this.F,t?"focusin":"focus",this,!t);this.mw=v(this.F,a,this,!t)}r(Vl,md);Vl.prototype.handleEvent=function(a){var b=new Gc(a.bd);b.type="focusin"==a.type||"focus"==a.type?"focusin":"focusout";this.dispatchEvent(b)};Vl.prototype.V=function(){Vl.m.V.call(this);ed(this.lw);ed(this.mw);delete this.F};function Wl(a,b){this.F=a;this.S=b};function Xl(a,b){kh.call(this,b);this.px=!!a;this.gh=null;this.at=Nl({tw:this.fl},this)}r(Xl,kh);f=Xl.prototype;f.qn=null;f.Oa=!1;f.Bc=null;f.hc=null;f.vd=null;f.wm=!1;f.ta=function(){return"goog-modalpopup"};f.ik=function(){return this.Bc};f.D=function(){Xl.m.D.call(this);var a=this.o(),b=Da(this.ta()).split(" ");li(a,b);Qf(a,!0);w(a,!1);Yl(this);Zl(this)};
function Yl(a){if(a.px&&!a.hc){var b=a.I().D("IFRAME",{frameborder:0,style:"border:0;vertical-align:bottom;",src:Ql});a.hc=b;a.hc.className=a.ta()+"-bg";w(a.hc,!1);ug(a.hc,0)}a.Bc||(a.Bc=a.I().D("DIV",a.ta()+"-bg"),w(a.Bc,!1))}function Zl(a){a.vd||(a.vd=a.I().createElement("SPAN"),w(a.vd,!1),Qf(a.vd,!0),a.vd.style.position="absolute")}f.Zs=function(){this.wm=!1};f.rb=function(a){return!!a&&"DIV"==a.tagName};
f.Ca=function(a){Xl.m.Ca.call(this,a);a=Da(this.ta()).split(" ");li(this.o(),a);Yl(this);Zl(this);Qf(this.o(),!0);w(this.o(),!1)};f.Aa=function(){this.hc&&Df(this.hc,this.o());Df(this.Bc,this.o());Xl.m.Aa.call(this);Ef(this.vd,this.o());this.qn=new Vl(this.I().R);this.Va().A(this.qn,"focusin",this.onFocus);$l(this,!1)};f.lb=function(){this.va()&&this.$(!1);ac(this.qn);Xl.m.lb.call(this);Ff(this.hc);Ff(this.Bc);Ff(this.vd)};
f.$=function(a){a!=this.Oa&&(this.ph&&this.ph.stop(),this.Sh&&this.Sh.stop(),this.oh&&this.oh.stop(),this.Rh&&this.Rh.stop(),this.la&&$l(this,a),a?this.Lp():this.$g())};
function $l(a,b){a.os||(a.os=new Wl(a.F,a.S));var c=a.os;if(b){c.Zg||(c.Zg=[]);for(var d=c.S.se(c.S.R.body),e=0;e<d.length;e++){var g=d[e],h;if(h=g!=c.F)h=g.getAttribute("aria-hidden"),h=!(null==h||void 0==h?0:String(h));h&&(gi(g,"hidden",!0),c.Zg.push(g))}}else if(c.Zg){for(e=0;e<c.Zg.length;e++)c.Zg[e].removeAttribute("aria-hidden");c.Zg=null}}
f.Lp=function(){if(this.dispatchEvent("beforeshow")){try{this.gh=this.I().R.activeElement}catch(a){}this.fl();this.yc();this.Va().A(ne(this.I()),"resize",this.fl).A(ne(this.I()),"orientationchange",this.at);am(this,!0);this.focus();this.Oa=!0;this.ph&&this.Sh?(cd(this.ph,"end",this.ng,!1,this),this.Sh.play(),this.ph.play()):this.ng()}};
f.$g=function(){if(this.dispatchEvent("beforehide")){this.Va().Ba(ne(this.I()),"resize",this.fl).Ba(ne(this.I()),"orientationchange",this.at);this.Oa=!1;this.oh&&this.Rh?(cd(this.oh,"end",this.jf,!1,this),this.Rh.play(),this.oh.play()):this.jf();a:{try{var a=this.I(),b=a.R.body,c=a.R.activeElement||b;if(!this.gh||this.gh==b){this.gh=null;break a}(c==b||a.contains(this.o(),c))&&this.gh.focus()}catch(d){}this.gh=null}}};function am(a,b){a.hc&&w(a.hc,b);a.Bc&&w(a.Bc,b);w(a.o(),b);w(a.vd,b)}f.ng=function(){this.dispatchEvent("show")};
f.jf=function(){am(this,!1);this.dispatchEvent("hide")};f.va=function(){return this.Oa};f.focus=function(){this.Fr()};f.fl=function(){this.hc&&w(this.hc,!1);this.Bc&&w(this.Bc,!1);var a=this.I().R,b=sf(wf(a)||window||window),c=Math.max(b.width,Math.max(a.body.scrollWidth,a.documentElement.scrollWidth)),a=Math.max(b.height,Math.max(a.body.scrollHeight,a.documentElement.scrollHeight));this.hc&&(w(this.hc,!0),sg(this.hc,c,a));this.Bc&&(w(this.Bc,!0),sg(this.Bc,c,a))};
f.yc=function(){var a;var b=this.I().R;var c=wf(b)||window;"fixed"==fg(this.o())?b=a=0:(b=uf(this.I().R),a=b.x,b=b.y);var d=tg(this.o()),c=sf(c||window);a=Math.max(a+c.width/2-d.width/2,0);b=Math.max(b+c.height/2-d.height/2,0);gg(this.o(),a,b);gg(this.vd,a,b)};f.onFocus=function(a){this.wm?this.Zs():a.target==this.vd&&od(this.Fr,0,this)};f.Fr=function(){try{t&&this.I().R.body.focus(),this.o().focus()}catch(a){}};
f.V=function(){ac(this.ph);this.ph=null;ac(this.oh);this.oh=null;ac(this.Sh);this.Sh=null;ac(this.Rh);this.Rh=null;Xl.m.V.call(this)};function bm(a,b,c){Xl.call(this,b,c);this.kc=a||"modal-dialog";this.kb=cm(cm(new dm,em,!0),fm,!1,!0)}r(bm,Xl);f=bm.prototype;f.Ev=!0;f.xk=!0;f.ps=!0;f.Bv=!0;f.Qh=.5;f.mj="";f.ab=null;f.Se=null;f.zv=!1;f.Pb=null;f.fc=null;f.Dl=null;f.Ac=null;f.cc=null;f.jb=null;f.kp="dialog";f.ta=function(){return this.kc};f.dj=function(a){this.mj=a;this.fc&&Kf(this.fc,a)};f.Kr=function(){return this.mj};f.Yc=function(a){a instanceof Re||(a=Ue(a),a=Ve(Fa(Te(a)),a.te()));this.ab=a;this.cc&&(this.cc.innerHTML=Te(a))};
f.Fd=function(){return null!=this.ab?Te(this.ab):""};f.ue=function(){return this.kp};f.ub=function(){this.o()||this.render();return this.cc};function gm(a){a.o()||a.render();return a.o()}f.ik=function(){this.o()||this.render();return bm.m.ik.call(this)};function hm(a,b){a.Qh=b;if(a.o()){var c=a.ik();c&&ug(c,a.Qh)}}f.un=function(){return this.ta()};
function im(a,b){var c=Da(a.kc+"-title-draggable").split(" ");a.o()&&(b?li(a.Pb,c):ni(a.Pb,c));if(b&&!a.Se){var d=new Bl(a.o(),a.Pb);a.Se=d;li(a.Pb,c);v(a.Se,"start",a.dx,!1,a)}else!b&&a.Se&&(a.Se.ib(),a.Se=null)}
f.D=function(){bm.m.D.call(this);var a=this.o(),b=this.I();this.Pb=b.D("DIV",this.kc+"-title",this.fc=b.D("SPAN",{className:this.kc+"-title-text",id:nh(this)},this.mj),this.Ac=b.D("SPAN",this.kc+"-title-close"));Bf(a,this.Pb,this.cc=b.D("DIV",this.kc+"-content"),this.jb=b.D("DIV",this.kc+"-buttons"));fi(this.fc,"heading");fi(this.Ac,"button");Qf(this.Ac,!0);gi(this.Ac,"label",jm);this.Dl=this.fc.id;fi(a,this.ue());gi(a,"labelledby",this.Dl||"");this.ab&&(this.cc.innerHTML=Te(this.ab));w(this.Ac,this.xk);
this.kb&&(a=this.kb,a.F=this.jb,a.render());w(this.jb,!!this.kb);hm(this,this.Qh)};
f.Ca=function(a){bm.m.Ca.call(this,a);a=this.o();var b=this.kc+"-content";this.cc=of(document,null,b,a)[0];this.cc||(this.cc=this.I().D("DIV",b),this.ab&&(this.cc.innerHTML=Te(this.ab)),a.appendChild(this.cc));var b=this.kc+"-title",c=this.kc+"-title-text",d=this.kc+"-title-close";(this.Pb=of(document,null,b,a)[0])?(this.fc=of(document,null,c,this.Pb)[0],this.Ac=of(document,null,d,this.Pb)[0]):(this.Pb=this.I().D("DIV",b),a.insertBefore(this.Pb,this.cc));this.fc?(this.mj=Rf(this.fc),this.fc.id||(this.fc.id=
nh(this))):(this.fc=xf("SPAN",{className:c,id:nh(this)}),this.Pb.appendChild(this.fc));this.Dl=this.fc.id;gi(a,"labelledby",this.Dl||"");this.Ac||(this.Ac=this.I().D("SPAN",d),this.Pb.appendChild(this.Ac));w(this.Ac,this.xk);b=this.kc+"-buttons";(this.jb=of(document,null,b,a)[0])?(this.kb=new dm(this.I()),this.kb.J(this.jb)):(this.jb=this.I().D("DIV",b),a.appendChild(this.jb),this.kb&&(a=this.kb,a.F=this.jb,a.render()),w(this.jb,!!this.kb));hm(this,this.Qh)};
f.Aa=function(){bm.m.Aa.call(this);this.Va().A(this.o(),"keydown",this.Ri).A(this.o(),"keypress",this.Ri);this.Va().A(this.jb,"click",this.zw);im(this,this.Bv);this.Va().A(this.Ac,"click",this.Hw);var a=this.o();fi(a,this.ue());""!==this.fc.id&&gi(a,"labelledby",this.fc.id);if(!this.ps){this.ps=!1;if(this.la){var a=this.I(),b=this.ik();a.removeNode(this.hc);a.removeNode(b)}this.va()&&$l(this,!1)}};f.lb=function(){this.va()&&this.$(!1);im(this,!1);bm.m.lb.call(this)};
f.$=function(a){a!=this.va()&&(this.la||this.render(),bm.m.$.call(this,a))};f.ng=function(){bm.m.ng.call(this);this.dispatchEvent(km)};f.jf=function(){bm.m.jf.call(this);this.dispatchEvent(lm);this.zv&&this.ib()};
f.dx=function(){var a=this.I().R,b=sf(wf(a)||window||window),c=Math.max(a.body.scrollWidth,b.width),a=Math.max(a.body.scrollHeight,b.height),d=tg(this.o());"fixed"==fg(this.o())?this.Se.Mk=new $f(0,0,Math.max(0,b.width-d.width),Math.max(0,b.height-d.height)):this.Se.Mk=new $f(0,0,c-d.width,a-d.height)};f.Hw=function(){mm(this)};function mm(a){if(a.xk){var b=a.kb,c=b&&b.Fm;c?(b=b.get(c),a.dispatchEvent(new nm(c,b))&&a.$(!1)):a.$(!1)}}f.V=function(){this.jb=this.Ac=null;bm.m.V.call(this)};
function om(a,b){a.kb=b;if(a.jb){if(a.kb){var c=a.kb;c.F=a.jb;c.render()}else a.jb.innerHTML=Te(cf);w(a.jb,!!a.kb)}}f.zw=function(a){a:{for(a=a.target;null!=a&&a!=this.jb;){if("BUTTON"==a.tagName)break a;a=a.parentNode}a=null}if(a&&!a.disabled){a=a.name;var b=this.kb.get(a);this.dispatchEvent(new nm(a,b))&&this.$(!1)}};
f.Ri=function(a){var b=!1,c=!1,d=this.kb;var e=a.target;if("keydown"==a.type)if(this.Ev&&27==a.keyCode){var g=d&&d.Fm;e="SELECT"==e.tagName&&!e.disabled;g&&!e?(c=!0,b=d.get(g),b=this.dispatchEvent(new nm(g,b))):e||(b=!0)}else{if(9==a.keyCode&&a.shiftKey&&e==this.o()){this.wm=!0;try{this.vd.focus()}catch(p){}od(this.Zs,0,this)}}else if(13==a.keyCode){if("BUTTON"==e.tagName&&!e.disabled)g=e.name;else if(e==this.Ac)mm(this);else if(d){var h=d.Xj,k;if(k=h)a:{k=mf("BUTTON",d.F);for(var l=0,m;m=k[l];l++)if(m.name==
h||m.id==h){k=m;break a}k=null}e=("TEXTAREA"==e.tagName||"SELECT"==e.tagName||"A"==e.tagName)&&!e.disabled;!k||k.disabled||e||(g=h)}g&&d&&(c=!0,b=this.dispatchEvent(new nm(g,String(d.get(g)))))}else e==this.Ac&&32==a.keyCode&&mm(this);if(b||c)a.stopPropagation(),a.preventDefault();b&&this.$(!1)};function nm(a,b){this.type=pm;this.key=a;this.caption=b}r(nm,Cc);var pm="dialogselect",lm="afterhide",km="aftershow";function dm(a){this.S=a||hf();Pd.call(this)}r(dm,Pd);f=dm.prototype;f.kc="goog-buttonset";
f.Xj=null;f.F=null;f.Fm=null;f.set=function(a,b,c,d){Pd.prototype.set.call(this,a,b);c&&(this.Xj=a);d&&(this.Fm=a);return this};function cm(a,b,c,d){return a.set(b.key,b.caption,c,d)}f.render=function(){if(this.F){this.F.innerHTML=Te(cf);var a=hf(this.F);this.forEach(function(b,c){var d=a.D("BUTTON",{name:c},b);c==this.Xj&&(d.className=this.kc+"-default");this.F.appendChild(d)},this)}};
f.J=function(a){if(a&&1==a.nodeType){this.F=a;a=mf("BUTTON",this.F);for(var b=0,c,d,e;c=a[b];b++)if(d=c.name||c.id,e=Rf(c)||c.value,d){var g=0==b;this.set(d,e,g,c.name==qm);g&&ki(c,this.kc+"-default")}}};f.o=function(){return this.F};f.I=function(){return this.S};var qm="cancel",jm="Close",em={key:"ok",caption:"OK"},fm={key:qm,caption:"Cancel"},rm={key:"yes",caption:"Yes"},sm={key:"no",caption:"No"},tm={key:"save",caption:"Save"},um={key:"continue",caption:"Continue"};
"undefined"!=typeof document&&(cm(new dm,em,!0,!0),cm(cm(new dm,em,!0),fm,!1,!0),cm(cm(new dm,rm,!0),sm,!1,!0),cm(cm(cm(new dm,rm),sm,!0),fm,!1,!0),cm(cm(cm(new dm,um),tm),fm,!0,!0));function vm(){}r(vm,oi);ka(vm);f=vm.prototype;f.Qf=function(){return"button"};f.Od=function(a,b,c){switch(b){case 8:case 16:gi(a,"pressed",c);break;default:case 64:case 1:vm.m.Od.call(this,a,b,c)}};f.D=function(a){var b=vm.m.D.call(this,a);this.rl(b,a.rk());var c=a.ba();c&&this.qa(b,c);vi(a,16)&&this.Od(b,16,!!(a.ma&16));return b};f.J=function(a,b){b=vm.m.J.call(this,a,b);var c=this.ba(b);a.cq=c;a.Xp=this.rk(b);vi(a,16)&&this.Od(b,16,!!(a.ma&16));return b};f.ba=ja;f.qa=ja;f.rk=function(a){return a.title};
f.rl=function(a,b){a&&(b?a.title=b:a.removeAttribute("title"))};f.ta=function(){return"goog-button"};function wm(){}r(wm,vm);ka(wm);f=wm.prototype;f.Qf=function(){};f.D=function(a){Bi(a,!1);a.Ph&=-256;a.pc(32,!1);return a.I().D("BUTTON",{"class":this.fd(a).join(" "),disabled:!a.isEnabled(),title:a.rk()||"",value:a.ba()||""},a.qe()||"")};f.rb=function(a){return"BUTTON"==a.tagName||"INPUT"==a.tagName&&("button"==a.type||"submit"==a.type||"reset"==a.type)};f.J=function(a,b){Bi(a,!1);a.Ph&=-256;a.pc(32,!1);if(b.disabled){var c=this.ri(1);ki(b,c)}return wm.m.J.call(this,a,b)};
f.dg=function(a){a.Va().A(a.o(),"click",a.nh)};f.vh=ja;f.yh=ja;f.ye=function(a){return a.isEnabled()};f.He=ja;f.setState=function(a,b,c){wm.m.setState.call(this,a,b,c);(a=a.o())&&1==b&&(a.disabled=c)};f.ba=function(a){return a.value};f.qa=function(a,b){a&&(a.value=b)};f.Od=ja;function xm(a,b,c){Ai.call(this,a,b||wm.ua(),c)}r(xm,Ai);f=xm.prototype;f.ba=function(){return this.cq};f.qa=function(a){this.cq=a;this.W.qa(this.o(),a)};f.rk=function(){return this.Xp};f.rl=function(a){this.Xp=a;this.W.rl(this.o(),a)};f.V=function(){xm.m.V.call(this);delete this.cq;delete this.Xp};f.Aa=function(){xm.m.Aa.call(this);if(vi(this,32)){var a=this.Hb();a&&this.Va().A(a,"keyup",this.we)}};
f.we=function(a){return 13==a.keyCode&&"key"==a.type||32==a.keyCode&&"keyup"==a.type?this.nh(a):32==a.keyCode};xi("goog-button",function(){return new xm(null)});function ym(a,b){var c=Array.prototype.slice.call(arguments),d=c.shift();if("undefined"==typeof d)throw Error("[goog.string.format] Template required");return d.replace(/%([0\-\ \+]*)(\d+)?(\.(\d+))?([%sfdiu])/g,function(a,b,d,k,l,m,p,u){if("%"==m)return"%";var e=c.shift();if("undefined"==typeof e)throw Error("[goog.string.format] Not enough arguments");arguments[0]=e;return ym.ne[m].apply(null,arguments)})}ym.ne={};
ym.ne.s=function(a,b,c){return isNaN(c)||""==c||a.length>=Number(c)?a:a=-1<b.indexOf("-",0)?a+Ua(" ",Number(c)-a.length):Ua(" ",Number(c)-a.length)+a};
ym.ne.f=function(a,b,c,d,e){d=a.toString();isNaN(e)||""==e||(d=parseFloat(a).toFixed(e));var g=0>Number(a)?"-":0<=b.indexOf("+")?"+":0<=b.indexOf(" ")?" ":"";0<=Number(a)&&(d=g+d);if(isNaN(c)||d.length>=Number(c))return d;d=isNaN(e)?Math.abs(Number(a)).toString():Math.abs(Number(a)).toFixed(e);a=Number(c)-d.length-g.length;0<=b.indexOf("-",0)?d=g+d+Ua(" ",a):(b=0<=b.indexOf("0",0)?"0":" ",d=g+Ua(b,a)+d);return d};ym.ne.d=function(a,b,c,d,e,g,h,k){return ym.ne.f(parseInt(a,10),b,c,d,0,g,h,k)};
ym.ne.i=ym.ne.d;ym.ne.u=ym.ne.d;function zm(a){var b=["date","datetime"],c;var d=0;for(c=b.length;d<c;d++){var e=b[d];if(dh(a,e))return!0}return!1};function Am(a,b){var c,d;null==c&&(c=500);if(zm(a)||"date"===(d=a.type)||"datetime"===d){var e=!1;v(a,"change",function(a){e||b(a);return e=!1});return v(a,"keyup",function(a){b(a);return e=!0})}var g=null;var h=v(a,"change",function(a){g&&g.stop();return b(a)});return v(a,"keyup",function(a){g&&g.stop();g=new Kg(function(){b(a);return ed(h)},c);return g.start()})}function Bm(a,b){return v(a,"keypress",function(a){if(13===a.keyCode)return b(a)})}function Cm(a,b){v(a,"click",b);Bm(a,b)};function Dm(a,b,c){a.push(encodeURIComponent(b)+"\x3d"+encodeURIComponent(c))}function Em(a){var b=a.type;if(!da(b))return null;switch(b.toLowerCase()){case "checkbox":case "radio":return a.checked?a.value:null;case "select-one":return b=a.selectedIndex,0<=b?a.options[b].value:null;case "select-multiple":for(var b=[],c,d=0;c=a.options[d];d++)c.selected&&b.push(c.value);return b.length?b:null;default:return da(a.value)?a.value:null}}
function Fm(a,b){var c=a.type;if(da(c))switch(c.toLowerCase()){case "checkbox":case "radio":a.checked=b;break;case "select-one":a.selectedIndex=-1;if(ea(b))for(var d=0;c=a.options[d];d++)if(c.value==b){c.selected=!0;break}break;case "select-multiple":c=b;ea(c)&&(c=[c]);for(var e=0;d=a.options[e];e++)if(d.selected=!1,c)for(var g,h=0;g=c[h];h++)d.value==g&&(d.selected=!0);break;default:a.value=null!=b?b:""}};function Gm(a){return 2>a.length?a:Array.prototype.slice.call(a,0,a.length-1)}function Hm(a,b){null==b&&(b=null);return 0==a.length?b:a[a.length-1]}function Im(a,b,c){null==c&&(c=null);return 0==a.length?c:b(a[0])?a[0]:Im(sb(a,1),b,c)}function Jm(a,b){return la(a)?fb(a,b,void 0):[]}function Km(a,b){var c={};return la(a)?hb(a,b,c,void 0):[]}function Lm(a){return la(a)?0==a.length:!0};function Mm(a,b,c,d){null==c&&(c=ab);null==d&&(d=null);return Th(a,b)?c(Rh(a,b)):d};function Nm(a,b){null==b&&(b=null);var c=parseInt(a,10);isFinite(c)||null==b||(c=b);return c}function Om(a){return 0<a}function Pm(a,b){var c=a[0];var d=a[1];return c<=b&&b<=d};function Qm(a,b){try{return qd(a)}catch(c){return na(b)?b(a):null}};function Rm(a){this.Qq=a}ka(Rm);f=Rm.prototype;f.Qf=function(){return this.Qq};function Sm(a,b){a&&(a.tabIndex=b?0:-1)}f.D=function(a){return a.I().D("DIV",this.fd(a).join(" "))};f.ub=function(a){return a};f.rb=function(a){return"DIV"==a.tagName};f.J=function(a,b){b.id&&oh(a,b.id);var c=this.ta(),d=!1,e=ii(b);e&&db(e,function(b){b==c?d=!0:b&&(b==c+"-disabled"?a.pa(!1):b==c+"-horizontal"?a.setOrientation(Tm):b==c+"-vertical"&&a.setOrientation(Um))},this);d||ki(b,c);Vm(this,a,this.ub(b));return b};
function Vm(a,b,c){if(c)for(var d=c.firstChild,e;d&&d.parentNode==c;){e=d.nextSibling;if(1==d.nodeType){var g=a.wn(d);g&&(g.F=d,b.isEnabled()||g.pa(!1),b.$a(g),g.J(d))}else d.nodeValue&&""!=Da(d.nodeValue)||c.removeChild(d);d=e}}f.wn=function(a){a:{a=ii(a);for(var b=0,c=a.length;b<c;b++){var d=a[b];if(d=d in yi?yi[d]():null){a=d;break a}}a=null}return a};f.dg=function(a){a=a.o();xg(a,!0,jc);t&&(a.hideFocus=!0);var b=this.Qf();b&&fi(a,b)};f.Hb=function(a){return a.o()};f.ta=function(){return"goog-container"};
f.fd=function(a){var b=this.ta(),c=[b,a.og==Tm?b+"-horizontal":b+"-vertical"];a.isEnabled()||c.push(b+"-disabled");return c};function Wm(a,b,c){kh.call(this,c);this.W=b||Rm.ua();this.og=a||Um}r(Wm,kh);var Tm="horizontal",Um="vertical";f=Wm.prototype;f.Go=null;f.Yb=null;f.W=null;f.og=null;f.Oa=!0;f.Tb=!0;f.rn=!0;f.eb=-1;f.Mb=null;f.kg=!1;f.Ou=!1;f.Iw=!0;f.Sd=null;f.Hb=function(){return this.Go||this.W.Hb(this)};f.kk=function(){return this.Yb||(this.Yb=new Yh(this.Hb()))};f.D=function(){this.F=this.W.D(this)};f.ub=function(){return this.W.ub(this.o())};f.rb=function(a){return this.W.rb(a)};
f.Ca=function(a){this.F=this.W.J(this,a);"none"==a.style.display&&(this.Oa=!1)};f.Aa=function(){Wm.m.Aa.call(this);rh(this,function(a){a.la&&Xm(this,a)},this);var a=this.o();this.W.dg(this);this.$(this.Oa,!0);this.Va().A(this,"enter",this.Nn).A(this,"highlight",this.Rv).A(this,"unhighlight",this.$v).A(this,"open",this.Zv).A(this,"close",this.Nv).A(a,"mousedown",this.xe).A(kf(a),"mouseup",this.Ov).A(a,["mousedown","mouseup","mouseover","mouseout","contextmenu"],this.Mv);this.ye()&&Ym(this,!0)};
function Ym(a,b){var c=a.Va(),d=a.Hb();b?c.A(d,"focus",a.handleFocus).A(d,"blur",a.handleBlur).A(a.kk(),"key",a.gd):c.Ba(d,"focus",a.handleFocus).Ba(d,"blur",a.handleBlur).Ba(a.kk(),"key",a.gd)}f.lb=function(){this.xh(-1);this.Mb&&this.Mb.setOpen(!1);this.kg=!1;Wm.m.lb.call(this)};f.V=function(){Wm.m.V.call(this);this.Yb&&(this.Yb.ib(),this.Yb=null);this.W=this.Mb=this.Sd=this.Go=null};f.Nn=function(){return!0};
f.Rv=function(a){var b=vh(this,a.target);if(-1<b&&b!=this.eb){var c=th(this,this.eb);c&&c.ud(!1);this.eb=b;c=th(this,this.eb);this.kg&&c.setActive(!0);this.Iw&&this.Mb&&c!=this.Mb&&(vi(c,64)?c.setOpen(!0):this.Mb.setOpen(!1))}b=this.o();null!=a.target.o()&&gi(b,"activedescendant",a.target.o().id)};f.$v=function(a){a.target==th(this,this.eb)&&(this.eb=-1);this.o().removeAttribute("aria-activedescendant")};
f.Zv=function(a){(a=a.target)&&a!=this.Mb&&a.getParent()==this&&(this.Mb&&this.Mb.setOpen(!1),this.Mb=a)};f.Nv=function(a){a.target==this.Mb&&(this.Mb=null);var b=this.o(),c=a.target.o();b&&a.target.ma&2&&c&&hi(b,c)};f.xe=function(a){this.Tb&&(this.kg=!0);var b=this.Hb();b&&Nf(b)?b.focus():a.preventDefault()};f.Ov=function(){this.kg=!1};
f.Mv=function(a){a:{var b=a.target;if(this.Sd)for(var c=this.o();b&&b!==c;){var d=b.id;if(d in this.Sd){b=this.Sd[d];break a}b=b.parentNode}b=null}if(b)switch(a.type){case "mousedown":b.xe(a);break;case "mouseup":b.Xg(a);break;case "mouseover":b.Wg(a);break;case "mouseout":b.Xn(a);break;case "contextmenu":b.wi(a)}};f.handleFocus=function(){};f.handleBlur=function(){this.xh(-1);this.kg=!1;this.Mb&&this.Mb.setOpen(!1)};
f.gd=function(a){return this.isEnabled()&&this.va()&&(0!=sh(this)||this.Go)&&this.we(a)?(a.preventDefault(),a.stopPropagation(),!0):!1};
f.we=function(a){var b=th(this,this.eb);if(b&&"function"==typeof b.gd&&b.gd(a)||this.Mb&&this.Mb!=b&&"function"==typeof this.Mb.gd&&this.Mb.gd(a))return!0;if(a.shiftKey||a.ctrlKey||a.metaKey||a.altKey)return!1;switch(a.keyCode){case 27:if(this.ye())this.Hb().blur();else return!1;break;case 36:Zm(this);break;case 35:$m(this);break;case 38:if(this.og==Um)an(this);else return!1;break;case 37:if(this.og==Tm)this.Fk()?bn(this):an(this);else return!1;break;case 40:if(this.og==Um)bn(this);else return!1;
break;case 39:if(this.og==Tm)this.Fk()?an(this):bn(this);else return!1;break;default:return!1}return!0};function Xm(a,b){var c=b.o(),c=c.id||(c.id=nh(b));a.Sd||(a.Sd={});a.Sd[c]=b}f.$a=function(a,b){Wm.m.$a.call(this,a,b)};
f.sm=function(a,b,c){a.hj|=2;a.hj|=64;!this.ye()&&this.Ou||a.pc(32,!1);Bi(a,!1);var d=a.getParent()==this?vh(this,a):-1;Wm.m.sm.call(this,a,b,c);a.la&&this.la&&Xm(this,a);a=d;-1==a&&(a=sh(this));a==this.eb?this.eb=Math.min(sh(this)-1,b):a>this.eb&&b<=this.eb?this.eb++:a<this.eb&&b>this.eb&&this.eb--};
f.removeChild=function(a,b){if(a=ea(a)?qh(this,a):a){var c=vh(this,a);-1!=c&&(c==this.eb?(a.ud(!1),this.eb=-1):c<this.eb&&this.eb--);(c=a.o())&&c.id&&this.Sd&&Mb(this.Sd,c.id)}a=Wm.m.removeChild.call(this,a,b);Bi(a,!0);return a};f.setOrientation=function(a){if(this.o())throw Error("Component already rendered");this.og=a};f.va=function(){return this.Oa};
f.$=function(a,b){if(b||this.Oa!=a&&this.dispatchEvent(a?"show":"hide")){this.Oa=a;var c=this.o();c&&(w(c,a),this.ye()&&Sm(this.Hb(),this.Tb&&this.Oa),b||this.dispatchEvent(this.Oa?"aftershow":"afterhide"));return!0}return!1};f.isEnabled=function(){return this.Tb};
f.pa=function(a){this.Tb!=a&&this.dispatchEvent(a?"enable":"disable")&&(a?(this.Tb=!0,rh(this,function(a){a.Ut?delete a.Ut:a.pa(!0)})):(rh(this,function(a){a.isEnabled()?a.pa(!1):a.Ut=!0}),this.kg=this.Tb=!1),this.ye()&&Sm(this.Hb(),a&&this.Oa))};f.ye=function(){return this.rn};f.He=function(a){a!=this.rn&&this.la&&Ym(this,a);this.rn=a;this.Tb&&this.Oa&&Sm(this.Hb(),a)};f.xh=function(a){(a=th(this,a))?a.ud(!0):-1<this.eb&&th(this,this.eb).ud(!1)};f.ud=function(a){this.xh(vh(this,a))};
function Zm(a){cn(a,function(a,c){return(a+1)%c},sh(a)-1)}function $m(a){cn(a,function(a,c){a--;return 0>a?c-1:a},0)}function bn(a){cn(a,function(a,c){return(a+1)%c},a.eb)}function an(a){cn(a,function(a,c){a--;return 0>a?c-1:a},a.eb)}function cn(a,b,c){c=0>c?vh(a,a.Mb):c;var d=sh(a);c=b.call(a,c,d);for(var e=0;e<=d;){var g=th(a,c);if(g&&a.Wq(g)){a.xh(c);break}e++;c=b.call(a,c,d)}}f.Wq=function(a){return a.va()&&a.isEnabled()&&vi(a,2)};function dn(){}r(dn,oi);ka(dn);dn.prototype.ta=function(){return"goog-menuheader"};function en(a,b,c){Ai.call(this,a,c||dn.ua(),b);this.pc(1,!1);this.pc(2,!1);this.pc(4,!1);this.pc(32,!1);this.ma=1}r(en,Ai);xi("goog-menuheader",function(){return new en(null)});function fn(){this.cr=[]}r(fn,oi);ka(fn);function gn(a,b){var c=a.cr[b];if(!c){switch(b){case 0:c=a.ta()+"-highlight";break;case 1:c=a.ta()+"-checkbox";break;case 2:c=a.ta()+"-content"}a.cr[b]=c}return c}f=fn.prototype;f.Qf=function(){return"menuitem"};f.D=function(a){var b=a.I().D("DIV",this.fd(a).join(" "),this.Vj(a.Fd(),a.I()));hn(this,a,b,vi(a,8)||vi(a,16));return b};f.ub=function(a){return a&&a.firstChild};
f.J=function(a,b){var c=da(b.firstElementChild)?b.firstElementChild:Gf(b.firstChild,!0),d=gn(this,2);c&&ji(c,d)||b.appendChild(this.Vj(b.childNodes,a.I()));ji(b,"goog-option")&&(a.kl(!0),this.kl(a,b,!0));return fn.m.J.call(this,a,b)};f.oc=function(a,b){var c=this.ub(a),d=jn(this,a)?c.firstChild:null;fn.m.oc.call(this,a,b);d&&!jn(this,a)&&c.insertBefore(d,c.firstChild||null)};f.Vj=function(a,b){var c=gn(this,2);return b.D("DIV",c,a)};f.pl=function(a,b,c){a&&b&&hn(this,a,b,c)};
f.kl=function(a,b,c){a&&b&&hn(this,a,b,c)};function jn(a,b){var c=a.ub(b);if(c){var c=c.firstChild,d=gn(a,1);return!!c&&Hf(c)&&ji(c,d)}return!1}function hn(a,b,c,d){ti(a,c,b.ue());ui(a,b,c);d!=jn(a,c)&&(d?ki(c,"goog-option"):mi(c,"goog-option"),c=a.ub(c),d?(a=gn(a,1),c.insertBefore(b.I().D("DIV",a),c.firstChild||null)):c.removeChild(c.firstChild))}f.ri=function(a){switch(a){case 2:return gn(this,0);case 16:case 8:return"goog-option-selected";default:return fn.m.ri.call(this,a)}};
f.nk=function(a){var b=gn(this,0);switch(a){case "goog-option-selected":return 16;case b:return 2;default:return fn.m.nk.call(this,a)}};f.ta=function(){return"goog-menuitem"};function kn(a,b,c,d){Ai.call(this,a,d||fn.ua(),c);this.qa(b)}r(kn,Ai);f=kn.prototype;f.ba=function(){var a=this.Sa;return null!=a?a:this.qe()};f.qa=function(a){this.Sa=a};f.pc=function(a,b){kn.m.pc.call(this,a,b);switch(a){case 8:this.ma&16&&!b&&Fi(this,16,!1)&&this.setState(16,!1);var c=this.o();c&&this.W.pl(this,c,b);break;case 16:(c=this.o())&&this.W.kl(this,c,b)}};f.pl=function(a){this.pc(8,a)};f.kl=function(a){this.pc(16,a)};
f.qe=function(){var a=this.Fd();return la(a)?(a=fb(a,function(a){return Hf(a)&&(ji(a,"goog-menuitem-accel")||ji(a,"goog-menuitem-mnemonic-separator"))?"":Tf(a)}).join(""),Ca(a)):kn.m.qe.call(this)};f.Xg=function(a){var b=this.getParent();if(b){var c=b.Js;b.Js=null;if(c&&fa(a.clientX)&&ef(c,new df(a.clientX,a.clientY)))return}kn.m.Xg.call(this,a)};f.we=function(a){return a.keyCode==this.ms&&this.nh(a)?!0:kn.m.we.call(this,a)};f.Jv=function(){return this.ms};xi("goog-menuitem",function(){return new kn(null)});
kn.prototype.ue=function(){return vi(this,16)?"menuitemcheckbox":vi(this,8)?"menuitemradio":kn.m.ue.call(this)};kn.prototype.getParent=function(){return Ai.prototype.getParent.call(this)};kn.prototype.mk=function(){return Ai.prototype.mk.call(this)};function ln(){}r(ln,oi);ka(ln);ln.prototype.D=function(a){return a.I().D("DIV",this.ta())};ln.prototype.J=function(a,b){b.id&&oh(a,b.id);if("HR"==b.tagName){var c=b;b=this.D(a);Df(b,c);Ff(c)}else ki(b,this.ta());return b};ln.prototype.oc=function(){};ln.prototype.ta=function(){return"goog-menuseparator"};function mn(a,b){Ai.call(this,null,a||ln.ua(),b);this.pc(1,!1);this.pc(2,!1);this.pc(4,!1);this.pc(32,!1);this.ma=1}r(mn,Ai);mn.prototype.Aa=function(){mn.m.Aa.call(this);var a=this.o();fi(a,"separator")};xi("goog-menuseparator",function(){return new mn});function nn(a){this.Qq=a||"menu"}r(nn,Rm);ka(nn);nn.prototype.rb=function(a){return"UL"==a.tagName||nn.m.rb.call(this,a)};nn.prototype.wn=function(a){return"HR"==a.tagName?new mn:nn.m.wn.call(this,a)};nn.prototype.ta=function(){return"goog-menu"};nn.prototype.dg=function(a){nn.m.dg.call(this,a);a=a.o();gi(a,"haspopup","true")};xi("goog-menuseparator",function(){return new mn});function on(a,b){Wm.call(this,Um,b||nn.ua(),a);this.He(!1)}r(on,Wm);f=on.prototype;f.yf=!0;f.Pu=!1;f.ta=function(){return this.W.ta()};f.removeItem=function(a){(a=this.removeChild(a,!0))&&a.ib()};f.setPosition=function(a,b){var c=this.va();c||w(this.o(),!0);var d=this.o(),e=a,g=b,h=mg(d);e instanceof df&&(g=e.y,e=e.x);gg(d,d.offsetLeft+(e-h.x),d.offsetTop+(Number(g)-h.y));c||w(this.o(),!1)};f.mt=function(a){(this.yf=a)&&this.He(!0)};
f.$=function(a,b,c){(b=on.m.$.call(this,a,b))&&a&&this.la&&this.yf&&this.Hb().focus();this.Js=a&&c&&fa(c.clientX)?new df(c.clientX,c.clientY):null;return b};f.Nn=function(a){this.yf&&this.Hb().focus();return on.m.Nn.call(this,a)};f.Wq=function(a){return(this.Pu||a.isEnabled())&&a.va()&&vi(a,2)};f.Ca=function(a){var b=this.W;var c=this.I();c=of(c.R,"DIV",b.ta()+"-content",a);for(var d=c.length,e=0;e<d;e++)Vm(b,this,c[e]);on.m.Ca.call(this,a)};
f.we=function(a){var b=on.m.we.call(this,a);b||rh(this,function(c){!b&&c.Jv&&c.ms==a.keyCode&&(this.isEnabled()&&this.ud(c),b=c.gd(a))},this);return b};f.xh=function(a){on.m.xh.call(this,a);(a=th(this,a))&&ng(a.o(),this.o())};function pn(){pn.m.constructor.call(this)}r(pn,nn);pn.prototype.D=function(a){a=a.I().D("ul");a.setAttribute("class","dropdown-menu");return a};ka(pn);function qn(a,b){null==b&&(b=pn.ua());qn.m.constructor.call(this,a,b)}r(qn,on);qn.prototype.$=function(a){qn.m.$.call(this,a);null!=this.o()&&(this.o().style.display=a?"block":"none");return qn.m.$.call(this,a)};function rn(){rn.m.constructor.call(this)}r(rn,fn);rn.prototype.D=function(a){var b=a.I();var c=this.Vj(a.Fd(),b);c=b.D("li",{},c);hn(this,a,c,vi(a,8)||vi(a,16));ui(this,a,c);return c};rn.prototype.Vj=function(a,b){return b.D("a","menuitem",a)};ka(rn);function sn(){sn.m.constructor.call(this);this.Ad.push("paginator");this.lj=this.Ie=null;this.wg="%d per page";this.Li="LI";this.Do="SPAN";this.Rd=tn;this.fi=un;this.ei=vn}r(sn,wh);
var wn={Fh:"active",Gh:"disabled",xq:"previous",$l:"next",dm:"result",fm:"result-number",yq:"result-caption",gm:"result-total",zq:"result-displayed",Dg:"paging",bm:"pagination",am:"page-item",Rl:"base-switcher",uu:"paginator"},xn={qq:"c",Ih:"p",Dg:"paging",Sl:"baseType",Ql:"baseRange",Ul:"customBaseLabel",Tl:"customBaseErrorMessage"},tn=[1,1E3],un="Display number of rows",vn="Row number has to be between %s and %s";f=sn.prototype;
f.bn=function(a,b){var c=Eg(a,"paginator__error-message",b);if(null!=c)return a.removeNode(c)};f.Um=function(a,b){var c=this.Rd;c=a.G("div","paginator__error-message",[a.G("div","alert alert-danger",ym(this.ei,c[0],c[1]))]);return a.appendChild(b,c)};f.zi=function(a,b){var c=this;return Bm(b,function(){var d=a.I();var e=Eg(d,"paginator__custom-base-input",b);if(null!=e){var g=Yf(b);e=Nm(Em(e));return Pm(c.Rd,e)?(c.bn(d,g),a.Ge(e)):c.Um(d,g)}})};
f.Tm=function(a,b){var c=a.I();var d=Eg(c,"result-displayed",b);if(null==d)return null;var e=Yf(d);d=c.G("span","paginator__custom-base-wrapper");Vf(e,d);c.appendChild(d,c.G("span","paginator__custom-base-label",this.fi));var g=this.Rd;e=g[0];g=g[1];e=c.G("input",{type:"number",value:a.Rf(),"class":"paginator__custom-base-input",min:e,max:g});c.appendChild(d,e);return this.zi(a,e)};f.an=function(a,b){var c=a.I();var d=Eg(c,"paginator__custom-base-wrapper",b);if(null!=d)return c.removeNode(d)};
f.xm=function(a){if(la(a)&&2===a.length&&jb(a,Om))return a;ca.console.warn('Paginator "data-base-rage" contains invalid value. A tuple with two positive number was expected.');return tn};f.gp=function(a){a=Mm(a,"pattern");if(null!=a)return this.wg=a};f.Jq=function(a){return this.kj(a,wn.Fh,wn.Gh)};f.Bk=function(a){return this.kj(a,wn.Gh,wn.Fh)};f.$m=function(a,b,c){if(null!=b)return this.gp(b),this.Oh(a,b,c),Fm(this.lj,String(a.base))};
f.Oh=function(a,b,c){a=this.Vm(a,c);b.appendChild(a);this.lj=a;return this.Ie=b};
f.Vm=function(a,b){var c,d=this;var e=b.D("div","dropdown");this.Hc=b.D("button","btn btn-default dropdown-toggle");this.ug(this.Hc,a.base);b.appendChild(e,this.Hc);var g=new qn(this.l);g.$(!1);var h=a.Bf;var k=0;for(c=h.length;k<c;k++){var l=h[k];var m=new kn(ym(this.wg,l),l,this.l,rn.ua());g.$a(m,!0)}v(g,"action",function(c){l=c.target.Sa;g.$(!1);b.appendChild(e,d.Hc);d.ug(d.Hc,l);return a.Ge(l)});g.render(e);v(this.Hc,"click",function(){return g.$(!g.va())});v(b.R.body,"click",function(){if(g.va())return g.$(!1)},
!0);return e};f.Im=function(a){var b;var c=a.I();var d=a.o();if(null!=d){var e=Xf(d);c.removeNode(d);var g=Fg(c,wn.am,d);var h=0;for(b=g.length;h<b;h++)a=g[h],c.removeNode(a);g=Fg(c,wn.dm,d);h=0;for(b=g.length;h<b;h++)a=g[h],a.innerHTML="";return c.vo(d,e)}};f.ug=function(a,b){if(null!=b&&null!=a)return a.innerHTML=[ym(this.wg,b),'\x3cspan class\x3d"caret"\x3e\x3c/span\x3e'].join("")};f.Ge=function(a){return this.ug(this.Hc,a)};f.Bn=function(a){return Eg(a.I(),"pagination",a.o())};function yn(a){this.Af=q(this.Af,this);yn.m.constructor.call(this,a);this.sf=a.sf;this.base=this.sf.Rf();this.base=0<=this.base?this.base:zn;this.ge=[];this.Na=[];this.Ie=null;this.Bf=this.gi=[10,25,50,100,500,1E3];this.oi=!1}r(yn,xh);var zn=10;f=yn.prototype;f.Rf=function(){return this.base};f.Af=function(a){return Im([this.base,a],Om,zn)};f.gs=function(a){return this.base=Mm(a,"base",bb(this.Af,Nm))};f.rb=function(a){return null!=a&&dh(a,"paginator")};f.xf=function(){return this.Ai(this.K.Bn(this))};
f.Ca=function(a){this.oi="true"===Mm(a,"forceDisplay");if(!this.Bf){var b=a.querySelector(".base-switcher");this.fp(b)}this.K.J(this,a);this.F=a;return this.vg(!1)};f.fp=function(a){if(null!=a||this.Bf)return Mm(a,"bases",qd,this.gi)};f.clone=function(){var a=this.o().cloneNode(!0);this.Ai(a);Zg(this.o(),"top");Zg(a,"bottom");ah(a,"top");this.K.zi(this,a);return a};f.rr=function(){if(null==this.o())return null;var a=this.clone();this.ge.push(a);return a};
f.Hm=function(){var a;var b=this.Na;var c=0;for(a=b.length;c<a;c++){var d=b[c];ed(d)}return this.Na=[]};f.show=function(a,b){null==b&&(b=!1);return!b&&2>this.Vk&&!this.oi?this.vg(!1):this.vg(a)};f.vg=function(a){var b;var c=a?ah:Zg;var d=[this.o()].concat(this.ge);var e=[];var g=0;for(b=d.length;g<b;g++)a=d[g],e.push(c(a,"empty"));return e};function An(a,b){this.l=a;this.G=b;this.wc=q(this.wc,this);this.name="";if(!Th(this.G,"col")){var c=this.l.Vg(this.G);this.name=c?String(c):""}v(this.G,"click",this.wc);An.m.constructor.call(this);this.direction=Bn}r(An,md);var Cn={$t:"goog-tablesorter-sorted",du:"goog-tablesorter-sorted-reverse"},Bn=3,Dn={Zz:"sort-asc",$z:"sort-desc",aA:"sort-no"};An.prototype.o=function(){return this.G};An.prototype.getName=function(){if(!this.name){var a=this.name=Rh(this.G,"col");this.name=a?String(a):""}return String(this.name)};
An.prototype.te=function(){return this.direction};
An.prototype.wc=function(){if(dh(this.G,Cn.$t)){this.direction=2;ah(this.G,"goog-tablesorter-sorted");Zg(this.G,"goog-tablesorter-sorted-reverse");var a=this.dispatchEvent(new Cc("sort-desc"))}else dh(this.G,Cn.du)?(this.direction=Bn,ah(this.G,"goog-tablesorter-sorted"),ah(this.G,"goog-tablesorter-sorted-reverse"),a=this.dispatchEvent(new Cc("sort-no"))):(this.direction=1,Zg(this.G,"goog-tablesorter-sorted"),ah(this.G,"goog-tablesorter-sorted-reverse"),a=this.dispatchEvent(new Cc("sort-asc")));return a};function En(a){this.l=a;this.Yf=q(this.Yf,this);En.m.constructor.call(this);this.table=null;this.headers=[]}r(En,md);En.prototype.J=function(a){var b;this.table=a;var c=Fg(this.l,"sortable",a);var d=[];a=0;for(b=c.length;a<b;a++){var e=c[a];e=new An(this.l,e);v(e,Gb(Dn),this.Yf);d.push(this.headers.push(e))}return d};En.prototype.Yf=function(a){return this.dispatchEvent(new Cc("sort",a.target))};function Fn(){Fn.m.constructor.call(this)}r(Fn,wm);Fn.prototype.D=function(a){var b=Fn.m.D.call(this,a);var c=a.I();c.Yc(b,"");var d=c.G("span",{},b);c.Yc(d,a.qe());Zg(b,"btn");return b};Fn.prototype.rb=function(a){return"DIV"===a.tagName||"SPAN"===a.tagName?!0:Fn.m.rb.call(this,a)};ka(Fn);function Gn(){Gn.m.constructor.call(this);this.tag="a";this.Ad=["btn","btn-link"]}r(Gn,wh);Gn.prototype.D=function(a){var b=Gn.m.D.call(this,a);b.innerHTML="";b.href=a.href;b.title=a.Kr();null!=a.qk()&&(b.target=a.qk());var c=a.I().G("span",{});a.ah?c.innerHTML=a.ah:a.I().Yc(c,a.qe());b.appendChild(c);return b};ka(Gn);function Hn(a){null==a.K&&(a.K=Gn.ua());Hn.m.constructor.call(this,a);this.href=a.href;this.caption=a.caption;this.ah=a.ah;this.title=a.title;this.target=a.target;null==this.href&&(this.href="");null==this.caption&&(this.caption="");null==this.ah&&(this.ah="");null==this.title&&(this.title=this.caption)}r(Hn,xh);Hn.prototype.Kr=function(){return this.title};Hn.prototype.qk=function(){return this.target};Hn.prototype.Fd=function(){return this.o()?this.l.Vg(this.o()):""};function In(){}In.prototype.format=function(a,b){null==b&&(b=null);var c="";if(null!=b){var d=b.split("__");for(c=a;0<d.length;){var e=d.shift();c=null!=c[e]?c[e]:""}}if(la(c)){var g;var h=[];e=0;for(g=c.length;e<g;e++)d=c[e],h.push(Jn(d));c=h.join(", ")}else c=Jn(c);return c};function Jn(a){oa(a)&&(a=new fl(a));return null!=a._verbose?a._verbose:String(a)};function Kn(a,b,c){Kn.m.constructor.call(this,a,b,c)}r(Kn,Rm);Kn.prototype.dg=function(a){var b=a.o();if(a=this.Qf())return fi(b,a)};function Ln(){Ln.m.constructor.call(this);this.tag="tbody"}r(Ln,Kn);Ln.prototype.D=function(a){return a.I().G(this.tag,this.fd(a).join(" "))};ka(Ln);function Mn(a){null==a&&(a={});this.hd=q(this.hd,this);null==a.K&&(a.K=Ln.ua());var b=a.orientation;var c=a.K;this.l=a.l;Mn.m.constructor.call(this,b,c,this.l);this.rg=!0;this.He(!1);this.selected=null}r(Mn,Wm);f=Mn.prototype;f.add=function(a){a=new Nn({l:this.l,caption:a});return this.$a(a)};f.$a=function(a,b){var c;null==b&&(b=!1);Mn.m.$a.call(this,a,b);var d=[32,8];var e=0;for(c=d.length;e<c;e++){var g=d[e];a.pc(g,!0)}};f.se=function(){var a=[];rh(this,function(b){return a.push(b)});return a};
f.D=function(){var a;if(!this.la&&(null!=this.K?this.F=a=this.K.D(this):Mn.m.D.call(this),this.rg))return rh(this,function(a){return a.D()})};f.render=function(a){var b=this;null==a&&(a=null);return this.lf(function(c){return null!=a?a.insertBefore(c,null):b.l.R.body.appendChild(c)})};f.lf=function(a){var b=this;if(this.la)throw Error("Component already rendered");null==this.o()&&this.D();a(this.o());this.rg&&rh(this,function(a){return a.render(b.o())});this.Aa();return this.wk()};
f.wk=function(){return this.A("action",this.hd)};f.hd=function(a){a=a.target;this.selected&&this.setSelected(!1);this.selected=a;return this.setSelected(!0)};f.setSelected=function(a){return this.selected.setState(8,a)};f.destroy=function(){this.lb();return this.l.removeNode(this.o())};f.xe=function(){this.Tb&&(this.kg=!0)};f.yp=function(a){var b=this;return rh(this,function(c){if(c.Sa.id===a.id)return c.setSelected(!0),b.A("action",function(){return c.setSelected(!1)})})};function On(a){null==a&&(a={});On.m.constructor.call(this,a.content,a.K,a.l);this.rg=!0}r(On,Ai);f=On.prototype;f.addClass=function(a){return Di(this,a)};f.D=function(){var a;if(!this.la&&(null!=this.K?this.F=a=this.K.D(this):On.m.D.call(this),this.rg))return rh(this,function(a){return a.D()})};f.render=function(a){var b=this;null==a&&(a=null);return this.lf(function(c){return null!=a?a.insertBefore(c,null):b.l.R.body.appendChild(c)})};
f.lf=function(a){var b=this;if(this.la)throw Error("Component already rendered");null==this.o()&&this.D();a(this.o());this.rg&&rh(this,function(a){return a.render(b.o())});return this.Aa()};f.destroy=function(){this.lb();return this.l.removeNode(this.o())};function Pn(){Pn.m.constructor.call(this);this.tag="div"}r(Pn,oi);Pn.prototype.D=function(a){var b=a.I().G(this.tag,this.fd(a).join(" "),a.Fd());ui(this,a,b);return b};ka(Pn);function Qn(){Qn.m.constructor.call(this);this.tag="tr"}r(Qn,Pn);ka(Qn);function Rn(){Rn.m.constructor.call(this);this.tag="td"}r(Rn,wh);Rn.prototype.D=function(a){var b=a.I();var c=b.G(this.tag,this.xd(a));b=b.G("span","cell-content");b.innerHTML=a.qe();c.appendChild(b);return c};ka(Rn);function Nn(a){null==a.K&&(a.K=Rn.ua());Nn.m.constructor.call(this,a)}r(Nn,xh);function Sn(a){null==a&&(a={});this.Wf=q(this.Wf,this);this.hd=q(this.hd,this);null==a.K&&(a.K=Qn.ua());Sn.m.constructor.call(this,a);this.confirm=a.confirm;this.cells=[];this.nf=null;Di(this,Tn)}r(Sn,On);var Tn="row-unselected",Un={SELECTED:"row-selected",Kh:Tn};function Vn(a){var b=new Nn({l:a.l,caption:""});a.$a(b);return b}f=Sn.prototype;f.Ck=function(){return!0};f.pl=function(a){this.nf=a;return this.nf.A(Wn,this.hd)};
f.hd=function(a){var b,c;var d=this.dh()?[Un.SELECTED,Un.Kh]:[Un.Kh,Un.SELECTED];var e=[Zg,ah];var g=b=0;for(c=e.length;b<c;g=++b){var h=e[g];h(this.o(),d[g])}return this.dispatchEvent(new Cc("selection-change",a))};f.dh=function(){return null!=this.nf&&!!this.nf.ba()};
f.Wf=function(a){var b=this;var c=a.target.Sa.action;var d=c.confirm;var e=a.target;return d?(this.confirm.oc(d.text),this.confirm.dj(d.title),Xn(this.confirm,d.true_label,d.false_label),this.confirm.open(),this.confirm.focus(),cd(this.confirm,pm,function(a){if("yes"===a.key)return"DELETE"===c.method?b.dispatchEvent(new Cc("DeleteButton",e)):b.dispatchEvent(new Cc("RemoteButton",e))})):this.dispatchEvent(new Cc("RemoteButton",a.target))};
f.tk=function(a){return this.dispatchEvent(new Cc("DeleteButton",a.target))};function Yn(a){null==a&&(a={});null==a.K&&(a.K=Fn.ua());Yn.m.constructor.call(this,a.content,a.K,a.l);this.l=a.l}r(Yn,xm);function Zn(a){this.handleError=q(this.handleError,this);Zn.m.constructor.call(this,a);this.Ma={}}r(Zn,Yn);f=Zn.prototype;f.call=function(a,b,c,d,e,g){var h=this;null==e&&(e=null);null==g&&(g=!1);this.pa(!1);b=this.Qm(b);return a.request(b,c,d,function(a){h.pa(!0);null!=e&&e(a);return h.dispatchEvent(new Cc("remote-button-success",a))},this.handleError,g)};f.handleError=function(a){this.pa(!0);return this.dispatchEvent(new Cc("remote-button-error",a))};f.D=function(){Di(this,"remote-button");return Zn.m.D.call(this)};
f.Aa=function(){var a=this.o();a.setAttribute("type","button");var b;if(a=Rh(a,"fields")){var c=String(a).split(",");var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];a=Eg(this.l,"remote-button-param-"+g);null!=a?d.push(this.Ma[g]=a):d.push(void 0)}}return Zn.m.Aa.call(this)};f.Qm=function(a){a=new yk(a);var b=this.Ma;for(d in b){var c=b[d];Mk(a,d,Em(c))}var d=[a.pd];a.vc()&&d.push(a.vc());return d.join("?")};function $n(a){$n.m.constructor.call(this,a)}r($n,Zn);$n.prototype.call=function(a,b,c,d,e){null==e&&(e=null);return $n.m.call.call(this,a,b,c,d,e,!0)};function ao(a){this.Of=a};function bo(){bo.m.constructor.call(this);this.tag="ul";this.Ad.push("error-list")}r(bo,wh);ka(bo);function co(a){null==a&&(a={});null==a.K&&(a.K=bo.ua());co.m.constructor.call(this,a);this.ds=null}r(co,xh);co.prototype.Lh=function(a){this.l.sh(this.o());this.ds=this.l.G("li",{},a);return this.o().appendChild(this.ds)};function eo(a){null==a&&(a={});this.nc=q(this.nc,this);eo.m.constructor.call(this,a);this.name=a.name;this.required=a.required;this.size=a.size;this.caption=a.caption;this.Qd=a.Qd;this.ln=a.ln;this.placeholder=a.placeholder;oa(a.value)&&(a.value=rd(a.value));this.qa(a.value);null==this.required&&(this.required=!1);null==this.size&&(this.size=50);null==this.caption&&(this.caption="");null==this.Qd&&(this.Qd=new ao(this));null==this.ln&&(this.ln=new co({l:this.l}))}r(eo,xh);var Wn="change";f=eo.prototype;
f.$a=function(){};f.xf=function(){this.qa(this.value);return this.wk()};f.qa=function(a){this.value=a;if(null!=this.o())return Fm(this.o(),a)};f.ba=function(){return this.la?Em(this.o()):this.value};f.focus=function(){var a;return null!=(a=this.o())?a.focus():void 0};f.J=function(a){eo.m.J.call(this,a);return this.wk()};f.wk=function(){return v(this.o(),"change",this.nc)};f.nc=function(a){return this.dispatchEvent(new Cc(Wn,a))};function fo(){fo.m.constructor.call(this);this.tag="input"}r(fo,wh);fo.prototype.D=function(a){return a.I().D(this.tag,this.xd(a))};fo.prototype.xd=function(a){var b={"class":this.fd(a).join(" "),size:a.size,id:nh(a)};a.required&&(b.required="required");null!=a.placeholder&&(b.placeholder=a.placeholder);return b};ka(fo);function go(){go.m.constructor.call(this)}r(go,fo);go.prototype.xd=function(a){var b=go.m.xd.call(this,a);b.type=a.type;return b};ka(go);function ho(a){null==a&&(a={});null==(null!=a?a.K:void 0)&&(a.K=go.ua());ho.m.constructor.call(this,a);this.type=a.type;null==this.type&&(this.type="text")}r(ho,eo);f=ho.prototype;f.clear=function(){return this.o().value=""};f.render=function(a){ho.m.render.call(this,a);return this.ni()};f.qg=function(a){ho.m.qg.call(this,a);return this.ni()};f.ni=function(){this.Ek(this.o())||(this.F=this.S.se(this.o())[0])};f.Ek=function(a){return"INPUT"===a.tagName};
f.Ok=function(a){null==a&&(a=!0);return this.o().required=a?"required":void 0};function io(){io.m.constructor.call(this)}r(io,go);io.prototype.xd=function(a){var b=io.m.xd.call(this,a);a.ba()&&(b.checked="checked");return b};ka(io);function jo(a){null==a.K&&(a.K=io.ua());a.type="checkbox";null==a.value&&(a.value=!!a.value);jo.m.constructor.call(this,a)}r(jo,ho);jo.prototype.ba=function(){return!!jo.m.ba.call(this)};jo.prototype.qa=function(a){jo.m.qa.call(this,!!a);return this.dispatchEvent(new Cc(Wn,this))};function ko(a,b,c,d,e,g){this.l=a;this.rows=b;this.cols=c;this.Ir=d;this.confirm=e;this.nf=null!=g?g:!1;this.Dm=q(this.Dm,this);this.Bm=q(this.Bm,this);ko.m.constructor.call(this);this.mq={rest:this.Bm,web:this.Dm}}r(ko,xh);f=ko.prototype;f.Bp=function(a){this.cols=a};
f.ic=function(a,b){var c;null==b&&(b=!0);var d=new Sn({l:this.l,confirm:this.confirm});d.Sa=a;this.rows.$a(d,!1);if(this.nf){var e=new Nn({l:this.l});var g=new jo({l:this.l});e.$a(g);e.addClass("select-row");d.$a(e);d.pl(g)}var h=this.cols;g=0;for(c=h.length;g<c;g++)e=h[g],lo(this,a,e,d);if(null!=a._class_names)for(h=a._class_names,g=0,c=h.length;g<c;g++)e=h[g],Di(d,e);la(a._actions)&&0!=a._actions.length&&b&&mo(this,d,a);return d};
function lo(a,b,c,d){var e=new Nn({l:a.l});b._default_action?(a=new Hn({l:a.l,href:b._web_links[b._default_action],ah:a.Ir.format(b,c)}),e.$a(a)):e.nt(a.Ir.format(b,c));e.addClass(c);d.$a(e);return e}function mo(a,b,c){var d;var e=Vn(b);e.addClass("actions");var g=c._actions;var h=0;for(d=g.length;h<d;h++){var k=g[h];var l=a;null!=l.mq[k.type]?l.mq[k.type](k,c,e,b):ne(l.l).console.warn("Non-existent action type: "+k.type)}b.$a(e);return e}
f.Bm=function(a,b,c,d){var e=new $n({content:a.verbose_name,l:this.l});b={qs:b,action:a};var g=a.verbose_name;Di(e,this.un(a));e.Sa=b;e.rl(g);c.$a(e);e.A("action",d.Wf);return e};f.Dm=function(a,b,c){b=new Hn({l:this.l,href:b._web_links[a.name],caption:a.verbose_name,target:a.target});b.addClass(this.un(a));return c.$a(b)};f.un=function(a){return a.class_name||a.name||""};function no(a,b,c,d,e,g,h,k,l){this.l=a;this.up=b;this.cols=c;this.sf=d;this.confirm=e;this.query=g;this.gb=h;this.ka=k;this.ht=null!=l?l:!1;this.co=q(this.co,this);this.tk=q(this.tk,this);this.Yf=q(this.Yf,this);this.fo=q(this.fo,this);this.Wf=q(this.Wf,this);this.eo=q(this.eo,this);this.Ln=q(this.Ln,this);this.qp=q(this.qp,this);this.refresh=q(this.refresh,this);this.Mn=q(this.Mn,this);no.m.constructor.call(this);this.Op=this.tbody=this.table=null;this.rows=new Mn({l:this.l});this.gl=null;this.ut=
!0;this.data=[]}r(no,xh);function oo(a){var b;return null!=(b=Yf(a.table))?b.querySelector(".paginator"):void 0}f=no.prototype;
f.J=function(a){var b,c=this;this.table=a;this.gl=new ko(this.l,this.rows,this.cols,new In,this.confirm,this.ht);this.sf.A(po,this.refresh);Eg(this.l,"actions",this.table)||(this.ut=!1);qo(this);var d=oo(this);null!=d?this.gb.gs(d):this.gb=null;ro(this,this.yd(),function(a){c.Op=new En(c.l);c.Op.J(c.table);c.Op.A("sort",c.Yf);if(null!=c.gb){c.gb.init(a);null!=d?c.gb.J(d):c.gb.qg(c.table);var b=c.gb.rr();c.l.Ki(b,c.table)}c.dispatchEvent(new Cc("loaded",a));return c.A("delete-item",c.Mn)});var e=[this.table,
d];var g=0;for(b=e.length;g<b;g++)a=e[g],Cm(a,function(a){return c.dispatchEvent(new Cc("action",a))});g=Dg(this.l,"input.select-all",this.table);null!=g&&(a=new jo({l:this.l}),a.J(g),a.A(Wn,this.co))};f.vc=function(){return this.sf.vc()};f.Mn=function(a){var b=this;if(this.gb.Ys())return this.up["delete"](a.target,function(){return ro(b,b.yd(),function(a){if(null!=b.gb)return b.gb.refresh(a)})});this.data=this.data.filter(function(b){return b!==a.target});return so(this)};
f.yd=function(){null!=this.gb&&this.gb.yd(this.query);return this.query};f.refresh=function(){var a=this;this.query=this.vc();return ro(this,this.yd(),function(b){var c;return null!=(c=a.gb)?c.refresh(b):void 0})};f.qp=function(){var a=this;this.query=this.vc();this.query.el=!0;ro(this,this.yd(),function(b){var c;return null!=(c=a.gb)?c.refresh(b):void 0});return this.query.el=!1};function ro(a,b,c){null==c&&(c=null);null!=c&&(a.yr=c);return a.up.load(b,a.Ln)}
function so(a){var b;if(a.rows.la){var c=a.rows;c.lb();Hg(c.o());c.sh()}var d=a.data;c=0;for(b=d.length;c<b;c++){var e=d[c];e=a.gl.ic(e,a.ut);e.A("DeleteButton",a.tk);e.A("RemoteButton",a.Wf);e.A("selection-change",a.fo)}a.rows.la||a.rows.render(a.table);return a.rows.A("action",a.eo)}f.Ln=function(a,b){this.gb.clearData()?this.data=a:this.data=this.data.concat(a);so(this);b.count=a.length;null!=this.yr&&this.yr(b);return this.data};
f.eo=function(a){a.target.setState(8,!0);return this.dispatchEvent(new Cc("selected-item",a.target.Sa))};function qo(a){var b=a.table.querySelector("tbody");null!=b&&a.l.removeNode(b)}
f.Wf=function(a){var b=this;a=a.target;var c=a.Sa.qs;var d=a.Sa.action;return a.call(this.up.client,c._rest_links[d.name].url,d.method,d.data,function(a){b.ka.kx(d.success_text);var c=b.gl,e;var k=uh(c.rows);var l=[];var m=0;for(e=k.length;m<e;m++){var p=k[m];p=qh(c.rows,p);if(p.Sa.id===a.id){var u=p;u.Sa=a;Hg(u.o());u.sh();k=u.o();p=c.cols;e=0;for(m=p.length;e<m;e++){var A=p[e];lo(c,a,A,u).render(k)}la(a._actions)&&0!=a._actions.length?mo(c,u,a).render(k):Vn(u).render();break}else l.push(void 0)}return l})};
f.fo=function(a){return this.dispatchEvent(a)};f.Yf=function(a){var b=this;a=a.target;this.query.sort(a.getName(),a.te());this.gb.$s()&&this.gb.reset(this.query);return ro(this,this.yd(),function(a){return b.gb.refresh(a)})};f.tk=function(a){a=a.target;this.dispatchEvent(new Cc("delete-item",a.Sa.qs));return this.l.removeNode(a.Sa.xb)};f.qt=function(a){this.query=a};f.yp=function(a){return this.rows.yp(a)};f.Bp=function(a){this.cols=a};
f.Ug=function(){var a;if(this.ht){var b=this.rows.se();var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];e.dh()&&c.push(e)}return c}return[]};f.co=function(a){var b;var c=this.rows.se();var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];null!=g.nf?d.push(g.nf.qa(a.currentTarget.ba())):d.push(void 0)}return d};function to(a){this.table=a;this.vp=this.cols=null}function uo(a){var b;null==b&&(b=!1);return a.cols&&!b?a.cols:a.cols=vo(a,"cols")}function wo(a){var b;null==b&&(b=!1);return a.vp&&!b?a.vp:a.vp=vo(a,"restFields")}to.prototype.Tg=function(a){var b;return String(null!=(b=Rh(this.table,a))?b:"")};function vo(a,b){var c=a.Tg(b).split(",");1===c.length&&""===c[0]&&(c=[]);return c};function xo(a){this.client=a}xo.prototype.load=function(a,b){return this.client.find(a.Qm(),b,null,a)};xo.prototype["delete"]=function(a,b){return this.client["delete"](a,b)};Re.prototype.bh=function(a,b){this.kf=a;this.en=b;return this};function yo(){yo.m.constructor.call(this)}r(yo,dm);var zo={yes:"btn btn-primary",no:"btn btn-default",ok:"btn btn-primary",cancel:"btn btn-default"};yo.prototype.render=function(){if(this.F){this.F.innerHTML="";var a=hf(this.F);var b=function(b,d){var c=a.D("button",{name:d,className:zo[d]},b);d===this.Xj&&Zg(c,this.kc+"-default");return this.F.appendChild(c)};return Kd(this,b,this)}};function Ao(a,b,c){Ao.m.constructor.call(this,a,b,c);a=new yo;cm(a,rm,!0);cm(a,sm,!1,!0);om(this,a)}r(Ao,bm);function Xn(a,b,c){a=a.kb;null!=b&&a.set("yes",b);null!=c&&a.set("no",c)}f=Ao.prototype;f.oc=function(a){this.ab=(new Re).bh(a,null);null!=this.cc&&(this.cc.innerHTML=Te(this.ab))};f.open=function(){return this.$(!0)};f.hide=function(){return this.$(!1)};f.$=function(a){Ao.m.$.call(this,a);ag(this.o(),"display",a?"block":"none");return w(this.Gf,a)};
f.D=function(){this.F=this.S.G("div","modal fade in");var a=this.S.G("div","modal-dialog",this.F);this.Gf=this.S.G("div","modal-content",a);Zg(this.Gf,this.ta());Qf(this.Gf,!0);w(this.Gf,!1);Yl(this);Zl(this);this.Bc.className="modal-backdrop fade in";this.fc=this.S.D("h4","modal-title",this.mj);a={className:"modal-header",id:nh(this)};this.Pb=this.S.D("div",a);this.xk&&(a=new Ah({l:this.S}),a.render(this.Pb),this.Ac=a.o());this.Pb.appendChild(this.fc);this.nx=this.Pb.id;this.cc=this.S.D("div","modal-body");
this.jb=this.S.D("div","modal-footer");this.kb&&(a=this.kb,a.F=this.jb,a.render());w(this.jb,!!this.kb);Bf(this.Gf,this.Pb,this.cc,this.jb);fi(this.Gf,this.ue());gi(this.Gf,"labelledby",this.nx||"");this.ab&&this.cc&&(this.cc.innerHTML=Te(this.ab));return hm(this,this.Qh)};function Bo(a,b,c){Bo.m.constructor.call(this,a,b,c);this.confirm="Do you really want to delete %s?"}r(Bo,Ao);function Co(){}Co.prototype.Tg=function(a,b){var c;return null!=(c=Rh(a,b))?c:""};function Do(a){this.grid=a;this.Un=q(this.Un,this);this.grid.A("loaded",this.Un)}Do.prototype.J=function(a){return this.rs=a.querySelector(".table-empty")};Do.prototype.Un=function(a){var b;a=0===a.target.count;null!=(b=this.grid.gb)&&b.show(!a);if(this.rs)return w(this.rs,a)};function Eo(a,b){this.l=a;this.G=b;this.Cn=q(this.Cn,this);this.nc=q(this.nc,this);Eo.m.constructor.call(this);Am(this.G,this.nc);this.pt()}r(Eo,md);f=Eo.prototype;f.nc=function(a){return this.dispatchEvent(new Cc("change",a))};function Fo(a,b,c){a.qa(c);return a.cj(b)}f.o=function(){return this.G};f.ba=function(){return Em(this.G)};f.qa=function(a){return Fm(this.G,a)};f.cj=function(a){this.Be=a};f.reset=function(){return this.qa("")};f.getName=function(){return this.name};f.We=function(){return String(this.Be)};
f.pt=function(){var a=Go(this).split("__");this.cj(Hm(a,""));return this.name=this.Ve()};function Go(a){return String(Rh(a.G,"filter"))}f.Ve=function(){this.filter||(this.filter=Go(this));return this.filter};f.lk=function(){return this.Cn().lk()};f.Cn=function(){return new kl(this.getName(),this.We(),this.ba(),!0)};f.apply=function(a){var b;var c=new kl(this.getName(),this.We(),this.ba(),!0);(b=pl(a,c))&&a.filter(c);return b};f.$r=function(a){return a===this.Ve()};function Ho(){Ho.m.constructor.call(this);this.tag="select"}r(Ho,fo);Ho.prototype.D=function(a){var b;var c=Ho.m.D.call(this,a);var d=a.options;for(b in d){var e=d[b];c.appendChild(Io(a.I(),b,e))}return c};Ho.prototype.xd=function(a){var b=Ho.m.xd.call(this,a);Mb(b,"type");a.multiple&&(b.multiple="multiple");return b};function Io(a,b,c){b=a.D("option",{value:b});a.Yc(b,c);return b}ka(Ho);function Jo(a){null==a&&(a={});null==(null!=a?a.K:void 0)&&(a.K=Ho.ua());null==a.size&&(a.size=1);Jo.m.constructor.call(this,a);this.multiple=a.multiple;this.options=a.options;null==this.multiple&&(this.multiple=!1);null==this.options&&(this.options={});a=this.options;if(null==this.value){a:{for(b in a)break a;var b=void 0}this.qa(b)}this.options=a}r(Jo,eo);Jo.prototype.qa=function(a){if(void 0!==a)return Jo.m.qa.call(this,String(a))};function Ko(a,b){Ko.m.constructor.call(this,a,b);this.options={gt:"\x3e",gte:"\x3e\x3d",eq:"\x3d",lte:"\x3c\x3d",lt:"\x3c"};this.select=new Jo({l:a,options:this.options});this.select.addClass(Lo);this.select.qg(b);this.select.qa(String(this.Be));this.select.A(Wn,this.nc)}r(Ko,Eo);var Lo="filter-extended";f=Ko.prototype;f.wr="eq";f.Ve=function(){return this.name};f.We=function(){return String(this.select.ba())};f.reset=function(){this.qa(String(this.Be));return this.cj(this.wr)};
f.cj=function(a){this.Be=ll(a)?this.wr:a;if(null!=this.select)return this.select.qa(String(this.Be))};f.pt=function(){var a=Go(this).split("__");this.cj(Hm(a,""));return this.name=Gm(a).join("__")};f.apply=function(a){var b;var c=new kl(this.getName(),String(this.select.ba()),this.ba(),!1);(b=pl(a,c))&&a.filter(c);return b};f.$r=function(a){return Gm(a.split("__")).join("__")===this.name};function Mo(a,b,c,d){this.grid=a;this.query=b;this.Qc=c;this.l=d;this.nc=q(this.nc,this);Mo.m.constructor.call(this);this.Ma={};this.Ct={};this.hi={}}r(Mo,md);f=Mo.prototype;
f.gq=function(a){var b;var c=dh(a,"extended-filters");var d=this.l.all("thead *[data-filter]",a);var e=0;for(b=d.length;e<b;e++){var g=d[e];var h;if(h=c){var k=void 0;h=g;h=zm(h)||"number"===(k=h.type)||"date"===k||"datetime"===k}g=h?new Ko(this.l,g):new Eo(this.l,g);Jb(this.Ma,g.Ve())?ne(this.l).console.warn("Table #"+a.id+" contains duplicated columns with name '"+g.Ve()+"'."):(this.Ma[g.Ve()]=g,v(g,"change",this.nc))}a=No(this.Qc);c={};b=0;for(d=a.length;b<d;b++){e=a[b];g=e.split("__");if(null!=
g[g.length-2]){var l=g;g=[];var m=h=0;for(k=l.length-2;0<=k?h<=k:h>=k;m=0<=k?++h:--h)g.push(l[m]);g=g.join("__")}else g=g[0];c[e]={operator:e.replace(g,"").replace("__",""),value:this.Qc.get(e),name:e}}this.Ct=c;this.hi=this.query.hi;c=this.hi;e=[];for(u in c)a=c[u],null!=this.Ma[u]?e.push(this.Ma[u].qa(a)):e.push(this.query.filter(new kl(u,"",a)));Oo(this);var p;a=this.Ma;c=[];for(p in a){var u=a[p];c.push(u.apply(this.query))}return this.query};
function Oo(a){var b,c,d;var e=a.Ma;var g=[];for(b in e){var h=e[b];g.push(function(){var a=this.Ct;var b=[];for(c in a)d=a[c],h.$r(c)?b.push(Fo(h,d.operator,d.value)):b.push(void 0);return b}.call(a))}}f.nc=function(a){return this.filter(a.currentTarget)};f.filter=function(a){var b;if(a.apply(this.query)){var c=this.Ma;for(b in c)a=c[b],a.apply(this.query);this.dispatchEvent(new Cc("reset-paginator",{}));return this.Dd()}};f.Dd=function(){return this.dispatchEvent(new Cc("filter-changed",{}))};
function Po(a){var b;var c=a.Ma;var d=[];for(b in c){var e=c[b];e.reset();d.push(e.apply(a.query))}return a.Dd()}f.vc=function(){return this.query};function Qo(a,b,c,d,e){this.client=a;this.l=b;this.$b=c;this.Qc=d;this.query=e;this.Nk=q(this.Nk,this);this.hd=q(this.hd,this);this.Vn=q(this.Vn,this)}var Ro={iu:"webLink"};function So(a){a=Rh(a,"mode");return null!=a&&"pane"===a}Qo.prototype.gq=function(a){a.A("selected-item",this.hd);a.A("loaded",this.Vn);a=(a=this.Qc.get("i"))?this.client.get([this.query.uri.pd,a].join("/"),this.Nk):void 0;return a};Qo.prototype.Vn=function(a){if(this.Qk)return a.currentTarget.yp(this.Qk)};Qo.prototype.hd=function(a){return this.Nk(a.target)};
Qo.prototype.Nk=function(a){var b;this.Qk=a;this.Qc.set("i",this.Qk.id);var c=this.l.all(".snippet[data-web-link]");var d=[];a=0;for(b=c.length;a<b;a++){var e=c[a];var g=String(Rh(e,Ro.iu)).split(".");e=g[0];g=null!=g[1]?g[1]:null;e=this.Qk._web_links[e];null!=g&&(e=e[g]);null!=e?d.push(this.client.request(e,"GET")):d.push(void 0)}return d};function To(){Cc.call(this,"navigate")}r(To,Cc);function Uo(){return!(Cb("iPad")||Cb("Android")&&!Cb("Mobile")||Cb("Silk"))&&(Cb("iPod")||Cb("iPhone")||Cb("Android")||Cb("IEMobile"))};function Vo(a,b){for(var c=[a],d=b.length-1;0<=d;--d)c.push(typeof b[d],b[d]);return c.join("\x0B")};function Wo(a,b,c,d){md.call(this);if(a&&!b)throw Error("Can't use invisible history without providing a blank page.");if(c)var e=c;else{e="history_state"+Xo;var g=$e("input",{type:"text",name:e,id:e,style:Be("display:none")});document.write(Te(g));e=lf(document,e)}this.Bi=e;this.Me=c?wf(kf(c)):window;this.mo=b;t&&!b&&(this.mo="https"==window.location.protocol?Ke(Ae(Be("https:///"))):Ke(Ae(Be('javascript:""'))));this.wa=new nd(Yo);$b(this,va(ac,this.wa));this.Ag=!a;this.Ub=new fh(this);if(a||Zo){if(d)var h=
d;else{a="history_iframe"+Xo;d=this.mo;b={id:a,style:Be("display:none"),sandbox:void 0};d&&Ie(d);c={};c.src=d||null;c.srcdoc=null;d={sandbox:""};e={};for(h in c)e[h]=c[h];for(h in d)e[h]=d[h];for(h in b){g=h.toLowerCase();if(g in c)throw Error('Cannot override "'+g+'" attribute, got "'+h+'" with value "'+b[h]+'"');g in d&&delete e[g];e[h]=b[h]}h=af("iframe",e,void 0);document.write(Te(h));h=lf(document,a)}this.Wa=h;this.Pt=!0}Zo&&(this.Ub.A(this.Me,"load",this.Bw),this.tt=this.hn=!1);this.Ag?$o(this,
this.Sf(),!0):ap(this,this.Bi.value);Xo++}r(Wo,md);Wo.prototype.Tb=!1;Wo.prototype.kh=!1;Wo.prototype.ih=null;var bp=function(a,b){var c=b||Vo;return function(){var b=this||ca,b=b.closure_memoize_cache_||(b.closure_memoize_cache_={}),e=c(pa(a),arguments);return b.hasOwnProperty(e)?b[e]:b[e]=a.apply(this,arguments)}}(function(){return t?vc(8):"onhashchange"in ca}),Zo=t&&!vc(8);f=Wo.prototype;f.jh=null;f.V=function(){Wo.m.V.call(this);this.Ub.ib();this.pa(!1)};
f.pa=function(a){if(a!=this.Tb)if(Zo&&!this.hn)this.tt=a;else if(a)if(gc?this.Ub.A(this.Me.document,cp,this.Jw):jc&&this.Ub.A(this.Me,"pageshow",this.Gw),bp()&&this.Ag)this.Ub.A(this.Me,"hashchange",this.Cw),this.Tb=!0,this.dispatchEvent(new To(this.Sf()));else{if(!t||Uo()||this.hn)this.Ub.A(this.wa,"tick",q(this.Zq,this,!0)),this.Tb=!0,Zo||(this.ih=this.Sf(),this.dispatchEvent(new To(this.Sf()))),this.wa.start()}else this.Tb=!1,this.Ub.sd(),this.wa.stop()};
f.Bw=function(){this.hn=!0;this.Bi.value&&ap(this,this.Bi.value,!0);this.pa(this.tt)};f.Gw=function(a){a.bd.persisted&&(this.pa(!1),this.pa(!0))};f.Cw=function(){var a=dp(this.Me);a!=this.ih&&ep(this,a)};f.Sf=function(){return null!=this.jh?this.jh:this.Ag?dp(this.Me):fp(this)||""};f.ql=function(a,b){this.Sf()!=a&&(this.Ag?($o(this,a,!1),bp()||t&&!Uo()&&ap(this,a,!1,b),this.Tb&&this.Zq()):(ap(this,a,!1),this.jh=this.ih=this.Bi.value=a,this.dispatchEvent(new To)))};
function dp(a){a=a.location.href;var b=a.indexOf("#");return 0>b?"":a.substring(b+1)}function $o(a,b,c){a=a.Me.location;var d=a.href.split("#")[0],e=Pa(a.href,"#");if(Zo||e||b)d+="#"+b;d!=a.href&&(c?a.replace(d):a.href=d)}
function ap(a,b,c,d){if(a.Pt||b!=fp(a))if(a.Pt=!1,b=encodeURIComponent(String(b)),t){var e=Jf(a.Wa);e.open("text/html",c?"replace":void 0);c=bf($e("title",{},d||a.Me.document.title),$e("body",{},b));e.write(Te(c));e.close()}else if(e=Ie(a.mo)+"#"+b,a=a.Wa.contentWindow)c?a.location.replace(e):a.location.href=e}
function fp(a){if(t)return a=Jf(a.Wa),a.body?Ea(a.body.innerHTML):null;var b=a.Wa.contentWindow;if(b){try{var c=Ea(dp(b))}catch(d){return a.kh||(1!=a.kh&&a.wa.setInterval(gp),a.kh=!0),null}a.kh&&(0!=a.kh&&a.wa.setInterval(Yo),a.kh=!1);return c||null}return null}f.Zq=function(){if(this.Ag){var a=dp(this.Me);a!=this.ih&&ep(this,a)}if(!this.Ag||Zo)if(a=fp(this)||"",null==this.jh||a==this.jh)this.jh=null,a!=this.ih&&ep(this,a)};
function ep(a,b){a.ih=a.Bi.value=b;a.Ag?(Zo&&ap(a,b),$o(a,b)):ap(a,b);a.dispatchEvent(new To(a.Sf()))}f.Jw=function(){this.wa.stop();this.wa.start()};var cp=["mousedown","keydown","mousemove"],Xo=0,Yo=150,gp=1E4;function hp(a,b){var c;for(c in b){var d=b[c];Jb(a,c)||(a[c]=d)}return a}function ip(a,b){var c,d;var e={};if(null!=a.length){var g=c=0;for(d=a.length;c<d;g=++c){var h=a[g];g=b(h,g);h=g[0];var k=g[1];e[h]=k}}else for(g in a)h=a[g],c=b(h,g),h=c[0],k=c[1],e[h]=k;return e}function jp(a,b,c){a=a||{};if(0==b.length)b=a;else{if(1===b.length)var d=a||{};else d=a,c=jp(Ob(a,b[0]),sb(b,1),c),d=d||{};d[b[0]]=c;b=d}return b}
function kp(a,b,c){null==c&&(c=null);a=a||{};return 0==b.length?a:1===b.length?Ob(a,b[0],c):kp(Ob(a,b[0],c),sb(b,1),c)};function lp(a,b){this.Qc=a;this.bs=null!=b?b:"";this.Hl=q(this.Hl,this);this.$n=q(this.$n,this);this.Pn=q(this.Pn,this);this.On=q(this.On,this);lp.m.constructor.call(this);this.history=new Wo;this.history.pa(!0);this.base=mp;this.Yl="grid"}r(lp,md);var np={qj:"b",Ih:"p"},mp=0,op={qj:mp,Ih:1},po="state-changed";f=lp.prototype;f.Ge=function(a){this.base=a};f.An=function(){return Nm(this.Qc.get("p"),1)};
f.Rf=function(){var a=Qm(ca.localStorage.getItem(this.Yl))||{};a=Nm(kp(a,[this.bs,"b"]));return 0<a?a:this.base};f.handle=function(a,b){this.gb=a;this.Le=b;this.gb.A("changed",this.$n);this.Le.A("filter-changed",this.On);return this.Le.A("reset-paginator",this.Pn)};f.Dd=function(){return this.dispatchEvent(new Cc(po,{}))};f.On=function(){this.gb.reset(this.vc());return this.Hl({page:1})};f.Pn=function(){return this.gb.reset(this.vc)};f.$n=function(a){return this.Hl(a.target)};
f.Hl=function(a){if(null!=a){if(null!=a.base){var b={};b[np.qj]=a.base;var c=Qm(ca.localStorage.getItem(this.Yl))||{};for(g in b){var d=b[g];jp(c,[this.bs,g],d)}ca.localStorage.setItem(this.Yl,rd(c))}b={};b[np.Ih]=a.page!==op.Ih?a.page:null;c=this.Le;a={};var e=c.Ma;for(k in e){var g=e[k];var h=d=g.ba();(null!=c.hi[g.getName()]&&""===h||""!==h)&&(a[g.lk()]=d)}Qb(a,b);this.Qc.Ui();var k=this.Qc;b=!0;var l;null==b&&(b=!1);for(l in a)c=a[l],pp(k.Gb,k.key(l),c,b);k.hs.hash=k.Gb.toString();return this.Dd()}};
f.vc=function(){return this.Le.vc()};function qp(a,b,c,d){this.grid=a;this.client=b;this.url=c;this.interval=null!=d?d:3E3;this.update=q(this.update,this);this.fetch=q(this.fetch,this);this.timer=new nd(this.interval);this.timer.A("tick",this.fetch)}qp.prototype.start=function(){if(1<this.interval)return this.timer.start()};qp.prototype.fetch=function(){this.timer.stop();return this.client.find(this.url,this.update)};qp.prototype.update=function(a){0<a["count-changed"]&&this.grid.refresh();return this.timer.start()};function rp(a){this.Ye=q(this.Ye,this);rp.m.constructor.call(this,a);this.Le=a.Le;this.client=a.client;this.ext=this.type=null}r(rp,Hn);rp.prototype.Ca=function(a){var b;rp.m.Ca.call(this,a);v(a,"click",this.Ye);this.Ks=new yk(this.o().href);this.qo=this.Ks.qb;var c=this.Le.grid.cols;var d=0;for(b=c.length;d<b;d++)a=c[d],this.qo.remove(a)};rp.prototype.getExtension=function(){var a;this.ext||(this.ext=null!=(a=Rh(this.o(),"type"))?a:"",this.ext=String(this.ext));return this.ext};
rp.prototype.Ye=function(){var a=this.o(),b;var c=this.Le.vc().uri.clone();Ck(c,this.Ks.pd);var d=this.qo.uc();var e=0;for(b=d.length;e<b;e++){var g=d[e];null==c.qb.get(g)&&Mk(c,g,this.qo.Ib(g))}return a.href=c.toString()};function sp(a){this.Qo=a;this.get=q(this.get,this);this.set=q(this.set,this)}sp.prototype.set=function(a,b){return/string|number|boolean/.test(typeof b)?this.Qo.setItem(a,b):this.Qo.setItem(a,rd(b))};sp.prototype.get=function(a){var b;a=this.Qo.getItem(a);if(null===a)return a;if("string"===typeof a||a instanceof String)if("["===(b=a.trim().substring(0,1))||"{"===b)try{a=qd(a)}catch(c){}return a};function tp(a,b,c){this.l=a;this.mi=b;this.grid=c;this.toArray=q(this.toArray,this);this.show=q(this.show,this);this.cn=q(this.cn,this);this.Ip=q(this.Ip,this);this.$p=q(this.$p,this);this.ui=q(this.ui,this);this.hl=q(this.hl,this);this.Dp=q(this.Dp,this);this.pn=q(this.pn,this);this.nc=q(this.nc,this);this.Dn=q(this.Dn,this);this.yn=q(this.yn,this);this.zn=q(this.zn,this);this.Ui=q(this.Ui,this);this.render=q(this.render,this);this.Qn=q(this.Qn,this);this.J=q(this.J,this);tp.m.constructor.call(this);
this.headers=[];this.cols={};this.fk={};this.$q=[];this.fq=[];this.xj=this.grid.cols;this.Qp=["columns_storage",this.grid.table.id].join("-");this.Dr="";this.Ue=null;this.tf=new sp(ne(this.l).localStorage)}r(tp,xh);f=tp.prototype;
f.J=function(a){this.G=a;if(this.grid.table.id||""===this.grid.table.id){tp.m.J.call(this,this.G);this.headers=this.l.all("thead \x3e tr \x3e th",this.grid.table);this.fq=this.Dn();this.cols=this.zn();this.fk=this.yn();if(this.Ue=this.l.o(["columns-filtering-text",this.grid.table.id].join("-")))return this.Dr=this.Ue.innerHTML,this.l.hide(this.Ue),v(this.Ue,"click",this.cn),v(this.grid,"loaded",this.Qn),v(this.mi,"filter-changed",this.pn),this.render();ne(this.l).console.warn("Missing filtering text element for #"+
this.G.id)}else ne(this.l).console.warn("Missing table id attribute for "+this.grid.table.className)};f.Qn=function(){return this.show(this.ui())};function up(a,b,c){var d=a.l.G("li","managed-col");var e=[a.grid.table.id,"manage-",b].join("");var g=a.l.G("input",{type:"checkbox",name:b,checked:null!=a.cols[b]?a.cols[b]:!0,id:e},d);b=a.l.G("label",{"class":a.fk[b]?"is-filtered":"","for":e},a.fq[b]);a.l.append(d,b);v(g,"click",a.nc);a.l.append(c,d);return g}
f.render=function(){var a=this.l.G("ul","managed-columns");var b,c;var d=this.xj;var e=[];var g=b=0;for(c=d.length;b<c;g=++b)g=d[g],e.push(up(this,g,a));this.$q=e;this.hl(this.ui());this.Ui();return this.l.append(this.G,a)};f.Ui=function(){return this.l.removeNode(Eg(this.l,"managed-columns",this.G))};f.zn=function(){var a;var b=this.tf.get(this.Qp);if(!b){b={};var c=this.xj;var d=0;for(a=c.length;d<a;d++){var e=c[d];b[e]=!0}this.tf.set(this.Qp,b)}return b};
f.yn=function(){return ip(this.mi.Ma,function(a){return[a.getName(),""!==a.ba()]})};f.Dn=function(){var a=this;return ip(this.l.all("th \x3e span",this.grid.table),function(b,c){return[a.xj[c],a.l.Vg(b)]})};f.nc=function(){this.cols=this.ui();this.show(this.cols);return this.hl(this.cols)};f.pn=function(){this.Dp(this.mi.vc());return this.render()};f.Dp=function(a){var b;var c=this.fk;var d=[];for(b in c)d.push(this.fk[b]=null!=a.filters[b]);return d};
f.hl=function(a){this.$p(a);return this.tf.set(this.Qp,a)};f.ui=function(){return ip(this.$q,function(a){return[a.name,null!=a.checked&&a.checked]})};f.$p=function(a){var b;var c=[];for(b in a){var d=a[b];!d&&this.mi.vc().filters.ke(b)&&c.push(this.fq[b].toLowerCase())}return 0<c.length?(this.Ue.innerHTML=this.Dr.replace("%(columns)s",c.join(", ")),this.l.show(this.Ue)):this.l.hide(this.Ue)};
f.Ip=function(){var a;var b=this.headers;var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];null!=this.cols[e.className]?c.push(w(e,this.cols[e.className])):c.push(void 0)}return c};f.cn=function(){return Po(this.mi)};f.show=function(a){var b=this.grid;a=this.toArray(a);b.Bp(a);b.gl.Bp(a);so(b);return this.Ip()};f.toArray=function(a){var b;var c=this.xj;var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];a[g]&&d.push(g)}return d};function vp(a,b,c,d,e){var g=this;this.l=a;this.client=b;this.url=c;this.rf=d;this.register=e;this.yi=q(this.yi,this);vp.m.constructor.call(this);this.dc=new Ao(void 0,void 0,this.l);om(this.dc,null);Cm(gm(this.dc),function(a){return g.dispatchEvent(new Cc("action",a))})}r(vp,md);vp.prototype.open=function(){return xl(this.client,this.url,this.yi)};vp.prototype.yi=function(a){if(a.snippets[this.rf])return this.dc.oc(a.snippets[this.rf]),this.np(a)};
vp.prototype.np=function(){var a=this.dc.ub();this.register.xc(a);this.dc.$(!0);Zg(this.dc.o(),this.rf);return this.dispatchEvent(new Cc("open",this.dc))};vp.prototype.dj=function(a){return this.dc.dj(a)};var wp={};function xp(a,b){a.removeAttribute("disabled");var c=Eg(b,"btn-spinner",a);if(null!=c)return b.removeNode(c)}function yp(a,b){var c;var d=b.all(".form-btns button.btn.disable-on-submit",a);var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];e.push(xp(h,b))}return e}
function zp(a,b){var c;if(!dh(a,wp.Ax)){var d=b.all(".form-btns button.btn.disable-on-submit",a);var e=function(a){var c=a.currentTarget;Eg(b,"btn-spinner",c)||b.appendChild(c,b.G("i","fa fa-refresh fa-spin btn-spinner"));if(null!=a.target.name)return b.appendChild(a.target.parentNode,b.G("input",{type:"hidden",name:a.target.name,value:a.target.value}))};var g=0;for(c=d.length;g<c;g++){var h=d[g];Cm(h,e)}v(a,"submit",function(){var a;var b=[];var c=0;for(a=d.length;c<a;c++)h=d[c],b.push(h.setAttribute("disabled",
!0));return b})}};function Ap(a,b,c,d,e){this.l=a;this.client=b;this.grid=c;this.$b=d;this.ka=e;this.bo=q(this.bo,this);this.handleSelect=q(this.handleSelect,this);this.Jb=q(this.Jb,this);this.handleError=q(this.handleError,this);this.gn=q(this.gn,this);this.Zk=q(this.Zk,this);this.Om=q(this.Om,this);this.Uf=q(this.Uf,this);this.Zf=q(this.Zf,this);this.wc=q(this.wc,this);this.Cc=new Yn({l:this.l});this.nn={};this.$i={};this.aj=[];this.Ma=this.form=null}
var Bp={SELECTED:"field-selected",Kh:"field-unselected"},Cp={tj:"formSnippet",Zt:"apiUrl",gu:"formUrl"};f=Ap.prototype;f.J=function(a){this.G=a;a=[Cp.tj,Cp.Zt,Cp.gu];var b,c;var d=[];var e=0;for(c=a.length;e<c;e++){var g=a[e];if(b=Mm(this.G,g))d.push(String(b));else throw Error("Missing `data-"+g+"` for BulkChange");}this.Hr=d[0];this.Tu=d[1];this.Hv=d[2];this.Cc.J(this.G);this.Cc.A("action",this.wc);this.Cc.pa(!1);return v(this.grid,"selection-change",this.bo)};
f.wc=function(){if(this.Cc.isEnabled()){var a=new vp(this.l,this.client,[this.Hv,"?snippet\x3d",this.Hr].join(""),this.Hr,this.$b);a.A("open",this.Jb);return a.open()}};f.Zf=function(){var a,b;var c={};var d=this.$i;for(a in d){var e=d[a];c[a]=Em(e)}return this.client.request([this.Tu,"?id__in\x3d",rd(function(){var a;var c=this.aj;var d=[];var e=0;for(a=c.length;e<a;e++)b=c[e],d.push(b.Sa.id);return d}.call(this))].join(""),"PUT",c,this.Uf,this.handleError)};f.Uf=function(){return ne(this.l).location.reload(!0)};
f.Om=function(a){var b;a=Jm(null!=(b=a.messages)?b.errors:void 0,function(a){return a.id});return Lm(a)?null:ym(Mm(this.G,"bulkErrorMessage",String,"Please fix errors for following objects: %s"),a.join(", "))};f.Zk=function(a){a=Eg(this.l,"errorlist",a);if(null!=a)return this.l.removeNode(a)};f.gn=function(a){var b;var c=this.nn;var d=[];for(b in c){var e=c[b];Jb(a,b)?(ki(e,"error"),this.Zk(e),d.push(this.l.appendChild(e,this.l.G("ul","errorlist",[this.l.G("li",null,a[b])])))):(mi(e,"error"),d.push(this.Zk(e)))}return d};
f.handleError=function(a){var b=this.Om(a);null!=b&&Mh(this.ka,b);this.gn(Km(a.messages.errors,function(a,b){return hp(a,b.errors)}));if(null!=this.form)return yp(this.form,this.l)};
f.Jb=function(a){var b;this.$i={};this.aj=this.grid.Ug();a=a.target.ub();var c=Eg(this.l,"btn-save",a);null!=c&&(this.zl=new Yn({l:this.l}),this.zl.J(c),this.zl.pa(!1),this.zl.A("action",this.Zf));this.form=Dg(this.l,"form",a);null!=this.form&&v(this.form,"submit",function(a){return a.preventDefault()});c=Eg(this.l,"bulk-change-object-summary",a);null!=c&&this.l.Yc(c,ym(this.l.Vg(c),this.aj.length));var d=Fg(this.l,"field",a);var e=[];var g=0;for(b=d.length;g<b;g++)a=d[g],c=Dp(Ep(this,a)),Fp(this,
a,Dp(Ep(this,a))),e.push(this.nn[c]=a);return e};function Fp(a,b,c){Zg(b,"field-unselected");var d=new jo({l:a.l});d.render(b);Qh(d.o(),"target",c);d.A(Wn,a.handleSelect)}f.Bl=function(a,b){var c,d;var e=b?[Bp.SELECTED,Bp.Kh]:[Bp.Kh,Bp.SELECTED];var g=[Zg,ah];var h=[];var k=c=0;for(d=g.length;c<d;k=++c){var l=g[k];h.push(l(a,e[k]))}return h};
f.handleSelect=function(a){var b=this.nn[Mm(a.target.target,"target")];if(null==b)ne(this.l).console.warn("Given an invalid checkbox ID "+nh(a.target)+" for BulkChange");else{a=a.target.target.checked;this.Bl(b,a);var c=Ep(this,b);if(null==c)ne(this.l).console.warn("Missing an input in field ."+b.className);else return a?this.$i[Dp(c)]=c:Mb(this.$i,Dp(c)),this.zl.pa(a||Fb(this.$i))}};function Ep(a,b){return Dg(a.l,"input, select, textarea",b)}
function Dp(a){return Hm(a.name.split("-").slice(-1))}f.bo=function(){this.aj=this.grid.Ug();return this.Cc.pa(0<this.aj.length)};function Gp(){this.Pm=q(this.Pm,this);Gp.m.constructor.call(this);this.Ad.push("paginator");this.Xr=[[!1,!1],[!1,!1]];this.lj=this.Ie=null;this.wg="%d per page";this.Li="LI";this.Do="SPAN";this.Ms=Hp;this.et=[this.tv,this.Pm];this.Rd=Ip;this.fi=Jp;this.ei=Kp}r(Gp,wh);
var Lp={Fh:"active",Gh:"disabled",xq:"previous",$l:"next",dm:"result",fm:"result-number",yq:"result-caption",gm:"result-total",zq:"result-displayed",Dg:"paging",bm:"pagination",am:"page-item",Rl:"base-switcher",uu:"paginator"},Mp={Ih:"p",Dg:"paging",Sl:"baseType",Ql:"baseRange",Ul:"customBaseLabel",Tl:"customBaseErrorMessage"},Hp="simple",Ip=[1,1E3],Jp="Display number of rows",Kp="Row number has to be between %s and %s",Np=[["Items total:","%d"],["Displayed","%d to %d"]];f=Gp.prototype;
f.D=function(a){var b,c;var d=Gp.m.D.call(this,a);var e=a.I();for(b=c=0;1>=c;b=++c){var g=ym.apply(ym,(0,this.et[b])(a));b=d;var h=a,k=g,l=g=h.I().D(this.Do,{});h.I().Yc(l,k);b.appendChild(g)}Op(this,a,d,e);this.Oh(a,d,e);return d};
function Pp(a,b){var c,d,e;var g=[];if(2>b.Vk)g=[b.page];else{var h=(b.Vk-1)/4;var k=Math.max(b.Er,b.page-3);var l=Math.min(b.PA,b.page+3);g=function(){e=[];for(var a=k;k<=l?a<=l:a>=l;k<=l?a++:a--)e.push(a);return e}.apply(a);for(c=d=0;4>=d;c=++d)g.push(Math.round(h*c)+b.Er);ub(g);g.sort(function(a,b){return a<b?-1:a>b?1:0})}return g}function Qp(a,b,c,d,e,g){a.Xr[e][g]||(c=Eg(b,c,d),d="",null!=c&&(d=b.Vg(c)),null!=d&&""!==d&&(Np[e][g]=d),a.Xr[e][g]=!0)}
f.bn=function(a,b){var c=Eg(a,"paginator__error-message",b);if(null!=c)return a.removeNode(c)};f.Um=function(a,b){var c=this.Rd;c=a.G("div","paginator__error-message",[a.G("div","alert alert-danger",ym(this.ei,c[0],c[1]))]);return a.appendChild(b,c)};f.zi=function(a,b){var c=this;return Bm(b,function(){var d=a.I();var e=Eg(d,"paginator__custom-base-input",b);if(null!=e){var g=Yf(b);e=Nm(Em(e));return Pm(c.Rd,e)?(c.bn(d,g),a.Ge(e)):c.Um(d,g)}})};
f.Tm=function(a,b){var c=a.I();var d=Eg(c,"result-displayed",b);if(null==d)return null;var e=Yf(d);d=c.G("span","paginator__custom-base-wrapper");Vf(e,d);c.appendChild(d,c.G("span","paginator__custom-base-label",this.fi));var g=this.Rd;e=g[0];g=g[1];e=c.G("input",{type:"number",value:a.Rf(),"class":"paginator__custom-base-input",min:e,max:g});c.appendChild(d,e);return this.zi(a,e)};f.an=function(a,b){var c=a.I();var d=Eg(c,"paginator__custom-base-wrapper",b);if(null!=d)return c.removeNode(d)};
f.xm=function(a){if(la(a)&&2===a.length&&jb(a,Om))return a;ca.console.warn('Paginator "data-base-rage" contains invalid value. A tuple with two positive number was expected.');return Ip};
f.J=function(a,b){var c,d,e,g;var h=a.I();var k=[Eg(h,Lp.gm,b),Eg(h,Lp.zq,b)];var l=c=0;for(g=k.length;c<g;l=++c){var m=k[l];null!=m&&(Qp(this,h,Lp.yq,m,l,0),Qp(this,h,Lp.fm,m,l,1),m=Eg(h,Lp.fm,m),null!=m&&(l=ym.apply(ym,(0,this.et[l])(a)),a.I().Yc(m,l)))}c=Eg(h,Lp.gm,b);null!=c&&ag(c,"display",isNaN(a.total)?"none":"inline");this.Ms=Mm(b,Mp.Dg,String,Hp);c=null!=(d=Eg(h,Lp.bm,b))?d:b;this.Zm(a,c,h);this.fi=Mm(b,Mp.Ul,String,Jp);this.ei=Mm(b,Mp.Tl,String,Kp);this.Rd=Mm(b,Mp.Ql,bb(this.xm,Qm));"custom"===
Mm(b,Mp.Sl)&&(this.an(a,b),this.Tm(a,b));d=null!=(e=Eg(h,Lp.Dg,b))?e:b;this.Ie?Vf(d,this.Ie):(e=Eg(h,Lp.Rl,b),this.$m(a,e,h));return ag(b,"visibility","inherit")};f.gp=function(a){a=Mm(a,"pattern");if(null!=a)return this.wg=a};f.Pm=function(a){return[Np[1][1],0===(0===a.total?0:a.offset+a.count)||0===a.total?0:a.offset+1,0===a.total?0:a.offset+a.count]};f.tv=function(a){return[Np[0][1],a.total]};
f.Zm=function(a,b,c){var d=Eg(c,Lp.xq,b);isNaN(a.pg)||0===a.total?this.Bk(d):this.ml(d,a.page-1);var e=c.R.createDocumentFragment();"full"===this.Ms&&Op(this,a,e,c);c.Ki(e,d);b=Eg(c,Lp.$l,b);return a.yo()||0===a.total?this.Bk(b):this.ml(b,a.page+1)};
function Op(a,b,c,d){var e,g;var h=Pp(a,b);var k=[];var l=e=0;for(g=h.length;e<g;l=++e){var m=h[l];var p=d.G(a.Do,{},String(m));var u=d.G(a.Li,"page-item");m===b.page?a.Jq(u):a.ml(u,m);u.appendChild(p);c.appendChild(u);h[l+1]>m+1?k.push(c.appendChild(d.D(a.Li,{},"..."))):k.push(void 0)}}f.ml=function(a,b){ah(a,"disabled");return Qh(a,"p",String(b))};f.Jq=function(a){return this.kj(a,Lp.Fh,Lp.Gh)};f.Bk=function(a){return this.kj(a,Lp.Gh,Lp.Fh)};f.kj=function(a,b,c){Zg(a,b);ah(a,c);return Sh(a,"p")};
f.Im=function(a){var b;var c=a.I();var d=a.o();if(null!=d){var e=Xf(d);c.removeNode(d);var g=Fg(c,Lp.am,d);var h=0;for(b=g.length;h<b;h++)a=g[h],c.removeNode(a);g=Fg(c,Lp.dm,d);h=0;for(b=g.length;h<b;h++)a=g[h],a.innerHTML="";return c.vo(d,e)}};f.$m=function(a,b,c){if(null!=b)return this.gp(b),this.Oh(a,b,c),Fm(this.lj,String(a.base))};f.Oh=function(a,b,c){a=this.Vm(a,c);b.appendChild(a);this.lj=a;return this.Ie=b};
f.Vm=function(a,b){var c,d=this;var e=b.D("div","dropdown");this.Hc=b.D("button","btn btn-default dropdown-toggle");this.ug(this.Hc,a.base);b.appendChild(e,this.Hc);var g=new qn(this.l);g.$(!1);var h=a.Bf;var k=0;for(c=h.length;k<c;k++){var l=h[k];var m=new kn(ym(this.wg,l),l,this.l,rn.ua());g.$a(m,!0)}v(g,"action",function(c){l=c.target.Sa;g.$(!1);b.appendChild(e,d.Hc);d.ug(d.Hc,l);return a.Ge(l)});g.render(e);v(this.Hc,"click",function(){return g.$(!g.va())});v(b.R.body,"click",function(){if(g.va())return g.$(!1)},
!0);return e};f.ug=function(a,b){if(null!=b&&null!=a)return a.innerHTML=[ym(this.wg,b),'\x3cspan class\x3d"caret"\x3e\x3c/span\x3e'].join("")};f.Ge=function(a){return this.ug(this.Hc,a)};f.Bn=function(a){return Eg(a.I(),"pagination",a.o())};f.An=function(a){for(;;){if(!a||dh(a,"pagination"))return null;if(a.tagName===this.Li&&Th(a,"p"))return Mm(a,"p");a=Yf(a)}};function Rp(a){this.Af=q(this.Af,this);null==a.K&&(a.K=new Gp);Rp.m.constructor.call(this,a);a=a.sf;this.base=a.Rf();this.page=a.An();this.base=0<=this.base?this.base:zn;this.page=0<=this.page?this.page:1;this.Er=1;this.ge=[];this.Na=[];this.Ie=null;this.Bf=this.gi=[10,25,50,100,500,1E3];this.oi=!1}r(Rp,yn);f=Rp.prototype;f.Rf=function(){return this.base};f.Af=function(a){return Im([this.base,a],Om,zn)};f.gs=function(a){this.base=Mm(a,"base",bb(this.Af,Nm));return this.offset=(this.page-1)*this.base};
f.init=function(a){this.total=a.total;this.count=a.count;this.pg=a.pg;this.lg=a.lg;Sp(this);return null!=this.count?this.count:this.count=this.base};function Sp(a){return a.Vk=Math.ceil(a.total/a.base)}f.rb=function(a){return null!=a&&dh(a,"paginator")};f.xf=function(){return this.Ai(this.K.Bn(this))};f.yo=function(){return isNaN(this.lg)};
f.refresh=function(a){var b;this.total=a.total;this.count=a.count;this.pg=a.pg;this.lg=a.lg;null!=a.offset&&(this.offset=a.offset);Sp(this);this.K.Im(this);this.Ca(this.o());this.xf();var c=[];var d=this.ge;var e=0;for(b=d.length;e<b;e++){var g=d[e];a=this.clone();Wf(a,g);c.push(a)}this.ge=c;return this.show(!0)};f.Ge=function(a){this.base=a;this.offset=0;this.page=1;Sp(this);return this.Dd()};
f.Ca=function(a){this.oi="true"===Mm(a,"forceDisplay");if(!this.Bf){var b=a.querySelector(".base-switcher");this.fp(b)}this.K.J(this,a);this.F=a;return this.vg(!1)};f.fp=function(a){if(null!=a||this.Bf)return Mm(a,"bases",qd,this.gi)};f.ml=function(a){this.page=a;this.offset=(this.page-1)*this.base;return Sp(this)};f.Ai=function(a){var b=this;a=v(a,"click",function(a){a=b.K.An(a.target,b.l);if(null!=a)return b.page=Nm(a),b.offset=(b.page-1)*b.base,b.Dd()});return this.Na.push(a)};
f.Dd=function(){this.Hm();return this.dispatchEvent(new Cc("changed",{offset:this.offset,base:this.base,page:this.page}))};f.clone=function(){var a=this.o().cloneNode(!0);this.Ai(a);Zg(this.o(),"top");Zg(a,"bottom");ah(a,"top");this.K.zi(this,a);return a};f.rr=function(){if(null==this.o())return null;var a=this.clone();this.ge.push(a);return a};f.Hm=function(){var a;var b=this.Na;var c=0;for(a=b.length;c<a;c++){var d=b[c];ed(d)}return this.Na=[]};
f.show=function(a,b){null==b&&(b=!1);return!b&&2>this.Vk&&!this.oi?this.vg(!1):this.vg(a)};f.vg=function(a){var b;var c=a?ah:Zg;var d=[this.o()].concat(this.ge);var e=[];var g=0;for(b=d.length;g<b;g++)a=d[g],e.push(c(a,"empty"));return e};f.reset=function(a){this.offset=0;this.page=1;return this.yd(a)};f.yd=function(a){a.base=this.base;return a.offset=this.offset};f.clearData=function(){return!0};f.Ys=function(){return!0};f.$s=function(){return!1};var Tp;function Up(){return Tp=Up.m.constructor.apply(this,arguments)}r(Up,sn);Up.prototype.D=function(a){var b=Up.m.D.call(this,a);var c=a.I();this.Oh(a,b,c);return b};
Up.prototype.J=function(a,b){var c,d;var e=a.I();var g=null!=(d=Eg(e,wn.bm,b))?d:b;this.Zm(a,g,e);this.fi=Mm(b,xn.Ul,String,un);this.ei=Mm(b,xn.Tl,String,vn);this.Rd=Mm(b,xn.Ql,bb(this.xm,Qm));"custom"===Mm(b,xn.Sl)&&(this.an(a,b),this.Tm(a,b));g=null!=(c=Eg(e,wn.Dg,b))?c:b;this.Ie?Vf(g,this.Ie):(c=Eg(e,wn.Rl,b),this.$m(a,c,e));return ag(b,"visibility","inherit")};Up.prototype.Zm=function(a,b,c){b=Eg(c,wn.$l,b);a.yo()?a=this.Bk(b):(a=a.Pc,ah(b,"disabled"),a=Qh(b,"c",a));return a};
Up.prototype.kj=function(a,b,c){Zg(a,b);ah(a,c);return Sh(a,"c")};function Vp(a){null==a.K&&(a.K=new Up);Vp.m.constructor.call(this,a);this.Pc=this.cursor=null}r(Vp,yn);f=Vp.prototype;f.init=function(a){this.total=a.total;this.count=a.count;this.Pc=a.Pc;null==this.Pc&&(this.Pc=null);return null!=this.count?this.count:this.count=this.base};f.yo=function(){return null===this.Pc};
f.refresh=function(a){var b;this.total=a.total;this.count=a.count;this.pg=a.pg;this.lg=a.lg;this.Pc=a.Pc?a.Pc:null;this.K.Im(this);this.Ca(this.o());this.xf();var c=[];var d=this.ge;var e=0;for(b=d.length;e<b;e++){var g=d[e];a=this.clone();Wf(a,g);c.push(a)}this.ge=c;return this.show(!0)};f.Ge=function(a){this.base=a;this.Pc=this.cursor=null;return this.Dd()};
f.Ai=function(a){var b=this;a=v(a,"click",function(a){a:{var c=b.K;for(a=a.target;;){if(!a||dh(a,"pagination")){c=null;break a}if(a.tagName===c.Li&&Th(a,"c")){c=Mm(a,"c");break a}a=Yf(a)}}if(null!=c)return b.cursor=c,b.Dd()});return this.Na.push(a)};f.Dd=function(){this.Hm();return this.dispatchEvent(new Cc("changed",{base:this.base,cursor:this.cursor}))};f.reset=function(a){this.Pc=this.cursor=null;return this.yd(a)};f.yd=function(a){a.base=this.base;return a.cursor=this.cursor};
f.clearData=function(){return null===this.Pc};f.Ys=function(){return!1};f.$s=function(){return!0};function Wp(){var a;null==a&&(a={});return this.Br=a.Br,a}
function Xp(a,b,c,d,e,g,h){var k=no,l,m;null==h&&(h=new Wp);var p=new Co;var u=p.Tg(a,"context");u=new ql(c,u);var A=new to(a);var D=new xo(u);var E=new ml(p.Tg(a,"resource"));var J=wo(A);var M=0;for(m=J.length;M<m;M++){var O=J[M];E.Nf.push(O)}0<E.Nf.length&&(u.headers["X-Fields"]=E.Nf.join(","));tl(u,E.rx());u.headers["X-Fields"]=E.Nf.join(",");M=new lp(e,a.id);O=null!=(null!=(l=Yf(a))?l.querySelector(".paginator.cursor-based-paginator"):void 0)?new Vp({sf:M}):new Rp({sf:M});l=new Bo(void 0,void 0,
b);var aa=new k(b,D,uo(A),M,l,E,O,g,"true"===A.Tg("rowSelectable"));var ra=new Mo(aa,E,e,b);aa.qt(ra.gq(a));aa.J(a);if(k=p.Tg(a,"columnsManager"))if(k=b.o(k))p=new tp(b,ra,aa),p.J(k);M.handle(O,ra);M.Ge(O.Rf());(new Do(aa)).J(Yf(a));So(a)&&(nb(E.Nf,"_default_action"),e=new Qo(u,b,d,e,E),e.gq(aa));Th(a,"updateUrl")?(u=String(Rh(a,"updateUrl")),e=Nm(String(Rh(a,"updateInterval")),3E3),E=new je({ig:!1}),e=new qp(aa,new ql(c,"",E),u,e),e.start()):Th(a,"interval")&&Yp(aa);h=h.Br;null==h&&(h=Dg(b,".export-buttons"));
null==h&&(h=Yf(a));u=Fg(b,"export",h);E=0;for(k=u.length;E<k;E++)h=u[E],e=new rp({l:b,Le:ra,client:new ql(c)}),e.J(h);Th(a,"bulkChange")&&(h=b.o(String(Rh(a,"bulkChange"))),null!=h&&(b=new Ap(b,new ql(c),aa,d,g),b.J(h)));b=Yf(a).querySelector(".reset-filters");v(b,"click",function(){return Po(ra)});b=Yf(a).querySelector(".reload");v(b,"click",function(){return aa.refresh()});b=Yf(a).querySelector(".reload-count");v(b,"click",function(){return aa.qp()});return aa}
function Yp(a){var b=Nm(String(Rh(a.table,"interval")),0);1<b&&(b=new nd(b),b.A("tick",function(){return a.refresh()}),b.start())};function Zp(a,b){this.client=a;this.l=b;this.od=q(this.od,this);Zp.m.constructor.call(this);this.Bj=[];this.Hj=!1}r(Zp,md);f=Zp.prototype;f.J=function(a){var b,c;if(null!=a&&(b=this.l.all(".btn-save",a))){this.Bj=[];var d=0;for(c=b.length;d<c;d++){var e=b[d];var g=new xm;g.J(e);this.Bj.push(g)}this.Yg(a);return $p(a)}};function $p(a){return v(a,"submit",function(a){return a.preventDefault()})}
f.Yg=function(a){var b,c=this;var d=this.Bj;var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(h.A("action",function(b){h=b.target;h.pa(!1);b=h.o().getAttribute("name");var d=h.o().getAttribute("value");null!=b&&(b=c.l.D("input",{type:"hidden",name:b,value:d}),c.l.appendChild(a,b));return c.send(a)}))}return e};f.send=function(a){this.form=a};
f.od=function(){var a;if(this.Hj){var b=this.l.all("input[type\x3dtext], textarea",this.form);var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];c.push(Fm(e,""))}return c}};f.Qi=function(){};function aq(a){var b;var c=a.Bj;var d=[];var e=0;for(b=c.length;e<b;e++)a=c[e],d.push(a.pa(!0))};function bq(a,b){this.od=q(this.od,this);this.Qi=q(this.Qi,this);bq.m.constructor.call(this,a,b)}r(bq,Zp);bq.prototype.J=function(a){bq.m.J.call(this,a);return this.url=a.action};
bq.prototype.send=function(a){bq.m.send.call(this,a);var b=this.client;var c=this.od,d=this.Qi;b:{var e;var g=a.querySelectorAll("input[type\x3dfile]");var h=0;for(e=g.length;h<e;h++){var k=g[h];if(Em(k)){k=!0;break b}}k=!1}if(k)b=Al(b,a,c,d);else{k=a.action;h=[];var l=a.elements,m;for(g=0;m=l[g];g++)if(m.form==a&&!m.disabled&&"FIELDSET"!=m.tagName)switch(e=m.name,m.type.toLowerCase()){case "file":case "submit":case "reset":case "button":break;case "select-multiple":m=Em(m);if(null!=m)for(var p,u=
0;p=m[u];u++)Dm(h,e,p);break;default:p=Em(m),null!=p&&Dm(h,e,p)}l=a.getElementsByTagName("INPUT");for(g=0;m=l[g];g++)m.form==a&&"image"==m.type.toLowerCase()&&(e=m.name,Dm(h,e,m.value),Dm(h,e+".x","0"),Dm(h,e+".y","0"));a=h.join("\x26");b=yl(b,k,a,c,d)}return b};bq.prototype.Qi=function(a){bq.m.Qi.call(this,a);if(null!=a)return aq(this),a=this.l.Sr(a).querySelector("form"),this.J(a),this.dispatchEvent(new Cc("error",a))};
bq.prototype.od=function(a){bq.m.od.call(this,a);aq(this);return this.dispatchEvent(new Cc("saved",a))};function cq(a,b,c,d,e){this.Xf=q(this.Xf,this);cq.m.constructor.call(this,a,b,c,d,e);this.tm=new bq(this.client,this.l);this.tm.A("saved",this.Xf);this.tm.A("error",this.yi)}r(cq,vp);cq.prototype.np=function(a){var b=this.dc.ub();b=Dg(this.l,"form",b);dh(b,"ajax")&&ah(b,"ajax");this.tm.J(b);return cq.m.np.call(this,a)};cq.prototype.Xf=function(a){if(a.target&&a.target.snippets)return this.yi(a.target);this.dc.$(!1);return this.dispatchEvent(new Cc("success-close",a.target))};function dq(a,b){this.l=a;this.Qb=b;this.mn=q(this.mn,this);this.od=q(this.od,this);this.Ye=q(this.Ye,this);this.Cc=new Zn({l:a});this.client=new ql(this.Qb);this.method=this.url="";this.El=this.Ne=this.Ps=this.jp=this.data=null}var eq={vu:"postAction",wu:"postElement",ju:"method",URL:"url",bu:"data",Eu:"state"},fq={reload:function(a){return ne(a.l).location.reload(!0)},remove:function(a){a.El=a.l.o(a.Ps);a=null!=a.El?ne(a.l).setTimeout(a.mn(),1E3):void 0;return a}};
dq.prototype.J=function(a){this.Ne=a;this.Cc.J(this.Ne);var b;this.url=String(Rh(this.Ne,eq.URL));this.method=String(Rh(this.Ne,eq.ju));this.data=null!=(b=Rh(this.Ne,eq.bu))?b:{};this.jp=String(Rh(this.Ne,eq.vu));this.Ps=String(Rh(this.Ne,eq.wu));this.state=Rh(this.Ne,eq.Eu);this.Cc.A("action",this.Ye);a=this.state&&ne(this.l).location.hash==="#"+this.state?this.Ye():void 0;return a};
dq.prototype.Ye=function(){this.state&&(ne(this.l).location.hash=this.state);return this.Cc.call(this.client,this.url,this.method,this.data,this.od)};dq.prototype.od=function(){if(Jb(fq,this.jp))return fq[this.jp](this)};dq.prototype.mn=function(){var a=this;var b=new Wg(this.El,1E3);b.A("end",function(){a.l.removeNode(a.El)});return b.play()};function gq(a){var b=0,c=0;if(hq(a))b=a.selectionStart,c=-1;else if(t&&!uc("9")){var d=iq(a),e=d[0],d=d[1];if(e.inRange(d)){e.setEndPoint("EndToStart",d);if("textarea"==a.type){d.duplicate();b=a=e.text;for(c=!1;!c;)0==e.compareEndPoints("StartToEnd",e)?c=!0:(e.moveEnd("character",-1),e.text==a?b+="\r\n":c=!0);e=[b.length,-1];return e}b=e.text.length;c=-1}}return[b,c]}
function iq(a){var b=a.ownerDocument||a.document,c=b.selection.createRange();"textarea"==a.type?(b=b.body.createTextRange(),b.moveToElementText(a)):b=a.createTextRange();return[b,c]}function jq(a,b){"textarea"==a.type&&(b=a.value.substring(0,b).replace(/(\r\n|\r|\n)/g,"\n").length);return b}function hq(a){try{return"number"==typeof a.selectionStart}catch(b){return!1}};function kq(a,b,c,d){Vb.call(this);d=d||150;this.mh=null!=c?c:!0;this.bj=a||",;";this.wv=this.bj.substring(0,1);a=this.mh?"[\\s"+this.bj+"]+":"[\\s]+";this.Ot=new RegExp("^"+a+"|"+a+"$","g");this.ax=new RegExp("\\s*["+this.bj+"]$");this.fs=b||"";this.Rw=!1;this.Qw=this.mh;this.wa=0<d?new nd(d):null;this.Eb=new fh(this);this.vj=new fh(this);this.Yb=new Yh;this.cs=-1}r(kq,Vb);var lq=(Nh||Oh)&&!uc("533.17.9");f=kq.prototype;f.tx=!0;f.Iv=!0;f.Qt=!1;f.cx=!0;f.bx=!0;f.rm=null;f.Ea=null;f.Jo="";f.wf=!1;
f.wp=!1;f.ox=!0;f.sn=function(){return this.Ea};f.ba=function(){return this.Ea.value};f.qa=function(a){this.Ea.value=a};function mq(a,b){Hf(b)&&(fi(b,"combobox"),gi(b,"autocomplete","list"));a.Eb.A(b,"focus",a.handleFocus);a.Eb.A(b,"blur",a.handleBlur);a.Ea||(a.vj.A(b,"keydown",a.zs),Hf(b)&&Uf(kf(b))==b&&nq(a,b))}
function oq(a,b){if(a.Ea){var c=b.toString();if(da(void 0)?0:a.mh){var d=pq(a,a.ba(),gq(a.Ea)[0]),e=qq(a,a.ba());a.ax.test(c)||(c=c.replace(/[\s\xa0]+$/,"")+a.wv);a.tx&&(0==d||Aa(e[d-1])||(c=" "+c),d==e.length-1&&(c+=" "));if(c!=e[d]){e[d]=c;c=a.Ea;(jc||t&&uc("9"))&&c.blur();c.value=e.join("");for(var g=0,h=0;h<=d;h++)g+=e[h].length;c.focus();d=g;e=a.Ea;c=d;hq(e)?e.selectionStart=c:t&&!uc("9")&&(g=iq(e),h=g[0],h.inRange(g[1])&&(c=jq(e,c),h.collapse(!0),h.move("character",c),h.select()));e=a.Ea;hq(e)?
e.selectionEnd=d:t&&!uc("9")&&(g=iq(e),c=g[1],g[0].inRange(c)&&(d=jq(e,d),e=jq(e,gq(e)[0]),c.collapse(!0),c.moveEnd("character",d-e),c.select()))}}else a.qa(c);a.wp=!0}return!1}f.V=function(){kq.m.V.call(this);null!=this.rm&&window.clearTimeout(this.rm);this.Eb.ib();delete this.Eb;this.vj.ib();this.Yb.ib();ac(this.wa)};
f.gd=function(a){switch(a.keyCode){case 40:if(this.Ia.cf())return this.Qt?rq(this.Ia):sq(this.Ia),a.preventDefault(),!0;if(!this.mh)return this.update(!0),a.preventDefault(),!0;break;case 38:if(this.Ia.cf())return this.Qt?sq(this.Ia):rq(this.Ia),a.preventDefault(),!0;break;case 9:if(!this.Ia.cf()||a.shiftKey||this.Rw)this.Ia.Sc();else if(this.update(),this.Ia.tg()&&this.Qw)return a.preventDefault(),!0;break;case 13:if(this.Ia.cf()){if(this.update(),this.Ia.tg())return a.preventDefault(),a.stopPropagation(),
!0}else this.Ia.Sc();break;case 27:if(this.Ia.cf())return this.Ia.Sc(),a.preventDefault(),a.stopPropagation(),!0;break;case 229:if(!this.wf)return this.wf||(this.Eb.A(this.Ea,"keyup",this.Bs),this.Eb.A(this.Ea,"keypress",this.As),this.wf=!0),!0;break;default:this.wa&&!this.ox&&(this.wa.stop(),this.wa.start())}return tq(this,a)};function tq(a,b){var c=a.mh&&b.charCode&&-1!=a.bj.indexOf(String.fromCharCode(b.charCode));a.cx&&c&&a.update();return a.bx&&c&&a.Ia.tg()?(b.preventDefault(),!0):!1}f.Tv=function(){return!1};
f.handleFocus=function(a){nq(this,a.target||null)};function nq(a,b){a.vj.sd();a.Ia&&uq(a.Ia);b!=a.Ea&&(a.Ea=b,a.wa&&(a.wa.start(),a.Eb.A(a.wa,"tick",a.Fs)),a.Jo=a.ba(),a.Yb.zf(a.Ea),a.Eb.A(a.Yb,"key",a.Ri),a.Eb.A(a.Ea,"mousedown",a.Cs),t&&a.Eb.A(a.Ea,"keypress",a.ys))}f.handleBlur=function(){lq?this.rm=window.setTimeout(q(this.Ss,this),0):this.Ss()};
f.Ss=function(){this.Ea&&(this.Eb.Ba(this.Yb,"key",this.Ri),this.Yb.detach(),this.Eb.Ba(this.Ea,"keyup",this.Tv),this.Eb.Ba(this.Ea,"mousedown",this.Cs),t&&this.Eb.Ba(this.Ea,"keypress",this.ys),this.wf&&vq(this),this.Ea=null,this.wa&&(this.wa.stop(),this.Eb.Ba(this.wa,"tick",this.Fs)),this.Ia&&wq(this.Ia))};f.Fs=function(){this.update()};f.zs=function(a){this.handleFocus(a)};f.Ri=function(a){this.cs=a.keyCode;this.Ia&&this.gd(a)};f.As=function(){this.wf&&229!=this.cs&&vq(this)};
f.Bs=function(a){this.wf&&(13==a.keyCode||77==a.keyCode&&a.ctrlKey)&&vq(this)};f.Cs=function(a){this.Ia&&this.xe(a)};f.xe=function(){};function vq(a){a.wf&&(a.wf=!1,a.Eb.Ba(a.Ea,"keypress",a.As),a.Eb.Ba(a.Ea,"keyup",a.Bs))}f.ys=function(a){tq(this,a)};
f.update=function(a){if(this.Ea&&(a||this.ba()!=this.Jo)){if(a||!this.wp){a=gq(this.Ea)[0];var b=this.ba();a=qq(this,b)[pq(this,b,a)];a=this.Ot?String(a).replace(this.Ot,""):a;this.Ia&&(this.Ia.Ya=this.Ea,this.Ia.ql(a,this.ba()))}this.Jo=this.ba()}this.wp=!1};function pq(a,b,c){a=qq(a,b);if(c==b.length)return a.length-1;for(var d=b=0,e=0;d<a.length&&e<=c;d++)e+=a[d].length,b=d;return b}
function qq(a,b){if(!a.mh)return[b];for(var c=String(b).split(""),d=[],e=[],g=0,h=!1;g<c.length;g++)a.fs&&-1!=a.fs.indexOf(c[g])?(a.Iv&&!h&&(d.push(e.join("")),e.length=0),e.push(c[g]),h=!h):h||-1==a.bj.indexOf(c[g])?e.push(c[g]):(e.push(c[g]),d.push(e.join("")),e.length=0);d.push(e.join(""));return d};function xq(a,b,c,d){null==a&&(a=null);null==b&&(b=null);null==c&&(c=null);null==d&&(d=null);xq.m.constructor.call(this,a,b,c,d)}r(xq,kq);xq.prototype.handleFocus=function(a){return xq.m.handleFocus.call(this,a)};xq.prototype.ba=function(){var a;return null!=(a=xq.m.ba.call(this))?a:""};function yq(){yq.m.constructor.call(this)}r(yq,oi);yq.prototype.ta=function(){return"tag"};yq.prototype.D=function(a){var b=yq.m.D.call(this,a);a.Yr()||(a=a.I().D("span",{"class":"goog-icon-remove"}),b.appendChild(a));return b};ka(yq);function zq(a,b){this.l=a;this.Mw=b;zq.m.constructor.call(this)}r(zq,yq);zq.prototype.D=function(a){var b=zq.m.D.call(this,a);a=Aq(this.Mw,a.Sa);Vf(b,a);return b};function Bq(a){this.l=a}var Cq={hu:"ac-image ac-image-list",Ey:"ac-image-placeholder"};Bq.prototype.al=function(a,b,c){a instanceof fl||(a=a.data);c.appendChild(Aq(this,a));return c.appendChild(this.l.createTextNode(a.toString()))};function Dq(a){return a.l.G("div","ac-image-placeholder")}function Aq(a,b){null==b&&(b=null);return null!=b&&null!=b.photo?a.l.D("img",{src:b.photo,"class":Cq.hu,alt:b.toString()}):Dq(a)};function Eq(a,b){this.fa=a||[];this.Dh=!b}Eq.prototype.sg=function(a,b,c){if(this.Dh){var d=this.fa,e=Fq(a,b,d);0==e.length&&(e=Gq(a,b,d));b=e}else b=Fq(a,b,this.fa);c(a,b)};function Fq(a,b,c){var d=[];if(""!=a){a=Ta(a);a=new RegExp("(^|\\W+)"+a,"i");for(var e=0;e<c.length&&d.length<b;e++){var g=c[e];String(g).match(a)&&d.push(g)}}return d}
function Gq(a,b,c){for(var d=[],e=0;e<c.length;e++){var g=c[e],h=a.toLowerCase(),k=String(g).toLowerCase(),l=0;if(-1!=k.indexOf(h))l=parseInt((k.indexOf(h)/4).toString(),10);else for(var m=h.split(""),p=-1,u=10,A=0,D;D=m[A];A++)D=k.indexOf(D),D>p?(p=D-p-1,p>u-5&&(p=u-5),l+=p,p=D):(l+=u,u+=5);l<6*h.length&&d.push({Md:g,it:l,index:e})}d.sort(function(a,b){var c=a.it-b.it;return 0!=c?c:a.index-b.index});a=[];for(A=0;A<b&&A<d.length;A++)a.push(d[A].Md);return a};function Hq(a,b,c){null==b&&(b=!1);this.gx=null!=c?c:!0;Hq.m.constructor.call(this,a,b)}r(Hq,Eq);Hq.prototype.sg=function(a,b,c,d){null==d&&(d=void 0);return this.gx&&""===a?c(a,this.fa.length<=b?this.fa:this.fa.slice(0,+(b-1)+1||9E9)):Hq.m.sg.call(this,a,b,c,d)};function Iq(a,b,c){md.call(this);this.ff=a;this.uh=c;this.W=b;v(b,[Jq,Kq,Lq,Mq],this.handleEvent,!1,this);this.Za=null;this.fa=[];this.kd=-1;this.Fb=0;this.Hf=this.Ya=null;this.uo={}}r(Iq,md);f=Iq.prototype;f.pw=10;f.Tq=!0;f.Kq=!1;f.Xt=!1;f.Gl=!1;var Jq="hilite",Kq="select",Mq="dismiss",Lq="canceldismiss";f=Iq.prototype;f.Sf=function(){return this.Za};
f.handleEvent=function(a){var b=this.ff;if(a.target==this.W)switch(a.type){case Jq:this.Hd(a.xb);break;case Kq:var c=!1;if(fa(a.xb)){a=a.xb;var d=this.fa[Nq(this,a)],c=!!d&&b.Co&&b.Co(d);d&&!c&&this.kd!=a&&this.Hd(a)}c||this.tg();break;case Lq:uq(this);break;case Mq:wq(this)}};f.ql=function(a,b){this.Za!=a&&(this.Za=a,this.ff.sg(this.Za,this.pw,q(this.Ro,this),b),uq(this))};f.qk=function(){return this.Ya};f.cf=function(){return this.W.va()};
function sq(a){for(var b=a.Fb+a.fa.length-1,c=a.kd,d=0;d<a.fa.length;d++){if(c>=a.Fb&&c<b)c++;else if(-1==c)c=a.Fb;else if(a.Kq&&c==b){a.Hd(-1);break}else if(a.Xt&&c==b)c=a.Fb;else break;if(a.Hd(c))break}}function rq(a){for(var b=a.Fb+a.fa.length-1,c=a.kd,d=0;d<a.fa.length;d++){if(c>a.Fb)c--;else if(a.Kq&&c==a.Fb){a.Hd(-1);break}else if(!a.Xt||-1!=c&&c!=a.Fb)break;else c=b;if(a.Hd(c))break}}
f.Hd=function(a){var b=Nq(this,a),c=this.fa[b];return c&&this.ff.Co&&this.ff.Co(c)?!1:(this.kd=a,this.W.Hd(a),-1!=b)};f.tg=function(){var a=Nq(this,this.kd);if(-1!=a){var b=this.fa[a],c=oq(this.uh,b);this.Gl?(this.Za=null,wq(this)):this.Sc();c||(this.dispatchEvent({type:"update",xb:b,index:a}),this.Gl&&this.uh.update(!0));return!0}this.Sc();this.dispatchEvent({type:"update",xb:null,index:null});return!1};
f.Sc=function(){this.kd=-1;this.Za=null;this.Fb+=this.fa.length;this.fa=[];window.clearTimeout(this.Hf);this.Hf=null;this.W.Sc();this.dispatchEvent("suggestionsupdate");this.dispatchEvent(Mq)};function wq(a){a.Hf||(a.Hf=window.setTimeout(q(a.Sc,a),100))}f.Vr=function(){return this.Hf?(window.clearTimeout(this.Hf),this.Hf=null,!0):!1};function uq(a){a.Vr()||window.setTimeout(q(a.Vr,a),10)}f.V=function(){Iq.m.V.call(this);delete this.uo;this.W.ib();this.uh.ib();this.ff=null};
f.Ro=function(a,b,c){this.Za==a&&this.De(b,c)};f.De=function(a,b){var c="object"==n(b)&&b,d=(c?c.Kv():b)?Nq(this,this.kd):-1;this.Fb+=this.fa.length;this.fa=a;for(var e=[],g=0;g<a.length;++g)e.push({id:this.Fb+g,data:a[g]});g=null;this.Ya&&(g=this.uo[pa(this.Ya)]||this.Ya);this.W.Oq=g;this.W.De(e,this.Za,this.Ya);g=this.Tq;c&&void 0!==c.tn()&&(g=c.tn());this.kd=-1;(g||0<=d)&&0!=e.length&&this.Za&&(0<=d?this.Hd(this.Fb+d):sq(this));this.dispatchEvent("suggestionsupdate")};
function Nq(a,b){var c=b-a.Fb;return 0>c||c>=a.fa.length?-1:c}f.update=function(a){this.uh.update(a)};function Oq(a,b,c){this.tg=q(this.tg,this);this.Yo=q(this.Yo,this);this.De=q(this.De,this);Oq.m.constructor.call(this,a,b,c)}r(Oq,Iq);f=Oq.prototype;f.ql=function(a,b){null==b&&(b=void 0);this.Za=a;this.ff.sg(this.Za,this.ff.fa.length,q(this.Ro,this),b);return uq(this)};
f.De=function(a,b){var c;var d=((c="object"===n(b)&&b)?c.Kv():b)?Nq(this,this.kd):-1;this.Fb+=this.fa.length;this.fa=this.Yo(a);var e=[];var g=this.fa;for(l in g){var h=g[l];var k={id:this.Fb+Nm(l),data:h};null!=h.group&&(k.group=h.group);e.push(k);k=null}this.Ya&&(k=this.uo[pa(this.Ya)]||this.Ya);this.W.Oq=k;this.W.De(e,this.Za,this.Ya);var l=this.Tq;c&&null!=c.tn()&&(l=c.tn());this.kd=-1;if(l||0<=d&&0!==e.length&&this.Za)0<=d?this.Hd(this.Fb+d):sq(this);this.dispatchEvent("suggestionsupdate")};
f.Yo=function(a){var b;var c=[];var d=0;for(b=a.length;d<b;d++){var e=a[d];this.dh(e,this.W.Ug())||c.push(e)}return c};f.dh=function(a,b){return null!=b[a.toString()]};f.tg=function(){var a=Nq(this,this.kd);if(-1!==a){var b=this.fa[a];var c=oq(this.uh,b);this.Gl?(this.Za=null,wq(this)):this.Sc();c||(this.dispatchEvent({type:"update",xb:b,index:a}),this.Gl&&this.uh.update(!0));return!0}this.Sc();this.dispatchEvent({type:"update",xb:null,index:null});return!1};function Pq(a,b){this.l=a;this.select=b}Pq.prototype.load=function(a){var b;var c=Em(this.select);if(null==c)return null;var d=String(c);var e=0;for(b=a.length;e<b;e++)if(c=a[e],String(c.id)===d)return c;return null};Pq.prototype.uf=function(a){null==a&&(a=null);return Fm(this.select,null!=a?String(a.id):"")};Pq.prototype.Hj=function(){var a=Em(this.select);if(null!=a)return this.l.unselect(this.select,a)};function Qq(a,b){this.l=a;this.K=b;this.jd=q(this.jd,this);this.Jb=q(this.Jb,this);this.Vc=q(this.Vc,this);this.Jn=q(this.Jn,this);Qq.m.constructor.call(this);this.input=this.ij=this.cb=this.select=null}r(Qq,md);function Rq(a,b){var c=a.ij.load(b);if(null!=c)return Sq(a,c)}
function Sq(a,b){var c=a.cb,d=a.input.o();Hf(d)&&(d.removeAttribute("role"),d.removeAttribute("aria-autocomplete"));d==c.Ea&&c.handleBlur();c.Eb.Ba(d,"focus",c.handleFocus);c.Eb.Ba(d,"blur",c.handleBlur);c.Ea||c.vj.Ba(d,"keydown",c.zs);a.input.qa(b.toString());mq(a.cb,a.input.o());a.input.ba()&&Qi(a.input);a.l.select(a.select,b.pk);return a.wj(b)}f=Qq.prototype;
f.J=function(a){this.select=a;if(a.hasAttribute("multiple"))throw Error("For select-multiple use wzk.ui.ac.ExtSelectbox");this.K.J(this.select);this.ij=new Pq(this.l,this.select);this.K.A(Tq,this.Vc);this.K.A(Uq,this.Jb);this.input=this.K.input;this.select.hasAttribute("placeholder")&&Ni(this.input,this.select.getAttribute("placeholder"));a=Rh(this.select,"chooseLabel");this.av=Rh(this.select,"chooseValue");if(a)return a=this.l.G("span","ac-choose-value",a),this.l.Ki(a,this.input.o()),v(a,"click",
this.Jn)};f.Jn=function(){return Vq(this,this.av)};function Vq(a,b){a.l.select(a.select,b);return Rq(a,a.data)}f.Vc=function(){this.ij.Hj();return gd(this.select,"change",!1,{type:"change",target:this.select})};f.Jb=function(){this.K.input.o().focus();return this.Rc.De(this.data)};
f.load=function(a){this.data=a;this.Id=new Hq(this.data,!1);this.cb=new xq(null,null,!1);this.Rc=new Oq(this.Id,this.K,this.cb);a=this.K.input.o();this.Rc.Ya=a;this.cb.Ia=this.Rc;mq(this.cb,this.K.input.o());this.Rc.A("update",this.jd);Rq(this,this.data);return this.dispatchEvent(new Cc("loaded"))};f.jd=function(a){this.ij.uf(a.xb);this.wj(a.xb);this.K.input.Ze();return gd(this.select,"change",!1,{type:"change",target:this.select})};f.wj=function(a){return Wq(this.K,a)};f.destroy=function(){return this.input.destroy()};function Xq(a,b,c,d){md.call(this);this.Zb=a||document.body;this.S=hf(this.Zb);this.Ww=!a;this.F=null;this.Za="";this.fa=[];this.Fe=[];this.Et=this.zk=-1;this.Oa=!1;this.className="ac-renderer";this.Zi="ac-row";this.es="active";this.qm="ac-active";this.dw="ac-highlighted";this.le=b||null;this.St=null!=d?d:!0;this.ow=!0;this.yk=!1;this.Xw=!!c;this.Bh=!1;this.Pk=0;this.Jp=!1}r(Xq,md);f=Xq.prototype;f.Xu=0;f.o=function(){return this.F};
f.De=function(a,b,c){this.Za=b;this.fa=a;this.zk=-1;this.Et=wa();this.Ya=c;this.Fe=[];this.redraw()};f.Sc=function(){this.Oa&&(this.Oa=!1,Yq(this,!1),0<this.Pk?(ac(this.Hg),this.Hg=new Wg(this.F,this.Pk),this.Hg.play()):w(this.F,!1))};f.show=function(){this.Oa||(this.Oa=!0,Yq(this,!0),0<this.Pk?(ac(this.Hg),this.Hg=new Xg(this.F,this.Pk),this.Hg.play()):w(this.F,!0))};
function Yq(a,b){a.Ya&&(gi(a.Ya,"haspopup",b),gi(a.F,"expanded",b),gi(a.Ya,"expanded",b),b?gi(a.Ya,"owns",a.F.id):(a.Ya.removeAttribute("aria-owns"),hi(a.Ya,null)))}f.va=function(){return this.Oa};function Zq(a,b){var c=0<=b&&b<a.fa.length?a.fa[b]:void 0,d=0<=b&&b<a.Fe.length?a.Fe[b]:void 0;a.dispatchEvent({type:"rowhilite",TA:d,xb:c?c.data:null})&&(0<=a.zk&&ni(a.Fe[a.zk],[a.qm,a.es]),a.zk=b,d&&(li(d,[a.qm,a.es]),a.Ya&&hi(a.Ya,d),ng(d,a.F)))}
f.Hd=function(a){if(-1==a)Zq(this,-1);else for(var b=0;b<this.fa.length;b++)if(this.fa[b].id==a){Zq(this,b);break}};f.So=function(){if(!this.F){var a=this.S.D("DIV",{style:"display:none"});this.Jp&&(a.style.overflowY="auto");this.F=a;li(a,Da(this.className).split(" "));fi(a,"listbox");a.id=jh(ih.ua());this.S.appendChild(this.Zb,a);v(a,"click",this.vi,!1,this);v(a,"mousedown",this.xi,!1,this);v(a,"mouseover",this.Yn,!1,this)}};
f.redraw=function(){this.So();this.Bh&&(this.F.style.visibility="hidden");this.iq&&(this.F.style.minWidth=this.iq.clientWidth-this.Xu+"px");this.Fe.length=0;this.S.sh(this.F);if(this.le&&this.le.render)this.le.render(this,this.F,this.fa,this.Za);else{var a=null;db(this.fa,function(b){b=this.bl(b,this.Za);this.Bh?this.F.insertBefore(b,a):this.S.appendChild(this.F,b);a=b},this)}0==this.fa.length?this.Sc():(this.show(),this.yc(),xg(this.F,!0))};
f.yc=function(){if(this.Ya&&this.Ww){var a=this.Oq||this.Ya;var b=this.Xw?5:1;this.Bh&&(b^=1);var c=65;this.Jp&&(this.F.style.height="",c|=32);hk(a,b,this.F,b^1,null,null,c);this.Bh&&(this.F.style.visibility="visible")}};f.qk=function(){return this.Ya||null};f.V=function(){this.F&&(dd(this.F,"click",this.vi,!1,this),dd(this.F,"mousedown",this.xi,!1,this),dd(this.F,"mouseover",this.Yn,!1,this),this.S.removeNode(this.F),this.F=null,this.Oa=!1);ac(this.Hg);this.Zb=null;Xq.m.V.call(this)};
function $q(a,b,c){if(a.yk||!a.Vt)if(3==b.nodeType){var d=null;la(c)&&1<c.length&&!a.yk&&(d=sb(c,1));c=ar(a,c);if(0!=c.length){var e=b.nodeValue,g=a.ow?new RegExp("\\b(?:"+c+")","gi"):new RegExp(c,"gi");c=[];for(var h=0,k=g.exec(e),l=0;k;)l++,c.push(e.substring(h,k.index)),c.push(e.substring(k.index,g.lastIndex)),h=g.lastIndex,k=g.exec(e);c.push(e.substring(h));if(1<c.length){d=a.yk?l:1;for(e=0;e<d;e++)g=2*e,b.nodeValue=c[g],h=a.S.createElement("B"),h.className=a.dw,a.S.appendChild(h,a.S.createTextNode(c[g+
1])),h=b.parentNode.insertBefore(h,b.nextSibling),b.parentNode.insertBefore(a.S.createTextNode(""),h.nextSibling),b=h.nextSibling;b.nodeValue=sb(c,2*d).join("");a.Vt=!0}else d&&$q(a,b,d)}}else for(b=b.firstChild;b;)d=b.nextSibling,$q(a,b,c),b=d}
function ar(a,b){var c="";if(!b)return c;la(b)&&(b=eb(b,function(a){return!Aa(null==a?"":String(a))}));a.yk?la(b)?c=fb(b,Ta).join("|"):(c=b.replace(/[\s\xa0]+/g," ").replace(/^\s+|\s+$/g,""),c=Ta(c),c=c.replace(/ /g,"|")):la(b)?c=0<b.length?Ta(b[0]):"":/^\W/.test(b)||(c=Ta(b));return c}
f.bl=function(a,b){var c=this.S.D("DIV",{className:this.Zi,id:jh(ih.ua())});fi(c,"option");this.le&&this.le.al?this.le.al(a,0,c):Kf(c,a.data.toString());b&&this.St&&(this.Vt=!1,$q(this,c,b));ki(c,this.Zi);this.Fe.push(c);return c};function br(a,b){for(;b&&b!=a.F&&!ji(b,a.Zi);)b=b.parentNode;return b?cb(a.Fe,b):-1}f.vi=function(a){var b=br(this,a.target);0<=b&&this.dispatchEvent({type:Kq,xb:this.fa[b].id});a.stopPropagation()};f.xi=function(a){a.stopPropagation();a.preventDefault()};
f.Yn=function(a){a=br(this,a.target);0<=a&&!(300>wa()-this.Et)&&this.dispatchEvent({type:Jq,xb:this.fa[a].id})};function cr(a){null==a&&(a={});null==a.Cf&&(a.Cf=dr);a.K=new yh(a.Cf,"");cr.m.constructor.call(this,a)}r(cr,zh);var dr="open-list";function er(a,b,c,d,e){this.l=a;this.Jc=c;this.Am=q(this.Am,this);this.yj=q(this.yj,this);this.tp=q(this.tp,this);this.redraw=q(this.redraw,this);this.Ug=q(this.Ug,this);this.Vf=q(this.Vf,this);this.Jb=q(this.Jb,this);this.Fp=q(this.Fp,this);this.Gp=q(this.Gp,this);er.m.constructor.call(this,null,c,d,e);this.className=fr;this.Zi=gr;this.qm=hr;this.input=this.bg=null}r(er,Xq);var fr="ac dropdown-menu",gr="ac-row",hr="active",Tq="clean",Uq="open";f=er.prototype;
f.J=function(a){this.select=a;w(this.select,!1);this.rh=this.select.hasAttribute("readonly");this.je=this.l.G("span","ac-item ac-buttons");null!=this.Jc&&(this.bg=Aq(this.Jc),this.l.appendChild(this.je,this.bg));a=Yf(this.select);this.l.appendChild(a,this.je);this.input=new Pi(this.l);this.input.A("VALUE_CHANGE",this.Vf);this.input.render(this.je);this.select.hasAttribute(!this.rh)&&this.input.o().setAttribute("placeholder",this.select.getAttribute("placeholder"));this.items=this.l.D("div");null!=
this.Jc&&Zg(this.input.o(),"with-image");if(this.rh)return this.input.o().setAttribute("readonly","true");this.Ae=new cr({l:this.l});this.Ae.A("action",this.Jb);return this.Ae.render(this.je)};f.Gp=function(a){this.Jt=a};f.Fp=function(){};f.Jb=function(){return this.dispatchEvent(new Cc(Uq))};f.Vf=function(a){if(!this.rh){if(a.target.Fd())return w(this.Ae.o(),!1);w(this.Ae.o(),!0);null!=this.Jc&&(a=Dq(this.Jc),Wf(a,this.bg),this.bg=a);return this.dispatchEvent(new Cc(Tq))}};
function Wq(a,b){null==b&&(b=null);if(null!=a.bg){var c=Aq(a.Jc,b);Wf(c,a.bg);return a.bg=c}}f.So=function(){if(!this.F){var a=this.l.D("ul",{style:"display:none"});this.Jp&&ag(a,"overflow-y","auto");this.F=a;li(a,Da(this.className).split(" "));fi(a,"listbox");a.id=jh(ih.ua());this.l.appendChild(this.je,a);v(a,"click",this.vi,!1,this);v(a,"mousedown",this.xi,!1,this);v(a,"mouseover",this.Yn,!1,this)}};f.Ug=function(){return null==this.Jt?{}:this.Jt.getTags()};
f.redraw=function(){this.So();this.Bh&&(this.F.style.visibility="hidden");if(this.iq){var a=this.iq.clientWidth+"px";this.F.style.minWidth=a}this.Fe.length=0;this.S.sh(this.F);if(this.le&&this.le.render)this.le.render(this,this.F,this.fa,this.Za);else{a=this.fa;var b,c;var d={};var e=[];var g=b=0;for(c=a.length;g<c;g++){var h=a[g];null!=h.group?null!=d[h.group]?e[d[h.group]].push(h):(d[h.group]=b,e[d[h.group]]=[h],b++):(e.push(h),b++)}this.tp(e,null)}if(0===this.fa.length)this.Sc();else return this.show(),
this.yc(),xg(this.F,!0)};f.tp=function(a,b){var c,d;var e=[];var g=0;for(c=a.length;g<c;g++){var h=a[g];if(h instanceof Array){var k=this.Am(h[0]?h[0].group:"");var l=this.l.G("ul");k.appendChild(l);var m=0;for(d=h.length;m<d;m++){var p=h[m];l.appendChild(this.bl(p,this.Za))}this.yj(k,b);e.push(b=k)}else h=this.bl(h,this.Za),this.yj(h,b),e.push(b=h)}return e};f.yj=function(a,b){return this.Bh?this.F.insertBefore(a,b):this.S.appendChild(this.F,a)};f.Am=function(a){return this.l.G("li","",a)};
f.bl=function(a,b){var c=jh(ih.ua());c=this.l.D("li",{className:gr,id:c});fi(c,"option");Qh(c,"value",a.data.id);this.Jc&&this.Jc.al?this.Jc.al(a,0,c):Kf(c,a.data.toString());b&&this.St&&$q(this,c,b);Zg(c,this.Zi);this.Fe.push(c);return c};function ir(){ir.m.constructor.call(this)}r(ir,oi);ir.prototype.ta=function(){return"tag-container"};ka(ir);function jr(a,b,c,d){this.rh=null!=d?d:!1;this.wc=q(this.wc,this);null==b&&(b=yq.ua());jr.m.constructor.call(this,a,b,c);this.icon=this.listener=null}r(jr,Ai);f=jr.prototype;f.D=function(){jr.m.D.call(this);this.icon=Eg(this.S,"goog-icon-remove",this.o());return this.Yg(this.o())};f.pa=function(a){jr.m.pa.call(this,a);this.icon&&w(this.icon,a);a?this.Yg(this.o()):fd(this.o())};f.Yg=function(a){if(null!=a)return v(a,[Fc.oq,Fc.tq],this.wc)};
f.wc=function(a){if(a.target===this.icon&&(a.type===Fc.tq&&13===a.keyCode||a.type===Fc.oq))return this.dispatchEvent("remove")};f.Yr=function(){return this.rh};f.st=function(a){return this.rh=a};function kr(a,b,c){null==b&&(b=ir.ua());kr.m.constructor.call(this,a,b,c);this.zh={}}r(kr,Ai);function lr(a,b,c,d,e){null==e&&(e=!1);b=new jr(b,d,a.S);b.st(e);b.Sa=c;return a.add(b)}f=kr.prototype;f.add=function(a){if(null==this.zh[this.getKey(a.Sa)])return this.Yg(a),this.$a(a,!0),this.zh[this.getKey(a.Sa)]=a,this.dispatchEvent("add"),this.dispatchEvent(new Cc("add-tag",a))};f.getTags=function(){return this.zh};f.isEmpty=function(){return 0===Fb(this.zh)};f.getKey=function(a){return a.toString()};
f.Yg=function(a){var b=this;return cd(a,"remove",function(c){c.stopPropagation();Mb(b.zh,b.getKey(c.target.Sa));b.removeChild(c.target,!0);b.dispatchEvent("remove");return b.dispatchEvent(new Cc("remove-tag",a))})};f.pa=function(a){var b;kr.m.pa.call(this,a);var c=this.zh;for(b in c){var d=c[b];d.pa(a)}};f.sp=function(a){return this.Vi(Yf(a),Xf(a))};function mr(a,b,c,d){var e=this;this.l=a;this.K=b;this.Jc=c;this.cb=null!=d?d:null;this.Jb=q(this.Jb,this);this.Vf=q(this.Vf,this);mr.m.constructor.call(this);this.Ic=new kr(null,null,this.l);this.input=new Pi(a);this.input.A("VALUE_CHANGE",this.Vf);this.Ae=new cr({l:this.l});this.Ae.A("action",this.Jb);this.Ic.A("add-tag",function(a){return e.cb.add(a.target)});this.Ic.A("remove-tag",function(a){return e.cb.remove(a.target)});this.so=new xq(null,null,!1);this.Id=this.autoComplete=null;this.enabled=
!0}r(mr,md);f=mr.prototype;f.J=function(a,b){if(!a.hasAttribute("multiple"))throw Error("For select-one use wzk.ui.ac.SelectAutoComplete");this.render(a,b);if(a.disabled)return this.pa(!1)};
f.render=function(a,b){this.data=b;this.Id=new Hq(this.data,!1);this.autoComplete=new Oq(this.Id,this.K,this.so);w(a,!1);this.Ic.sp(a);this.K.Gp(this.Ic);this.K.Fp(this.data);var c=a.hasAttribute("readonly");if(!c){var d=this.l.D("div","ac-buttons");this.input.render(d);this.Ae.render(d);this.K.je=d;this.l.vo(d,a);a.hasAttribute("placeholder")&&Ni(this.input,a.getAttribute("placeholder"));this.so.Ia=this.autoComplete;mq(this.so,this.input.o());nr(this,this.autoComplete);or(this,a)}if(this.cb)return this.cb.load(b,
this.Ic,this.Jc,c)};f.Vf=function(a){return a.target.Fd()?w(this.Ae.o(),!1):w(this.Ae.o(),!0)};f.clear=function(){return this.input.qa("")};f.pa=function(a){this.enabled=a;this.input.pa(this.enabled);this.input.$(!1);return this.Ic.pa(!1)};f.refresh=function(a){this.Id.fa=a||[]};function nr(a,b){v(b,"update",function(b){if(null!=a.Jc){var c=a.Jc;c=new zq(c.l,c)}else c=null;lr(a.Ic,b.xb.toString(),b.xb,c);return a.input.clear()})}f.Jb=function(){this.input.o().focus();return this.autoComplete.De(this.data)};
function or(a,b){var c=b.required;a.input.Ok(c);b.required=void 0;v(a.Ic,"add-tag",function(){return a.input.Ok(!1)});v(a.Ic,"remove-tag",function(){if(a.Ic.isEmpty()&&c)return a.input.Ok(!0)})};function pr(a,b){this.l=a;this.select=b;this.table={}}f=pr.prototype;f.load=function(a,b,c,d){var e;null==d&&(d=!1);var g=[];var h=0;for(e=a.length;h<e;h++){var k=a[h];g.push(this.table[String(this.getKey(k))]=k)}a=Em(this.select);if(null!=a){e=[];k=0;for(h=a.length;k<h;k++){g=a[k];g=this.table[g];var l=null!=c?new zq(c.l,c):null;e.push(lr(b,g.toString(),g,l,d))}return e}};
f.uf=function(a){var b={};a=a.getTags();for(e in a){var c=a[e];c=c.Sa;b[this.getKey(c)]=c}var d=this.l.se(this.select);a=0;for(c=d.length;a<c;a++){var e=d[a];e.selected=Jb(b,e.value)}return gd(this.select,"change",!1,{type:"change",target:this.select})};f.add=function(a){return this.l.select(this.select,this.getKey(a))};f.remove=function(a){return this.l.unselect(this.select,this.getKey(a))};f.getKey=function(a){return a.id};function qr(a,b){this.select=a;this.tf=b;this.ho=q(this.ho,this);this.Ic=null}f=qr.prototype;f.load=function(a,b,c,d){this.Ic=b;this.tf.load(a,b,c,d);if(null!=this.select.form)return v(this.select.form,"submit",this.ho)};f.uf=function(a){return this.tf.uf(a)};f.add=function(a){return this.tf.add(a.Sa)};f.remove=function(a){return this.tf.remove(a.Sa)};f.ho=function(){return this.uf(this.Ic)};function rr(){this.Po=q(this.Po,this)}rr.prototype.load=function(a,b,c){var d;this.l=b;b=[];var e=a.children;a=0;for(d=e.length;a<d;a++){var g=e[a];"OPTGROUP"===g.nodeName?(g=this.Po(g),null!=g&&(b=b.concat(g))):(g=this.Th(g),null!=g&&b.push(g))}return c(b)};rr.prototype.Po=function(a){var b;var c=a.getAttribute("label");var d=[];var e=this.l.all("option",a);a=0;for(b=e.length;a<b;a++){var g=e[a];g=this.Th(g,c);null!=g&&d.push(g)}return 0<d.length?d:null};
rr.prototype.Th=function(a,b){null==b&&(b=void 0);var c=a.getAttribute("value");if(null==c||""===c||"__all__"===c)return null;c={DA:this.l.Vg(a),id:c};null!=b&&(c.group=b);var d=Rh(a,"image");null!=d&&0<d.length&&(c.photo=d);return new fl(c)};function sr(a,b,c){Vb.call(this);this.Jl=a;this.Dh=!b;this.U=new Vd(c)}r(sr,Vb);f=sr.prototype;f.ks="GET";f.ab=void 0;f.aw=null;f.hh=null;f.oc=function(a){this.ab=a};f.Cm=function(a,b,c,d){a=new yk(a);Mk(a,"token",b);Mk(a,"max_matches",String(c));Mk(a,"use_similar",String(Number(d)));return a.toString()};f.Hp=function(){return!0};f.vx=function(a,b,c){c=c.target.ti();var d=[];if(c)try{d=JSON.parse(c)}catch(e){}b(a,d)};
f.sg=function(a,b,c,d){this.Hp(this.Jl,a,b,this.Dh,d)&&(b=this.Cm(this.Jl,a,b,this.Dh,d))&&(a=q(this.vx,this,a,c),this.U.fg()&&this.U.abort(),this.hh&&ed(this.hh),this.hh=cd(this.U,"success",a),this.U.send(b,this.ks,this.ab,this.aw))};f.V=function(){this.U.ib();sr.m.V.call(this)};function tr(a,b,c,d,e){this.Wp=null!=b?b:"token";this.Mo=null!=c?c:100;this.Vp=null!=d?d:3;this.Ma=null!=e?e:[];tr.m.constructor.call(this,a,!1);this.headers={"X-Fields":this.Ma.join(","),"X-Base":this.Mo}}r(tr,sr);tr.prototype.Hp=function(a,b){return null!=b&&b.length>=this.Vp};tr.prototype.Vv=function(a,b,c){var d;var e=c.target.Mc();var g=[];var h=0;for(d=e.length;h<d;h++)c=e[h],g.push(new fl(c));return b(a,g)};tr.prototype.Cm=function(a,b){var c=new yk(a);Mk(c,this.Wp,b);return c.toString()};
tr.prototype.sg=function(a,b,c,d){if(this.Hp(this.Jl,a,b,this.Dh,d))return b=this.Cm(this.Jl,a,b,this.Dh,d),this.hh&&(ed(this.hh),this.U.abort()),a=q(this.Vv,this,a,c),this.hh=cd(this.U,"success",a),this.U.send(b,this.ks,this.ab,this.headers)};function ur(a,b,c){this.Id=a;this.K=b;this.cb=c;ur.m.constructor.call(this,this.Id,this.K,this.cb)}r(ur,Oq);ur.prototype.ql=function(a,b){null==b&&(b=void 0);if(this.Za!==a)return this.Za=a,this.ff.sg(this.Za,0,q(this.Ro,this),b),uq(this)};function vr(a,b,c,d,e,g){this.l=a;this.K=b;this.Su=c;this.Wp=d;this.Mo=e;this.Vp=g;this.jd=q(this.jd,this);this.Vc=q(this.Vc,this);vr.m.constructor.call(this);this.input=this.ij=this.cb=this.input=null}r(vr,md);f=vr.prototype;
f.J=function(a){this.G=a;this.K.J(this.G);this.K.A(Tq,this.Vc);this.input=this.K.input;this.G.hasAttribute("placeholder")&&Ni(this.input,this.G.getAttribute("placeholder"));this.Id=new tr(this.Su,this.Wp,this.Mo,this.Vp,["id","_autocomplete_value","_obj_name"].concat(wr(a)));this.cb=new xq(null,null,!1);this.Rc=new ur(this.Id,this.K,this.cb);a=this.K.input.o();this.Rc.Ya=a;this.cb.Ia=this.Rc;mq(this.cb,this.K.input.o());return this.Rc.A("update",this.jd)};
f.Vc=function(){Fm(this.G,"");this.input.qa("");return gd(this.input,"change",!1,{type:"change",target:this.input})};f.jd=function(a){Fm(this.G,a.xb.id);this.wj(a.xb);this.K.input.Ze();return gd(this.input,"change",!1,{type:"change",target:this.input})};f.wj=function(a){return Wq(this.K,a)};f.destroy=function(){return this.input.destroy()};function xr(a,b){var c=yr(a,b);(new rr).load(a,b,function(a){return c.load(a)});return c}function zr(a,b){var c=Ar(a,b);return(new rr).load(a,b,function(b){return c.J(a,b)})}function yr(a,b){var c=new er(b,0,Br(a,b));c=new Qq(b,c);c.J(a);return c}function Ar(a,b){if(null!=a){var c=Br(a,b);var d=new er(b,0,c);return new mr(b,d,c,new qr(a,new pr(b,a)))}}function Br(a,b){var c=null;null!=a.querySelector("option[data-image]")&&(c=new Bq(b));return c}
function wr(a){var b;a=Rh(a,"fields");if(null!=a){var c=qd(a);var d=[];for(b in c)a=c[b],d.push(a.split("__")[0]);return d}return[]};function Cr(a,b,c){this.l=a;this.client=b;this.register=c;this.Jb=q(this.Jb,this);this.Xf=q(this.Xf,this)}Cr.prototype.data=function(a){return String(Rh(this.G,a))};
Cr.prototype.J=function(a){this.G=a;this.Uo=new cq(this.l,this.client,this.data("url"),this.data("form"),this.register);this.G.title&&this.Uo.dj(this.G.title);(this.Ti=this.l.o(this.data("related")))&&dh(this.Ti,"related-fulltext-search")&&(this.Ti=this.Ti,this.Rc=xr(this.Ti,this.l),this.Uo.A("success-close",this.Xf));return v(this.G,"click",this.Jb)};
Cr.prototype.Xf=function(a){a=new fl(a.target.obj);var b=this.Rc;var c=b.l.G("option",{value:a.pk});b.l.Yc(c,a.toString());b.select.appendChild(c);b.data.push(a);b.Id.fa=b.data||[];return Sq(this.Rc,a)};Cr.prototype.Jb=function(){return this.Uo.open()};function Dr(a,b,c,d){var e=String(Rh(a,"url"));var g=String(Rh(a,"snippetName"));var h=new cq(b,new ql(c),e,g,d);(b=Rh(a,"modalTitle"))&&h.dj(String(b));return v(a,"click",function(a){a.preventDefault();return h.open()})};function Er(a){this.start=null!=a?a:0}Er.prototype.next=function(){return this.start++};Er.prototype.xc=function(a){return a.replace("__prefix__",this.start.toString())};function Fr(a){null==a&&(a={});Fr.m.constructor.call(this,a)}r(Fr,xh);Fr.prototype.Ca=function(a){var b;var c=Fg(this.l,"inline-line",a);var d=0;for(b=c.length;d<b;d++)a=c[d],Gr(this,a)};function Gr(a,b){var c=Hr(a.l,b);if(null==c)return null;w(c,!1);var d=new Ah({l:a.l,Xc:b});d.sp(c);return d};function Ir(a,b,c,d){this.xb=a;this.ki=b;this.$b=c;this.l=d;this.Sn=q(this.Sn,this);Ir.m.constructor.call(this);this.parent=Yf(this.xb);a=this.xb.cloneNode(!0);ah(a,this.Wk);this.clone=a;this.Yw=new Fr({l:this.l});a=Jr;this.Wk=a.wq;this.Sk=a.sq;ah(this.clone,a.sq,a.wq)}r(Ir,md);var Jr={sq:"even",wq:"odd"};function Kr(a,b,c){var d=Gr(a.Yw,b);null!=d&&d.A("action",a.Sn);c&&a.$b.xc(b)}
Ir.prototype.Sn=function(a){var b=a.target;a=Hr(this.l,b.Xc);if(null!=a)return a.checked=!0,this.dispatchEvent(new Cc("delete",b.Xc))};Ir.prototype.Wj=function(a){Zg(a,this.Sk);return Lr(this)};function Lr(a){var b;return b=[a.Wk,a.Sk],a.Sk=b[0],a.Wk=b[1],b}Ir.prototype.pa=function(a){var b;var c=this.l.se(this.parent);var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];d.push(w(da(g.lastElementChild)?g.lastElementChild:Gf(g.lastChild,!1),a))}return d};function Mr(a){this.fieldset=a;this.Ed=0;this.Hh=1E3;this.Gr=null}Mr.prototype.load=function(){var a;var b=this.fieldset.querySelectorAll("input[type\x3dhidden]");var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];za(e.name,"TOTAL_FORMS")?(this.Ed=parseInt(e.value,10),c.push(this.Gr=e)):za(e.name,"MAX_NUM_FORMS")?(this.Hh=parseInt(e.value,10),c.push(e)):za(e.name,"INITIAL_FORMS")?c.push(e):c.push(void 0)}return c};Mr.prototype.uf=function(a){var b;return null!=(b=this.Gr)?b.value=a:void 0};function Nr(a,b){this.l=a;this.$b=b;this.wc=q(this.wc,this);this.ao=q(this.ao,this);this.Ed=0;this.Hh=1E3;this.listener=this.zd=this.Pg=null}
Nr.prototype.ao=function(a){var b;a=a.target;if(ji(a,"empty")){this.Ed--;this.Pg.uf(this.Ed);var c=this.zd;var d,e;c.ki.start--;for(b=a;b=Xf(b);){eh(b,c.Sk);eh(b,c.Wk);var g=c.l.all("input, select, textarea",b);var h=0;for(d=g.length;h<d;h++){var k=g[h];var l=["name","id","data-for"];var m=0;for(e=l.length;m<e;m++){var p=l[m];if(null!=k[p]){var u=k[p].split("-");u[u.length-2]=parseInt(u[u.length-2],10)-1;k[p]=u.join("-")}}}}c.l.removeNode(a);return Lr(c)}b=this.l.all("input[type\x3dfile]",a);p=0;
for(k=b.length;p<k;p++)c=b[p],this.l.removeNode(c);w(a,!1);k=this.l.all("input, select",a);b=[];c=0;for(p=k.length;c<p;c++)a=k[c],b.push(a.removeAttribute("required"));return b};
Nr.prototype.wc=function(){var a=this.zd;var b=a.clone.cloneNode(!0);Kr(a,b,!0);a.Wj(b);var c,d;var e=a.l.all("input, select, textarea",b);var g=0;for(c=e.length;g<c;g++){var h=e[g];var k=["name","id","data-for"];var l=0;for(d=k.length;l<d;l++){var m=k[l];null!=h[m]&&(h[m]=a.ki.xc(h[m]))}}c=a.l.all("div[data-for]",b);d=[];g=0;for(l=c.length;g<l;g++)m=c[g],h=Rh(m,"for"),null!=h?d.push(Qh(m,"for",a.ki.xc(h))):d.push(void 0);a.parent.appendChild(b);a.ki.next();this.Ed++;this.Pg.uf(this.Ed);this.Ed===
this.Hh&&(Zg(this.Cc,"disabled"),ed(this.listener));if(1===this.l.all("input[type\x3dfile]",b).length)return a=(a=Dg(this.l,"input[type\x3dfile]",b))?a.click():void 0,a};function Hr(a,b){var c;var d=a.all("input[type\x3dcheckbox]",b);var e=[];var g=0;for(c=d.length;g<c;g++){var h=d[g];za(h.id,"DELETE")&&e.push(h)}return e.length?e[0]:null};function Or(a,b){this.Rm=a instanceof df?a:new df(a,b)}r(Or,kk);Or.prototype.yc=function(a,b,c,d){hk(ig(a),0,a,b,this.Rm,c,null,d)};function Pr(a){this.ob=new Pd;a&&this.addAll(a)}function Qr(a){var b=typeof a;return"object"==b&&a||"function"==b?"o"+pa(a):b.substr(0,1)+a}f=Pr.prototype;f.Gd=function(){return this.ob.Gd()};f.add=function(a){this.ob.set(Qr(a),a)};f.addAll=function(a){a=Id(a);for(var b=a.length,c=0;c<b;c++)this.add(a[c])};f.sd=function(a){a=Id(a);for(var b=a.length,c=0;c<b;c++)this.remove(a[c])};f.remove=function(a){return this.ob.remove(Qr(a))};f.clear=function(){this.ob.clear()};f.isEmpty=function(){return this.ob.isEmpty()};
f.contains=function(a){return this.ob.ke(Qr(a))};f.Wr=function(a){var b=new Pr;a=Id(a);for(var c=0;c<a.length;c++){var d=a[c];this.contains(d)&&b.add(d)}return b};f.Ib=function(){return this.ob.Ib()};f.clone=function(){return new Pr(this)};f.tb=function(a){return this.Gd()==Hd(a)&&Rr(this,a)};
function Rr(a,b){var c=Hd(b);if(a.Gd()>c)return!1;!(b instanceof Pr)&&5<c&&(b=new Pr(b));return Md(a,function(a){var c=b;if(c.contains&&"function"==typeof c.contains)a=c.contains(a);else if(c.di&&"function"==typeof c.di)a=c.di(a);else if(ma(c)||ea(c))a=mb(c,a);else a:{for(var d in c)if(c[d]==a){a=!0;break a}a=!1}return a})};function Sr(a,b,c){this.S=c||(a?hf(lf(document,a)):hf());mk.call(this,this.S.D("DIV",{style:"position:absolute;display:none;"}));this.Wm=new df(1,1);this.Lf=new Pr;this.xg=null;a&&this.zf(a);null!=b&&Kf(this.o(),b)}r(Sr,mk);var Tr=[];f=Sr.prototype;f.rc=null;f.className="goog-tooltip";f.vt=500;f.bw=0;f.I=function(){return this.S};
f.zf=function(a){a=lf(document,a);this.Lf.add(a);v(a,"mouseover",this.Wg,!1,this);v(a,"mouseout",this.uk,!1,this);v(a,"mousemove",this.Or,!1,this);v(a,"focus",this.handleFocus,!1,this);v(a,"blur",this.uk,!1,this)};f.detach=function(a){if(a)a=lf(document,a),Ur(this,a),this.Lf.remove(a);else{for(var b=this.Lf.Ib(),c=0;a=b[c];c++)Ur(this,a);this.Lf.clear()}};
function Ur(a,b){dd(b,"mouseover",a.Wg,!1,a);dd(b,"mouseout",a.uk,!1,a);dd(b,"mousemove",a.Or,!1,a);dd(b,"focus",a.handleFocus,!1,a);dd(b,"blur",a.uk,!1,a)}f.Cp=function(a){var b=this.o();b&&Ff(b);Sr.m.Cp.call(this,a);a?(b=this.S.R.body,b.insertBefore(a,b.lastChild),ac(this.xg),this.xg=new Vl(this.o()),$b(this,va(ac,this.xg)),v(this.xg,"focusin",this.Uh,void 0,this),v(this.xg,"focusout",this.xl,void 0,this)):(ac(this.xg),this.xg=null)};
f.getState=function(){return this.qf?this.va()?4:1:this.Ci?3:this.va()?2:0};f.$o=function(){if(!fk.prototype.$o.call(this))return!1;if(this.anchor)for(var a,b=0;a=Tr[b];b++)If(a.o(),this.anchor)||a.$(!1);mb(Tr,this)||Tr.push(this);a=this.o();a.className=this.className;this.Uh();v(a,"mouseover",this.Qr,!1,this);v(a,"mouseout",this.Pr,!1,this);Vr(this);return!0};
f.jf=function(){nb(Tr,this);for(var a=this.o(),b,c=0;b=Tr[c];c++)b.anchor&&If(a,b.anchor)&&b.$(!1);this.Ns&&this.Ns.xl();dd(a,"mouseover",this.Qr,!1,this);dd(a,"mouseout",this.Pr,!1,this);this.anchor=void 0;0==this.getState()&&(this.jl=!1);fk.prototype.jf.call(this)};f.js=function(a,b){this.anchor==a&&this.Lf.contains(this.anchor)&&(this.jl||!this.SA?(this.$(!1),this.va()||(this.anchor=a,this.setPosition(b||Wr(this,0)),this.$(!0))):this.anchor=void 0);this.qf=void 0};f.sn=function(){return this.rc};
f.qw=function(a){this.Ci=void 0;if(a==this.anchor){a=this.I();var b=a.sn();a=b&&this.o()&&a.contains(this.o(),b);null!=this.rc&&(this.rc==this.o()||this.Lf.contains(this.rc))||a||this.ar&&this.ar.rc||this.$(!1)}};function Xr(a,b){var c=uf(a.S.R);a.Wm.x=b.clientX+c.x;a.Wm.y=b.clientY+c.y}f.Wg=function(a){var b=Yr(this,a.target);this.rc=b;this.Uh();b!=this.anchor&&(this.anchor=b,this.qf||(this.qf=od(q(this.js,this,b,void 0),this.vt)),Zr(this),Xr(this,a))};
function Yr(a,b){try{for(;b&&!a.Lf.contains(b);)b=b.parentNode;return b}catch(c){return null}}f.Or=function(a){Xr(this,a);this.jl=!0};f.handleFocus=function(a){this.rc=a=Yr(this,a.target);this.jl=!0;if(this.anchor!=a){this.anchor=a;var b=Wr(this,1);this.Uh();this.qf||(this.qf=od(q(this.js,this,a,b),this.vt));Zr(this)}};function Wr(a,b){if(0==b){var c=a.Wm.clone();return new $r(c)}return new as(a.rc)}function Zr(a){if(a.anchor)for(var b,c=0;b=Tr[c];c++)If(b.o(),a.anchor)&&(b.ar=a,a.Ns=b)}
f.uk=function(a){var b=Yr(this,a.target),c=Yr(this,a.relatedTarget);b!=c&&(b==this.rc&&(this.rc=null),Vr(this),this.jl=!1,!this.va()||a.relatedTarget&&If(this.o(),a.relatedTarget)?this.anchor=void 0:this.xl())};f.Qr=function(){var a=this.o();this.rc!=a&&(this.Uh(),this.rc=a)};f.Pr=function(a){var b=this.o();this.rc!=b||a.relatedTarget&&If(b,a.relatedTarget)||(this.rc=null,this.xl())};function Vr(a){a.qf&&(ca.clearTimeout(a.qf),a.qf=void 0)}
f.xl=function(){2==this.getState()&&(this.Ci=od(q(this.qw,this,this.anchor),this.bw))};f.Uh=function(){this.Ci&&(ca.clearTimeout(this.Ci),this.Ci=void 0)};f.V=function(){this.$(!1);Vr(this);this.detach();this.o()&&Ff(this.o());this.rc=null;delete this.S;Sr.m.V.call(this)};function $r(a,b){Or.call(this,a,b)}r($r,Or);$r.prototype.yc=function(a,b,c){b=ig(a);b=lg(b);c=c?new Zf(c.top+10,c.right,c.bottom,c.left+10):new Zf(10,0,0,10);jk(this.Rm,a,8,c,b,9)&496&&jk(this.Rm,a,8,c,b,5)};
function as(a){lk.call(this,a,5)}r(as,lk);as.prototype.yc=function(a,b,c){var d=new df(10,0);hk(this.element,this.qr,a,b,d,c,9)&496&&hk(this.element,4,a,1,d,c,5)};function bs(a){null==a&&(a={});this.sk=q(this.sk,this);this.vk=q(this.vk,this);this.l=a.l;this.tl=!1;bs.m.constructor.call(this,a)}r(bs,On);bs.prototype.Ca=function(a){this.element=a;v(a,"click",this.vk);v(Dg(this.l,"body"),"click",this.sk);v(a,"touchstart",this.vk);v(Dg(this.l,"body"),"touchstart",this.sk);return bs.m.Ca.call(this,this.element)};bs.prototype.vk=function(a){a.stopPropagation();this.tl=!this.tl;return this.toggle()};bs.prototype.sk=function(){this.tl=!1;return this.toggle()};
bs.prototype.toggle=function(){return this.tl?Zg(this.element,"open"):ah(this.element,"open")};function cs(a){this.Gb=null!=a?a:"?";this.query={};this.wh(this.Gb)}cs.prototype.wh=function(a){this.Gb=a;this.Gb=this.Gb.replace("#","?");return this.uri=new yk(this.Gb)};function pp(a,b,c,d){null==d&&(d=!1);null==c||""===c&&!d?a.uri.removeParameter(b):Mk(a.uri,b,c)}cs.prototype.toString=function(){return String(this.uri.vc())};function ds(a,b,c){this.prefix=a;this.Gb=b;this.hs=c}f=ds.prototype;f.get=function(a){a=this.key(a);(a=this.Gb.uri.qb.get(a))||(a="");return a};function No(a){var b;var c=a.Gb.uri.qb.uc();var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];var h=a.prefix;g.substring(0,h.length)===h&&d.push(g.replace(a.prefix+"-",""))}return d}f.Ui=function(){var a;var b=No(this);var c=[];var d=0;for(a=b.length;d<a;d++){var e=b[d];c.push(this.remove(e))}return c};
f.set=function(a,b){pp(this.Gb,this.key(a),b);return this.hs.hash=this.Gb.toString()};f.remove=function(a){a=this.key(a);return this.Gb.uri.removeParameter(a)};f.key=function(a){return[this.prefix,a].join("-")};function es(){}es.prototype.apply=function(){};function fs(a){this.R=a||{cookie:""}}f=fs.prototype;f.isEnabled=function(){return navigator.cookieEnabled};f.set=function(a,b,c,d,e,g){if(/[;=\s]/.test(a))throw Error('Invalid cookie name "'+a+'"');if(/[;\r\n]/.test(b))throw Error('Invalid cookie value "'+b+'"');da(c)||(c=-1);e=e?";domain\x3d"+e:"";d=d?";path\x3d"+d:"";g=g?";secure":"";c=0>c?"":0==c?";expires\x3d"+(new Date(1970,1,1)).toUTCString():";expires\x3d"+(new Date(wa()+1E3*c)).toUTCString();this.R.cookie=a+"\x3d"+b+e+d+c+g};
f.get=function(a,b){for(var c=a+"\x3d",d=(this.R.cookie||"").split(";"),e=0,g;e<d.length;e++){g=Da(d[e]);if(0==g.lastIndexOf(c,0))return g.substr(c.length);if(g==a)return""}return b};f.remove=function(a,b,c){var d=this.ke(a);this.set(a,"",0,b,c);return d};f.uc=function(){return gs(this).keys};f.Ib=function(){return gs(this).values};f.isEmpty=function(){return!this.R.cookie};f.Gd=function(){return this.R.cookie?(this.R.cookie||"").split(";").length:0};f.ke=function(a){return da(this.get(a))};
f.di=function(a){for(var b=gs(this).values,c=0;c<b.length;c++)if(b[c]==a)return!0;return!1};f.clear=function(){for(var a=gs(this).keys,b=a.length-1;0<=b;b--)this.remove(a[b])};function gs(a){a=(a.R.cookie||"").split(";");for(var b=[],c=[],d,e,g=0;g<a.length;g++)e=Da(a[g]),d=e.indexOf("\x3d"),-1==d?(b.push(""),c.push(e)):(b.push(e.substring(0,d)),c.push(e.substring(d+1)));return{keys:b,values:c}};function hs(a){hs.m.constructor.call(this);this.pr=new fs(a);var b,c,d;var e=[document.body,document.getElementById("wrapper")];var g=[];var h=0;for(d=e.length;h<d;h++){a=e[h];var k;if(k=null!=a)k=Rh(a,"initial"),k=!(null==k||""===k);k&&g.push(a)}this.vv=0==g.length?null:null!=(b=qd(Rh(g[0],"initial")))?null!=(c=b.net)?c["csrf-token"]:void 0:void 0}r(hs,es);
hs.prototype.apply=function(a){var b;var c=[this.vv,this.pr.get("csrftoken")];var d=[];var e=0;for(b=c.length;e<b;e++){var g=c[e];null!=g&&d.push(g)}0==d.length?(g=this.pr.get("Authorization"),null!=g&&(a.Authorization=g)):a["X-CsrfToken"]=d[0]};function is(a,b,c){this.$b=a;this.l=b;this.Uk=c}var js={tj:"snippet",sA:"snippetType",Xl:"snippetLoader"},ks={tj:"data-snippet",Xl:"data-snippet-loader"};f=is.prototype;
f.apply=function(a){var b,c;if(a.snippets){var d={};var e=this.l.all("*["+ks.Xl+"]");var g=0;for(b=e.length;g<b;g++){var h=e[g];var k=String(Rh(h,js.Xl)).split(" ");var l=0;for(c=k.length;l<c;l++){var m=k[l];d[m]=h}}k=a.snippets;g=[];for(m in k)h=k[m],null!=d[m]&&this.xc(d[m],h),a=Dg(this.l,"*["+ks.tj+"\x3d"+m+"]"),null!=a?g.push(this.xc(a,h)):g.push(void 0);return g}};
f.xc=function(a,b){var c;var d=null!=(c=Rh(a,"snippetType"))?c:"replace";return function(){switch(d){case "replace":return this.replace;case "append":return this.append;case "prepend":return this.Pw}}.call(this).call(this,b,a)};f.replace=function(a,b){b.innerHTML=a;return this.$b.xc(b)};f.append=function(a,b){var c=this.l.Sr(a);b.appendChild(c);return this.$b.xc(c)};f.Pw=function(a,b){var c=this.l.Sr(a);Vf(b,c);return this.$b.xc(c)};function ls(){}ls.prototype.apply=function(){};function ms(a){this.$c=a}r(ms,ls);ms.prototype.apply=function(a){if(401===fe(a.target))return this.$c.location.reload(!0)};function ns(a,b){this.ka=a;this.gf=b;hp(this.gf,os)}var os={error:"Internal error occured. Service is unavailable, sorry.",loading:"Loading...",offline:"Offline..."};f=ns.prototype;
f.apply=function(a,b){var c;if(!Jb(this.gf,String(b))){null!=a.errors?(la(a.errors)||ea(a.errors))&&Mh(this.ka,a.errors):null!=a.error&&Mh(this.ka,a.error);var d=null!=(c=a.messages)?c:a.message;if(null!=d){for(h in d)c=d[h],ea(c)&&this.ka.Lh(c,h);var e;if(d=null!=(e=d.errors)?e["non-field-errors"]:void 0){var g=[];var h=0;for(c=d.length;h<c;h++)e=d[h],g.push(Mh(this.ka,e));d=g}else d=void 0;return d}}};
f.error=function(a){if(Jb(this.gf,String(a))){if(a=String(a),null!=this.gf[a])return Mh(this.ka,this.gf[a])}else return Mh(this.ka,this.gf.error)};f.ig=function(){return[ps(this,this.gf.loading,"loading",!1,!1)]};f.yw=function(){return[ps(this,this.gf.offline,"info",!0,!0)]};function ps(a,b,c,d,e){b=new Gh({l:a.ka.I(),Rk:b,severity:"info",ek:d,Og:e});null!=c&&b.addClass(c);a.ka.$a(b);return b}f.Ij=function(){return this.ka.Ij()};function qs(){this.log=function(){}};function rs(a){this.jq=null!=a?a:null;this.Rn=q(this.Rn,this);this.Ig=q(this.Ig,this);this.Wi=new qs;this.$b=new xa(this.Ig,this.Wi);this.Vs=new xa(this.Ig,this.Wi);null==this.jq&&(this.jq=me);this.Gb=this.ii=this.Qb=null;this.Uk={};this.Qc={};this.$c=null}
function ss(a,b,c){var d=ts,e={Xs:!0};d.$c=a;null==c&&(c={});null==e&&(e={});d.dateFormat=null;a=null!=d.$c.console?q(d.$c.console.error,d.$c.console):function(){};d.Wi.log=a;d.ii=d.$c.document;d.Gb=new cs(d.$c.location.hash);d.Uk={app:d,Gb:d.Gb,ka:b};null==d.dateFormat&&(d.dateFormat={lx:Ri,Lw:Pj,format:"dd'.'MM'.'yyyy"});Ti=d.dateFormat.lx;Rj=d.dateFormat.Lw;var g=new Cg(d.ii);var h=new is(d.$b,g,d.Uk);a=new hs(d.$c.document);b=new ns(b,c);d.Qb=new d.jq(b,a,h,g);null!=e.Xs&&e.Xs&&se(d.Qb,new ms(d.$c));
e=new Wo;e.pa(!0);e.A("navigate",d.Rn);d.Vs.xc(d.ii);return d.$b.xc(d.ii)}rs.prototype.Ig=function(a,b){return a(b,new Cg(this.ii),this.Qb,this.Uk)};rs.prototype.Rn=function(){return this.Gb.wh(this.$c.location.hash)};rs.prototype.on=function(a,b){return this.$b.register(a,b)};rs.prototype.once=function(a,b){return this.Vs.register(a,b)};
function us(a){var b=ts;b.once(".flash",function(b){return a.l.jo(b)?a.J(b):a.render(b)});b.on("table.js-grid",function(a,b,e,g){var c;a:{var d;var l=b.all("table.js-grid");var m=c=0;for(d=l.length;c<d;m=++c){var p=l[m];if(p===a){c=m;break a}}c=-1}p=g.app.$b;m=g.app;c="g-"+c;Jb(m.Qc,c)||(m.Qc[c]=new ds(c,m.Gb,m.$c.location));m=m.Qc[c];var u;null==u&&(u=new Wp);return Xp(a,b,e,p,m,g.ka,u)});b.on(".remote-button",function(a,b,e){b=new dq(b,e);b.J(a);return b});b.on("form.ajax",function(a,b,e){e=new ql(e);
b=new bq(e,b);b.Hj=!0;return b.J(a)});b.on("fieldset.inline-js",function(a,b,e,g){b=new Nr(b,g.app.$b);e=!dh(a,"disabled");var c,d;null==e&&(e=!0);b.Cc=Eg(b.l,"dynamic",a);if(null!=b.Cc){b.Pg=new Mr(a);b.Pg.load();b.Ed=b.Pg.Ed;b.Hh=b.Pg.Hh;var l=new Er(parseInt(b.Ed,10));g=Fg(b.l,"inline-line",a);a=g[g.length-1];b.zd=new Ir(a,l,b.$b,b.l);if(1<g.length)for(l=c=0,d=g.length-2;0<=d?c<=d:c>=d;l=0<=d?++c:--c)Kr(b.zd,g[l],!1);e?(b.listener=v(b.Cc,"click",b.wc),b.zd.A("delete",b.ao)):b.zd.pa(!1);b.l.removeNode(a);
b=w(b.Cc,e)}else b=void 0;return b});b.on("select.fulltext-search",function(a,b){return xr(a,b)});b.on("select.fulltext-search-multiple",function(a,b){return zr(a,b)});b.on("*[data-title]",function(a,b){var c=Rh(a,"title");return new Sr(a,c,b)});b.on(".navbar-toggle",function(a,b){return xk(a,b)});b.on("*[data-snippet-onload]",function(a,b,e){e=(a=String(Rh(a,"snippetOnload")))?e.ic(new je).send(a,"GET","",{}):void 0;return e});b.on(".related-button",function(a,b,e,g){g=g.app.$b;e=new ql(e);return(new Cr(b,
e,g)).J(a)});b.on(".dropdown",function(a,b){return(new bs({l:b})).J(a)});b.on("input[type\x3ddatetime], input.datetime, input.date",function(a,b,e,g){return(new wk(b,g.app.dateFormat.format)).J(a)})};function vs(){return Tp=vs.m.constructor.apply(this,arguments)}r(vs,me);vs.prototype.Em=function(a){a=vs.m.Em.call(this,a);a.A(le,function(a){return window.__dispatch(ie(a.target)["X-Authorization-Expiration"])});return a};var ts=new rs(vs);function ws(a,b){this.l=a;this.duration=null!=b?b:xs;this.hk=q(this.hk,this);this.toggle=q(this.toggle,this)}var ys={Lx:"dropup-button",au:"caret-marker",nq:"fa-angle-up",Pl:"fa-angle-left",pq:"collapse",OPENED:"opened",CLOSED:"closed"},xs=500;f=ws.prototype;
f.J=function(a){this.Yj=a;this.Ej=Eg(this.l,ys.au,this.Yj);a=Rh(this.Yj,"target");if(null==a)throw Error("Dropup button element has no 'target' attribute!");this.Ec=Dg(this.l,"#"+a+", ."+a,this.l.R);if(null==this.Ec)console.warn("Dropup element with specified id or class  '"+a+"', does not exist!");else return w(this.Ec,!1),ag(this.Ec,"overflow","hidden"),this.vm=!1,v(this.Yj,"click",this.toggle)};f.toggle=function(a){a.preventDefault();return"none"==this.Ec.style.display||this.vm?this.open():this.close()};
f.open=function(){var a=this;null==this.height&&(this.height=this.Jr());w(this.Ec,!0);var b=zs(this,0,this.height);b.A("begin",function(){var b;var d=a.hk();var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(ch(h,ys.CLOSED,ys.OPENED))}return e});b.A("finish",function(){ah(a.Ec,ys.pq);return a.Bl()});return b.play()};
f.close=function(){var a=this;var b=zs(this,this.height,0);b.A("finish",function(){var b;var d=a.hk();var e=[];var g=0;for(b=d.length;g<b;g++){var h=d[g];e.push(ch(h,ys.OPENED,ys.CLOSED))}return e});b.A("finish",function(){w(a.Ec,!1);Zg(a.Ec,ys.pq);return a.Bl()});return b.play()};f.hk=function(){return[this.Ec,this.Yj]};f.Bl=function(){if(null!=this.Ej)return dh(this.Ej,ys.Pl)?ch(this.Ej,ys.Pl,ys.nq):ch(this.Ej,ys.nq,ys.Pl)};
function zs(a,b,c){b=new Pg([b],[c],a.duration);setTimeout(function(){a.Ec.style.height=hg(c,!0)},a.duration);b.A("animate",function(b){a.Ec.style.height=hg(b.coords[0],!0)});b.A("begin",function(){return a.vm=!0});b.A("finish",function(){return a.vm=!1});return b}f.Jr=function(){w(this.Ec,!0);var a=tg(this.Ec).height;w(this.Ec,!1);return a};function As(a){this.Kn=q(this.Kn,this);As.m.constructor.call(this,a);this.kq=a.kq;this.ka=a.ka;this.Sm=this.lq=null}r(As,xh);As.prototype.ba=function(a){return"select"===a.tagName.toLowerCase()?Rf(a.options[a.selectedIndex]):Em(a)||Rf(a)};As.prototype.J=function(a){var b=this;this.G=a;this.Sm=Rh(this.G,"copiedMessage");this.lq=new this.kq(this.G);this.Bt=this.l.o(String(Rh(this.G,"copyTarget")));if(null!=this.Bt)return this.lq.on("ready",function(){return b.lq.on("copy",b.Kn)})};
As.prototype.Kn=function(a){a.clipboardData.setData("text/plain",this.ba(this.Bt));if(this.Sm)return this.ka.Lh(this.Sm,"success",void 0,!0,1E3)};ts.on("input[type\x3ddatetime], input.datetime, input.date",function(a,b){Ti=Ri;Rj=Pj;return(new wk(b,"dd'.'MM'.'yyyy")).J(a)});ts.on(".hint",function(a){return v(a,"click",function(a){return a.preventDefault()})});ts.on(".copy-button-js",function(a,b,c,d){return(new As({kq:ne(b).ZeroClipboard,l:b,ka:d.ka})).J(a)});ts.on(".dropup",function(a,b){var c;null==c&&(c=void 0);null==c&&(c=Nm(String(Rh(a,"duration")),250));return(new ws(b,c)).J(a)});ts.on(".dropdown",function(a,b){return(new bs({l:b})).J(a)});function Bs(a){this.l=a;this.Tn=q(this.Tn,this);this.forwardTo=null}Bs.prototype.forward=function(a,b){this.forwardTo=b;if(null!=this.forwardTo)return v(a,["keypress"],this.Tn)};Bs.prototype.Tn=function(a){if(13===a.keyCode)return this.forwardTo.click(),a.preventDefault()};function Cs(a,b,c){this.l=a;this.Mt=b;this.Nt=c;this.Gn=q(this.Gn,this);this.Zn=q(this.Zn,this);this.Zf=q(this.Zf,this);var d=this.Mt;var e=[];b=0;for(c=d.length;b<c;b++)a=d[b],e.push(Em(a[0]));this.Ls=e}
Cs.prototype.J=function(a){this.Of=Dg(this.l,"input",a);if(null==this.Of)throw Error("Missing field for a parent element for component app.ui.form.PopupField");this.hidden=this.l.G("input",{type:"hidden",name:this.Of.name,value:this.Of.value});this.l.Ki(this.hidden,a);this.dc=new Ao(void 0,void 0,this.l);this.dc.kb.clear();this.dc.kb.set("ok","OK",!0);v(this.Of.form,"submit",this.Zf);v(this.dc,pm,this.Zn);v(this.dc,lm,this.Gn);return this.l.appendChild(this.dc.ub(),a)};
Cs.prototype.Zf=function(a){var b=this;var c=hb(this.Mt,function(a,c,d){var e=c[0];c=c[1];e=Em(e);a[0]+=e===b.Nt&&e!==b.Ls[d]?c:0;a[1]=a[1]||e===b.Nt&&e!==b.Ls[d];return a},[0,!1]);var d=c[0];if(c[1])return a.preventDefault(),Fm(this.Of,d),this.dc.open()};Cs.prototype.Zn=function(a){if("ok"===a.key)return Fm(this.hidden,Em(this.Of)),this.hidden.form.submit()};Cs.prototype.Gn=function(){return yp(this.hidden.form,this.l)};function Ds(a,b){var c;null==b&&(b=null);null==b&&(b=function(a){return a});var d=0;for(c=a.length;d<c;d++){var e=a[d];if(!b(e))return!1}return!0}var Es={string:String,json:qd,decimal:Nm};function Fs(a,b,c,d,e,g){var h={"true":{add:"fa-check-circle",remove:"fa-times-circle",msg:"valid"},"false":{add:"fa-times-circle",remove:"fa-check-circle",msg:"invalid"}};return function(){var k=h[c(Em(b),d,a)];Zg(e,k.add);ah(e,k.remove);null!=g?(k=g[k.msg],k=e.setAttribute("title",null!=k?String(k):"")):k=void 0;return k}}
var Gs={eq:function(a,b,c){b=c.o(String(b));return null!=b?a===Em(b):!1},max:function(a,b){return null!=a&&a.length<=Number(b)},min:function(a,b){return null!=a&&a.length>=Number(b)},re:function(a,b){return Ds(b,function(b){return(new RegExp(b)).test(a)})}};function Hs(a,b){this.l=a;this.Qb=b;this.jd=q(this.jd,this)}var Is={api:"string",fields:"json",tokenName:"string",limit:"decimal",tokenMinLength:"decimal"};
Hs.prototype.J=function(a){this.G=a;var b={};for(e in Is){var c=Is[e];var d=Rh(a,e);if(null==d)throw Error("Missing attribute `data-"+e+"` for element #"+a.id);if(null==Es[c])throw Error("Unknown param type `"+c+"` for element #"+a.id);b[e]=Es[c](d)}this.ec=b;a=this.G.id.split("-").slice(0,-1);this.Ma={};var e=this.ec.fields;for(g in e){d=a.concat(g).join("-");e=this.l.o(d);if(null==e)throw Error("Missing a field #"+d+" for an element #"+this.G.id);this.Ma[g]=xr(e,this.l)}var g=this.G;a=this.l;e=
this.ec.api;d=this.ec.tokenName;b=this.ec.limit;c=this.ec.tokenMinLength;a=new vr(a,new er(a,0,Br(g,a)),e,d,b,c);a.J(g);return v(a.Rc,"update",this.jd)};Hs.prototype.jd=function(a){var b;var c=new In;var d=this.Ma;var e=[];for(b in d){var g=d[b];e.push(Vq(g,c.format(a.xb,this.ec.fields[b])))}return e};function Js(a,b){return v(a,"submit",function(){return b.onbeforeunload=null})}ts.on(".member-registration-form",function(a,b){return(new Bs(b)).forward(a,Eg(b,"send-verification-sms-btn",a))});ts.on(".click-once-btn",function(a,b){return v(a,"click",function(){return b.hide(a)})});ts.on("form",function(a,b){zp(a,b);return Js(a,ne(b))});ts.on(".gate-form input, .gate-form button",function(a,b){return b.ex(a,{tabIndex:1})});ts.on(".modal-form",function(a,b,c,d){return Dr(a,b,c,d.app.$b)});
ts.on(".field.issued_eshop_credit_note",function(a,b){var c=b.all(".inline.orderitem .inline-line .field__total_price");c=wb(fb(b.all(".inline.orderitem .field__state select"),ab),fb(c,function(a){return parseFloat(a.children[0].children[0].getAttribute("title"))}));return(new Cs(b,c,"2")).J(a)});
ts.on("input[data-validators]",function(a,b){var c;if(Th(a,"validators")){var d=b.G("i",{"class":"fa field-validation-icon"},Yf(a));var e=Th(a,"messages")?qd(Rh(a,"messages")):{};var g=qd(Rh(a,"validators"));var h=[];for(c in g){var k=g[c];null!=Gs[c]?h.push(Am(a,Fs(b,a,Gs[c],k,d,e))):h.push(void 0)}d=h}else d=void 0;return d});ts.on(".api-fulltext-search-prefill",function(a,b,c){return(new Hs(b,c)).J(a)});function Ks(a,b){this.$k=q(this.$k,this);Ks.m.constructor.call(this,a,b)}r(Ks,Sj);Ks.prototype.uq={Gu:"Dnes",NONE:"Smazat"};
Ks.prototype.$k=function(a,b){var c=this.I().createElement("TD");c.colSpan=null!=b?b:{2:3};c.className=this.X()+"-today-cont";Tj(this,c,this.uq.Gu,this.X()+"-today-btn");a.appendChild(c);c=this.I().createElement("TD");c.colSpan=null!=b?b:{4:3};a.appendChild(c);c=this.I().createElement("TD");c.colSpan=2;c.className=this.X()+"-none-cont";Tj(this,c,this.uq.NONE,this.X()+"-none-btn");return a.appendChild(c)};function Ls(a,b,c){var d,e,g,h;var k=d=null;var l=h=g=e=0;var m=c?".datetime-range-to":".date-range-to";c=c?"dd'.'MM'.'yyyy' 'HH':'mm":"dd'.'MM'.'yyyy";var p=new nj(c);var u=new uj(c);var A=Yf(a).querySelector(m);if(null==A)ne(b).console.warn("Missing "+m+" for input "+a.name);else{var D=function(a){return ll(Em(a))?null:Ib(Em(a))};var E=function(a){Zg(a,"open")};var J=function(a){ah(a,"open")};var M=function(a){ah(a,"hidden")};var O=function(a){return Zg(a,"hidden")};var aa=function(a,b){J(a);return O(b)};
var ra=function(a){a=new rk(a,null,b,new Ks("goog-date-picker",b));a.sl=!0;a.ck&&Vj(a);a.Mh=!0;a.bk&&Vj(a);a.fj=!0;ak(a);ck(a);Wj(a);return a};var Za=function(a){return null!=a?p.format(a):""};var Vc=function(){return null!=d||null!=k?""+Za(d)+" - "+Za(k):""};var W=function(a){return b.Yc(a,Vc())};var fj=function(a){var b;ah(a,"disabled");var c=a.querySelectorAll("select");var d=[];var e=0;for(b=c.length;e<b;e++)a=c[e],d.push(a.removeAttribute("disabled"))};var gb=function(a){var b;Zg(a,"disabled");
var c=a.querySelectorAll("select");var d=[];var e=0;for(b=c.length;e<b;e++)a=c[e],d.push(a.setAttribute("disabled","disabled"))};var tb=function(a,b,c){null!=c.date?(null==d&&(d=new ij,d.setHours(e),d.setMinutes(g)),d.setYear(c.date.getYear()),d.setMonth(c.date.getMonth()),d.setDate(c.date.getDate()),fj(b)):(d=null,gb(b));return W(a)};var zb=function(a,b,c){null!=c.date?(null==k&&(k=new ij,k.setHours(h),k.setMinutes(l)),k.setYear(c.date.getYear()),k.setMonth(c.date.getMonth()),k.setDate(c.date.getDate()),
fj(b)):(k=null,gb(b));return W(a)};var Ab=function(a,b,c){b(Nm(c.target.value));return W(a)};var Ib=function(a){var b=new ij;xj(u,a,b,0,!0);ll(a)&&(b.setHours(0),b.setMinutes(0));return b};var Wb=function(a,c,d){var e=new Jo({options:fb(vb(0,24),Iz)});e.D();e.qa(d.getHours());v(e.o(),"change",a);a=new Jo({options:fb(vb(0,60),Iz)});a.D();a.qa(d.getMinutes());v(a.o(),"change",c);return b.G("div","time-picker",[e.o(),b.G("div","separator",":"),a.o()])};var Xb=function(a,b,c,d,e,g,h,k){var l=Ib(Em(c));
c=ra(l);c.render(a.children[d]);e(l);e=Wb(va(Ab,b,g),va(Ab,b,h),l);a.children[d].appendChild(e);v(c,"select",va(k,b,e));return c};var pc=function(b,c,m,p,u){u.preventDefault();E(b);M(m);if(null==b.querySelector(".goog-date-picker"))return u=Xb(c,p,a,0,function(a){return d=a},function(a){d.setHours(a);return e=a},function(a){d.setMinutes(a);return g=a},tb),p=Xb(c,p,A,1,function(a){return k=a},function(a){k.setHours(a);return h=a},function(a){k.setMinutes(a);return l=a},zb),c.appendChild(bH(u,p)),c.appendChild(cH(b,
m))};var Ic=function(a,b,c){c.preventDefault();return aa(a,b)};var Wc=function(a,b,c,d){d.preventDefault();c=c();a.setDate(c.getStartDate());return b.setDate(c.Te)};var ld=function(b,c,e){e.preventDefault();aa(b,c);Em(a);Za(d);Fm(a,Za(d));Fm(A,Za(k));qk(a);return qk(A)};var Ld=function(a,b,c){c.preventDefault();return aa(a,b)};var he=function(a,b,c){c.preventDefault();a.setDate(null);return b.setDate(null)};var Je=function(){return b.G("button","btn btn-primary btn-xs",[b.G("i","glyphicon glyphicon-calendar fa fa-calendar"),
b.G("span","label",Vc()),b.G("span","caret")])};var pf=function(){return b.G("div","popup dropdown-menu",[b.G("div","date-picker-wrapper date-picker-wrapper__from"),b.G("div","date-picker-wrapper date-picker-wrapper__to")])};var og=function(){return b.G("div","overlay")};var $h=function(a,c){var d=b.G("button","btn btn-default",a);v(d,"click",c);return d};var pg=function(a,b){var c=a.clone();c.add(new Wi(Zi,b));return c};var Sl=function(a){var b=Lj().Te;var c=pg(b,-1);b.add(new Wi(Zi,-1));return new Dj(pg(b,
-1*a),c)};m=function(a){a=a();var b=a.Pp;var c=a.Te;return function(){return new Dj(b,pg(c,1))}};var Tl=[["Dnes",m(Lj)],["Včera",m(Jj)],["Posledních 7 dní",m(Mj)],["Posledních 30 dní",m(function(){return Sl(30)})],["Minulý týden",m(Oj)],["Minulý měsíc",m(Nj)]];var Ul=function(a,c,d){a=b.G("div",c,a);v(a,"click",d);return a};var bH=function(a,c){var d,e;return b.G("div","filter-buttons",[b.G("div","btn-group-vertical",pb(function(){var b;var g=[];var h=0;for(b=Tl.length;h<b;h++){var k=Tl[h];d=k[0];
e=k[1];g.push($h(d,va(Wc,a,c,e)))}return g}(),[Ul("Odstranit","btn btn-default",va(he,a,c))]))])};var cH=function(a,c){return b.G("div","action-buttons",[b.G("div","btn-group",[Ul("Filtrovat","btn btn-primary",va(ld,a,c)),Ul("Zavřít","btn btn-default",va(Ld,a,c))])])};var Iz=function(a){return 10>a?ym("0%s",a):String(a)};return function(){d=D(a);k=D(A);var c=Je();var e=pf();var g=og();O(g);var h=b.G("div",{"class":"date-range-wrapper"},[c,e,g]);Yf(a).appendChild(h);v(c,"click",va(pc,h,e,g,c.querySelector(".label")));
v(g,"click",va(Ic,h,g));O(a);return O(A)}()}};ha("app.start",function(a,b){var c=new Lh({l:new Cg(a.document)});us(c);ts.on("input.date-range-from",function(a,b){return Ls(a,b,!1)});ts.on("input.datetime-range-from",function(a,b){return Ls(a,b,!0)});return ss(a,c,b)});function Ms(a,b){this.Ja=[];this.Zc=b;for(var c=!0,d=a.length-1;0<=d;d--){var e=a[d]|0;c&&e==b||(this.Ja[d]=e,c=!1)}}var Ns={};function Os(a){if(-128<=a&&128>a){var b=Ns[a];if(b)return b}b=new Ms([a|0],0>a?-1:0);-128<=a&&128>a&&(Ns[a]=b);return b}function Ps(a){if(isNaN(a)||!isFinite(a))return Qs;if(0>a)return Ps(-a).ra();for(var b=[],c=1,d=0;a>=c;d++)b[d]=a/c|0,c*=Rs;return new Ms(b,0)}var Rs=4294967296,Qs=Os(0),Ss=Os(1),Ts=Os(16777216);f=Ms.prototype;
f.nj=function(){return 0<this.Ja.length?this.Ja[0]:this.Zc};f.Nd=function(){if(this.Ra())return-this.ra().Nd();for(var a=0,b=1,c=0;c<this.Ja.length;c++)var d=Us(this,c),a=a+(0<=d?d:Rs+d)*b,b=b*Rs;return a};
f.toString=function(a){a=a||10;if(2>a||36<a)throw Error("radix out of range: "+a);if(this.Xb())return"0";if(this.Ra())return"-"+this.ra().toString(a);for(var b=Ps(Math.pow(a,6)),c=this,d="";;){var e=Vs(c,b),g=(c.subtract(e.multiply(b)).nj()>>>0).toString(a),c=e;if(c.Xb())return g+d;for(;6>g.length;)g="0"+g;d=""+g+d}};function Us(a,b){return 0>b?0:b<a.Ja.length?a.Ja[b]:a.Zc}f.Xb=function(){if(0!=this.Zc)return!1;for(var a=0;a<this.Ja.length;a++)if(0!=this.Ja[a])return!1;return!0};
f.Ra=function(){return-1==this.Zc};f.Ao=function(){return 0==this.Ja.length&&-1==this.Zc||0<this.Ja.length&&0!=(this.Ja[0]&1)};f.tb=function(a){if(this.Zc!=a.Zc)return!1;for(var b=Math.max(this.Ja.length,a.Ja.length),c=0;c<b;c++)if(Us(this,c)!=Us(a,c))return!1;return!0};f.En=function(a){return 0<this.compare(a)};f.Fn=function(a){return 0<=this.compare(a)};f.Ni=function(a){return 0>this.compare(a)};f.Lo=function(a){return 0>=this.compare(a)};
f.compare=function(a){a=this.subtract(a);return a.Ra()?-1:a.Xb()?0:1};f.ra=function(){return this.Xo().add(Ss)};f.add=function(a){for(var b=Math.max(this.Ja.length,a.Ja.length),c=[],d=0,e=0;e<=b;e++){var g=d+(Us(this,e)&65535)+(Us(a,e)&65535),h=(g>>>16)+(Us(this,e)>>>16)+(Us(a,e)>>>16),d=h>>>16,g=g&65535,h=h&65535;c[e]=h<<16|g}return new Ms(c,c[c.length-1]&-2147483648?-1:0)};f.subtract=function(a){return this.add(a.ra())};
f.multiply=function(a){if(this.Xb()||a.Xb())return Qs;if(this.Ra())return a.Ra()?this.ra().multiply(a.ra()):this.ra().multiply(a).ra();if(a.Ra())return this.multiply(a.ra()).ra();if(this.Ni(Ts)&&a.Ni(Ts))return Ps(this.Nd()*a.Nd());for(var b=this.Ja.length+a.Ja.length,c=[],d=0;d<2*b;d++)c[d]=0;for(d=0;d<this.Ja.length;d++)for(var e=0;e<a.Ja.length;e++){var g=Us(this,d)>>>16,h=Us(this,d)&65535,k=Us(a,e)>>>16,l=Us(a,e)&65535;c[2*d+2*e]+=h*l;Ws(c,2*d+2*e);c[2*d+2*e+1]+=g*l;Ws(c,2*d+2*e+1);c[2*d+2*e+
1]+=h*k;Ws(c,2*d+2*e+1);c[2*d+2*e+2]+=g*k;Ws(c,2*d+2*e+2)}for(d=0;d<b;d++)c[d]=c[2*d+1]<<16|c[2*d];for(d=b;d<2*b;d++)c[d]=0;return new Ms(c,0)};function Ws(a,b){for(;(a[b]&65535)!=a[b];)a[b+1]+=a[b]>>>16,a[b]&=65535,b++}
function Vs(a,b){if(b.Xb())throw Error("division by zero");if(a.Xb())return Qs;if(a.Ra())return b.Ra()?Vs(a.ra(),b.ra()):Vs(a.ra(),b).ra();if(b.Ra())return Vs(a,b.ra()).ra();if(30<a.Ja.length){if(a.Ra()||b.Ra())throw Error("slowDivide_ only works with positive integers.");for(var c=Ss,d=b;d.Lo(a);)c=c.shiftLeft(1),d=d.shiftLeft(1);for(var e=c.pf(1),g=d.pf(1),h,d=d.pf(2),c=c.pf(2);!d.Xb();)h=g.add(d),h.Lo(a)&&(e=e.add(c),g=h),d=d.pf(1),c=c.pf(1);return e}c=Qs;for(d=a;d.Fn(b);){e=Math.max(1,Math.floor(d.Nd()/
b.Nd()));g=Math.ceil(Math.log(e)/Math.LN2);g=48>=g?1:Math.pow(2,g-48);h=Ps(e);for(var k=h.multiply(b);k.Ra()||k.En(d);)e-=g,h=Ps(e),k=h.multiply(b);h.Xb()&&(h=Ss);c=c.add(h);d=d.subtract(k)}return c}f.Xo=function(){for(var a=this.Ja.length,b=[],c=0;c<a;c++)b[c]=~this.Ja[c];return new Ms(b,~this.Zc)};f.Ru=function(a){for(var b=Math.max(this.Ja.length,a.Ja.length),c=[],d=0;d<b;d++)c[d]=Us(this,d)&Us(a,d);return new Ms(c,this.Zc&a.Zc)};
f.shiftLeft=function(a){var b=a>>5;a%=32;for(var c=this.Ja.length+b+(0<a?1:0),d=[],e=0;e<c;e++)d[e]=0<a?Us(this,e-b)<<a|Us(this,e-b-1)>>>32-a:Us(this,e-b);return new Ms(d,this.Zc)};f.pf=function(a){var b=a>>5;a%=32;for(var c=this.Ja.length-b,d=[],e=0;e<c;e++)d[e]=0<a?Us(this,e+b)>>>a|Us(this,e+b+1)<<32-a:Us(this,e+b);return new Ms(d,this.Zc)};function Xs(a,b){null!=a&&this.append.apply(this,arguments)}f=Xs.prototype;f.Oe="";f.set=function(a){this.Oe=""+a};f.append=function(a,b,c){this.Oe+=String(a);if(null!=b)for(var d=1;d<arguments.length;d++)this.Oe+=arguments[d];return this};f.clear=function(){this.Oe=""};f.toString=function(){return this.Oe};function Ys(a,b){this.nb=a|0;this.Wb=b|0}var Zs={},$s={};function at(a){return fc(Zs,a,function(a){return new Ys(a,0>a?-1:0)})}function bt(a){a|=0;return-128<=a&&128>a?at(a):new Ys(a,0>a?-1:0)}function ct(a){return isNaN(a)?at(0):a<=-dt?et():a+1>=dt?ft():0>a?ct(-a).ra():new Ys(a%gt|0,a/gt|0)}function ht(a,b){return new Ys(a,b)}
function it(a,b){if(0==a.length)throw Error("number format error: empty string");var c=b||10;if(2>c||36<c)throw Error("radix out of range: "+c);if("-"==a.charAt(0))return it(a.substring(1),c).ra();if(0<=a.indexOf("-"))throw Error('number format error: interior "-" character: '+a);for(var d=ct(Math.pow(c,8)),e=at(0),g=0;g<a.length;g+=8){var h=Math.min(8,a.length-g),k=parseInt(a.substring(g,g+h),c);8>h?(h=ct(Math.pow(c,h)),e=e.multiply(h).add(ct(k))):(e=e.multiply(d),e=e.add(ct(k)))}return e}
var gt=4294967296,dt=gt*gt/2;function ft(){return fc($s,jt,function(){return ht(-1,2147483647)})}function et(){return fc($s,kt,function(){return ht(0,-2147483648)})}function lt(){return fc($s,mt,function(){return bt(16777216)})}f=Ys.prototype;f.nj=function(){return this.nb};f.Nd=function(){return this.Wb*gt+(0<=this.nb?this.nb:gt+this.nb)};
f.toString=function(a){a=a||10;if(2>a||36<a)throw Error("radix out of range: "+a);if(this.Xb())return"0";if(this.Ra()){if(this.tb(et())){var b=ct(a);var c=this.div(b);b=c.multiply(b).subtract(this);return c.toString(a)+b.nj().toString(a)}return"-"+this.ra().toString(a)}c=ct(Math.pow(a,6));b=this;for(var d="";;){var e=b.div(c),g=(b.subtract(e.multiply(c)).nj()>>>0).toString(a);b=e;if(b.Xb())return g+d;for(;6>g.length;)g="0"+g;d=""+g+d}};f.Xb=function(){return 0==this.Wb&&0==this.nb};
f.Ra=function(){return 0>this.Wb};f.Ao=function(){return 1==(this.nb&1)};f.tb=function(a){return this.Wb==a.Wb&&this.nb==a.nb};f.Ni=function(a){return 0>this.compare(a)};f.Lo=function(a){return 0>=this.compare(a)};f.En=function(a){return 0<this.compare(a)};f.Fn=function(a){return 0<=this.compare(a)};f.compare=function(a){if(this.tb(a))return 0;var b=this.Ra(),c=a.Ra();return b&&!c?-1:!b&&c?1:this.subtract(a).Ra()?-1:1};f.ra=function(){return this.tb(et())?et():this.Xo().add(at(1))};
f.add=function(a){var b=this.Wb>>>16,c=this.Wb&65535,d=this.nb>>>16,e=a.Wb>>>16,g=a.Wb&65535,h=a.nb>>>16;a=0+((this.nb&65535)+(a.nb&65535));h=0+(a>>>16)+(d+h);d=0+(h>>>16);d+=c+g;b=0+(d>>>16)+(b+e)&65535;return ht((h&65535)<<16|a&65535,b<<16|d&65535)};f.subtract=function(a){return this.add(a.ra())};
f.multiply=function(a){if(this.Xb()||a.Xb())return at(0);if(this.tb(et()))return a.Ao()?et():at(0);if(a.tb(et()))return this.Ao()?et():at(0);if(this.Ra())return a.Ra()?this.ra().multiply(a.ra()):this.ra().multiply(a).ra();if(a.Ra())return this.multiply(a.ra()).ra();if(this.Ni(lt())&&a.Ni(lt()))return ct(this.Nd()*a.Nd());var b=this.Wb>>>16,c=this.Wb&65535,d=this.nb>>>16,e=this.nb&65535,g=a.Wb>>>16,h=a.Wb&65535,k=a.nb>>>16;a=a.nb&65535;var l=0+e*a;var m=0+(l>>>16)+d*a;var p=0+(m>>>16);m=(m&65535)+
e*k;p+=m>>>16;p+=c*a;var u=0+(p>>>16);p=(p&65535)+d*k;u+=p>>>16;p=(p&65535)+e*h;u=u+(p>>>16)+(b*a+c*k+d*h+e*g)&65535;return ht((m&65535)<<16|l&65535,u<<16|p&65535)};
f.div=function(a){if(a.Xb())throw Error("division by zero");if(this.Xb())return at(0);if(this.tb(et())){if(a.tb(at(1))||a.tb(at(-1)))return et();if(a.tb(et()))return at(1);var b=this.pf(1).div(a).shiftLeft(1);if(b.tb(at(0)))return a.Ra()?at(1):at(-1);var c=this.subtract(a.multiply(b));return b.add(c.div(a))}if(a.tb(et()))return at(0);if(this.Ra())return a.Ra()?this.ra().div(a.ra()):this.ra().div(a).ra();if(a.Ra())return this.div(a.ra()).ra();var d=at(0);for(c=this;c.Fn(a);){b=Math.max(1,Math.floor(c.Nd()/
a.Nd()));for(var e=Math.ceil(Math.log(b)/Math.LN2),e=48>=e?1:Math.pow(2,e-48),g=ct(b),h=g.multiply(a);h.Ra()||h.En(c);)b-=e,g=ct(b),h=g.multiply(a);g.Xb()&&(g=at(1));d=d.add(g);c=c.subtract(h)}return d};f.Xo=function(){return ht(~this.nb,~this.Wb)};f.Ru=function(a){return ht(this.nb&a.nb,this.Wb&a.Wb)};f.shiftLeft=function(a){a&=63;if(0==a)return this;var b=this.nb;return 32>a?ht(b<<a,this.Wb<<a|b>>>32-a):ht(0,b<<a-32)};
f.pf=function(a){a&=63;if(0==a)return this;var b=this.Wb;return 32>a?ht(this.nb>>>a|b<<32-a,b>>a):ht(b>>a-32,0<=b?0:-1)};var jt=1,kt=2,mt=6;var nt;if("undefined"===typeof x)var x={};if("undefined"===typeof ot)var ot=function(){throw Error("No *print-fn* fn set for evaluation environment");};if("undefined"===typeof pt)var pt=function(){throw Error("No *print-err-fn* fn set for evaluation environment");};var qt=null;if("undefined"===typeof rt)var rt=null;function st(){return new y(null,5,[tt,!0,ut,!0,vt,!1,wt,!1,xt,null],null)}function z(a){return null!=a&&!1!==a}function yt(a){return null==a}function zt(a){return a instanceof Array}
function At(a){return null==a?!0:!1===a?!0:!1}function B(a,b){return a[n(null==b?null:b)]?!0:a._?!0:!1}function Bt(a,b){var c=null==b?null:b.constructor,c=z(z(c)?c.lr:c)?c.Sj:n(b);return Error(["No protocol method ",a," defined for type ",c,": ",b].join(""))}function Ct(a){var b=a.Sj;return z(b)?b:""+C.c(a)}var Dt="undefined"!==typeof Symbol&&"function"===n(Symbol)?Symbol.iterator:"@@iterator";function Et(a){for(var b=a.length,c=Array(b),d=0;;)if(d<b)c[d]=a[d],d+=1;else break;return c}
function Ft(a){function b(a,b){a.push(b);return a}var c=[];return Gt?Gt(b,c,a):Ht.call(null,b,c,a)}function It(){}function Jt(){}function Kt(){}
var Lt=function Lt(b){if(null!=b&&null!=b.ya)return b.ya(b);var c=Lt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Lt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("ICounted.-count",b);},Mt=function Mt(b){if(null!=b&&null!=b.Ka)return b.Ka(b);var c=Mt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Mt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IEmptyableCollection.-empty",b);};function Nt(){}
var Ot=function Ot(b,c){if(null!=b&&null!=b.xa)return b.xa(b,c);var d=Ot[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Ot._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("ICollection.-conj",b);};function Pt(){}
var Qt=function Qt(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return Qt.h(arguments[0],arguments[1]);case 3:return Qt.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};
Qt.h=function(a,b){if(null!=a&&null!=a.aa)return a.aa(a,b);var c=Qt[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=Qt._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw Bt("IIndexed.-nth",a);};Qt.j=function(a,b,c){if(null!=a&&null!=a.ac)return a.ac(a,b,c);var d=Qt[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=Qt._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw Bt("IIndexed.-nth",a);};Qt.L=3;function Rt(){}
var St=function St(b){if(null!=b&&null!=b.hb)return b.hb(b);var c=St[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=St._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("ISeq.-first",b);},Tt=function Tt(b){if(null!=b&&null!=b.Db)return b.Db(b);var c=Tt[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Tt._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("ISeq.-rest",b);};function Ut(){}function Vt(){}
var Wt=function Wt(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return Wt.h(arguments[0],arguments[1]);case 3:return Wt.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};
Wt.h=function(a,b){if(null!=a&&null!=a.na)return a.na(a,b);var c=Wt[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=Wt._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw Bt("ILookup.-lookup",a);};Wt.j=function(a,b,c){if(null!=a&&null!=a.Y)return a.Y(a,b,c);var d=Wt[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=Wt._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw Bt("ILookup.-lookup",a);};Wt.L=3;function Xt(){}
var Yt=function Yt(b,c){if(null!=b&&null!=b.Vh)return b.Vh(b,c);var d=Yt[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Yt._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IAssociative.-contains-key?",b);},Zt=function Zt(b,c,d){if(null!=b&&null!=b.Rb)return b.Rb(b,c,d);var e=Zt[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=Zt._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw Bt("IAssociative.-assoc",b);};function $t(){}
var au=function au(b,c){if(null!=b&&null!=b.Wh)return b.Wh(b,c);var d=au[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=au._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IFind.-find",b);};function bu(){}var cu=function cu(b,c){if(null!=b&&null!=b.Df)return b.Df(b,c);var d=cu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=cu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IMap.-dissoc",b);};function du(){}
var eu=function eu(b){if(null!=b&&null!=b.Km)return b.Km();var c=eu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=eu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IMapEntry.-key",b);},fu=function fu(b){if(null!=b&&null!=b.Lm)return b.Lm();var c=fu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=fu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IMapEntry.-val",b);};function gu(){}
var hu=function hu(b,c){if(null!=b&&null!=b.Mm)return b.Mm(b,c);var d=hu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=hu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("ISet.-disjoin",b);},iu=function iu(b){if(null!=b&&null!=b.Mg)return b.Mg(b);var c=iu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=iu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IStack.-peek",b);},ju=function ju(b){if(null!=b&&null!=b.Ng)return b.Ng(b);var c=ju[n(null==
b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=ju._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IStack.-pop",b);};function ku(){}var lu=function lu(b,c,d){if(null!=b&&null!=b.Yh)return b.Yh(b,c,d);var e=lu[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=lu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw Bt("IVector.-assoc-n",b);};function mu(){}
var nu=function nu(b){if(null!=b&&null!=b.Pe)return b.Pe(b);var c=nu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=nu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IDeref.-deref",b);};function ou(){}var pu=function pu(b){if(null!=b&&null!=b.sa)return b.sa(b);var c=pu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=pu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IMeta.-meta",b);};function qu(){}
var ru=function ru(b,c){if(null!=b&&null!=b.za)return b.za(b,c);var d=ru[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=ru._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IWithMeta.-with-meta",b);};function su(){}
var tu=function tu(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return tu.h(arguments[0],arguments[1]);case 3:return tu.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};
tu.h=function(a,b){if(null!=a&&null!=a.Bb)return a.Bb(a,b);var c=tu[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=tu._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw Bt("IReduce.-reduce",a);};tu.j=function(a,b,c){if(null!=a&&null!=a.Cb)return a.Cb(a,b,c);var d=tu[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=tu._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw Bt("IReduce.-reduce",a);};tu.L=3;
var uu=function uu(b,c,d){if(null!=b&&null!=b.Kg)return b.Kg(b,c,d);var e=uu[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=uu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw Bt("IKVReduce.-kv-reduce",b);},vu=function vu(b,c){if(null!=b&&null!=b.P)return b.P(b,c);var d=vu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=vu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IEquiv.-equiv",b);},wu=function wu(b){if(null!=b&&null!=b.ha)return b.ha(b);
var c=wu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=wu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IHash.-hash",b);};function xu(){}var yu=function yu(b){if(null!=b&&null!=b.oa)return b.oa(b);var c=yu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=yu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("ISeqable.-seq",b);};function zu(){}function Au(){}function Bu(){}function Cu(){}
var Du=function Du(b){if(null!=b&&null!=b.Lg)return b.Lg(b);var c=Du[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Du._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IReversible.-rseq",b);},Eu=function Eu(b,c){if(null!=b&&null!=b.kr)return b.kr(0,c);var d=Eu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Eu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IWriter.-write",b);};function Fu(){}
var Gu=function Gu(b,c,d){if(null!=b&&null!=b.$h)return b.$h(b,c,d);var e=Gu[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=Gu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw Bt("IWatchable.-notify-watches",b);},Hu=function Hu(b,c,d){if(null!=b&&null!=b.Zh)return b.Zh(b,c,d);var e=Hu[n(null==b?null:b)];if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=Hu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw Bt("IWatchable.-add-watch",b);},Iu=function Iu(b,
c){if(null!=b&&null!=b.ai)return b.ai(b,c);var d=Iu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Iu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IWatchable.-remove-watch",b);},Ju=function Ju(b){if(null!=b&&null!=b.Jg)return b.Jg(b);var c=Ju[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Ju._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IEditableCollection.-as-transient",b);},Ku=function Ku(b,c){if(null!=b&&null!=b.Ff)return b.Ff(b,
c);var d=Ku[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Ku._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("ITransientCollection.-conj!",b);},Lu=function Lu(b){if(null!=b&&null!=b.Xh)return b.Xh(b);var c=Lu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Lu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("ITransientCollection.-persistent!",b);},Mu=function Mu(b,c,d){if(null!=b&&null!=b.Ef)return b.Ef(b,c,d);var e=Mu[n(null==b?null:b)];if(null!=
e)return e.j?e.j(b,c,d):e.call(null,b,c,d);e=Mu._;if(null!=e)return e.j?e.j(b,c,d):e.call(null,b,c,d);throw Bt("ITransientAssociative.-assoc!",b);},Nu=function Nu(b){if(null!=b&&null!=b.dr)return b.dr();var c=Nu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Nu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IChunk.-drop-first",b);},Ou=function Ou(b){if(null!=b&&null!=b.Jm)return b.Jm(b);var c=Ou[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Ou._;if(null!=
c)return c.c?c.c(b):c.call(null,b);throw Bt("IChunkedSeq.-chunked-first",b);},Pu=function Pu(b){if(null!=b&&null!=b.Kj)return b.Kj(b);var c=Pu[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Pu._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IChunkedSeq.-chunked-rest",b);},Qu=function Qu(b,c){if(null!=b&&null!=b.mc)return b.mc(b,c);var d=Qu[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=Qu._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IReset.-reset!",
b);},Ru=function Ru(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return Ru.h(arguments[0],arguments[1]);case 3:return Ru.j(arguments[0],arguments[1],arguments[2]);case 4:return Ru.N(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return Ru.ca(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};
Ru.h=function(a,b){if(null!=a&&null!=a.Oj)return a.Oj(a,b);var c=Ru[n(null==a?null:a)];if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);c=Ru._;if(null!=c)return c.h?c.h(a,b):c.call(null,a,b);throw Bt("ISwap.-swap!",a);};Ru.j=function(a,b,c){if(null!=a&&null!=a.Pj)return a.Pj(a,b,c);var d=Ru[n(null==a?null:a)];if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);d=Ru._;if(null!=d)return d.j?d.j(a,b,c):d.call(null,a,b,c);throw Bt("ISwap.-swap!",a);};
Ru.N=function(a,b,c,d){if(null!=a&&null!=a.Qj)return a.Qj(a,b,c,d);var e=Ru[n(null==a?null:a)];if(null!=e)return e.N?e.N(a,b,c,d):e.call(null,a,b,c,d);e=Ru._;if(null!=e)return e.N?e.N(a,b,c,d):e.call(null,a,b,c,d);throw Bt("ISwap.-swap!",a);};
Ru.ca=function(a,b,c,d,e){if(null!=a&&null!=a.Rj)return a.Rj(a,b,c,d,e);var g=Ru[n(null==a?null:a)];if(null!=g)return g.ca?g.ca(a,b,c,d,e):g.call(null,a,b,c,d,e);g=Ru._;if(null!=g)return g.ca?g.ca(a,b,c,d,e):g.call(null,a,b,c,d,e);throw Bt("ISwap.-swap!",a);};Ru.L=5;var Su=function Su(b){if(null!=b&&null!=b.lc)return b.lc(b);var c=Su[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=Su._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IIterable.-iterator",b);};
function Tu(a){this.Zw=a;this.C=1073741824;this.Z=0}Tu.prototype.kr=function(a,b){return this.Zw.append(b)};function Uu(a){var b=new Xs;a.ja(null,new Tu(b),st());return""+C.c(b)}var Vu="undefined"!==typeof Math.imul&&0!==Math.imul(4294967295,5)?function(a,b){return Math.imul(a,b)}:function(a,b){var c=a&65535,d=b&65535;return c*d+((a>>>16&65535)*d+c*(b>>>16&65535)<<16>>>0)|0};function Wu(a){a=Vu(a|0,-862048943);return Vu(a<<15|a>>>-15,461845907)}
function Xu(a,b){var c=(a|0)^(b|0);return Vu(c<<13|c>>>-13,5)+-430675100|0}function Yu(a,b){var c=(a|0)^b,c=Vu(c^c>>>16,-2048144789),c=Vu(c^c>>>13,-1028477387);return c^c>>>16}function Zu(a){a:{var b=1;for(var c=0;;)if(b<a.length){var d=b+2,c=Xu(c,Wu(a.charCodeAt(b-1)|a.charCodeAt(b)<<16));b=d}else{b=c;break a}}b=1===(a.length&1)?b^Wu(a.charCodeAt(a.length-1)):b;return Yu(b,Vu(2,a.length))}var $u={},av=0;
function bv(a){255<av&&($u={},av=0);if(null==a)return 0;var b=$u[a];if("number"!==typeof b){a:if(null!=a)if(b=a.length,0<b)for(var c=0,d=0;;)if(c<b)var e=c+1,d=Vu(31,d)+a.charCodeAt(c),c=e;else{b=d;break a}else b=0;else b=0;$u[a]=b;av+=1}return a=b}
function cv(a){if(null!=a&&(a.C&4194304||x===a.iv))return a.ha(null)^0;if("number"===typeof a){if(z(isFinite(a)))return Math.floor(a)%2147483647;switch(a){case Infinity:return 2146435072;case -Infinity:return-1048576;default:return 2146959360}}else return!0===a?a=1231:!1===a?a=1237:"string"===typeof a?(a=bv(a),0!==a&&(a=Wu(a),a=Xu(0,a),a=Yu(a,4))):a=a instanceof Date?a.valueOf()^0:null==a?0:wu(a)^0,a}function dv(a,b){return a^b+2654435769+(a<<6)+(a>>2)}function ev(a,b){return b instanceof a}
function fv(a,b,c,d,e){this.ns=a;this.name=b;this.Md=c;this.Fg=d;this.gc=e;this.C=2154168321;this.Z=4096}f=fv.prototype;f.toString=function(){return this.Md};f.equiv=function(a){return this.P(null,a)};f.P=function(a,b){return b instanceof fv?this.Md===b.Md:!1};
f.call=function(){function a(a,b,c){return F.j?F.j(b,this,c):F.call(null,b,this,c)}function b(a,b){return F.h?F.h(b,this):F.call(null,b,this)}var c=null,c=function(c,e,g){switch(arguments.length){case 2:return b.call(this,0,e);case 3:return a.call(this,0,e,g)}throw Error("Invalid arity: "+(arguments.length-1));};c.h=b;c.j=a;return c}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return F.h?F.h(a,this):F.call(null,a,this)};
f.h=function(a,b){return F.j?F.j(a,this,b):F.call(null,a,this,b)};f.sa=function(){return this.gc};f.za=function(a,b){return new fv(this.ns,this.name,this.Md,this.Fg,b)};f.ha=function(){var a=this.Fg;return null!=a?a:this.Fg=a=dv(Zu(this.name),bv(this.ns))};f.ja=function(a,b){return Eu(b,this.Md)};
var gv=function gv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return gv.c(arguments[0]);case 2:return gv.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};gv.c=function(a){if(a instanceof fv)return a;var b=a.indexOf("/");return 1>b?gv.h(null,a):gv.h(a.substring(0,b),a.substring(b+1,a.length))};
gv.h=function(a,b){var c=null!=a?[C.c(a),C.c("/"),C.c(b)].join(""):b;return new fv(a,b,c,null,null)};gv.L=2;function G(a){if(null==a)return null;if(null!=a&&(a.C&8388608||x===a.nv))return a.oa(null);if(zt(a)||"string"===typeof a)return 0===a.length?null:new H(a,0,null);if(B(xu,a))return yu(a);throw Error([C.c(a),C.c(" is not ISeqable")].join(""));}function I(a){if(null==a)return null;if(null!=a&&(a.C&64||x===a.Sb))return a.hb(null);a=G(a);return null==a?null:St(a)}
function hv(a){return null!=a?null!=a&&(a.C&64||x===a.Sb)?a.Db(null):(a=G(a))?Tt(a):iv:iv}function K(a){return null==a?null:null!=a&&(a.C&128||x===a.Nj)?a.bc(null):G(hv(a))}var L=function L(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return L.c(arguments[0]);case 2:return L.h(arguments[0],arguments[1]);default:return L.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};L.c=function(){return!0};
L.h=function(a,b){return null==a?null==b:a===b||vu(a,b)};L.w=function(a,b,c){for(;;)if(L.h(a,b))if(K(c))a=b,b=I(c),c=K(c);else return L.h(b,I(c));else return!1};L.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return L.w(b,a,c)};L.L=2;function jv(a){this.s=a}jv.prototype.next=function(){if(null!=this.s){var a=I(this.s);this.s=K(this.s);return{value:a,done:!1}}return{value:null,done:!0}};function kv(a){return new jv(G(a))}function lv(a,b){var c=Wu(a),c=Xu(0,c);return Yu(c,b)}
function mv(a){var b=0,c=1;for(a=G(a);;)if(null!=a)b+=1,c=Vu(31,c)+cv(I(a))|0,a=K(a);else return lv(c,b)}var nv=lv(1,0);function ov(a){var b=0,c=0;for(a=G(a);;)if(null!=a)b+=1,c=c+cv(I(a))|0,a=K(a);else return lv(c,b)}var pv=lv(0,0);Kt["null"]=!0;Lt["null"]=function(){return 0};Date.prototype.P=function(a,b){return b instanceof Date&&this.valueOf()===b.valueOf()};vu.number=function(a,b){return a===b};It["function"]=!0;ou["function"]=!0;pu["function"]=function(){return null};wu._=function(a){return pa(a)};
function qv(a){return a+1}function N(a){return nu(a)}function rv(a,b){var c=Lt(a);if(0===c)return b.T?b.T():b.call(null);for(var d=Qt.h(a,0),e=1;;)if(e<c)var g=Qt.h(a,e),d=b.h?b.h(d,g):b.call(null,d,g),e=e+1;else return d}function sv(a,b,c){var d=Lt(a),e=c;for(c=0;;)if(c<d){var g=Qt.h(a,c),e=b.h?b.h(e,g):b.call(null,e,g);c+=1}else return e}
function tv(a,b){var c=a.length;if(0===a.length)return b.T?b.T():b.call(null);for(var d=a[0],e=1;;)if(e<c)var g=a[e],d=b.h?b.h(d,g):b.call(null,d,g),e=e+1;else return d}function uv(a,b,c){var d=a.length,e=c;for(c=0;;)if(c<d){var g=a[c],e=b.h?b.h(e,g):b.call(null,e,g);c+=1}else return e}function vv(a,b,c,d){for(var e=a.length;;)if(d<e){var g=a[d];c=b.h?b.h(c,g):b.call(null,c,g);d+=1}else return c}function wv(a){return null!=a?a.C&2||x===a.cv?!0:a.C?!1:B(Kt,a):B(Kt,a)}
function xv(a){return null!=a?a.C&16||x===a.gr?!0:a.C?!1:B(Pt,a):B(Pt,a)}function P(a,b,c){var d=Q.c?Q.c(a):Q.call(null,a);if(c>=d)return-1;!(0<c)&&0>c&&(c+=d,c=0>c?0:c);for(;;)if(c<d){if(L.h(yv?yv(a,c):zv.call(null,a,c),b))return c;c+=1}else return-1}function Av(a,b,c){var d=Q.c?Q.c(a):Q.call(null,a);if(0===d)return-1;0<c?(--d,c=d<c?d:c):c=0>c?d+c:c;for(;;)if(0<=c){if(L.h(yv?yv(a,c):zv.call(null,a,c),b))return c;--c}else return-1}function Bv(a,b){this.v=a;this.i=b}
Bv.prototype.mb=function(){return this.i<this.v.length};Bv.prototype.next=function(){var a=this.v[this.i];this.i+=1;return a};function H(a,b,c){this.v=a;this.i=b;this.meta=c;this.C=166592766;this.Z=8192}f=H.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q.c?Q.c(this):Q.call(null,this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.aa=function(a,b){var c=b+this.i;if(0<=c&&c<this.v.length)return this.v[c];throw Error("Index out of bounds");};f.ac=function(a,b,c){a=b+this.i;return 0<=a&&a<this.v.length?this.v[a]:c};
f.lc=function(){return new Bv(this.v,this.i)};f.sa=function(){return this.meta};f.bc=function(){return this.i+1<this.v.length?new H(this.v,this.i+1,null):null};f.ya=function(){var a=this.v.length-this.i;return 0>a?0:a};f.Lg=function(){var a=this.ya(null);return 0<a?new Cv(this,a-1,null):null};f.ha=function(){return mv(this)};f.P=function(a,b){return Dv.h?Dv.h(this,b):Dv.call(null,this,b)};f.Ka=function(){return iv};f.Bb=function(a,b){return vv(this.v,b,this.v[this.i],this.i+1)};
f.Cb=function(a,b,c){return vv(this.v,b,c,this.i)};f.hb=function(){return this.v[this.i]};f.Db=function(){return this.i+1<this.v.length?new H(this.v,this.i+1,null):iv};f.oa=function(){return this.i<this.v.length?this:null};f.za=function(a,b){return new H(this.v,this.i,b)};f.xa=function(a,b){return Ev.h?Ev.h(b,this):Ev.call(null,b,this)};H.prototype[Dt]=function(){return kv(this)};function Fv(a,b){return b<a.length?new H(a,b,null):null}
function R(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return Fv(arguments[0],0);case 2:return Fv(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}function Cv(a,b,c){this.Fj=a;this.i=b;this.meta=c;this.C=32374990;this.Z=8192}f=Cv.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q.c?Q.c(this):Q.call(null,this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){return 0<this.i?new Cv(this.Fj,this.i-1,null):null};f.ya=function(){return this.i+1};f.ha=function(){return mv(this)};
f.P=function(a,b){return Dv.h?Dv.h(this,b):Dv.call(null,this,b)};f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return Gv?Gv(b,this):Hv.call(null,b,this)};f.Cb=function(a,b,c){return Iv?Iv(b,c,this):Hv.call(null,b,c,this)};f.hb=function(){return Qt.h(this.Fj,this.i)};f.Db=function(){return 0<this.i?new Cv(this.Fj,this.i-1,null):iv};f.oa=function(){return this};f.za=function(a,b){return new Cv(this.Fj,this.i,b)};f.xa=function(a,b){return Ev.h?Ev.h(b,this):Ev.call(null,b,this)};
Cv.prototype[Dt]=function(){return kv(this)};function Jv(a){return I(K(a))}function Kv(a){for(;;){var b=K(a);if(null!=b)a=b;else return I(a)}}vu._=function(a,b){return a===b};var Lv=function Lv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Lv.T();case 1:return Lv.c(arguments[0]);case 2:return Lv.h(arguments[0],arguments[1]);default:return Lv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Lv.T=function(){return Mv};
Lv.c=function(a){return a};Lv.h=function(a,b){return null!=a?Ot(a,b):Ot(iv,b)};Lv.w=function(a,b,c){for(;;)if(z(c))a=Lv.h(a,b),b=I(c),c=K(c);else return Lv.h(a,b)};Lv.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Lv.w(b,a,c)};Lv.L=2;
function Q(a){if(null!=a)if(null!=a&&(a.C&2||x===a.cv))a=a.ya(null);else if(zt(a))a=a.length;else if("string"===typeof a)a=a.length;else if(null!=a&&(a.C&8388608||x===a.nv))a:{a=G(a);for(var b=0;;){if(wv(a)){a=b+Lt(a);break a}a=K(a);b+=1}}else a=Lt(a);else a=0;return a}function Nv(a,b,c){for(;;){if(null==a)return c;if(0===b)return G(a)?I(a):c;if(xv(a))return Qt.j(a,b,c);if(G(a))a=K(a),--b;else return c}}
function zv(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 2:return yv(arguments[0],arguments[1]);case 3:return S(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}
function yv(a,b){if("number"!==typeof b)throw Error("Index argument to nth must be a number");if(null==a)return a;if(null!=a&&(a.C&16||x===a.gr))return a.aa(null,b);if(zt(a)){if(0<=b&&b<a.length)return a[b];throw Error("Index out of bounds");}if("string"===typeof a){if(0<=b&&b<a.length)return a.charAt(b);throw Error("Index out of bounds");}if(null!=a&&(a.C&64||x===a.Sb)){a:{var c=a;for(var d=b;;){if(null==c)throw Error("Index out of bounds");if(0===d){if(G(c)){c=I(c);break a}throw Error("Index out of bounds");
}if(xv(c)){c=Qt.h(c,d);break a}if(G(c))c=K(c),--d;else throw Error("Index out of bounds");}}return c}if(B(Pt,a))return Qt.h(a,b);throw Error([C.c("nth not supported on this type "),C.c(Ct(null==a?null:a.constructor))].join(""));}
function S(a,b,c){if("number"!==typeof b)throw Error("Index argument to nth must be a number.");if(null==a)return c;if(null!=a&&(a.C&16||x===a.gr))return a.ac(null,b,c);if(zt(a))return 0<=b&&b<a.length?a[b]:c;if("string"===typeof a)return 0<=b&&b<a.length?a.charAt(b):c;if(null!=a&&(a.C&64||x===a.Sb))return Nv(a,b,c);if(B(Pt,a))return Qt.h(a,b);throw Error([C.c("nth not supported on this type "),C.c(Ct(null==a?null:a.constructor))].join(""));}
var F=function F(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return F.h(arguments[0],arguments[1]);case 3:return F.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};F.h=function(a,b){return null==a?null:null!=a&&(a.C&256||x===a.jv)?a.na(null,b):zt(a)?null!=b&&b<a.length?a[b|0]:null:"string"===typeof a?null!=b&&b<a.length?a.charAt(b|0):null:B(Vt,a)?Wt.h(a,b):null};
F.j=function(a,b,c){return null!=a?null!=a&&(a.C&256||x===a.jv)?a.Y(null,b,c):zt(a)?null!=b&&0<=b&&b<a.length?a[b|0]:c:"string"===typeof a?null!=b&&0<=b&&b<a.length?a.charAt(b|0):c:B(Vt,a)?Wt.j(a,b,c):c:c};F.L=3;var T=function T(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 3:return T.j(arguments[0],arguments[1],arguments[2]);default:return T.w(arguments[0],arguments[1],arguments[2],new H(c.slice(3),0,null))}};
T.j=function(a,b,c){return null!=a?Zt(a,b,c):Ov([b,c])};T.w=function(a,b,c,d){for(;;)if(a=T.j(a,b,c),z(d))b=I(d),c=Jv(d),d=K(K(d));else return a};T.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),d=K(d);return T.w(b,a,c,d)};T.L=3;
var Pv=function Pv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Pv.c(arguments[0]);case 2:return Pv.h(arguments[0],arguments[1]);default:return Pv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Pv.c=function(a){return a};Pv.h=function(a,b){return null==a?null:cu(a,b)};Pv.w=function(a,b,c){for(;;){if(null==a)return null;a=Pv.h(a,b);if(z(c))b=I(c),c=K(c);else return a}};
Pv.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Pv.w(b,a,c)};Pv.L=2;function Qv(a){var b=na(a);return b?b:null!=a?x===a.bv?!0:a.Tj?!1:B(It,a):B(It,a)}function Rv(a,b){this.B=a;this.meta=b;this.C=393217;this.Z=0}f=Rv.prototype;f.sa=function(){return this.meta};f.za=function(a,b){return new Rv(this.B,b)};f.bv=x;
f.call=function(){function a(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W,ra,Za){a=this;return U.Mj?U.Mj(a.B,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W,ra,Za):U.call(null,a.B,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W,ra,Za)}function b(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W,ra){a=this;return a.B.ce?a.B.ce(b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W,ra):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W,ra)}function c(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W){a=this;return a.B.be?a.B.be(b,c,d,e,g,h,k,l,m,p,u,A,
D,E,M,J,O,aa,W):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa,W)}function d(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa){a=this;return a.B.ae?a.B.ae(b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O,aa)}function e(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O){a=this;return a.B.$d?a.B.$d(b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J,O)}function g(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J){a=this;return a.B.Zd?a.B.Zd(b,c,d,e,g,h,k,l,m,p,u,A,D,E,
M,J):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M,J)}function h(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M){a=this;return a.B.Yd?a.B.Yd(b,c,d,e,g,h,k,l,m,p,u,A,D,E,M):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E,M)}function k(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E){a=this;return a.B.Xd?a.B.Xd(b,c,d,e,g,h,k,l,m,p,u,A,D,E):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D,E)}function l(a,b,c,d,e,g,h,k,l,m,p,u,A,D){a=this;return a.B.Wd?a.B.Wd(b,c,d,e,g,h,k,l,m,p,u,A,D):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A,D)}function m(a,b,c,d,e,g,
h,k,l,m,p,u,A){a=this;return a.B.Vd?a.B.Vd(b,c,d,e,g,h,k,l,m,p,u,A):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u,A)}function p(a,b,c,d,e,g,h,k,l,m,p,u){a=this;return a.B.Ud?a.B.Ud(b,c,d,e,g,h,k,l,m,p,u):a.B.call(null,b,c,d,e,g,h,k,l,m,p,u)}function u(a,b,c,d,e,g,h,k,l,m,p){a=this;return a.B.Td?a.B.Td(b,c,d,e,g,h,k,l,m,p):a.B.call(null,b,c,d,e,g,h,k,l,m,p)}function A(a,b,c,d,e,g,h,k,l,m){a=this;return a.B.fe?a.B.fe(b,c,d,e,g,h,k,l,m):a.B.call(null,b,c,d,e,g,h,k,l,m)}function D(a,b,c,d,e,g,h,k,l){a=this;return a.B.ee?
a.B.ee(b,c,d,e,g,h,k,l):a.B.call(null,b,c,d,e,g,h,k,l)}function E(a,b,c,d,e,g,h,k){a=this;return a.B.de?a.B.de(b,c,d,e,g,h,k):a.B.call(null,b,c,d,e,g,h,k)}function J(a,b,c,d,e,g,h){a=this;return a.B.Dc?a.B.Dc(b,c,d,e,g,h):a.B.call(null,b,c,d,e,g,h)}function M(a,b,c,d,e,g){a=this;return a.B.ca?a.B.ca(b,c,d,e,g):a.B.call(null,b,c,d,e,g)}function O(a,b,c,d,e){a=this;return a.B.N?a.B.N(b,c,d,e):a.B.call(null,b,c,d,e)}function aa(a,b,c,d){a=this;return a.B.j?a.B.j(b,c,d):a.B.call(null,b,c,d)}function ra(a,
b,c){a=this;return a.B.h?a.B.h(b,c):a.B.call(null,b,c)}function Za(a,b){a=this;return a.B.c?a.B.c(b):a.B.call(null,b)}function Vc(a){a=this;return a.B.T?a.B.T():a.B.call(null)}var W=null,W=function(W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf,og,$h,pg,Sl,Tl,Ul){switch(arguments.length){case 1:return Vc.call(this,W);case 2:return Za.call(this,W,gb);case 3:return ra.call(this,W,gb,tb);case 4:return aa.call(this,W,gb,tb,zb);case 5:return O.call(this,W,gb,tb,zb,Ab);case 6:return M.call(this,W,gb,tb,
zb,Ab,Ib);case 7:return J.call(this,W,gb,tb,zb,Ab,Ib,Wb);case 8:return E.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb);case 9:return D.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc);case 10:return A.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic);case 11:return u.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc);case 12:return p.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld);case 13:return m.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld);case 14:return l.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he);case 15:return k.call(this,
W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je);case 16:return h.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf);case 17:return g.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf,og);case 18:return e.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf,og,$h);case 19:return d.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf,og,$h,pg);case 20:return c.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf,og,$h,pg,Sl);case 21:return b.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,
Ic,Wc,ld,Ld,he,Je,pf,og,$h,pg,Sl,Tl);case 22:return a.call(this,W,gb,tb,zb,Ab,Ib,Wb,Xb,pc,Ic,Wc,ld,Ld,he,Je,pf,og,$h,pg,Sl,Tl,Ul)}throw Error("Invalid arity: "+(arguments.length-1));};W.c=Vc;W.h=Za;W.j=ra;W.N=aa;W.ca=O;W.Dc=M;W.de=J;W.ee=E;W.fe=D;W.Td=A;W.Ud=u;W.Vd=p;W.Wd=m;W.Xd=l;W.Yd=k;W.Zd=h;W.$d=g;W.ae=e;W.be=d;W.ce=c;W.hv=b;W.Mj=a;return W}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.T=function(){return this.B.T?this.B.T():this.B.call(null)};
f.c=function(a){return this.B.c?this.B.c(a):this.B.call(null,a)};f.h=function(a,b){return this.B.h?this.B.h(a,b):this.B.call(null,a,b)};f.j=function(a,b,c){return this.B.j?this.B.j(a,b,c):this.B.call(null,a,b,c)};f.N=function(a,b,c,d){return this.B.N?this.B.N(a,b,c,d):this.B.call(null,a,b,c,d)};f.ca=function(a,b,c,d,e){return this.B.ca?this.B.ca(a,b,c,d,e):this.B.call(null,a,b,c,d,e)};f.Dc=function(a,b,c,d,e,g){return this.B.Dc?this.B.Dc(a,b,c,d,e,g):this.B.call(null,a,b,c,d,e,g)};
f.de=function(a,b,c,d,e,g,h){return this.B.de?this.B.de(a,b,c,d,e,g,h):this.B.call(null,a,b,c,d,e,g,h)};f.ee=function(a,b,c,d,e,g,h,k){return this.B.ee?this.B.ee(a,b,c,d,e,g,h,k):this.B.call(null,a,b,c,d,e,g,h,k)};f.fe=function(a,b,c,d,e,g,h,k,l){return this.B.fe?this.B.fe(a,b,c,d,e,g,h,k,l):this.B.call(null,a,b,c,d,e,g,h,k,l)};f.Td=function(a,b,c,d,e,g,h,k,l,m){return this.B.Td?this.B.Td(a,b,c,d,e,g,h,k,l,m):this.B.call(null,a,b,c,d,e,g,h,k,l,m)};
f.Ud=function(a,b,c,d,e,g,h,k,l,m,p){return this.B.Ud?this.B.Ud(a,b,c,d,e,g,h,k,l,m,p):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p)};f.Vd=function(a,b,c,d,e,g,h,k,l,m,p,u){return this.B.Vd?this.B.Vd(a,b,c,d,e,g,h,k,l,m,p,u):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u)};f.Wd=function(a,b,c,d,e,g,h,k,l,m,p,u,A){return this.B.Wd?this.B.Wd(a,b,c,d,e,g,h,k,l,m,p,u,A):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A)};
f.Xd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D){return this.B.Xd?this.B.Xd(a,b,c,d,e,g,h,k,l,m,p,u,A,D):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D)};f.Yd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E){return this.B.Yd?this.B.Yd(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E)};f.Zd=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J){return this.B.Zd?this.B.Zd(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J)};
f.$d=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M){return this.B.$d?this.B.$d(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M)};f.ae=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O){return this.B.ae?this.B.ae(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O)};
f.be=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa){return this.B.be?this.B.be(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa)};f.ce=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra){return this.B.ce?this.B.ce(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra):this.B.call(null,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra)};
f.hv=function(a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za){return U.Mj?U.Mj(this.B,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za):U.call(null,this.B,a,b,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za)};function Sv(a,b){return na(a)?new Rv(a,b):null==a?null:ru(a,b)}function Tv(a){var b=null!=a;return(b?null!=a?a.C&131072||x===a.lv||(a.C?0:B(ou,a)):B(ou,a):b)?pu(a):null}function Uv(a){return null==a?null:iu(a)}
var Vv=function Vv(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Vv.c(arguments[0]);case 2:return Vv.h(arguments[0],arguments[1]);default:return Vv.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Vv.c=function(a){return a};Vv.h=function(a,b){return null==a?null:hu(a,b)};Vv.w=function(a,b,c){for(;;){if(null==a)return null;a=Vv.h(a,b);if(z(c))b=I(c),c=K(c);else return a}};
Vv.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Vv.w(b,a,c)};Vv.L=2;function Wv(a){return null==a||At(G(a))}function Xv(a){return null==a?!1:null!=a?a.C&8||x===a.GA?!0:a.C?!1:B(Nt,a):B(Nt,a)}function Yv(a){return null==a?!1:null!=a?a.C&4096||x===a.pv?!0:a.C?!1:B(gu,a):B(gu,a)}function Zv(a){return null!=a?a.C&16777216||x===a.ov?!0:a.C?!1:B(zu,a):B(zu,a)}function $v(a){return null==a?!1:null!=a?a.C&1024||x===a.ir?!0:a.C?!1:B(bu,a):B(bu,a)}
function aw(a){return null!=a?a.C&67108864||x===a.KA?!0:a.C?!1:B(Bu,a):B(Bu,a)}function bw(a){return null!=a?a.C&16384||x===a.MA?!0:a.C?!1:B(ku,a):B(ku,a)}function cw(a){return null!=a?a.Z&512||x===a.FA?!0:!1:!1}function dw(a){var b=[];Db(a,function(a,b){return function(a,c){return b.push(c)}}(a,b));return b}function ew(a,b,c,d,e){for(;0!==e;)c[d]=a[b],d+=1,--e,b+=1}var fw={};function gw(a){return!0===a}function hw(a){return null==a?!1:null!=a?a.C&64||x===a.Sb?!0:a.C?!1:B(Rt,a):B(Rt,a)}
function iw(a){return null==a?!1:!1===a?!1:!0}function jw(a){var b=Qv(a);return b?b:null!=a?a.C&1||x===a.IA?!0:a.C?!1:B(Jt,a):B(Jt,a)}function kw(a){return"number"===typeof a&&!isNaN(a)&&Infinity!==a&&parseFloat(a)===parseInt(a,10)}function lw(a,b){return F.j(a,b,fw)===fw?!1:!0}function mw(a,b){var c;if(c=null!=a)c=null!=a?a.C&512||x===a.EA?!0:a.C?!1:B(Xt,a):B(Xt,a);return c&&lw(a,b)?(null!=a?x===a.Lj||(a.Tj?0:B($t,a)):B($t,a))?au(a,b):new V(null,2,5,X,[b,F.h(a,b)],null):null}
function Hv(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 2:return Gv(arguments[0],arguments[1]);case 3:return Iv(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}function Gv(a,b){var c=G(b);if(c){var d=I(c),c=K(c);return Gt?Gt(a,d,c):Ht.call(null,a,d,c)}return a.T?a.T():a.call(null)}
function Iv(a,b,c){for(c=G(c);;)if(c){var d=I(c);b=a.h?a.h(b,d):a.call(null,b,d);c=K(c)}else return b}function Ht(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 2:return nw(arguments[0],arguments[1]);case 3:return Gt(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}
function nw(a,b){return null!=b&&(b.C&524288||x===b.mv)?b.Bb(null,a):zt(b)?tv(b,a):"string"===typeof b?tv(b,a):B(su,b)?tu.h(b,a):Gv(a,b)}function Gt(a,b,c){return null!=c&&(c.C&524288||x===c.mv)?c.Cb(null,a,b):zt(c)?uv(c,a,b):"string"===typeof c?uv(c,a,b):B(su,c)?tu.j(c,a,b):Iv(a,b,c)}function ow(a,b,c){return null!=c?uu(c,a,b):b}function pw(a){return a}function qw(a,b,c,d){a=a.c?a.c(b):a.call(null,b);c=Gt(a,c,d);return a.c?a.c(c):a.call(null,c)}
var rw=function rw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return rw.T();case 1:return rw.c(arguments[0]);case 2:return rw.h(arguments[0],arguments[1]);default:return rw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};rw.T=function(){return 0};rw.c=function(a){return a};rw.h=function(a,b){return a+b};rw.w=function(a,b,c){return Gt(rw,a+b,c)};rw.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return rw.w(b,a,c)};rw.L=2;
var sw=function sw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return sw.c(arguments[0]);case 2:return sw.h(arguments[0],arguments[1]);default:return sw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};sw.c=function(a){return-a};sw.h=function(a,b){return a-b};sw.w=function(a,b,c){return Gt(sw,a-b,c)};sw.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return sw.w(b,a,c)};sw.L=2;function tw(a){return a-1}
var uw=function uw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return uw.c(arguments[0]);case 2:return uw.h(arguments[0],arguments[1]);default:return uw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};uw.c=function(a){return a};uw.h=function(a,b){return a>b?a:b};uw.w=function(a,b,c){return Gt(uw,a>b?a:b,c)};uw.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return uw.w(b,a,c)};uw.L=2;function vw(a,b){return(a%b+b)%b}
function ww(a){a=(a-a%2)/2;return 0<=a?Math.floor(a):Math.ceil(a)}function xw(a){a-=a>>1&1431655765;a=(a&858993459)+(a>>2&858993459);return 16843009*(a+(a>>4)&252645135)>>24}var C=function C(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return C.T();case 1:return C.c(arguments[0]);default:return C.w(arguments[0],new H(c.slice(1),0,null))}};C.T=function(){return""};C.c=function(a){return null==a?"":""+a};
C.w=function(a,b){for(var c=new Xs(""+C.c(a)),d=b;;)if(z(d))c=c.append(""+C.c(I(d))),d=K(d);else return c.toString()};C.O=function(a){var b=I(a);a=K(a);return C.w(b,a)};C.L=1;function yw(a,b,c){return a.substring(b,c)}function Dv(a,b){if(Zv(b))if(wv(a)&&wv(b)&&Q(a)!==Q(b))var c=!1;else a:{c=G(a);for(var d=G(b);;){if(null==c){c=null==d;break a}if(null!=d&&L.h(I(c),I(d)))c=K(c),d=K(d);else{c=!1;break a}}}else c=null;return iw(c)}
function zw(a){var b=0;for(a=G(a);;)if(a){var c=I(a),b=(b+(cv(Aw.c?Aw.c(c):Aw.call(null,c))^cv(Bw.c?Bw.c(c):Bw.call(null,c))))%4503599627370496;a=K(a)}else return b}function Cw(a,b,c,d,e){this.meta=a;this.first=b;this.Ee=c;this.count=d;this.M=e;this.C=65937646;this.Z=8192}f=Cw.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,this.count)}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){return 1===this.count?null:this.Ee};f.ya=function(){return this.count};f.Mg=function(){return this.first};f.Ng=function(){return this.Db(null)};
f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return this.first};f.Db=function(){return 1===this.count?iv:this.Ee};f.oa=function(){return this};f.za=function(a,b){return new Cw(b,this.first,this.Ee,this.count,this.M)};f.xa=function(a,b){return new Cw(this.meta,b,this,this.count+1,null)};
function Dw(a){return null!=a?a.C&33554432||x===a.JA?!0:a.C?!1:B(Au,a):B(Au,a)}Cw.prototype[Dt]=function(){return kv(this)};function Ew(a){this.meta=a;this.C=65937614;this.Z=8192}f=Ew.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){return null};f.ya=function(){return 0};f.Mg=function(){return null};f.Ng=function(){throw Error("Can't pop empty list");};f.ha=function(){return nv};
f.P=function(a,b){return Dw(b)||Zv(b)?null==G(b):!1};f.Ka=function(){return this};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return null};f.Db=function(){return iv};f.oa=function(){return null};f.za=function(a,b){return new Ew(b)};f.xa=function(a,b){return new Cw(this.meta,b,null,1,null)};var iv=new Ew(null);Ew.prototype[Dt]=function(){return kv(this)};
function Fw(a){return(null!=a?a.C&134217728||x===a.LA||(a.C?0:B(Cu,a)):B(Cu,a))?Du(a):Gt(Lv,iv,a)}var Gw=function Gw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return Gw.w(0<c.length?new H(c.slice(0),0,null):null)};Gw.w=function(a){if(a instanceof H&&0===a.i)var b=a.v;else a:for(b=[];;)if(null!=a)b.push(a.hb(null)),a=a.bc(null);else break a;a=b.length;for(var c=iv;;)if(0<a){var d=a-1,c=c.xa(null,b[a-1]);a=d}else return c};Gw.L=0;Gw.O=function(a){return Gw.w(G(a))};
function Hw(a,b,c,d){this.meta=a;this.first=b;this.Ee=c;this.M=d;this.C=65929452;this.Z=8192}f=Hw.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){return null==this.Ee?null:G(this.Ee)};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};
f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return this.first};f.Db=function(){return null==this.Ee?iv:this.Ee};f.oa=function(){return this};f.za=function(a,b){return new Hw(b,this.first,this.Ee,this.M)};f.xa=function(a,b){return new Hw(null,b,this,null)};Hw.prototype[Dt]=function(){return kv(this)};
function Ev(a,b){return null==b||null!=b&&(b.C&64||x===b.Sb)?new Hw(null,a,b,null):new Hw(null,a,G(b),null)}function Y(a,b,c,d){this.ns=a;this.name=b;this.Vb=c;this.Fg=d;this.C=2153775105;this.Z=4096}f=Y.prototype;f.toString=function(){return[C.c(":"),C.c(this.Vb)].join("")};f.equiv=function(a){return this.P(null,a)};f.P=function(a,b){return b instanceof Y?this.Vb===b.Vb:!1};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return F.h(c,this);case 3:return F.j(c,this,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return F.h(c,this)};a.j=function(a,c,d){return F.j(c,this,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return F.h(a,this)};f.h=function(a,b){return F.j(a,this,b)};
f.ha=function(){var a=this.Fg;return null!=a?a:this.Fg=a=dv(Zu(this.name),bv(this.ns))+2654435769|0};f.ja=function(a,b){return Eu(b,[C.c(":"),C.c(this.Vb)].join(""))};function Iw(a,b){return a===b?!0:a instanceof Y&&b instanceof Y?a.Vb===b.Vb:!1}
var Jw=function Jw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Jw.c(arguments[0]);case 2:return Jw.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};
Jw.c=function(a){if(a instanceof Y)return a;if(a instanceof fv){if(null!=a&&(a.Z&4096||x===a.jr))var b=a.ns;else throw Error([C.c("Doesn't support namespace: "),C.c(a)].join(""));return new Y(b,Kw.c?Kw.c(a):Kw.call(null,a),a.Md,null)}return"string"===typeof a?(b=a.split("/"),2===b.length?new Y(b[0],b[1],a,null):new Y(null,b[0],a,null)):null};
Jw.h=function(a,b){var c=a instanceof Y?Kw.c?Kw.c(a):Kw.call(null,a):a instanceof fv?Kw.c?Kw.c(a):Kw.call(null,a):a,d=b instanceof Y?Kw.c?Kw.c(b):Kw.call(null,b):b instanceof fv?Kw.c?Kw.c(b):Kw.call(null,b):b;return new Y(c,d,[C.c(z(c)?[C.c(c),C.c("/")].join(""):null),C.c(d)].join(""),null)};Jw.L=2;function Lw(a,b,c,d){this.meta=a;this.fn=b;this.s=c;this.M=d;this.C=32374988;this.Z=1}f=Lw.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
function Mw(a){null!=a.fn&&(a.s=a.fn.T?a.fn.T():a.fn.call(null),a.fn=null);return a.s}f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){this.oa(null);return null==this.s?null:K(this.s)};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};
f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){this.oa(null);return null==this.s?null:I(this.s)};f.Db=function(){this.oa(null);return null!=this.s?hv(this.s):iv};f.oa=function(){Mw(this);if(null==this.s)return null;for(var a=this.s;;)if(a instanceof Lw)a=Mw(a);else return this.s=a,G(this.s)};f.za=function(a,b){return new Lw(b,this.fn,this.s,this.M)};f.xa=function(a,b){return Ev(b,this)};Lw.prototype[Dt]=function(){return kv(this)};
function Nw(a,b){this.zm=a;this.end=b;this.C=2;this.Z=0}Nw.prototype.add=function(a){this.zm[this.end]=a;return this.end+=1};Nw.prototype.sb=function(){var a=new Ow(this.zm,0,this.end);this.zm=null;return a};Nw.prototype.ya=function(){return this.end};function Pw(a){return new Nw(Array(a),0)}function Ow(a,b,c){this.v=a;this.wb=b;this.end=c;this.C=524306;this.Z=0}f=Ow.prototype;f.ya=function(){return this.end-this.wb};f.aa=function(a,b){return this.v[this.wb+b]};
f.ac=function(a,b,c){return 0<=b&&b<this.end-this.wb?this.v[this.wb+b]:c};f.dr=function(){if(this.wb===this.end)throw Error("-drop-first of empty chunk");return new Ow(this.v,this.wb+1,this.end)};f.Bb=function(a,b){return vv(this.v,b,this.v[this.wb],this.wb+1)};f.Cb=function(a,b,c){return vv(this.v,b,c,this.wb)};function Qw(a,b,c,d){this.sb=a;this.Jd=b;this.meta=c;this.M=d;this.C=31850732;this.Z=1536}f=Qw.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){if(1<Lt(this.sb))return new Qw(Nu(this.sb),this.Jd,this.meta,null);var a=yu(this.Jd);return null==a?null:a};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};
f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.meta)};f.hb=function(){return Qt.h(this.sb,0)};f.Db=function(){return 1<Lt(this.sb)?new Qw(Nu(this.sb),this.Jd,this.meta,null):null==this.Jd?iv:this.Jd};f.oa=function(){return this};f.Jm=function(){return this.sb};f.Kj=function(){return null==this.Jd?iv:this.Jd};f.za=function(a,b){return new Qw(this.sb,this.Jd,b,this.M)};f.xa=function(a,b){return Ev(b,this)};f.er=function(){return null==this.Jd?null:this.Jd};Qw.prototype[Dt]=function(){return kv(this)};
function Rw(a,b){return 0===Lt(a)?b:new Qw(a,b,null,null)}function Sw(a,b){a.add(b)}function Tw(a){for(var b=[];;)if(G(a))b.push(I(a)),a=K(a);else return b}function Uw(a,b){if(wv(b))return Q(b);for(var c=0,d=G(b);;)if(null!=d&&c<a)c+=1,d=K(d);else return c}
var Vw=function Vw(b){if(null==b)var c=null;else if(null==K(b))c=G(I(b));else{c=Ev;var d=I(b);b=K(b);b=Vw.c?Vw.c(b):Vw.call(null,b);c=c(d,b)}return c},Ww=function Ww(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Ww.T();case 1:return Ww.c(arguments[0]);case 2:return Ww.h(arguments[0],arguments[1]);default:return Ww.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};
Ww.T=function(){return new Lw(null,function(){return null},null,null)};Ww.c=function(a){return new Lw(null,function(){return a},null,null)};Ww.h=function(a,b){return new Lw(null,function(){var c=G(a);return c?cw(c)?Rw(Ou(c),Ww.h(Pu(c),b)):Ev(I(c),Ww.h(hv(c),b)):b},null,null)};Ww.w=function(a,b,c){return function e(a,b){return new Lw(null,function(){var c=G(a);return c?cw(c)?Rw(Ou(c),e(Pu(c),b)):Ev(I(c),e(hv(c),b)):z(b)?e(I(b),K(b)):null},null,null)}(Ww.h(a,b),c)};
Ww.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Ww.w(b,a,c)};Ww.L=2;var Xw=function Xw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return Xw.T();case 1:return Xw.c(arguments[0]);case 2:return Xw.h(arguments[0],arguments[1]);default:return Xw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Xw.T=function(){return Ju(Mv)};Xw.c=function(a){return a};Xw.h=function(a,b){return Ku(a,b)};
Xw.w=function(a,b,c){for(;;)if(a=Ku(a,b),z(c))b=I(c),c=K(c);else return a};Xw.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Xw.w(b,a,c)};Xw.L=2;
function Yw(a,b,c){var d=G(c);if(0===b)return a.T?a.T():a.call(null);c=St(d);var e=Tt(d);if(1===b)return a.c?a.c(c):a.c?a.c(c):a.call(null,c);var d=St(e),g=Tt(e);if(2===b)return a.h?a.h(c,d):a.h?a.h(c,d):a.call(null,c,d);var e=St(g),h=Tt(g);if(3===b)return a.j?a.j(c,d,e):a.j?a.j(c,d,e):a.call(null,c,d,e);var g=St(h),k=Tt(h);if(4===b)return a.N?a.N(c,d,e,g):a.N?a.N(c,d,e,g):a.call(null,c,d,e,g);var h=St(k),l=Tt(k);if(5===b)return a.ca?a.ca(c,d,e,g,h):a.ca?a.ca(c,d,e,g,h):a.call(null,c,d,e,g,h);var k=
St(l),m=Tt(l);if(6===b)return a.Dc?a.Dc(c,d,e,g,h,k):a.Dc?a.Dc(c,d,e,g,h,k):a.call(null,c,d,e,g,h,k);var l=St(m),p=Tt(m);if(7===b)return a.de?a.de(c,d,e,g,h,k,l):a.de?a.de(c,d,e,g,h,k,l):a.call(null,c,d,e,g,h,k,l);var m=St(p),u=Tt(p);if(8===b)return a.ee?a.ee(c,d,e,g,h,k,l,m):a.ee?a.ee(c,d,e,g,h,k,l,m):a.call(null,c,d,e,g,h,k,l,m);var p=St(u),A=Tt(u);if(9===b)return a.fe?a.fe(c,d,e,g,h,k,l,m,p):a.fe?a.fe(c,d,e,g,h,k,l,m,p):a.call(null,c,d,e,g,h,k,l,m,p);var u=St(A),D=Tt(A);if(10===b)return a.Td?a.Td(c,
d,e,g,h,k,l,m,p,u):a.Td?a.Td(c,d,e,g,h,k,l,m,p,u):a.call(null,c,d,e,g,h,k,l,m,p,u);var A=St(D),E=Tt(D);if(11===b)return a.Ud?a.Ud(c,d,e,g,h,k,l,m,p,u,A):a.Ud?a.Ud(c,d,e,g,h,k,l,m,p,u,A):a.call(null,c,d,e,g,h,k,l,m,p,u,A);var D=St(E),J=Tt(E);if(12===b)return a.Vd?a.Vd(c,d,e,g,h,k,l,m,p,u,A,D):a.Vd?a.Vd(c,d,e,g,h,k,l,m,p,u,A,D):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D);var E=St(J),M=Tt(J);if(13===b)return a.Wd?a.Wd(c,d,e,g,h,k,l,m,p,u,A,D,E):a.Wd?a.Wd(c,d,e,g,h,k,l,m,p,u,A,D,E):a.call(null,c,d,e,g,h,k,l,
m,p,u,A,D,E);var J=St(M),O=Tt(M);if(14===b)return a.Xd?a.Xd(c,d,e,g,h,k,l,m,p,u,A,D,E,J):a.Xd?a.Xd(c,d,e,g,h,k,l,m,p,u,A,D,E,J):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J);var M=St(O),aa=Tt(O);if(15===b)return a.Yd?a.Yd(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M):a.Yd?a.Yd(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M);var O=St(aa),ra=Tt(aa);if(16===b)return a.Zd?a.Zd(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O):a.Zd?a.Zd(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O);
var aa=St(ra),Za=Tt(ra);if(17===b)return a.$d?a.$d(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa):a.$d?a.$d(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa);var ra=St(Za),Vc=Tt(Za);if(18===b)return a.ae?a.ae(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra):a.ae?a.ae(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra);Za=St(Vc);Vc=Tt(Vc);if(19===b)return a.be?a.be(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za):a.be?a.be(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,
aa,ra,Za):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za);var W=St(Vc);Tt(Vc);if(20===b)return a.ce?a.ce(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za,W):a.ce?a.ce(c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za,W):a.call(null,c,d,e,g,h,k,l,m,p,u,A,D,E,J,M,O,aa,ra,Za,W);throw Error("Only up to 20 arguments supported on functions");}
var U=function U(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return U.h(arguments[0],arguments[1]);case 3:return U.j(arguments[0],arguments[1],arguments[2]);case 4:return U.N(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return U.ca(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);default:return U.w(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],new H(c.slice(5),0,null))}};
U.h=function(a,b){var c=a.L;if(a.O){var d=Uw(c+1,b);return d<=c?Yw(a,d,b):a.O(b)}return a.apply(a,Tw(b))};U.j=function(a,b,c){b=Ev(b,c);c=a.L;if(a.O){var d=Uw(c+1,b);return d<=c?Yw(a,d,b):a.O(b)}return a.apply(a,Tw(b))};U.N=function(a,b,c,d){b=Ev(b,Ev(c,d));c=a.L;return a.O?(d=Uw(c+1,b),d<=c?Yw(a,d,b):a.O(b)):a.apply(a,Tw(b))};U.ca=function(a,b,c,d,e){b=Ev(b,Ev(c,Ev(d,e)));c=a.L;return a.O?(d=Uw(c+1,b),d<=c?Yw(a,d,b):a.O(b)):a.apply(a,Tw(b))};
U.w=function(a,b,c,d,e,g){b=Ev(b,Ev(c,Ev(d,Ev(e,Vw(g)))));c=a.L;return a.O?(d=Uw(c+1,b),d<=c?Yw(a,d,b):a.O(b)):a.apply(a,Tw(b))};U.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),g=K(e),e=I(g),g=K(g);return U.w(b,a,c,d,e,g)};U.L=5;
var Zw=function Zw(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Zw.c(arguments[0]);case 2:return Zw.h(arguments[0],arguments[1]);default:return Zw.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};Zw.c=function(){return!1};Zw.h=function(a,b){return!L.h(a,b)};Zw.w=function(a,b,c){return At(U.N(L,a,b,c))};Zw.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return Zw.w(b,a,c)};Zw.L=2;function $w(a){return G(a)?a:null}
function ax(){"undefined"===typeof nt&&(nt=function(a){this.sw=a;this.C=393216;this.Z=0},nt.prototype.za=function(a,b){return new nt(b)},nt.prototype.sa=function(){return this.sw},nt.prototype.mb=function(){return!1},nt.prototype.next=function(){return Error("No such element")},nt.prototype.remove=function(){return Error("Unsupported operation")},nt.OA=function(){return new V(null,1,5,X,[bx],null)},nt.lr=!0,nt.Sj="cljs.core/t_cljs$core14185",nt.qv=function(a){return Eu(a,"cljs.core/t_cljs$core14185")});
return new nt(cx)}function dx(a,b){for(;;){if(null==G(b))return!0;var c=I(b);c=a.c?a.c(c):a.call(null,c);if(z(c)){c=a;var d=K(b);a=c;b=d}else return!1}}function ex(a,b){for(;;)if(G(b)){var c=I(b);c=a.c?a.c(c):a.call(null,c);if(z(c))return c;c=a;var d=K(b);a=c;b=d}else return null}
function fx(a){return function(){function b(b,c){return At(a.h?a.h(b,c):a.call(null,b,c))}function c(b){return At(a.c?a.c(b):a.call(null,b))}function d(){return At(a.T?a.T():a.call(null))}var e=null,g=function(){function b(a,b,d){var e=null;if(2<arguments.length){for(var e=0,g=Array(arguments.length-2);e<g.length;)g[e]=arguments[e+2],++e;e=new H(g,0,null)}return c.call(this,a,b,e)}function c(b,c,d){return At(U.N(a,b,c,d))}b.L=2;b.O=function(a){var b=I(a);a=K(a);var d=I(a);a=hv(a);return c(b,d,a)};
b.w=c;return b}(),e=function(a,e,l){switch(arguments.length){case 0:return d.call(this);case 1:return c.call(this,a);case 2:return b.call(this,a,e);default:var h=null;if(2<arguments.length){for(var h=0,k=Array(arguments.length-2);h<k.length;)k[h]=arguments[h+2],++h;h=new H(k,0,null)}return g.w(a,e,h)}throw Error("Invalid arity: "+(arguments.length-1));};e.L=2;e.O=g.O;e.T=d;e.c=c;e.h=b;e.w=g.w;return e}()}
var gx=function gx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return gx.T();case 1:return gx.c(arguments[0]);case 2:return gx.h(arguments[0],arguments[1]);case 3:return gx.j(arguments[0],arguments[1],arguments[2]);default:return gx.w(arguments[0],arguments[1],arguments[2],new H(c.slice(3),0,null))}};gx.T=function(){return pw};gx.c=function(a){return a};
gx.h=function(a,b){return function(){function c(c,d,e){c=b.j?b.j(c,d,e):b.call(null,c,d,e);return a.c?a.c(c):a.call(null,c)}function d(c,d){var e=b.h?b.h(c,d):b.call(null,c,d);return a.c?a.c(e):a.call(null,e)}function e(c){c=b.c?b.c(c):b.call(null,c);return a.c?a.c(c):a.call(null,c)}function g(){var c=b.T?b.T():b.call(null);return a.c?a.c(c):a.call(null,c)}var h=null,k=function(){function c(a,b,c,e){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+
3],++g;g=new H(h,0,null)}return d.call(this,a,b,c,g)}function d(c,d,e,g){c=U.ca(b,c,d,e,g);return a.c?a.c(c):a.call(null,c)}c.L=3;c.O=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var e=I(a);a=hv(a);return d(b,c,e,a)};c.w=d;return c}(),h=function(a,b,h,u){switch(arguments.length){case 0:return g.call(this);case 1:return e.call(this,a);case 2:return d.call(this,a,b);case 3:return c.call(this,a,b,h);default:var l=null;if(3<arguments.length){for(var l=0,m=Array(arguments.length-3);l<m.length;)m[l]=
arguments[l+3],++l;l=new H(m,0,null)}return k.w(a,b,h,l)}throw Error("Invalid arity: "+(arguments.length-1));};h.L=3;h.O=k.O;h.T=g;h.c=e;h.h=d;h.j=c;h.w=k.w;return h}()};
gx.j=function(a,b,c){return function(){function d(d,e,g){d=c.j?c.j(d,e,g):c.call(null,d,e,g);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}function e(d,e){var g=c.h?c.h(d,e):c.call(null,d,e);g=b.c?b.c(g):b.call(null,g);return a.c?a.c(g):a.call(null,g)}function g(d){d=c.c?c.c(d):c.call(null,d);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}function h(){var d=c.T?c.T():c.call(null);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}var k=null,l=function(){function d(a,
b,c,d){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return e.call(this,a,b,c,g)}function e(d,e,g,h){d=U.ca(c,d,e,g,h);d=b.c?b.c(d):b.call(null,d);return a.c?a.c(d):a.call(null,d)}d.L=3;d.O=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var d=I(a);a=hv(a);return e(b,c,d,a)};d.w=e;return d}(),k=function(a,b,c,k){switch(arguments.length){case 0:return h.call(this);case 1:return g.call(this,a);case 2:return e.call(this,
a,b);case 3:return d.call(this,a,b,c);default:var m=null;if(3<arguments.length){for(var m=0,p=Array(arguments.length-3);m<p.length;)p[m]=arguments[m+3],++m;m=new H(p,0,null)}return l.w(a,b,c,m)}throw Error("Invalid arity: "+(arguments.length-1));};k.L=3;k.O=l.O;k.T=h;k.c=g;k.h=e;k.j=d;k.w=l.w;return k}()};
gx.w=function(a,b,c,d){return function(a){return function(){function b(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(b){b=U.h(I(a),b);for(var c=K(a);;)if(c)b=I(c).call(null,b),c=K(c);else return b}b.L=0;b.O=function(a){a=G(a);return c(a)};b.w=c;return b}()}(Fw(Ev(a,Ev(b,Ev(c,d)))))};gx.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),d=K(d);return gx.w(b,a,c,d)};gx.L=3;
var hx=function hx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return hx.c(arguments[0]);case 2:return hx.h(arguments[0],arguments[1]);case 3:return hx.j(arguments[0],arguments[1],arguments[2]);case 4:return hx.N(arguments[0],arguments[1],arguments[2],arguments[3]);default:return hx.w(arguments[0],arguments[1],arguments[2],arguments[3],new H(c.slice(4),0,null))}};hx.c=function(a){return a};
hx.h=function(a,b){return function(){function c(c,d,e){return a.N?a.N(b,c,d,e):a.call(null,b,c,d,e)}function d(c,d){return a.j?a.j(b,c,d):a.call(null,b,c,d)}function e(c){return a.h?a.h(b,c):a.call(null,b,c)}function g(){return a.c?a.c(b):a.call(null,b)}var h=null,k=function(){function c(a,b,c,e){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return d.call(this,a,b,c,g)}function d(c,d,e,g){return U.w(a,b,c,d,e,R([g],
0))}c.L=3;c.O=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var e=I(a);a=hv(a);return d(b,c,e,a)};c.w=d;return c}(),h=function(a,b,h,u){switch(arguments.length){case 0:return g.call(this);case 1:return e.call(this,a);case 2:return d.call(this,a,b);case 3:return c.call(this,a,b,h);default:var l=null;if(3<arguments.length){for(var l=0,m=Array(arguments.length-3);l<m.length;)m[l]=arguments[l+3],++l;l=new H(m,0,null)}return k.w(a,b,h,l)}throw Error("Invalid arity: "+(arguments.length-1));};h.L=3;h.O=
k.O;h.T=g;h.c=e;h.h=d;h.j=c;h.w=k.w;return h}()};
hx.j=function(a,b,c){return function(){function d(d,e,g){return a.ca?a.ca(b,c,d,e,g):a.call(null,b,c,d,e,g)}function e(d,e){return a.N?a.N(b,c,d,e):a.call(null,b,c,d,e)}function g(d){return a.j?a.j(b,c,d):a.call(null,b,c,d)}function h(){return a.h?a.h(b,c):a.call(null,b,c)}var k=null,l=function(){function d(a,b,c,d){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return e.call(this,a,b,c,g)}function e(d,e,g,h){return U.w(a,
b,c,d,e,R([g,h],0))}d.L=3;d.O=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var d=I(a);a=hv(a);return e(b,c,d,a)};d.w=e;return d}(),k=function(a,b,c,k){switch(arguments.length){case 0:return h.call(this);case 1:return g.call(this,a);case 2:return e.call(this,a,b);case 3:return d.call(this,a,b,c);default:var m=null;if(3<arguments.length){for(var m=0,p=Array(arguments.length-3);m<p.length;)p[m]=arguments[m+3],++m;m=new H(p,0,null)}return l.w(a,b,c,m)}throw Error("Invalid arity: "+(arguments.length-
1));};k.L=3;k.O=l.O;k.T=h;k.c=g;k.h=e;k.j=d;k.w=l.w;return k}()};
hx.N=function(a,b,c,d){return function(){function e(e,g,h){return a.Dc?a.Dc(b,c,d,e,g,h):a.call(null,b,c,d,e,g,h)}function g(e,g){return a.ca?a.ca(b,c,d,e,g):a.call(null,b,c,d,e,g)}function h(e){return a.N?a.N(b,c,d,e):a.call(null,b,c,d,e)}function k(){return a.j?a.j(b,c,d):a.call(null,b,c,d)}var l=null,m=function(){function e(a,b,c,d){var e=null;if(3<arguments.length){for(var e=0,h=Array(arguments.length-3);e<h.length;)h[e]=arguments[e+3],++e;e=new H(h,0,null)}return g.call(this,a,b,c,e)}function g(e,
g,h,k){return U.w(a,b,c,d,e,R([g,h,k],0))}e.L=3;e.O=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var d=I(a);a=hv(a);return g(b,c,d,a)};e.w=g;return e}(),l=function(a,b,c,d){switch(arguments.length){case 0:return k.call(this);case 1:return h.call(this,a);case 2:return g.call(this,a,b);case 3:return e.call(this,a,b,c);default:var l=null;if(3<arguments.length){for(var l=0,p=Array(arguments.length-3);l<p.length;)p[l]=arguments[l+3],++l;l=new H(p,0,null)}return m.w(a,b,c,l)}throw Error("Invalid arity: "+
(arguments.length-1));};l.L=3;l.O=m.O;l.T=k;l.c=h;l.h=g;l.j=e;l.w=m.w;return l}()};hx.w=function(a,b,c,d,e){return function(){function g(a){var b=null;if(0<arguments.length){for(var b=0,c=Array(arguments.length-0);b<c.length;)c[b]=arguments[b+0],++b;b=new H(c,0,null)}return h.call(this,b)}function h(g){return U.ca(a,b,c,d,Ww.h(e,g))}g.L=0;g.O=function(a){a=G(a);return h(a)};g.w=h;return g}()};hx.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),e=K(e);return hx.w(b,a,c,d,e)};
hx.L=4;
function ix(){var a=jx,b=kx;return function(){function c(c,d,e){c=null==c?b:c;return a.j?a.j(c,d,e):a.call(null,c,d,e)}function d(c,d){var e=null==c?b:c;return a.h?a.h(e,d):a.call(null,e,d)}function e(c){c=null==c?b:c;return a.c?a.c(c):a.call(null,c)}var g=null,h=function(){function c(a,b,c,e){var g=null;if(3<arguments.length){for(var g=0,h=Array(arguments.length-3);g<h.length;)h[g]=arguments[g+3],++g;g=new H(h,0,null)}return d.call(this,a,b,c,g)}function d(c,d,e,g){return U.ca(a,null==c?b:c,d,e,
g)}c.L=3;c.O=function(a){var b=I(a);a=K(a);var c=I(a);a=K(a);var e=I(a);a=hv(a);return d(b,c,e,a)};c.w=d;return c}(),g=function(a,b,g,p){switch(arguments.length){case 1:return e.call(this,a);case 2:return d.call(this,a,b);case 3:return c.call(this,a,b,g);default:var k=null;if(3<arguments.length){for(var k=0,l=Array(arguments.length-3);k<l.length;)l[k]=arguments[k+3],++k;k=new H(l,0,null)}return h.w(a,b,g,k)}throw Error("Invalid arity: "+(arguments.length-1));};g.L=3;g.O=h.O;g.c=e;g.h=d;g.j=c;g.w=
h.w;return g}()}function lx(a,b){return function d(b,g){return new Lw(null,function(){var e=G(g);if(e){if(cw(e)){for(var k=Ou(e),l=Q(k),m=Pw(l),p=0;;)if(p<l)Sw(m,function(){var d=b+p,e=Qt.h(k,p);return a.h?a.h(d,e):a.call(null,d,e)}()),p+=1;else break;return Rw(m.sb(),d(b+l,Pu(e)))}return Ev(function(){var d=I(e);return a.h?a.h(b,d):a.call(null,b,d)}(),d(b+1,hv(e)))}return null},null,null)}(0,b)}function mx(a,b,c,d){this.state=a;this.meta=b;this.Qd=c;this.yb=d;this.Z=16386;this.C=6455296}f=mx.prototype;
f.equiv=function(a){return this.P(null,a)};f.P=function(a,b){return this===b};f.Pe=function(){return this.state};f.sa=function(){return this.meta};f.$h=function(a,b,c){for(var d,e=G(this.yb),g=null,h=0,k=0;;)if(k<h)d=g.aa(null,k),a=S(d,0,null),d=S(d,1,null),d.N?d.N(a,this,b,c):d.call(null,a,this,b,c),k+=1;else if(a=G(e))e=a,cw(e)?(g=Ou(e),e=Pu(e),a=g,d=Q(g),g=a,h=d):(g=I(e),a=S(g,0,null),d=S(g,1,null),d.N?d.N(a,this,b,c):d.call(null,a,this,b,c),e=K(e),g=null,h=0),k=0;else return null};
f.Zh=function(a,b,c){this.yb=T.j(this.yb,b,c);return this};f.ai=function(a,b){return this.yb=Pv.h(this.yb,b)};f.ha=function(){return pa(this)};function nx(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return ox(arguments[0]);default:return c=arguments[0],b=new H(b.slice(1),0,null),d=null!=b&&(b.C&64||x===b.Sb)?U.h(px,b):b,b=F.h(d,vt),d=F.h(d,qx),new mx(c,b,d,null)}}function ox(a){return new mx(a,null,null,null)}
function rx(a,b){if(a instanceof mx){var c=a.Qd;if(null!=c&&!z(c.c?c.c(b):c.call(null,b)))throw Error("Validator rejected reference state");c=a.state;a.state=b;null!=a.yb&&Gu(a,c,b);return b}return Qu(a,b)}
var sx=function sx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return sx.h(arguments[0],arguments[1]);case 3:return sx.j(arguments[0],arguments[1],arguments[2]);case 4:return sx.N(arguments[0],arguments[1],arguments[2],arguments[3]);default:return sx.w(arguments[0],arguments[1],arguments[2],arguments[3],new H(c.slice(4),0,null))}};
sx.h=function(a,b){if(a instanceof mx){var c=a.state;c=b.c?b.c(c):b.call(null,c);c=rx(a,c)}else c=Ru.h(a,b);return c};sx.j=function(a,b,c){if(a instanceof mx){var d=a.state;b=b.h?b.h(d,c):b.call(null,d,c);a=rx(a,b)}else a=Ru.j(a,b,c);return a};sx.N=function(a,b,c,d){if(a instanceof mx){var e=a.state;b=b.j?b.j(e,c,d):b.call(null,e,c,d);a=rx(a,b)}else a=Ru.N(a,b,c,d);return a};sx.w=function(a,b,c,d,e){return a instanceof mx?rx(a,U.ca(b,a.state,c,d,e)):Ru.ca(a,b,c,d,e)};
sx.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),e=K(e);return sx.w(b,a,c,d,e)};sx.L=4;
function tx(a,b){return function d(b,g){return new Lw(null,function(){var e=G(g);if(e){if(cw(e)){for(var k=Ou(e),l=Q(k),m=Pw(l),p=0;;)if(p<l){var u=function(){var d=b+p,e=Qt.h(k,p);return a.h?a.h(d,e):a.call(null,d,e)}();null!=u&&m.add(u);p+=1}else break;return Rw(m.sb(),d(b+l,Pu(e)))}l=function(){var d=I(e);return a.h?a.h(b,d):a.call(null,b,d)}();return null==l?d(b+1,hv(e)):Ev(l,d(b+1,hv(e)))}return null},null,null)}(0,b)}
var Z=function Z(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return Z.c(arguments[0]);case 2:return Z.h(arguments[0],arguments[1]);case 3:return Z.j(arguments[0],arguments[1],arguments[2]);case 4:return Z.N(arguments[0],arguments[1],arguments[2],arguments[3]);default:return Z.w(arguments[0],arguments[1],arguments[2],arguments[3],new H(c.slice(4),0,null))}};
Z.c=function(a){return function(b){return function(){function c(c,d){var e=a.c?a.c(d):a.call(null,d);return b.h?b.h(c,e):b.call(null,c,e)}function d(a){return b.c?b.c(a):b.call(null,a)}function e(){return b.T?b.T():b.call(null)}var g=null,h=function(){function c(a,b,c){var e=null;if(2<arguments.length){for(var e=0,g=Array(arguments.length-2);e<g.length;)g[e]=arguments[e+2],++e;e=new H(g,0,null)}return d.call(this,a,b,e)}function d(c,d,e){d=U.j(a,d,e);return b.h?b.h(c,d):b.call(null,c,d)}c.L=2;c.O=
function(a){var b=I(a);a=K(a);var c=I(a);a=hv(a);return d(b,c,a)};c.w=d;return c}(),g=function(a,b,g){switch(arguments.length){case 0:return e.call(this);case 1:return d.call(this,a);case 2:return c.call(this,a,b);default:var k=null;if(2<arguments.length){for(var k=0,l=Array(arguments.length-2);k<l.length;)l[k]=arguments[k+2],++k;k=new H(l,0,null)}return h.w(a,b,k)}throw Error("Invalid arity: "+(arguments.length-1));};g.L=2;g.O=h.O;g.T=e;g.c=d;g.h=c;g.w=h.w;return g}()}};
Z.h=function(a,b){return new Lw(null,function(){var c=G(b);if(c){if(cw(c)){for(var d=Ou(c),e=Q(d),g=Pw(e),h=0;;)if(h<e)Sw(g,function(){var b=Qt.h(d,h);return a.c?a.c(b):a.call(null,b)}()),h+=1;else break;return Rw(g.sb(),Z.h(a,Pu(c)))}return Ev(function(){var b=I(c);return a.c?a.c(b):a.call(null,b)}(),Z.h(a,hv(c)))}return null},null,null)};
Z.j=function(a,b,c){return new Lw(null,function(){var d=G(b),e=G(c);if(d&&e){var g=Ev;var h=I(d);var k=I(e);h=a.h?a.h(h,k):a.call(null,h,k);d=g(h,Z.j(a,hv(d),hv(e)))}else d=null;return d},null,null)};Z.N=function(a,b,c,d){return new Lw(null,function(){var e=G(b),g=G(c),h=G(d);if(e&&g&&h){var k=Ev;var l=I(e);var m=I(g),p=I(h);l=a.j?a.j(l,m,p):a.call(null,l,m,p);e=k(l,Z.N(a,hv(e),hv(g),hv(h)))}else e=null;return e},null,null)};
Z.w=function(a,b,c,d,e){var g=function k(a){return new Lw(null,function(){var b=Z.h(G,a);return dx(pw,b)?Ev(Z.h(I,b),k(Z.h(hv,b))):null},null,null)};return Z.h(function(){return function(b){return U.h(a,b)}}(g),g(Lv.w(e,d,R([c,b],0))))};Z.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),e=K(e);return Z.w(b,a,c,d,e)};Z.L=4;
function ux(a,b){if("number"!==typeof a)throw Error("Assert failed: (number? n)");return new Lw(null,function(){if(0<a){var c=G(b);return c?Ev(I(c),ux(a-1,hv(c))):null}return null},null,null)}function vx(a,b){if("number"!==typeof a)throw Error("Assert failed: (number? n)");return new Lw(null,function(c){return function(){return c(a,b)}}(function(a,b){for(;;){var c=G(b);if(0<a&&c){var d=a-1,c=hv(c);a=d;b=c}else return c}}),null,null)}function wx(a){return Z.j(function(a){return a},a,vx(2,a))}
function xx(a){return new Lw(null,function(){return Ev(a,xx(a))},null,null)}function yx(a,b){return U.h(Ww,U.j(Z,a,b))}function zx(a,b){return new Lw(null,function(){var c=G(b);if(c){if(cw(c)){for(var d=Ou(c),e=Q(d),g=Pw(e),h=0;;)if(h<e){var k=Qt.h(d,h);k=a.c?a.c(k):a.call(null,k);z(k)&&(k=Qt.h(d,h),g.add(k));h+=1}else break;return Rw(g.sb(),zx(a,Pu(c)))}d=I(c);c=hv(c);return z(a.c?a.c(d):a.call(null,d))?Ev(d,zx(a,c)):zx(a,c)}return null},null,null)}function Ax(a,b){return zx(fx(a),b)}
function Bx(a){return function c(a){return new Lw(null,function(){return Ev(a,z(Zv.c?Zv.c(a):Zv.call(null,a))?yx(c,R([G.c?G.c(a):G.call(null,a)],0)):null)},null,null)}(a)}function Cx(a){return zx(function(a){return!Zv(a)},hv(Bx(a)))}
var jx=function jx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return jx.T();case 1:return jx.c(arguments[0]);case 2:return jx.h(arguments[0],arguments[1]);case 3:return jx.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};jx.T=function(){return Mv};jx.c=function(a){return a};
jx.h=function(a,b){return null!=a?null!=a&&(a.Z&4||x===a.dv)?ru(Lu(Gt(Ku,Ju(a),b)),Tv(a)):Gt(Ot,a,b):Gt(Lv,iv,b)};jx.j=function(a,b,c){return null!=a&&(a.Z&4||x===a.dv)?ru(Lu(qw(b,Xw,Ju(a),c)),Tv(a)):qw(b,Lv,a,c)};jx.L=3;function Dx(a,b){return Lu(Gt(function(b,d){return Xw.h(b,a.c?a.c(d):a.call(null,d))},Ju(Mv),b))}function Ex(a,b,c){return new Lw(null,function(){var d=G(c);if(d){var e=ux(a,d);return a===Q(e)?Ev(e,Ex(a,b,vx(b,d))):null}return null},null,null)}
function Fx(a,b,c,d){return new Lw(null,function(){var e=G(d);if(e){var g=ux(a,e);if(a===Q(g))return Ev(g,Fx(a,b,c,vx(b,e)));e=ux(a,Ww.h(g,c));return Ot(iv,e)}return null},null,null)}function Gx(a,b){return Gt(F,a,b)}function Hx(a,b,c){var d=fw;for(b=G(b);;)if(null!=b){a=F.j(a,I(b),d);if(d===a)return c;b=K(b)}else return a}
var Ix=function Ix(b,c,d){c=G(c);var e=I(c),g=K(c);return g?T.j(b,e,function(){var c=F.h(b,e);return Ix.j?Ix.j(c,g,d):Ix.call(null,c,g,d)}()):T.j(b,e,d)},Jx=function Jx(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 3:return Jx.j(arguments[0],arguments[1],arguments[2]);case 4:return Jx.N(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return Jx.ca(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);case 6:return Jx.Dc(arguments[0],
arguments[1],arguments[2],arguments[3],arguments[4],arguments[5]);default:return Jx.w(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5],new H(c.slice(6),0,null))}};Jx.j=function(a,b,c){b=G(b);var d=I(b);return(b=K(b))?T.j(a,d,Jx.j(F.h(a,d),b,c)):T.j(a,d,function(){var b=F.h(a,d);return c.c?c.c(b):c.call(null,b)}())};Jx.N=function(a,b,c,d){b=G(b);var e=I(b);return(b=K(b))?T.j(a,e,Jx.N(F.h(a,e),b,c,d)):T.j(a,e,function(){var b=F.h(a,e);return c.h?c.h(b,d):c.call(null,b,d)}())};
Jx.ca=function(a,b,c,d,e){b=G(b);var g=I(b);return(b=K(b))?T.j(a,g,Jx.ca(F.h(a,g),b,c,d,e)):T.j(a,g,function(){var b=F.h(a,g);return c.j?c.j(b,d,e):c.call(null,b,d,e)}())};Jx.Dc=function(a,b,c,d,e,g){b=G(b);var h=I(b);return(b=K(b))?T.j(a,h,Jx.Dc(F.h(a,h),b,c,d,e,g)):T.j(a,h,function(){var b=F.h(a,h);return c.N?c.N(b,d,e,g):c.call(null,b,d,e,g)}())};Jx.w=function(a,b,c,d,e,g,h){var k=G(b);b=I(k);return(k=K(k))?T.j(a,b,U.w(Jx,F.h(a,b),k,c,d,R([e,g,h],0))):T.j(a,b,U.w(c,F.h(a,b),d,e,g,R([h],0)))};
Jx.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),e=K(d),d=I(e),g=K(e),e=I(g),h=K(g),g=I(h),h=K(h);return Jx.w(b,a,c,d,e,g,h)};Jx.L=6;function Kx(a,b,c){return T.j(a,b,function(){var d=F.h(a,b);return c.c?c.c(d):c.call(null,d)}())}function Lx(a,b){var c=Mx,d=ix();return T.j(a,c,function(){var e=F.h(a,c);return d.h?d.h(e,b):d.call(null,e,b)}())}function Nx(a,b){this.Da=a;this.v=b}
function Ox(a){return new Nx(a,[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null])}function Px(a){return new Nx(a.Da,Et(a.v))}function Qx(a){a=a.H;return 32>a?0:a-1>>>5<<5}function Rx(a,b,c){for(;;){if(0===b)return c;var d=Ox(a);d.v[0]=c;c=d;b-=5}}
var Sx=function Sx(b,c,d,e){var g=Px(d),h=b.H-1>>>c&31;5===c?g.v[h]=e:(d=d.v[h],null!=d?(c-=5,b=Sx.N?Sx.N(b,c,d,e):Sx.call(null,b,c,d,e)):b=Rx(null,c-5,e),g.v[h]=b);return g};function Tx(a,b){throw Error([C.c("No item "),C.c(a),C.c(" in vector of length "),C.c(b)].join(""));}function Ux(a,b){if(b>=Qx(a))return a.Ob;for(var c=a.root,d=a.shift;;)if(0<d)var e=d-5,c=c.v[b>>>d&31],d=e;else return c.v}function Vx(a,b){return 0<=b&&b<a.H?Ux(a,b):Tx(b,a.H)}
var Wx=function Wx(b,c,d,e,g){var h=Px(d);if(0===c)h.v[e&31]=g;else{var k=e>>>c&31;c-=5;d=d.v[k];b=Wx.ca?Wx.ca(b,c,d,e,g):Wx.call(null,b,c,d,e,g);h.v[k]=b}return h},Xx=function Xx(b,c,d){var e=b.H-2>>>c&31;if(5<c){c-=5;var g=d.v[e];b=Xx.j?Xx.j(b,c,g):Xx.call(null,b,c,g);if(null==b&&0===e)return null;d=Px(d);d.v[e]=b;return d}if(0===e)return null;d=Px(d);d.v[e]=null;return d};function Yx(a,b,c,d,e,g){this.i=a;this.base=b;this.v=c;this.Fc=d;this.start=e;this.end=g}
Yx.prototype.mb=function(){return this.i<this.end};Yx.prototype.next=function(){32===this.i-this.base&&(this.v=Ux(this.Fc,this.i),this.base+=32);var a=this.v[this.i&31];this.i+=1;return a};function Zx(a,b,c){return new Yx(b,b-b%32,b<Q(a)?Ux(a,b):null,a,b,c)}function V(a,b,c,d,e,g){this.meta=a;this.H=b;this.shift=c;this.root=d;this.Ob=e;this.M=g;this.C=167668511;this.Z=8196}f=V.prototype;f.Lj=x;f.Wh=function(a,b){return new V(null,2,5,X,[b,F.h(this,b)],null)};f.toString=function(){return Uu(this)};
f.equiv=function(a){return this.P(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){return"number"===typeof b?this.ac(null,b,c):c};
f.Kg=function(a,b,c){a=0;for(var d=c;;)if(a<this.H){var e=Ux(this,a);c=e.length;a:for(var g=0;;)if(g<c)var h=g+a,k=e[g],d=b.j?b.j(d,h,k):b.call(null,d,h,k),g=g+1;else{e=d;break a}a+=c;d=e}else return d};f.aa=function(a,b){return Vx(this,b)[b&31]};f.ac=function(a,b,c){return 0<=b&&b<this.H?Ux(this,b)[b&31]:c};
f.Yh=function(a,b,c){if(0<=b&&b<this.H)return Qx(this)<=b?(a=Et(this.Ob),a[b&31]=c,new V(this.meta,this.H,this.shift,this.root,a,null)):new V(this.meta,this.H,this.shift,Wx(this,this.shift,this.root,b,c),this.Ob,null);if(b===this.H)return this.xa(null,c);throw Error([C.c("Index "),C.c(b),C.c(" out of bounds  [0,"),C.c(this.H),C.c("]")].join(""));};f.lc=function(){return Zx(this,0,this.H)};f.sa=function(){return this.meta};f.ya=function(){return this.H};f.Km=function(){return this.aa(null,0)};
f.Lm=function(){return this.aa(null,1)};f.Mg=function(){return 0<this.H?this.aa(null,this.H-1):null};
f.Ng=function(){if(0===this.H)throw Error("Can't pop empty vector");if(1===this.H)return ru(Mv,this.meta);if(1<this.H-Qx(this))return new V(this.meta,this.H-1,this.shift,this.root,this.Ob.slice(0,-1),null);var a=Ux(this,this.H-2),b=Xx(this,this.shift,this.root),b=null==b?X:b,c=this.H-1;return 5<this.shift&&null==b.v[1]?new V(this.meta,c,this.shift-5,b.v[0],a,null):new V(this.meta,c,this.shift,b,a,null)};f.Lg=function(){return 0<this.H?new Cv(this,this.H-1,null):null};
f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){if(b instanceof V)if(this.H===Q(b))for(var c=this.lc(null),d=Su(b);;)if(c.mb()){var e=c.next(),g=d.next();if(!L.h(e,g))return!1}else return!0;else return!1;else return Dv(this,b)};f.Jg=function(){return new $x(this.H,this.shift,ay.c?ay.c(this.root):ay.call(null,this.root),by.c?by.c(this.Ob):by.call(null,this.Ob))};f.Ka=function(){return ru(Mv,this.meta)};f.Bb=function(a,b){return rv(this,b)};
f.Cb=function(a,b,c){a=0;for(var d=c;;)if(a<this.H){var e=Ux(this,a);c=e.length;a:for(var g=0;;)if(g<c)var h=e[g],d=b.h?b.h(d,h):b.call(null,d,h),g=g+1;else{e=d;break a}a+=c;d=e}else return d};f.Rb=function(a,b,c){if("number"===typeof b)return this.Yh(null,b,c);throw Error("Vector's key for assoc must be a number.");};f.Vh=function(a,b){return kw(b)?0<=b&&b<this.H:!1};
f.oa=function(){if(0===this.H)return null;if(32>=this.H)return new H(this.Ob,0,null);a:{var a=this.root;for(var b=this.shift;;)if(0<b)b-=5,a=a.v[0];else{a=a.v;break a}}return cy?cy(this,a,0,0):dy.call(null,this,a,0,0)};f.za=function(a,b){return new V(b,this.H,this.shift,this.root,this.Ob,this.M)};
f.xa=function(a,b){if(32>this.H-Qx(this)){for(var c=this.Ob.length,d=Array(c+1),e=0;;)if(e<c)d[e]=this.Ob[e],e+=1;else break;d[c]=b;return new V(this.meta,this.H+1,this.shift,this.root,d,null)}c=(d=this.H>>>5>1<<this.shift)?this.shift+5:this.shift;d?(d=Ox(null),d.v[0]=this.root,e=Rx(null,this.shift,new Nx(null,this.Ob)),d.v[1]=e):d=Sx(this,this.shift,this.root,new Nx(null,this.Ob));return new V(this.meta,this.H+1,c,d,[b],null)};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.aa(null,c);case 3:return this.ac(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.aa(null,c)};a.j=function(a,c,d){return this.ac(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.aa(null,a)};f.h=function(a,b){return this.ac(null,a,b)};
var X=new Nx(null,[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null]),Mv=new V(null,0,5,X,[],nv);function ey(a){var b=a.length;if(32>b)return new V(null,b,5,X,a,null);for(var c=32,d=(new V(null,32,5,X,a.slice(0,32),null)).Jg(null);;)if(c<b)var e=c+1,d=Xw.h(d,a[c]),c=e;else return Lu(d)}V.prototype[Dt]=function(){return kv(this)};function fy(a){return zt(a)?ey(a):Lu(Gt(Ku,Ju(Mv),a))}
var gy=function gy(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return gy.w(0<c.length?new H(c.slice(0),0,null):null)};gy.w=function(a){return a instanceof H&&0===a.i?ey(a.v):fy(a)};gy.L=0;gy.O=function(a){return gy.w(G(a))};function hy(a,b,c,d,e,g){this.Gc=a;this.node=b;this.i=c;this.wb=d;this.meta=e;this.M=g;this.C=32375020;this.Z=1536}f=hy.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.bc=function(){if(this.wb+1<this.node.length){var a=this.Gc;var b=this.node,c=this.i,d=this.wb+1;a=cy?cy(a,b,c,d):dy.call(null,a,b,c,d);return null==a?null:a}return this.er(null)};
f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(Mv,this.meta)};f.Bb=function(a,b){var c=this.Gc;var d=this.i+this.wb,e=Q(this.Gc);c=iy.j?iy.j(c,d,e):iy.call(null,c,d,e);return rv(c,b)};f.Cb=function(a,b,c){a=this.Gc;var d=this.i+this.wb,e=Q(this.Gc);a=iy.j?iy.j(a,d,e):iy.call(null,a,d,e);return sv(a,b,c)};f.hb=function(){return this.node[this.wb]};
f.Db=function(){if(this.wb+1<this.node.length){var a=this.Gc;var b=this.node,c=this.i,d=this.wb+1;a=cy?cy(a,b,c,d):dy.call(null,a,b,c,d);return null==a?iv:a}return this.Kj(null)};f.oa=function(){return this};f.Jm=function(){var a=this.node;return new Ow(a,this.wb,a.length)};f.Kj=function(){var a=this.i+this.node.length;if(a<Lt(this.Gc)){var b=this.Gc,c=Ux(this.Gc,a);return cy?cy(b,c,a,0):dy.call(null,b,c,a,0)}return iv};
f.za=function(a,b){return jy?jy(this.Gc,this.node,this.i,this.wb,b):dy.call(null,this.Gc,this.node,this.i,this.wb,b)};f.xa=function(a,b){return Ev(b,this)};f.er=function(){var a=this.i+this.node.length;if(a<Lt(this.Gc)){var b=this.Gc,c=Ux(this.Gc,a);return cy?cy(b,c,a,0):dy.call(null,b,c,a,0)}return null};hy.prototype[Dt]=function(){return kv(this)};
function dy(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 3:return b=arguments[0],c=arguments[1],d=arguments[2],new hy(b,Vx(b,c),c,d,null,null);case 4:return cy(arguments[0],arguments[1],arguments[2],arguments[3]);case 5:return jy(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}function cy(a,b,c,d){return new hy(a,b,c,d,null,null)}
function jy(a,b,c,d,e){return new hy(a,b,c,d,e,null)}function ky(a,b,c,d,e){this.meta=a;this.Fc=b;this.start=c;this.end=d;this.M=e;this.C=167666463;this.Z=8192}f=ky.prototype;f.Lj=x;f.Wh=function(a,b){return new V(null,2,5,X,[b,F.h(this,b)],null)};f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){return"number"===typeof b?this.ac(null,b,c):c};
f.Kg=function(a,b,c){a=this.start;for(var d=0;;)if(a<this.end){var e=d,g=Qt.h(this.Fc,a);c=b.j?b.j(c,e,g):b.call(null,c,e,g);d+=1;a+=1}else return c};f.aa=function(a,b){return 0>b||this.end<=this.start+b?Tx(b,this.end-this.start):Qt.h(this.Fc,this.start+b)};f.ac=function(a,b,c){return 0>b||this.end<=this.start+b?c:Qt.j(this.Fc,this.start+b,c)};
f.Yh=function(a,b,c){a=this.start+b;if(0>b||this.end+1<=a)throw Error([C.c("Index "),C.c(b),C.c(" out of bounds [0,"),C.c(this.ya(null)),C.c("]")].join(""));b=this.meta;c=T.j(this.Fc,a,c);var d=this.start,e=this.end;a+=1;a=e>a?e:a;return ly.ca?ly.ca(b,c,d,a,null):ly.call(null,b,c,d,a,null)};f.lc=function(){return Zx(this.Fc,this.start,this.end)};f.sa=function(){return this.meta};f.ya=function(){return this.end-this.start};f.Mg=function(){return Qt.h(this.Fc,this.end-1)};
f.Ng=function(){if(this.start===this.end)throw Error("Can't pop empty vector");var a=this.meta,b=this.Fc,c=this.start,d=this.end-1;return ly.ca?ly.ca(a,b,c,d,null):ly.call(null,a,b,c,d,null)};f.Lg=function(){return this.start!==this.end?new Cv(this,this.end-this.start-1,null):null};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(Mv,this.meta)};f.Bb=function(a,b){return rv(this,b)};
f.Cb=function(a,b,c){return sv(this,b,c)};f.Rb=function(a,b,c){if("number"===typeof b)return this.Yh(null,b,c);throw Error("Subvec's key for assoc must be a number.");};f.oa=function(){var a=this;return function(b){return function d(e){return e===a.end?null:Ev(Qt.h(a.Fc,e),new Lw(null,function(){return function(){return d(e+1)}}(b),null,null))}}(this)(a.start)};f.za=function(a,b){return ly.ca?ly.ca(b,this.Fc,this.start,this.end,this.M):ly.call(null,b,this.Fc,this.start,this.end,this.M)};
f.xa=function(a,b){var c=this.meta,d=lu(this.Fc,this.end,b),e=this.start,g=this.end+1;return ly.ca?ly.ca(c,d,e,g,null):ly.call(null,c,d,e,g,null)};f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.aa(null,c);case 3:return this.ac(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.aa(null,c)};a.j=function(a,c,d){return this.ac(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};
f.c=function(a){return this.aa(null,a)};f.h=function(a,b){return this.ac(null,a,b)};ky.prototype[Dt]=function(){return kv(this)};function ly(a,b,c,d,e){for(;;)if(b instanceof ky)c=b.start+c,d=b.start+d,b=b.Fc;else{if(!bw(b))throw Error("v must satisfy IVector");var g=Q(b);if(0>c||0>d||c>g||d>g)throw Error("Index out of bounds");return new ky(a,b,c,d,e)}}
var iy=function iy(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return iy.h(arguments[0],arguments[1]);case 3:return iy.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};iy.h=function(a,b){return iy.j(a,b,Q(a))};iy.j=function(a,b,c){return ly(null,a,b,c,null)};iy.L=3;function my(a,b){return a===b.Da?b:new Nx(a,Et(b.v))}function ay(a){return new Nx({},Et(a.v))}
function by(a){var b=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];ew(a,0,b,0,a.length);return b}var ny=function ny(b,c,d,e){d=my(b.root.Da,d);var g=b.H-1>>>c&31;if(5===c)b=e;else{var h=d.v[g];null!=h?(c-=5,b=ny.N?ny.N(b,c,h,e):ny.call(null,b,c,h,e)):b=Rx(b.root.Da,c-5,e)}d.v[g]=b;return d};function $x(a,b,c,d){this.H=a;this.shift=b;this.root=c;this.Ob=d;this.Z=88;this.C=275}f=$x.prototype;
f.Ff=function(a,b){if(this.root.Da){if(32>this.H-Qx(this))this.Ob[this.H&31]=b;else{var c=new Nx(this.root.Da,this.Ob),d=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];d[0]=b;this.Ob=d;if(this.H>>>5>1<<this.shift){var d=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null],e=this.shift+
5;d[0]=this.root;d[1]=Rx(this.root.Da,this.shift,c);this.root=new Nx(this.root.Da,d);this.shift=e}else this.root=ny(this,this.shift,this.root,c)}this.H+=1;return this}throw Error("conj! after persistent!");};f.Xh=function(){if(this.root.Da){this.root.Da=null;var a=this.H-Qx(this),b=Array(a);ew(this.Ob,0,b,0,a);return new V(null,this.H,this.shift,this.root,b,null)}throw Error("persistent! called twice");};
f.Ef=function(a,b,c){if("number"===typeof b)return oy(this,b,c);throw Error("TransientVector's key for assoc! must be a number.");};
function oy(a,b,c){if(a.root.Da){if(0<=b&&b<a.H){if(Qx(a)<=b)a.Ob[b&31]=c;else{var d=function(){return function g(d,k){var h=my(a.root.Da,k);if(0===d)h.v[b&31]=c;else{var m=b>>>d&31,p=g(d-5,h.v[m]);h.v[m]=p}return h}}(a).call(null,a.shift,a.root);a.root=d}return a}if(b===a.H)return a.Ff(null,c);throw Error([C.c("Index "),C.c(b),C.c(" out of bounds for TransientVector of length"),C.c(a.H)].join(""));}throw Error("assoc! after persistent!");}
f.ya=function(){if(this.root.Da)return this.H;throw Error("count after persistent!");};f.aa=function(a,b){if(this.root.Da)return Vx(this,b)[b&31];throw Error("nth after persistent!");};f.ac=function(a,b,c){return 0<=b&&b<this.H?this.aa(null,b):c};f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){return"number"===typeof b?this.ac(null,b,c):c};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.Y(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.Y(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.Y(null,a,b)};function py(a,b){this.Sg=a;this.Yi=b}
py.prototype.mb=function(){var a=null!=this.Sg&&G(this.Sg);return a?a:(a=null!=this.Yi)?this.Yi.mb():a};py.prototype.next=function(){if(null!=this.Sg){var a=I(this.Sg);this.Sg=K(this.Sg);return a}if(null!=this.Yi&&this.Yi.mb())return this.Yi.next();throw Error("No such element");};py.prototype.remove=function(){return Error("Unsupported operation")};function qy(a,b,c,d){this.meta=a;this.tc=b;this.rd=c;this.M=d;this.C=31850572;this.Z=0}f=qy.prototype;f.toString=function(){return Uu(this)};
f.equiv=function(a){return this.P(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.meta)};f.hb=function(){return I(this.tc)};
f.Db=function(){var a=K(this.tc);return a?new qy(this.meta,a,this.rd,null):null==this.rd?this.Ka(null):new qy(this.meta,this.rd,null,null)};f.oa=function(){return this};f.za=function(a,b){return new qy(b,this.tc,this.rd,this.M)};f.xa=function(a,b){return Ev(b,this)};qy.prototype[Dt]=function(){return kv(this)};function ry(a,b,c,d,e){this.meta=a;this.count=b;this.tc=c;this.rd=d;this.M=e;this.C=31858766;this.Z=8192}f=ry.prototype;f.toString=function(){return Uu(this)};
f.equiv=function(a){return this.P(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,this.count.c?this.count.c(this):this.count.call(null,this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.lc=function(){return new py(this.tc,Su(this.rd))};f.sa=function(){return this.meta};f.ya=function(){return this.count};f.Mg=function(){return I(this.tc)};
f.Ng=function(){if(z(this.tc)){var a=K(this.tc);return a?new ry(this.meta,this.count-1,a,this.rd,null):new ry(this.meta,this.count-1,G(this.rd),Mv,null)}return this};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(sy,this.meta)};f.hb=function(){return I(this.tc)};f.Db=function(){return hv(G(this))};f.oa=function(){var a=G(this.rd),b=this.tc;return z(z(b)?b:a)?new qy(null,this.tc,G(a),null):null};
f.za=function(a,b){return new ry(b,this.count,this.tc,this.rd,this.M)};f.xa=function(a,b){if(z(this.tc)){var c=this.rd;c=new ry(this.meta,this.count+1,this.tc,Lv.h(z(c)?c:Mv,b),null)}else c=new ry(this.meta,this.count+1,Lv.h(this.tc,b),Mv,null);return c};var sy=new ry(null,0,null,Mv,nv);ry.prototype[Dt]=function(){return kv(this)};function ty(){this.C=2097152;this.Z=0}ty.prototype.equiv=function(a){return this.P(null,a)};ty.prototype.P=function(){return!1};var uy=new ty;
function vy(a,b){return iw($v(b)?Q(a)===Q(b)?dx(function(a){return L.h(F.j(b,I(a),uy),Jv(a))},a):null:null)}function wy(a,b,c,d,e){this.i=a;this.Uw=b;this.Uq=c;this.Ma=d;this.Cr=e}wy.prototype.mb=function(){var a=this.i<this.Uq;return a?a:this.Cr.mb()};wy.prototype.next=function(){if(this.i<this.Uq){var a=yv(this.Ma,this.i);this.i+=1;return new V(null,2,5,X,[a,Wt.h(this.Uw,a)],null)}return this.Cr.next()};wy.prototype.remove=function(){return Error("Unsupported operation")};
function xy(a){this.s=a}xy.prototype.next=function(){if(null!=this.s){var a=I(this.s),b=S(a,0,null),a=S(a,1,null);this.s=K(this.s);return{value:[b,a],done:!1}}return{value:null,done:!0}};function yy(a){this.s=a}yy.prototype.next=function(){if(null!=this.s){var a=I(this.s);this.s=K(this.s);return{value:[a,a],done:!1}}return{value:null,done:!0}};
function zy(a,b){if(b instanceof Y)a:{var c=a.length;for(var d=b.Vb,e=0;;){if(c<=e){c=-1;break a}if(a[e]instanceof Y&&d===a[e].Vb){c=e;break a}e+=2}}else if(ea(b)||"number"===typeof b)a:for(c=a.length,d=0;;){if(c<=d){c=-1;break a}if(b===a[d]){c=d;break a}d+=2}else if(b instanceof fv)a:for(c=a.length,d=b.Md,e=0;;){if(c<=e){c=-1;break a}if(a[e]instanceof fv&&d===a[e].Md){c=e;break a}e+=2}else if(null==b)a:for(c=a.length,d=0;;){if(c<=d){c=-1;break a}if(null==a[d]){c=d;break a}d+=2}else a:for(c=a.length,
d=0;;){if(c<=d){c=-1;break a}if(L.h(b,a[d])){c=d;break a}d+=2}return c}function Ay(a,b,c){this.v=a;this.i=b;this.gc=c;this.C=32374990;this.Z=0}f=Ay.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.gc};f.bc=function(){return this.i<this.v.length-2?new Ay(this.v,this.i+2,this.gc):null};f.ya=function(){return(this.v.length-this.i)/2};f.ha=function(){return mv(this)};
f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.gc)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return new V(null,2,5,X,[this.v[this.i],this.v[this.i+1]],null)};f.Db=function(){return this.i<this.v.length-2?new Ay(this.v,this.i+2,this.gc):iv};f.oa=function(){return this};f.za=function(a,b){return new Ay(this.v,this.i,b)};f.xa=function(a,b){return Ev(b,this)};Ay.prototype[Dt]=function(){return kv(this)};
function By(a,b,c){this.v=a;this.i=b;this.H=c}By.prototype.mb=function(){return this.i<this.H};By.prototype.next=function(){var a=new V(null,2,5,X,[this.v[this.i],this.v[this.i+1]],null);this.i+=2;return a};function y(a,b,c,d){this.meta=a;this.H=b;this.v=c;this.M=d;this.C=16647951;this.Z=8196}f=y.prototype;f.Lj=x;f.Wh=function(a,b){var c=zy(this.v,b);return new V(null,2,5,X,[this.v[c],F.h(this,b)],null)};f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.keys=function(){return kv(Cy.c?Cy.c(this):Cy.call(null,this))};f.entries=function(){return new xy(G(G(this)))};f.values=function(){return kv(Dy.c?Dy.c(this):Dy.call(null,this))};f.has=function(a){return lw(this,a)};f.get=function(a,b){return this.Y(null,a,b)};
f.forEach=function(a){for(var b,c,d=G(this),e=null,g=0,h=0;;)if(h<g)b=e.aa(null,h),c=S(b,0,null),b=S(b,1,null),a.h?a.h(b,c):a.call(null,b,c),h+=1;else if(c=G(d))d=c,cw(d)?(e=Ou(d),d=Pu(d),c=e,b=Q(e),e=c,g=b):(e=I(d),c=S(e,0,null),b=S(e,1,null),a.h?a.h(b,c):a.call(null,b,c),d=K(d),e=null,g=0),h=0;else return null};f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){a=zy(this.v,b);return-1===a?c:this.v[a+1]};
f.Kg=function(a,b,c){a=this.v.length;for(var d=0;;)if(d<a){var e=this.v[d],g=this.v[d+1];c=b.j?b.j(c,e,g):b.call(null,c,e,g);d+=2}else return c};f.lc=function(){return new By(this.v,0,2*this.H)};f.sa=function(){return this.meta};f.ya=function(){return this.H};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=ov(this)};
f.P=function(a,b){if(null!=b&&(b.C&1024||x===b.ir)){var c=this.v.length;if(this.H===b.ya(null))for(var d=0;;)if(d<c){var e=b.Y(null,this.v[d],fw);if(e!==fw)if(L.h(this.v[d+1],e))d+=2;else return!1;else return!1}else return!0;else return!1}else return vy(this,b)};f.Jg=function(){return new Ey({},this.v.length,Et(this.v))};f.Ka=function(){return ru(cx,this.meta)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};
f.Df=function(a,b){if(0<=zy(this.v,b)){var c=this.v.length,d=c-2;if(0===d)return this.Ka(null);for(var d=Array(d),e=0,g=0;;){if(e>=c)return new y(this.meta,this.H-1,d,null);L.h(b,this.v[e])||(d[g]=this.v[e],d[g+1]=this.v[e+1],g+=2);e+=2}}else return this};
f.Rb=function(a,b,c){a=zy(this.v,b);if(-1===a){if(this.H<Fy){a=this.v;for(var d=a.length,e=Array(d+2),g=0;;)if(g<d)e[g]=a[g],g+=1;else break;e[d]=b;e[d+1]=c;return new y(this.meta,this.H+1,e,null)}return ru(Zt(jx.h(Gy,this),b,c),this.meta)}if(c===this.v[a+1])return this;b=Et(this.v);b[a+1]=c;return new y(this.meta,this.H,b,null)};f.Vh=function(a,b){return-1!==zy(this.v,b)};f.oa=function(){var a=this.v;return 0<=a.length-2?new Ay(a,0,null):null};f.za=function(a,b){return new y(b,this.H,this.v,this.M)};
f.xa=function(a,b){if(bw(b))return this.Rb(null,Qt.h(b,0),Qt.h(b,1));for(var c=this,d=G(b);;){if(null==d)return c;var e=I(d);if(bw(e))c=c.Rb(null,Qt.h(e,0),Qt.h(e,1)),d=K(d);else throw Error("conj on a map takes map entries or seqables of map entries");}};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.Y(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.Y(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.Y(null,a,b)};var cx=new y(null,0,[],pv),Fy=8;
function Ov(a){for(var b=[],c=0;;)if(c<a.length){var d=a[c],e=a[c+1],g=zy(b,d);-1===g?(g=b,g.push(d),g.push(e)):b[g+1]=e;c+=2}else break;return new y(null,b.length/2,b,null)}y.prototype[Dt]=function(){return kv(this)};function Ey(a,b,c){this.Rg=a;this.hg=b;this.v=c;this.C=258;this.Z=56}f=Ey.prototype;f.ya=function(){if(z(this.Rg))return ww(this.hg);throw Error("count after persistent!");};f.na=function(a,b){return this.Y(null,b,null)};
f.Y=function(a,b,c){if(z(this.Rg))return a=zy(this.v,b),-1===a?c:this.v[a+1];throw Error("lookup after persistent!");};f.Ff=function(a,b){if(z(this.Rg)){if(null!=b?b.C&2048||x===b.kv||(b.C?0:B(du,b)):B(du,b))return this.Ef(null,Aw.c?Aw.c(b):Aw.call(null,b),Bw.c?Bw.c(b):Bw.call(null,b));for(var c=G(b),d=this;;){var e=I(c);if(z(e))c=K(c),d=d.Ef(null,Aw.c?Aw.c(e):Aw.call(null,e),Bw.c?Bw.c(e):Bw.call(null,e));else return d}}else throw Error("conj! after persistent!");};
f.Xh=function(){if(z(this.Rg))return this.Rg=!1,new y(null,ww(this.hg),this.v,null);throw Error("persistent! called twice");};f.Ef=function(a,b,c){if(z(this.Rg)){a=zy(this.v,b);if(-1===a){if(this.hg+2<=2*Fy)return this.hg+=2,this.v.push(b),this.v.push(c),this;a=Hy.h?Hy.h(this.hg,this.v):Hy.call(null,this.hg,this.v);return Mu(a,b,c)}c!==this.v[a+1]&&(this.v[a+1]=c);return this}throw Error("assoc! after persistent!");};
function Hy(a,b){for(var c=Ju(Gy),d=0;;)if(d<a)c=Mu(c,b[d],b[d+1]),d+=2;else return c}function Iy(){this.Pd=!1}function Jy(a,b){return a===b?!0:Iw(a,b)?!0:L.h(a,b)}function Ky(a,b,c){a=Et(a);a[b]=c;return a}function Ly(a,b){var c=Array(a.length-2);ew(a,0,c,0,2*b);ew(a,2*(b+1),c,2*b,c.length-2*b);return c}function My(a,b,c,d){a=a.Mf(b);a.v[c]=d;return a}
function Ny(a,b,c){for(var d=a.length,e=0,g=c;;)if(e<d){c=a[e];if(null!=c){var h=a[e+1];c=b.j?b.j(g,c,h):b.call(null,g,c,h)}else c=a[e+1],c=null!=c?c.Mi(b,g):g;e+=2;g=c}else return g}function Oy(a,b,c,d){this.v=a;this.i=b;this.Oi=c;this.nd=d}Oy.prototype.advance=function(){for(var a=this.v.length;;)if(this.i<a){var b=this.v[this.i],c=this.v[this.i+1];null!=b?b=this.Oi=new V(null,2,5,X,[b,c],null):null!=c?(b=Su(c),b=b.mb()?this.nd=b:!1):b=!1;this.i+=2;if(b)return!0}else return!1};
Oy.prototype.mb=function(){var a=null!=this.Oi;return a?a:(a=null!=this.nd)?a:this.advance()};Oy.prototype.next=function(){if(null!=this.Oi){var a=this.Oi;this.Oi=null;return a}if(null!=this.nd)return a=this.nd.next(),this.nd.mb()||(this.nd=null),a;if(this.advance())return this.next();throw Error("No such element");};Oy.prototype.remove=function(){return Error("Unsupported operation")};function Py(a,b,c){this.Da=a;this.Fa=b;this.v=c}f=Py.prototype;
f.Mf=function(a){if(a===this.Da)return this;var b=xw(this.Fa),c=Array(0>b?4:2*(b+1));ew(this.v,0,c,0,2*b);return new Py(a,this.Fa,c)};f.Ii=function(){return Qy?Qy(this.v):Ry.call(null,this.v)};f.Mi=function(a,b){return Ny(this.v,a,b)};f.bf=function(a,b,c,d){var e=1<<(b>>>a&31);if(0===(this.Fa&e))return d;var g=xw(this.Fa&e-1),e=this.v[2*g],g=this.v[2*g+1];return null==e?g.bf(a+5,b,c,d):Jy(c,e)?g:d};
f.md=function(a,b,c,d,e,g){var h=1<<(c>>>b&31),k=xw(this.Fa&h-1);if(0===(this.Fa&h)){var l=xw(this.Fa);if(2*l<this.v.length){a=this.Mf(a);b=a.v;g.Pd=!0;a:for(c=2*(l-k),g=2*k+(c-1),l=2*(k+1)+(c-1);;){if(0===c)break a;b[l]=b[g];--l;--c;--g}b[2*k]=d;b[2*k+1]=e;a.Fa|=h;return a}if(16<=l){k=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];k[c>>>b&31]=Sy.md(a,b+5,c,d,e,g);for(e=d=0;;)if(32>d)0!==
(this.Fa>>>d&1)&&(k[d]=null!=this.v[e]?Sy.md(a,b+5,cv(this.v[e]),this.v[e],this.v[e+1],g):this.v[e+1],e+=2),d+=1;else break;return new Ty(a,l+1,k)}b=Array(2*(l+4));ew(this.v,0,b,0,2*k);b[2*k]=d;b[2*k+1]=e;ew(this.v,2*k,b,2*(k+1),2*(l-k));g.Pd=!0;a=this.Mf(a);a.v=b;a.Fa|=h;return a}l=this.v[2*k];h=this.v[2*k+1];if(null==l)return l=h.md(a,b+5,c,d,e,g),l===h?this:My(this,a,2*k+1,l);if(Jy(d,l))return e===h?this:My(this,a,2*k+1,e);g.Pd=!0;g=b+5;d=Uy?Uy(a,g,l,h,c,d,e):Vy.call(null,a,g,l,h,c,d,e);e=2*k;
k=2*k+1;a=this.Mf(a);a.v[e]=null;a.v[k]=d;return a};
f.ld=function(a,b,c,d,e){var g=1<<(b>>>a&31),h=xw(this.Fa&g-1);if(0===(this.Fa&g)){var k=xw(this.Fa);if(16<=k){h=[null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null];h[b>>>a&31]=Sy.ld(a+5,b,c,d,e);for(d=c=0;;)if(32>c)0!==(this.Fa>>>c&1)&&(h[c]=null!=this.v[d]?Sy.ld(a+5,cv(this.v[d]),this.v[d],this.v[d+1],e):this.v[d+1],d+=2),c+=1;else break;return new Ty(null,k+1,h)}a=Array(2*(k+1));ew(this.v,
0,a,0,2*h);a[2*h]=c;a[2*h+1]=d;ew(this.v,2*h,a,2*(h+1),2*(k-h));e.Pd=!0;return new Py(null,this.Fa|g,a)}var l=this.v[2*h],g=this.v[2*h+1];if(null==l)return k=g.ld(a+5,b,c,d,e),k===g?this:new Py(null,this.Fa,Ky(this.v,2*h+1,k));if(Jy(c,l))return d===g?this:new Py(null,this.Fa,Ky(this.v,2*h+1,d));e.Pd=!0;e=this.Fa;k=this.v;a+=5;a=Wy?Wy(a,l,g,b,c,d):Vy.call(null,a,l,g,b,c,d);c=2*h;h=2*h+1;d=Et(k);d[c]=null;d[h]=a;return new Py(null,e,d)};
f.Hi=function(a,b,c,d){var e=1<<(b>>>a&31);if(0===(this.Fa&e))return d;var g=xw(this.Fa&e-1),e=this.v[2*g],g=this.v[2*g+1];return null==e?g.Hi(a+5,b,c,d):Jy(c,e)?new V(null,2,5,X,[e,g],null):d};
f.Ji=function(a,b,c){var d=1<<(b>>>a&31);if(0===(this.Fa&d))return this;var e=xw(this.Fa&d-1),g=this.v[2*e],h=this.v[2*e+1];return null==g?(a=h.Ji(a+5,b,c),a===h?this:null!=a?new Py(null,this.Fa,Ky(this.v,2*e+1,a)):this.Fa===d?null:new Py(null,this.Fa^d,Ly(this.v,e))):Jy(c,g)?new Py(null,this.Fa^d,Ly(this.v,e)):this};f.lc=function(){return new Oy(this.v,0,null,null)};var Sy=new Py(null,0,[]);function Xy(a,b,c){this.v=a;this.i=b;this.nd=c}
Xy.prototype.mb=function(){for(var a=this.v.length;;){if(null!=this.nd&&this.nd.mb())return!0;if(this.i<a){var b=this.v[this.i];this.i+=1;null!=b&&(this.nd=Su(b))}else return!1}};Xy.prototype.next=function(){if(this.mb())return this.nd.next();throw Error("No such element");};Xy.prototype.remove=function(){return Error("Unsupported operation")};function Ty(a,b,c){this.Da=a;this.H=b;this.v=c}f=Ty.prototype;f.Mf=function(a){return a===this.Da?this:new Ty(a,this.H,Et(this.v))};
f.Ii=function(){return Yy?Yy(this.v):Zy.call(null,this.v)};f.Mi=function(a,b){for(var c=this.v.length,d=0,e=b;;)if(d<c){var g=this.v[d];null!=g&&(e=g.Mi(a,e));d+=1}else return e};f.bf=function(a,b,c,d){var e=this.v[b>>>a&31];return null!=e?e.bf(a+5,b,c,d):d};f.md=function(a,b,c,d,e,g){var h=c>>>b&31,k=this.v[h];if(null==k)return a=My(this,a,h,Sy.md(a,b+5,c,d,e,g)),a.H+=1,a;b=k.md(a,b+5,c,d,e,g);return b===k?this:My(this,a,h,b)};
f.ld=function(a,b,c,d,e){var g=b>>>a&31,h=this.v[g];if(null==h)return new Ty(null,this.H+1,Ky(this.v,g,Sy.ld(a+5,b,c,d,e)));a=h.ld(a+5,b,c,d,e);return a===h?this:new Ty(null,this.H,Ky(this.v,g,a))};f.Hi=function(a,b,c,d){var e=this.v[b>>>a&31];return null!=e?e.Hi(a+5,b,c,d):d};
f.Ji=function(a,b,c){var d=b>>>a&31,e=this.v[d];if(null!=e){a=e.Ji(a+5,b,c);if(a===e)d=this;else if(null==a)if(8>=this.H)a:{e=this.v;a=e.length;b=Array(2*(this.H-1));c=0;for(var g=1,h=0;;)if(c<a)c!==d&&null!=e[c]&&(b[g]=e[c],g+=2,h|=1<<c),c+=1;else{d=new Py(null,h,b);break a}}else d=new Ty(null,this.H-1,Ky(this.v,d,a));else d=new Ty(null,this.H,Ky(this.v,d,a));return d}return this};f.lc=function(){return new Xy(this.v,0,null)};
function $y(a,b,c){b*=2;for(var d=0;;)if(d<b){if(Jy(c,a[d]))return d;d+=2}else return-1}function az(a,b,c,d){this.Da=a;this.ie=b;this.H=c;this.v=d}f=az.prototype;f.Mf=function(a){if(a===this.Da)return this;var b=Array(2*(this.H+1));ew(this.v,0,b,0,2*this.H);return new az(a,this.ie,this.H,b)};f.Ii=function(){return Qy?Qy(this.v):Ry.call(null,this.v)};f.Mi=function(a,b){return Ny(this.v,a,b)};f.bf=function(a,b,c,d){a=$y(this.v,this.H,c);return 0>a?d:Jy(c,this.v[a])?this.v[a+1]:d};
f.md=function(a,b,c,d,e,g){if(c===this.ie){b=$y(this.v,this.H,d);if(-1===b){if(this.v.length>2*this.H)return b=2*this.H,c=2*this.H+1,a=this.Mf(a),a.v[b]=d,a.v[c]=e,g.Pd=!0,a.H+=1,a;c=this.v.length;b=Array(c+2);ew(this.v,0,b,0,c);b[c]=d;b[c+1]=e;g.Pd=!0;d=this.H+1;a===this.Da?(this.v=b,this.H=d,a=this):a=new az(this.Da,this.ie,d,b);return a}return this.v[b+1]===e?this:My(this,a,b+1,e)}return(new Py(a,1<<(this.ie>>>b&31),[null,this,null,null])).md(a,b,c,d,e,g)};
f.ld=function(a,b,c,d,e){return b===this.ie?(a=$y(this.v,this.H,c),-1===a?(a=2*this.H,b=Array(a+2),ew(this.v,0,b,0,a),b[a]=c,b[a+1]=d,e.Pd=!0,new az(null,this.ie,this.H+1,b)):L.h(this.v[a+1],d)?this:new az(null,this.ie,this.H,Ky(this.v,a+1,d))):(new Py(null,1<<(this.ie>>>a&31),[null,this])).ld(a,b,c,d,e)};f.Hi=function(a,b,c,d){a=$y(this.v,this.H,c);return 0>a?d:Jy(c,this.v[a])?new V(null,2,5,X,[this.v[a],this.v[a+1]],null):d};
f.Ji=function(a,b,c){a=$y(this.v,this.H,c);return-1===a?this:1===this.H?null:new az(null,this.ie,this.H-1,Ly(this.v,ww(a)))};f.lc=function(){return new Oy(this.v,0,null,null)};
function Vy(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 6:return Wy(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5]);case 7:return Uy(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],arguments[5],arguments[6]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}
function Wy(a,b,c,d,e,g){var h=cv(b);if(h===d)return new az(null,h,2,[b,c,e,g]);var k=new Iy;return Sy.ld(a,h,b,c,k).ld(a,d,e,g,k)}function Uy(a,b,c,d,e,g,h){var k=cv(c);if(k===e)return new az(null,k,2,[c,d,g,h]);var l=new Iy;return Sy.md(a,b,k,c,d,l).md(a,b,e,g,h,l)}function bz(a,b,c,d,e){this.meta=a;this.nodes=b;this.i=c;this.s=d;this.M=e;this.C=32374860;this.Z=0}f=bz.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return Gv(b,this)};
f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return null==this.s?new V(null,2,5,X,[this.nodes[this.i],this.nodes[this.i+1]],null):I(this.s)};f.Db=function(){var a=this,b=null==a.s?function(){var b=a.nodes,d=a.i+2;return cz?cz(b,d,null):Ry.call(null,b,d,null)}():function(){var b=a.nodes,d=a.i,e=K(a.s);return cz?cz(b,d,e):Ry.call(null,b,d,e)}();return null!=b?b:iv};f.oa=function(){return this};f.za=function(a,b){return new bz(b,this.nodes,this.i,this.s,this.M)};
f.xa=function(a,b){return Ev(b,this)};bz.prototype[Dt]=function(){return kv(this)};function Ry(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return Qy(arguments[0]);case 3:return cz(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}function Qy(a){return cz(a,0,null)}
function cz(a,b,c){if(null==c)for(c=a.length;;)if(b<c){if(null!=a[b])return new bz(null,a,b,null,null);var d=a[b+1];if(z(d)&&(d=d.Ii(),z(d)))return new bz(null,a,b+2,d,null);b+=2}else return null;else return new bz(null,a,b,c,null)}function dz(a,b,c,d,e){this.meta=a;this.nodes=b;this.i=c;this.s=d;this.M=e;this.C=32374860;this.Z=0}f=dz.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.meta};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return Gv(b,this)};
f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return I(this.s)};f.Db=function(){var a=this.nodes;var b=this.i,c=K(this.s);a=ez?ez(null,a,b,c):Zy.call(null,null,a,b,c);return null!=a?a:iv};f.oa=function(){return this};f.za=function(a,b){return new dz(b,this.nodes,this.i,this.s,this.M)};f.xa=function(a,b){return Ev(b,this)};dz.prototype[Dt]=function(){return kv(this)};
function Zy(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;switch(b.length){case 1:return Yy(arguments[0]);case 4:return ez(arguments[0],arguments[1],arguments[2],arguments[3]);default:throw Error([C.c("Invalid arity: "),C.c(b.length)].join(""));}}function Yy(a){return ez(null,a,0,null)}
function ez(a,b,c,d){if(null==d)for(d=b.length;;)if(c<d){var e=b[c];if(z(e)&&(e=e.Ii(),z(e)))return new dz(a,b,c+1,e,null);c+=1}else return null;else return new dz(a,b,c,d,null)}function fz(a,b,c){this.Lb=a;this.ft=b;this.xp=c}fz.prototype.mb=function(){return!this.xp||this.ft.mb()};fz.prototype.next=function(){if(this.xp)return this.ft.next();this.xp=!0;return new V(null,2,5,X,[null,this.Lb],null)};fz.prototype.remove=function(){return Error("Unsupported operation")};
function gz(a,b,c,d,e,g){this.meta=a;this.H=b;this.root=c;this.Kb=d;this.Lb=e;this.M=g;this.C=16123663;this.Z=8196}f=gz.prototype;f.Lj=x;f.Wh=function(a,b){return this.Kb?new V(null,2,5,X,[null,this.Lb],null):this.root.Hi(0,cv(b),b,null)};f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};f.keys=function(){return kv(Cy.c?Cy.c(this):Cy.call(null,this))};f.entries=function(){return new xy(G(G(this)))};f.values=function(){return kv(Dy.c?Dy.c(this):Dy.call(null,this))};
f.has=function(a){return lw(this,a)};f.get=function(a,b){return this.Y(null,a,b)};f.forEach=function(a){for(var b,c,d=G(this),e=null,g=0,h=0;;)if(h<g)b=e.aa(null,h),c=S(b,0,null),b=S(b,1,null),a.h?a.h(b,c):a.call(null,b,c),h+=1;else if(c=G(d))d=c,cw(d)?(e=Ou(d),d=Pu(d),c=e,b=Q(e),e=c,g=b):(e=I(d),c=S(e,0,null),b=S(e,1,null),a.h?a.h(b,c):a.call(null,b,c),d=K(d),e=null,g=0),h=0;else return null};f.na=function(a,b){return this.Y(null,b,null)};
f.Y=function(a,b,c){return null==b?this.Kb?this.Lb:c:null==this.root?c:this.root.bf(0,cv(b),b,c)};f.Kg=function(a,b,c){a=this.Kb?b.j?b.j(c,null,this.Lb):b.call(null,c,null,this.Lb):c;return null!=this.root?this.root.Mi(b,a):a};f.lc=function(){var a=this.root?Su(this.root):ax();return this.Kb?new fz(this.Lb,a,!1):a};f.sa=function(){return this.meta};f.ya=function(){return this.H};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=ov(this)};f.P=function(a,b){return vy(this,b)};
f.Jg=function(){return new hz({},this.root,this.H,this.Kb,this.Lb)};f.Ka=function(){return ru(Gy,this.meta)};f.Df=function(a,b){if(null==b)return this.Kb?new gz(this.meta,this.H-1,this.root,!1,null,null):this;if(null==this.root)return this;var c=this.root.Ji(0,cv(b),b);return c===this.root?this:new gz(this.meta,this.H-1,c,this.Kb,this.Lb,null)};
f.Rb=function(a,b,c){if(null==b)return this.Kb&&c===this.Lb?this:new gz(this.meta,this.Kb?this.H:this.H+1,this.root,!0,c,null);a=new Iy;b=(null==this.root?Sy:this.root).ld(0,cv(b),b,c,a);return b===this.root?this:new gz(this.meta,a.Pd?this.H+1:this.H,b,this.Kb,this.Lb,null)};f.Vh=function(a,b){return null==b?this.Kb:null==this.root?!1:this.root.bf(0,cv(b),b,fw)!==fw};f.oa=function(){if(0<this.H){var a=null!=this.root?this.root.Ii():null;return this.Kb?Ev(new V(null,2,5,X,[null,this.Lb],null),a):a}return null};
f.za=function(a,b){return new gz(b,this.H,this.root,this.Kb,this.Lb,this.M)};f.xa=function(a,b){if(bw(b))return this.Rb(null,Qt.h(b,0),Qt.h(b,1));for(var c=this,d=G(b);;){if(null==d)return c;var e=I(d);if(bw(e))c=c.Rb(null,Qt.h(e,0),Qt.h(e,1)),d=K(d);else throw Error("conj on a map takes map entries or seqables of map entries");}};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.Y(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.Y(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.Y(null,a,b)};var Gy=new gz(null,0,null,!1,null,pv);gz.prototype[Dt]=function(){return kv(this)};
function hz(a,b,c,d,e){this.Da=a;this.root=b;this.count=c;this.Kb=d;this.Lb=e;this.C=258;this.Z=56}function iz(a,b,c){if(a.Da){if(null==b)a.Lb!==c&&(a.Lb=c),a.Kb||(a.count+=1,a.Kb=!0);else{var d=new Iy;b=(null==a.root?Sy:a.root).md(a.Da,0,cv(b),b,c,d);b!==a.root&&(a.root=b);d.Pd&&(a.count+=1)}return a}throw Error("assoc! after persistent!");}f=hz.prototype;f.ya=function(){if(this.Da)return this.count;throw Error("count after persistent!");};
f.na=function(a,b){return null==b?this.Kb?this.Lb:null:null==this.root?null:this.root.bf(0,cv(b),b)};f.Y=function(a,b,c){return null==b?this.Kb?this.Lb:c:null==this.root?c:this.root.bf(0,cv(b),b,c)};
f.Ff=function(a,b){a:if(this.Da)if(null!=b?b.C&2048||x===b.kv||(b.C?0:B(du,b)):B(du,b))var c=iz(this,Aw.c?Aw.c(b):Aw.call(null,b),Bw.c?Bw.c(b):Bw.call(null,b));else{c=G(b);for(var d=this;;){var e=I(c);if(z(e))c=K(c),d=iz(d,Aw.c?Aw.c(e):Aw.call(null,e),Bw.c?Bw.c(e):Bw.call(null,e));else{c=d;break a}}}else throw Error("conj! after persistent");return c};
f.Xh=function(){if(this.Da){this.Da=null;var a=new gz(null,this.count,this.root,this.Kb,this.Lb,null)}else throw Error("persistent! called twice");return a};f.Ef=function(a,b,c){return iz(this,b,c)};var px=function px(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return px.w(0<c.length?new H(c.slice(0),0,null):null)};px.w=function(a){for(var b=G(a),c=Ju(Gy);;)if(b){a=K(K(b));var d=I(b),b=Jv(b),c=Mu(c,d,b),b=a}else return Lu(c)};px.L=0;px.O=function(a){return px.w(G(a))};
var jz=function jz(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return jz.w(0<c.length?new H(c.slice(0),0,null):null)};jz.w=function(a){a=a instanceof H&&0===a.i?a.v:Ft(a);return Ov(a)};jz.L=0;jz.O=function(a){return jz.w(G(a))};function kz(a,b){this.ia=a;this.gc=b;this.C=32374988;this.Z=0}f=kz.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.gc};f.bc=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.Nj||(this.ia.C?0:B(Ut,this.ia)):B(Ut,this.ia))?this.ia.bc(null):K(this.ia);return null==a?null:new kz(a,this.gc)};f.ha=function(){return mv(this)};
f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.gc)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return this.ia.hb(null).Km()};f.Db=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.Nj||(this.ia.C?0:B(Ut,this.ia)):B(Ut,this.ia))?this.ia.bc(null):K(this.ia);return null!=a?new kz(a,this.gc):iv};f.oa=function(){return this};f.za=function(a,b){return new kz(this.ia,b)};f.xa=function(a,b){return Ev(b,this)};
kz.prototype[Dt]=function(){return kv(this)};function Cy(a){return(a=G(a))?new kz(a,null):null}function Aw(a){return eu(a)}function lz(a,b){this.ia=a;this.gc=b;this.C=32374988;this.Z=0}f=lz.prototype;f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};
f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.sa=function(){return this.gc};f.bc=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.Nj||(this.ia.C?0:B(Ut,this.ia)):B(Ut,this.ia))?this.ia.bc(null):K(this.ia);return null==a?null:new lz(a,this.gc)};f.ha=function(){return mv(this)};
f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.gc)};f.Bb=function(a,b){return Gv(b,this)};f.Cb=function(a,b,c){return Iv(b,c,this)};f.hb=function(){return this.ia.hb(null).Lm()};f.Db=function(){var a=(null!=this.ia?this.ia.C&128||x===this.ia.Nj||(this.ia.C?0:B(Ut,this.ia)):B(Ut,this.ia))?this.ia.bc(null):K(this.ia);return null!=a?new lz(a,this.gc):iv};f.oa=function(){return this};f.za=function(a,b){return new lz(this.ia,b)};f.xa=function(a,b){return Ev(b,this)};
lz.prototype[Dt]=function(){return kv(this)};function Dy(a){return(a=G(a))?new lz(a,null):null}function Bw(a){return fu(a)}var mz=function mz(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return mz.w(0<c.length?new H(c.slice(0),0,null):null)};mz.w=function(a){return z(ex(pw,a))?nw(function(a,c){return Lv.h(z(a)?a:cx,c)},a):null};mz.L=0;mz.O=function(a){return mz.w(G(a))};
function nz(a){for(var b=cx,c=G(new V(null,3,5,X,[oz,pz,qz],null));;)if(c)var d=I(c),e=F.j(a,d,rz),b=Zw.h(e,rz)?T.j(b,d,e):b,c=K(c);else return ru(b,Tv(a))}function sz(a){this.Eo=a}sz.prototype.mb=function(){return this.Eo.mb()};sz.prototype.next=function(){if(this.Eo.mb())return this.Eo.next().Ob[0];throw Error("No such element");};sz.prototype.remove=function(){return Error("Unsupported operation")};function tz(a,b,c){this.meta=a;this.$e=b;this.M=c;this.C=15077647;this.Z=8196}f=tz.prototype;
f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};f.keys=function(){return kv(G(this))};f.entries=function(){return new yy(G(G(this)))};f.values=function(){return kv(G(this))};f.has=function(a){return lw(this,a)};
f.forEach=function(a){for(var b,c,d=G(this),e=null,g=0,h=0;;)if(h<g)b=e.aa(null,h),c=S(b,0,null),b=S(b,1,null),a.h?a.h(b,c):a.call(null,b,c),h+=1;else if(c=G(d))d=c,cw(d)?(e=Ou(d),d=Pu(d),c=e,b=Q(e),e=c,g=b):(e=I(d),c=S(e,0,null),b=S(e,1,null),a.h?a.h(b,c):a.call(null,b,c),d=K(d),e=null,g=0),h=0;else return null};f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){return Yt(this.$e,b)?b:c};f.lc=function(){return new sz(Su(this.$e))};f.sa=function(){return this.meta};f.ya=function(){return Lt(this.$e)};
f.ha=function(){var a=this.M;return null!=a?a:this.M=a=ov(this)};f.P=function(a,b){return Yv(b)&&Q(this)===Q(b)&&dx(function(a){return function(b){return lw(a,b)}}(this),b)};f.Jg=function(){return new uz(Ju(this.$e))};f.Ka=function(){return ru(vz,this.meta)};f.Mm=function(a,b){return new tz(this.meta,cu(this.$e,b),null)};f.oa=function(){return Cy(this.$e)};f.za=function(a,b){return new tz(b,this.$e,this.M)};f.xa=function(a,b){return new tz(this.meta,T.j(this.$e,b,null),null)};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.Y(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.Y(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.Y(null,a,b)};var vz=new tz(null,cx,pv);
function wz(a){for(var b=a.length,c=Ju(vz),d=0;;)if(d<b)Ku(c,a[d]),d+=1;else break;return Lu(c)}tz.prototype[Dt]=function(){return kv(this)};function uz(a){this.Ke=a;this.Z=136;this.C=259}f=uz.prototype;f.Ff=function(a,b){this.Ke=Mu(this.Ke,b,null);return this};f.Xh=function(){return new tz(null,Lu(this.Ke),null)};f.ya=function(){return Q(this.Ke)};f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){return Wt.j(this.Ke,b,fw)===fw?c:b};
f.call=function(){function a(a,b,c){return Wt.j(this.Ke,b,fw)===fw?c:b}function b(a,b){return Wt.j(this.Ke,b,fw)===fw?null:b}var c=null,c=function(c,e,g){switch(arguments.length){case 2:return b.call(this,0,e);case 3:return a.call(this,0,e,g)}throw Error("Invalid arity: "+(arguments.length-1));};c.h=b;c.j=a;return c}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return Wt.j(this.Ke,a,fw)===fw?null:a};f.h=function(a,b){return Wt.j(this.Ke,a,fw)===fw?b:a};
function xz(a){a=G(a);if(null==a)return vz;if(a instanceof H&&0===a.i)return wz(a.v);for(var b=Ju(vz);;)if(null!=a){var c=K(a),b=b.Ff(null,a.hb(null));a=c}else return Lu(b)}function yz(a){for(var b=Mv;;)if(K(a))b=Lv.h(b,I(a)),a=K(a);else return G(b)}function Kw(a){if(null!=a&&(a.Z&4096||x===a.jr))return a.name;if("string"===typeof a)return a;throw Error([C.c("Doesn't support name: "),C.c(a)].join(""));}
var zz=function zz(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return zz.h(arguments[0],arguments[1]);case 3:return zz.j(arguments[0],arguments[1],arguments[2]);default:return zz.w(arguments[0],arguments[1],arguments[2],new H(c.slice(3),0,null))}};zz.h=function(a,b){return b};zz.j=function(a,b,c){return(a.c?a.c(b):a.call(null,b))>(a.c?a.c(c):a.call(null,c))?b:c};
zz.w=function(a,b,c,d){return Gt(function(b,c){return zz.j(a,b,c)},zz.j(a,b,c),d)};zz.O=function(a){var b=I(a),c=K(a);a=I(c);var d=K(c),c=I(d),d=K(d);return zz.w(b,a,c,d)};zz.L=3;function Az(a,b,c){this.i=a;this.end=b;this.step=c}Az.prototype.mb=function(){return 0<this.step?this.i<this.end:this.i>this.end};Az.prototype.next=function(){var a=this.i;this.i+=this.step;return a};function Bz(a,b,c,d,e){this.meta=a;this.start=b;this.end=c;this.step=d;this.M=e;this.C=32375006;this.Z=8192}f=Bz.prototype;
f.toString=function(){return Uu(this)};f.equiv=function(a){return this.P(null,a)};f.indexOf=function(){var a=null,a=function(a,c){switch(arguments.length){case 1:return P(this,a,0);case 2:return P(this,a,c)}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(a){return P(this,a,0)};a.h=function(a,c){return P(this,a,c)};return a}();
f.lastIndexOf=function(){function a(a){return Av(this,a,Q(this))}var b=null,b=function(b,d){switch(arguments.length){case 1:return a.call(this,b);case 2:return Av(this,b,d)}throw Error("Invalid arity: "+(arguments.length-1));};b.c=a;b.h=function(a,b){return Av(this,a,b)};return b}();f.aa=function(a,b){if(b<this.ya(null))return this.start+b*this.step;if(this.start>this.end&&0===this.step)return this.start;throw Error("Index out of bounds");};
f.ac=function(a,b,c){return b<this.ya(null)?this.start+b*this.step:this.start>this.end&&0===this.step?this.start:c};f.lc=function(){return new Az(this.start,this.end,this.step)};f.sa=function(){return this.meta};f.bc=function(){return 0<this.step?this.start+this.step<this.end?new Bz(this.meta,this.start+this.step,this.end,this.step,null):null:this.start+this.step>this.end?new Bz(this.meta,this.start+this.step,this.end,this.step,null):null};
f.ya=function(){return At(this.oa(null))?0:Math.ceil((this.end-this.start)/this.step)};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=mv(this)};f.P=function(a,b){return Dv(this,b)};f.Ka=function(){return ru(iv,this.meta)};f.Bb=function(a,b){return rv(this,b)};f.Cb=function(a,b,c){for(a=this.start;;)if(0<this.step?a<this.end:a>this.end)c=b.h?b.h(c,a):b.call(null,c,a),a+=this.step;else return c};f.hb=function(){return null==this.oa(null)?null:this.start};
f.Db=function(){return null!=this.oa(null)?new Bz(this.meta,this.start+this.step,this.end,this.step,null):iv};f.oa=function(){return 0<this.step?this.start<this.end?this:null:0>this.step?this.start>this.end?this:null:this.start===this.end?null:this};f.za=function(a,b){return new Bz(b,this.start,this.end,this.step,this.M)};f.xa=function(a,b){return Ev(b,this)};Bz.prototype[Dt]=function(){return kv(this)};function Cz(a){a:for(var b=a;;)if(G(b))b=K(b);else break a;return a}
function Dz(a,b){if("string"===typeof b){var c=a.exec(b);return L.h(I(c),b)?1===Q(c)?I(c):fy(c):null}throw new TypeError("re-matches must match against a string.");}function Ez(a,b){if("string"===typeof b){var c=a.exec(b);return null==c?null:1===Q(c)?I(c):fy(c)}throw new TypeError("re-find must match against a string.");}function Fz(a){if(a instanceof RegExp)return a;var b=Ez(/^\(\?([idmsux]*)\)/,a),c=S(b,0,null),b=S(b,1,null),c=Q(c);return new RegExp(a.substring(c),z(b)?b:"")}
function Gz(a,b,c,d,e,g,h){var k=qt;qt=null==qt?null:qt-1;try{if(null!=qt&&0>qt)return Eu(a,"#");Eu(a,c);if(0===xt.c(g))G(h)&&Eu(a,function(){var a=Hz.c(g);return z(a)?a:"..."}());else{if(G(h)){var l=I(h);b.j?b.j(l,a,g):b.call(null,l,a,g)}for(var m=K(h),p=xt.c(g)-1;;)if(!m||null!=p&&0===p){G(m)&&0===p&&(Eu(a,d),Eu(a,function(){var a=Hz.c(g);return z(a)?a:"..."}()));break}else{Eu(a,d);var u=I(m);c=a;h=g;b.j?b.j(u,c,h):b.call(null,u,c,h);var A=K(m);c=p-1;m=A;p=c}}return Eu(a,e)}finally{qt=k}}
function Jz(a,b){for(var c,d=G(b),e=null,g=0,h=0;;)if(h<g)c=e.aa(null,h),Eu(a,c),h+=1;else if(d=G(d))e=d,cw(e)?(d=Ou(e),e=Pu(e),c=d,g=Q(d),d=e,e=c):(c=I(e),Eu(a,c),d=K(e),e=null,g=0),h=0;else return null}var Kz={'"':'\\"',"\\":"\\\\","\b":"\\b","\f":"\\f","\n":"\\n","\r":"\\r","\t":"\\t"};function Lz(a){return[C.c('"'),C.c(a.replace(RegExp('[\\\\"\b\f\n\r\t]',"g"),function(a){return Kz[a]})),C.c('"')].join("")}
function Mz(a,b){var c=iw(F.h(a,vt));return c?(c=null!=b?b.C&131072||x===b.lv?!0:!1:!1)?null!=Tv(b):c:c}
function Nz(a,b,c){if(null==a)return Eu(b,"nil");if(Mz(c,a)){Eu(b,"^");var d=Tv(a);Oz.j?Oz.j(d,b,c):Oz.call(null,d,b,c);Eu(b," ")}if(a.lr)return a.qv(b);if(null!=a&&(a.C&2147483648||x===a.Ta))return a.ja(null,b,c);if(!0===a||!1===a||"number"===typeof a)return Eu(b,""+C.c(a));if(null!=a&&a.constructor===Object)return Eu(b,"#js "),d=Z.h(function(b){return new V(null,2,5,X,[Jw.c(b),a[b]],null)},dw(a)),Pz.N?Pz.N(d,Oz,b,c):Pz.call(null,d,Oz,b,c);if(zt(a))return Gz(b,Oz,"#js ["," ","]",c,a);if(ea(a))return z(ut.c(c))?
Eu(b,Lz(a)):Eu(b,a);if(na(a)){var e=a.name;c=z(function(){var a=null==e;return a?a:Aa(e)}())?"Function":e;return Jz(b,R(["#object[",c,' "',""+C.c(a),'"]'],0))}if(a instanceof Date)return c=function(a,b){for(var c=""+C.c(a);;)if(Q(c)<b)c=[C.c("0"),C.c(c)].join("");else return c},Jz(b,R(['#inst "',""+C.c(a.getUTCFullYear()),"-",c(a.getUTCMonth()+1,2),"-",c(a.getUTCDate(),2),"T",c(a.getUTCHours(),2),":",c(a.getUTCMinutes(),2),":",c(a.getUTCSeconds(),2),".",c(a.getUTCMilliseconds(),3),"-",'00:00"'],0));
if(a instanceof RegExp)return Jz(b,R(['#"',a.source,'"'],0));if(z(a.constructor.Sj))return Jz(b,R(["#object[",a.constructor.Sj.replace(RegExp("/","g"),"."),"]"],0));e=a.constructor.name;c=z(function(){var a=null==e;return a?a:Aa(e)}())?"Object":e;return Jz(b,R(["#object[",c," ",""+C.c(a),"]"],0))}function Oz(a,b,c){var d=Qz.c(c);return z(d)?(c=T.j(c,Rz,Nz),d.j?d.j(a,b,c):d.call(null,a,b,c)):Nz(a,b,c)}
function Sz(a,b){var c=new Xs;a:{var d=new Tu(c);Oz(I(a),d,b);for(var e=G(K(a)),g=null,h=0,k=0;;)if(k<h){var l=g.aa(null,k);Eu(d," ");Oz(l,d,b);k+=1}else if(e=G(e))g=e,cw(g)?(e=Ou(g),g=Pu(g),l=e,h=Q(e),e=g,g=l):(l=I(g),Eu(d," "),Oz(l,d,b),e=K(g),g=null,h=0),k=0;else break a}return c}function Tz(a,b){return Wv(a)?"":""+C.c(Sz(a,b))}function Uz(a){var b=T.j(st(),ut,!1);a=Tz(a,b);ot.c?ot.c(a):ot.call(null);a=st();ot.c?ot.c("\n"):ot.call(null);return F.h(a,tt),null}
function Vz(a,b,c,d,e){return Gz(d,function(a,b,d){var e=eu(a);c.j?c.j(e,b,d):c.call(null,e,b,d);Eu(b," ");a=fu(a);return c.j?c.j(a,b,d):c.call(null,a,b,d)},[C.c(a),C.c("{")].join(""),", ","}",e,G(b))}function Pz(a,b,c,d){var e=S(null,0,null),g=S(null,1,null);return z(e)?Vz([C.c("#:"),C.c(e)].join(""),g,b,c,d):Vz(null,a,b,c,d)}H.prototype.Ta=x;H.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};Lw.prototype.Ta=x;Lw.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};
bz.prototype.Ta=x;bz.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};Ay.prototype.Ta=x;Ay.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};hy.prototype.Ta=x;hy.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};Hw.prototype.Ta=x;Hw.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};Cv.prototype.Ta=x;Cv.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};gz.prototype.Ta=x;gz.prototype.ja=function(a,b,c){return Pz(this,Oz,b,c)};
dz.prototype.Ta=x;dz.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};ky.prototype.Ta=x;ky.prototype.ja=function(a,b,c){return Gz(b,Oz,"["," ","]",c,this)};tz.prototype.Ta=x;tz.prototype.ja=function(a,b,c){return Gz(b,Oz,"#{"," ","}",c,this)};Qw.prototype.Ta=x;Qw.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};mx.prototype.Ta=x;mx.prototype.ja=function(a,b,c){Eu(b,"#object [cljs.core.Atom ");Oz(new y(null,1,[Wz,this.state],null),b,c);return Eu(b,"]")};
lz.prototype.Ta=x;lz.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};V.prototype.Ta=x;V.prototype.ja=function(a,b,c){return Gz(b,Oz,"["," ","]",c,this)};qy.prototype.Ta=x;qy.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};Ew.prototype.Ta=x;Ew.prototype.ja=function(a,b){return Eu(b,"()")};ry.prototype.Ta=x;ry.prototype.ja=function(a,b,c){return Gz(b,Oz,"#queue ["," ","]",c,G(this))};y.prototype.Ta=x;y.prototype.ja=function(a,b,c){return Pz(this,Oz,b,c)};
Bz.prototype.Ta=x;Bz.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};kz.prototype.Ta=x;kz.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};Cw.prototype.Ta=x;Cw.prototype.ja=function(a,b,c){return Gz(b,Oz,"("," ",")",c,this)};var Xz=null;function Yz(){null==Xz&&(Xz=ox?ox(0):nx.call(null,0));return gv.c([C.c("reagent"),C.c(sx.h(Xz,qv))].join(""))}function Zz(){}
var $z=function $z(b){if(null!=b&&null!=b.gv)return b.gv(b);var c=$z[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=$z._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IEncodeJS.-clj-\x3ejs",b);};function aA(a){return(null!=a?x===a.fv||(a.Tj?0:B(Zz,a)):B(Zz,a))?$z(a):"string"===typeof a||"number"===typeof a||a instanceof Y||a instanceof fv?bA.c?bA.c(a):bA.call(null,a):Tz(R([a],0),st())}
var bA=function bA(b){if(null==b)return null;if(null!=b?x===b.fv||(b.Tj?0:B(Zz,b)):B(Zz,b))return $z(b);if(b instanceof Y)return Kw(b);if(b instanceof fv)return""+C.c(b);if($v(b)){var c={};b=G(b);for(var d=null,e=0,g=0;;)if(g<e){var h=d.aa(null,g),k=S(h,0,null),h=S(h,1,null);c[aA(k)]=bA.c?bA.c(h):bA.call(null,h);g+=1}else if(b=G(b))cw(b)?(e=Ou(b),b=Pu(b),d=e,e=Q(e)):(e=I(b),d=S(e,0,null),e=S(e,1,null),c[aA(d)]=bA.c?bA.c(e):bA.call(null,e),b=K(b),d=null,e=0),g=0;else break;return c}if(Xv(b)){c=[];
b=G(Z.h(bA,b));d=null;for(g=e=0;;)if(g<e)k=d.aa(null,g),c.push(k),g+=1;else if(b=G(b))d=b,cw(d)?(b=Ou(d),g=Pu(d),d=b,e=Q(b),b=g):(b=I(d),c.push(b),b=K(d),d=null,e=0),g=0;else break;return c}return b};function cA(){}var dA=function dA(b,c){if(null!=b&&null!=b.ev)return b.ev(b,c);var d=dA[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=dA._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IEncodeClojure.-js-\x3eclj",b);};
function eA(a,b){var c=null!=b&&(b.C&64||x===b.Sb)?U.h(px,b):b,d=F.h(c,fA);return function(a,c,d,k){return function m(e){return(null!=e?x===e.HA||(e.Tj?0:B(cA,e)):B(cA,e))?dA(e,U.h(jz,b)):hw(e)?Cz(Z.h(m,e)):Xv(e)?jx.h(null==e?null:Mt(e),Z.h(m,e)):zt(e)?fy(Z.h(m,e)):(null==e?null:e.constructor)===Object?jx.h(cx,function(){return function(a,b,c,d){return function M(g){return new Lw(null,function(a,b,c,d){return function(){for(;;){var a=G(g);if(a){if(cw(a)){var b=Ou(a),c=Q(b),h=Pw(c);a:for(var k=0;;)if(k<
c){var p=Qt.h(b,k),p=new V(null,2,5,X,[d.c?d.c(p):d.call(null,p),m(e[p])],null);h.add(p);k+=1}else{b=!0;break a}return b?Rw(h.sb(),M(Pu(a))):Rw(h.sb(),null)}h=I(a);return Ev(new V(null,2,5,X,[d.c?d.c(h):d.call(null,h),m(e[h])],null),M(hv(a)))}return null}}}(a,b,c,d),null,null)}}(a,c,d,k)(dw(e))}()):e}}(b,c,d,z(d)?Jw:C)(a)}function gA(a,b){this.vf=a;this.M=b;this.C=2153775104;this.Z=2048}f=gA.prototype;f.toString=function(){return this.vf};f.equiv=function(a){return this.P(null,a)};
f.P=function(a,b){return b instanceof gA&&this.vf===b.vf};f.ja=function(a,b){return Eu(b,[C.c('#uuid "'),C.c(this.vf),C.c('"')].join(""))};f.ha=function(){null==this.M&&(this.M=cv(this.vf));return this.M};function hA(a,b,c){var d=Error(a);this.message=a;this.data=b;this.Xq=c;this.name=d.name;this.description=d.description;this.number=d.number;this.fileName=d.fileName;this.lineNumber=d.lineNumber;this.columnNumber=d.columnNumber;this.stack=d.stack;return this}hA.prototype.__proto__=Error.prototype;
hA.prototype.Ta=x;hA.prototype.ja=function(a,b,c){Eu(b,"#error {:message ");Oz(this.message,b,c);z(this.data)&&(Eu(b,", :data "),Oz(this.data,b,c));z(this.Xq)&&(Eu(b,", :cause "),Oz(this.Xq,b,c));return Eu(b,"}")};hA.prototype.toString=function(){return Uu(this)};function iA(a,b){return new hA(a,b,null)};var jA=new Y(null,"response","response",-1068424192),kA=new Y(null,"tick","tick",-835886976),lA=new fv(null,"localTimeParser","localTimeParser",-1738135328,null),mA=new Y(null,"description","description",-1428560544),nA=new Y(null,"unnamed","unnamed",-26044928),oA=new Y(null,"date-element-parser","date-element-parser",2072167040),pA=new Y(null,"auth-token","auth-token",30990976),qA=new Y(null,"add-event","add-event",938429088),rA=new Y(null,"hour-minute","hour-minute",-1164421312),sA=new Y("common",
"default-response","common/default-response",107834336),tA=new Y(null,"yield","yield",177875009),uA=new Y(null,"ordinal-suffix","ordinal-suffix",-1311843199),vA=new Y(null,"labels","labels",-626734591),wA=new Y(null,"paused","paused",-1710376127),xA=new Y(null,"non-field-errors","non-field-errors",882146145),yA=new Y(null,"errors","errors",-908790718),zA=new Y(null,"month-name","month-name",-605509534),AA=new Y(null,"formatters","formatters",-1875637118),BA=new Y(null,"on-set","on-set",-140953470),
CA=new Y(null,"day","day",-274800446),DA=new Y(null,"format","format",-1306924766),EA=new Y("flash","messages","flash/messages",247896418),FA=new Y(null,"t-time","t-time",-42016318),GA=new fv(null,"timeParser","timeParser",1585048034,null),HA=new Y(null,"basic-ordinal-date","basic-ordinal-date",243220162),IA=new Y(null,"date","date",-1463434462),JA=new Y(null,"hour","hour",-555989214),KA=new Y(null,"timezone","timezone",1831928099),LA=new Y(null,"long","long",-171452093),MA=new Y(null,"cljsLegacyRender",
"cljsLegacyRender",-1527295613),bx=new fv(null,"meta14186","meta14186",718375395,null),NA=new Y("common","on-load","common/on-load",-2052806173),OA=new Y(null,"time-no-ms","time-no-ms",870271683),PA=new Y(null,"weekyear-week-day","weekyear-week-day",-740233533),QA=new Y(null,"overrides","overrides",1738628867),RA=new Y(null,"idle","idle",-2007156861),SA=new Y(null,"where","where",-2044795965),TA=new Y(null,"week-date-time","week-date-time",540228836),UA=new Y(null,"date-hour-minute-second-fraction",
"date-hour-minute-second-fraction",1937143076),VA=new Y(null,"eventCategory","eventCategory",2015922692),WA=new Y(null,"group","group",582596132),vt=new Y(null,"meta","meta",1499536964),XA=new Y(null,"full","full",436801220),YA=new Y(null,"basic-date-time","basic-date-time",1525413604),ZA=new Y(null,"date-time","date-time",177938180),$A=new Y(null,"basic-time-no-ms","basic-time-no-ms",-1720654076),aB=new Y(null,"dddd","dddd",217016228),bB=new Y(null,"date-parser","date-parser",-981534587),wt=new Y(null,
"dup","dup",556298533),cB=new Y(null,"loading-count","loading-count",-1446402651),dB=new Y(null,"basic-week-date","basic-week-date",1775847845),eB=new Y(null,"key","key",-1516042587),fB=new Y(null,"fsm-state","fsm-state",1656310533),gB=new fv(null,"dateOptionalTimeParser","dateOptionalTimeParser",1783230854,null),hB=new Y(null,"parse-quoted","parse-quoted",1180570118),iB=new Y(null,"int","int",-1741416922),jB=new Y(null,"basic-t-time-no-ms","basic-t-time-no-ms",-424650106),kB=new Y(null,"local-time",
"local-time",-1873195290),lB=new Y(null,"\x3c-","\x3c-",760412998),mB=new Y(null,"date-time-no-ms","date-time-no-ms",1655953671),nB=new Y(null,"year-month-day","year-month-day",-415594169),oB=new Y(null,"db","db",993250759),pB=new Y(null,"sub","sub",-2093760025),qB=new Y(null,"date-opt-time","date-opt-time",-1507102105),rB=new Y(null,"eventLabel","eventLabel",-31846777),sB=new Y("re-frame.std-interceptors","not-found","re-frame.std-interceptors/not-found",-1614827865),Mx=new Y(null,"queue","queue",
1455835879),tB=new Y(null,"displayName","displayName",-809144601),uB=new Y("common","handle-default-response","common/handle-default-response",980613895),vB=new Y(null,"rfc822","rfc822",-404628697),qx=new Y(null,"validator","validator",-1966190681),wB=new Y(null,"eventAction","eventAction",1999627175),xB=new Y(null,"sequential","sequential",-1082983960),yB=new Y(null,"illegal-pattern","illegal-pattern",-1810990520),zB=new Y(null,"warn","warn",-436710552),AB=new Y(null,"columns","columns",1998437288),
BB=new Y("auth","update-expires-in","auth/update-expires-in",1932168168),CB=new Y(null,"date-hour-minute-second-ms","date-hour-minute-second-ms",-425334775),DB=new Y(null,"name","name",1843675177),EB=new Y(null,"basic-ordinal-date-time","basic-ordinal-date-time",1054564521),FB=new fv(null,"timeElementParser","timeElementParser",302132553,null),GB=new Y(null,"ordinal-date","ordinal-date",-77899447),HB=new Y(null,"meridiem","meridiem",1668960617),IB=new Y(null,"change-points","change-points",-349409815),
JB=new Y(null,"hour-minute-second-fraction","hour-minute-second-fraction",-1253038551),KB=new Y("form","finish-progress","form/finish-progress",937307977),LB=new Y(null,"value","value",305978217),MB=new Y(null,"date-hour-minute","date-hour-minute",1629918346),NB=new Y(null,"time","time",1385887882),OB=new Y(null,"component-did-mount","component-did-mount",-1126910518),PB=new Y(null,"fields","fields",-1932066230),QB=new Y("settings","app_settings","settings/app_settings",634678890),RB=new Y(null,"localtime",
"localtime",1720049482),SB=new Y(null,"basic-week-date-time","basic-week-date-time",-502077622),TB=new Y(null,"do-fx","do-fx",1194163050),UB=new Y(null,"net","net",344042539),VB=new Y(null,"months","months",-45571637),WB=new Y(null,"flash","flash",934660619),XB=new Y(null,"app_settings","app_settings",-682153397),YB=new Y(null,"PM","PM",1356677707),ZB=new Y(null,"settings","settings",1556144875),$B=new Y(null,"month","month",-1960248533),aC=new Y(null,"component-did-update","component-did-update",
-1468549173),bC=new Y(null,"initial-messages","initial-messages",2057377771),cC=new Y(null,"days","days",-1394072564),Wz=new Y(null,"val","val",128701612),dC=new Y(null,"format-str","format-str",695206156),eC=new Y(null,"dispatch-n","dispatch-n",-504469236),fC=new Y(null,"weekyear","weekyear",-74064500),gC=new Y(null,"type","type",1174270348),hC=new Y(null,"abbr","abbr",2088591884),iC=new Y(null,"debug","debug",-1608172596),jC=new Y(null,"basic-time","basic-time",-923134899),kC=new fv(null,"localDateParser",
"localDateParser",477820077,null),lC=new Y(null,"points","points",-1486596883),Rz=new Y(null,"fallback-impl","fallback-impl",-1501286995),mC=new Y(null,"invalid-date","invalid-date",2030506573),tt=new Y(null,"flush-on-newline","flush-on-newline",-151457939),nC=new Y(null,"componentWillUnmount","componentWillUnmount",1573788814),oC=new Y(null,"string","string",-1989541586),pC=new Y(null,"hour-minute-second","hour-minute-second",-1906654770),qC=new Y(null,"ordinal-date-time","ordinal-date-time",-1386753458),
rC=new Y(null,"deregister-event-handler","deregister-event-handler",-1096518994),sC=new Y("common","initialize","common/initialize",-132515154),tC=new Y(null,"seconds","seconds",-445266194),uC=new Y(null,"parse-error","parse-error",255902478),vC=new fv(null,"dateParser","dateParser",-1248418930,null),wC=new Y(null,"array","array",-2080713842),xC=new Y(null,"ordinal-date-time-no-ms","ordinal-date-time-no-ms",-1539005490),yC=new Y("net","load-auth-token","net/load-auth-token",-541175633),zC=new Y(null,
"on-click","on-click",1632826543),AC=new Y(null,"hour-minute-second-ms","hour-minute-second-ms",1209749775),BC=new Y(null,"running","running",1554969103),CC=new Y(null,"is_premium","is_premium",-1548830001),DC=new Y(null,"hitType","hitType",-1417600145),EC=new Y(null,"shouldComponentUpdate","shouldComponentUpdate",1795750960),FC=new Y("flash","add-message","flash/add-message",905417936),GC=new Y(null,"time-parser","time-parser",-1636511536),HC=new Y(null,"flush-dom","flush-dom",-933676816),IC=new Y(null,
"day-name","day-name",1806125744),JC=new Y(null,"div","div",1057191632),KC=new Y(null,"trim-v","trim-v",-1274938640),ut=new Y(null,"readably","readably",1129599760),LC=new Y(null,"date-time-parser","date-time-parser",-656147568),Hz=new Y(null,"more-marker","more-marker",-14717935),MC=new Y(null,"dispatch","dispatch",1319337009),NC=new Y(null,"year","year",335913393),pz=new Y(null,"reagentRender","reagentRender",-358306383),OC=new Y(null,"token","token",-1211463215),PC=new Y(null,"t-time-no-ms","t-time-no-ms",
990689905),QC=new fv(null,"dateElementParser","dateElementParser",984800945,null),RC=new Y(null,"basic-week-date-time-no-ms","basic-week-date-time-no-ms",-2043113679),SC=new fv(null,"localDateOptionalTimeParser","localDateOptionalTimeParser",435955537,null),TC=new Y(null,"no-cache","no-cache",1588056370),oz=new Y(null,"render","render",-1408033454),UC=new Y(null,"messages","messages",345434482),VC=new Y(null,"db-handler","db-handler",579530098),WC=new Y(null,"period-match-erroro","period-match-erroro",
1058444722),XC=new Y(null,"basic-date","basic-date",1566551506),YC=new Y(null,"event","event",301435442),ZC=new Y(null,"after","after",594996914),$C=new Y(null,"reagent-render","reagent-render",-985383853),aD=new Y(null,"weekyear-week","weekyear-week",795291571),bD=new Y(null,"sub-type","sub-type",-997954412),cD=new Y(null,"invalid-format","invalid-format",-72676108),xt=new Y(null,"print-length","print-length",1931866356),dD=new Y(null,"local-date","local-date",1829761428),eD=new Y(null,"basic-ordinal-date-time-no-ms",
"basic-ordinal-date-time-no-ms",-395135436),fD=new Y(null,"id","id",-1388402092),gD=new Y(null,"class","class",-2030961996),hD=new Y(null,"effects","effects",-282369292),iD=new Y(null,"progress?","progress?",374452148),jD=new Y(null,"pm","pm",1813737428),kD=new Y(null,"bindto","bindto",583512213),lD=new Y(null,"year-month","year-month",735283381),mD=new Y(null,"auto-run","auto-run",1958400437),nD=new fv(null,"/","/",-1371932971,null),oD=new Y(null,"run-queue","run-queue",-1701798027),pD=new Y(null,
"severity","severity",175684886),qD=new Y("form","init-form","form/init-form",339616054),rD=new Y(null,"info","info",-317069002),sD=new Y(null,"stack","stack",-793405930),tD=new Y("settings","settings","settings/settings",1867392726),uD=new fv(null,"dateTimeParser","dateTimeParser",-1493718282,null),vD=new Y("flash","delete-message","flash/delete-message",1089438775),wD=new Y(null,"local-date-opt-time","local-date-opt-time",1178432599),xD=new Y(null,"display-name","display-name",694513143),yD=new Y(null,
"right","right",-452581833),zD=new Y(null,"scheduled","scheduled",553898551),AD=new Y(null,"hours","hours",58380855),BD=new Y(null,"years","years",-1298579689),CD=new Y(null,"week-date","week-date",-1176745129),DD=new Y(null,"csrf-token","csrf-token",-1872302856),ED=new Y(null,"on-dispose","on-dispose",2105306360),FD=new Y(null,"pause","pause",-2095325672),GD=new Y(null,"error","error",-978969032),HD=new Y(null,"quoted","quoted",2117344952),qz=new Y(null,"componentFunction","componentFunction",825866104),
ID=new Y(null,"exception","exception",-335277064),JD=new Y(null,"coeffects","coeffects",497912985),KD=new Y(null,"x","x",2099068185),LD=new Y(null,"form","form",-1624062471),MD=new Y(null,"date-hour","date-hour",-344234471),ND=new Y(null,"tag","tag",-1290361223),OD=new Y(null,"contents","contents",-1567174023),PD=new Y(null,"period","period",-352129191),QD=new fv(null,"quote","quote",1377916282,null),RD=new Y(null,"set","set",304602554),SD=new Y(null,"minutes","minutes",1319166394),TD=new Y(null,
"timeout","timeout",-318625318),UD=new Y(null,"not-implemented","not-implemented",1918806714),VD=new Y(null,"Authorization","Authorization",-1017527462),WD=new Y(null,"axis","axis",-1215390822),XD=new Y("net","loading-count","net/loading-count",-1446220870),YD=new Y(null,"groupEnd","groupEnd",-337721382),ZD=new Y(null,"atom","atom",-397043653),$D=new Y(null,"trigger","trigger",103466139),aE=new Y(null,"autobind","autobind",-570650245),bE=new Y(null,"cofx","cofx",2013202907),cE=new Y("analytics","click",
"analytics/click",320864955),Qz=new Y(null,"alt-impl","alt-impl",670969595),dE=new Y(null,"goog.date.UtcDateTime","goog.date.UtcDateTime",731790075),eE=new Y(null,"resume","resume",-118572261),fE=new Y(null,"time-element-parser","time-element-parser",-2042883205),gE=new Y(null,"subchart","subchart",-126478469),hE=new Y(null,"ms","ms",-1152709733),iE=new Y(null,"enrich","enrich",-2108921925),jE=new Y(null,"fx","fx",-1237829572),kE=new Y(null,"auth-header-name","auth-header-name",759786620),lE=new Y(null,
"date-hour-minute-second","date-hour-minute-second",-1565419364),mE=new Y(null,"before","before",-1633692388),nE=new Y(null,"week-date-time-no-ms","week-date-time-no-ms",-1226853060),fA=new Y(null,"keywordize-keys","keywordize-keys",1310784252),oE=new Y(null,"log","log",-1595516004),pE=new fv(null,"deref","deref",1494944732,null),qE=new Y("re-frame.std-interceptors","untrimmed-event","re-frame.std-interceptors/untrimmed-event",-840935075),rE=new Y(null,"HOURS","HOURS",-1611068963),sE=new Y(null,"map",
"map",1371690461),tE=new Y(null,"finish-run","finish-run",753148477),uE=new Y(null,"basic-date-time-no-ms","basic-date-time-no-ms",-899402179),vE=new Y(null,"object","object",1474613949),wE=new Y(null,"componentWillMount","componentWillMount",-285327619),xE=new Y(null,"millis","millis",-1338288387),yE=new Y(null,"names","names",-1943074658),zE=new Y(null,"tooltip","tooltip",-1809677058),AE=new Y(null,"required","required",1807647006),BE=new Y(null,"created_at","created_at",1484050750),CE=new Y(null,
"mysql","mysql",-1431590210),DE=new Y(null,"dispatch-later","dispatch-later",291951390),EE=new Y("flash","add-initial-messages","flash/add-initial-messages",1637693695),FE=new Y(null,"basic-t-time","basic-t-time",191791391),GE=new Y(null,"in","in",-1531184865),HE=new Y(null,"left","left",-399115937),rz=new Y("cljs.core","not-found","cljs.core/not-found",-1572889185),IE=new Y(null,"default-year","default-year",1658037695),JE=new Y(null,"text","text",-1790561697),KE=new Y(null,"show","show",-576705889),
LE=new Y("views","set-view","views/set-view",473229023),ME=new Y(null,"data","data",-232669377);var NE=RegExp("([\\uD800-\\uDBFF])([\\uDC00-\\uDFFF])","g");function OE(a,b,c){var d=RegExp,e=b.source,g=z(b.ignoreCase)?[C.c("g"),C.c("i")].join(""):"g",g=z(b.multiline)?[C.c(g),C.c("m")].join(""):g;b=z(b.UA)?[C.c(g),C.c("u")].join(""):g;d=new d(e,b);return a.replace(d,c)}
function PE(a){return function(){function b(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(b){b=wx(b);if(L.h(Q(b),1))return b=I(b),a.c?a.c(b):a.call(null,b);b=fy(b);return a.c?a.c(b):a.call(null,b)}b.L=0;b.O=function(a){a=G(a);return c(a)};b.w=c;return b}()}
function QE(a,b,c){if("string"===typeof b)return a.replace(new RegExp(Ta(b),"g"),c);if(b instanceof RegExp)return"string"===typeof c?OE(a,b,c):OE(a,b,PE(c));throw[C.c("Invalid match arg: "),C.c(b)].join("");}function RE(a){var b=new Xs;for(a=G(a);;)if(null!=a)b=b.append(""+C.c(I(a))),a=K(a);else return b.toString()}function SE(a,b){for(var c=new Xs,d=G(b);;)if(null!=d)c.append(""+C.c(I(d))),d=K(d),null!=d&&c.append(a);else return c.toString()}
function TE(a){return 2>Q(a)?a.toUpperCase():[C.c(a.substring(0,1).toUpperCase()),C.c(a.substring(1).toLowerCase())].join("")}function UE(a,b){var c="/(?:)/"===""+C.c(b)?Lv.h(fy(Ev("",Z.h(C,G(a)))),""):fy((""+C.c(a)).split(b));if(1<Q(c))a:for(;;)if(""===Uv(c))c=null==c?null:ju(c);else break a;return c}function VE(a){return Ba(a)};function WE(a){var b=Oa("\x26nbsp;");return SE(b,Z.h(function(a){return RE(Fw(a))},Fw(Fx(3,3,null,a.replace(NE,"$2$1").split("").reverse().join("")))))}function XE(a){if(Ba(a))return null;var b=[C.c("%."),C.c(0),C.c("f")].join("");a=ym(b,a);a=UE(a,".");b=S(a,0,null);S(a,1,null);return[C.c(WE(b)),C.c(Oa("\x26nbsp;")),C.c("Kč")].join("")};var YE="undefined"!==typeof console;if("undefined"===typeof ZE)var ZE=ox?ox(null):nx.call(null,null);
if("undefined"===typeof $E)var $E=function(){var a={};a.warn=function(){return function(){function a(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(a){return sx.w(ZE,Jx,new V(null,1,5,X,[zB],null),Lv,R([U.h(C,a)],0))}a.L=0;a.O=function(a){a=G(a);return c(a)};a.w=c;return a}()}(a);a.error=function(){return function(){function a(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-
0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(a){return sx.w(ZE,Jx,new V(null,1,5,X,[GD],null),Lv,R([U.h(C,a)],0))}a.L=0;a.O=function(a){a=G(a);return c(a)};a.w=c;return a}()}(a);return a}();if("undefined"===typeof aF){var bF;if("undefined"!==typeof React)bF=React;else{var cF;if("undefined"!==typeof require){var dF=require("react");if(z(dF))cF=dF;else throw Error("require('react') failed");}else throw Error("js/React is missing");bF=cF}var aF=bF}var eF=new tz(null,new y(null,2,["aria",null,"data",null],null),null);function fF(a){return 2>Q(a)?a.toUpperCase():[C.c(a.substring(0,1).toUpperCase()),C.c(a.substring(1))].join("")}
function gF(a){if("string"===typeof a)return a;a=Kw(a);var b=UE(a,/-/);var c=G(b);b=I(c);c=K(c);return z(eF.c?eF.c(b):eF.call(null,b))?a:U.j(C,b,Z.h(fF,c))}function hF(a){var b=function(){var b=function(){var b=Qv(a);return b?(b=a.displayName,z(b)?b:a.name):b}();if(z(b))return b;b=function(){var b=null!=a?a.Z&4096||x===a.jr?!0:!1:!1;return b?Kw(a):b}();if(z(b))return b;b=Tv(a);return $v(b)?DB.c(b):null}();return QE(""+C.c(b),"$",".")}var iF=!1;if("undefined"===typeof jF)var jF=0;function kF(a){return setTimeout(a,16)}var lF="undefined"===typeof window||null==window.document?kF:function(){var a=window,b=a.requestAnimationFrame;if(z(b))return b;b=a.webkitRequestAnimationFrame;if(z(b))return b;b=a.mozRequestAnimationFrame;if(z(b))return b;a=a.msRequestAnimationFrame;return z(a)?a:kF}();function mF(a,b){return a.cljsMountOrder-b.cljsMountOrder}if("undefined"===typeof nF)var nF=function(){return null};function oF(a){this.il=a}
function pF(a,b){var c=a[b];if(null==c)return null;a[b]=null;for(var d=c.length,e=0;;)if(e<d)c[e].call(null),e+=1;else return null}function qF(a){if(a.il)return null;a.il=!0;a=function(a){return function(){a.il=!1;pF(a,"beforeFlush");nF();var b=a.componentQueue;if(null!=b){a.componentQueue=null;b.sort(mF);for(var d=b.length,e=0;;)if(e<d){var g=b[e];!0===g.cljsIsDirty&&g.forceUpdate();e+=1}else break}return pF(a,"afterRender")}}(a);return lF.c?lF.c(a):lF.call(null,a)}
oF.prototype.enqueue=function(a,b){if(null==b)throw Error("Assert failed: (some? f)");null==this[a]&&(this[a]=[]);this[a].push(b);return qF(this)};function rF(a){return sF.enqueue("afterRender",a)}if("undefined"===typeof sF)var sF=new oF(!1);function tF(a){if(z(a.cljsIsDirty))return null;a.cljsIsDirty=!0;return sF.enqueue("componentQueue",a)};function uF(a,b){var c=U.j(zz,a,b);return Ev(c,Ax(function(a){return function(b){return a===b}}(c),b))}function vF(a,b){return Q(a)<Q(b)?Gt(Lv,b,a):Gt(Lv,a,b)}var wF=function wF(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return wF.c(arguments[0]);case 2:return wF.h(arguments[0],arguments[1]);default:return wF.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};wF.c=function(a){return a};
wF.h=function(a,b){for(;;)if(Q(b)<Q(a)){var c=a;a=b;b=c}else return Gt(function(a,b){return function(a,c){return lw(b,c)?a:Vv.h(a,c)}}(a,b),a,a)};wF.w=function(a,b,c){a=uF(function(a){return-Q(a)},Lv.w(c,b,R([a],0)));return Gt(wF,I(a),hv(a))};wF.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return wF.w(b,a,c)};wF.L=2;
var xF=function xF(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return xF.c(arguments[0]);case 2:return xF.h(arguments[0],arguments[1]);default:return xF.w(arguments[0],arguments[1],new H(c.slice(2),0,null))}};xF.c=function(a){return a};xF.h=function(a,b){return Q(a)<Q(b)?Gt(function(a,d){return lw(b,d)?Vv.h(a,d):a},a,a):Gt(Vv,a,b)};xF.w=function(a,b,c){return Gt(xF,a,Lv.h(c,b))};
xF.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return xF.w(b,a,c)};xF.L=2;var yF;if("undefined"===typeof zF)var zF=!1;if("undefined"===typeof AF)var AF=0;if("undefined"===typeof BF)var BF=ox?ox(0):nx.call(null,0);function CF(a,b){var c=yF;yF=a;try{return b.T?b.T():b.call(null)}finally{yF=c}}
function DF(a,b){b.Dj=null;b.RA=AF+=1;var c=CF(b,a),d=b.Dj;b.Qe=!1;a:{var e=b.Bg;var g=null==d?0:d.length,h=g===(null==e?0:e.length);if(h)for(h=0;;){var k=h===g;if(k){e=k;break a}if(d[h]===e[h])h+=1;else{e=!1;break a}}else e=h}if(!e)a:{e=xz(d);g=xz(b.Bg);b.Bg=d;for(var d=G(xF.h(e,g)),h=null,l=k=0;;)if(l<k){var m=h.aa(null,l);Hu(m,b,EF);l+=1}else if(d=G(d))h=d,cw(h)?(d=Ou(h),l=Pu(h),h=d,k=Q(d),d=l):(d=I(h),Hu(d,b,EF),d=K(h),h=null,k=0),l=0;else break;e=G(xF.h(g,e));d=null;for(k=h=0;;)if(k<h)g=d.aa(null,
k),Iu(g,b),k+=1;else if(e=G(e))d=e,cw(d)?(e=Ou(d),d=Pu(d),g=e,h=Q(e),e=d,d=g):(g=I(d),Iu(g,b),e=K(d),d=null,h=0),k=0;else break a}return c}function FF(a){var b=yF;if(null!=b){var c=b.Dj;null==c?b.Dj=[a]:c.push(a)}}function GF(a,b){zF&&sx.j(BF,rw,Q(b)-Q(a));return b}function HF(a,b,c){var d=a.yb;a.yb=GF(d,T.j(d,b,c));return a.hq=null}function IF(a,b){var c=a.yb;a.yb=GF(c,Pv.h(c,b));return a.hq=null}
function JF(a,b,c){for(var d=a.hq,d=null==d?a.hq=ow(function(){return function(a,b,c){a.push(b);a.push(c);return a}}(d),[],a.yb):d,e=d.length,g=0;;)if(g<e){var h=d[g],k=d[g+1];k.N?k.N(h,a,b,c):k.call(null,h,a,b,c);g=2+g}else return null}function KF(a,b,c,d){Eu(b,[C.c("#\x3c"),C.c(d),C.c(" ")].join(""));a:{d=yF;yF=null;try{var e=nu(a);break a}finally{yF=d}e=void 0}Oz(e,b,c);return Eu(b,"\x3e")}if("undefined"===typeof LF)var LF=null;
function MF(){for(;;){var a=LF;if(null==a)return null;LF=null;for(var b=a.length,c=0;;)if(c<b){var d=a[c];d.Qe&&null!=d.Bg&&NF(d,!0);c+=1}else break}}nF=MF;function OF(a,b,c,d){this.state=a;this.meta=b;this.Qd=c;this.yb=d;this.C=2153938944;this.Z=114690}f=OF.prototype;f.Yk=x;f.ja=function(a,b,c){return KF(this,b,c,"Atom:")};f.sa=function(){return this.meta};f.ha=function(){return pa(this)};f.P=function(a,b){return this===b};
f.mc=function(a,b){if(null!=this.Qd&&!z(this.Qd.c?this.Qd.c(b):this.Qd.call(null,b)))throw Error([C.c("Assert failed: "),C.c("Validator rejected reference state"),C.c("\n"),C.c("(validator new-value)")].join(""));var c=this.state;this.state=b;null!=this.yb&&JF(this,c,b);return b};f.Oj=function(a,b){return this.mc(null,b.c?b.c(this.state):b.call(null,this.state))};f.Pj=function(a,b,c){return this.mc(null,b.h?b.h(this.state,c):b.call(null,this.state,c))};
f.Qj=function(a,b,c,d){return this.mc(null,b.j?b.j(this.state,c,d):b.call(null,this.state,c,d))};f.Rj=function(a,b,c,d,e){return this.mc(null,U.ca(b,this.state,c,d,e))};f.$h=function(a,b,c){return JF(this,b,c)};f.Zh=function(a,b,c){return HF(this,b,c)};f.ai=function(a,b){return IF(this,b)};f.Pe=function(){FF(this);return this.state};
var PF=function PF(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return PF.c(arguments[0]);default:return PF.w(arguments[0],new H(c.slice(1),0,null))}};PF.c=function(a){return new OF(a,null,null,null)};PF.w=function(a,b){var c=null!=b&&(b.C&64||x===b.Sb)?U.h(px,b):b,d=F.h(c,vt),c=F.h(c,qx);return new OF(a,d,c,null)};PF.O=function(a){var b=I(a);a=K(a);return PF.w(b,a)};PF.L=1;
function QF(a,b,c,d){var e=b.reagReactionCache,g=null==e?cx:e,h=g.h?g.h(c,null):g.call(null,c,null);if(null!=h)return nu(h);if(null==yF)return a.T?a.T():a.call(null);var k=function(){var k=function(){return function(){zF&&sx.h(BF,tw);var a=Pv.h(b.reagReactionCache,c);b.reagReactionCache=a;null!=d&&(d.Si=null);return null}}(a,ED,e,g,h);return RF.j?RF.j(a,ED,k):RF.call(null,a,ED,k)}(),l=nu(k);b.reagReactionCache=T.j(g,c,k);zF&&sx.h(BF,qv);null!=d&&(d.Si=k);return l}
function SF(a,b,c){this.Uc=a;this.Ab=b;this.Si=c;this.C=2153807872;this.Z=0}f=SF.prototype;f.Yk=x;f.Pe=function(){var a=this,b=a.Si;return null==b?QF(function(){return function(){return U.h(a.Uc,a.Ab)}}(b,this),a.Uc,a.Ab,this):nu(b)};f.P=function(a,b){return b instanceof SF&&L.h(this.Uc,b.Uc)&&L.h(this.Ab,b.Ab)};f.ha=function(){return cv(new V(null,2,5,X,[this.Uc,this.Ab],null))};f.ja=function(a,b,c){return KF(this,b,c,"Track:")};
function TF(a,b,c,d,e){this.Ha=a;this.path=b;this.Si=c;this.state=d;this.yb=e;this.C=2153807872;this.Z=114690}function UF(a){var b=yF;yF=null;try{return a.Pe(null)}finally{yF=b}}function VF(a,b,c){b!==c&&(a.state=c,null!=a.yb&&JF(a,b,c))}f=TF.prototype;f.Yk=x;f.ja=function(a,b,c){return KF(this,b,c,[C.c("Cursor: "),C.c(this.path)].join(""))};f.ha=function(){return cv(new V(null,2,5,X,[this.Ha,this.path],null))};f.P=function(a,b){return b instanceof TF&&L.h(this.path,b.path)&&L.h(this.Ha,b.Ha)};
f.mc=function(a,b){VF(this,this.state,b);(null!=this.Ha?this.Ha.C&32768||x===this.Ha.fr||(this.Ha.C?0:B(mu,this.Ha)):B(mu,this.Ha))?L.h(this.path,Mv)?rx.h?rx.h(this.Ha,b):rx.call(null,this.Ha,b):sx.N(this.Ha,Ix,this.path,b):this.Ha.h?this.Ha.h(this.path,b):this.Ha.call(null,this.path,b);return b};f.Oj=function(a,b){var c=this;return c.mc(null,function(){var a=UF(c);return b.c?b.c(a):b.call(null,a)}())};
f.Pj=function(a,b,c){var d=this;return d.mc(null,function(){var a=UF(d);return b.h?b.h(a,c):b.call(null,a,c)}())};f.Qj=function(a,b,c,d){var e=this;return e.mc(null,function(){var a=UF(e);return b.j?b.j(a,c,d):b.call(null,a,c,d)}())};f.Rj=function(a,b,c,d,e){return this.mc(null,U.ca(b,UF(this),c,d,e))};f.$h=function(a,b,c){return JF(this,b,c)};f.Zh=function(a,b,c){return HF(this,b,c)};f.ai=function(a,b){return IF(this,b)};
f.Pe=function(){var a=this,b=this,c=a.state,d=function(){var d=a.Si;return null==d?(d=(null!=a.Ha?a.Ha.C&32768||x===a.Ha.fr||(a.Ha.C?0:B(mu,a.Ha)):B(mu,a.Ha))?function(){return function(){return Gx(N.c?N.c(a.Ha):N.call(null,a.Ha),a.path)}}(d,c,b):function(){return function(){return a.Ha.c?a.Ha.c(a.path):a.Ha.call(null,a.path)}}(d,c,b),QF(d,a.Ha,a.path,b)):nu(d)}();VF(b,c,d);return d};
var WF=function WF(b){if(null!=b&&null!=b.pp)return b.pp();var c=WF[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=WF._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("IDisposable.dispose!",b);},XF=function XF(b,c){if(null!=b&&null!=b.Us)return b.Us(0,c);var d=XF[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=XF._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IDisposable.add-on-dispose!",b);};
function EF(a,b,c,d){c===d||a.Qe?a=null:null==a.wd?(a.Qe=!0,null==LF&&(LF=[],!1===sF.il&&qF(sF)),a=LF.push(a)):a=!0===a.wd?NF(a,!1):a.wd.c?a.wd.c(a):a.wd.call(null,a);return a}function YF(a,b,c,d,e,g,h,k){this.Uc=a;this.state=b;this.Qe=c;this.us=d;this.Bg=e;this.yb=g;this.wd=h;this.Gm=k;this.C=2153807872;this.Z=114690}function ZF(a){var b=yF;yF=null;try{return a.Pe(null)}finally{yF=b}}
function NF(a,b){var c=a.state;if(z(b)){var d=a.Uc;try{a.Gm=null;var e=DF(d,a)}catch(g){e=g,a.state=e,a.Gm=e,e=a.Qe=!1}}else e=DF(a.Uc,a);a.us||(a.state=e,null==a.yb||L.h(c,e)||JF(a,c,e));return e}function $F(a,b){var c=null!=b&&(b.C&64||x===b.Sb)?U.h(px,b):b,d=F.h(c,mD),e=F.h(c,BA),g=F.h(c,ED),c=F.h(c,TC);null!=d&&(a.wd=d);null!=e&&(a.Is=e);null!=g&&(a.Gs=g);null!=c&&(a.us=c)}f=YF.prototype;f.Yk=x;f.ja=function(a,b,c){return KF(this,b,c,[C.c("Reaction "),C.c(cv(this)),C.c(":")].join(""))};f.ha=function(){return pa(this)};
f.P=function(a,b){return this===b};f.pp=function(){var a=this.state,b=this.Bg;this.wd=this.state=this.Bg=null;this.Qe=!0;for(var b=G(xz(b)),c=null,d=0,e=0;;)if(e<d){var g=c.aa(null,e);Iu(g,this);e+=1}else if(b=G(b))c=b,cw(c)?(b=Ou(c),e=Pu(c),c=b,d=Q(b),b=e):(b=I(c),Iu(b,this),b=K(c),c=null,d=0),e=0;else break;null!=this.Gs&&this.Gs(a);a=this.Hs;if(null==a)return null;b=a.length;for(c=0;;)if(c<b)a[c].call(null,this),c+=1;else return null};
f.Us=function(a,b){var c=this.Hs;return null==c?this.Hs=[b]:c.push(b)};f.mc=function(a,b){if(!Qv(this.Is))throw Error([C.c("Assert failed: "),C.c("Reaction is read only."),C.c("\n"),C.c("(fn? (.-on-set a))")].join(""));var c=this.state;this.state=b;this.Is(c,b);JF(this,c,b);return b};f.Oj=function(a,b){var c=this;return c.mc(null,function(){var a=ZF(c);return b.c?b.c(a):b.call(null,a)}())};f.Pj=function(a,b,c){var d=this;return d.mc(null,function(){var a=ZF(d);return b.h?b.h(a,c):b.call(null,a,c)}())};
f.Qj=function(a,b,c,d){var e=this;return e.mc(null,function(){var a=ZF(e);return b.j?b.j(a,c,d):b.call(null,a,c,d)}())};f.Rj=function(a,b,c,d,e){return this.mc(null,U.ca(b,ZF(this),c,d,e))};f.$h=function(a,b,c){return JF(this,b,c)};f.Zh=function(a,b,c){return HF(this,b,c)};f.ai=function(a,b){var c=Wv(this.yb);IF(this,b);return!c&&Wv(this.yb)&&null==this.wd?this.pp():null};
f.Pe=function(){var a=this.Gm;if(null!=a)throw a;(a=null==yF)&&MF();a&&null==this.wd?this.Qe&&(a=this.state,this.state=this.Uc.T?this.Uc.T():this.Uc.call(null),null==this.yb||L.h(a,this.state)||JF(this,a,this.state)):(FF(this),this.Qe&&NF(this,!1));return this.state};
function RF(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;var c=arguments[0],b=1<b.length?new H(b.slice(1),0,null):null,e=null!=b&&(b.C&64||x===b.Sb)?U.h(px,b):b,b=F.h(e,mD),d=F.h(e,BA),e=F.h(e,ED),c=new YF(c,null,!0,!1,null,null,null,null);$F(c,new y(null,3,[mD,b,BA,d,ED,e],null));return c}var aG=RF(null);
function bG(a,b){var c=cG,d=aG,e=DF(a,d);null!=d.Bg&&(aG=RF(null),$F(d,c),d.Uc=a,d.wd=function(){return function(){return tF.c?tF.c(b):tF.call(null,b)}}(d,e),b.cljsRatom=d);return e}function dG(a){var b={};a=CF(b,a);return new V(null,2,5,X,[a,null!=b.Dj],null)};var eG;function fG(a,b){var c=b.argv;if(null==c){var c=X,d=a.constructor;a:for(var e=dw(b),g=e.length,h=cx,k=0;;)if(k<g)var l=e[k],h=T.j(h,Jw.c(l),b[l]),k=k+1;else break a;c=new V(null,2,5,c,[d,h],null)}return c}function gG(a){var b;if(b=Qv(a))a=null==a?null:a.prototype,b=null!=(null==a?null:a.reagentRender);return b}function hG(a){var b;if(b=Qv(a))a=null==a?null:a.prototype,b=null!=(null==a?null:a.render);return b}if("undefined"===typeof iG)var iG=null;
function jG(a){for(;;){var b=a.reagentRender;if(jw(b))var c=null;else throw Error("Assert failed: (ifn? f)");var d=!0===a.cljsLegacyRender?b.call(a,a):function(){var c=fG(a,a.props);switch(Q(c)){case 1:return b.call(a);case 2:return b.call(a,yv(c,1));case 3:return b.call(a,yv(c,1),yv(c,2));case 4:return b.call(a,yv(c,1),yv(c,2),yv(c,3));case 5:return b.call(a,yv(c,1),yv(c,2),yv(c,3),yv(c,4));default:return b.apply(a,Ft(c).slice(1))}}();if(bw(d))return iG.c?iG.c(d):iG.call(null,d);if(jw(d))c=gG(d)?
function(a,b,c,d){return function(){function a(a){var c=null;if(0<arguments.length){for(var c=0,d=Array(arguments.length-0);c<d.length;)d[c]=arguments[c+0],++c;c=new H(d,0,null)}return b.call(this,c)}function b(a){a=U.j(gy,d,a);return iG.c?iG.c(a):iG.call(null,a)}a.L=0;a.O=function(a){a=G(a);return b(a)};a.w=b;return a}()}(a,b,c,d):d,a.reagentRender=c;else return d}}
var cG=new y(null,1,[TC,!0],null),lG=new y(null,1,[oz,function(){var a=this.cljsRatom;this.cljsIsDirty=!1;return null==a?bG(function(a,c){return function(){a:{var a=eG;eG=c;try{var b=[!1];try{var g=jG(c);b[0]=!0;var h=g;break a}finally{z(b[0])||z(YE)&&(z(!1)?$E:console).error(""+C.c([C.c("Error rendering component"),C.c(kG.T?kG.T():kG.call(null))].join("")))}}finally{eG=a}h=void 0}return h}}(a,this),this):NF(a,!1)}],null);
function mG(a,b){var c=a instanceof Y?a.Vb:null;switch(c){case "getDefaultProps":throw Error([C.c("Assert failed: "),C.c("getDefaultProps not supported"),C.c("\n"),C.c("false")].join(""));case "getInitialState":return function(){return function(){var a=this.cljsState;a=null!=a?a:this.cljsState=PF.c(null);var c=b.call(this,this);return rx.h?rx.h(a,c):rx.call(null,a,c)}}(a,c);case "componentWillReceiveProps":return function(){return function(a){return b.call(this,this,fG(this,a))}}(a,c);case "shouldComponentUpdate":return function(){return function(a){var c=
iF;if(c)return c;var c=this.props.argv,d=a.argv,h=null==c||null==d;return null==b?h||Zw.h(c,d):h?b.call(this,this,fG(this,this.props),fG(this,a)):b.call(this,this,c,d)}}(a,c);case "componentWillUpdate":return function(){return function(a){return b.call(this,this,fG(this,a))}}(a,c);case "componentDidUpdate":return function(){return function(a){return b.call(this,this,fG(this,a))}}(a,c);case "componentWillMount":return function(){return function(){this.cljsMountOrder=jF+=1;return null==b?null:b.call(this,
this)}}(a,c);case "componentDidMount":return function(){return function(){return b.call(this,this)}}(a,c);case "componentWillUnmount":return function(){return function(){var a=this.cljsRatom;null!=a&&WF(a);this.cljsIsDirty=!1;return null==b?null:b.call(this,this)}}(a,c);default:return null}}
function nG(a,b,c){var d=mG(a,b);if(z(z(d)?b:d)&&!jw(b))throw Error([C.c("Assert failed: "),C.c([C.c("Expected function in "),C.c(c),C.c(a),C.c(" but got "),C.c(b)].join("")),C.c("\n"),C.c("(ifn? f)")].join(""));return z(d)?d:b}var oG=new y(null,3,[EC,null,wE,null,nC,null],null),pG=function(a){return function(b){return function(c){var d=F.h(N.c?N.c(b):N.call(null,b),c);if(null!=d)return d;d=a.c?a.c(c):a.call(null,c);sx.N(b,T,c,d);return d}}(function(){var a=cx;return ox?ox(a):nx.call(null,a)}())}(gF);
function qG(a){return ow(function(a,c,d){return T.j(a,Jw.c(pG.c?pG.c(c):pG.call(null,c)),d)},cx,a)}
function rG(a){var b=nz(a),c=I(Dy(b));if(!(0<Q(b)))throw Error([C.c("Assert failed: "),C.c("Missing reagent-render"),C.c("\n"),C.c("(pos? (count renders))")].join(""));if(1!==Q(b))throw Error([C.c("Assert failed: "),C.c("Too many render functions supplied"),C.c("\n"),C.c("(\x3d\x3d 1 (count renders))")].join(""));if(!jw(c))throw Error([C.c("Assert failed: "),C.c([C.c("Render must be a function, not "),C.c(Tz(R([c],0),st()))].join("")),C.c("\n"),C.c("(ifn? render-fun)")].join(""));var d=function(){var b=
pz.c(a);return z(b)?b:qz.c(a)}(),b=null==d,e=z(d)?d:oz.c(a),g=""+C.c(function(){var b=tB.c(a);return z(b)?b:hF(e)}());a:switch(g){case "":c=""+C.c(Yz());break a;default:c=g}d=ow(function(a,b,c,d,e){return function(a,b,c){return T.j(a,b,nG(b,c,e))}}(d,b,e,g,c),cx,a);return T.w(d,tB,c,R([aE,!1,MA,b,pz,e,oz,oz.c(lG)],0))}function sG(a){return ow(function(a,c,d){a[Kw(c)]=d;return a},{},a)}
function tG(a){if(!$v(a))throw Error("Assert failed: (map? body)");return aF.createClass(sG(rG(mz.w(R([oG,qG(a)],0)))))}
var uG=function uG(b){var c=function(){var c=null==b?null:b._reactInternalInstance;c=z(c)?c:b;return null==c?null:c._currentElement}(),d=function(){var b=null==c?null:c.type;return null==b?null:b.displayName}(),e=function(){var b=null==c?null:c._owner,b=null==b?null:uG.c?uG.c(b):uG.call(null,b);return null==b?null:[C.c(b),C.c(" \x3e ")].join("")}(),d=[C.c(e),C.c(d)].join("");return Wv(d)?null:d};
function kG(){var a=eG;var b=uG(a);z(b)?a=b:(a=null==a?null:a.constructor,a=null==a?null:hF(a));return Wv(a)?"":[C.c(" (in "),C.c(a),C.c(")")].join("")}
function vG(a){if(!jw(a))throw Error([C.c("Assert failed: "),C.c([C.c("Expected a function, not "),C.c(Tz(R([a],0),st()))].join("")),C.c("\n"),C.c("(ifn? f)")].join(""));hG(a)&&!gG(a)&&z(YE)&&(z(!1)?$E:console).warn([C.c("Warning: "),C.c("Using native React classes directly in Hiccup forms "),C.c("is not supported. Use create-element or "),C.c("adapt-react-class instead: "),C.c(function(){var b=hF(a);return Wv(b)?a:b}()),C.c(kG())].join(""));if(gG(a))return a.cljsReactClass=a;var b=Tv(a),b=T.j(b,
$C,a),b=tG(b);return a.cljsReactClass=b};function wG(a,b,c){if(Dw(c))return c=U.h(Gw,Z.h(a,c)),b.c?b.c(c):b.call(null,c);if(hw(c))return c=Cz(Z.h(a,c)),b.c?b.c(c):b.call(null,c);if(aw(c))return c=Gt(function(b,c){return Lv.h(b,a.c?a.c(c):a.call(null,c))},c,c),b.c?b.c(c):b.call(null,c);Xv(c)&&(c=jx.h(null==c?null:Mt(c),Z.h(a,c)));return b.c?b.c(c):b.call(null,c)}var xG=function xG(b,c){return wG(hx.h(xG,b),pw,b.c?b.c(c):b.call(null,c))};var yG=/([^\s\.#]+)(?:#([^\s\.#]+))?(?:\.([^\s#]+))?/;function zG(a){return a instanceof Y||a instanceof fv}var AG={"class":"className","for":"htmlFor",charset:"charSet"};function BG(a,b,c){if(zG(b)){var d=Kw(b);d=AG.hasOwnProperty(d)?AG[d]:null;b=null==d?AG[Kw(b)]=gF(b):d}a[b]=CG.c?CG.c(c):CG.call(null,c);return a}
function CG(a){return"object"!==n(a)?a:zG(a)?Kw(a):$v(a)?ow(BG,{},a):Xv(a)?bA(a):jw(a)?function(){function b(a){var b=null;if(0<arguments.length){for(var b=0,d=Array(arguments.length-0);b<d.length;)d[b]=arguments[b+0],++b;b=new H(d,0,null)}return c.call(this,b)}function c(b){return U.h(a,b)}b.L=0;b.O=function(a){a=G(a);return c(a)};b.w=c;return b}():bA(a)}function DG(a,b,c){a=null==a?{}:a;a[b]=c;return a}
var EG=new tz(null,new y(null,6,["url",null,"tel",null,"text",null,"textarea",null,"password",null,"search",null],null),null),FG=function FG(b){var c=b.cljsInputElement;if(null==c)return null;b.cljsInputDirty=!1;var d=b.cljsRenderedValue,e=b.cljsDOMValue;if(Zw.h(d,e)){if(c===document.activeElement&&lw(EG,c.type)&&"string"===typeof d&&"string"===typeof e){var g=c.value;if(Zw.h(g,e))return rF(function(){return function(){return FG.c?FG.c(b):FG.call(null,b)}}(g,d,e,c,c));e=Q(g)-c.selectionStart;e=Q(d)-
e;b.cljsDOMValue=d;c.value=d;c.selectionStart=e;return c.selectionEnd=e}b.cljsDOMValue=d;return c.value=d}return null};function GG(a,b,c){a.cljsDOMValue=c.target.value;z(a.cljsInputDirty)||(a.cljsInputDirty=!0,rF(function(){return FG(a)}));return b.c?b.c(c):b.call(null,c)}
function HG(a){var b=eG;if(z(function(){var b=null!=a;return b?(b=a.hasOwnProperty("onChange"),z(b)?a.hasOwnProperty("value"):b):b}())){var c=a.value,d=null==c?"":c,e=a.onChange;null==b.cljsInputElement&&(b.cljsDOMValue=d);b.cljsRenderedValue=d;delete a.value;a.defaultValue=d;a.onChange=function(a,c,d,e){return function(a){return GG(b,e,a)}}(a,c,d,e);a.ref=function(){return function(a){return b.cljsInputElement=a}}(a,c,d,e)}}
var IG=null,KG=new y(null,3,[xD,"ReagentInput",aC,FG,$C,function(a,b,c,d){HG(c);return JG.N?JG.N(a,b,c,d):JG.call(null,a,b,c,d)}],null);function LG(a){if($v(a))try{var b=F.h(a,eB)}catch(c){b=null}else b=null;return b}function MG(a){var b=LG(Tv(a));return null==b?LG(S(a,1,null)):b}var NG={};
function OG(a,b,c){var d=a.name,e=S(b,c,null),g=null==e||$v(e);var e=CG(g?e:null),h=a.id,e=null!=h&&null==(null==e?null:e.id)?DG(e,"id",h):e;a=a.className;null==a?a=e:(h=null==e?null:e.className,a=DG(e,"className",null==h?a:[C.c(a),C.c(" "),C.c(h)].join("")));c+=g?1:0;a:switch(d){case "input":case "textarea":g=!0;break a;default:g=!1}if(g)return g=X,null==IG&&(IG=tG(KG)),b=Sv(new V(null,5,5,g,[IG,b,d,a,c],null),Tv(b)),PG.c?PG.c(b):PG.call(null,b);g=LG(Tv(b));g=null==g?a:DG(a,"key",g);return JG.N?
JG.N(b,d,g,c):JG.call(null,b,d,g,c)}function QG(a){return""+C.c(xG(function(a){if(Qv(a)){var b=hF(a);switch(b){case "":return a;default:return gv.c(b)}}else return a},a))}function RG(a,b){return[C.c(U.h(C,b)),C.c(": "),C.c(QG(a)),C.c("\n"),C.c(kG())].join("")}
function SG(a){for(;;){if(!(0<Q(a)))throw Error([C.c("Assert failed: "),C.c(RG(a,R(["Hiccup form should not be empty"],0))),C.c("\n"),C.c("(pos? (count v))")].join(""));var b=S(a,0,null);if(!zG(b)&&"string"!==typeof b&&!jw(b))throw Error([C.c("Assert failed: "),C.c(RG(a,R(["Invalid Hiccup form"],0))),C.c("\n"),C.c("(valid-tag? tag)")].join(""));if(zG(b)||"string"===typeof b){var b=Kw(b),c=b.indexOf("\x3e");switch(c){case -1:var c=b,b=NG,d=c,b=b.hasOwnProperty(d)?b[d]:null;if(null==b){var b=c,e=K(Dz(yG,
Kw(c))),d=S(e,0,null),g=S(e,1,null),e=S(e,2,null),e=null==e?null:QE(e,/\./," ");if(!z(d))throw Error([C.c("Assert failed: "),C.c([C.c("Invalid tag: '"),C.c(c),C.c("'"),C.c(kG())].join("")),C.c("\n"),C.c("tag")].join(""));b=NG[b]={name:d,id:g,className:e}}return OG(b,a,1);case 0:c=S(a,1,null);if(!L.h("\x3e",b))throw Error([C.c("Assert failed: "),C.c(RG(a,R(["Invalid Hiccup tag"],0))),C.c("\n"),C.c('(\x3d "\x3e" n)')].join(""));if("string"!==typeof c&&!Qv(c))throw Error([C.c("Assert failed: "),C.c(RG(a,
R(["Expected React component in"],0))),C.c("\n"),C.c("(or (string? comp) (fn? comp))")].join(""));return OG({name:c},a,2);default:a=new V(null,2,5,X,[b.substring(0,c),T.j(a,0,b.substring(c+1))],null)}}else return c=b.cljsReactClass,b=null==c?vG(b):c,c={argv:a},a=MG(a),null!=a&&(c.key=a),aF.createElement(b,c)}}function PG(a){return"object"!==n(a)?a:bw(a)?SG(a):hw(a)?TG.c?TG.c(a):TG.call(null,a):zG(a)?Kw(a):(null!=a?a.C&2147483648||x===a.Ta||(a.C?0:B(Fu,a)):B(Fu,a))?Tz(R([a],0),st()):a}iG=PG;
function TG(a){var b={},c=dG(function(b){return function(){for(var c=Ft(a),d=c.length,e=0;;)if(e<d){var l=c[e];bw(l)&&null==MG(l)&&(b["no-key"]=!0);c[e]=PG(l);e+=1}else break;return c}}(b)),d=S(c,0,null),c=S(c,1,null);z(c)&&z(YE)&&(z(!1)?$E:console).warn([C.c("Warning: "),C.c(RG(a,R(["Reactive deref not supported in lazy seq, ","it should be wrapped in doall"],0)))].join(""));z(b["no-key"])&&z(YE)&&(z(!1)?$E:console).warn([C.c("Warning: "),C.c(RG(a,R(["Every element in a seq should have a unique :key"],
0)))].join(""));return d}function JG(a,b,c,d){var e=Q(a)-d;switch(e){case 0:return aF.createElement(b,c);case 1:return aF.createElement(b,c,PG(S(a,d,null)));default:return aF.createElement.apply(null,ow(function(){return function(a,b,c){b>=d&&a.push(PG(c));return a}}(e),[b,c],a))}};if("undefined"===typeof UG)var UG=null;if("undefined"===typeof VG)var VG=null;function WG(){if(null!=VG)return VG;if("undefined"!==typeof ReactDOM)return VG=ReactDOM;if("undefined"!==typeof require){var a=VG=require("react-dom");if(z(a))return a;throw Error("require('react-dom') failed");}throw Error("js/ReactDOM is missing");}if("undefined"===typeof XG){var XG,YG=cx;XG=ox?ox(YG):nx.call(null,YG)}
function ZG(a,b){var c=iF;iF=!0;try{return WG().render(a.T?a.T():a.call(null),b,function(){return function(){var c=iF;iF=!1;try{return sx.N(XG,T,b,new V(null,2,5,X,[a,b],null)),pF(sF,"afterRender"),null}finally{iF=c}}}(c))}finally{iF=c}}function $G(a,b){return ZG(a,b)}function aH(a,b){MF();return ZG(function(){return PG(Qv(a)?a.T?a.T():a.call(null):a)},b)};function dH(a,b){return aH(a,b)}ha("reagent.core.force_update_all",function(){MF();MF();for(var a=G(Dy(N.c?N.c(XG):N.call(null,XG))),b=null,c=0,d=0;;)if(d<c){var e=b.aa(null,d);U.h($G,e);d+=1}else if(a=G(a))b=a,cw(b)?(a=Ou(b),d=Pu(b),b=a,c=Q(a),a=d):(a=I(b),U.h($G,a),a=K(b),b=null,c=0),d=0;else break;return pF(sF,"afterRender")});function eH(a){return rF(a)};var fH=function fH(b){if(null!=b&&null!=b.mr)return b.mr();var c=fH[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=fH._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("PushbackReader.read-char",b);},gH=function gH(b,c){if(null!=b&&null!=b.nr)return b.nr(0,c);var d=gH[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=gH._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("PushbackReader.unread",b);};
function hH(a,b,c){this.s=a;this.buffer=b;this.lo=c}hH.prototype.mr=function(){return 0===this.buffer.length?(this.lo+=1,this.s[this.lo]):this.buffer.pop()};hH.prototype.nr=function(a,b){return this.buffer.push(b)};function iH(a){var b=!/[^\t\n\r ]/.test(a);return z(b)?b:","===a}function jH(a){throw Error(U.h(C,a));}
function kH(a,b){for(var c=new Xs(b),d=fH(a);;){var e;if(!(e=null==d||iH(d))){e=d;var g="#"!==e;e=g?(g="'"!==e)?(g=":"!==e)?lH.c?lH.c(e):lH.call(null,e):g:g:g}if(e)return gH(a,d),c.toString();c.append(d);d=fH(a)}}function mH(a){for(;;){var b=fH(a);if("\n"===b||"\r"===b||null==b)return a}}var nH=Fz("^([-+]?)(?:(0)|([1-9][0-9]*)|0[xX]([0-9A-Fa-f]+)|0([0-7]+)|([1-9][0-9]?)[rR]([0-9A-Za-z]+))(N)?$"),oH=Fz("^([-+]?[0-9]+)/([0-9]+)$"),pH=Fz("^([-+]?[0-9]+(\\.[0-9]*)?([eE][-+]?[0-9]+)?)(M)?$"),qH=Fz("^[:]?([^0-9/].*/)?([^0-9/][^/]*)$");
function rH(a,b){var c=a.exec(b);return null!=c&&c[0]===b?1===c.length?c[0]:c:null}var sH=Fz("^[0-9A-Fa-f]{2}$"),tH=Fz("^[0-9A-Fa-f]{4}$");function uH(a,b,c){return z(Dz(a,c))?c:jH(R(["Unexpected unicode escape \\",b,c],0))}
function vH(a){var b=fH(a),c="t"===b?"\t":"r"===b?"\r":"n"===b?"\n":"\\"===b?"\\":'"'===b?'"':"b"===b?"\b":"f"===b?"\f":null;z(c)?b=c:"x"===b?(a=(new Xs(fH(a),fH(a))).toString(),b=String.fromCharCode(parseInt(uH(sH,b,a),16))):"u"===b?(a=(new Xs(fH(a),fH(a),fH(a),fH(a))).toString(),b=String.fromCharCode(parseInt(uH(tH,b,a),16))):b=/[^0-9]/.test(b)?jH(R(["Unexpected unicode escape \\",b],0)):String.fromCharCode(b);return b}
function wH(a,b){for(var c=[];;){a:{var d=b;for(var e=iH,g=fH(d);;)if(z(e.c?e.c(g):e.call(null,g)))g=fH(d);else{d=g;break a}}z(d)||jH(R(["EOF while reading"],0));if(a===d)return c;e=lH.c?lH.c(d):lH.call(null,d);z(e)?d=e.h?e.h(b,d):e.call(null,b,d):(gH(b,d),d=xH.N?xH.N(b,!0,null,!0):xH.call(null,b,!0,null));d!==b&&c.push(d)}}function yH(a,b){return jH(R(["Reader for ",b," not implemented yet"],0))}
function zH(a,b){var c=fH(a),d=AH.c?AH.c(c):AH.call(null,c);if(z(d))return d.h?d.h(a,b):d.call(null,a,b);d=BH.h?BH.h(a,c):BH.call(null,a,c);return z(d)?d:jH(R(["No dispatch macro for ",c],0))}function CH(a,b){return jH(R(["Unmatched delimiter ",b],0))}function DH(a){a=wH(")",a);for(var b=a.length,c=iv;;)if(0<b)var d=b-1,c=c.xa(null,a[b-1]),b=d;else return c}function EH(a){return fy(wH("]",a))}
function FH(a){a=wH("}",a);var b=a.length;if(!kw(b))throw Error([C.c("Argument must be an integer: "),C.c(b)].join(""));0!==(b&1)&&jH(R(["Map literal must contain an even number of forms"],0));if(b<=2*Fy){for(var b=[],c=0;;)if(c<a.length){var d=a[c],e=a[c+1];if(-1===zy(b,d)){var g=b;g.push(d);g.push(e)}else throw Error([C.c("Duplicate key: "),C.c(d)].join(""));c+=2}else break;a=new y(null,a.length/2,a,null)}else{b=a.length;c=Ju(Gy);for(d=0;;)if(d<b){Mu(c,a[d],a[d+1]);if(Zw.h(Lt(c),d/2+1))throw Error([C.c("Duplicate key: "),
C.c(a[d])].join(""));d+=2}else break;a=Lu(c)}return a}function GH(a){for(var b=new Xs,c=fH(a);;){if(null==c)return jH(R(["EOF while reading"],0));if("\\"===c)b.append(vH(a));else{if('"'===c)return b.toString();b.append(c)}c=fH(a)}}
function HH(a){for(var b=new Xs,c=fH(a);;){if(null==c)return jH(R(["EOF while reading"],0));if("\\"===c){b.append(c);var d=fH(a);if(null==d)return jH(R(["EOF while reading"],0));var e=function(){var a=b;a.append(d);return a}(),g=fH(a)}else{if('"'===c)return b.toString();e=function(){var a=b;a.append(c);return a}();g=fH(a)}b=e;c=g}}
function IH(a,b){var c=kH(a,b),d=Pa(c,"/");z(z(d)?1!==c.length:d)?c=gv.h(yw(c,0,c.indexOf("/")),yw(c,c.indexOf("/")+1,c.length)):(d=gv.c(c),c="nil"===c?null:"true"===c?!0:"false"===c?!1:"/"===c?nD:d);return c}
function JH(a,b){var c=kH(a,b),d=c.substring(1);return 1===d.length?d:"tab"===d?"\t":"return"===d?"\r":"newline"===d?"\n":"space"===d?" ":"backspace"===d?"\b":"formfeed"===d?"\f":"u"===d.charAt(0)?String.fromCharCode(parseInt(d.substring(1),16)):"o"===d.charAt(0)?yH(0,c):jH(R(["Unknown character literal: ",c],0))}
function KH(a){a=kH(a,fH(a));var b=rH(qH,a);a=b[0];var c=b[1],b=b[2];return void 0!==c&&":/"===c.substring(c.length-2,c.length)||":"===b[b.length-1]||-1!==a.indexOf("::",1)?jH(R(["Invalid token: ",a],0)):null!=c&&0<c.length?Jw.h(c.substring(0,c.indexOf("/")),b):Jw.c(a)}function LH(a){return function(b){b=xH.N?xH.N(b,!0,null,!0):xH.call(null,b,!0,null);b=Ot(iv,b);return Ot(b,a)}}function MH(){return function(){return jH(R(["Unreadable form"],0))}}
function NH(a){var b=xH.N?xH.N(a,!0,null,!0):xH.call(null,a,!0,null);b=b instanceof fv?new y(null,1,[ND,b],null):"string"===typeof b?new y(null,1,[ND,b],null):b instanceof Y?Ov([b,!0]):b;$v(b)||jH(R(["Metadata must be Symbol,Keyword,String or Map"],0));a=xH.N?xH.N(a,!0,null,!0):xH.call(null,a,!0,null);return(null!=a?a.C&262144||x===a.NA||(a.C?0:B(qu,a)):B(qu,a))?Sv(a,mz.w(R([Tv(a),b],0))):jH(R(["Metadata can only be applied to IWithMetas"],0))}
function OH(a){a=wH("}",a);for(var b=a.length,c=Ju(vz),d=0;;)if(d<b){Ku(c,a[d]);if(!L.h(Q(c),d+1))throw Error([C.c("Duplicate key: "),C.c(a[d])].join(""));d+=1}else break;return Lu(c)}function PH(a){return Fz(HH(a))}function QH(a){xH.N?xH.N(a,!0,null,!0):xH.call(null,a,!0,null);return a}function lH(a){return'"'===a?GH:":"===a?KH:";"===a?mH:"'"===a?LH(QD):"@"===a?LH(pE):"^"===a?NH:"`"===a?yH:"~"===a?yH:"("===a?DH:")"===a?CH:"["===a?EH:"]"===a?CH:"{"===a?FH:"}"===a?CH:"\\"===a?JH:"#"===a?zH:null}
function AH(a){return"{"===a?OH:"\x3c"===a?MH():'"'===a?PH:"!"===a?mH:"_"===a?QH:null}
function xH(a,b,c){for(;;){var d=fH(a);if(null==d)return z(b)?jH(R(["EOF while reading"],0)):c;if(!iH(d))if(";"===d)a=mH.h?mH.h(a,d):mH.call(null,a);else{var e=lH(d);if(z(e))e=e.h?e.h(a,d):e.call(null,a,d);else{var g,e=a;!(g=!/[^0-9]/.test(d))&&(g="+"===d||"-"===d)&&(g=fH(e),gH(e,g),g=!/[^0-9]/.test(g));if(g)a:for(e=a,d=new Xs(d),g=fH(e);;){var h=null==g;h||(h=(h=iH(g))?h:lH.c?lH.c(g):lH.call(null,g));if(z(h)){gH(e,g);d=e=d.toString();z(rH(nH,d))?(d=rH(nH,d),g=d[2],null!=(L.h(g,"")?null:g)?d=0:(g=
z(d[3])?[d[3],10]:z(d[4])?[d[4],16]:z(d[5])?[d[5],8]:z(d[6])?[d[7],parseInt(d[6],10)]:[null,null],h=g[0],null==h?d=null:(g=parseInt(h,g[1]),d="-"===d[1]?-g:g))):z(rH(oH,d))?(d=rH(oH,d),d=parseInt(d[1],10)/parseInt(d[2],10)):d=z(rH(pH,d))?parseFloat(d):null;e=z(d)?d:jH(R(["Invalid number format [",e,"]"],0));break a}d.append(g);g=fH(e)}else e=IH(a,d)}if(e!==a)return e}}}
var RH=function(a,b){return function(c,d){return F.h(z(d)?b:a,c)}}(new V(null,13,5,X,[null,31,28,31,30,31,30,31,31,30,31,30,31],null),new V(null,13,5,X,[null,31,29,31,30,31,30,31,31,30,31,30,31],null)),SH=/(\d\d\d\d)(?:-(\d\d)(?:-(\d\d)(?:[T](\d\d)(?::(\d\d)(?::(\d\d)(?:[.](\d+))?)?)?)?)?)?(?:[Z]|([-+])(\d\d):(\d\d))?/;function TH(a){a=parseInt(a,10);return At(isNaN(a))?a:null}
function UH(a,b,c,d){a<=b&&b<=c||jH(R([[C.c(d),C.c(" Failed:  "),C.c(a),C.c("\x3c\x3d"),C.c(b),C.c("\x3c\x3d"),C.c(c)].join("")],0));return b}
function VH(a){var b=Dz(SH,a);S(b,0,null);var c=S(b,1,null),d=S(b,2,null),e=S(b,3,null),g=S(b,4,null),h=S(b,5,null),k=S(b,6,null),l=S(b,7,null),m=S(b,8,null),p=S(b,9,null),u=S(b,10,null);if(At(b))return jH(R([[C.c("Unrecognized date/time syntax: "),C.c(a)].join("")],0));var A=TH(c),D=function(){var a=TH(d);return z(a)?a:1}();a=function(){var a=TH(e);return z(a)?a:1}();var b=function(){var a=TH(g);return z(a)?a:0}(),c=function(){var a=TH(h);return z(a)?a:0}(),E=function(){var a=TH(k);return z(a)?a:
0}(),J=function(){a:if(L.h(3,Q(l)))var a=l;else if(3<Q(l))a=l.substring(0,3);else for(a=new Xs(l);;)if(3>a.Oe.length)a=a.append("0");else{a=a.toString();break a}a=TH(a);return z(a)?a:0}(),m=(L.h(m,"-")?-1:1)*(60*function(){var a=TH(p);return z(a)?a:0}()+function(){var a=TH(u);return z(a)?a:0}());return new V(null,8,5,X,[A,UH(1,D,12,"timestamp month field must be in range 1..12"),UH(1,a,function(){var a=0===vw(A,4);z(a)&&(a=At(0===vw(A,100)),a=z(a)?a:0===vw(A,400));return RH.h?RH.h(D,a):RH.call(null,
D,a)}(),"timestamp day field must be in range 1..last day in month"),UH(0,b,23,"timestamp hour field must be in range 0..23"),UH(0,c,59,"timestamp minute field must be in range 0..59"),UH(0,E,L.h(c,59)?60:59,"timestamp second field must be in range 0..60"),UH(0,J,999,"timestamp millisecond field must be in range 0..999"),m],null)}
var WH,XH=mz.w(R([new y(null,4,["inst",function(a){if("string"===typeof a){var b=VH(a);if(z(b)){a=S(b,0,null);var c=S(b,1,null),d=S(b,2,null),e=S(b,3,null),g=S(b,4,null),h=S(b,5,null),k=S(b,6,null);b=S(b,7,null);b=new Date(Date.UTC(a,c-1,d,e,g,h,k)-6E4*b)}else b=jH(R([[C.c("Unrecognized date/time syntax: "),C.c(a)].join("")],0))}else b=jH(R(["Instance literal expects a string for its timestamp."],0));return b},"uuid",function(a){return"string"===typeof a?new gA(a,null):jH(R(["UUID literal expects a string as its representation."],
0))},"queue",function(a){return bw(a)?jx.h(sy,a):jH(R(["Queue literal expects a vector for its elements."],0))},"js",function(a){if(bw(a)){var b=[];a=G(a);for(var c=null,d=0,e=0;;)if(e<d){var g=c.aa(null,e);b.push(g);e+=1}else if(a=G(a))c=a,cw(c)?(a=Ou(c),e=Pu(c),c=a,d=Q(a),a=e):(a=I(c),b.push(a),a=K(c),c=null,d=0),e=0;else break;return b}if($v(a)){b={};a=G(a);c=null;for(e=d=0;;)if(e<d){var h=c.aa(null,e),g=S(h,0,null),h=S(h,1,null);b[Kw(g)]=h;e+=1}else if(a=G(a))cw(a)?(d=Ou(a),a=Pu(a),c=d,d=Q(d)):
(d=I(a),c=S(d,0,null),d=S(d,1,null),b[Kw(c)]=d,a=K(a),c=null,d=0),e=0;else break;return b}return jH(R([[C.c("JS literal expects a vector or map containing "),C.c("only string or unqualified keyword keys")].join("")],0))}],null),cx],0));WH=ox?ox(XH):nx.call(null,XH);var YH=ox?ox(null):nx.call(null,null);
function BH(a,b){var c=IH(a,b),d=F.h(N.c?N.c(WH):N.call(null,WH),""+C.c(c)),e=N.c?N.c(YH):N.call(null,YH);return z(d)?(c=xH(a,!0,null),d.c?d.c(c):d.call(null,c)):z(e)?(d=xH(a,!0,null),e.h?e.h(c,d):e.call(null,c,d)):jH(R(["Could not find tag parser for ",""+C.c(c)," in ",Tz(R([Cy(N.c?N.c(WH):N.call(null,WH))],0),st())],0))}function ZH(a,b){var c=""+C.c(a);F.h(N.c?N.c(WH):N.call(null,WH),c);sx.N(WH,T,c,b)};function $H(a){return Z.h(function(b){return a.item(b)},new Bz(null,0,a.length,1,null))};function aI(a,b){try{var c=eA(qd(a),R([fA,!0],0)),d=b.c?b.c(c):b.call(null,c);return new V(null,2,5,X,[z(d)?c:a,d],null)}catch(e){return new V(null,2,5,X,[a,!1],null)}}function bI(a,b){var c=Kw(DB.c(b));c=UE(c,"-");c=RE(Ww.h(new V(null,1,5,X,[I(c)],null),Z.h(TE,hv(c))));return Rh(a,c)}
var cI=PF.c(new y(null,4,[oC,function(a,b){var c=null==a,d=c?a:""+C.c(a);if(z(b))c=new V(null,2,5,X,[d,!Ba(a)],null);else var e=ea(a),c=new V(null,2,5,X,[d,z(e)?e:c],null);return c},vE,function(a,b){return At(b)&&Ba(a)?new V(null,2,5,X,[null,!0],null):aI(a,$v)},wC,function(a,b){return At(b)&&Ba(a)?new V(null,2,5,X,[null,!0],null):aI(a,bw)},iB,function(a,b){var c=(c="string"===typeof a)?Ez(/^-?\d+$/,a):c;if(z(c)){if("string"!==typeof a)throw Error("Cannot read from non-string object.");c=xH(new hH(a,
[],-1),!1,null)}else c=null;return At(b)&&Ba(a)?new V(null,2,5,X,[null,!0],null):new V(null,2,5,X,[c,kw(c)],null)}],null));function dI(a,b,c){return F.h(N.c?N.c(cI):N.call(null,cI),a).call(null,bI(c,b),F.j(b,AE,!1))}function eI(a){return function(b,c){var d=gC.c(c);return T.j(b,DB.c(c),lw(N.c?N.c(cI):N.call(null,cI),d)?dI(d,c,a):new V(null,2,5,X,[bI(a,c),!0],null))}};var fI=function(){var a=new y(null,5,[oE,console.log.bind(console),zB,console.warn.bind(console),GD,console.error.bind(console),WA,z(console.group)?console.group.bind(console):console.log.bind(console),YD,z(console.groupEnd)?console.groupEnd.bind(console):function(){return iv}],null);return ox?ox(a):nx.call(null,a)}();function gI(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;hI(arguments[0],1<b.length?new H(b.slice(1),0,null):null)}
function hI(a,b){if(!lw(N.c?N.c(fI):N.call(null,fI),a))throw Error([C.c("Assert failed: "),C.c([C.c("re-frame: log called with unknown level: "),C.c(a)].join("")),C.c("\n"),C.c("(contains? (clojure.core/deref loggers) level)")].join(""));return U.h(function(){var b=N.c?N.c(fI):N.call(null,fI);return a.c?a.c(b):a.call(null,b)}(),b)};var iI=PF.c(Mv),jI=PF.c(Mv);function kI(a){a=I(F.h(a,bC));return Ba(a)?null:a}function lI(a,b){sx.j(jI,Lv,new V(null,2,5,X,[a,b],null))}function mI(a,b,c,d){Ba(c.id)&&hI(zB,R([[C.c("Component with selector '"),C.c(a),C.c("' has no ID ")].join(""),c],0));return dH(new V(null,1,5,X,[function(){return b.h?b.h(c,d):b.call(null,c,d)}],null),c)}function nI(a){return ow(function(a,c,d){var b=S(d,0,null);d=S(d,1,null);return z(d)?a:Lv.h(a,[C.c(Kw(c)),C.c(" -\x3e '"),C.c(b),C.c("'")].join(""))},Mv,a)}
function oI(a){return dx(gw,Z.h(function(a){S(a,0,null);a=S(a,1,null);S(a,0,null);return S(a,1,null)},a))}function pI(a,b,c,d){return b.h?b.h(c,d):b.call(null,c,d)}function qI(a,b,c,d,e){var g=S(d,0,null);d=S(d,1,null);d=Gt(eI(e),cx,d);z(oI(d))?a.N?a.N(c,g,e,d):a.call(null,c,g,e,d):(a=hI,e=e.id,c=SE(" ",nI(d)),b=ym("Invalid arguments for %s #%s: %s",b,e,c),a(zB,R([b],0)))}
function rI(){for(var a,b,c,d=G(new V(null,2,5,X,[new V(null,3,5,X,[pI,"control",jI],null),new V(null,3,5,X,[mI,"component",iI],null)],null)),e=null,g=0,h=0;;)if(h<g){a=e.aa(null,h);c=S(a,0,null);b=S(a,1,null);a=S(a,2,null);a=G(N.c?N.c(a):N.call(null,a));for(var k=null,l=0,m=0;;)if(m<l){var p=k.aa(null,m),u=S(p,0,null),A=S(p,1,null);S(p,2,null);for(var p=$H(document.querySelectorAll(u)),p=G(p),D=null,E=0,J=0;;)if(J<E){var M=D.aa(null,J);qI(c,b,u,A,M);J+=1}else if(p=G(p))D=p,cw(D)?(p=Ou(D),J=Pu(D),
D=p,E=Q(p),p=J):(p=I(D),qI(c,b,u,A,p),p=K(D),D=null,E=0),J=0;else break;m+=1}else if(a=G(a)){if(cw(a))l=Ou(a),a=Pu(a),k=l,l=Q(l);else{m=I(a);k=S(m,0,null);l=S(m,1,null);S(m,2,null);m=$H(document.querySelectorAll(k));m=G(m);u=null;for(p=A=0;;)if(p<A)D=u.aa(null,p),qI(c,b,k,l,D),p+=1;else if(m=G(m))u=m,cw(u)?(m=Ou(u),p=Pu(u),u=m,A=Q(m),m=p):(m=I(u),qI(c,b,k,l,m),m=K(u),u=null,A=0),p=0;else break;a=K(a);k=null;l=0}m=0}else break;h+=1}else if(b=G(d)){d=b;if(cw(d))c=Ou(d),d=Pu(d),b=c,c=Q(c),e=b,g=c;else{e=
I(d);c=S(e,0,null);b=S(e,1,null);a=S(e,2,null);e=G(N.c?N.c(a):N.call(null,a));g=null;for(a=h=0;;)if(a<h){m=g.aa(null,a);k=S(m,0,null);l=S(m,1,null);S(m,2,null);m=$H(document.querySelectorAll(k));m=G(m);u=null;for(p=A=0;;)if(p<A)D=u.aa(null,p),qI(c,b,k,l,D),p+=1;else if(m=G(m))u=m,cw(u)?(m=Ou(u),p=Pu(u),u=m,A=Q(m),m=p):(m=I(u),qI(c,b,k,l,m),m=K(u),u=null,A=0),p=0;else break;a+=1}else if(e=G(e)){if(cw(e))h=Ou(e),e=Pu(e),g=h,h=Q(h);else{a=I(e);g=S(a,0,null);h=S(a,1,null);S(a,2,null);a=$H(document.querySelectorAll(g));
a=G(a);k=null;for(m=l=0;;)if(m<l)u=k.aa(null,m),qI(c,b,g,h,u),m+=1;else if(a=G(a))k=a,cw(k)?(a=Ou(k),m=Pu(k),k=a,l=Q(a),a=m):(a=I(k),qI(c,b,g,h,a),a=K(k),k=null,l=0),m=0;else break;e=K(e);g=null;h=0}a=0}else break;d=K(d);e=null;g=0}h=0}else break};var kx=jx.h(sy,Mv);function sI(a,b){XF(a,b)}function tI(a,b){setTimeout(a,b)}function uI(a){return null!=a&&x===a.Yk?[C.c(z(ev.h?ev.h(OF,a):ev.call(null,OF,a))?"ra":z(ev.h?ev.h(TF,a):ev.call(null,TF,a))?"rc":z(ev.h?ev.h(YF,a):ev.call(null,YF,a))?"rx":z(ev.h?ev.h(SF,a):ev.call(null,SF,a))?"tr":"other"),C.c(cv(a))].join(""):null};function vI(a){var b=null!=a&&(a.C&64||x===a.Sb)?U.h(px,a):a;a=F.h(b,fD);var c=F.h(b,mE),b=F.h(b,ZC);return new y(null,3,[fD,z(a)?a:nA,mE,c,ZC,b],null)}function wI(a,b,c){return Ix(a,new V(null,2,5,X,[hD,b],null),c)}function xI(a,b){return Gx(a,new V(null,2,5,X,[JD,b],null))}function yI(a,b){for(var c=a;;){var d=Mx.c(c);if(Wv(d))return c;var e=Uv(d),g=sD.c(c),c=T.w(c,Mx,null==d?null:ju(d),R([sD,Lv.h(g,e)],0)),e=F.h(e,b),c=z(e)?e.c?e.c(c):e.call(null,c):c}};ox||nx.call(null,0);var zI=cx;ox||nx.call(null,zI);var AI=new tz(null,new y(null,4,[pB,null,YC,null,bE,null,jE,null],null),null),BI,CI=cx;BI=ox?ox(CI):nx.call(null,CI);function DI(a,b){return F.h(F.h(N.c?N.c(BI):N.call(null,BI),a),b)}function EI(a,b,c){sx.N(BI,Ix,new V(null,2,5,X,[a,b],null),c);return c}
var FI=function FI(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 0:return FI.T();case 1:return FI.c(arguments[0]);case 2:return FI.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};FI.T=function(){var a=cx;return rx.h?rx.h(BI,a):rx.call(null,BI,a)};FI.c=function(a){if(!z(AI.c?AI.c(a):AI.call(null,a)))throw Error("Assert failed: (kinds kind)");return sx.j(BI,Pv,a)};
FI.h=function(a,b){if(!z(AI.c?AI.c(a):AI.call(null,a)))throw Error("Assert failed: (kinds kind)");return z(DI(a,b))?sx.w(BI,Jx,new V(null,1,5,X,[a],null),Pv,R([b],0)):hI(zB,R(["re-frame: can't clear",""+C.c(a),"handler for",[C.c(b),C.c(". Handler not found.")].join("")],0))};FI.L=2;var GI=function GI(b,c){var d=G(c),e=I(d);if(d=K(d)){var g=F.h(b,e);return z(g)?(d=GI.h?GI.h(g,d):GI.call(null,g,d),G(d)?T.j(b,e,d):Pv.h(b,e)):b}return Pv.h(b,e)};function HI(a){return bw(a)?I(a):hI(GD,R(["re-frame: expected a vector, but got:",a],0))};var II=PF.c(cx);if(!z(AI.c?AI.c(YC):AI.call(null,YC)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");var JI=null;
function KI(a){var b=HI(a),b=DI(YC,b);if(z(b))if(z(JI))hI(GD,R([[C.c('re-frame: while handling "'),C.c(JI),C.c('", dispatch-sync was called for "'),C.c(a),C.c("\". You can't call dispatch-sync within an event handler.")].join("")],0));else{var c=JI;JI=a;try{var d=Ix(cx,new V(null,2,5,X,[JD,YC],null),a);var e=Lx(d,b);var g=yI(e,mE),h=Pv.h(g,Mx),k=sD.c(g);var l=Lx(h,k);yI(l,ZC)}finally{JI=c}}};var LI=new y(null,2,[HC,function(a){function b(){return Ub.c?Ub.c(a):Ub.call(null,a)}return eH.c?eH.c(b):eH.call(null,b)},tA,Ub],null),MI=function MI(b,c){if(null!=b&&null!=b.Ts)return b.Ts(0,c);var d=MI[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=MI._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IEventQueue.push",b);},NI=function NI(b,c){if(null!=b&&null!=b.op)return b.op(0,c);var d=NI[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=NI._;
if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("IEventQueue.-call-post-event-callbacks",b);};function OI(a,b,c){this.gk=a;this.Nb=b;this.Ow=c}function PI(a){for(var b=Q(a.Nb);;){if(0===b)return QI(a,tE,null);var c=ex(LI,Cy(Tv(Uv(a.Nb))));if(z(c))return QI(a,FD,c);RI(a);--b}}OI.prototype.Ts=function(a,b){return QI(this,qA,b)};function SI(a){a=function(a){return function(){return QI(a,oD,null)}}(a);return Ub.c?Ub.c(a):Ub.call(null,a)}
function QI(a,b,c){var d=function(){var d=new V(null,2,5,X,[a.gk,b],null);if(L.h(new V(null,2,5,X,[RA,qA],null),d))return new V(null,2,5,X,[zD,function(a,b){return function(){b.Nb=Lv.h(b.Nb,c);return SI(b)}}(d,a)],null);if(L.h(new V(null,2,5,X,[zD,qA],null),d))return new V(null,2,5,X,[zD,function(a,b){return function(){return b.Nb=Lv.h(b.Nb,c)}}(d,a)],null);if(L.h(new V(null,2,5,X,[zD,oD],null),d))return new V(null,2,5,X,[BC,function(a,b){return function(){return PI(b)}}(d,a)],null);if(L.h(new V(null,
2,5,X,[BC,qA],null),d))return new V(null,2,5,X,[BC,function(a,b){return function(){return b.Nb=Lv.h(b.Nb,c)}}(d,a)],null);if(L.h(new V(null,2,5,X,[BC,FD],null),d))return new V(null,2,5,X,[wA,function(a,b){return function(){return TI(b,c)}}(d,a)],null);if(L.h(new V(null,2,5,X,[BC,ID],null),d))return new V(null,2,5,X,[RA,function(a,b){return function(){b.Nb=kx;throw c;}}(d,a)],null);if(L.h(new V(null,2,5,X,[BC,tE],null),d))return Wv(a.Nb)?new V(null,1,5,X,[RA],null):new V(null,2,5,X,[zD,function(a,
b){return function(){return SI(b)}}(d,a)],null);if(L.h(new V(null,2,5,X,[wA,qA],null),d))return new V(null,2,5,X,[wA,function(a,b){return function(){return b.Nb=Lv.h(b.Nb,c)}}(d,a)],null);if(L.h(new V(null,2,5,X,[wA,eE],null),d))return new V(null,2,5,X,[BC,function(a,b){return function(){RI(b);return PI(b)}}(d,a)],null);throw iA([C.c("re-frame: router state transition not found. "),C.c(a.gk),C.c(" "),C.c(b)].join(""),new y(null,2,[fB,a.gk,$D,b],null));}();var e=S(d,0,null);d=S(d,1,null);a.gk=e;return z(d)?
d.T?d.T():d.call(null):null}function TI(a,b){var c=function(a){return function(){return QI(a,eE,null)}}(a);return b.c?b.c(c):b.call(null,c)}function RI(a){var b=Uv(a.Nb);try{KI(b);var c=a.Nb;var d=null==c?null:ju(c);a.Nb=d;a.op(0,b)}catch(e){QI(a,ID,e)}}
OI.prototype.op=function(a,b){for(var c,d=G(Dy(this.Ow)),e=null,g=0,h=0;;)if(h<g)c=e.aa(null,h),c.h?c.h(b,this.Nb):c.call(null,b,this.Nb),h+=1;else if(d=G(d))e=d,cw(e)?(d=Ou(e),e=Pu(e),c=d,g=Q(d),d=e,e=c):(c=I(e),c.h?c.h(b,this.Nb):c.call(null,b,this.Nb),d=K(e),e=null,g=0),h=0;else return null};var UI=new OI(RA,kx,cx);function VI(a){if(null==a)throw new hA('re-frame: you called "dispatch" without an event vector.',cx,null);MI(UI,a);return null}function WI(a){KI(a);NI(UI,a);return null};if(!z(AI.c?AI.c(jE):AI.call(null,jE)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");
var XI=hx.h(EI,jE),YI=vI(R([fD,TB,ZC,function(a){for(var b,c=G(hD.c(a)),d=null,e=0,g=0;;)if(g<e)a=d.aa(null,g),b=S(a,0,null),a=S(a,1,null),b=DI(jE,b),z(b)&&(b.c?b.c(a):b.call(null,a)),g+=1;else if(a=G(c))c=a,cw(c)?(d=Ou(c),c=Pu(c),a=d,b=Q(d),d=a,e=b):(a=I(c),b=S(a,0,null),a=S(a,1,null),d=DI(jE,b),z(d)&&(d.c?d.c(a):d.call(null,a)),c=K(c),d=null,e=0),g=0;else return null}],0)),ZI=function(a){return function(b){b=G(b);for(var c=null,d=0,e=0;;)if(e<d){var g=c.aa(null,e),h=null!=g&&(g.C&64||x===g.Sb)?
U.h(px,g):g;var k=h;var l=F.h(h,hE);var m=F.h(h,MC);Wv(m)||"number"!==typeof l?hI(GD,R(["re-frame: ignoring bad :dispatch-later value:",k],0)):tI(function(a,b,c,d,e,g,h,k,l){return function(){return VI(l)}}(b,c,d,e,g,h,k,l,m,a),l);e+=1}else if(h=G(b)){g=h;if(cw(g))b=Ou(g),l=Pu(g),c=b,m=Q(b),b=l,d=m;else{var p=I(g),u=null!=p&&(p.C&64||x===p.Sb)?U.h(px,p):p;k=u;l=F.h(u,hE);m=F.h(u,MC);Wv(m)||"number"!==typeof l?hI(GD,R(["re-frame: ignoring bad :dispatch-later value:",k],0)):tI(function(a,b,c,d,e,g,
h,k,l){return function(){return VI(l)}}(b,c,d,e,p,u,k,l,m,g,h,a),l);b=K(g);c=null;d=0}e=0}else return null}}(DE);XI.h?XI.h(DE,ZI):XI.call(null,DE,ZI);function $I(a){return bw(a)?VI(a):hI(GD,R(["re-frame: ignoring bad :dispatch value. Expected a vector, but got:",a],0))}XI.h?XI.h(MC,$I):XI.call(null,MC,$I);
function aJ(a){Zv(a)||hI(GD,R(["re-frame: ignoring bad :dispatch-n value. Expected a collection, got got:",a],0));a=G(a);for(var b=null,c=0,d=0;;)if(d<c){var e=b.aa(null,d);VI(e);d+=1}else if(a=G(a))b=a,cw(b)?(a=Ou(b),b=Pu(b),e=a,c=Q(a),a=b,b=e):(e=I(b),VI(e),a=K(b),b=null,c=0),d=0;else return null}XI.h?XI.h(eC,aJ):XI.call(null,eC,aJ);
function bJ(a){var b=hx.h(FI,YC);if(Zv(a)){a=G(Zv(a)?a:new V(null,1,5,X,[a],null));for(var c=null,d=0,e=0;;)if(e<d){var g=c.aa(null,e);b.c?b.c(g):b.call(null,g);e+=1}else if(a=G(a))c=a,cw(c)?(a=Ou(c),c=Pu(c),g=a,d=Q(a),a=c,c=g):(g=I(c),b.c?b.c(g):b.call(null,g),a=K(c),c=null,d=0),e=0;else return null}else return null}XI.h?XI.h(rC,bJ):XI.call(null,rC,bJ);function cJ(a){return rx.h?rx.h(II,a):rx.call(null,II,a)}XI.h?XI.h(oB,cJ):XI.call(null,oB,cJ);if(!z(AI.c?AI.c(bE):AI.call(null,bE)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");var dJ=hx.h(EI,bE);function eJ(a){return T.j(a,oB,N.c?N.c(II):N.call(null,II))}dJ.h?dJ.h(oB,eJ):dJ.call(null,oB,eJ);var fJ=function(a){return vI(R([fD,JD,mE,function(b){return Kx(b,JD,DI(bE,a))}],0))}(oB);function gJ(a,b){return L.h(a,b)?new V(null,3,5,X,[null,null,a],null):new V(null,3,5,X,[a,b,null],null)}function hJ(a){return G(a)?Gt(function(a,c){var b=S(c,0,null),e=S(c,1,null);return T.j(a,b,e)},fy(ux(U.h(uw,Cy(a)),xx(null))),a):null}
function iJ(a,b,c){var d=F.h(a,c),e=F.h(b,c),g=jJ.h?jJ.h(d,e):jJ.call(null,d,e),h=S(g,0,null),k=S(g,1,null),g=S(g,2,null);a=lw(a,c);b=lw(b,c);d=a&&b&&(null!=g||null==d&&null==e);return new V(null,3,5,X,[!a||null==h&&d?null:Ov([c,h]),!b||null==k&&d?null:Ov([c,k]),d?Ov([c,g]):null],null)}
var kJ=function kJ(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return kJ.h(arguments[0],arguments[1]);case 3:return kJ.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};kJ.h=function(a,b){return kJ.j(a,b,vF(Cy(a),Cy(b)))};kJ.j=function(a,b,c){return Gt(function(a,b){return Cz(Z.j(mz,a,b))},new V(null,3,5,X,[null,null,null],null),Z.h(hx.j(iJ,a,b),c))};kJ.L=3;
function lJ(a,b){return fy(Z.h(hJ,kJ.j(bw(a)?a:fy(a),bw(b)?b:fy(b),new Bz(null,0,function(){var c=Q(a),d=Q(b);return c>d?c:d}(),1,null))))}function mJ(a,b){return new V(null,3,5,X,[$w(xF.h(a,b)),$w(xF.h(b,a)),$w(wF.h(a,b))],null)}
var nJ=function nJ(b){if(null!=b&&null!=b.sv)return b.sv(b);var c=nJ[n(null==b?null:b)];if(null!=c)return c.c?c.c(b):c.call(null,b);c=nJ._;if(null!=c)return c.c?c.c(b):c.call(null,b);throw Bt("EqualityPartition.equality-partition",b);},oJ=function oJ(b,c){if(null!=b&&null!=b.rv)return b.rv(b,c);var d=oJ[n(null==b?null:b)];if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);d=oJ._;if(null!=d)return d.h?d.h(b,c):d.call(null,b,c);throw Bt("Diff.diff-similar",b);};nJ["null"]=function(){return ZD};
nJ.string=function(){return ZD};nJ.number=function(){return ZD};nJ.array=function(){return xB};nJ["function"]=function(){return ZD};nJ["boolean"]=function(){return ZD};nJ._=function(a){return(null!=a?a.C&1024||x===a.ir||(a.C?0:B(bu,a)):B(bu,a))?sE:(null!=a?a.C&4096||x===a.pv||(a.C?0:B(gu,a)):B(gu,a))?RD:(null!=a?a.C&16777216||x===a.ov||(a.C?0:B(zu,a)):B(zu,a))?xB:ZD};oJ["null"]=function(a,b){return gJ(a,b)};oJ.string=function(a,b){return gJ(a,b)};oJ.number=function(a,b){return gJ(a,b)};
oJ.array=function(a,b){return lJ(a,b)};oJ["function"]=function(a,b){return gJ(a,b)};oJ["boolean"]=function(a,b){return gJ(a,b)};oJ._=function(a,b){return function(){var b=nJ(a),b=b instanceof Y?b.Vb:null;switch(b){case "atom":return gJ;case "set":return mJ;case "sequential":return lJ;case "map":return kJ;default:throw Error([C.c("No matching clause: "),C.c(b)].join(""));}}().call(null,a,b)};function jJ(a,b){return L.h(a,b)?new V(null,3,5,X,[null,null,a],null):L.h(nJ(a),nJ(b))?oJ(a,b):gJ(a,b)};vI(R([fD,iC,mE,function(a){hI(oE,R(["Handling re-frame event:",xI(a,YC)],0));return a},ZC,function(a){var b=xI(a,YC),c=xI(a,oB);var d=Hx(a,new V(null,2,5,X,[hD,oB],null),sB);L.h(d,sB)?hI(oE,R(["No :db changes caused by:",b],0)):(d=jJ(c,d),c=S(d,0,null),d=S(d,1,null),null!=c||null!=d?(hI(WA,R(["db clojure.data/diff for:",b],0)),hI(oE,R(["only before:",c],0)),hI(oE,R(["only after :",d],0)),gI(YD)):hI(oE,R(["no app-db changes caused by:",b],0)));return a}],0));
var pJ=vI(R([fD,KC,mE,function(a){return Ix(Jx.N(a,new V(null,2,5,X,[JD,YC],null),iy,1),new V(null,2,5,X,[JD,qE],null),xI(a,YC))},ZC,function(a){return Ix(GI(a,new V(null,2,5,X,[JD,qE],null)),new V(null,2,5,X,[JD,YC],null),xI(a,qE))}],0));function qJ(a){return vI(R([fD,VC,mE,function(b){var c=JD.c(b),d=null!=c&&(c.C&64||x===c.Sb)?U.h(px,c):c,c=F.h(d,oB),d=F.h(d,YC);return wI(b,oB,a.h?a.h(c,d):a.call(null,c,d))}],0))}
function rJ(a){return vI(R([fD,iE,ZC,function(b){var c=xI(b,YC);var d=Gx(b,new V(null,2,5,X,[hD,oB],null));d=z(d)?d:xI(b,oB);return wI(b,oB,a.h?a.h(d,c):a.call(null,d,c))}],0))}function sJ(a){return vI(R([fD,ZC,ZC,function(b){var c=Gx(b,new V(null,2,5,X,[hD,oB],null));c=z(c)?c:xI(b,oB);var d=xI(b,YC);a.h?a.h(c,d):a.call(null,c,d);return b}],0))};if(!z(AI.c?AI.c(pB):AI.call(null,pB)))throw Error("Assert failed: (re-frame.registrar/kinds kind)");var tJ,uJ=cx;tJ=ox?ox(uJ):nx.call(null,uJ);function vJ(a,b,c){a=new V(null,2,5,X,[a,b],null);sI(c,function(a){return function(){sx.j(tJ,Pv,a);return null}}(a));sx.N(tJ,T,a,c);return c}function wJ(a,b){return F.h(N.c?N.c(tJ):N.call(null,tJ),new V(null,2,5,X,[a,b],null))}
var xJ=function xJ(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return xJ.c(arguments[0]);case 2:return xJ.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};
xJ.c=function(a){var b=wJ(a,Mv);if(z(b))return b;var c=HI(a);b=DI(pB,c);return null==b?hI(GD,R([[C.c('re-frame: no subscription handler registered for: "'),C.c(c),C.c('". Returning a nil subscription.')].join("")],0)):vJ(a,Mv,b.h?b.h(II,a):b.call(null,II,a))};
xJ.h=function(a,b){var c=wJ(a,b);if(z(c))return c;var d=HI(a);var e=DI(pB,d);if(null==e)return hI(GD,R([[C.c('re-frame: no subscription handler registered for: "'),C.c(d),C.c('". Returning a nil subscription.')].join("")],0));var g=RF(function(){return function(){return Dx(N,b)}}(d,e,c));var h=RF(function(b,c,d){return function(){var c=N.c?N.c(b):N.call(null,b);return d.j?d.j(II,a,c):d.call(null,II,a,c)}}(g,d,e,c));return vJ(a,b,RF(function(a,b){return function(){var a=N.c?N.c(b):N.call(null,b);return N.c?
N.c(a):N.call(null,a)}}(g,h,d,e,c)))};xJ.L=2;function yJ(a){return jx.j(null==a?null:Mt(a),Z.c(function(a){var b=S(a,0,null);a=S(a,1,null);return new V(null,2,5,X,[b,N.c?N.c(a):N.call(null,a)],null)}),a)}function zJ(a,b){if(Zv(a))var c=Z.h(N,a);else $v(a)?c=yJ(a):(c=null!=a?a.C&32768||x===a.fr?!0:a.C?!1:B(mu,a):B(mu,a),c=z(c)?N.c?N.c(a):N.call(null,a):hI(GD,R(["re-frame: in the reg-sub for ",b,", the input-signals function returns: ",a],0)));return c}
function AJ(a){for(var b=[],c=arguments.length,d=0;;)if(d<c)b.push(arguments[d]),d+=1;else break;return BJ(arguments[0],1<b.length?new H(b.slice(1),0,null):null)}
function BJ(a,b){var c=Kv(b),d=yz(b),e=[C.c("re-frame: reg-sub for "),C.c(a),C.c(", ")].join(""),g=function(){var a=Q(d);switch(a){case 0:return function(){return function(){var a=null,a=function(a,b){switch(arguments.length){case 1:return II;case 2:return II}throw Error("Invalid arity: "+(arguments.length-1));};a.c=function(){return II};a.h=function(){return II};return a}()}(a,c,d,e);case 1:return a=I(d),Qv(a)||hI(GD,R([e,"2nd argument expected to be an inputs function, got:",a],0)),a;case 2:var b=
S(d,0,null),g=S(d,1,null);L.h(lB,b)||hI(GD,R([e,"expected :\x3c-, got:",b],0));return function(a,b,c){return function(){function a(){return xJ.c(c)}function b(){return xJ.c(c)}var d=null,d=function(c,d){switch(arguments.length){case 1:return b.call(this);case 2:return a.call(this)}throw Error("Invalid arity: "+(arguments.length-1));};d.c=b;d.h=a;return d}()}(d,b,g,a,c,d,e);default:var b=Ex(2,2,d),g=Z.h(I,b),m=Z.h(Kv,b);dx(new tz(null,new y(null,1,[lB,null],null),null),g)&&dx(bw,m)||hI(GD,R([e,"expected pairs of :\x3c- and vectors, got:",
b],0));return function(a,b,c){return function(){function a(){return Z.h(xJ,c)}function b(){return Z.h(xJ,c)}var d=null,d=function(c,d){switch(arguments.length){case 1:return b.call(this);case 2:return a.call(this)}throw Error("Invalid arity: "+(arguments.length-1));};d.c=b;d.h=a;return d}()}(b,g,m,a,c,d,e)}}();return EI(pB,a,function(b,c,d,e){return function(){function g(g,h,k){var l=e.h?e.h(h,k):e.call(null,h,k);g=ox?ox(null):nx.call(null,null);var l=RF(function(b,c,d){return function(){var c=zJ(b,
a);return d.j?d.j(c,h,k):d.call(null,c,h,k)}}(l,g,b,c,d,e)),m=uI(l);rx.h?rx.h(g,m):rx.call(null,g,m);return l}function h(g,h){var k=e.c?e.c(h):e.call(null,h),l=ox?ox(null):nx.call(null,null),k=RF(function(b,c,d){return function(){var c=zJ(b,a);return d.h?d.h(c,h):d.call(null,c,h)}}(k,l,b,c,d,e)),m=uI(k);rx.h?rx.h(l,m):rx.call(null,l,m);return k}var k=null,k=function(a,b,c){switch(arguments.length){case 2:return h.call(this,0,b);case 3:return g.call(this,0,b,c)}throw Error("Invalid arity: "+(arguments.length-
1));};k.h=h;k.j=g;return k}()}(c,d,e,g))};hx.h(FI,pB);hx.h(FI,jE);hx.h(FI,bE);hx.h(FI,YC);var CJ=function CJ(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 2:return CJ.h(arguments[0],arguments[1]);case 3:return CJ.j(arguments[0],arguments[1],arguments[2]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};CJ.h=function(a,b){return CJ.j(a,null,b)};CJ.j=function(a,b,c){b=new V(null,4,5,X,[fJ,YI,b,qJ(c)],null);return EI(YC,a,Ax(yt,Cx(b)))};CJ.L=3;function DJ(){var a=I(Ax(VE,Z.h(function(a){return Rh(a,"initial")},Ax(yt,new V(null,2,5,X,[document.querySelector("body"),document.querySelector("#wrapper")],null)))));z(a)&&(a=new V(null,2,5,X,[sC,eA(qd(a),R([fA,!0],0))],null),WI.c?WI.c(a):WI.call(null,a));rI();a=new V(null,1,5,X,[yC],null);WI.c?WI.c(a):WI.call(null,a);a=new V(null,1,5,X,[NA],null);return WI.c?WI.c(a):WI.call(null,a)};function EJ(a){return bw(a)&&L.h(2,Q(a))&&Zw.h(eu(a),xA)}function FJ(a,b,c){return Ix(a,new V(null,3,5,X,[LD,b,iD],null),c)}function GJ(a,b){return ow(function(a,b,e){return Ix(a,new V(null,3,5,X,[PB,b,LB],null),e)},a,b)};function HJ(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null),h=S(b,2,null);return Jx.j(a,new V(null,3,5,X,[LD,h,yA],null),function(a,b,c,d){return function(){var a=I(Ax(yt,new V(null,3,5,X,[jA.c(d),d,cx],null)));return jx.h(cx,zx(EJ,Ww.h(Gx(a,new V(null,2,5,X,[UC,GD],null)),Gx(a,new V(null,2,5,X,[UC,yA],null)))))}}(b,c,d,e,g,h))}rJ.c?rJ.c(HJ):rJ.call(null,HJ);function IJ(a,b){S(b,0,null);var c=S(b,1,null);return FJ(a,c,!0)}rJ.c?rJ.c(IJ):rJ.call(null,IJ);
function JJ(a,b){S(b,0,null);var c=S(b,1,null);S(c,0,null);S(c,1,null);c=S(b,2,null);return FJ(a,c,!1)}rJ.c?rJ.c(JJ):rJ.call(null,JJ);function KJ(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null),h=S(b,2,null);return setTimeout(function(a,b,c,d,e,g){return function(){var a=new V(null,2,5,X,[KB,g],null);return VI.c?VI.c(a):VI.call(null,a)}}(b,c,d,e,g,h),15E3)}sJ.c?sJ.c(KJ):sJ.call(null,KJ);
function LJ(a,b){S(b,0,null);var c=S(b,1,null);S(c,0,null);S(c,1,null);c=S(b,2,null);return null==c?a:Ix(a,new V(null,3,5,X,[LD,c,yA],null),null)}rJ.c?rJ.c(LJ):rJ.call(null,LJ);function MJ(a,b){S(b,0,null);var c=S(b,1,null),d=S(b,2,null);return Ix(a,new V(null,2,5,X,[LD,fD.c(c)],null),GJ(c,d))}var NJ=rJ.c?rJ.c(MJ):rJ.call(null,MJ);CJ.j(KB,pJ,function(a,b){var c=S(b,0,null);return FJ(a,c,!1)});CJ.j(qD,new V(null,1,5,X,[NJ],null),function(a,b){S(b,0,null);S(b,1,null);S(b,2,null);return a});function OJ(a){return Gx(a,new V(null,2,5,X,[ZB,ZB],null))}AJ.h?AJ.h(tD,OJ):AJ.call(null,tD,OJ);function PJ(a){return Gx(a,new V(null,2,5,X,[ZB,XB],null))}AJ.h?AJ.h(QB,PJ):AJ.call(null,QB,PJ);function QJ(a){return Hx(a,new V(null,2,5,X,[UB,cB],null),0)}AJ.h?AJ.h(XD,QJ):AJ.call(null,XD,QJ);CJ.h(yC,function(a){return Ix(a,new V(null,2,5,X,[UB,pA],null),(new fs(document)).get(Kw(Hx(a,new V(null,2,5,X,[UB,kE],null),VD))))});var RJ=new y(null,1,[UC,Mv],null);function SJ(a,b,c,d,e,g,h){this.id=a;this.text=b;this.severity=c;this.timeout=d;this.Pa=e;this.ea=g;this.M=h;this.C=2229667594;this.Z=8192}f=SJ.prototype;f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){switch(b instanceof Y?b.Vb:null){case "id":return this.id;case "text":return this.text;case "severity":return this.severity;case "timeout":return this.timeout;default:return F.j(this.ea,b,c)}};
f.ja=function(a,b,c){return Gz(b,function(){return function(a){return Gz(b,Oz,""," ","",c,a)}}(this),"#common.flash.db.FlashMessage{",", ","}",c,Ww.h(new V(null,4,5,X,[new V(null,2,5,X,[fD,this.id],null),new V(null,2,5,X,[JE,this.text],null),new V(null,2,5,X,[pD,this.severity],null),new V(null,2,5,X,[TD,this.timeout],null)],null),this.ea))};f.lc=function(){return new wy(0,this,4,new V(null,4,5,X,[fD,JE,pD,TD],null),z(this.ea)?Su(this.ea):ax())};f.sa=function(){return this.Pa};
f.ya=function(){return 4+Q(this.ea)};f.ha=function(){var a=this.M;return null!=a?a:this.M=a=zw(this)};f.P=function(a,b){return z(z(b)?this.constructor===b.constructor&&vy(this,b):b)?!0:!1};f.Df=function(a,b){return lw(new tz(null,new y(null,4,[fD,null,pD,null,TD,null,JE,null],null),null),b)?Pv.h(ru(jx.h(cx,this),this.Pa),b):new SJ(this.id,this.text,this.severity,this.timeout,this.Pa,$w(Pv.h(this.ea,b)),null)};
f.Rb=function(a,b,c){return z(Iw.h?Iw.h(fD,b):Iw.call(null,fD,b))?new SJ(c,this.text,this.severity,this.timeout,this.Pa,this.ea,null):z(Iw.h?Iw.h(JE,b):Iw.call(null,JE,b))?new SJ(this.id,c,this.severity,this.timeout,this.Pa,this.ea,null):z(Iw.h?Iw.h(pD,b):Iw.call(null,pD,b))?new SJ(this.id,this.text,c,this.timeout,this.Pa,this.ea,null):z(Iw.h?Iw.h(TD,b):Iw.call(null,TD,b))?new SJ(this.id,this.text,this.severity,c,this.Pa,this.ea,null):new SJ(this.id,this.text,this.severity,this.timeout,this.Pa,T.j(this.ea,
b,c),null)};f.oa=function(){return G(Ww.h(new V(null,4,5,X,[new V(null,2,5,X,[fD,this.id],null),new V(null,2,5,X,[JE,this.text],null),new V(null,2,5,X,[pD,this.severity],null),new V(null,2,5,X,[TD,this.timeout],null)],null),this.ea))};f.za=function(a,b){return new SJ(this.id,this.text,this.severity,this.timeout,b,this.ea,this.M)};f.xa=function(a,b){return bw(b)?this.Rb(null,Qt.h(b,0),Qt.h(b,1)):Gt(Ot,this,b)};
function TJ(a,b,c){return new SJ(Date.now(),b,F.j(new y(null,1,[yA,GD],null),a,a),c,null,null,null)}function UJ(a,b){return TJ(a,b,L.h(a,GD)?0:3E3)}function VJ(a,b,c,d){return Jx.j(a,new V(null,2,5,X,[WB,UC],null),function(a){return Lv.h(a,TJ(b,c,d))})}function WJ(a,b,c){return Jx.j(a,new V(null,2,5,X,[WB,UC],null),function(a){return Lv.h(a,UJ(b,c))})}function XJ(a){return Gt(function(a,c){return Lv.h(a,UJ(GD,c))},Mv,Gx(a,new V(null,3,5,X,[UC,GD,xA],null)))}
function YJ(a){var b=S(a,0,null),c=S(a,1,null);return"string"===typeof c?new V(null,1,5,X,[UJ(Jw.c(b),c)],null):Zv(c)?Z.h(function(a,b){return function(a){return UJ(Jw.c(b),a)}}(a,b,c),c):Mv}function ZJ(a){a=GD.c(a);return"string"===typeof a?new V(null,1,5,X,[UJ(GD,a)],null):Mv}function $J(a,b){return Ww.w(b,yx(pw,R([Z.h(YJ,F.j(a,UC,cx))],0)),R([XJ(a),ZJ(a)],0))};function aK(a){return Jx.j(a,new V(null,2,5,X,[UB,cB],null),qv)}rJ.c?rJ.c(aK):rJ.call(null,aK);function bK(a){return Jx.j(a,new V(null,2,5,X,[UB,cB],null),function(a){return 0<a?a-1:0})}var cK=rJ.c?rJ.c(bK):rJ.call(null,bK);var dK=PF.c(Mv);CJ.h(NA,function(a){return Gt(function(a,c){return c.c?c.c(a):c.call(null,a)},a,N.c?N.c(dK):N.call(null,dK))});CJ.j(sA,new V(null,1,5,X,[cK],null),function(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null);return Jx.j(a,new V(null,2,5,X,[WB,UC],null),function(a,b,c,d){return function(a){return $J(I(Ax(yt,new V(null,3,5,X,[jA.c(d),d,cx],null))),a)}}(b,c,d,e,g))});
CJ.h(uB,function(a,b){S(b,0,null);var c=S(b,1,null),d=S(c,0,null);S(c,1,null);c=S(c,2,null);return L.h(-1,c)?WJ(a,GD,[C.c("Zdá se, že jste off-line. Prosíme, připojte se k Internetu a zkuste"),C.c(" to znovu")].join("")):L.h(401,c)?WJ(a,GD,"Vaše přihlášení vypršelo, prosíme, aktualizujte stránku."):L.h(404,c)?window.location.reload():L.h(429,c)?WJ(a,GD,Hx(d,new V(null,2,5,X,[UC,GD],null),"Příliš mnoho požadavků.")):L.h(502,c)||L.h(503,c)?(d=new V(null,2,5,X,[LE,new V(null,2,5,X,["error","not-available"],
null)],null),VI.c?VI.c(d):VI.call(null,d)):400<=c?WJ(a,GD,[C.c("Objevila se interní chyba. Prosíme, aktualizujte stránku a zkuste"),C.c(" to znovu")].join("")):a});function eK(a){return Hx(a,new V(null,2,5,X,[WB,UC],null),Mv)}AJ.h?AJ.h(EA,eK):AJ.call(null,EA,eK);function fK(a,b){return SE(" ",jx.h(new V(null,1,5,X,[a],null),Z.h(function(b){return[C.c(a),C.c("--"),C.c(b)].join("")},b)))};function gK(a){return function(){var b=new V(null,2,5,X,[vD,a],null);return VI.c?VI.c(b):VI.call(null,b)}}var hK=function hK(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;return hK.w(arguments[0],arguments[1],2<c.length?new H(c.slice(2),0,null):null)};
hK.w=function(a,b,c){return jx.h(new V(null,2,5,X,[JC,new y(null,1,[gD,fK([C.c(b),C.c("__msg")].join(""),new V(null,1,5,X,[Kw(pD.c(a))],null))],null)],null),Ww.h(new V(null,1,5,X,[new V(null,3,5,X,[JC,new y(null,1,[gD,[C.c(b),C.c("__content")].join("")],null),JE.c(a)],null)],null),c))};hK.L=2;hK.O=function(a){var b=I(a),c=K(a);a=I(c);c=K(c);return hK.w(b,a,c)};
function iK(a,b){return tG(new y(null,3,[xD,"close-button",OB,function(){if(0<TD.c(a)){var b=gK(fD.c(a)),d=TD.c(a);return setTimeout(b,d)}return null},$C,function(a){return new V(null,3,5,X,[JC,new y(null,2,[gD,[C.c(b),C.c("__close")].join(""),zC,gK(fD.c(a))],null),Oa("\x26times;")],null)}],null))}function jK(a,b){return new V(null,4,5,X,[hK,a,b,new V(null,3,5,X,[iK,a,b],null)],null)}
function kK(a){return new V(null,3,5,X,[JC,new y(null,1,[gD,fK([C.c(a),C.c("__msg")].join(""),new V(null,1,5,X,["loading"],null))],null),new V(null,3,5,X,[JC,new y(null,1,[gD,[C.c(a),C.c("__content")].join("")],null),"Načítám..."],null)],null)}
function lK(a,b,c){return new V(null,4,5,X,[JC,new y(null,1,[gD,c],null),0<b?new V(null,2,5,X,[kK,c],null):null,G(a)?function(){return function e(a){return new Lw(null,function(){for(;;){var b=G(a);if(b){if(cw(b)){var g=Ou(b),l=Q(g),m=Pw(l);a:for(var p=0;;)if(p<l){var u=Qt.h(g,p),u=Sv(new V(null,3,5,X,[jK,u,c],null),new y(null,1,[eB,fD.c(u)],null));m.add(u);p+=1}else{g=!0;break a}return g?Rw(m.sb(),e(Pu(b))):Rw(m.sb(),null)}m=I(b);return Ev(Sv(new V(null,3,5,X,[jK,m,c],null),new y(null,1,[eB,fD.c(m)],
null)),e(hv(b)))}return null}},null,null)}(a)}():null],null)};function mK(a,b){var c=S(b,0,null),d=S(b,1,null),e=S(d,0,null),g=S(d,1,null);return Jx.j(a,new V(null,2,5,X,[WB,UC],null),function(a,b,c,d){return function(a){return $J(I(Ax(yt,new V(null,3,5,X,[jA.c(d),d,cx],null))),a)}}(b,c,d,e,g))}rJ.c?rJ.c(mK):rJ.call(null,mK);function nK(a){return Ix(a,new V(null,2,5,X,[WB,UC],null),Mv)}rJ.c?rJ.c(nK):rJ.call(null,nK);
function oK(a,b){var c=S(b,0,null),d=S(b,1,null);return Jx.j(a,new V(null,2,5,X,[WB,UC],null),function(a,b,c){return function(d){return Ax(function(a,b,c){return function(a){return L.h(fD.c(a),c)}}(a,b,c),d)}}(b,c,d))}var pK=rJ.c?rJ.c(oK):rJ.call(null,oK);function qK(a,b){var c=S(b,0,null),d=S(b,1,null);return Gt(function(){return function(a,b){return WJ(a,Jw.c(function(){var a=pD.c(b);return z(a)?a:rD}()),JE.c(b))}}(b,c,d),a,d)}var rK=rJ.c?rJ.c(qK):rJ.call(null,qK);CJ.j(vD,new V(null,1,5,X,[pK],null),function(a,b){S(b,0,null);S(b,1,null);return a});CJ.h(FC,function(a,b){S(b,0,null);var c=S(b,1,null),d=S(b,2,null),e=S(b,3,null);return VJ(a,c,d,e)});CJ.j(EE,rK,function(a,b){S(b,0,null);S(b,1,null);return a});sx.j(iI,Lv,new V(null,2,5,X,["#sticky-flash-cljs",new V(null,2,5,X,[function(a,b){var c=function(){var a=new V(null,1,5,X,[XD],null);return xJ.c?xJ.c(a):xJ.call(null,a)}(),d=function(){var a=new V(null,1,5,X,[EA],null);return xJ.c?xJ.c(a):xJ.call(null,a)}(),e=kI(b);return tG(new y(null,3,[xD,"flash-component",OB,function(a,b,c){return function(){if(G(c)){var a=new V(null,2,5,X,[EE,c],null);return VI.c?VI.c(a):VI.call(null,a)}return null}}(c,d,e),$C,function(a,b){return function(){return new V(null,
4,5,X,[lK,N.c?N.c(b):N.call(null,b),N.c?N.c(a):N.call(null,a),"common-flash"],null)}}(c,d,e)],null))},new V(null,1,5,X,[new y(null,3,[DB,bC,AE,!1,gC,wC],null)],null)],null)],null));var sK=null!=window.ga;function tK(a){var b=a.className,c=[C.c(window.location.pathname),C.c(window.location.hash)].join(""),d=Rh(a,"analytics");a=Ba(d)?Rf(a):SE("_",new V(null,2,5,X,[d,Rf(a)],null));return new y(null,4,[DC,"event",VA,b,wB,c,rB,a],null)}function uK(a){a=a.currentTarget;a=mz.w(R([new y(null,1,[wB,a.href],null),tK(a)],0));return z(sK)?U.h(window.ga,R(["send",a],0)):hI(oE,R(["GA event: ",a],0))};function vK(a,b){S(b,0,null);var c=S(b,1,null);return v(c,"click",uK)}var wK=sJ.c?sJ.c(vK):sJ.call(null,vK);CJ.j(cE,wK,function(a,b){S(b,0,null);S(b,1,null);return a});lI("a",new V(null,1,5,X,[function(a){return v(a,"click",uK)}],null));lI(".collapsible",new V(null,1,5,X,[function(a){return v(a,"click",function(){return function(){return eh(a,"expanded")}}(a,"click"))}],null));lI(".collapsible button",new V(null,1,5,X,[function(a){return a.disabled=!1}],null));if(L.h("default","nodejs")){var xK=require("@pupeno/xmlhttprequest").XMLHttpRequest;global.XMLHttpRequest=xK};var yK="undefined"!=typeof Object.keys?function(a){return Object.keys(a)}:function(a){return Hb(a)},zK="undefined"!=typeof Array.isArray?function(a){return Array.isArray(a)}:function(a){return"array"===n(a)};function AK(){Math.round(15*Math.random()).toString(16)};var BK=1;function CK(a,b){if(null==a)return null==b;if(a===b)return!0;if("object"===typeof a){if(zK(a)){if(zK(b)&&a.length===b.length){for(var c=0;c<a.length;c++)if(!CK(a[c],b[c]))return!1;return!0}return!1}if(a.bi)return a.bi(b);if(null!=b&&"object"===typeof b){if(b.bi)return b.bi(a);var c=0,d=yK(b).length,e;for(e in a)if(a.hasOwnProperty(e)&&(c++,!b.hasOwnProperty(e)||!CK(a[e],b[e])))return!1;return c===d}}return!1}function DK(a,b){return a^b+2654435769+(a<<6)+(a>>2)}var EK={},FK=0;
function GK(a){var b=0;if(null!=a.forEach)a.forEach(function(a,c){b=(b+(HK(c)^HK(a)))%4503599627370496});else for(var c=yK(a),d=0;d<c.length;d++)var e=c[d],g=a[e],b=(b+(HK(e)^HK(g)))%4503599627370496;return b}function IK(a){var b=0;if(zK(a))for(var c=0;c<a.length;c++)b=DK(b,HK(a[c]));else a.forEach&&a.forEach(function(a){b=DK(b,HK(a))});return b}
function HK(a){if(null==a)return 0;switch(typeof a){case "number":return a;case "boolean":return!0===a?1:0;case "string":var b=EK[a];if(null==b){for(var c=b=0;c<a.length;++c)b=31*b+a.charCodeAt(c),b%=4294967296;FK++;256<=FK&&(EK={},FK=1);EK[a]=b}a=b;return a;case "function":return b=a.transit$hashCode$,b||(b=BK,"undefined"!=typeof Object.defineProperty?Object.defineProperty(a,"transit$hashCode$",{value:b,enumerable:!1}):a.transit$hashCode$=b,BK++),b;default:return a instanceof Date?a.valueOf():zK(a)?
IK(a):a.Nm?a.Nm():GK(a)}};it("9007199254740991");it("-9007199254740991");Ys.prototype.equiv=function(a){return CK(this,a)};Ys.prototype.equiv=Ys.prototype.equiv;Ys.prototype.bi=function(a){return a instanceof Ys&&this.tb(a)};Ys.prototype.Nm=function(){return this.nj()};Date.prototype.bi=function(a){return a instanceof Date?this.valueOf()===a.valueOf():!1};Date.prototype.Nm=function(){return this.valueOf()};(8|3&Math.round(14*Math.random())).toString(16);AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();AK();gA.prototype.P=function(a,b){return b instanceof gA?this.vf===b.vf:!1};Ys.prototype.P=function(a,b){return this.equiv(b)};Ys.prototype.iv=x;Ys.prototype.ha=function(){return HK(this)};var JK=Mv;ox||nx.call(null,JK);function KK(a){return Uz(R(["CLJS-AJAX response:",a],0))}ox||nx.call(null,KK);function LK(a){return"undefined"!==typeof console?console.error(a):"undefined"!==typeof window?window.alert(""+C.c(a)):Uz(R(["CLJS-AJAX ERROR:",a],0))}ox||nx.call(null,LK);PF.c(null);PF.c(null);function MK(a,b,c,d,e,g){this.value=a;this.left=b;this.right=c;this.Pa=d;this.ea=e;this.M=g;this.C=2229667594;this.Z=8192}f=MK.prototype;f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){switch(b instanceof Y?b.Vb:null){case "value":return this.value;case "left":return this.left;case "right":return this.right;default:return F.j(this.ea,b,c)}};
f.ja=function(a,b,c){return Gz(b,function(){return function(a){return Gz(b,Oz,""," ","",c,a)}}(this),"#linked.map.Node{",", ","}",c,Ww.h(new V(null,3,5,X,[new V(null,2,5,X,[LB,this.value],null),new V(null,2,5,X,[HE,this.left],null),new V(null,2,5,X,[yD,this.right],null)],null),this.ea))};f.lc=function(){return new wy(0,this,3,new V(null,3,5,X,[LB,HE,yD],null),z(this.ea)?Su(this.ea):ax())};f.sa=function(){return this.Pa};f.ya=function(){return 3+Q(this.ea)};
f.ha=function(){var a=this.M;return null!=a?a:this.M=a=zw(this)};f.P=function(a,b){return z(z(b)?this.constructor===b.constructor&&vy(this,b):b)?!0:!1};f.Df=function(a,b){return lw(new tz(null,new y(null,3,[LB,null,yD,null,HE,null],null),null),b)?Pv.h(ru(jx.h(cx,this),this.Pa),b):new MK(this.value,this.left,this.right,this.Pa,$w(Pv.h(this.ea,b)),null)};
f.Rb=function(a,b,c){return z(Iw.h?Iw.h(LB,b):Iw.call(null,LB,b))?new MK(c,this.left,this.right,this.Pa,this.ea,null):z(Iw.h?Iw.h(HE,b):Iw.call(null,HE,b))?new MK(this.value,c,this.right,this.Pa,this.ea,null):z(Iw.h?Iw.h(yD,b):Iw.call(null,yD,b))?new MK(this.value,this.left,c,this.Pa,this.ea,null):new MK(this.value,this.left,this.right,this.Pa,T.j(this.ea,b,c),null)};
f.oa=function(){return G(Ww.h(new V(null,3,5,X,[new V(null,2,5,X,[LB,this.value],null),new V(null,2,5,X,[HE,this.left],null),new V(null,2,5,X,[yD,this.right],null)],null),this.ea))};f.za=function(a,b){return new MK(this.value,this.left,this.right,b,this.ea,this.M)};f.xa=function(a,b){return bw(b)?this.Rb(null,Qt.h(b,0),Qt.h(b,1)):Gt(Ot,this,b)};function NK(a,b){this.head=a;this.Bd=b;this.C=2314602255;this.Z=8192}f=NK.prototype;
f.toString=function(){var a=this;return[C.c("{"),C.c(SE(", ",function(){return function(a){return function d(b){return new Lw(null,function(){return function(){for(;;){var a=G(b);if(a){if(cw(a)){var e=Ou(a),k=Q(e),l=Pw(k);a:for(var m=0;;)if(m<k){var p=Qt.h(e,m),u=S(p,0,null),p=S(p,1,null),u=[C.c(u),C.c(" "),C.c(p)].join("");l.add(u);m+=1}else{e=!0;break a}return e?Rw(l.sb(),d(Pu(a))):Rw(l.sb(),null)}e=I(a);l=S(e,0,null);e=S(e,1,null);return Ev([C.c(l),C.c(" "),C.c(e)].join(""),d(hv(a)))}return null}}}(a),
null,null)}}(a)(a)}())),C.c("}")].join("")};f.equiv=function(a){return this.P(null,a)};f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){a=mw(this.Bd,b);return z(a)?LB.c(fu(a)):c};f.Kg=function(a,b,c){return Gt(function(){return function(a,c){return U.h(hx.h(b,a),c)}}(this),c,G(this))};f.ja=function(a,b){return Eu(b,[C.c("#linked/map "),C.c(jx.h(Mv,this))].join(""))};f.sa=function(){return Tv(this.Bd)};f.ya=function(){return Q(this.Bd)};
f.Lg=function(){return OK.c?OK.c(this):OK.call(null,this)};f.ha=function(){return cv(jx.h(cx,this))};f.P=function(a,b){return vy(this,b)};f.Ka=function(){return ru(PK,Tv(this.Bd))};f.Df=function(a,b){return QK.h?QK.h(this,b):QK.call(null,this,b)};f.Rb=function(a,b,c){return RK.j?RK.j(this,b,c):RK.call(null,this,b,c)};f.Vh=function(a,b){return lw(this.Bd,b)};f.oa=function(){return SK.c?SK.c(this):SK.call(null,this)};f.za=function(a,b){return new NK(this.head,Sv(this.Bd,b))};
f.xa=function(a,b){if(bw(b))return this.Rb(null,Qt.h(b,0),Qt.h(b,1));for(var c=this,d=G(b);;){if(null==d)return c;var e=I(d);if(bw(e))c=c.Rb(null,Qt.h(e,0),Qt.h(e,1)),d=K(d);else throw Error("conj on a map takes map entries or seqables of map entries");}};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.Y(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.Y(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.Y(null,a,b)};
function RK(a,b,c){var d=a.head;a=a.Bd;var e=mw(a,b);if(z(e))return new NK(d,Ix(a,new V(null,2,5,X,[b,LB],null),c));if(Wv(a))return new NK(b,T.j(a,b,new MK(c,b,b,null,null,null)));e=Gx(a,new V(null,2,5,X,[d,HE],null));return new NK(d,Ix(Ix(T.j(a,b,new MK(c,e,d,null,null,null)),new V(null,2,5,X,[d,HE],null),b),new V(null,2,5,X,[e,yD],null),b))}
function QK(a,b){var c=a.head,d=a.Bd,e=mw(d,b);if(z(e)){if(L.h(1,Q(d)))return null==a?null:Mt(a);var g=yD.c(fu(e)),e=HE.c(fu(e)),c=L.h(b,c)?g:c;return new NK(c,Ix(Ix(Pv.h(d,b),new V(null,2,5,X,[g,HE],null),e),new V(null,2,5,X,[e,yD],null),g))}return a}function TK(a,b){return new V(null,2,5,X,[a,b],null)}
var UK=function UK(b,c,d,e){var g=mw(b,c),h=S(g,0,null),k=S(g,1,null),l=TK(h,LB.c(k)),m=e.c?e.c(k):e.call(null,k);return L.h(c,d)?Ot(iv,l):Ev(l,new Lw(null,function(c,g,h,k,l){return function(){return UK.N?UK.N(b,l,d,e):UK.call(null,b,l,d,e)}}(g,h,k,l,m),null,null))};function SK(a){var b=a.Bd;a=a.head;var c=Gx(b,new V(null,2,5,X,[a,HE],null));return G(b)?UK(b,a,c,yD):null}function OK(a){var b=a.Bd;a=a.head;var c=Gx(b,new V(null,2,5,X,[a,HE],null));return G(b)?UK(b,c,a,HE):null}
var PK=new NK(null,Gy),VK=hx.h(jx,PK);ZH("linked/map",VK);function WK(a){this.ef=a;this.C=2313556239;this.Z=8192}f=WK.prototype;f.toString=function(){return[C.c("["),C.c(SE(" ",Z.h(C,this))),C.c("]")].join("")};f.equiv=function(a){return this.P(null,a)};f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){return Yt(this.ef,b)?b:c};f.ja=function(a,b){return Eu(b,[C.c("#linked/set "),C.c(jx.h(Mv,this))].join(""))};f.sa=function(){return Tv(this.ef)};f.ya=function(){return Lt(this.ef)};f.Lg=function(){var a=Du(this.ef);return a?Z.h(Aw,a):null};
f.ha=function(){return cv(jx.h(vz,this))};f.P=function(a,b){return Yv(b)&&Q(this)===Q(b)&&dx(function(a){return function(b){return lw(a,b)}}(this),b)};f.Ka=function(){return Sv(XK,Tv)};f.Mm=function(a,b){return new WK(cu(this.ef,b))};f.oa=function(){var a=G(this.ef);return a?Z.h(Aw,a):null};f.za=function(a,b){return new WK(Sv(this.ef,b))};f.xa=function(a,b){return new WK(T.j(this.ef,b,null))};
f.call=function(){var a=null,a=function(a,c,d){switch(arguments.length){case 2:return this.na(null,c);case 3:return this.Y(null,c,d)}throw Error("Invalid arity: "+(arguments.length-1));};a.h=function(a,c){return this.na(null,c)};a.j=function(a,c,d){return this.Y(null,c,d)};return a}();f.apply=function(a,b){return this.call.apply(this,[this].concat(Et(b)))};f.c=function(a){return this.na(null,a)};f.h=function(a,b){return this.Y(null,a,b)};var XK=new WK(PK),YK=hx.h(jx,XK);ZH("linked/set",YK);var ZK=new y(null,4,[kE,"Authorization",cB,0,DD,null,pA,null],null);CJ.h(sC,function(a,b){S(b,0,null);var c=S(b,1,null);return mz.w(R([new y(null,2,[WB,RJ,UB,ZK],null),c],0))});function $K(a,b,c,d,e,g,h){a=fa(a)?Date.UTC(a,b||0,c||1,d||0,e||0,g||0,h||0):a?a.getTime():wa();this.date=new Date(a)}r($K,ij);f=$K.prototype;f.clone=function(){var a=new $K(this.date);a.ed=this.ed;a.Pf=this.Pf;return a};f.add=function(a){(a.years||a.months)&&cj.prototype.add.call(this,new Wi(a.years,a.months));this.date=new Date(this.date.getTime()+1E3*(a.seconds+60*(a.minutes+60*(a.hours+24*a.days))))};f.getTimezoneOffset=function(){return 0};f.getFullYear=ij.prototype.getUTCFullYear;
f.getMonth=ij.prototype.getUTCMonth;f.getDate=ij.prototype.getUTCDate;f.getHours=ij.prototype.getUTCHours;f.getMinutes=ij.prototype.getUTCMinutes;f.getSeconds=ij.prototype.getUTCSeconds;f.getMilliseconds=ij.prototype.getUTCMilliseconds;f.getDay=ij.prototype.getUTCDay;f.setFullYear=ij.prototype.setUTCFullYear;f.setMonth=ij.prototype.setUTCMonth;f.setDate=ij.prototype.setUTCDate;f.setHours=ij.prototype.setUTCHours;f.setMinutes=ij.prototype.setUTCMinutes;f.setSeconds=ij.prototype.setUTCSeconds;
f.setMilliseconds=ij.prototype.setUTCMilliseconds;var aL=new V(null,12,5,X,"January February March April May June July August September October November December".split(" "),null),bL=new V(null,7,5,X,"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),null);function cL(a){return dx(function(a){return a instanceof cj},a)?U.h(L,Z.h(function(a){return a.getTime()},a)):U.h(L,a)}var dL=new V(null,12,5,X,[31,28,31,30,31,30,31,31,30,31,30,31],null);function eL(a){a=z(cL(R([a,1],0)))?11:a-1;return dL.c?dL.c(a):dL.call(null,a)}
function fL(a){a=null!=a&&(a.C&64||x===a.Sb)?U.h(px,a):a;var b=F.h(a,BD),c=F.h(a,VB),d=F.h(a,cC),e=F.h(a,AD),g=F.h(a,SD),h=F.h(a,tC),k=F.h(a,xE),l=z(c)?1<=c&&12>=c:null;if(z(b))if(z(c))if(z(l)){var m=eL(c);b=0===vw(b,400)?!0:0===vw(b,100)?!1:0===vw(b,4)?!0:!1;b=z(b)?cL(R([c,2],0)):b;m=z(b)?m+1:m}else m=l;else m=c;else m=z(c)?z(l)?eL(c):l:c;var p=z(d)?z(m)?1<=d&&d<=m:1<=d&&31>=d:null,u=z(e)?0<=e&&23>=e:null,A=z(g)?0<=g&&59>=g:null,b=z(h)?0<=h&&60>=h:null;m=z(k)?0<=k&&999>=k:null;if(!dx(gw,Ax(yt,new V(null,
6,5,X,[l,p,u,A,b,m],null)))){var D=cx,c=!1===l?T.j(D,VB,c):D,d=!1===p?T.j(c,cC,d):c,e=!1===u?T.j(d,AD,e):d,g=!1===A?T.j(e,SD,g):e,h=!1===b?T.j(g,tC,h):g,k=!1===m?T.j(h,xE,k):h;throw iA("Date is not valid",new y(null,3,[gC,mC,IA,a,yA,k],null));}}function gL(a,b){return I(tx(function(a,d){return z(cL(R([d,b],0)))?a:null},a))}function hL(a){a=Z.h(function(a){return a instanceof Y||a instanceof fv?""+C.c(a):a},a);return U.j(ym,"%s not implemented yet",a)};function iL(a,b){return QE("string"===typeof a?a:RE(a),b,"")}function jL(a){return new V(null,2,5,X,[HD,a],null)}function kL(a,b){var c=G(b);I(c);K(c);for(var c=b,d=Mv;;){var e=c,g=G(e),c=I(g),g=K(g);var h=c;h=z(h)?a.c?a.c(c):a.call(null,c):h;if(z(h))d=Lv.h(d,c),c=g;else return new V(null,2,5,X,[d,e],null)}}function lL(a,b){var c=kL(wz([a]),b),d=S(c,0,null),c=S(c,1,null);return new V(null,2,5,X,[new V(null,2,5,X,[OC,U.j(C,a,d)],null),c],null)}
function mL(a,b){var c=G(b),d=I(c),c=K(c);if(L.h(d,"'"))return new V(null,2,5,X,[jL("'"),c],null);c=kL(fx(new tz(null,new y(null,1,["'",null],null),null)),b);d=S(c,0,null);c=S(c,1,null);return new V(null,2,5,X,[jL(d),hv(c)],null)}function nL(a,b){var c=kL(function(a){return At(Ez(/[a-zA-Z']/,a))},b),d=S(c,0,null),c=S(c,1,null);return new V(null,2,5,X,[jL(U.j(C,a,d)),c],null)}
function oL(a){var b=a;for(a=Mv;;){var c=G(b),b=I(c),c=K(c),d=L.h(b,"'")?mL:z(Ez(/[a-zA-Z]/,""+C.c(b)))?lL:nL,b=d.h?d.h(b,c):d.call(null,b,c),c=S(b,0,null),b=S(b,1,null);a=Lv.h(a,c);if(!G(b))return a}}
function pL(a,b){var c=I(a);var d=RE(hv(a));var e=Q(b)-1,g=[C.c(c),C.c(d.substring(0,e))].join("");d=L.h(b,g)?new V(null,2,5,X,[g,yw(d,e,Q(d))],null):new V(null,2,5,X,[null,[C.c(c),C.c(d)].join("")],null);c=S(d,0,null);d=S(d,1,null);return z(c)?new V(null,2,5,X,[new V(null,2,5,X,[uA,c],null),d],null):null}
function qL(a,b,c){a=kL(function(a){return Ez(/\d/,a)},a);var d=S(a,0,null);a=S(a,1,null);return Q(d)>=b?new V(null,2,5,X,[function(){var a=U.h(C,ux(c,d));return parseInt(a)}(),Ww.h(vx(c,d),a)],null):new V(null,2,5,X,[function(){var a=U.h(C,d);return parseInt(a)}(),a],null)}function rL(a,b,c,d){c=qL(a,c,d);a=S(c,0,null);c=S(c,1,null);return new V(null,2,5,X,[new V(null,2,5,X,[b,a],null),c],null)}function sL(a,b){return function(c){return rL(c,BD,a,b)}}
function tL(a,b){return function(c){return rL(c,fC,a,b)}}function uL(a){return function(b){return rL(b,aD,a,2)}}function vL(){return function(a){return rL(a,VB,1,2)}}function wL(a,b){return function(c){return rL(c,cC,a,b)}}function xL(a){return function(b){return rL(b,AD,a,2)}}function yL(a){return function(b){return rL(b,rE,a,2)}}function zL(a){return function(b){return rL(b,SD,a,2)}}function AL(a){return function(b){return rL(b,tC,a,2)}}function BL(a,b){return function(c){return rL(c,xE,a,b)}}
function CL(a,b,c){b=60*parseInt(b,10)+parseInt(c,10);a=L.h(a,"+")?sw:rw;return new V(null,2,5,X,[KA,new Wi(aj,a.c?a.c(b):a.call(null,b))],null)}
function DL(a){return function(b){var c=G(b);var d=I(c);var e=K(c),g=function(){return function(){return iA([C.c("Invalid timezone format: "),C.c(b)].join(""),new y(null,1,[gC,uC],null))}}(b,c,d,e,d,e),h=function(a,b,c,d,e,g){return function(a){var b=RE(ux(4,g)),c=Ez(/^(\d{2})(\d{2})/,b);return z(c)?(S(c,0,null),b=S(c,1,null),c=S(c,2,null),new V(null,2,5,X,[CL(a,b,c),vx(4,g)],null)):null}}(b,c,d,e,d,e,g),k=function(a,b,c,d,e,g){return function(a){var b=RE(ux(5,g)),c=Ez(/^(\d{2}):(\d{2})/,b);return z(c)?
(S(c,0,null),b=S(c,1,null),c=S(c,2,null),new V(null,2,5,X,[CL(a,b,c),vx(5,g)],null)):null}}(b,c,d,e,d,e,g,h);if(z((new tz(null,new y(null,2,["+",null,"-",null],null),null)).call(null,d)))switch(c=a instanceof Y?a.Vb:null,c){case "dddd":h=h(d);if(z(h))return h;d=k(d);if(z(d))return d;throw g();case "long":h=h(d);if(z(h))return h;d=k(d);if(z(d))return d;throw g();default:throw Error([C.c("No matching clause: "),C.c(c)].join(""));}else{if(L.h(d,"Z"))return new V(null,1,5,X,[new V(null,2,5,X,[KA,CL(rw,
"0","0")],null)],null);var l=a instanceof Y?a.Vb:null;switch(l){case "abbr":var m=ux(3,b);d=kL(function(){return function(a){return Ez(/[A-Z]/,a)}}(m,a,l,b,c,d,e,d,e,g,h,k),m);k=S(d,0,null);S(d,1,null);if(L.h(Q(k),3))return new V(null,2,5,X,[new V(null,2,5,X,[KA,RE(k)],null),vx(3,b)],null);throw g();case "full":throw iA([C.c("Cannot parse long form timezone:"),C.c(b)].join(""),new y(null,1,[gC,uC],null));default:throw g();}}}}
function EL(){return function(a){var b=new V(null,2,5,X,[ux(2,a),vx(2,a)],null);var c=S(b,0,null);a=S(c,0,null);c=S(c,1,null);b=S(b,1,null);var d=[C.c(a),C.c(c)].join(""),c=z((new tz(null,new y(null,4,["AM",null,"am",null,"pm",null,"PM",null],null),null)).call(null,d))?new V(null,2,5,X,[d,b],null):z((new tz(null,new y(null,2,["a",null,"p",null],null),null)).call(null,a))?new V(null,2,5,X,[(new y(null,2,["a","am","p","pm"],null)).call(null,a),Ev(c,b)],null):z((new tz(null,new y(null,2,["A",null,"P",
null],null),null)).call(null,a))?new V(null,2,5,X,[(new y(null,2,["A","am","P","pm"],null)).call(null,a),Ev(c,b)],null):null;a=S(c,0,null);c=S(c,1,null);return new V(null,2,5,X,[new V(null,2,5,X,[HB,Jw.c(a)],null),RE(c)],null)}}
function FL(a,b,c){c=Ww.h(c,Z.h(function(a){return a.substring(0,3)},c));var d=I(Ax(gx.h(hx.h(L,a),Jv),Z.h(function(){return function(b){return new V(null,2,5,X,[b,iL(a,Fz([C.c("^"),C.c(b)].join("")))],null)}}(c),c))),e=S(d,0,null),d=S(d,1,null);if(z(e))return new V(null,2,5,X,[new V(null,2,5,X,[b,vw(gL(c,e),12)],null),d],null);throw iA([C.c("Could not parse "),C.c(Kw(b)),C.c(" name")].join(""),new y(null,4,[gC,uC,bD,WC,PD,b,GE,d],null));}
function GL(){return function(a){return Jx.j(FL(a,VB,aL),new V(null,2,5,X,[0,1],null),qv)}}function HL(){return function(a){return FL(a,cC,bL)}}function IL(a){return function(b){var c=iL(b,Fz([C.c("^"),C.c(a)].join("")));if(L.h(b,c))throw iA("Quoted text not found",new y(null,2,[gC,uC,SA,hB],null));return new V(null,2,5,X,[new V(null,2,5,X,[HD,a],null),c],null)}}
function JL(){return function(a){var b=pL(a,"st");if(z(b))return b;b=pL(a,"nd");if(z(b))return b;b=pL(a,"rd");return z(b)?b:pL(a,"th")}}
function KL(a){var b=S(a,0,null);a=S(a,1,null);if(L.h(b,OC))switch(a){case "S":return BL(1,2);case "SSS":return BL(3,3);case "s":return AL(1);case "ss":return AL(2);case "m":return zL(1);case "mm":return zL(2);case "h":return xL(1);case "hh":return xL(2);case "H":return yL(1);case "HH":return yL(2);case "d":return wL(1,2);case "dd":return wL(2,2);case "D":return wL(1,3);case "DD":return wL(2,3);case "DDD":return wL(3,3);case "M":return vL();case "MM":return vL();case "MMM":return GL();case "MMMM":return GL();
case "y":return sL(1,4);case "yy":return sL(2,2);case "yyyy":return sL(4,4);case "Y":return sL(1,4);case "YY":return sL(2,2);case "YYYY":return sL(4,4);case "x":return tL(1,4);case "xx":return tL(2,2);case "xxxx":return tL(4,4);case "w":return uL(1);case "ww":return uL(2);case "E":return HL();case "EEE":return HL();case "EEEE":return HL();case "a":return EL();case "A":return EL();case "Z":return DL(aB);case "ZZ":return DL(LA);case "ZZZ":return DL(hC);case "ZZZZ":return DL(hC);case "z":return DL(hC);
case "zz":return DL(hC);case "zzz":return DL(hC);case "zzzz":return DL(XA);case "o":return JL();default:throw iA([C.c("Illegal pattern component: "),C.c(a)].join(""),new y(null,1,[gC,yB],null));}else return IL(a)}
function LL(a,b){for(var c=Z.h(KL,oL(a)),d=G(c),e=I(d),g=K(d),h=Mv,k=b,l=c,m=h;;){var p=k,u=l,A=G(u),D=I(A),E=K(A),J=D,M=E,O=m,k=function(a,c,d,e){return function(){return iA([C.c("Invalid format: "),C.c(b),C.c(" is malformed at "),C.c(Tz(R([e],0),st()))].join(""),new y(null,2,[gC,uC,bD,cD],null))}}(k,l,m,p,u,A,D,E,J,M,O,b,c,c,d,e,g,e,g,h);if(G(p)){if(null==J)throw k();l=J.c?J.c(p):J.call(null,p);k=S(l,0,null);l=S(l,1,null);O=Lv.h(O,k);k=l;l=M;m=O}else{if(z(J))throw k();return O}}}
function ML(a,b,c){b=null!=b&&(b.C&64||x===b.Sb)?U.h(px,b):b;b=F.h(b,IE);c=jx.h(cx,Ax(gx.h(new tz(null,new y(null,1,[HD,null],null),null),I),c));var d=null!=c&&(c.C&64||x===c.Sb)?U.h(px,c):c,e=F.h(d,SD),g=F.h(d,rE),h=F.h(d,xE);c=F.h(d,KA);var k=F.h(d,HB),l=F.h(d,VB),m=F.h(d,cC),p=F.h(d,tC),u=F.h(d,AD),A=F.h(d,BD),D=z(A)?A:z(b)?b:0,E=z(l)?l-1:null,J=z(k)?z((new tz(null,new y(null,2,[YB,null,jD,null],null),null)).call(null,k))?function(){var a=u+12;return L.h(a,24)?12:a}():L.h(u,12)?0:u:g;b=Pv.w(T.j(d,
AD,J),rE,R([HB],0));c=c instanceof Wi?c:new Wi(bj,0);fL(b);b=function(){var b=a instanceof Y?a.Vb:null;switch(b){case "goog.date.Date":return new cj(D,E,m);case "goog.date.DateTime":return new ij(D,E,m,J,e,p,h);case "goog.date.UtcDateTime":return new $K(D,E,m,J,e,p,h);default:throw Error([C.c("No matching clause: "),C.c(b)].join(""));}}();b.add(c);return b};function NL(a,b,c,d,e){c=1>d?""+C.c(c):[C.c(RE(ux(d-Q(""+C.c(c)),xx("0")))),C.c(c)].join("");d=Q(c);e=c.substring(d-e);return new V(null,2,5,X,[[C.c(a),C.c(e)].join(""),b],null)}var OL=function OL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return OL.c(arguments[0]);case 2:return OL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};OL.c=function(a){return OL.h(a,a)};
OL.h=function(a,b){return function(c,d){return NL(c,d,d.getMilliseconds(),a,b)}};OL.L=2;var PL=function PL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return PL.c(arguments[0]);case 2:return PL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};PL.c=function(a){return PL.h(a,a)};PL.h=function(a,b){return function(c,d){return NL(c,d,d.getSeconds(),a,b)}};PL.L=2;
var QL=function QL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return QL.c(arguments[0]);case 2:return QL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};QL.c=function(a){return QL.h(a,a)};QL.h=function(a,b){return function(c,d){return NL(c,d,d.getMinutes(),a,b)}};QL.L=2;
var RL=function RL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return RL.c(arguments[0]);case 2:return RL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};RL.c=function(a){return RL.h(a,a)};RL.h=function(a,b){return function(c,d){var e=vw(d.getHours(),12);return NL(c,d,0===e?12:e,a,b)}};RL.L=2;
var SL=function SL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return SL.c(arguments[0]);case 2:return SL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};SL.c=function(a){return SL.h(a,a)};SL.h=function(a,b){return function(c,d){return NL(c,d,d.getHours(),a,b)}};SL.L=2;
var TL=function TL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return TL.c(arguments[0]);case 2:return TL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};TL.c=function(a){return TL.h(a,a)};TL.h=function(a,b){return function(c,d){return NL(c,d,d.getDate(),a,b)}};TL.L=2;
var UL=function UL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return UL.c(arguments[0]);case 2:return UL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};UL.c=function(a){return UL.h(a,a)};UL.h=function(a,b){return function(c,d){return NL(c,d,d.getMonth()+1,a,b)}};UL.L=2;
var VL=function VL(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return VL.c(arguments[0]);case 2:return VL.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};VL.c=function(a){return VL.h(a,a)};VL.h=function(a,b){return function(c,d){return NL(c,d,d.getYear(),a,b)}};VL.L=2;function WL(a){return function(b,c){return new V(null,2,5,X,[U.j(C,b,a),c],null)}}
function XL(a,b,c,d,e){var g=fy(function(){return z(e)?Z.h(function(){return function(a){return a.substring(0,3)}}(d),d):d}());return new V(null,2,5,X,[[C.c(a),C.c(g.c?g.c(c):g.call(null,c))].join(""),b],null)}function YL(a){return function(b,c){return XL(b,c,c.getMonth(),aL,a)}}function ZL(a){return function(b,c){return XL(b,c,c.getDay(),bL,a)}}
var $L=function $L(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return $L.c(arguments[0]);case 2:return $L.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};$L.c=function(a){return $L.h(a,a)};$L.h=function(a,b){return function(c,d){var e=d.getYear(),g=d.getMonth(),h=d.getDate(),k=L.h(g,0),l=L.h(g,11),g=Vi(e,g,h),e=k&&52<=g?e-1:l&&L.h(g,1)?e+1:e;return NL(c,d,e,a,b)}};$L.L=2;
var aM=function aM(b){for(var c=[],d=arguments.length,e=0;;)if(e<d)c.push(arguments[e]),e+=1;else break;switch(c.length){case 1:return aM.c(arguments[0]);case 2:return aM.h(arguments[0],arguments[1]);default:throw Error([C.c("Invalid arity: "),C.c(c.length)].join(""));}};aM.c=function(a){return aM.h(a,a)};aM.h=function(a,b){return function(c,d){var e=d.getYear(),g=d.getMonth(),h=d.getDate();return NL(c,d,Vi(e,g,h),a,b)}};aM.L=2;
function bM(a){return function(b,c){var d=c.getHours(),d=z(a)?12>d?"AM":"PM":12>d?"am":"pm";return new V(null,2,5,X,[[C.c(b),C.c(d)].join(""),c],null)}}function cM(){return function(a,b){return b instanceof $K?new V(null,2,5,X,[[C.c(a),C.c(hj(b))].join(""),b],null):new V(null,2,5,X,[a,b],null)}}
function dM(a){return function(b,c){var d=a.c?a.c(c):a.call(null,c),e=function(){switch(d){case 1:return"st";case 2:return"nd";case 3:return"rd";case 21:return"st";case 22:return"nd";case 23:return"rd";case 31:return"st";default:return"th"}}();return new V(null,2,5,X,[[C.c(b),C.c(e)].join(""),c],null)}}
function eM(a){var b=S(a,0,null);a=S(a,1,null);if(L.h(b,OC))switch(a){case "S":return new V(null,3,5,X,[xE,1,2],null);case "SSS":return new V(null,3,5,X,[xE,3,3],null);case "s":return new V(null,3,5,X,[tC,1,2],null);case "ss":return new V(null,3,5,X,[tC,2,2],null);case "m":return new V(null,3,5,X,[SD,1,2],null);case "mm":return new V(null,3,5,X,[SD,2,2],null);case "h":return new V(null,3,5,X,[AD,1,2],null);case "hh":return new V(null,3,5,X,[AD,2,2],null);case "H":return new V(null,3,5,X,[rE,1,2],
null);case "HH":return new V(null,3,5,X,[rE,2,2],null);case "d":return new V(null,3,5,X,[CA,1,2],null);case "dd":return new V(null,3,5,X,[CA,2,2],null);case "D":return new V(null,3,5,X,[CA,1,3],null);case "DD":return new V(null,3,5,X,[CA,2,3],null);case "DDD":return new V(null,3,5,X,[CA,3,3],null);case "M":return new V(null,3,5,X,[$B,1,2],null);case "MM":return new V(null,3,5,X,[$B,2,2],null);case "MMM":return new V(null,2,5,X,[zA,!0],null);case "MMMM":return new V(null,2,5,X,[zA,!1],null);case "y":return new V(null,
3,5,X,[NC,1,4],null);case "yy":return new V(null,3,5,X,[NC,2,2],null);case "yyyy":return new V(null,3,5,X,[NC,4,4],null);case "Y":return new V(null,3,5,X,[NC,1,4],null);case "YY":return new V(null,3,5,X,[NC,2,2],null);case "YYYY":return new V(null,3,5,X,[NC,4,4],null);case "x":return new V(null,3,5,X,[fC,1,4],null);case "xx":return new V(null,3,5,X,[fC,2,2],null);case "xxxx":return new V(null,3,5,X,[fC,4,4],null);case "w":return new V(null,3,5,X,[aD,1,2],null);case "ww":return new V(null,3,5,X,[aD,
2,2],null);case "E":return new V(null,2,5,X,[IC,!0],null);case "EEE":return new V(null,2,5,X,[IC,!0],null);case "EEEE":return new V(null,2,5,X,[IC,!1],null);case "a":return new V(null,2,5,X,[HB,!1],null);case "A":return new V(null,2,5,X,[HB,!0],null);case "Z":return new V(null,1,5,X,[KA],null);case "ZZ":return new V(null,1,5,X,[KA],null);case "o":return new V(null,1,5,X,[uA],null);default:throw Error([C.c("No matching clause: "),C.c(a)].join(""));}else return new V(null,2,5,X,[HD,a],null)}
function fM(a){var b=a instanceof Y?a.Vb:null;switch(b){case "millis":return function(){return function(a){return a.getMilliseconds()}}(a,b);case "seconds":return function(){return function(a){return a.getSeconds()}}(a,b);case "minutes":return function(){return function(a){return a.getMinutes()}}(a,b);case "hours":return function(){return function(a){return a.getHours()}}(a,b);case "HOURS":return function(){return function(a){return a.getHours()}}(a,b);case "day":return function(){return function(a){return a.getDate()}}(a,
b);case "month":return function(){return function(a){return a.getMonth()}}(a,b);case "year":return function(){return function(a){return a.getYear()}}(a,b);default:throw Error([C.c("No matching clause: "),C.c(b)].join(""));}}
function gM(a,b,c){var d=G(c);c=I(d);d=K(d);c=c instanceof Y?c.Vb:null;switch(c){case "ordinal-suffix":return--b,a=a.c?a.c(b):a.call(null,b),a=S(a,0,null),dM(fM(a));case "month-name":return U.h(YL,d);case "day":return U.h(TL,d);case "timezone":return U.h(cM,d);case "meridiem":return U.h(bM,d);case "month":return U.h(UL,d);case "weekyear":return U.h($L,d);case "seconds":return U.h(PL,d);case "day-name":return U.h(ZL,d);case "year":return U.h(VL,d);case "weekyear-week":return U.h(aM,d);case "hours":return U.h(RL,
d);case "quoted":return U.h(WL,d);case "minutes":return U.h(QL,d);case "HOURS":return U.h(SL,d);case "millis":return U.h(OL,d);default:throw Error([C.c("No matching clause: "),C.c(c)].join(""));}};var hM={id:"UTC",std_offset:0,names:new V(null,1,5,X,["UTC"],null),transitions:Mv};function iM(a,b,c,d,e,g,h){this.pe=a;this.Ce=b;this.me=c;this.Je=d;this.Pa=e;this.ea=g;this.M=h;this.C=2229667594;this.Z=8192}f=iM.prototype;f.na=function(a,b){return this.Y(null,b,null)};f.Y=function(a,b,c){switch(b instanceof Y?b.Vb:null){case "format-str":return this.pe;case "overrides":return this.Ce;case "default-year":return this.me;case "timezone":return this.Je;default:return F.j(this.ea,b,c)}};
f.ja=function(a,b,c){return Gz(b,function(){return function(a){return Gz(b,Oz,""," ","",c,a)}}(this),"#cljs-time.format.Formatter{",", ","}",c,Ww.h(new V(null,4,5,X,[new V(null,2,5,X,[dC,this.pe],null),new V(null,2,5,X,[QA,this.Ce],null),new V(null,2,5,X,[IE,this.me],null),new V(null,2,5,X,[KA,this.Je],null)],null),this.ea))};f.lc=function(){return new wy(0,this,4,new V(null,4,5,X,[dC,QA,IE,KA],null),z(this.ea)?Su(this.ea):ax())};f.sa=function(){return this.Pa};f.ya=function(){return 4+Q(this.ea)};
f.ha=function(){var a=this.M;return null!=a?a:this.M=a=zw(this)};f.P=function(a,b){return z(z(b)?this.constructor===b.constructor&&vy(this,b):b)?!0:!1};f.Df=function(a,b){return lw(new tz(null,new y(null,4,[KA,null,QA,null,dC,null,IE,null],null),null),b)?Pv.h(ru(jx.h(cx,this),this.Pa),b):new iM(this.pe,this.Ce,this.me,this.Je,this.Pa,$w(Pv.h(this.ea,b)),null)};
f.Rb=function(a,b,c){return z(Iw.h?Iw.h(dC,b):Iw.call(null,dC,b))?new iM(c,this.Ce,this.me,this.Je,this.Pa,this.ea,null):z(Iw.h?Iw.h(QA,b):Iw.call(null,QA,b))?new iM(this.pe,c,this.me,this.Je,this.Pa,this.ea,null):z(Iw.h?Iw.h(IE,b):Iw.call(null,IE,b))?new iM(this.pe,this.Ce,c,this.Je,this.Pa,this.ea,null):z(Iw.h?Iw.h(KA,b):Iw.call(null,KA,b))?new iM(this.pe,this.Ce,this.me,c,this.Pa,this.ea,null):new iM(this.pe,this.Ce,this.me,this.Je,this.Pa,T.j(this.ea,b,c),null)};
f.oa=function(){return G(Ww.h(new V(null,4,5,X,[new V(null,2,5,X,[dC,this.pe],null),new V(null,2,5,X,[QA,this.Ce],null),new V(null,2,5,X,[IE,this.me],null),new V(null,2,5,X,[KA,this.Je],null)],null),this.ea))};f.za=function(a,b){return new iM(this.pe,this.Ce,this.me,this.Je,b,this.ea,this.M)};f.xa=function(a,b){return bw(b)?this.Rb(null,Qt.h(b,0),Qt.h(b,1)):Gt(Ot,this,b)};
function jM(a){a=new y(null,2,[dC,a,KA,hM],null);return new iM(dC.c(a),QA.c(a),IE.c(a),KA.c(a),null,Pv.w(a,dC,R([QA,IE,KA],0)),null)}function kM(a){return function(){throw{name:UD,message:hL(R([Kw(a)],0))};}}var lM;
a:for(var mM=[oA,rA,FA,HA,IA,JA,OA,PA,TA,UA,YA,ZA,$A,bB,dB,jB,kB,mB,nB,qB,vB,CB,EB,GB,JB,MB,NB,SB,fC,jC,pC,qC,xC,AC,GC,LC,NC,PC,RC,XC,aD,dD,eD,lD,wD,CD,MD,fE,lE,nE,uE,CE,FE],nM=[kM(QC),jM("HH:mm"),jM("'T'HH:mm:ss.SSSZZ"),jM("yyyyDDD"),jM("yyyy-MM-dd"),jM("HH"),jM("HH:mm:ssZZ"),jM("xxxx-'W'ww-e"),jM("xxxx-'W'ww-e'T'HH:mm:ss.SSSZZ"),jM("yyyy-MM-dd'T'HH:mm:ss.SSS"),jM("yyyyMMdd'T'HHmmss.SSSZ"),jM("yyyy-MM-dd'T'HH:mm:ss.SSSZZ"),jM("HHmmssZ"),kM(vC),jM("xxxx'W'wwe"),jM("'T'HHmmssZ"),kM(lA),jM("yyyy-MM-dd'T'HH:mm:ssZZ"),
jM("yyyy-MM-dd"),kM(gB),jM("EEE, dd MMM yyyy HH:mm:ss Z"),jM("yyyy-MM-dd'T'HH:mm:ss.SSS"),jM("yyyyDDD'T'HHmmss.SSSZ"),jM("yyyy-DDD"),jM("HH:mm:ss.SSS"),jM("yyyy-MM-dd'T'HH:mm"),jM("HH:mm:ss.SSSZZ"),jM("xxxx'W'wwe'T'HHmmss.SSSZ"),jM("xxxx"),jM("HHmmss.SSSZ"),jM("HH:mm:ss"),jM("yyyy-DDD'T'HH:mm:ss.SSSZZ"),jM("yyyy-DDD'T'HH:mm:ssZZ"),jM("HH:mm:ss.SSS"),jM(GA),kM(uD),jM("yyyy"),jM("'T'HH:mm:ssZZ"),jM("xxxx'W'wwe'T'HHmmssZ"),jM("yyyyMMdd"),jM("xxxx-'W'ww"),kM(kC),jM("yyyyDDD'T'HHmmssZ"),jM("yyyy-MM"),
kM(SC),jM("xxxx-'W'ww-e"),jM("yyyy-MM-dd'T'HH"),kM(FB),jM("yyyy-MM-dd'T'HH:mm:ss"),jM("xxxx-'W'ww-e'T'HH:mm:ssZZ"),jM("yyyyMMdd'T'HHmmssZ"),jM("yyyy-MM-dd HH:mm:ss"),jM("'T'HHmmss.SSSZ")],oM=mM.length,pM=0,qM=Ju(Gy);;)if(pM<oM)var rM=pM+1,sM=qM.Ef(null,mM[pM],nM[pM]),pM=rM,qM=sM;else{lM=Lu(qM);break a}var tM=new tz(null,new y(null,9,[oA,null,bB,null,kB,null,qB,null,GC,null,LC,null,dD,null,wD,null,fE,null],null),null);xF.h(xz(Cy(lM)),tM);function uM(a,b){return ML(dE,a,LL(dC.c(a),b))}
function vM(a){return I(function(){return function c(d){return new Lw(null,function(){for(var e=d;;)if(e=G(e)){if(cw(e)){var g=Ou(e),h=Q(g),k=Pw(h);a:for(var l=0;;)if(l<h){var m=Qt.h(g,l);try{var p=uM(m,a)}catch(u){p=null}z(p)&&k.add(p);l+=1}else{p=!0;break a}return p?Rw(k.sb(),c(Pu(e))):Rw(k.sb(),null)}g=I(e);try{k=uM(g,a)}catch(u){k=null}if(z(k))return Ev(k,c(hv(e)));e=hv(e)}else return null},null,null)}(Dy(lM))}())};jM("dd. MM. yyyy");var wM=jM("dd.MM.yyyy HH:mm");function xM(a,b){return F.h(I(zx(function(a){return L.h(b,F.h(a,"id"))},a)),"value")}function yM(a,b){return Z.h(function(a){var c=S(a,0,null),e=S(a,1,null);return Ev(c,Z.h(function(a,b){return function(a){return F.j(a,b,0)}}(a,c,e),b))},a)}function zM(a){return Z.h(function(a){return new Date(BE.c(a))},a)}
function AM(a,b){var c=eA(b,R([fA,!1],0)),d=[C.c('\x3cdiv class\x3d"tooltip"\x3e'),C.c("\x3cdiv\x3e%s\x3c/div\x3e"),C.c("\x3cdiv\x3eBodový zisk: %s bodů\x3c/div\x3e"),C.c("\x3cdiv\x3eCelkový nákupní limit: %s\x3c/div\x3e"),C.c("\x3cdiv\x3eDostupný limit: %s\x3c/div\x3e"),C.c("\x3cdiv\x3eAktivní revolving: %s\x3c/div\x3e"),C.c("\x3cdiv\x3eDůvod: %s\x3c/div\x3e"),C.c("\x3c/div\x3e")].join("");var e=(new ij(F.h(I(c),"x"))).toIsoString();if(Ba(e))e=null;else{e=vM(e);var g=null!=wM&&(wM.C&64||x===wM.Sb)?
U.h(px,wM):wM,h=F.h(g,dC);F.h(g,AA);if(null==e)throw Error("Assert failed: (not (nil? dt))");if(!(e instanceof cj))throw Error("Assert failed: (instance? goog.date.Date dt)");a:for(g=Dx(eM,oL(h)),h=lx(hx.h(gM,g),g),g=G(h),I(g),K(g),g=e,e="";;){var k=G(h),h=I(k),l=K(k),k=h,h=l;if(null==k)break a;g=k.h?k.h(e,g):k.call(null,e,g);e=S(g,0,null);g=S(g,1,null)}}g=lC.c(F.h(a,F.h(I(c),"index")));h=XE(xM(c,"price_limit"));k=XE(xM(c,"available_limit"));l=CC.c(F.h(a,F.h(I(c),"index")));c=mA.c(F.h(a,F.h(I(c),
"index")));return ym(d,e,g,h,k,l,c)}
lI("#history-limit",new V(null,2,5,X,[function(a,b){var c=I(F.h(b,vA)),d=I(F.h(b,IB)),c=new y(null,5,[ME,new y(null,3,[KD,"x",yE,c,AB,Ev(Ev("x",zM(d)),yM(c,d))],null),WD,new y(null,1,[KD,new y(null,3,[gC,"timeseries",kA,new y(null,1,[DA,"%d-%m-%Y"],null),RB,!0],null)],null),zE,new y(null,1,[OD,hx.h(AM,d)],null),kD,a,gE,new y(null,1,[KE,!0],null)],null);return c3.generate(bA(c))},new V(null,2,5,X,[new y(null,3,[DB,vA,gC,vE,AE,!0],null),new y(null,3,[DB,IB,gC,wC,AE,!0],null)],null)],null));ha("backend.core.run",function(){window.__dispatch=function(a){a=new V(null,2,5,X,[BB,a],null);return VI.c?VI.c(a):VI.call(null,a)};return DJ()});
})();
